@@ -18,14 +18,15 @@
 :- module hlds.assertion.
 :- interface.
 
-:- import_module hlds.hlds_data.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module hlds.hlds_promise.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.set_of_var.
 
 :- import_module list.
-:- import_module pair.
 
 %-----------------------------------------------------------------------------%
 
@@ -39,37 +40,57 @@
 :- pred record_preds_used_in(hlds_goal::in, assert_id::in,
     module_info::in, module_info::out) is det.
 
-    % is_commutativity_assertion(MI, Id, Vs, CVs):
+    % is_commutativity_assertion(ModuleInfo, AssertId, CallVars, CommuteVars):
+    % is_commutativity_assertion_goal(Goal, CallVars, CommuteVars):
     %
-    % Does the assertion represented by the assertion id, Id,
-    % state the commutativity of a pred/func?
-    % We extend the usual definition of commutativity to apply to
-    % predicates or functions with more than two arguments as
-    % follows by allowing extra arguments which must be invariant.
-    % If so, this predicate returns (in CVs) the two variables which
-    % can be swapped in order if it was a call to Vs.
+    % Does the assertion represented by AssertId or Goal say that
+    % the one predicate or function called in the code is commutative?
+    % If so, given CallVars, the argument list of a call to this predicate
+    % or function, this predicate returns (in CommuteVars) the two variables
+    % which can be swapped. (This identifies the two commutative argument
+    % positions.)
     %
     % The assertion must be in a form similar to this
+    %
     %   all [Is,A,B,C] ( p(Is,A,B,C) <=> p(Is,B,A,C) )
-    % for the predicate to return true (note that the invariant
-    % arguments, Is, can be any where providing they are in
-    % identical locations on both sides of the equivalence).
+    %
+    % for this predicate to return true.
+    %
+    % We extend the usual definition of commutativity to apply to predicates
+    % or functions with more than two input arguments by allowing extra
+    % input arguments which must be invariant. The invariant arguments, Is,
+    % can be anywhere, providing they are in identical locations on both sides
+    % of the equivalence.
     %
 :- pred is_commutativity_assertion(module_info::in, assert_id::in,
-    list(prog_var)::in, pair(prog_var)::out) is semidet.
+    list(prog_var)::in, set_of_progvar::out) is semidet.
+:- pred is_commutativity_assertion_goal(hlds_goal::in,
+    list(prog_var)::in, set_of_progvar::out) is semidet.
 
-    % is_associativity_assertion(MI, Id, Vs, CVs, OV):
+:- type associative_vars_output_var
+    --->    associative_vars_output_var(
+                % The associative vars.
+                set_of_progvar,
+
+                % The output var.
+                prog_var
+            ).
+
+    % is_associativity_assertion(ModuleInfo, AssertId, CallVars,
+    %   AssocVarsOutputVar):
+    % is_associativity_assertion_goal(Goal, CallVars,
+    %   AssocVarsOutputVar):
     %
-    % Does the assertion represented by the assertion id, Id,
-    % state the associativity of a pred/func?
-    % We extend the usual definition of associativity to apply to
-    % predicates or functions with more than two arguments as
-    % follows by allowing extra arguments which must be invariant.
-    % If so, this predicate returns (in CVs) the two variables which
-    % can be swapped in order if it was a call to Vs, and the
-    % output variable, OV, related to these two variables (for the
-    % case below it would be the variable in the same position as
-    % AB, BC or ABC).
+    % Does the assertion represented by AssertId or Goal say that
+    % the one predicate or function called in the code is associative?
+    % If so, this predicate returns
+    %
+    %   associative_vars_output_var(AssocVars, OutputVar)
+    %
+    % Given CallVars, the argument list of a call to this predicate or function
+    % (which may be e.g. [Is, A, B, C] in the example below), AssocVars will
+    % indicate the positions of the two input variables which can be swapped,
+    % and OutputVar will indicate the position of the output variable.
     %
     % The assertion must be in a form similar to this
     %
@@ -80,22 +101,36 @@
     %     some [BC] p(Is,B,C,BC), p(Is,A,BC,ABC)
     %   )
     %
-    % for the predicate to return true (note that the invariant
-    % arguments, Is, can be any where providing they are in
-    % identical locations on both sides of the equivalence).
+    % for this predicate to return true.
+    %
+    % We extend the usual definition of associativity to apply to predicates
+    % or functions with more than two input arguments by allowing extra
+    % input arguments which must be invariant. The invariant arguments, Is,
+    % can be anywhere, providing they are in identical locations on both sides
+    % of the equivalence.
     %
 :- pred is_associativity_assertion(module_info::in, assert_id::in,
-    list(prog_var)::in, pair(prog_var)::out, prog_var::out) is semidet.
+    list(prog_var)::in, associative_vars_output_var::out) is semidet.
+:- pred is_associativity_assertion_goal(hlds_goal::in,
+    list(prog_var)::in, associative_vars_output_var::out) is semidet.
 
-    % is_update_assertion(MI, Id, PId, Ss):
+:- type state_update_vars
+    --->    state_update_vars(prog_var, prog_var).
+
+    % is_update_assertion(ModuleInfo, AssertId, PredId, CallVars, StateVars):
+    % is_update_assertion_goal(Goal, PredId, CallVars, StateVars):
     %
-    % is true iff the assertion, Id, is about a predicate, PId,
-    % which takes some state as input and produces some state as output
-    % and we are guaranteed to get the same final state regardless of
-    % the order that the state is updated.
+    % Does the assertion represented by AssertId or Goal say that
+    % predicate PredId is a predicate that updates some state
+    % (taking one version of the state as input and returning another
+    % version as output) for which we are guaranteed to get the *same*
+    % final state regardless of the order in which the state updates
+    % are applied? Given the full list of arguments of a call to PredId
+    % in CallVars, return in StateVars the pair of variables which represent
+    % the initial and updated versions of the state. In the example below,
+    % given a call update(S0, A, SA), these would be S0 and SA.
     %
-    % i.e. the promise should look something like this, note that A
-    % and B could be vectors of variables.
+    % The assertion must be in a form similar to this
     %
     % :- promise all [A,B,SO,S]
     %   (
@@ -104,27 +139,35 @@
     %       (some [SB] (update(S0,B,SB), update(SB,A,S)))
     %   ).
     %
-    % Given the actual variables, Vs, to the call to update, return
-    % the pair of variables which are state variables, SPair.
+    % for this predicate to return true. Note that A and B could be
+    % vectors of variables.
     %
 :- pred is_update_assertion(module_info::in, assert_id::in,
-    pred_id::in, list(prog_var)::in, pair(prog_var)::out) is semidet.
+    pred_id::in, list(prog_var)::in, state_update_vars::out) is semidet.
+:- pred is_update_assertion_goal(hlds_goal::in,
+    pred_id::in, list(prog_var)::in, state_update_vars::out) is semidet.
 
-    % is_construction_equivalence_assertion(MI, Id, C, P):
+    % is_construction_equivalence_assertion(ModuleInfo, AssertId,
+    %   ConsId, PredId):
+    % is_construction_equivalence_assertion_goal(Goal, ConsId, PredId):
     %
-    % Can a single construction unification whose functor is determined
-    % by the cons_id, C, be expressed as a call to the predid, P (with possibly
-    % some construction unifications to initialise the arguments).
+    % Can a single construction unification whose functor is ConsId
+    % be expressed as a call to PredId, with possibly some construction
+    % unifications to initialise its arguments?
     %
-    % The assertion will be in a form similar to
+    % The assertion must be in a form similar to this
     %
     %   all [L,H,T] ( L = [H | T] <=> append([H], T, L) )
     %
+    % for this predicate to return true.
+    %
 :- pred is_construction_equivalence_assertion(module_info::in, assert_id::in,
     cons_id::in, pred_id::in) is semidet.
+:- pred is_construction_equivalence_assertion_goal(hlds_goal::in,
+    cons_id::in, pred_id::in) is semidet.
 
-    % Place a hlds_goal into a standard form.  Currently all the
-    % code does is replace conj([G]) with G.
+    % Place a hlds_goal into a standard form. Currently all the code does
+    % is replace conj([G]) with G.
     %
 :- pred normalise_goal(hlds_goal::in, hlds_goal::out) is det.
 
@@ -136,12 +179,12 @@
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_clauses.
 :- import_module mdbcomp.
-:- import_module mdbcomp.prim_data.
-:- import_module parse_tree.set_of_var.
+:- import_module mdbcomp.sym_name.
 
 :- import_module assoc_list.
 :- import_module map.
 :- import_module maybe.
+:- import_module pair.
 :- import_module require.
 :- import_module set.
 :- import_module solutions.
@@ -151,8 +194,60 @@
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-is_commutativity_assertion(Module, AssertId, CallVars, CommutativeVars) :-
-    assert_id_goal(Module, AssertId, Goal),
+assert_id_goal(ModuleInfo, AssertId, Goal) :-
+    module_info_get_assertion_table(ModuleInfo, AssertTable),
+    assertion_table_lookup(AssertTable, AssertId, PredId),
+    module_info_pred_info(ModuleInfo, PredId, PredInfo),
+    pred_info_get_clauses_info(PredInfo, ClausesInfo),
+    clauses_info_get_clauses_rep(ClausesInfo, ClausesRep, _ItemNumbers),
+    get_clause_list_maybe_repeated(ClausesRep, Clauses),
+    (
+        Clauses = [Clause],
+        Goal0 = Clause ^ clause_body,
+        normalise_goal(Goal0, Goal)
+    ;
+        ( Clauses = []
+        ; Clauses = [_, _ | _]
+        ),
+        unexpected($pred, "goal is not an assertion")
+    ).
+
+%-----------------------------------------------------------------------------%
+%-----------------------------------------------------------------------------%
+
+record_preds_used_in(Goal, AssertId, !ModuleInfo) :-
+    pred_ids_called_from_goal(Goal, CalleePredIds),
+    % Sanity check.
+    ( if list.member(invalid_pred_id, CalleePredIds) then
+        unexpected($pred, "invalid pred_id")
+    else
+        true
+    ),
+    list.foldl(record_use_in_assertion(AssertId), CalleePredIds, !ModuleInfo).
+
+    % record_use_in_assertion(AssertId, PredId, !ModuleInfo):
+    %
+    % Record in PredId's pred_info that that predicate is used
+    % in assertion AssertId.
+    %
+:- pred record_use_in_assertion(assert_id::in, pred_id::in,
+    module_info::in, module_info::out) is det.
+
+record_use_in_assertion(AssertId, PredId, !ModuleInfo) :-
+    module_info_pred_info(!.ModuleInfo, PredId, PredInfo0),
+    pred_info_get_assertions(PredInfo0, Assertions0),
+    set.insert(AssertId, Assertions0, Assertions),
+    pred_info_set_assertions(Assertions, PredInfo0, PredInfo),
+    module_info_set_pred_info(PredId, PredInfo, !ModuleInfo).
+
+%-----------------------------------------------------------------------------%
+%-----------------------------------------------------------------------------%
+
+is_commutativity_assertion(ModuleInfo, AssertId, CallVars, CommutativeVars) :-
+    assert_id_goal(ModuleInfo, AssertId, Goal),
+    is_commutativity_assertion_goal(Goal, CallVars, CommutativeVars).
+
+is_commutativity_assertion_goal(Goal, CallVars, CommutativeVars) :-
     goal_is_equivalence(Goal, P, Q),
     P = hlds_goal(plain_call(PredId, _, VarsP, _, _, _), _),
     Q = hlds_goal(plain_call(PredId, _, VarsQ, _, _, _), _),
@@ -162,30 +257,31 @@ is_commutativity_assertion(Module, AssertId, CallVars, CommutativeVars) :-
     %
     % Check that the two list of variables are identical except that
     % the position of two variables has been swapped.
-    % e.g [A,B,C] and [B,A,C] is true.
+    % For example, commutative_var_ordering is true for [A,B,C] and [B,A,C].
     % It also takes a list of variables, Vs, to a call and returns
-    % the two variables in that list that can be swapped, ie [A,B].
+    % the two variables in that list that can be swapped. In this case,
+    % that will be [A,B].
     %
 :- pred commutative_var_ordering(list(prog_var)::in, list(prog_var)::in,
-    list(prog_var)::in, pair(prog_var)::out) is semidet.
+    list(prog_var)::in, set_of_progvar::out) is semidet.
 
 commutative_var_ordering([P | Ps], [Q | Qs], [V | Vs], CommutativeVars) :-
-    ( P = Q ->
+    ( if P = Q then
         commutative_var_ordering(Ps, Qs, Vs, CommutativeVars)
-    ;
+    else
         commutative_var_ordering_2(P, Q, Ps, Qs, Vs, CallVarB),
-        CommutativeVars = V - CallVarB
+        CommutativeVars = set_of_var.list_to_set([V, CallVarB])
     ).
 
 :- pred commutative_var_ordering_2(prog_var::in, prog_var::in,
-    list(prog_var)::in, list(prog_var)::in, list(prog_var)::in,
-    prog_var::out) is semidet.
+    list(prog_var)::in, list(prog_var)::in,
+    list(prog_var)::in, prog_var::out) is semidet.
 
-commutative_var_ordering_2(VarP, VarQ, [P | Ps], [Q | Qs], [V | Vs],
-        CallVarB) :-
-    ( P = Q ->
+commutative_var_ordering_2(VarP, VarQ, [P | Ps], [Q | Qs],
+        [V | Vs], CallVarB) :-
+    ( if P = Q then
         commutative_var_ordering_2(VarP, VarQ, Ps, Qs, Vs, CallVarB)
-    ;
+    else
         CallVarB = V,
         P = VarQ,
         Q = VarP,
@@ -195,18 +291,22 @@ commutative_var_ordering_2(VarP, VarQ, [P | Ps], [Q | Qs], [V | Vs],
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-is_associativity_assertion(Module, AssertId, CallVars,
-        AssociativeVars, OutputVar) :-
-    assert_id_goal(Module, AssertId, hlds_goal(GoalExpr, GoalInfo)),
-    goal_is_equivalence(hlds_goal(GoalExpr, GoalInfo), P, Q),
+is_associativity_assertion(ModuleInfo, AssertId, CallVars,
+        AssociativeVarsOutputVar) :-
+    assert_id_goal(ModuleInfo, AssertId, Goal),
+    is_associativity_assertion_goal(Goal, CallVars, AssociativeVarsOutputVar).
 
+is_associativity_assertion_goal(Goal, CallVars, AssociativeVarsOutputVar) :-
+    goal_is_equivalence(Goal, P, Q),
+
+    Goal = hlds_goal(_GoalExpr, GoalInfo),
     UniversiallyQuantifiedVars = goal_info_get_nonlocals(GoalInfo),
 
     get_conj_goals(P, PCalls),
     get_conj_goals(Q, QCalls),
-    promise_equivalent_solutions [AssociativeVars, OutputVar] (
+    promise_equivalent_solutions [AssociativeVarsOutputVar] (
         associative(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
-            AssociativeVars - OutputVar)
+            AssociativeVarsOutputVar)
     ).
 
     % associative(Ps, Qs, Us, R):
@@ -219,12 +319,12 @@ is_associativity_assertion(Module, AssertId, CallVars,
     %   compose( A, B,  AB),        compose(B,  C,  BC),
     %   compose(AB, C, ABC)     <=> compose(A, BC, ABC)
     %
-:- pred associative(hlds_goals::in, hlds_goals::in,
+:- pred associative(list(hlds_goal)::in, list(hlds_goal)::in,
     set_of_progvar::in, list(prog_var)::in,
-    pair(pair(prog_var), prog_var)::out) is cc_nondet.
+    associative_vars_output_var::out) is cc_nondet.
 
 associative(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
-        (CallVarA - CallVarB) - OutputVar) :-
+        AssociativeVarsOutputVar) :-
     reorder(PCalls, QCalls, LHSCalls, RHSCalls),
     process_one_side(LHSCalls, UniversiallyQuantifiedVars, PredId,
         AB, PairsL, Vs),
@@ -232,7 +332,7 @@ associative(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
         BC, PairsR, _),
 
     % If you read the predicate documentation, you will note that
-    % for each pair of variables on the left hand side there are an equivalent
+    % for each pair of variables on the left hand side, there is an equivalent
     % pair of variables on the right hand side. As the pairs of variables
     % are not symmetric, the call to list.perm will only succeed once,
     % if at all.
@@ -246,14 +346,18 @@ associative(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
     list.filter((pred(X - _Y::in) is semidet :- X = A),
         AssocList, [_A - CallVarA]),
     list.filter((pred(X - _Y::in) is semidet :- X = B),
-        AssocList, [_B - CallVarB]).
+        AssocList, [_B - CallVarB]),
+
+    AssociativeVars = set_of_var.list_to_set([CallVarA, CallVarB]),
+    AssociativeVarsOutputVar =
+        associative_vars_output_var(AssociativeVars, OutputVar).
 
     % reorder(Ps, Qs, Ls, Rs):
     %
     % Given both sides of the equivalence return another possible ordering.
     %
-:- pred reorder(hlds_goals::in, hlds_goals::in,
-    hlds_goals::out, hlds_goals::out) is multi.
+:- pred reorder(list(hlds_goal)::in, list(hlds_goal)::in,
+    list(hlds_goal)::out, list(hlds_goal)::out) is multi.
 
 reorder(PCalls, QCalls, LHSCalls, RHSCalls) :-
     list.perm(PCalls, LHSCalls),
@@ -273,7 +377,7 @@ reorder(PCalls, QCalls, LHSCalls, RHSCalls) :-
     % i.e. for app(TypeInfo, X, Y, XY), app(TypeInfo, XY, Z, XYZ)
     % L <= XY and Ps <= [X - XY, Y - Z, XY - XYZ]
     %
-:- pred process_one_side(hlds_goals::in, set_of_progvar::in, pred_id::out,
+:- pred process_one_side(list(hlds_goal)::in, set_of_progvar::in, pred_id::out,
     prog_var::out, assoc_list(prog_var)::out, list(prog_var)::out) is semidet.
 
 process_one_side(Goals, UniversiallyQuantifiedVars, PredId,
@@ -293,25 +397,31 @@ number_of_associative_vars = 3.
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-is_update_assertion(Module, AssertId, _PredId, CallVars, StateA - StateB) :-
-    assert_id_goal(Module, AssertId, hlds_goal(GoalExpr, GoalInfo)),
-    goal_is_equivalence(hlds_goal(GoalExpr, GoalInfo), P, Q),
+is_update_assertion(ModuleInfo, AssertId, PredId, CallVars, StateVars) :-
+    assert_id_goal(ModuleInfo, AssertId, Goal),
+    is_update_assertion_goal(Goal, PredId, CallVars, StateVars).
+
+is_update_assertion_goal(Goal, _PredId, CallVars, StateVars) :-
+    % XXX Ignoring _PredId looks like a bug.
+    goal_is_equivalence(Goal, P, Q),
+
+    Goal = hlds_goal(_GoalExpr, GoalInfo),
     UniversiallyQuantifiedVars = goal_info_get_nonlocals(GoalInfo),
 
     get_conj_goals(P, PCalls),
     get_conj_goals(Q, QCalls),
     solutions.solutions(
         update(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars),
-        [StateA - StateB | _]).
+        [StateVars | _]).
 
     %   compose(S0, A, SA),     compose(SB, A, S),
     %   compose(SA, B, S)   <=> compose(S0, B, SB)
     %
-:- pred update(hlds_goals::in, hlds_goals::in, set_of_progvar::in,
-    list(prog_var)::in, pair(prog_var)::out) is nondet.
+:- pred update(list(hlds_goal)::in, list(hlds_goal)::in, set_of_progvar::in,
+    list(prog_var)::in, state_update_vars::out) is nondet.
 
 update(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
-        StateA - StateB) :-
+        StateVars) :-
     reorder(PCalls, QCalls, LHSCalls, RHSCalls),
     process_two_linked_calls(LHSCalls, UniversiallyQuantifiedVars, PredId,
         SA, PairsL, Vs),
@@ -323,7 +433,7 @@ update(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
     list.length(Pairs) = 2,
 
     % If you read the predicate documentation, you will note that
-    % for each pair of variables on the left hand side there is an equivalent
+    % for each pair of variables on the left hand side, there is an equivalent
     % pair of variables on the right hand side. As the pairs of variables
     % are not symmetric, the call to list.perm will only succeed once,
     % if at all.
@@ -333,49 +443,56 @@ update(PCalls, QCalls, UniversiallyQuantifiedVars, CallVars,
     list.filter((pred(X - _Y::in) is semidet :- X = S0),
         AssocList, [_S0 - StateA]),
     list.filter((pred(X - _Y::in) is semidet :- X = SA),
-        AssocList, [_SA - StateB]).
+        AssocList, [_SA - StateB]),
+    StateVars = state_update_vars(StateA, StateB).
 
 %-----------------------------------------------------------------------------%
 
-    % process_two_linked_calls(Gs, UQVs, PId, LV, AL, VAs):
+    % process_two_linked_calls(Goals, UQVs, PredId, LinkingVar, AL, VAs):
     %
-    % is true iff the list of goals, Gs, with universally quantified
-    % variables, UQVs, is two calls to the same predicate, PId, with
-    % one variable that links them, LV.  AL will be the assoc list
-    % that is the each variable from the first call with its
-    % corresponding variable in the second call, and VAs are the
-    % variables of the first call.
+    % is true iff the list of goals Goals, with universally quantified
+    % variables, UQVs, is two calls to the same predicate, PredId, with
+    % one variable that links them, LinkingVar. AL will be an assoc list
+    % that pairs each variable from the first call with its corresponding
+    % variable in the second call, and VAs are the variables of the first call.
     %
-:- pred process_two_linked_calls(hlds_goals::in, set_of_progvar::in,
+    % XXX To me (zs), this looks like a strange way to return the arg vars.
+    %
+:- pred process_two_linked_calls(list(hlds_goal)::in, set_of_progvar::in,
     pred_id::out, prog_var::out, assoc_list(prog_var)::out,
     list(prog_var)::out) is semidet.
 
 process_two_linked_calls(Goals, UniversiallyQuantifiedVars, PredId,
-        LinkingVar, Vars, VarsA) :-
+        LinkingVar, VarsAB, VarsA) :-
     Goals = [
         hlds_goal(plain_call(PredId, _, VarsA, _, _, _), _),
         hlds_goal(plain_call(PredId, _, VarsB, _, _, _), _)
     ],
 
-    % Determine the linking variable, L. By definition it must be
-    % existentially quantified and a member of both variable lists.
-    CommonVars = list_to_set(VarsA) `intersect` list_to_set(VarsB),
+    % Determine the linking variable, L. By definition, it must be
+    % existentially quantified, and a member of both variable lists.
+    SetVarsA = set_of_var.list_to_set(VarsA),
+    SetVarsB = set_of_var.list_to_set(VarsB),
+    CommonVars = set_of_var.intersect(SetVarsA, SetVarsB),
     set_of_var.is_singleton(
         set_of_var.difference(CommonVars, UniversiallyQuantifiedVars),
         LinkingVar),
 
     % Set up mapping between the variables in the two calls.
-    assoc_list.from_corresponding_lists(VarsA, VarsB, Vars).
+    assoc_list.from_corresponding_lists(VarsA, VarsB, VarsAB).
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-is_construction_equivalence_assertion(Module, AssertId, ConsId, PredId) :-
-    assert_id_goal(Module, AssertId, Goal),
+is_construction_equivalence_assertion(ModuleInfo, AssertId, ConsId, PredId) :-
+    assert_id_goal(ModuleInfo, AssertId, Goal),
+    is_construction_equivalence_assertion_goal(Goal, ConsId, PredId).
+
+is_construction_equivalence_assertion_goal(Goal, ConsId, PredId) :-
     goal_is_equivalence(Goal, P, Q),
-    ( single_construction(P, ConsId) ->
+    ( if single_construction(P, ConsId) then
         predicate_call(Q, PredId)
-    ;
+    else
         single_construction(Q, ConsId),
         predicate_call(P, PredId)
     ).
@@ -394,7 +511,7 @@ single_construction(Goal, ConsId) :-
     % and would thus match even if the two functors are NOT of the same type.
     % Note that by insisting on cons, we effectively disallow assertions
     % about tuples.
-    match_sym_name(UnqualifiedSymName, QualifiedSymName).
+    partial_sym_name_matches_full(UnqualifiedSymName, QualifiedSymName).
 
     % The side containing the predicate call must be a single call
     % to the predicate with 0 or more construction unifications
@@ -403,18 +520,19 @@ single_construction(Goal, ConsId) :-
 :- pred predicate_call(hlds_goal::in, pred_id::in) is semidet.
 
 predicate_call(Goal, PredId) :-
-    ( Goal = hlds_goal(conj(plain_conj, Goals), _) ->
+    ( if Goal = hlds_goal(conj(plain_conj, Goals), _) then
         list.member(Call, Goals),
         Call = hlds_goal(plain_call(PredId, _, _, _, _, _), _),
         list.delete(Goals, Call, Unifications),
-        P = (pred(G::in) is semidet :-
-            not (
-                G = hlds_goal(unify(_, UnifyRhs, _, _, _), _),
-                UnifyRhs = rhs_functor(_, _, _)
-            )
-        ),
+        P =
+            ( pred(G::in) is semidet :-
+                not (
+                    G = hlds_goal(unify(_, UnifyRhs, _, _, _), _),
+                    UnifyRhs = rhs_functor(_, _, _)
+                )
+            ),
         list.filter(P, Unifications, [])
-    ;
+    else
         Goal = hlds_goal(plain_call(PredId, _, _, _, _, _), _)
     ).
 
@@ -431,39 +549,18 @@ get_conj_goals(Goal0, ConjList) :-
 
 ignore_exist_quant_scope(Goal0, Goal) :-
     Goal0 = hlds_goal(GoalExpr0, _Context),
-    (
+    ( if
         GoalExpr0 = scope(Reason, Goal1),
         Reason = exist_quant(_)
-    ->
+    then
         Goal = Goal1
-    ;
+    else
         Goal = Goal0
     ).
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-assert_id_goal(Module, AssertId, Goal) :-
-    module_info_get_assertion_table(Module, AssertTable),
-    assertion_table_lookup(AssertTable, AssertId, PredId),
-    module_info_pred_info(Module, PredId, PredInfo),
-    pred_info_get_clauses_info(PredInfo, ClausesInfo),
-    clauses_info_get_clauses_rep(ClausesInfo, ClausesRep, _ItemNumbers),
-    get_clause_list(ClausesRep, Clauses),
-    (
-        Clauses = [Clause],
-        Goal0 = Clause ^ clause_body,
-        normalise_goal(Goal0, Goal)
-    ;
-        ( Clauses = []
-        ; Clauses = [_, _ | _]
-        ),
-        unexpected($module, $pred, "goal is not an assertion")
-    ).
-
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
 :- pred goal_is_implication(hlds_goal::in, hlds_goal::out, hlds_goal::out)
     is semidet.
 
@@ -471,9 +568,9 @@ goal_is_implication(Goal, P, Q) :-
     % Goal = (P => Q)
     Goal = hlds_goal(negation(hlds_goal(conj(plain_conj, GoalList), _)), GI),
     list.reverse(GoalList) = [NotQ | Ps],
-    ( Ps = [P0] ->
+    ( if Ps = [P0] then
         P = P0
-    ;
+    else
         P = hlds_goal(conj(plain_conj, list.reverse(Ps)), GI)
     ),
     NotQ = hlds_goal(negation(Q), _).
@@ -495,7 +592,7 @@ goal_is_equivalence(Goal, P, Q) :-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-    % equal_goals(GA, GB):
+    % equal_goals(GA, GB, !Subst):
     %
     % Do these two goals represent the same hlds_goal modulo renaming?
     %
@@ -505,12 +602,7 @@ goal_is_equivalence(Goal, P, Q) :-
 equal_goals(GoalA, GoalB, !Subst) :-
     GoalA = hlds_goal(GoalExprA, _GoalInfoA),
     GoalB = hlds_goal(GoalExprB, _GoalInfoB),
-    equal_goal_exprs(GoalExprA, GoalExprB, !Subst).
-
-:- pred equal_goal_exprs(hlds_goal_expr::in, hlds_goal_expr::in,
-    subst::in, subst::out) is semidet.
-
-equal_goal_exprs(GoalExprA, GoalExprB, !Subst) :-
+    require_complete_switch [GoalExprA]
     (
         GoalExprA = conj(ConjType, GoalsA),
         GoalExprB = conj(ConjType, GoalsB),
@@ -575,13 +667,56 @@ equal_goal_exprs(GoalExprA, GoalExprB, !Subst) :-
 :- pred equal_reason(scope_reason::in, scope_reason::in, subst::in, subst::out)
     is semidet.
 
-equal_reason(exist_quant(VarsA), exist_quant(VarsB), !Subst) :-
-    equal_vars(VarsA, VarsB, !Subst).
-equal_reason(barrier(Removable), barrier(Removable), !Subst).
-equal_reason(commit(ForcePruning), commit(ForcePruning), !Subst).
-equal_reason(from_ground_term(VarA, Kind), from_ground_term(VarB, Kind),
-        !Subst) :-
-    equal_var(VarA, VarB, !Subst).
+equal_reason(ReasonA, ReasonB, !Subst) :-
+    require_complete_switch [ReasonA]
+    (
+        ReasonA = exist_quant(VarsA),
+        ReasonB = exist_quant(VarsB),
+        equal_vars(VarsA, VarsB, !Subst)
+    ;
+        ReasonA = disable_warnings(HeadWarning, TailWarnings),
+        ReasonB = disable_warnings(HeadWarning, TailWarnings)
+    ;
+        ReasonA = promise_solutions(VarsA, Kind),
+        ReasonB = promise_solutions(VarsB, Kind),
+        equal_vars(VarsA, VarsB, !Subst)
+    ;
+        ReasonA = promise_purity(Purity),
+        ReasonB = promise_purity(Purity)
+    ;
+        ReasonA = require_detism(Detism),
+        ReasonB = require_detism(Detism)
+    ;
+        ReasonA = require_complete_switch(VarA),
+        ReasonB = require_complete_switch(VarB),
+        equal_var(VarA, VarB, !Subst)
+    ;
+        ReasonA = require_switch_arms_detism(VarA, Detism),
+        ReasonB = require_switch_arms_detism(VarB, Detism),
+        equal_var(VarA, VarB, !Subst)
+    ;
+        ReasonA = commit(ForcePruning),
+        ReasonB = commit(ForcePruning)
+    ;
+        ReasonA = barrier(Removable),
+        ReasonB = barrier(Removable)
+    ;
+        ReasonA = from_ground_term(VarA, Kind),
+        ReasonB = from_ground_term(VarB, Kind),
+        equal_var(VarA, VarB, !Subst)
+    ;
+        ReasonA = trace_goal(_CompileTime, _Runtime, _MaybeIO,
+            _MutableVars, _QuantVars),
+        % We could match all the vars, but there would be no point;
+        % trace goals should not occur in assertions.
+        fail
+    ;
+        ReasonA = loop_control(_LCVar, _LCSVar, _UseParentStack),
+        % We could match the vars, but there would be no point.
+        % These scopes can never occur in user-written code;
+        % they can only added by the compiler itself.
+        fail
+    ).
 
 :- pred equal_goals_shorthand(shorthand_goal_expr::in, shorthand_goal_expr::in,
     subst::in, subst::out) is semidet.
@@ -596,9 +731,9 @@ equal_goals_shorthand(ShortHandA, ShortHandB, !Subst) :-
     is semidet.
 
 equal_var(VA, VB, !Subst) :-
-    ( map.search(!.Subst, VA, SubstVA) ->
+    ( if map.search(!.Subst, VA, SubstVA) then
         SubstVA = VB
-    ;
+    else
         map.insert(VA, VB, !Subst)
     ).
 
@@ -613,22 +748,29 @@ equal_vars([VA | VAs], [VB | VBs], !Subst) :-
 :- pred equal_unification(unify_rhs::in, unify_rhs::in, subst::in, subst::out)
     is semidet.
 
-equal_unification(rhs_var(A), rhs_var(B), !Subst) :-
-    equal_vars([A], [B], !Subst).
-equal_unification(rhs_functor(ConsId, E, VarsA), rhs_functor(ConsId, E, VarsB),
-        !Subst) :-
-    equal_vars(VarsA, VarsB, !Subst).
-equal_unification(LambdaGoalA, LambdaGoalB, !Subst) :-
-    LambdaGoalA = rhs_lambda_goal(Purity, Groundness, PredOrFunc, EvalMethod,
-        NLVarsA, LVarsA, Modes, Det, GoalA),
-    LambdaGoalB = rhs_lambda_goal(Purity, Groundness, PredOrFunc, EvalMethod,
-        NLVarsB, LVarsB, Modes, Det, GoalB),
-    equal_vars(NLVarsA, NLVarsB, !Subst),
-    equal_vars(LVarsA, LVarsB, !Subst),
-    equal_goals(GoalA, GoalB, !Subst).
+equal_unification(RhsA, RhsB, !Subst) :-
+    (
+        RhsA = rhs_var(A),
+        RhsB = rhs_var(B),
+        equal_var(A, B, !Subst)
+    ;
+        RhsA = rhs_functor(ConsId, E, VarsA),
+        RhsB = rhs_functor(ConsId, E, VarsB),
+        equal_vars(VarsA, VarsB, !Subst)
+    ;
+        RhsA = rhs_lambda_goal(Purity, Groundness, PredOrFunc, EvalMethod,
+            NonLocalVarsA, ArgVarsModesA, Det, GoalA),
+        RhsB = rhs_lambda_goal(Purity, Groundness, PredOrFunc, EvalMethod,
+            NonLocalVarsB, ArgVarsModesB, Det, GoalB),
+        assoc_list.keys(ArgVarsModesA, ArgVarsA),
+        assoc_list.keys(ArgVarsModesB, ArgVarsB),
+        equal_vars(NonLocalVarsA, NonLocalVarsB, !Subst),
+        equal_vars(ArgVarsA, ArgVarsB, !Subst),
+        equal_goals(GoalA, GoalB, !Subst)
+    ).
 
-:- pred equal_goals_list(hlds_goals::in, hlds_goals::in, subst::in, subst::out)
-    is semidet.
+:- pred equal_goals_list(list(hlds_goal)::in, list(hlds_goal)::in,
+    subst::in, subst::out) is semidet.
 
 equal_goals_list([], [], !Subst).
 equal_goals_list([GoalA | GoalAs], [GoalB | GoalBs], !Subst) :-
@@ -650,44 +792,8 @@ equal_goals_cases([CaseA | CaseAs], [CaseB | CaseBs], !Subst) :-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-record_preds_used_in(Goal, AssertId, !Module) :-
-    predids_from_goal(Goal, PredIds),
-    % Sanity check.
-    ( list.member(invalid_pred_id, PredIds) ->
-        unexpected($module, $pred, "invalid pred_id")
-    ;
-        true
-    ),
-    list.foldl(update_pred_info(AssertId), PredIds, !Module).
-
-%-----------------------------------------------------------------------------%
-
-    % update_pred_info(Id, A, !Module):
-    %
-    % Record in the pred_info pointed to by Id that that predicate
-    % is used in the assertion pointed to by A.
-    %
-:- pred update_pred_info(assert_id::in, pred_id::in,
-    module_info::in, module_info::out) is det.
-
-update_pred_info(AssertId, PredId, !Module) :-
-    module_info_pred_info(!.Module, PredId, PredInfo0),
-    pred_info_get_assertions(PredInfo0, Assertions0),
-    set.insert(AssertId, Assertions0, Assertions),
-    pred_info_set_assertions(Assertions, PredInfo0, PredInfo),
-    module_info_set_pred_info(PredId, PredInfo, !Module).
-
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
 normalise_goal(Goal0, Goal) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo),
-    normalise_goal_expr(GoalExpr0, GoalExpr),
-    Goal = hlds_goal(GoalExpr, GoalInfo).
-
-:- pred normalise_goal_expr(hlds_goal_expr::in, hlds_goal_expr::out) is det.
-
-normalise_goal_expr(GoalExpr0, GoalExpr) :-
     (
         ( GoalExpr0 = plain_call(_, _, _, _, _, _)
         ; GoalExpr0 = generic_call(_, _, _, _, _)
@@ -696,23 +802,28 @@ normalise_goal_expr(GoalExpr0, GoalExpr) :-
         ),
         GoalExpr = GoalExpr0
     ;
-        GoalExpr0 = conj(ConjType, Goals0),
+        GoalExpr0 = conj(ConjType, SubGoals0),
         (
             ConjType = plain_conj,
-            normalise_conj(Goals0, Goals)
+            normalise_conj(SubGoals0, SubGoals)
         ;
             ConjType = parallel_conj,
-            normalise_goals(Goals0, Goals)
+            normalise_goals(SubGoals0, SubGoals)
         ),
-        GoalExpr = conj(ConjType, Goals)
+        ( if SubGoals = [SubGoal] then
+            SubGoal = hlds_goal(SubGoalExpr, _),
+            GoalExpr = SubGoalExpr
+        else
+            GoalExpr = conj(ConjType, SubGoals)
+        )
     ;
         GoalExpr0 = switch(Var, CanFail, Cases0),
         normalise_cases(Cases0, Cases),
         GoalExpr = switch(Var, CanFail, Cases)
     ;
-        GoalExpr0 = disj(Goals0),
-        normalise_goals(Goals0, Goals),
-        GoalExpr = disj(Goals)
+        GoalExpr0 = disj(SubGoals0),
+        normalise_goals(SubGoals0, SubGoals),
+        GoalExpr = disj(SubGoals)
     ;
         GoalExpr0 = negation(SubGoal0),
         normalise_goal(SubGoal0, SubGoal),
@@ -747,11 +858,12 @@ normalise_goal_expr(GoalExpr0, GoalExpr) :-
             ShortHand = bi_implication(LHS, RHS)
         ),
         GoalExpr = shorthand(ShortHand)
-    ).
+    ),
+    Goal = hlds_goal(GoalExpr, GoalInfo).
 
 %-----------------------------------------------------------------------------%
 
-:- pred normalise_conj(hlds_goals::in, hlds_goals::out) is det.
+:- pred normalise_conj(list(hlds_goal)::in, list(hlds_goal)::out) is det.
 
 normalise_conj([], []).
 normalise_conj([Goal0 | Goals0], Goals) :-
@@ -768,7 +880,7 @@ normalise_cases([Case0 | Cases0], [Case | Cases]) :-
     Case = case(MainConsId, OtherConsIds, Goal),
     normalise_cases(Cases0, Cases).
 
-:- pred normalise_goals(hlds_goals::in, hlds_goals::out) is det.
+:- pred normalise_goals(list(hlds_goal)::in, list(hlds_goal)::out) is det.
 
 normalise_goals([], []).
 normalise_goals([Goal0 | Goals0], [Goal | Goals]) :-
@@ -776,3 +888,5 @@ normalise_goals([Goal0 | Goals0], [Goal | Goals]) :-
     normalise_goals(Goals0, Goals).
 
 %-----------------------------------------------------------------------------%
+:- end_module hlds.assertion.
+%-----------------------------------------------------------------------------%
