@@ -14,14 +14,11 @@
 :- module backend_libs.
 :- interface.
 
-% modules that provide functionality used by several different back-ends
-:- include_module arg_pack.
 :- include_module base_typeclass_info.
 :- include_module builtin_ops.
 :- include_module bytecode_data.
 :- include_module c_util.
 :- include_module compile_target_code.
-:- include_module erlang_rtti.
 :- include_module export.
 :- include_module foreign.
 :- include_module interval.
@@ -30,6 +27,7 @@
 :- include_module proc_label.
 :- include_module pseudo_type_info.
 :- include_module rtti.
+:- include_module string_encoding.
 :- include_module switch_util.
 :- include_module type_ctor_info.
 :- include_module type_class_info.
