@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2004-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -188,7 +189,7 @@
 
 :- implementation.
 
-:- import_module check_hlds.inst_match.
+:- import_module check_hlds.inst_test.
 :- import_module check_hlds.mode_util.
 :- import_module hlds.hlds_args.
 :- import_module hlds.hlds_clauses.
@@ -314,13 +315,13 @@ add_mc_vars_for_goal(PredId, ProgVarset, Goal, !VarInfo) :-
         GoalExpr = shorthand(ShortHand),
         (
             ShortHand = atomic_goal(_, _, _, _, _, _, _),
-            sorry($module, $pred, "NYI: atomic_goal")
+            sorry($pred, "NYI: atomic_goal")
         ;
             ShortHand = bi_implication(_, _),
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         ;
             ShortHand = try_goal(_, _, _),
-            sorry($module, $pred, "try_goal")
+            sorry($pred, "try_goal")
         )
     ;
         ( GoalExpr = plain_call(_, _, _, _, _, _)
@@ -337,7 +338,7 @@ add_clauses_constraints(ModuleInfo, PredId, PredInfo, !VarInfo,
     pred_info_get_clauses_info(PredInfo, ClausesInfo),
     clauses_info_get_headvars(ClausesInfo, HeadVars),
     clauses_info_get_clauses_rep(ClausesInfo, ClausesRep, _ItemNumbers),
-    get_clause_list(ClausesRep, Clauses),
+    get_clause_list_maybe_repeated(ClausesRep, Clauses),
     clauses_info_get_varset(ClausesInfo, ProgVarset),
 
     (
@@ -433,14 +434,14 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
         ;
             ConjType = parallel_conj,
             % XXX Need to do something here.
-            sorry($module, $pred, "par_conj")
+            sorry($pred, "par_conj")
         )
     ;
         GoalExpr = plain_call(CalleePredId, _, Args, _, _, _),
         module_info_pred_info(ModuleInfo, CalleePredId, CalleePredInfo),
         % The predicate we are in now is the caller.
         CallerPredId = PredId,
-        ( pred_info_infer_modes(CalleePredInfo) ->
+        ( if pred_info_infer_modes(CalleePredInfo) then
             % No modes declared so just constrain the hearvars
             pred_info_get_clauses_info(CalleePredInfo, CalleeClausesInfo),
             clauses_info_get_headvar_list(CalleeClausesInfo, CalleeHeadVars),
@@ -448,9 +449,9 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
             add_call_headvar_constraints(ProgVarset, Context, GoalId,
                 CallerPredId, Args, CalleePredId, CalleeHeadVars,
                 !VarInfo, !Constraints)
-        ;
+        else
             % At least one declared mode
-            pred_info_get_procedures(CalleePredInfo, CalleeProcTable),
+            pred_info_get_proc_table(CalleePredInfo, CalleeProcTable),
             map.values(CalleeProcTable, CalleeProcInfos),
             list.map(proc_info_get_argmodes, CalleeProcInfos,
                 CalleeArgModeDecls),
@@ -464,23 +465,33 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
         (
             % XXX Need to do something here.
             Details = higher_order(_, _, _, _),
-            sorry($module, $pred, "higher_order generic_call")
+            sorry($pred, "higher_order generic_call")
         ;
             % XXX Need to do something here.
             Details = class_method(_, _, _, _),
-            sorry($module, $pred, "class_method generic_call")
+            sorry($pred, "class_method generic_call")
         ;
             % XXX We need to impose the constraint that all the argument
             % variables are bound elsewhere.
             Details = event_call(_),
-            sorry($module, $pred, "event_call generic_call")
+            sorry($pred, "event_call generic_call")
         ;
-            % No mode constraints
-            Details = cast(_)
+            Details = cast(CastType),
+            (
+                ( CastType = unsafe_type_cast
+                ; CastType = unsafe_type_inst_cast
+                ; CastType = equiv_type_cast
+                ; CastType = exists_cast
+                )
+                % No mode constraints.
+            ;
+                CastType = subtype_coerce,
+                sorry($pred, "subtype_coerce generic_call")
+            )
         )
     ;
         GoalExpr = switch(_, _, _),
-        unexpected($module, $pred, "switch")
+        unexpected($pred, "switch")
     ;
         GoalExpr = unify(LHSvar, RHS, _Mode, _Kind, _UnifyContext),
         prog_var_at_path(ProgVarset, PredId, LHSvar, GoalId,
@@ -520,8 +531,8 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
                 % - it could be a test, so no constraints.
             )
         ;
-            RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _, _),
-            sorry($module, $pred, "unify with lambda goal")
+            RHS = rhs_lambda_goal(_, _, _, _, _, _, _, _),
+            sorry($pred, "unify with lambda goal")
         )
     ;
         GoalExpr = disj(Goals),
@@ -646,15 +657,15 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
         CallArgs = list.map(foreign_arg_var, ForeignArgs),
         module_info_pred_proc_info(ModuleInfo, CalledPred, ProcId, _,
             ProcInfo),
-        ( proc_info_get_maybe_declared_argmodes(ProcInfo, yes(_OrigDecl)) ->
+        ( if proc_info_get_maybe_declared_argmodes(ProcInfo, yes(_OrigDecl)) then
             proc_info_get_argmodes(ProcInfo, Decl),
 
             % This pred should strip the disj(conj()) for the single
             % declaration.
             add_call_mode_decls_constraints(ModuleInfo, ProgVarset, Context,
                 PredId, [Decl], GoalId, CallArgs, !VarInfo, !Constraints)
-        ;
-            unexpected($module, $pred, "no mode declaration for foreign proc")
+        else
+            unexpected($pred, "no mode declaration for foreign proc")
         )
     ;
         GoalExpr = shorthand(Shorthand),
@@ -666,14 +677,14 @@ add_goal_expr_constraints(ModuleInfo, ProgVarset, PredId, GoalExpr,
             % - InnerUO should definitely be produced inside this goal,
             %   by the main goal and each orelse goal
             % - OuterUO is definitely produced by this goal
-            sorry($module, $pred, "NYI: atomic_goal")
+            sorry($pred, "NYI: atomic_goal")
         ;
             Shorthand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "shorthand goal")
+            unexpected($pred, "shorthand goal")
         ;
             Shorthand = try_goal(_, _, _),
-            sorry($module, $pred, "NYI: try_goal")
+            sorry($pred, "NYI: try_goal")
         )
     ).
 
@@ -694,9 +705,9 @@ mode_decls_constraints(ModuleInfo, VarMap, PredId, Decls, HeadVarsList,
         mode_decl_constraints(ModuleInfo), HeadVarsMCVars, Decls),
 
     Constraints0 = list.condense(ConstraintsList),
-    ( Constraints0 = [mc_conj(OneModeOnlyConstraints)] ->
+    ( if Constraints0 = [mc_conj(OneModeOnlyConstraints)] then
         Constraints = OneModeOnlyConstraints
-    ;
+    else
         Constraints = [mc_disj(Constraints0)]
     ).
 
@@ -739,8 +750,8 @@ add_mode_decl_constraints(ModuleInfo, PredId, ProcId, Decl, Args,
     % mode. If, according to the mode declaration, a variable is not
     % initially free then it cannot be produced by a call to this
     % goal; If a variable is initially free and becomes not free
-    % then it is produced by the predicate. Otherwise it is free ->
-    % free and is not produced.
+    % then it is produced by the predicate. Otherwise it is free -> free
+    % and is not produced.
     %
 :- pred add_call_mode_decls_constraints(module_info::in,
     prog_varset::in, prog_context::in, pred_id::in, list(list(mer_mode))::in,
@@ -754,10 +765,10 @@ add_call_mode_decls_constraints(ModuleInfo, ProgVarset, CallContext,
     ModeSpecificConstraints = list.condense(list.map(
         mode_decl_constraints(ModuleInfo, CallArgsHere),
         Decls)),
-    ( ModeSpecificConstraints = [mc_conj(OneModeOnlyConstraints)] ->
+    ( if ModeSpecificConstraints = [mc_conj(OneModeOnlyConstraints)] then
         list.foldl(add_constraint(!.VarInfo ^ mc_varset, CallContext),
             OneModeOnlyConstraints, !Constraints)
-    ;
+    else
         add_constraint(!.VarInfo ^ mc_varset, CallContext,
             mc_disj(ModeSpecificConstraints), !Constraints)
     ).
@@ -817,17 +828,17 @@ mode_decl_constraints(ModuleInfo, ConstraintVars, ArgModes) =
 
 single_mode_constraints(ModuleInfo, MCVar, Mode) = Constraint :-
     mode_util.mode_get_insts(ModuleInfo, Mode, InitialInst, FinalInst),
-    (
+    ( if
         % Already produced?
-        not inst_match.inst_is_free(ModuleInfo, InitialInst)
-    ->
+        not inst_is_free(ModuleInfo, InitialInst)
+    then
         IsProduced = no     % Not produced here.
-    ;
+    else if
         % free -> non-free
-        not inst_match.inst_is_free(ModuleInfo, FinalInst)
-    ->
+        not inst_is_free(ModuleInfo, FinalInst)
+    then
         IsProduced = yes    % Produced here.
-    ;
+    else
         % free -> free
         IsProduced = no     % Not produced here.
     ),
@@ -1000,12 +1011,11 @@ prog_var_at_paths(ProgVarset, PredID, ProgVar, GoalIds, MCVars, !VarInfo) :-
     % ProgVarset).
     %
 :- pred prog_vars_at_path(prog_varset::in, pred_id::in, list(prog_var)::in,
-    goal_id::in, list(mc_var)::out, mc_var_info::in, mc_var_info::out)
-    is det.
+    goal_id::in, list(mc_var)::out, mc_var_info::in, mc_var_info::out) is det.
 
 prog_vars_at_path(ProgVarset, PredId, ProgVars, GoalId, MCVars, !VarInfo) :-
     list.map_foldl(
-        (pred(ProgVar::in, MCVar::out, !.VarInfo::in, !:VarInfo::out) is det :-
+        ( pred(ProgVar::in, MCVar::out, !.VarInfo::in, !:VarInfo::out) is det :-
             prog_var_at_path(ProgVarset, PredId, ProgVar, GoalId, MCVar,
                 !VarInfo)
         ), ProgVars, MCVars, !VarInfo).
@@ -1025,9 +1035,9 @@ prog_vars_at_path(ProgVarset, PredId, ProgVars, GoalId, MCVars, !VarInfo) :-
 ensure_prog_var_at_path(ProgVarset, PredId, GoalId, ProgVar,
         !Varset, !VarMap) :-
     RepVar = (ProgVar `in` PredId) `at` GoalId,
-    ( bimap.search(!.VarMap, RepVar, _) ->
+    ( if bimap.search(!.VarMap, RepVar, _) then
         true
-    ;
+    else
         MCVarName = rep_var_to_string(ProgVarset, RepVar),
         varset.new_named_var(MCVarName, NewMCVar, !Varset),
         bimap.det_insert(RepVar, NewMCVar, !VarMap)
@@ -1056,22 +1066,6 @@ cons_prog_var_at_path(VarMap, PredId, GoalId, ProgVar, MCVars0, MCVars) :-
     MCVar = lookup_prog_var_at_path(VarMap, PredId, GoalId, ProgVar),
     MCVars = [MCVar | MCVars0].
 
-    % prog_var_at_paths(VarMap, GoalIds, ProgVar) = ConstraintVars
-    % consults the map to form a list of the constraint variable
-    % propositions that ProgVar is produced at each of the paths in
-    % GoalIds respectively.  The lookup function will report an
-    % error if the key (ProgVar `in` PredId) `at` GoalId does not
-    % exist in the map for any of the goal ids in GoalIds.
-    %
-:- func prog_var_at_paths(mc_var_map, pred_id, list(goal_id), prog_var) =
-    list(mc_var).
-
-prog_var_at_paths(VarMap, PredId, GoalIds, ProgVar) =
-    list.map(
-        func(GoalId) =
-            lookup_prog_var_at_path(VarMap, PredId, GoalId, ProgVar),
-        GoalIds).
-
     % nonlocals_at_path_and_subpaths(ProgVarset, GoalId, SubIds,
     %   Nonlocals, NonlocalsAtId, NonlocalsAtSubIds, !VarInfo)
     %
@@ -1093,7 +1087,7 @@ nonlocals_at_path_and_subpaths(ProgVarset, PredId, GoalId, SubIds,
     prog_vars_at_path(ProgVarset, PredId, NonlocalsList, GoalId,
         NonlocalsAtId, !VarInfo),
     list.map_foldl(
-        (pred(Nl::in, NlAtSubIds::out, !.VInfo::in, !:VInfo::out) is det :-
+        ( pred(Nl::in, NlAtSubIds::out, !.VInfo::in, !:VInfo::out) is det :-
             prog_var_at_paths(ProgVarset, PredId, Nl, SubIds, NlAtSubIds,
                 !VInfo)
         ), NonlocalsList, NonlocalsAtSubIds, !VarInfo),
