@@ -1,7 +1,8 @@
 %---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
-% Copyright (C) 1996-2007, 2009-2011 The University of Melbourne.
+% Copyright (C) 1996-2007, 2009-2012 The University of Melbourne.
+% Copyright (C) 2014-2018 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %---------------------------------------------------------------------------%
@@ -9,7 +10,8 @@
 % File: bytecode.m.
 % Author: zs.
 %
-% This module defines the bytecode used by the debugger.
+% This module defines the bytecode used by a possible future
+% bytecode-based debugger.
 %
 %---------------------------------------------------------------------------%
 
@@ -18,10 +20,8 @@
 
 :- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
-:- import_module hlds.
-:- import_module hlds.hlds_data.
 :- import_module mdbcomp.
-:- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
 :- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
@@ -33,7 +33,7 @@
 
 %---------------------------------------------------------------------------%
 
-:- type byte_tree   ==  cord(byte_code).
+:- type byte_tree == cord(byte_code).
 
 :- type byte_code
     --->    byte_enter_pred(byte_pred_id, int, byte_is_func, int)
@@ -108,15 +108,24 @@
 
 :- type byte_cons_tag
     --->    byte_no_tag
-    ;       byte_unshared_tag(tag_bits)
-    ;       byte_shared_remote_tag(tag_bits, int)
-    ;       byte_shared_local_tag(tag_bits, int)
+    ;       byte_unshared_tag(int)
+    ;       byte_shared_remote_tag(int, int)
+    ;       byte_shared_local_tag(int, int)
     ;       byte_enum_tag(int).
 
 :- type byte_arg
     --->    byte_arg_var(byte_var)
     ;       byte_arg_int_const(int)
-    ;       byte_arg_float_const(float).
+    ;       byte_arg_float_const(float)
+    ;       byte_arg_uint_const(uint)
+    ;       byte_arg_int8_const(int8)
+    ;       byte_arg_uint8_const(uint8)
+    ;       byte_arg_int16_const(int16)
+    ;       byte_arg_uint16_const(uint16)
+    ;       byte_arg_int32_const(int32)
+    ;       byte_arg_uint32_const(uint32)
+    ;       byte_arg_int64_const(int64)
+    ;       byte_arg_uint64_const(uint64).
 
 :- type byte_dir
     --->    to_arg
@@ -263,7 +272,7 @@ output_args(byte_enter_switch_arm(MainConsId, OtherConsIds, NextLabelId),
     output_cons_id(MainConsId, !IO),
     % The interpreter doesn't yet implement switch arms with more than one
     % function symbol.
-    expect(unify(OtherConsIds, []), $module, $pred, "OtherConsIds"),
+    expect(unify(OtherConsIds, []), $pred, "OtherConsIds"),
     output_label_id(NextLabelId, !IO).
 output_args(byte_endof_switch_arm(LabelId), !IO) :-
     output_label_id(LabelId, !IO).
@@ -549,22 +558,22 @@ debug_reg(byte_reg_r, N, !IO) :-
 :- pred output_is_func(byte_is_func::in, io::di, io::uo) is det.
 
 output_is_func(IsFunc, !IO) :-
-    ( ( IsFunc = 1 ; IsFunc = 0 ) ->
+    ( if ( IsFunc = 1 ; IsFunc = 0 ) then
         output_byte(IsFunc, !IO)
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "invalid predicate or function specified in bytecode")
     ).
 
 :- pred debug_is_func(byte_is_func::in, io::di, io::uo) is det.
 
 debug_is_func(IsFunc, !IO) :-
-    ( IsFunc = 1 ->
+    ( if IsFunc = 1 then
         debug_string("func", !IO)
-    ; IsFunc = 0 ->
+    else if IsFunc = 0 then
         debug_string("pred", !IO)
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "invalid predicate or function specifier in bytecode.")
     ).
 
@@ -593,6 +602,24 @@ output_arg(byte_arg_int_const(IntVal), !IO) :-
 output_arg(byte_arg_float_const(FloatVal), !IO) :-
     output_byte(2, !IO),
     output_float(FloatVal, !IO).
+output_arg(byte_arg_uint_const(_), _, _) :-
+    unexpected($pred, "NYI uint constants in bytecode").
+output_arg(byte_arg_int8_const(_), _, _) :-
+    unexpected($pred, "NYI int8 constants in bytecode").
+output_arg(byte_arg_uint8_const(_), _, _) :-
+    unexpected($pred, "NYI uint8 constants in bytecode").
+output_arg(byte_arg_int16_const(_), _, _) :-
+    unexpected($pred, "NYI int16 constants in bytecode").
+output_arg(byte_arg_uint16_const(_), _, _) :-
+    unexpected($pred, "NYI uint16 constants in bytecode").
+output_arg(byte_arg_int32_const(_), _, _) :-
+    unexpected($pred, "NYI int32 constants in bytecode").
+output_arg(byte_arg_uint32_const(_), _, _) :-
+    unexpected($pred, "NYI uint32 constants in bytecode").
+output_arg(byte_arg_int64_const(_), _, _) :-
+    unexpected($pred, "NYI int64 constants in bytecode").
+output_arg(byte_arg_uint64_const(_), _, _) :-
+    unexpected($pred, "NYI uint64 constants in bytecode").
 
 :- pred debug_arg(byte_arg::in, io::di, io::uo) is det.
 
@@ -602,6 +629,33 @@ debug_arg(byte_arg_var(Var), !IO) :-
 debug_arg(byte_arg_int_const(IntVal), !IO) :-
     debug_string("int", !IO),
     debug_int(IntVal, !IO).
+debug_arg(byte_arg_uint_const(UIntVal), !IO) :-
+    debug_string("uint", !IO),
+    debug_uint(UIntVal, !IO).
+debug_arg(byte_arg_int8_const(Int8Val), !IO) :-
+    debug_string("int8", !IO),
+    debug_int8(Int8Val, !IO).
+debug_arg(byte_arg_uint8_const(UInt8Val), !IO) :-
+    debug_string("uint8", !IO),
+    debug_uint8(UInt8Val, !IO).
+debug_arg(byte_arg_int16_const(Int16Val), !IO) :-
+    debug_string("int16", !IO),
+    debug_int16(Int16Val, !IO).
+debug_arg(byte_arg_uint16_const(UInt16Val), !IO) :-
+    debug_string("uint16", !IO),
+    debug_uint16(UInt16Val, !IO).
+debug_arg(byte_arg_int32_const(Int32Val), !IO) :-
+    debug_string("int32", !IO),
+    debug_int32(Int32Val, !IO).
+debug_arg(byte_arg_uint32_const(UInt32Val), !IO) :-
+    debug_string("uint32", !IO),
+    debug_uint32(UInt32Val, !IO).
+debug_arg(byte_arg_int64_const(Int64Val), !IO) :-
+    debug_string("int64", !IO),
+    debug_int64(Int64Val, !IO).
+debug_arg(byte_arg_uint64_const(UInt64Val), !IO) :-
+    debug_string("uint64", !IO),
+    debug_uint64(UInt64Val, !IO).
 debug_arg(byte_arg_float_const(FloatVal), !IO) :-
     debug_string("float", !IO),
     debug_float(FloatVal, !IO).
@@ -789,14 +843,14 @@ output_cons_id(byte_char_const(Char), !IO) :-
     output_byte(Byte, !IO).
     % XXX
 output_cons_id(byte_base_typeclass_info_const(_, _, _), !IO) :-
-    sorry($module, $pred, "bytecode for typeclass not yet implemented."),
+    sorry($pred, "bytecode for typeclass not yet implemented."),
     output_byte(8, !IO).
 output_cons_id(byte_type_info_cell_constructor, !IO) :-
-    sorry($module, $pred, "bytecode for type_info_cell_constructor " ++
+    sorry($pred, "bytecode for type_info_cell_constructor " ++
         "not yet implemented."),
     output_byte(9, !IO).
 output_cons_id(byte_typeclass_info_cell_constructor, !IO) :-
-    sorry($module, $pred, "bytecode for typeclass_info_cell_constructor " ++
+    sorry($pred, "bytecode for typeclass_info_cell_constructor " ++
         "not yet implemented."),
     output_byte(10, !IO).
 
@@ -1029,22 +1083,32 @@ determinism_debug(detism_cc_non,       "cc_nondet").
 determinism_debug(detism_erroneous,    "erroneous").
 determinism_debug(detism_failure,      "failure").
 
-:- pred binop_code(binary_op::in, int::out) is det.
+:- pred binop_code(binary_op, int).
+:- mode binop_code(in, out) is det.
+% :- mode binop_code(out, in) is semidet.
+% This mode would be a nice way to guard against accidentally assigning
+% the same bytecode value to two different operators. Unfortunately,
+% it would work only if we explicitly listed all possible integers
+% as arguments of offset_str_eq, or if we used an output inst as big as
+% this table.
 
-binop_code(int_add,                  0).
-binop_code(int_sub,                  1).
-binop_code(int_mul,                  2).
-binop_code(int_div,                  3).
-binop_code(int_mod,                  4).
-binop_code(unchecked_left_shift,     5).
-binop_code(unchecked_right_shift,    6).
-binop_code(bitwise_and,              7).
-binop_code(bitwise_or,               8).
-binop_code(bitwise_xor,              9).
+% If we ever complete the bytecode backend, then this representation
+% scheme should be simplified by putting related operations next
+% to each other.
+binop_code(int_add(int_type_int),    0).
+binop_code(int_sub(int_type_int),    1).
+binop_code(int_mul(int_type_int),    2).
+binop_code(int_div(int_type_int),    3).
+binop_code(int_mod(int_type_int),    4).
+binop_code(unchecked_left_shift(int_type_int, shift_by_int),  5).
+binop_code(unchecked_right_shift(int_type_int, shift_by_int), 6).
+binop_code(bitwise_and(int_type_int), 7).
+binop_code(bitwise_or(int_type_int),  8).
+binop_code(bitwise_xor(int_type_int), 9).
 binop_code(logical_and,             10).
 binop_code(logical_or,              11).
-binop_code(eq,                      12).
-binop_code(ne,                      13).
+binop_code(eq(int_type_int),        12).
+binop_code(ne(int_type_int),        13).
 binop_code(array_index(_Type),      14).
 binop_code(str_eq,                  15).
 binop_code(str_ne,                  16).
@@ -1052,14 +1116,14 @@ binop_code(str_lt,                  17).
 binop_code(str_gt,                  18).
 binop_code(str_le,                  19).
 binop_code(str_ge,                  20).
-binop_code(int_lt,                  21).
-binop_code(int_gt,                  22).
-binop_code(int_le,                  23).
-binop_code(int_ge,                  24).
-binop_code(float_plus,              25).
-binop_code(float_minus,             26).
-binop_code(float_times,             27).
-binop_code(float_divide,            28).
+binop_code(int_lt(int_type_int),    21).
+binop_code(int_gt(int_type_int),    22).
+binop_code(int_le(int_type_int),    23).
+binop_code(int_ge(int_type_int),    24).
+binop_code(float_add,               25).
+binop_code(float_sub,               26).
+binop_code(float_mul,               27).
+binop_code(float_div,               28).
 binop_code(float_eq,                29).
 binop_code(float_ne,                30).
 binop_code(float_lt,                31).
@@ -1071,25 +1135,202 @@ binop_code(unsigned_le,             36).
 binop_code(compound_eq,             37).
 binop_code(compound_lt,             38).
 binop_code(str_cmp,                 39).
-binop_code(float_word_bits,         40).
+% was: binop_code(float_word_bits,         40).
 binop_code(float_from_dword,        41).
+binop_code(pointer_equal_conservative, 42).
+binop_code(offset_str_eq(_),        43).
+binop_code(string_unsafe_index_code_unit, 44).
+binop_code(eq(int_type_uint),       45).
+binop_code(ne(int_type_uint),       46).
+binop_code(int_lt(int_type_uint),   47).
+binop_code(int_gt(int_type_uint),   48).
+binop_code(int_le(int_type_uint),   49).
+binop_code(int_ge(int_type_uint),   50).
+binop_code(int_add(int_type_uint),  51).
+binop_code(int_sub(int_type_uint),  52).
+binop_code(int_mul(int_type_uint),  53).
+binop_code(int_div(int_type_uint),                54).
+binop_code(int_mod(int_type_uint),                55).
+binop_code(bitwise_and(int_type_uint),  56).
+binop_code(bitwise_or(int_type_uint), 57).
+binop_code(bitwise_xor(int_type_uint), 58).
+binop_code(unchecked_left_shift(int_type_uint, shift_by_int), 59).
+binop_code(unchecked_right_shift(int_type_uint, shift_by_int), 60).
+binop_code(eq(int_type_int8),       61).
+binop_code(ne(int_type_int8),       62).
+binop_code(int_lt(int_type_int8),   63).
+binop_code(int_gt(int_type_int8),   64).
+binop_code(int_le(int_type_int8),   65).
+binop_code(int_ge(int_type_int8),   66).
+binop_code(int_add(int_type_int8),  67).
+binop_code(int_sub(int_type_int8),  68).
+binop_code(int_mul(int_type_int8),  69).
+binop_code(int_div(int_type_int8),  70).
+binop_code(int_mod(int_type_int8),  71).
+binop_code(bitwise_and(int_type_int8), 72).
+binop_code(bitwise_or(int_type_int8), 73).
+binop_code(bitwise_xor(int_type_int8), 74).
+binop_code(unchecked_left_shift(int_type_int8, shift_by_int), 75).
+binop_code(unchecked_right_shift(int_type_int8, shift_by_int), 76).
+binop_code(eq(int_type_uint8),       77).
+binop_code(ne(int_type_uint8),       78).
+binop_code(int_lt(int_type_uint8),   79).
+binop_code(int_gt(int_type_uint8),   80).
+binop_code(int_le(int_type_uint8),   81).
+binop_code(int_ge(int_type_uint8),   82).
+binop_code(int_add(int_type_uint8),  83).
+binop_code(int_sub(int_type_uint8),  84).
+binop_code(int_mul(int_type_uint8),  85).
+binop_code(int_div(int_type_uint8),  86).
+binop_code(int_mod(int_type_uint8),  87).
+binop_code(bitwise_and(int_type_uint8), 88).
+binop_code(bitwise_or(int_type_uint8),  89).
+binop_code(bitwise_xor(int_type_uint8), 90).
+binop_code(unchecked_left_shift(int_type_uint8, shift_by_int), 91).
+binop_code(unchecked_right_shift(int_type_uint8, shift_by_int), 92).
+binop_code(eq(int_type_int16),       93).
+binop_code(ne(int_type_int16),       94).
+binop_code(int_lt(int_type_int16),   95).
+binop_code(int_gt(int_type_int16),   96).
+binop_code(int_le(int_type_int16),   97).
+binop_code(int_ge(int_type_int16),   98).
+binop_code(int_add(int_type_int16),  99).
+binop_code(int_sub(int_type_int16), 100).
+binop_code(int_mul(int_type_int16), 101).
+binop_code(int_div(int_type_int16), 102).
+binop_code(int_mod(int_type_int16), 103).
+binop_code(bitwise_and(int_type_int16), 104).
+binop_code(bitwise_or(int_type_int16),  105).
+binop_code(bitwise_xor(int_type_int16), 106).
+binop_code(unchecked_left_shift(int_type_int16, shift_by_int),  107).
+binop_code(unchecked_right_shift(int_type_int16, shift_by_int), 108).
+binop_code(eq(int_type_uint16),       109).
+binop_code(ne(int_type_uint16),       110).
+binop_code(int_lt(int_type_uint16),   111).
+binop_code(int_gt(int_type_uint16),   112).
+binop_code(int_le(int_type_uint16),   113).
+binop_code(int_ge(int_type_uint16),   114).
+binop_code(int_add(int_type_uint16),  115).
+binop_code(int_sub(int_type_uint16),  116).
+binop_code(int_mul(int_type_uint16),  117).
+binop_code(int_div(int_type_uint16),  118).
+binop_code(int_mod(int_type_uint16),  119).
+binop_code(bitwise_and(int_type_uint16), 120).
+binop_code(bitwise_or(int_type_uint16),  121).
+binop_code(bitwise_xor(int_type_uint16), 122).
+binop_code(unchecked_left_shift(int_type_uint16, shift_by_int), 123).
+binop_code(unchecked_right_shift(int_type_uint16, shift_by_int), 124).
+binop_code(eq(int_type_int32),       125).
+binop_code(ne(int_type_int32),       126).
+binop_code(int_lt(int_type_int32),   127).
+binop_code(int_gt(int_type_int32),   128).
+binop_code(int_le(int_type_int32),   129).
+binop_code(int_ge(int_type_int32),   130).
+binop_code(int_add(int_type_int32),  131).
+binop_code(int_sub(int_type_int32),  132).
+binop_code(int_mul(int_type_int32),  133).
+binop_code(int_div(int_type_int32),  134).
+binop_code(int_mod(int_type_int32),                135).
+binop_code(bitwise_and(int_type_int32), 136).
+binop_code(bitwise_or(int_type_int32),  137).
+binop_code(bitwise_xor(int_type_int32), 138).
+binop_code(unchecked_left_shift(int_type_int32, shift_by_int), 139).
+binop_code(unchecked_right_shift(int_type_int32, shift_by_int), 140).
+binop_code(eq(int_type_uint32),       141).
+binop_code(ne(int_type_uint32),       142).
+binop_code(int_lt(int_type_uint32),   143).
+binop_code(int_gt(int_type_uint32),   144).
+binop_code(int_le(int_type_uint32),   145).
+binop_code(int_ge(int_type_uint32),   146).
+binop_code(int_add(int_type_uint32),  147).
+binop_code(int_sub(int_type_uint32),  148).
+binop_code(int_mul(int_type_uint32),  149).
+binop_code(int_div(int_type_uint32),  150).
+binop_code(int_mod(int_type_uint32),  151).
+binop_code(bitwise_and(int_type_uint32), 152).
+binop_code(bitwise_or(int_type_uint32),  153).
+binop_code(bitwise_xor(int_type_uint32), 154).
+binop_code(unchecked_left_shift(int_type_uint32, shift_by_int),  155).
+binop_code(unchecked_right_shift(int_type_uint32, shift_by_int), 156).
+binop_code(eq(int_type_int64),       157).
+binop_code(ne(int_type_int64),       158).
+binop_code(int_lt(int_type_int64),   159).
+binop_code(int_gt(int_type_int64),   160).
+binop_code(int_le(int_type_int64),   161).
+binop_code(int_ge(int_type_int64),   162).
+binop_code(int_add(int_type_int64),  163).
+binop_code(int_sub(int_type_int64),  164).
+binop_code(int_mul(int_type_int64),  165).
+binop_code(int_div(int_type_int64),  166).
+binop_code(int_mod(int_type_int64),  167).
+binop_code(bitwise_and(int_type_int64), 168).
+binop_code(bitwise_or(int_type_int64),  169).
+binop_code(bitwise_xor(int_type_int64), 170).
+binop_code(unchecked_left_shift(int_type_int64, shift_by_int), 171).
+binop_code(unchecked_right_shift(int_type_int64, shift_by_int), 172).
+binop_code(eq(int_type_uint64),       173).
+binop_code(ne(int_type_uint64),       174).
+binop_code(int_lt(int_type_uint64),   175).
+binop_code(int_gt(int_type_uint64),   176).
+binop_code(int_le(int_type_uint64),   177).
+binop_code(int_ge(int_type_uint64),   178).
+binop_code(int_add(int_type_uint64),  179).
+binop_code(int_sub(int_type_uint64),  180).
+binop_code(int_mul(int_type_uint64),  181).
+binop_code(int_div(int_type_uint64),  182).
+binop_code(int_mod(int_type_uint64),  183).
+binop_code(bitwise_and(int_type_uint64), 184).
+binop_code(bitwise_or(int_type_uint64),  185).
+binop_code(bitwise_xor(int_type_uint64), 186).
+binop_code(unchecked_left_shift(int_type_uint64, shift_by_int),  187).
+binop_code(unchecked_right_shift(int_type_uint64, shift_by_int), 188).
+binop_code(int64_from_dword,          189).
+binop_code(uint64_from_dword,         190).
+binop_code(unchecked_left_shift(int_type_int, shift_by_uint), 191).
+binop_code(unchecked_right_shift(int_type_int, shift_by_uint), 192).
+binop_code(unchecked_left_shift(int_type_uint, shift_by_uint), 193).
+binop_code(unchecked_right_shift(int_type_uint, shift_by_uint), 194).
+binop_code(unchecked_left_shift(int_type_int8, shift_by_uint), 195).
+binop_code(unchecked_right_shift(int_type_int8, shift_by_uint), 196).
+binop_code(unchecked_left_shift(int_type_uint8, shift_by_uint), 197).
+binop_code(unchecked_right_shift(int_type_uint8, shift_by_uint), 198).
+binop_code(unchecked_left_shift(int_type_int16, shift_by_uint), 199).
+binop_code(unchecked_right_shift(int_type_int16, shift_by_uint), 200).
+binop_code(unchecked_left_shift(int_type_uint16, shift_by_uint), 201).
+binop_code(unchecked_right_shift(int_type_uint16, shift_by_uint), 202).
+binop_code(unchecked_left_shift(int_type_int32, shift_by_uint), 203).
+binop_code(unchecked_right_shift(int_type_int32, shift_by_uint), 204).
+binop_code(unchecked_left_shift(int_type_uint32, shift_by_uint), 205).
+binop_code(unchecked_right_shift(int_type_uint32, shift_by_uint), 206).
+binop_code(unchecked_left_shift(int_type_int64, shift_by_uint), 207).
+binop_code(unchecked_right_shift(int_type_int64, shift_by_uint), 208).
+binop_code(unchecked_left_shift(int_type_uint64, shift_by_uint), 209).
+binop_code(unchecked_right_shift(int_type_uint64, shift_by_uint), 210).
+binop_code(unsigned_lt,               211).
 
-:- pred binop_debug(binary_op::in, string::out) is det.
+:- pred binop_debug(binary_op, string).
+:- mode binop_debug(in, out) is det.
+% :- mode binop_debug(out, in) is semidet.
+% This mode would be a nice way to guard against accidentally assigning
+% the same debug representation to two different operators. Unfortunately,
+% it would work only if we explicitly listed all possible integers
+% as arguments of offset_str_eq, or if we used an output inst as big as
+% this table.
 
-binop_debug(int_add,                "+").
-binop_debug(int_sub,                "-").
-binop_debug(int_mul,                "*").
-binop_debug(int_div,                "/").
-binop_debug(int_mod,                "mod").
-binop_debug(unchecked_left_shift,   "<<").
-binop_debug(unchecked_right_shift,  ">>").
-binop_debug(bitwise_and,            "&").
-binop_debug(bitwise_or,             "|").
-binop_debug(bitwise_xor,            "^").
+binop_debug(int_add(int_type_int),  "+").
+binop_debug(int_sub(int_type_int),  "-").
+binop_debug(int_mul(int_type_int),  "*").
+binop_debug(int_div(int_type_int),  "/").
+binop_debug(int_mod(int_type_int),  "mod").
+binop_debug(unchecked_left_shift(int_type_int, shift_by_int),   "<<").
+binop_debug(unchecked_right_shift(int_type_int, shift_by_int),  ">>").
+binop_debug(bitwise_and(int_type_int), "&").
+binop_debug(bitwise_or(int_type_int), "|").
+binop_debug(bitwise_xor(int_type_int), "^").
 binop_debug(logical_and,            "and").
 binop_debug(logical_or,             "or").
-binop_debug(eq,                     "eq").
-binop_debug(ne,                     "ne").
+binop_debug(eq(int_type_int),       "eq").
+binop_debug(ne(int_type_int),       "ne").
 binop_debug(array_index(_Type),     "array_index").
 binop_debug(str_eq,                 "str_eq").
 binop_debug(str_ne,                 "str_ne").
@@ -1097,14 +1338,14 @@ binop_debug(str_lt,                 "str_lt").
 binop_debug(str_gt,                 "str_gt").
 binop_debug(str_le,                 "str_le").
 binop_debug(str_ge,                 "str_ge").
-binop_debug(int_lt,                 "<").
-binop_debug(int_gt,                 ">").
-binop_debug(int_le,                 "<=").
-binop_debug(int_ge,                 ">=").
-binop_debug(float_plus,             "float_plus").
-binop_debug(float_minus,            "float_minus").
-binop_debug(float_times,            "float_times").
-binop_debug(float_divide,           "float_divide").
+binop_debug(int_lt(int_type_int),   "<").
+binop_debug(int_gt(int_type_int),   ">").
+binop_debug(int_le(int_type_int),   "<=").
+binop_debug(int_ge(int_type_int),   ">=").
+binop_debug(float_add,              "float_add").
+binop_debug(float_sub,              "float_sub").
+binop_debug(float_mul,              "float_mul").
+binop_debug(float_div,              "float_div").
 binop_debug(float_eq,               "float_eq").
 binop_debug(float_ne,               "float_ne").
 binop_debug(float_lt,               "float_lt").
@@ -1112,40 +1353,275 @@ binop_debug(float_gt,               "float_gt").
 binop_debug(float_le,               "float_le").
 binop_debug(float_ge,               "float_ge").
 binop_debug(body,                   "body").
+binop_debug(unsigned_lt,            "unsigned_lt").
 binop_debug(unsigned_le,            "unsigned_le").
 binop_debug(compound_eq,            "compound_eq").
 binop_debug(compound_lt,            "compound_lt").
 binop_debug(str_cmp,                "strcmp").
-binop_debug(float_word_bits,        "float_word_bits").
 binop_debug(float_from_dword,       "float_from_dword").
+binop_debug(int64_from_dword,       "int64_from_dword").
+binop_debug(uint64_from_dword,      "uint64_from_dword").
+binop_debug(pointer_equal_conservative, "pointer_equal_conservative").
+binop_debug(offset_str_eq(_),       "offset_str_eq").
+binop_debug(string_unsafe_index_code_unit, "string_unsafe_index_code_unit").
+binop_debug(eq(int_type_uint),      "==(uint)").
+binop_debug(ne(int_type_uint),      "!=(uint)").
+binop_debug(int_lt(int_type_uint),  "<(uint)").
+binop_debug(int_gt(int_type_uint),  ">(uint)").
+binop_debug(int_le(int_type_uint),  "<=(uint)").
+binop_debug(int_ge(int_type_uint),  ">=(uint)").
+binop_debug(int_add(int_type_uint), "+(uint)").
+binop_debug(int_sub(int_type_uint), "-(uint)").
+binop_debug(int_mul(int_type_uint), "*(uint)").
+binop_debug(int_div(int_type_uint), "/(uint)").
+binop_debug(int_mod(int_type_uint), "mod(uint)").
+binop_debug(bitwise_and(int_type_uint), "&(uint)").
+binop_debug(bitwise_or(int_type_uint), "|(uint)").
+binop_debug(bitwise_xor(int_type_uint), "^(uint)").
+binop_debug(unchecked_left_shift(int_type_uint, shift_by_int),
+    "<<(uint)").
+binop_debug(unchecked_right_shift(int_type_uint, shift_by_int),
+    ">>(uint)").
+binop_debug(eq(int_type_int8),      "==(int8)").
+binop_debug(ne(int_type_int8),      "!=(int8)").
+binop_debug(int_lt(int_type_int8),  "<(int8)").
+binop_debug(int_gt(int_type_int8),  ">(int8)").
+binop_debug(int_le(int_type_int8),  "<=(int8)").
+binop_debug(int_ge(int_type_int8),  ">=(int8)").
+binop_debug(int_add(int_type_int8), "+(int8)").
+binop_debug(int_sub(int_type_int8), "-(int8)").
+binop_debug(int_mul(int_type_int8), "*(int8)").
+binop_debug(int_div(int_type_int8), "/(int8)").
+binop_debug(int_mod(int_type_int8), "mod(int8)").
+binop_debug(bitwise_and(int_type_int8), "&(int8)").
+binop_debug(bitwise_or(int_type_int8), "|(int8)").
+binop_debug(bitwise_xor(int_type_int8), "^(int8)").
+binop_debug(unchecked_left_shift(int_type_int8, shift_by_int),
+    "<<(int8)").
+binop_debug(unchecked_right_shift(int_type_int8, shift_by_int),
+    ">>(int8)").
+binop_debug(eq(int_type_uint8),      "==(uint8)").
+binop_debug(ne(int_type_uint8),      "!=(uint8)").
+binop_debug(int_lt(int_type_uint8),  "<(uint8)").
+binop_debug(int_gt(int_type_uint8),  ">(uint8)").
+binop_debug(int_le(int_type_uint8),  "<=(uint8)").
+binop_debug(int_ge(int_type_uint8),  ">=(uint8)").
+binop_debug(int_add(int_type_uint8), "+(uint8)").
+binop_debug(int_sub(int_type_uint8), "-(uint8)").
+binop_debug(int_mul(int_type_uint8), "*(uint8)").
+binop_debug(int_div(int_type_uint8), "/(uint8)").
+binop_debug(int_mod(int_type_uint8), "mod(uint8)").
+binop_debug(bitwise_and(int_type_uint8), "&(uint8)").
+binop_debug(bitwise_or(int_type_uint8), "|(uint8)").
+binop_debug(bitwise_xor(int_type_uint8), "^(uint8)").
+binop_debug(unchecked_left_shift(int_type_uint8, shift_by_int),
+    "<<(uint8)").
+binop_debug(unchecked_right_shift(int_type_uint8, shift_by_int),
+    ">>(uint8)").
+binop_debug(eq(int_type_int16),      "==(int6)").
+binop_debug(ne(int_type_int16),      "!=(int16)").
+binop_debug(int_lt(int_type_int16),  "<(int16)").
+binop_debug(int_gt(int_type_int16),  ">(int16)").
+binop_debug(int_le(int_type_int16),  "<=(int16)").
+binop_debug(int_ge(int_type_int16),  ">=(int16)").
+binop_debug(int_add(int_type_int16), "+(int16)").
+binop_debug(int_sub(int_type_int16), "-(int16)").
+binop_debug(int_mul(int_type_int16), "*(int16)").
+binop_debug(int_div(int_type_int16), "/(int16)").
+binop_debug(int_mod(int_type_int16), "mod(int16)").
+binop_debug(bitwise_and(int_type_int16), "&(int16)").
+binop_debug(bitwise_or(int_type_int16), "|(int16)").
+binop_debug(bitwise_xor(int_type_int16), "^(int16)").
+binop_debug(unchecked_left_shift(int_type_int16, shift_by_int),
+    "<<(int16)").
+binop_debug(unchecked_right_shift(int_type_int16, shift_by_int),
+    ">>(int16)").
+binop_debug(eq(int_type_uint16),      "==(uint16)").
+binop_debug(ne(int_type_uint16),      "!=(uint16)").
+binop_debug(int_lt(int_type_uint16),  "<(uint16)").
+binop_debug(int_gt(int_type_uint16),  ">(uint16)").
+binop_debug(int_le(int_type_uint16),  "<=(uint16)").
+binop_debug(int_ge(int_type_uint16),  ">=(uint16)").
+binop_debug(int_add(int_type_uint16), "+(uint16)").
+binop_debug(int_sub(int_type_uint16), "-(uint16)").
+binop_debug(int_mul(int_type_uint16), "*(uint16)").
+binop_debug(int_div(int_type_uint16), "/(uint16)").
+binop_debug(int_mod(int_type_uint16), "mod(uint16)").
+binop_debug(bitwise_and(int_type_uint16), "&(uint16)").
+binop_debug(bitwise_or(int_type_uint16), "|(uint16)").
+binop_debug(bitwise_xor(int_type_uint16), "^(uint16)").
+binop_debug(unchecked_left_shift(int_type_uint16, shift_by_int),
+    "<<(uint16)").
+binop_debug(unchecked_right_shift(int_type_uint16, shift_by_int),
+    ">>(uint16)").
+binop_debug(eq(int_type_int32),      "==(int32)").
+binop_debug(ne(int_type_int32),      "!=(int32)").
+binop_debug(int_lt(int_type_int32),  "<(int32)").
+binop_debug(int_gt(int_type_int32),  ">(int32)").
+binop_debug(int_le(int_type_int32),  "<=(int32)").
+binop_debug(int_ge(int_type_int32),  ">=(int32)").
+binop_debug(int_add(int_type_int32), "+(int32)").
+binop_debug(int_sub(int_type_int32), "-(int32)").
+binop_debug(int_mul(int_type_int32), "*(int32)").
+binop_debug(int_div(int_type_int32), "/(int32)").
+binop_debug(int_mod(int_type_int32), "mod(int32)").
+binop_debug(bitwise_and(int_type_int32), "&(int32)").
+binop_debug(bitwise_or(int_type_int32), "|(int32)").
+binop_debug(bitwise_xor(int_type_int32), "^(int32)").
+binop_debug(unchecked_left_shift(int_type_int32, shift_by_int),
+    "<<(int32)").
+binop_debug(unchecked_right_shift(int_type_int32, shift_by_int),
+    ">>(int32)").
+binop_debug(eq(int_type_uint32),      "==(uint32)").
+binop_debug(ne(int_type_uint32),      "!=(uint32)").
+binop_debug(int_lt(int_type_uint32),  "<(uint32)").
+binop_debug(int_gt(int_type_uint32),  ">(uint32)").
+binop_debug(int_le(int_type_uint32),  "<=(uint32)").
+binop_debug(int_ge(int_type_uint32),  ">=(uint32)").
+binop_debug(int_add(int_type_uint32), "+(uint32)").
+binop_debug(int_sub(int_type_uint32), "-(uint32)").
+binop_debug(int_mul(int_type_uint32), "*(uint32)").
+binop_debug(int_div(int_type_uint32), "/(uint32)").
+binop_debug(int_mod(int_type_uint32), "mod(uint32").
+binop_debug(bitwise_and(int_type_uint32), "&(uint32)").
+binop_debug(bitwise_or(int_type_uint32),  "|(uint32)").
+binop_debug(bitwise_xor(int_type_uint32), "^(uint32)").
+binop_debug(unchecked_left_shift(int_type_uint32, shift_by_int),
+    "<<(uint32)").
+binop_debug(unchecked_right_shift(int_type_uint32, shift_by_int),
+    ">>(uint32)").
+binop_debug(eq(int_type_int64),      "==(int64)").
+binop_debug(ne(int_type_int64),      "!=(int64)").
+binop_debug(int_lt(int_type_int64),  "<(int64)").
+binop_debug(int_gt(int_type_int64),  ">(int64)").
+binop_debug(int_le(int_type_int64),  "<=(int64)").
+binop_debug(int_ge(int_type_int64),  ">=(int64)").
+binop_debug(int_add(int_type_int64), "+(int64)").
+binop_debug(int_sub(int_type_int64), "-(int64)").
+binop_debug(int_mul(int_type_int64), "*(int64)").
+binop_debug(int_div(int_type_int64), "/(int64)").
+binop_debug(int_mod(int_type_int64), "mod(int64)").
+binop_debug(bitwise_and(int_type_int64), "&(int64)").
+binop_debug(bitwise_or(int_type_int64), "|(int64)").
+binop_debug(bitwise_xor(int_type_int64), "^(int64)").
+binop_debug(unchecked_left_shift(int_type_int64, shift_by_int),
+    "<<(int64)").
+binop_debug(unchecked_right_shift(int_type_int64, shift_by_int),
+    ">>(int64)").
+binop_debug(eq(int_type_uint64),      "==(uint64)").
+binop_debug(ne(int_type_uint64),      "!=(uint64)").
+binop_debug(int_lt(int_type_uint64),  "<(uint64)").
+binop_debug(int_gt(int_type_uint64),  ">(uint64)").
+binop_debug(int_le(int_type_uint64),  "<=(uint64)").
+binop_debug(int_ge(int_type_uint64),  ">=(uint64)").
+binop_debug(int_add(int_type_uint64), "+(uint64)").
+binop_debug(int_sub(int_type_uint64), "-(uint64)").
+binop_debug(int_mul(int_type_uint64), "*(uint64)").
+binop_debug(int_div(int_type_uint64), "/(uint64)").
+binop_debug(int_mod(int_type_uint64), "mod(uint64").
+binop_debug(bitwise_and(int_type_uint64), "&(uint64)").
+binop_debug(bitwise_or(int_type_uint64),  "|(uint64)").
+binop_debug(bitwise_xor(int_type_uint64), "^(uint64)").
+binop_debug(unchecked_left_shift(int_type_uint64, shift_by_int), "<<(uint64)").
+binop_debug(unchecked_right_shift(int_type_uint64, shift_by_int), ">>(uint64)").
+binop_debug(unchecked_left_shift(int_type_int, shift_by_uint), "<<u").
+binop_debug(unchecked_right_shift(int_type_int, shift_by_uint), ">>u").
+binop_debug(unchecked_left_shift(int_type_uint, shift_by_uint), "<<u(uint)").
+binop_debug(unchecked_right_shift(int_type_uint, shift_by_uint), ">>u(uint)").
+binop_debug(unchecked_left_shift(int_type_int8, shift_by_uint), "<<u(int8)").
+binop_debug(unchecked_right_shift(int_type_int8, shift_by_uint), ">>u(int8)").
+binop_debug(unchecked_left_shift(int_type_uint8, shift_by_uint),
+    "<<u(uint8)").
+binop_debug(unchecked_right_shift(int_type_uint8, shift_by_uint),
+    ">>u(uint8)").
+binop_debug(unchecked_left_shift(int_type_int16, shift_by_uint),
+    "<<u(int16)").
+binop_debug(unchecked_right_shift(int_type_int16, shift_by_uint),
+    ">>u(int16)").
+binop_debug(unchecked_left_shift(int_type_uint16, shift_by_uint),
+    "<<u(uint16)").
+binop_debug(unchecked_right_shift(int_type_uint16, shift_by_uint),
+    ">>u(uint16)").
+binop_debug(unchecked_left_shift(int_type_int32, shift_by_uint),
+    "<<u(int32)").
+binop_debug(unchecked_right_shift(int_type_int32, shift_by_uint),
+    ">>u(int32)").
+binop_debug(unchecked_left_shift(int_type_uint32, shift_by_uint),
+    "<<u(uint32)").
+binop_debug(unchecked_right_shift(int_type_uint32, shift_by_uint),
+    ">>u(uint32)").
+binop_debug(unchecked_left_shift(int_type_int64, shift_by_uint),
+    "<<u(int64)").
+binop_debug(unchecked_right_shift(int_type_int64, shift_by_uint),
+    ">>u(int64)").
+binop_debug(unchecked_left_shift(int_type_uint64, shift_by_uint),
+    "<<u(uint64)").
+binop_debug(unchecked_right_shift(int_type_uint64, shift_by_uint),
+    ">>u(uint64)").
 
 :- pred unop_code(unary_op::in, int::out) is det.
 
-unop_code(mktag,                0).
+% unop_code(mktag,              0).
 unop_code(tag,                  1).
-unop_code(unmktag,              2).
+% unop_code(unmktag,            2).
 unop_code(mkbody,               3).
 unop_code(unmkbody,             4).
 unop_code(strip_tag,            5).
-unop_code(bitwise_complement,   6).
+unop_code(bitwise_complement(int_type_int), 6).
 unop_code(logical_not,          7).
 unop_code(hash_string,          8).
 unop_code(hash_string2,         9).
 unop_code(hash_string3,        10).
+unop_code(hash_string4,        11).
+unop_code(hash_string5,        12).
+unop_code(hash_string6,        13).
+unop_code(bitwise_complement(int_type_uint),    14).
+unop_code(bitwise_complement(int_type_int8),    15).
+unop_code(bitwise_complement(int_type_uint8),   16).
+unop_code(bitwise_complement(int_type_int16),   17).
+unop_code(bitwise_complement(int_type_uint16),  18).
+unop_code(bitwise_complement(int_type_int32),   19).
+unop_code(bitwise_complement(int_type_uint32),  20).
+unop_code(bitwise_complement(int_type_int64),   21).
+unop_code(bitwise_complement(int_type_uint64),  22).
+unop_code(dword_float_get_word0,                23).
+unop_code(dword_float_get_word1,                24).
+unop_code(dword_int64_get_word0,                25).
+unop_code(dword_int64_get_word1,                26).
+unop_code(dword_uint64_get_word0,               27).
+unop_code(dword_uint64_get_word1,               28).
 
 :- pred unop_debug(unary_op::in, string::out) is det.
 
-unop_debug(mktag,               "mktag").
+% unop_debug(mktag,             "mktag").
 unop_debug(tag,                 "tag").
-unop_debug(unmktag,             "unmktag").
+% unop_debug(unmktag,           "unmktag").
 unop_debug(mkbody,              "mkbody").
 unop_debug(unmkbody,            "unmkbody").
 unop_debug(strip_tag,           "strip_tag").
-unop_debug(bitwise_complement,  "bitwise_complement").
+unop_debug(bitwise_complement(int_type_int),  "bitwise_complement(int)").
 unop_debug(logical_not,         "not").
 unop_debug(hash_string,         "hash_string").
 unop_debug(hash_string2,        "hash_string2").
 unop_debug(hash_string3,        "hash_string3").
+unop_debug(hash_string4,        "hash_string4").
+unop_debug(hash_string5,        "hash_string5").
+unop_debug(hash_string6,        "hash_string6").
+unop_debug(dword_float_get_word0,   "dword_float_get_word0").
+unop_debug(dword_float_get_word1,   "dword_float_get_word1").
+unop_debug(dword_int64_get_word0,   "dword_int64_get_word0").
+unop_debug(dword_int64_get_word1,   "dword_int64_get_word1").
+unop_debug(dword_uint64_get_word0,  "dword_uint64_get_word0").
+unop_debug(dword_uint64_get_word1,  "dword_uint64_get_word1").
+unop_debug(bitwise_complement(int_type_uint),   "bitwise_complement(uint)").
+unop_debug(bitwise_complement(int_type_int8),   "bitwise_complement(int8)").
+unop_debug(bitwise_complement(int_type_uint8),  "bitwise_complement(uint8)").
+unop_debug(bitwise_complement(int_type_int16),  "bitwise_complement(int16)").
+unop_debug(bitwise_complement(int_type_uint16), "bitwise_complement(uint16)").
+unop_debug(bitwise_complement(int_type_int32),  "bitwise_complement(int32)").
+unop_debug(bitwise_complement(int_type_uint32), "bitwise_complement(uint32)").
+unop_debug(bitwise_complement(int_type_int64),  "bitwise_complement(int64)").
+unop_debug(bitwise_complement(int_type_uint64), "bitwise_complement(uint64)").
 
 %---------------------------------------------------------------------------%
 
@@ -1154,12 +1630,13 @@ unop_debug(hash_string3,        "hash_string3").
 :- pred debug_cstring(string::in, io::di, io::uo) is det.
 
 debug_cstring(Str, !IO) :-
-    io.write_char('"', !IO),
-    c_util.output_quoted_string(Str, !IO),
-    io.write_char('"', !IO),
+    io.output_stream(Stream, !IO),
+    io.write_char(Stream, '"', !IO),
+    c_util.output_quoted_string(Stream, Str, !IO),
+    io.write_char(Stream, '"', !IO),
     % XXX: We need the trailing space in case something follows
     % the string as a bytecode argument. This is not very elegant.
-    io.write_char(' ', !IO).
+    io.write_char(Stream, ' ', !IO).
 
 :- pred debug_string(string::in, io::di, io::uo) is det.
 
@@ -1173,6 +1650,60 @@ debug_int(Val, !IO) :-
     io.write_int(Val, !IO),
     io.write_char(' ', !IO).
 
+:- pred debug_uint(uint::in, io::di, io::uo) is det.
+
+debug_uint(Val, !IO) :-
+    io.write_uint(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_int8(int8::in, io::di, io::uo) is det.
+
+debug_int8(Val, !IO) :-
+    io.write_int8(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_uint8(uint8::in, io::di, io::uo) is det.
+
+debug_uint8(Val, !IO) :-
+    io.write_uint8(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_int16(int16::in, io::di, io::uo) is det.
+
+debug_int16(Val, !IO) :-
+    io.write_int16(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_uint16(uint16::in, io::di, io::uo) is det.
+
+debug_uint16(Val, !IO) :-
+    io.write_uint16(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_int32(int32::in, io::di, io::uo) is det.
+
+debug_int32(Val, !IO) :-
+    io.write_int32(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_uint32(uint32::in, io::di, io::uo) is det.
+
+debug_uint32(Val, !IO) :-
+    io.write_uint32(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_int64(int64::in, io::di, io::uo) is det.
+
+debug_int64(Val, !IO) :-
+    io.write_int64(Val, !IO),
+    io.write_char(' ', !IO).
+
+:- pred debug_uint64(uint64::in, io::di, io::uo) is det.
+
+debug_uint64(Val, !IO) :-
+    io.write_uint64(Val, !IO),
+    io.write_char(' ', !IO).
+
 :- pred debug_float(float::in, io::di, io::uo) is det.
 
 debug_float(Val, !IO) :-
