@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2005-2012 The University of Melbourne.
+% Copyright (C) 2017 The Mercury Team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -9,9 +10,9 @@
 % File: closure_analysis.m
 % Main author: juliensf
 %
-% Perform local closure analysis on procedures.  This involves tracking
+% Perform local closure analysis on procedures. This involves tracking
 % the possible values that a higher-order variable can take within a
-% procedure.  We attach this information to places where knowing the
+% procedure. We attach this information to places where knowing the
 % possible values of a higher-order call may be useful.
 %
 % This is similar to the analysis done by higher-order specialization, except
@@ -22,31 +23,37 @@
 :- module transform_hlds.closure_analysis.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
-:- import_module io.
 
-:- pred process_module(module_info::in, module_info::out,
-    io::di, io::uo) is det.
+:- pred closure_analyse_module(module_info::in, module_info::out) is det.
 
 %----------------------------------------------------------------------------%
 %----------------------------------------------------------------------------%
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.mode_test.
+:- import_module hlds.hlds_dependency_graph.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.passes_aux.
+:- import_module hlds.vartypes.
+:- import_module libs.
+:- import_module libs.dependency_graph.
 :- import_module libs.globals.
 :- import_module libs.options.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
 :- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
 :- import_module parse_tree.set_of_var.
-:- import_module transform_hlds.dependency_graph.
 
 :- import_module assoc_list.
 :- import_module bool.
+:- import_module io.
 :- import_module list.
 :- import_module map.
 :- import_module maybe.
@@ -58,28 +65,27 @@
 
 %----------------------------------------------------------------------------%
 
-process_module(!ModuleInfo, !IO) :-
+closure_analyse_module(!ModuleInfo) :-
     % XXX At the moment it is not necessary to do this on a per-SCC basis,
     % since the analysis is only procedure-local, but we would eventually
     % like to extend it.
 
     module_info_get_globals(!.ModuleInfo, Globals),
     globals.lookup_bool_option(Globals, debug_closure, Debug),
-    module_info_ensure_dependency_info(!ModuleInfo),
-    module_info_dependency_info(!.ModuleInfo, DepInfo),
-    hlds_dependency_info_get_dependency_ordering(DepInfo, SCCs),
-    list.foldl2(process_scc(Debug), SCCs, !ModuleInfo, !IO).
+    module_info_ensure_dependency_info(!ModuleInfo, DepInfo),
+    SCCs = dependency_info_get_bottom_up_sccs(DepInfo),
+    list.foldl(closure_analyse_scc(Debug), SCCs, !ModuleInfo).
 
 %----------------------------------------------------------------------------%
 %
 % Perform closure analysis on an SCC.
 %
 
-:- pred process_scc(bool::in, list(pred_proc_id)::in,
-    module_info::in, module_info::out, io::di, io::uo) is det.
+:- pred closure_analyse_scc(bool::in, scc::in,
+    module_info::in, module_info::out) is det.
 
-process_scc(Debug, SCC, !ModuleInfo, !IO) :-
-    list.foldl2(process_proc(Debug), SCC, !ModuleInfo, !IO).
+closure_analyse_scc(Debug, SCC, !ModuleInfo) :-
+    set.foldl(closure_analyse_proc(Debug), SCC, !ModuleInfo).
 
 %----------------------------------------------------------------------------%
 
@@ -88,20 +94,20 @@ process_scc(Debug, SCC, !ModuleInfo, !IO) :-
     %
 :- type closure_values
     --->    unknown
-                % The higher-order variable may be bound to something
-                % but we don't know what it is.
+            % The higher-order variable may be bound to something,
+            % but we don't know what it is.
 
     ;       partial(set(pred_proc_id))
-                % The higher-order variable may be bound to these
-                % values, or it may be bound to something else we don't
-                % know about.  (This is intended to be useful in producing
-                % error messages for the termination analysis; if one
-                % of the higher-order values is definitely non-terminating
-                % we can certainly let the user know about it.)
+            % The higher-order variable may be bound to these values,
+            % or it may be bound to something else we don't know about.
+            % (This is intended to be useful in producing error messages
+            % for the termination analysis; if one of the higher-order values
+            % is definitely non-terminating, we can certainly let the user
+            % know about it.)
 
     ;       exclusive(set(pred_proc_id)).
-                % The higher-order variable will be exclusively bound
-                % to this set of values.
+            % The higher-order variable can be bound only to one of the
+            % procedures identified by this set.
 
     % We attach a closure_info to each goal where it may be of interest;
     % at the moment calls and generic_calls.
@@ -127,7 +133,7 @@ var_has_ho_type(VarTypes, Var) :-
     lookup_var_type(VarTypes, Var, Type),
     type_is_higher_order(Type).
 
-    % Insert the given prog_var into the closure_info and set the
+    % Insert the given prog_var into the closure_info, and set the
     % possible values to unknown.
     %
 :- pred insert_unknown(prog_var::in, closure_info::in, closure_info::out)
@@ -141,26 +147,31 @@ insert_unknown(Var, !ClosureInfo) :-
 % Perform local closure analysis on a procedure.
 %
 
-:- pred process_proc(bool::in, pred_proc_id::in,
-    module_info::in, module_info::out, io::di, io::uo) is det.
+:- pred closure_analyse_proc(bool::in, pred_proc_id::in,
+    module_info::in, module_info::out) is det.
 
-process_proc(Debug, PPId, !ModuleInfo, !IO) :-
+closure_analyse_proc(Debug, PPId, !ModuleInfo) :-
     module_info_pred_proc_info(!.ModuleInfo, PPId, PredInfo, ProcInfo0),
     proc_info_get_headvars(ProcInfo0, HeadVars),
     proc_info_get_vartypes(ProcInfo0, VarTypes),
     proc_info_get_argmodes(ProcInfo0, ArgModes),
     ClosureInfo0 = closure_info_init(!.ModuleInfo, VarTypes, HeadVars,
         ArgModes),
-    write_proc_progress_message("% Analysing closures in ", PPId, !.ModuleInfo,
-        !IO),
+    trace [io(!TIO)] (
+        write_proc_progress_message(!.ModuleInfo,
+            "Analysing closures in", PPId, !TIO)
+    ),
     proc_info_get_goal(ProcInfo0, Body0),
-    process_goal(VarTypes, !.ModuleInfo, Body0, Body,
+    closure_analyse_goal(VarTypes, !.ModuleInfo, Body0, Body,
         ClosureInfo0, _ClosureInfo),
     (
         Debug = yes,
         proc_info_get_varset(ProcInfo, Varset),
-        dump_closure_info(Varset, Body, !IO),
-        io.flush_output(!IO)
+        trace [io(!IO)] (
+            get_debug_output_stream(!.ModuleInfo, DebugStream, !IO),
+            dump_closure_info(DebugStream, Varset, Body, !IO),
+            io.flush_output(DebugStream, !IO)
+        )
     ;
         Debug = no
     ),
@@ -172,15 +183,15 @@ process_proc(Debug, PPId, !ModuleInfo, !IO) :-
 % Track higher-order values through goals.
 %
 
-:- pred process_goal(vartypes::in, module_info::in,
+:- pred closure_analyse_goal(vartypes::in, module_info::in,
     hlds_goal::in, hlds_goal::out, closure_info::in, closure_info::out) is det.
 
-process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
+closure_analyse_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
     (
         GoalExpr0 = conj(ConjType, Goals0),
-        list.map_foldl(process_goal(VarTypes, ModuleInfo), Goals0, Goals,
-            !ClosureInfo),
+        list.map_foldl(closure_analyse_goal(VarTypes, ModuleInfo),
+            Goals0, Goals, !ClosureInfo),
         GoalExpr = conj(ConjType, Goals),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
@@ -205,7 +216,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
             % The closure_info won't yet contain any information about
             % higher-order outputs from this call.
 
-            ( map.search(!.ClosureInfo, Var, PossibleValues) ->
+            ( if map.search(!.ClosureInfo, Var, PossibleValues) then
                 (
                     PossibleValues = unknown
                 ;
@@ -214,7 +225,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
                     PossibleValues = exclusive(KnownValues),
                     map.det_insert(Var, KnownValues, !ValueMap)
                 )
-            ;
+            else
                 true
             )
         ),
@@ -236,16 +247,16 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
         % in 'GCallArgs' so we need to include in the set of input argument
         % separately.
 
-        ( Details = higher_order(CalledClosure0, _, _, _) ->
+        ( if Details = higher_order(CalledClosure0, _, _, _) then
             set_of_var.insert(CalledClosure0, InputArgs0, InputArgs)
-        ;
+        else
             InputArgs = InputArgs0
         ),
         AddValues = (pred(Var::in, !.ValueMap::in, !:ValueMap::out) is det :-
             % The closure_info won't yet contain any information about
             % higher-order outputs from this call.
 
-            ( map.search(!.ClosureInfo, Var, PossibleValues) ->
+            ( if map.search(!.ClosureInfo, Var, PossibleValues) then
                 (
                     PossibleValues = unknown
                 ;
@@ -254,7 +265,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
                     PossibleValues = exclusive(KnownValues),
                     map.det_insert(Var, KnownValues, !ValueMap)
                 )
-            ;
+            else
                 true
             )
         ),
@@ -262,14 +273,14 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
         goal_info_set_ho_values(Values, GoalInfo0, GoalInfo),
 
         % Insert any information about higher-order outputs from this call
-        % into the closure_info
+        % into the closure_info.
         set_of_var.fold(insert_unknown, OutputArgs, !ClosureInfo),
         Goal = hlds_goal(GoalExpr0, GoalInfo)
     ;
         GoalExpr0 = switch(SwitchVar, SwitchCanFail, Cases0),
         ProcessCase = (func(Case0) = Case - CaseInfo :-
             Case0 = case(MainConsId, OtherConsIds, CaseGoal0),
-            process_goal(VarTypes, ModuleInfo, CaseGoal0, CaseGoal,
+            closure_analyse_goal(VarTypes, ModuleInfo, CaseGoal0, CaseGoal,
               !.ClosureInfo, CaseInfo),
             Case = case(MainConsId, OtherConsIds, CaseGoal)
         ),
@@ -282,38 +293,38 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
         GoalExpr0 = unify(_, _, _, Unification, _),
         (
             Unification = construct(LHS, RHS, _, _, _, _, _),
-            (
+            ( if
                 RHS = closure_cons(ShroudedPPId, EvalMethod),
                 EvalMethod = lambda_normal
-            ->
+            then
                 PPId = unshroud_pred_proc_id(ShroudedPPId),
                 HO_Value = set.make_singleton_set(PPId),
                 map.det_insert(LHS, exclusive(HO_Value), !ClosureInfo)
-            ;
+            else
                 true
             )
         ;
             Unification = deconstruct(_, _, Args, _, _, _),
 
-            % XXX We don't currently support tracking the values of
-            % closures that are stored in data structures.
+            % XXX We don't currently support tracking the values of closures
+            % that are stored in data structures.
 
             HO_Args = list.filter(var_has_ho_type(VarTypes), Args),
             list.foldl(insert_unknown, HO_Args, !ClosureInfo)
         ;
             Unification = assign(LHS, RHS),
-            ( var_has_ho_type(VarTypes, LHS) ->
+            ( if var_has_ho_type(VarTypes, LHS) then
                 % Sanity check: make sure the rhs is also a higher-order
                 % variable.
 
-                ( var_has_ho_type(VarTypes, RHS) ->
+                ( if var_has_ho_type(VarTypes, RHS) then
                     true
-                ;
-                    unexpected($module, $pred, "not a higher-order var")
+                else
+                    unexpected($pred, "not a higher-order var")
                 ),
                 Values = map.lookup(!.ClosureInfo, RHS),
                 map.det_insert(LHS, Values, !ClosureInfo)
-            ;
+            else
                 true
             )
         ;
@@ -325,7 +336,7 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
     ;
         GoalExpr0 = disj(Goals0),
         ProcessDisjunct = (func(Disjunct0) = DisjunctResult :-
-            process_goal(VarTypes, ModuleInfo, Disjunct0, Disjunct,
+            closure_analyse_goal(VarTypes, ModuleInfo, Disjunct0, Disjunct,
                 !.ClosureInfo, ClosureInfoForDisjunct),
             DisjunctResult = Disjunct - ClosureInfoForDisjunct
         ),
@@ -337,59 +348,57 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = negation(NegatedGoal0),
-        process_goal(VarTypes, ModuleInfo, NegatedGoal0, NegatedGoal,
+        closure_analyse_goal(VarTypes, ModuleInfo, NegatedGoal0, NegatedGoal,
             !.ClosureInfo, _),
         GoalExpr = negation(NegatedGoal),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             SubGoal = SubGoal0
-        ;
-            process_goal(VarTypes, ModuleInfo, SubGoal0, SubGoal, !ClosureInfo)
+        else
+            closure_analyse_goal(VarTypes, ModuleInfo,
+                SubGoal0, SubGoal, !ClosureInfo)
         ),
         GoalExpr = scope(Reason, SubGoal),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = if_then_else(ExistQVars, Cond0, Then0, Else0),
-        process_goal(VarTypes, ModuleInfo, Cond0, Cond,
+        closure_analyse_goal(VarTypes, ModuleInfo, Cond0, Cond,
             !.ClosureInfo, CondInfo),
-        process_goal(VarTypes, ModuleInfo, Then0, Then,
+        closure_analyse_goal(VarTypes, ModuleInfo, Then0, Then,
             CondInfo, CondThenInfo),
-        process_goal(VarTypes, ModuleInfo, Else0, Else,
+        closure_analyse_goal(VarTypes, ModuleInfo, Else0, Else,
             !.ClosureInfo, ElseInfo),
         map.union(merge_closure_values, CondThenInfo, ElseInfo, !:ClosureInfo),
         GoalExpr = if_then_else(ExistQVars, Cond, Then, Else),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = call_foreign_proc(_, _, _, Args, _ExtraArgs, _, _),
-        % XXX 'ExtraArgs' should probably be ignored here since it is only
-        % used by the tabling transformation.
-        %
         % XXX We may eventually want to annotate foreign_procs with
-        % clousure_infos as well.  It isn't useful at the moment however.
+        % clousure_infos as well. It isn't useful at the moment however.
 
-        ForeignHOArgs = (pred(Arg::in, Out::out) is semidet :-
-            Arg = foreign_arg(Var, NameMode, Type, _BoxPolicy),
-            %
-            % A 'no' here means that the foreign argument is unused.
-            %
-            NameMode = yes(_ - Mode),
-            mode_util.mode_is_output(ModuleInfo, Mode),
-            type_is_higher_order(Type),
-            Out = Var - unknown
-        ),
+        ForeignHOArgs =
+            ( pred(Arg::in, Out::out) is semidet :-
+                Arg = foreign_arg(Var, NameMode, Type, _BoxPolicy),
+
+                % A 'no' here means that the foreign argument is unused.
+                NameMode = yes(foreign_arg_name_mode(_, Mode)),
+                mode_is_output(ModuleInfo, Mode),
+                type_is_higher_order(Type),
+                Out = Var - unknown
+            ),
         list.filter_map(ForeignHOArgs, Args, OutputForeignHOArgs),
         map.det_insert_from_assoc_list(OutputForeignHOArgs, !ClosureInfo),
         Goal = Goal0
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %----------------------------------------------------------------------------%
@@ -401,20 +410,20 @@ process_goal(VarTypes, ModuleInfo, Goal0, Goal, !ClosureInfo) :-
 
 partition_arguments(_, _, [],    [], !Inputs, !Outputs).
 partition_arguments(_, _, [_|_], [], _, _, _, _) :-
-    unexpected($module, $pred, "unequal length lists.").
+    unexpected($pred, "unequal length lists.").
 partition_arguments(_, _, [],    [_|_], _, _, _, _) :-
-    unexpected($module, $pred, "unequal length lists.").
+    unexpected($pred, "unequal length lists.").
 partition_arguments(ModuleInfo, VarTypes, [ Var | Vars ], [ Mode | Modes ],
         !Inputs, !Outputs) :-
-    ( var_has_ho_type(VarTypes, Var) ->
-        ( mode_is_input(ModuleInfo, Mode) ->
+    ( if var_has_ho_type(VarTypes, Var) then
+        ( if mode_is_input(ModuleInfo, Mode) then
             set_of_var.insert(Var, !Inputs)
-        ; mode_is_output(ModuleInfo, Mode) ->
+        else if mode_is_output(ModuleInfo, Mode) then
             set_of_var.insert(Var, !Outputs)
-        ;
+        else
             true
         )
-    ;
+    else
         true
     ),
     partition_arguments(ModuleInfo, VarTypes, Vars, Modes, !Inputs, !Outputs).
@@ -440,66 +449,78 @@ merge_closure_values(exclusive(A), exclusive(B), exclusive(A `set.union` B)).
 
 %----------------------------------------------------------------------------%
 %
-% Debugging code (used by '--debug-closure' option)
+% Debugging code, used if the '--debug-closure' option is given.
 %
 
-:- pred dump_closure_info(prog_varset::in, hlds_goal::in,
-    io::di, io::uo) is det.
+:- pred dump_closure_info(io.text_output_stream::in, prog_varset::in,
+    hlds_goal::in, io::di, io::uo) is det.
 
-dump_closure_info(Varset, Goal, !IO) :-
+dump_closure_info(DebugStream, Varset, Goal, !IO) :-
+    % XXX zs: It seems to me that the output from this predicate
+    % would be much easier to understand if each piece of the output
+    % was preceded by the identity of the goal that it came from.
     Goal = hlds_goal(GoalExpr, GoalInfo),
-    dump_closure_info_expr(Varset, GoalExpr, GoalInfo, !IO).
-
-:- pred dump_closure_info_expr(prog_varset::in, hlds_goal_expr::in,
-    hlds_goal_info::in, io::di, io::uo) is det.
-
-dump_closure_info_expr(Varset, conj(_ConjType, Goals), _, !IO) :-
-    list.foldl(dump_closure_info(Varset), Goals, !IO).
-dump_closure_info_expr(Varset, plain_call(_,_,_,_,_,_), GoalInfo, !IO) :-
-    dump_ho_values(GoalInfo, Varset, !IO).
-dump_closure_info_expr(Varset, generic_call(_,_,_,_,_), GoalInfo, !IO) :-
-    dump_ho_values(GoalInfo, Varset, !IO).
-dump_closure_info_expr(Varset, scope(_, Goal), _, !IO) :-
-    dump_closure_info(Varset, Goal, !IO).
-dump_closure_info_expr(Varset, switch(_, _, Cases), _, !IO) :-
-    CaseToGoal = (func(case(_, _, Goal)) = Goal),
-    Goals = list.map(CaseToGoal, Cases),
-    list.foldl(dump_closure_info(Varset), Goals, !IO).
-dump_closure_info_expr(Varset, if_then_else(_, Cond, Then, Else), _, !IO) :-
-    list.foldl(dump_closure_info(Varset), [Cond, Then, Else], !IO).
-dump_closure_info_expr(_, unify(_,_,_,_,_), _, !IO).
-dump_closure_info_expr(Varset, negation(Goal), _, !IO) :-
-    dump_closure_info(Varset, Goal, !IO).
-dump_closure_info_expr(_, call_foreign_proc(_, _, _, _, _, _, _), _, !IO).
-dump_closure_info_expr(Varset, disj(Goals), _, !IO) :-
-    list.foldl(dump_closure_info(Varset), Goals, !IO).
-dump_closure_info_expr(_, shorthand(_), _, _, _) :-
-    unexpected($module, $pred, "shorthand").
-
-:- pred dump_ho_values(hlds_goal_info::in, prog_varset::in,
-    io::di, io::uo) is det.
-
-dump_ho_values(GoalInfo, Varset, !IO) :-
-    HO_Values = goal_info_get_ho_values(GoalInfo),
-    ( not map.is_empty(HO_Values) ->
-        prog_out.write_context(goal_info_get_context(GoalInfo), !IO),
-        io.nl(!IO),
-        map.foldl(dump_ho_value(Varset), HO_Values, !IO)
+    (
+        GoalExpr = unify(_, _, _, _, _)
     ;
-        true
+        GoalExpr = plain_call(_, _, _, _, _, _),
+        dump_ho_values(DebugStream, Varset, GoalInfo, !IO)
+    ;
+        GoalExpr = generic_call(_, _, _, _, _),
+        dump_ho_values(DebugStream, Varset, GoalInfo, !IO)
+    ;
+        GoalExpr = call_foreign_proc(_, _, _, _, _, _, _)
+    ;
+        GoalExpr = conj(_ConjType, SubGoals),
+        list.foldl(dump_closure_info(DebugStream, Varset), SubGoals, !IO)
+    ;
+        GoalExpr = disj(SubGoals),
+        list.foldl(dump_closure_info(DebugStream, Varset), SubGoals, !IO)
+    ;
+        GoalExpr = switch(_, _, Cases),
+        SubGoals = list.map((func(case(_, _, CaseGoal)) = CaseGoal), Cases),
+        list.foldl(dump_closure_info(DebugStream, Varset), SubGoals, !IO)
+    ;
+        GoalExpr = if_then_else(_, CondGoal, ThenGoal, ElseGoal),
+        dump_closure_info(DebugStream, Varset, CondGoal, !IO),
+        dump_closure_info(DebugStream, Varset, ThenGoal, !IO),
+        dump_closure_info(DebugStream, Varset, ElseGoal, !IO)
+    ;
+        GoalExpr = negation(SubGoal),
+        dump_closure_info(DebugStream, Varset, SubGoal, !IO)
+    ;
+        GoalExpr = scope(_, SubGoal),
+        dump_closure_info(DebugStream, Varset, SubGoal, !IO)
+    ;
+        GoalExpr = shorthand(_),
+        unexpected($pred, "shorthand")
     ).
 
-:- pred dump_ho_value(prog_varset::in, prog_var::in, set(pred_proc_id)::in,
-    io::di, io::uo) is det.
+:- pred dump_ho_values(io.text_output_stream::in, prog_varset::in,
+    hlds_goal_info::in, io::di, io::uo) is det.
 
-dump_ho_value(Varset, ProgVar, Values, !IO) :-
+dump_ho_values(DebugStream, Varset, GoalInfo, !IO) :-
+    HO_Values = goal_info_get_ho_values(GoalInfo),
+    ( if map.is_empty(HO_Values) then
+        true
+    else
+        Context = goal_info_get_context(GoalInfo),
+        prog_out.write_context(DebugStream, Context, !IO),
+        io.nl(DebugStream, !IO),
+        map.foldl(dump_ho_value(DebugStream, Varset), HO_Values, !IO)
+    ).
+
+:- pred dump_ho_value(io.text_output_stream::in, prog_varset::in,
+    prog_var::in, set(pred_proc_id)::in, io::di, io::uo) is det.
+
+dump_ho_value(DebugStream, Varset, ProgVar, Values, !IO) :-
     VarName = varset.lookup_name(Varset, ProgVar),
-    io.format("%s =\n", [s(VarName)], !IO),
-    WritePPIds = (pred(PPId::in, !.IO::di, !:IO::uo) is det :-
-        io.write_string("\t", !IO),
-        io.write(PPId, !IO),
-        io.nl(!IO)
-    ),
+    io.format(DebugStream, "%s =\n", [s(VarName)], !IO),
+    WritePPIds =
+        ( pred(PPId::in, !.IO::di, !:IO::uo) is det :-
+            io.write_string(DebugStream, "\t", !IO),
+            io.write_line(DebugStream, PPId, !IO)
+        ),
     set.fold(WritePPIds, Values, !IO).
 
 %----------------------------------------------------------------------------%
