@@ -5,46 +5,60 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-% 
+%
 % File: code_util.m.
-% 
+%
 % Various utilities routines for code generation and recognition of builtins.
-% 
+%
 %-----------------------------------------------------------------------------%
 
 :- module ll_backend.code_util.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_llds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_rtti.
+:- import_module libs.
+:- import_module libs.optimization_options.
 :- import_module ll_backend.llds.
+:- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
 :- import_module bool.
 :- import_module list.
-:- import_module maybe.
-:- import_module pair.
 :- import_module set.
 
 %-----------------------------------------------------------------------------%
 
-    % Create a code address which holds the address of the specified procedure.
-    % The `immed' argument should be `no' if the the caller wants the returned
-    % address to be valid from everywhere in the program. If being valid from
-    % within the current procedure is enough, this argument should be `yes'
-    % wrapped around the value of the --procs-per-c-function option and the
-    % current procedure id. Using an address that is only valid from within
-    % the current procedure may make jumps more efficient.
-    %
-:- type immed == maybe(pair(int, pred_proc_id)).
-:- func make_entry_label(module_info, pred_id, proc_id, immed) = code_addr.
+:- type for_from_where
+    --->    for_from_everywhere
+    ;       for_from_proc(maybe_use_just_one_c_func, pred_id, proc_id).
 
-:- func make_entry_label_from_rtti(rtti_proc_label, immed) = code_addr.
+    % Create a code address which holds the address of the specified procedure.
+    % The `for_from_where' argument should be `for_from_everywhere'
+    % if the caller wants the returned address to be valid from everywhere
+    % in the program. If being valid from within a specified procedure
+    % (the current procedure) is enough, this argument should be
+    % `for_from_proc' wrapped around the value of the use_just_one_c_func
+    % option and the current procedure id. Using an address that is
+    % only valid from within the current procedure may make jumps faster.
+    %
+    % Note that we cannot take advantage of situations in which the caller
+    % and the callee are in the same C module even though use_just_one_c_func
+    % is not set, because what procedures end up in the same C module
+    % is decided only long after the LLDS has been constructed.
+    %
+:- func make_entry_label(module_info, pred_id, proc_id, for_from_where)
+    = code_addr.
+
+:- func make_entry_label_from_rtti(rtti_proc_label, for_from_where)
+    = code_addr.
 
     % Create a label which holds the address of the specified procedure,
     % which must be defined in the current module (procedures that are
@@ -52,7 +66,8 @@
     % not as labels, since their address is not known at C compilation time).
     % The fourth argument has the same meaning as for make_entry_label.
     %
-:- func make_local_entry_label(module_info, pred_id, proc_id, immed) = label.
+:- func make_local_entry_label(module_info, pred_id, proc_id, for_from_where)
+    = label.
 
     % Create a label internal to a Mercury procedure.
     %
@@ -78,9 +93,9 @@
     % and an lval respectively. Lvals referenced indirectly through
     % lvals of the form var(_) are not counted.
     %
+:- func lvals_in_lvals(list(lval)) = list(lval).
 :- func lvals_in_rval(rval) = list(lval).
 :- func lvals_in_lval(lval) = list(lval).
-:- func lvals_in_lvals(list(lval)) = list(lval).
 
     % Given a procedure that already has its arg_info field filled in,
     % return a list giving its input variables and their initial locations.
@@ -109,21 +124,24 @@
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
 :- import_module backend_libs.proc_label.
 :- import_module hlds.code_model.
 
 :- import_module int.
+:- import_module maybe.
+:- import_module pair.
 :- import_module require.
 :- import_module term.
 
 %---------------------------------------------------------------------------%
 
-make_entry_label(ModuleInfo, PredId, ProcId, Immed) = ProcAddr :-
+make_entry_label(ModuleInfo, PredId, ProcId, ForFromWhere) = ProcAddr :-
     RttiProcLabel = make_rtti_proc_label(ModuleInfo, PredId, ProcId),
-    ProcAddr = make_entry_label_from_rtti(RttiProcLabel, Immed).
+    ProcAddr = make_entry_label_from_rtti(RttiProcLabel, ForFromWhere).
 
-make_entry_label_from_rtti(RttiProcLabel, Immed) = ProcAddr :-
+make_entry_label_from_rtti(RttiProcLabel, ForFromWhere) = ProcAddr :-
     ProcIsImported = RttiProcLabel ^ rpl_proc_is_imported,
     (
         ProcIsImported = yes,
@@ -131,20 +149,21 @@ make_entry_label_from_rtti(RttiProcLabel, Immed) = ProcAddr :-
         ProcAddr = code_imported_proc(ProcLabel)
     ;
         ProcIsImported = no,
-        Label = make_local_entry_label_from_rtti(RttiProcLabel, Immed),
+        Label = make_local_entry_label_from_rtti(RttiProcLabel, ForFromWhere),
         ProcAddr = code_label(Label)
     ).
 
-make_local_entry_label(ModuleInfo, PredId, ProcId, Immed) = Label :-
+make_local_entry_label(ModuleInfo, PredId, ProcId, ForFromWhere) = Label :-
     RttiProcLabel = make_rtti_proc_label(ModuleInfo, PredId, ProcId),
-    Label = make_local_entry_label_from_rtti(RttiProcLabel, Immed).
+    Label = make_local_entry_label_from_rtti(RttiProcLabel, ForFromWhere).
 
-:- func make_local_entry_label_from_rtti(rtti_proc_label, immed) = label.
+:- func make_local_entry_label_from_rtti(rtti_proc_label, for_from_where)
+    = label.
 
-make_local_entry_label_from_rtti(RttiProcLabel, Immed) = Label :-
+make_local_entry_label_from_rtti(RttiProcLabel, ForFromWhere) = Label :-
     ProcLabel = make_proc_label_from_rtti(RttiProcLabel),
     (
-        Immed = no,
+        ForFromWhere = for_from_everywhere,
         % If we want to define the label or use it to put it into a data
         % structure, a label that is usable only within the current C module
         % won't do.
@@ -158,30 +177,32 @@ make_local_entry_label_from_rtti(RttiProcLabel, Immed) = Label :-
         ),
         Label = entry_label(EntryType, ProcLabel)
     ;
-        Immed = yes(ProcsPerFunc - proc(CurPredId, CurProcId)),
-        Label = choose_local_label_type(ProcsPerFunc, CurPredId, CurProcId,
+        ForFromWhere = for_from_proc(MaybeUseJustOneFunc,
+            CurPredId, CurProcId),
+        Label = choose_local_label_type(MaybeUseJustOneFunc,
+            CurPredId, CurProcId,
             RttiProcLabel ^ rpl_pred_id, RttiProcLabel ^ rpl_proc_id,
             ProcLabel)
     ).
 
-:- func choose_local_label_type(int, pred_id, proc_id, pred_id, proc_id,
-        proc_label) = label.
+:- func choose_local_label_type(maybe_use_just_one_c_func,
+    pred_id, proc_id, pred_id, proc_id, proc_label) = label.
 
-choose_local_label_type(ProcsPerFunc, CurPredId, CurProcId,
+choose_local_label_type(MaybeUseJustOneFunc, CurPredId, CurProcId,
         PredId, ProcId, ProcLabel) = Label :-
-    (
+    ( if
         % If we want to branch to the label now, we prefer a form that is
         % usable only within the current C module, since it is likely to be
         % faster.
         (
-            ProcsPerFunc = 0
+            MaybeUseJustOneFunc = use_just_one_c_func
         ;
             PredId = CurPredId,
             ProcId = CurProcId
         )
-    ->
+    then
         EntryType = entry_label_c_local
-    ;
+    else
         EntryType = entry_label_local
     ),
     Label = entry_label(EntryType, ProcLabel).
@@ -193,12 +214,12 @@ make_internal_label(ModuleInfo, PredId, ProcId, LabelNum) = Label :-
     Label = internal_label(LabelNum, ProcLabel).
 
 extract_proc_label_from_code_addr(CodeAddr) = ProcLabel :-
-    ( CodeAddr = code_label(Label) ->
+    ( if CodeAddr = code_label(Label) then
         ProcLabel = get_proc_label(Label)
-    ; CodeAddr = code_imported_proc(ProcLabelPrime) ->
+    else if CodeAddr = code_imported_proc(ProcLabelPrime) then
         ProcLabel = ProcLabelPrime
-    ;
-        unexpected($module, $pred, "failed")
+    else
+        unexpected($pred, "failed")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -215,7 +236,7 @@ max_mentioned_regs(Lvals, MaxRegR, MaxRegF) :-
 
 max_mentioned_reg_2([], !MaxRegR, !MaxRegF).
 max_mentioned_reg_2([Lval | Lvals], !MaxRegR, !MaxRegF) :-
-    ( Lval = reg(RegType, N) ->
+    ( if Lval = reg(RegType, N) then
         (
             RegType = reg_r,
             int.max(N, !MaxRegR)
@@ -223,7 +244,7 @@ max_mentioned_reg_2([Lval | Lvals], !MaxRegR, !MaxRegF) :-
             RegType = reg_f,
             int.max(N, !MaxRegF)
         )
-    ;
+    else
         true
     ),
     max_mentioned_reg_2(Lvals, !MaxRegR, !MaxRegF).
@@ -236,7 +257,7 @@ max_mentioned_abs_regs(Lvals, MaxRegR, MaxRegF) :-
 
 max_mentioned_abs_reg_2([], !MaxRegR, !MaxRegF).
 max_mentioned_abs_reg_2([Lval | Lvals], !MaxRegR, !MaxRegF) :-
-    ( Lval = abs_reg(RegType, N) ->
+    ( if Lval = abs_reg(RegType, N) then
         (
             RegType = reg_r,
             int.max(N, !MaxRegR)
@@ -244,7 +265,7 @@ max_mentioned_abs_reg_2([Lval | Lvals], !MaxRegR, !MaxRegF) :-
             RegType = reg_f,
             int.max(N, !MaxRegF)
         )
-    ;
+    else
         true
     ),
     max_mentioned_abs_reg_2(Lvals, !MaxRegR, !MaxRegF).
@@ -252,58 +273,70 @@ max_mentioned_abs_reg_2([Lval | Lvals], !MaxRegR, !MaxRegF) :-
 %-----------------------------------------------------------------------------%
 
 goal_may_alloc_temp_frame(hlds_goal(GoalExpr, _GoalInfo), May) :-
-    goal_may_alloc_temp_frame_2(GoalExpr, May).
+    goal_expr_may_alloc_temp_frame(GoalExpr, May).
 
-:- pred goal_may_alloc_temp_frame_2(hlds_goal_expr::in, bool::out)
-    is det.
+:- pred goal_expr_may_alloc_temp_frame(hlds_goal_expr::in, bool::out) is det.
 
-goal_may_alloc_temp_frame_2(generic_call(_, _, _, _, _), no).
-goal_may_alloc_temp_frame_2(plain_call(_, _, _, _, _, _), no).
-goal_may_alloc_temp_frame_2(unify(_, _, _, _, _), no).
-    % We cannot safely say that a foreign code fragment does not allocate
-    % temporary nondet frames without knowing all the #defined macros
-    % that expand to mktempframe and variants thereof. The performance
-    % impact of being too conservative is probably not too bad.
-goal_may_alloc_temp_frame_2(call_foreign_proc(_, _, _, _, _, _, _), yes).
-goal_may_alloc_temp_frame_2(scope(_, Goal), May) :-
-    Goal = hlds_goal(_, GoalInfo),
-    CodeModel = goal_info_get_code_model(GoalInfo),
+goal_expr_may_alloc_temp_frame(GoalExpr, May) :-
     (
-        CodeModel = model_non,
-        May = yes
-    ;
-        ( CodeModel = model_det
-        ; CodeModel = model_semi
+        ( GoalExpr = generic_call(_, _, _, _, _)
+        ; GoalExpr = plain_call(_, _, _, _, _, _)
+        ; GoalExpr = unify(_, _, _, _, _)
         ),
-        goal_may_alloc_temp_frame(Goal, May)
-    ).
-goal_may_alloc_temp_frame_2(negation(Goal), May) :-
-    goal_may_alloc_temp_frame(Goal, May).
-goal_may_alloc_temp_frame_2(conj(_ConjType, Goals), May) :-
-    goal_list_may_alloc_temp_frame(Goals, May).
-goal_may_alloc_temp_frame_2(disj(Goals), May) :-
-    goal_list_may_alloc_temp_frame(Goals, May).
-goal_may_alloc_temp_frame_2(switch(_Var, _Det, Cases), May) :-
-    cases_may_alloc_temp_frame(Cases, May).
-goal_may_alloc_temp_frame_2(if_then_else(_Vars, C, T, E), May) :-
-    ( goal_may_alloc_temp_frame(C, yes) ->
-        May = yes
-    ; goal_may_alloc_temp_frame(T, yes) ->
+        May = no
+    ;
+        GoalExpr = call_foreign_proc(_, _, _, _, _, _, _),
+        % We cannot safely say that a foreign code fragment does not allocate
+        % temporary nondet frames without knowing all the #defined macros
+        % that expand to mktempframe and variants thereof. The performance
+        % impact of being too conservative is probably not too bad.
         May = yes
     ;
-        goal_may_alloc_temp_frame(E, May)
+        GoalExpr = scope(_, SubGoal),
+        SubGoal = hlds_goal(_, SubGoalInfo),
+        SubCodeModel = goal_info_get_code_model(SubGoalInfo),
+        (
+            SubCodeModel = model_non,
+            May = yes
+        ;
+            ( SubCodeModel = model_det
+            ; SubCodeModel = model_semi
+            ),
+            goal_may_alloc_temp_frame(SubGoal, May)
+        )
+    ;
+        GoalExpr = negation(SubGoal),
+        goal_may_alloc_temp_frame(SubGoal, May)
+    ;
+        ( GoalExpr = conj(_ConjType, SubGoals)
+        ; GoalExpr = disj(SubGoals)
+        ),
+        goal_list_may_alloc_temp_frame(SubGoals, May)
+    ;
+        GoalExpr = switch(_Var, _Det, Cases),
+        cases_may_alloc_temp_frame(Cases, May)
+    ;
+        GoalExpr = if_then_else(_Vars, C, T, E),
+        ( if goal_may_alloc_temp_frame(C, yes) then
+            May = yes
+        else if goal_may_alloc_temp_frame(T, yes) then
+            May = yes
+        else
+            goal_may_alloc_temp_frame(E, May)
+        )
+    ;
+        GoalExpr = shorthand(_),
+        % These should have been expanded out by now.
+        unexpected($pred, "shorthand")
     ).
-goal_may_alloc_temp_frame_2(shorthand(_), _) :-
-    % These should have been expanded out by now.
-    unexpected($module, $pred, "shorthand").
 
 :- pred goal_list_may_alloc_temp_frame(list(hlds_goal)::in, bool::out) is det.
 
 goal_list_may_alloc_temp_frame([], no).
 goal_list_may_alloc_temp_frame([Goal | Goals], May) :-
-    ( goal_may_alloc_temp_frame(Goal, yes) ->
+    ( if goal_may_alloc_temp_frame(Goal, yes) then
         May = yes
-    ;
+    else
         goal_list_may_alloc_temp_frame(Goals, May)
     ).
 
@@ -311,24 +344,24 @@ goal_list_may_alloc_temp_frame([Goal | Goals], May) :-
 
 cases_may_alloc_temp_frame([], no).
 cases_may_alloc_temp_frame([case(_, _, Goal) | Cases], May) :-
-    ( goal_may_alloc_temp_frame(Goal, yes) ->
+    ( if goal_may_alloc_temp_frame(Goal, yes) then
         May = yes
-    ;
+    else
         cases_may_alloc_temp_frame(Cases, May)
     ).
 
 %-----------------------------------------------------------------------------%
 
 neg_rval(Rval, NegRval) :-
-    ( neg_rval_2(Rval, NegRval0) ->
+    ( if natural_neg_rval(Rval, NegRval0) then
         NegRval = NegRval0
-    ;
+    else
         NegRval = unop(logical_not, Rval)
     ).
 
-:- pred neg_rval_2(rval::in, rval::out) is semidet.
+:- pred natural_neg_rval(rval::in, rval::out) is semidet.
 
-neg_rval_2(const(Const), const(NegConst)) :-
+natural_neg_rval(const(Const), const(NegConst)) :-
     (
         Const = llconst_true,
         NegConst = llconst_false
@@ -336,18 +369,18 @@ neg_rval_2(const(Const), const(NegConst)) :-
         Const = llconst_false,
         NegConst = llconst_true
     ).
-neg_rval_2(unop(logical_not, Rval), Rval).
-neg_rval_2(binop(Op, X, Y), binop(NegOp, X, Y)) :-
+natural_neg_rval(unop(logical_not, Rval), Rval).
+natural_neg_rval(binop(Op, X, Y), binop(NegOp, X, Y)) :-
     neg_op(Op, NegOp).
 
 :- pred neg_op(binary_op::in, binary_op::out) is semidet.
 
-neg_op(eq, ne).
-neg_op(ne, eq).
-neg_op(int_lt, int_ge).
-neg_op(int_le, int_gt).
-neg_op(int_gt, int_le).
-neg_op(int_ge, int_lt).
+neg_op(eq(T), ne(T)).
+neg_op(ne(T), eq(T)).
+neg_op(int_lt(T), int_ge(T)).
+neg_op(int_le(T), int_gt(T)).
+neg_op(int_gt(T), int_le(T)).
+neg_op(int_ge(T), int_lt(T)).
 neg_op(str_eq, str_ne).
 neg_op(str_ne, str_eq).
 neg_op(str_lt, str_ge).
@@ -362,12 +395,12 @@ neg_op(float_gt, float_le).
 neg_op(float_ge, float_lt).
 
 negate_the_test([], _) :-
-    unexpected($module, $pred, "empty list").
+    unexpected($pred, "empty list").
 negate_the_test([Instr0 | Instrs0], Instrs) :-
-    ( Instr0 = llds_instr(if_val(Test, Target), Comment) ->
+    ( if Instr0 = llds_instr(if_val(Test, Target), Comment) then
         neg_rval(Test, NewTest),
         Instrs = [llds_instr(if_val(NewTest, Target), Comment)]
-    ;
+    else
         negate_the_test(Instrs0, Instrs1),
         Instrs = [Instr0 | Instrs1]
     ).
@@ -384,6 +417,7 @@ lvals_in_rval(var(_)) = [].
 lvals_in_rval(mkword(_, Rval)) = lvals_in_rval(Rval).
 lvals_in_rval(mkword_hole(_)) = [].
 lvals_in_rval(const(_)) = [].
+lvals_in_rval(cast(_, Rval)) = lvals_in_rval(Rval).
 lvals_in_rval(unop(_, Rval)) = lvals_in_rval(Rval).
 lvals_in_rval(binop(_, Rval1, Rval2)) =
     lvals_in_rval(Rval1) ++ lvals_in_rval(Rval2).
@@ -451,19 +485,28 @@ encode_num_generic_call_vars(NumR, NumF) = (NumR \/ (NumF << 16)).
 
 %-----------------------------------------------------------------------------%
 
-size_of_cell_args([]) = 0.
-size_of_cell_args([CellArg | CellArgs]) = Size + Sizes :-
+size_of_cell_args(CellArgs) = Size :-
+    size_of_cell_args_acc(CellArgs, 0, Size).
+
+:- pred size_of_cell_args_acc(list(cell_arg)::in, int::in, int::out) is det.
+
+size_of_cell_args_acc([], !Size).
+size_of_cell_args_acc([CellArg | CellArgs], !Size) :-
     (
         ( CellArg = cell_arg_full_word(_, _)
-        ; CellArg = cell_arg_take_addr(_, _)
-        ; CellArg = cell_arg_skip
+        ; CellArg = cell_arg_skip_one_word
+        ; CellArg = cell_arg_take_addr_one_word(_, _)
         ),
-        Size = 1
+        CellSize = 1
     ;
-        CellArg = cell_arg_double_word(_),
-        Size = 2
+        ( CellArg = cell_arg_double_word(_)
+        ; CellArg = cell_arg_skip_two_words
+        ; CellArg = cell_arg_take_addr_two_words(_, _)
+        ),
+        CellSize = 2
     ),
-    Sizes = size_of_cell_args(CellArgs).
+    !:Size = !.Size + CellSize,
+    size_of_cell_args_acc(CellArgs, !Size).
 
 %-----------------------------------------------------------------------------%
 
