@@ -18,6 +18,7 @@
 :- module transform_hlds.complexity.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -58,19 +59,27 @@
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
-:- import_module check_hlds.polymorphism.
+:- import_module check_hlds.
+:- import_module check_hlds.mode_test.
+:- import_module check_hlds.polymorphism_type_info.
 :- import_module hlds.code_model.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.instmap.
 :- import_module hlds.passes_aux.
 :- import_module hlds.pred_table.
+:- import_module hlds.vartypes.
+:- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
+:- import_module mdbcomp.
+:- import_module mdbcomp.builtin_modules.
 :- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
@@ -83,7 +92,6 @@
 :- import_module list.
 :- import_module map.
 :- import_module require.
-:- import_module set.
 :- import_module string.
 :- import_module term.
 :- import_module varset.
@@ -127,10 +135,10 @@ read_spec_file_lines(Stream, CurLineNum, NumLines, MaybeError, !ProcMap,
         ResLine = ok(Chars0),
         list.filter(unify('\n'), Chars0, _, Chars),
         string.from_char_list(Chars, ProcName),
-        ( map.insert(ProcName, CurLineNum, !ProcMap) ->
+        ( if map.insert(ProcName, CurLineNum, !ProcMap) then
             read_spec_file_lines(Stream, CurLineNum + 1,
                 NumLines, MaybeError, !ProcMap, !IO)
-        ;
+        else
             NumLines = CurLineNum,
             MaybeError = yes("repeated line: " ++ ProcName)
         )
@@ -138,24 +146,25 @@ read_spec_file_lines(Stream, CurLineNum, NumLines, MaybeError, !ProcMap,
 
 %-----------------------------------------------------------------------------%
 
+is_in_complexity_proc_map(ProcMap, ModuleInfo, PredId, ProcId) = IsInMap :-
+    FullName = complexity_proc_name(ModuleInfo, PredId, ProcId),
+    ( if map.search(ProcMap, FullName, ProcNum) then
+        IsInMap = yes(ProcNum)
+    else
+        IsInMap = no
+    ).
+
 complexity_proc_name(ModuleInfo, PredId, ProcId) = FullName :-
     module_info_get_name(ModuleInfo, ModuleSymName),
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
     PredName = pred_info_name(PredInfo),
     QualifiedName = qualified(ModuleSymName, PredName),
     Arity = pred_info_orig_arity(PredInfo),
-    NameAndArity = sym_name_and_arity_to_string(QualifiedName / Arity),
+    NameAndArity =
+        sym_name_arity_to_string(sym_name_arity(QualifiedName, Arity)),
     proc_id_to_int(ProcId, ProcIdInt),
     FullName = NameAndArity ++ "-" ++ int_to_string(ProcIdInt).
 
-is_in_complexity_proc_map(ProcMap, ModuleInfo, PredId, ProcId) = IsInMap :-
-    FullName = complexity_proc_name(ModuleInfo, PredId, ProcId),
-    ( map.search(ProcMap, FullName, ProcNum) ->
-        IsInMap = yes(ProcNum)
-    ;
-        IsInMap = no
-    ).
-
 %-----------------------------------------------------------------------------%
 
 complexity_process_proc_msg(NumProcs, ProcMap, PredProcId,
@@ -171,9 +180,9 @@ complexity_process_proc_msg(NumProcs, ProcMap, PredProcId,
         (
             Verbose = yes,
             trace [io(!IO)] (
-                write_proc_progress_message(
-                    "% Applying complexity experiment transformation to ",
-                    PredId, ProcId, !.ModuleInfo, !IO)
+                write_proc_progress_message(!.ModuleInfo,
+                    "Applying complexity experiment transformation to",
+                    PredId, ProcId, !IO)
             )
         ;
             Verbose = no
@@ -283,9 +292,11 @@ complexity_process_proc(NumProcs, ProcNum, FullName, PredId,
         SlotGoals),
 
     IsActiveOutputArg = foreign_arg(IsActiveVar,
-        yes(IsActiveVarName - out_mode), is_active_type, native_if_possible),
+        yes(foreign_arg_name_mode(IsActiveVarName, out_mode)),
+        is_active_type, bp_native_if_possible),
     SlotInputArg = foreign_arg(SlotVar,
-        yes(SlotVarName - in_mode), int_type, native_if_possible),
+        yes(foreign_arg_name_mode(SlotVarName, in_mode)),
+        int_type, bp_native_if_possible),
 
     ProcNumStr = int_to_string(ProcNum),
 
@@ -413,7 +424,8 @@ generate_slot_goals(ProcNum, NumberedVars, NumProfiledVars, Context, PredId,
         ProcVarName, SlotVarName, PredId, !ProcInfo, !ModuleInfo,
         PrefixGoals, ForeignArgs, FillCodeStr),
     SlotVarArg = foreign_arg(SlotVar,
-        yes(SlotVarName - out_mode), int_type, native_if_possible),
+        yes(foreign_arg_name_mode(SlotVarName, out_mode)),
+        int_type, bp_native_if_possible),
     PredName = "complexity_call_proc",
     DeclCodeStr = "\tMR_ComplexityProc *" ++ ProcVarName ++ ";\n",
     PredCodeStr = "\tMR_" ++ PredName ++ "(" ++
@@ -461,9 +473,11 @@ generate_size_goal(ArgVar, VarSeqNum, Context, NumProfiledVars, ProcVarName,
     ArgName = "arg" ++ int_to_string(VarSeqNum),
     TypeInfoArgName = "input_typeinfo" ++ int_to_string(VarSeqNum),
     ForeignArg = foreign_arg(ArgVar,
-        yes(ArgName - in_mode), VarType, native_if_possible),
+        yes(foreign_arg_name_mode(ArgName, in_mode)),
+        VarType, bp_native_if_possible),
     ForeignTypeInfoArg = foreign_arg(TypeInfoVar,
-        yes(TypeInfoArgName - in_mode), TypeInfoType, native_if_possible),
+        yes(foreign_arg_name_mode(TypeInfoArgName, in_mode)),
+        TypeInfoType, bp_native_if_possible),
     ForeignArgs = [ForeignTypeInfoArg, ForeignArg],
     CodeStr = "\t" ++ MacroName ++ "(" ++
         ProcVarName ++ ", " ++
@@ -497,10 +511,10 @@ complexity_generate_foreign_proc(PredName, Detism, Args, ExtraArgs,
     Attrs0 = default_attributes(lang_c),
     set_may_call_mercury(proc_will_not_call_mercury, Attrs0, Attrs),
     MaybeTraceRuntimeCond = no,
-    goal_util.generate_foreign_proc(BuiltinModule, PredName, pf_predicate,
-        only_mode, Detism, purity_impure, Attrs, Args, ExtraArgs,
+    goal_util.generate_foreign_proc(ModuleInfo, BuiltinModule, PredName,
+        pf_predicate, only_mode, Detism, purity_impure, Attrs, Args, ExtraArgs,
         MaybeTraceRuntimeCond, Code, [], instmap_delta_bind_vars(BoundVars),
-        ModuleInfo, Context, Goal).
+        Context, Goal).
 
 %-----------------------------------------------------------------------------%
 
@@ -510,25 +524,25 @@ complexity_generate_foreign_proc(PredName, Detism, Args, ExtraArgs,
 
 classify_args([], [], _, _, _, []).
 classify_args([_ | _], [], _, _, _, _) :-
-    unexpected($module, $pred, "lists not same length").
+    unexpected($pred, "lists not same length").
 classify_args([], [_ | _], _, _, _, _) :-
-    unexpected($module, $pred, "lists not same length").
+    unexpected($pred, "lists not same length").
 classify_args([Var | Vars], [Mode | Modes], ModuleInfo, VarSet, VarTypes,
         [Var - complexity_arg_info(MaybeName, Kind) | VarInfos]) :-
     classify_args(Vars, Modes, ModuleInfo, VarSet, VarTypes, VarInfos),
-    ( varset.search_name(VarSet, Var, Name) ->
+    ( if varset.search_name(VarSet, Var, Name) then
         MaybeName = yes(Name)
-    ;
+    else
         MaybeName = no
     ),
-    ( mode_is_fully_input(ModuleInfo, Mode) ->
+    ( if mode_is_fully_input(ModuleInfo, Mode) then
         lookup_var_type(VarTypes, Var, VarType),
-        ( zero_size_type(ModuleInfo, VarType) ->
+        ( if zero_size_type(ModuleInfo, VarType) then
             Kind = complexity_input_fixed_size
-        ;
+        else
             Kind = complexity_input_variable_size
         )
-    ;
+    else
         Kind = complexity_output
     ).
 
@@ -571,12 +585,10 @@ is_active_type = Type :-
 make_type_info_var(Type, Context, PredId, !ProcInfo, !ModuleInfo,
         TypeInfoVar, TypeInfoGoals) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo0),
-    create_poly_info(!.ModuleInfo, PredInfo0, !.ProcInfo, PolyInfo0),
-    polymorphism_make_type_info_var(Type, Context, TypeInfoVar,
-        TypeInfoGoals, PolyInfo0, PolyInfo),
-    poly_info_extract(PolyInfo, PredInfo0, PredInfo,
-        !ProcInfo, !:ModuleInfo),
-    expect(unify(PredInfo0, PredInfo), $module, $pred, "modified pred_info").
+    polymorphism_make_type_info_var_raw(Type, Context,
+        TypeInfoVar, TypeInfoGoals, !ModuleInfo,
+        PredInfo0, PredInfo, !ProcInfo),
+    expect(unify(PredInfo0, PredInfo), $pred, "modified pred_info").
 
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.complexity.
