@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2001-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -15,23 +16,18 @@
 
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module hlds.vartypes.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.program_representation.
-:- import_module parse_tree.prog_data.
 
 :- import_module list.
 :- import_module maybe.
 
-:- type proc_coverage_info.
-
-    % Initialize the coverage_profiling_options structure.
-    %
-:- pred coverage_profiling_options(module_info::in,
-    coverage_profiling_options::out) is det.
-
     % Perform the coverage profiling transformation on the given goal,
     % and return a list of the coverage points created.
     %
@@ -47,10 +43,14 @@
 
 :- import_module hlds.goal_util.
 :- import_module hlds.instmap.
+:- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.options.
 :- import_module ll_backend.deep_profiling.
+:- import_module parse_tree.
 :- import_module parse_tree.builtin_lib_types.
+:- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
 :- import_module parse_tree.set_of_var.
 
 :- import_module assoc_list.
@@ -59,7 +59,6 @@
 :- import_module map.
 :- import_module pair.
 :- import_module require.
-:- import_module set.
 :- import_module string.
 
     % Information used by the coverage profiling transformation.
@@ -116,7 +115,10 @@
                 cpo_run_first_pass          :: bool
             ).
 
-coverage_profiling_options(ModuleInfo, CoveragePointOptions) :-
+:- pred init_coverage_profiling_options(module_info::in,
+    coverage_profiling_options::out) is det.
+
+init_coverage_profiling_options(ModuleInfo, CoveragePointOptions) :-
     module_info_get_globals(ModuleInfo, Globals),
 
     % Options controlling what instrumentation code we generate.
@@ -153,9 +155,11 @@ coverage_profiling_options(ModuleInfo, CoveragePointOptions) :-
         CoverageAfterGoal, BranchIf, BranchSwitch, BranchDisj,
         UsePortCounts, UseTrivial, RunFirstPass).
 
+%-----------------------------------------------------------------------------%
+
 coverage_prof_transform_proc_body(ModuleInfo, PredProcId, ContainingGoalMap,
         MaybeRecInfo, !Goal, !VarInfo, CoveragePoints) :-
-    coverage_profiling_options(ModuleInfo, CoverageProfilingOptions),
+    init_coverage_profiling_options(ModuleInfo, CoverageProfilingOptions),
     CoverageInfo0 = init_proc_coverage_info(!.VarInfo, ModuleInfo,
         PredProcId, MaybeRecInfo, CoverageProfilingOptions, ContainingGoalMap),
     RunFirstPass = CoverageProfilingOptions ^ cpo_run_first_pass,
@@ -198,15 +202,15 @@ coverage_prof_second_pass_goal(Goal0, Goal,
     DPInfo = goal_info_get_dp_info(GoalInfo0),
     DPInfo = dp_goal_info(IsMDProfInst, _MaybeDPCoverageInfo),
 
-    (
+    ( if
         IsMDProfInst = goal_is_not_mdprof_inst,
         CoverageBeforeKnown = coverage_before_unknown
-    ->
+    then
         GoalId = goal_id(GoalNum),
         UnknownMsg = string.format(
             "Coverage information is unknown for goal_id %d\n", [i(GoalNum)]),
-        unexpected($module, $pred, UnknownMsg)
-    ;
+        unexpected($pred, UnknownMsg)
+    else
         true
     ),
 
@@ -231,9 +235,7 @@ coverage_prof_second_pass_goal(Goal0, Goal,
         (
             GoalExpr0 = plain_call(_, _, _, BuiltinState, _, _),
             (
-                ( BuiltinState = out_of_line_builtin
-                ; BuiltinState = not_builtin
-                ),
+                BuiltinState = not_builtin,
                 GathersCoverageAfter = yes
             ;
                 BuiltinState = inline_builtin,
@@ -299,11 +301,10 @@ coverage_prof_second_pass_goal(Goal0, Goal,
             AddedImpurityInner),
         % A scope may cut away solutions, if it does we don't know the number
         % of solutions of the scoped goal.
-        ScopedGoalDetism =
-            ScopeGoal0 ^ hlds_goal_info ^ goal_info_get_determinism,
-        ( ScopedGoalDetism = Detism ->
+        ScopedGoalDetism = goal_info_get_determinism(ScopeGoal0 ^ hg_info),
+        ( if ScopedGoalDetism = Detism then
             NextCoverageBeforeKnown0 = CoverageAfterScopedGoalKnown
-        ;
+        else
             NextCoverageBeforeKnown0 = coverage_before_unknown
         ),
         GoalExpr1 = scope(Reason, ScopeGoal)
@@ -314,14 +315,14 @@ coverage_prof_second_pass_goal(Goal0, Goal,
             AddedImpurityInner)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ),
 
     % Step 2.
     %
     % Decide whether we need to insert a coverage point after this goal
     % to measure how many times execution reaches there.
-    (
+    ( if
         (
             % Never insert coverage points on goals that are part of the deep
             % profiling instrumentation.
@@ -331,10 +332,10 @@ coverage_prof_second_pass_goal(Goal0, Goal,
             % goal; adding a counter would be redundant.
             NextCoverageBeforeKnown0 = coverage_before_known
         )
-    ->
+    then
         MaybeAddCP = no,
         NextCoverageBeforeKnown = NextCoverageBeforeKnown0
-    ;
+    else
         CoverageAfterGoals = CPOptions ^ cpo_coverage_after_goal,
         (
             CoverageAfterGoals  = yes,
@@ -409,12 +410,12 @@ coverage_prof_second_pass_conj(ConjType, [HeadGoal0 | TailGoals0], Goals,
         AddedImpurityTail),
     % Flatten the conjunction. We need to do this if we replaced the head
     % with a goal that is itself a conjunction.
-    (
+    ( if
         HeadGoal = hlds_goal(conj(plain_conj, HeadConjGoals), _),
         ConjType = plain_conj
-    ->
+    then
         Goals = HeadConjGoals ++ TailGoals
-    ;
+    else
         Goals = [HeadGoal | TailGoals]
     ),
     bool.or(AddedImpurityHead, AddedImpurityTail, AddedImpurity).
@@ -434,23 +435,22 @@ coverage_prof_second_pass_disj(DPInfo, CoverageBeforeKnown,
     % disjuncts and its second disjunct has 'failure' determinism, then
     % perform the coverage profiling pass on the first disjunct as if this
     % is the only goal.
-    (
+    ( if
         DPInfo = dp_goal_info(goal_is_mdprof_inst, _),
         Disjuncts0 = [FirstDisjunct0, SecondDisjunct],
-        goal_info_get_determinism(SecondDisjunct ^ hlds_goal_info) =
-            detism_failure
+        goal_info_get_determinism(SecondDisjunct ^ hg_info) = detism_failure
         % XXX: zs: Would this be a better test here?
         % goal_has_feature(SecondDisjunct, feature_preserve_backtrack_into)
         % pbone: I don't think so, the deep profiler doesn't seem to add this
         % feature to disjuncts that end in failure, it is probably a good idea
         % to add this annotation to prevent later compiler passes from breaking
         % the deep profiler.
-    ->
+    then
         coverage_prof_second_pass_goal(FirstDisjunct0, FirstDisjunct,
             CoverageBeforeKnown, NextCoverageBeforeKnown, !Info,
             AddedImpurity),
         Disjuncts = [FirstDisjunct, SecondDisjunct]
-    ;
+    else
         coverage_prof_second_pass_disj_2(DPInfo, CoverageBeforeKnown,
             coverage_before_known, NextCoverageBeforeKnown,
             Disjuncts0, Disjuncts, !Info, AddedImpurity)
@@ -472,14 +472,14 @@ coverage_prof_second_pass_disj_2(DPInfo,
     CPOptions = !.Info ^ ci_coverage_profiling_opts,
     CPOBranchDisj = CPOptions ^ cpo_branch_disj,
     DPInfo = dp_goal_info(IsMDProfInst, _),
-    (
+    ( if
         CPOBranchDisj = yes,
         CoverageBeforeKnown0 = coverage_before_unknown,
         IsMDProfInst = goal_is_not_mdprof_inst
-    ->
+    then
         InsertCP = yes,
         CoverageBeforeKnown = coverage_before_known
-    ;
+    else
         InsertCP = no,
         CoverageBeforeKnown = CoverageBeforeKnown0
     ),
@@ -495,7 +495,7 @@ coverage_prof_second_pass_disj_2(DPInfo,
     % Insert the coverage point if we decided to above.
     (
         InsertCP = yes,
-        DisjId = goal_info_get_goal_id(HeadDisjunct0 ^ hlds_goal_info),
+        DisjId = goal_info_get_goal_id(HeadDisjunct0 ^ hg_info),
         ContainingGoalMap = !.Info ^ ci_containing_goal_map,
         DisjPath = goal_id_to_reverse_path(ContainingGoalMap, DisjId),
         HeadCoveragePoint = coverage_point_info(DisjPath, cp_type_branch_arm),
@@ -572,14 +572,14 @@ coverage_prof_second_pass_switchcase_2(DPInfo, SwitchCanFail,
     CPOptions = !.Info ^ ci_coverage_profiling_opts,
     CPOBranchSwitch = CPOptions ^ cpo_branch_switch,
     DPInfo = dp_goal_info(IsMDProfInst, _),
-    (
+    ( if
         CPOBranchSwitch = yes,
         CoverageBeforeCaseKnown0 = coverage_before_unknown,
         IsMDProfInst = goal_is_not_mdprof_inst
-    ->
+    then
         InsertCP = yes,
         CoverageBeforeCaseKnown = coverage_before_known
-    ;
+    else
         InsertCP = no,
         CoverageBeforeCaseKnown = CoverageBeforeCaseKnown0
     ),
@@ -593,7 +593,7 @@ coverage_prof_second_pass_switchcase_2(DPInfo, SwitchCanFail,
     % Possibly insert coverage point at the start of the case.
     (
         InsertCP = yes,
-        CaseId = goal_info_get_goal_id(Goal0 ^ hlds_goal_info),
+        CaseId = goal_info_get_goal_id(Goal0 ^ hg_info),
         ContainingGoalMap = !.Info ^ ci_containing_goal_map,
         CasePath = goal_id_to_reverse_path(ContainingGoalMap, CaseId),
         CoveragePoint = coverage_point_info(CasePath, cp_type_branch_arm),
@@ -654,14 +654,14 @@ coverage_prof_second_pass_ite(DPInfo, ITEExistVars, Cond0, Then0, Else0,
     CPOptions = !.Info ^ ci_coverage_profiling_opts,
     CPOBranchIf = CPOptions ^ cpo_branch_ite,
     DPInfo = dp_goal_info(IsMDProfInst, _),
-    (
+    ( if
         CPOBranchIf = yes,
         IsMDProfInst = goal_is_not_mdprof_inst
-    ->
+    then
         ContainingGoalMap = !.Info ^ ci_containing_goal_map,
         (
             CoverageKnownBeforeThen0 = coverage_before_unknown,
-            ThenId = goal_info_get_goal_id(Then0 ^ hlds_goal_info),
+            ThenId = goal_info_get_goal_id(Then0 ^ hg_info),
             ThenPath = goal_id_to_reverse_path(ContainingGoalMap, ThenId),
             InsertCPThen = yes(coverage_point_info(ThenPath,
                 cp_type_branch_arm))
@@ -671,9 +671,9 @@ coverage_prof_second_pass_ite(DPInfo, ITEExistVars, Cond0, Then0, Else0,
         ),
         CoverageKnownBeforeThen = coverage_before_known,
 
-        ElseId = goal_info_get_goal_id(Else0 ^ hlds_goal_info),
+        ElseId = goal_info_get_goal_id(Else0 ^ hg_info),
         ElsePath = goal_id_to_reverse_path(ContainingGoalMap, ElseId),
-        CondDetism = goal_info_get_determinism(Cond ^ hlds_goal_info),
+        CondDetism = goal_info_get_determinism(Cond ^ hg_info),
         determinism_components(CondDetism, _, CondSolns),
         (
             CondSolns = at_most_many,
@@ -690,19 +690,19 @@ coverage_prof_second_pass_ite(DPInfo, ITEExistVars, Cond0, Then0, Else0,
 
             % Only insert a coverage point if we cannot infer the coverage
             % from before the ITE and before the then branch.
-            (
+            ( if
                 CoverageBeforeITEKnown = coverage_before_known,
                 CoverageKnownBeforeThen = coverage_before_known
-            ->
+            then
                 InsertCPElse = no,
                 CoverageKnownBeforeElse = coverage_before_known
-            ;
+            else
                 InsertCPElse = yes(coverage_point_info(ElsePath,
                     cp_type_branch_arm)),
                 CoverageKnownBeforeElse = coverage_before_known
             )
         )
-    ;
+    else
         % Don't insert any coverage points.
         InsertCPThen = no,
         InsertCPElse = no,
@@ -786,12 +786,12 @@ coverage_before_known_and(coverage_before_unknown, _) =
     goal_trivial::out) is det.
 
 goal_trivial_and(A, B, Trivial) :-
-    (
+    ( if
         A = goal_is_trivial,
         B = goal_is_trivial
-    ->
+    then
         Trivial = goal_is_trivial
-    ;
+    else
         Trivial = goal_is_nontrivial
     ).
 
@@ -800,12 +800,12 @@ goal_trivial_and(A, B, Trivial) :-
     port_counts_give_coverage_after::out) is det.
 
 port_counts_give_coverage_after_and(A, B, PortCountsCoverageAfter) :-
-    (
+    ( if
         A = port_counts_give_coverage_after,
         B = port_counts_give_coverage_after
-    ->
+    then
         PortCountsCoverageAfter = port_counts_give_coverage_after
-    ;
+    else
         PortCountsCoverageAfter = no_port_counts_give_coverage_after
     ).
 
@@ -833,7 +833,7 @@ has_port_counts_after(Goal, PCDirect, PCBefore, PC) :-
         % then they can be used to determine how often execution reaches the
         % point after this goal.
 
-        Detism = goal_info_get_determinism(Goal ^ hlds_goal_info),
+        Detism = goal_info_get_determinism(Goal ^ hg_info),
         has_port_counts_if_det(Detism, PCBefore, PC)
     ).
 
@@ -846,13 +846,13 @@ has_port_counts_after(Goal, PCDirect, PCBefore, PC) :-
 
 has_port_counts_if_det(Detism,
         PortCountsCoverageAfter0, PortCountsCoverageAfter) :-
-    (
+    ( if
         ( Detism = detism_det
         ; Detism = detism_cc_multi
         )
-    ->
+    then
         PortCountsCoverageAfter = PortCountsCoverageAfter0
-    ;
+    else
         PortCountsCoverageAfter = no_port_counts_give_coverage_after
     ).
 
@@ -879,13 +879,13 @@ coverage_prof_first_pass(CPOptions, Goal0, Goal, PortCountsCoverageAfterBefore,
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
     (
         % XXX: Not all call goals have associated call sites, therefore not all
-        % of these will have port counts.  For example inline foreign code does
-        % not get instrumented by the deep profiler.  See above in the
+        % of these will have port counts. For, example inline foreign code does
+        % not get instrumented by the deep profiler. See above in the
         % deep_profiling transformation.
         %
-        % This doesn't matter for the near future, since we're using a single
-        % pass coverage profiling algorithm.  This will need to be fixed when
-        % the two-pass coverage profiling is enabled.  Or if a naive assumption
+        % This doesn't matter for the near future, since we are using a single
+        % pass coverage profiling algorithm. This will need to be fixed when
+        % the two-pass coverage profiling is enabled. Or if a naive assumption
         % in the second pass is corrected, (See the XXX comment at the
         % beginning of coverage_prof_second_pass_goal regarding the defaults
         % that are assumed if the information from the first pass is not
@@ -893,9 +893,7 @@ coverage_prof_first_pass(CPOptions, Goal0, Goal, PortCountsCoverageAfterBefore,
         %
         GoalExpr0 = plain_call(_, _, _, BuiltinState, _, _),
         (
-            ( BuiltinState = out_of_line_builtin
-            ; BuiltinState = not_builtin
-            ),
+            BuiltinState = not_builtin,
             Trivial0 = goal_is_nontrivial,
             PortCountsCoverageAfterDirect = port_counts_give_coverage_after
         ;
@@ -1001,7 +999,7 @@ coverage_prof_first_pass(CPOptions, Goal0, Goal, PortCountsCoverageAfterBefore,
             PortCountsCoverageAfterElse, PortCountsCoverageAfterDirect)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ),
 
     (
@@ -1029,7 +1027,7 @@ coverage_prof_first_pass(CPOptions, Goal0, Goal, PortCountsCoverageAfterBefore,
         DPInfo = dp_goal_info(IsProfilingInstrumentation, yes(Info))
     ;
         MaybeDPInfo0 = no,
-        unexpected($module, $pred, "goal_dp_info not present")
+        unexpected($pred, "goal_dp_info not present")
     ),
     goal_info_set_maybe_dp_info(yes(DPInfo), GoalInfo0, GoalInfo),
     Goal = hlds_goal(GoalExpr, GoalInfo).
@@ -1073,7 +1071,7 @@ coverage_prof_first_pass_disj(CPOptions, [Goal0 | Goals0], [Goal | Goals],
     port_counts_give_coverage_after_and(PortCountsCoverageAfterGoal,
         PortCountsCoverageAfterDisj, PortCountsCoverageAfter).
 
-    % A switch is a special type of disjunction.  The important difference here
+    % A switch is a special type of disjunction. The important difference here
     % is that the coverage of the first case cannot be inferred from the
     % coverage before the switch.
     %
@@ -1109,9 +1107,9 @@ coverage_prof_first_pass_switchcase(CPOptions,
 
 insert_coverage_point_before(CPOptions, CPInfo, !Goal, !Info) :-
     make_coverage_point(CPOptions, CPInfo, CPGoals, !Info),
-    ( !.Goal = hlds_goal(conj(plain_conj, InnerGoals), _) ->
+    ( if !.Goal = hlds_goal(conj(plain_conj, InnerGoals), _) then
         Goals = CPGoals ++ InnerGoals
-    ;
+    else
         Goals = CPGoals ++ [!.Goal]
     ),
     create_conj_from_list(Goals, plain_conj, !:Goal).
@@ -1139,8 +1137,8 @@ make_coverage_point(CPOptions, CoveragePointInfo, Goals, !CoverageInfo) :-
         !:VarInfo = !.CoverageInfo ^ ci_var_info,
 
         generate_var("CPIndex", int_type, CPIndexVar, !VarInfo),
-        generate_deep_const_unify(int_const(CPIndex), CPIndexVar,
-            GoalUnifyIndex),
+        generate_deep_const_unify(some_int_const(int_const(CPIndex)),
+            CPIndexVar, GoalUnifyIndex),
         % When using dynamic coverage profiling we really on this variable
         % being optimised away later.
         generate_var("ProcLayout", c_pointer_type, ProcLayoutVar, !VarInfo),
@@ -1155,14 +1153,17 @@ make_coverage_point(CPOptions, CoveragePointInfo, Goals, !CoverageInfo) :-
 
     UseCalls = CPOptions ^ cpo_use_calls,
     ModuleInfo = !.CoverageInfo ^ ci_module_info,
-    Ground = ground(shared, none),
+    Ground = ground(shared, none_or_default_func),
     DataType = CPOptions ^ cpo_dynamic_coverage,
+    FromToGround = from_to_mode(Ground, Ground),
     (
         DataType = dynamic_coverage_data,
         PredName = "increment_dynamic_coverage_point_count",
         ArgVars = [CPIndexVar],
         make_foreign_args(ArgVars,
-            [(yes("CPIndex" - (Ground -> Ground)) - native_if_possible)],
+            [foreign_arg_name_mode_box(
+                yes(foreign_arg_name_mode("CPIndex", FromToGround)),
+                bp_native_if_possible)],
             [int_type], ForeignArgVars),
         PredArity = 1
     ;
@@ -1170,8 +1171,12 @@ make_coverage_point(CPOptions, CoveragePointInfo, Goals, !CoverageInfo) :-
         PredName = "increment_static_coverage_point_count",
         ArgVars = [ProcLayoutVar, CPIndexVar],
         make_foreign_args(ArgVars,
-            [(yes("ProcLayout" - (Ground -> Ground)) - native_if_possible),
-             (yes("CPIndex" - (Ground -> Ground)) - native_if_possible)],
+            [foreign_arg_name_mode_box(
+                yes(foreign_arg_name_mode("ProcLayout", FromToGround)),
+                bp_native_if_possible),
+            foreign_arg_name_mode_box(
+                yes(foreign_arg_name_mode("CPIndex", FromToGround)),
+                bp_native_if_possible)],
             [c_pointer_type, int_type], ForeignArgVars),
         PredArity = 2
     ),
@@ -1185,9 +1190,9 @@ make_coverage_point(CPOptions, CoveragePointInfo, Goals, !CoverageInfo) :-
         UseCalls = no,
         get_deep_profile_builtin_ppid(ModuleInfo, PredName, PredArity,
             PredId, ProcId),
-        coverage_point_ll_code(DataType, ForeignCallAttrs, ForeignCode),
+        coverage_point_ll_code(DataType, ForeignCallAttrs, ForeignProc),
         CallGoalExpr = call_foreign_proc(ForeignCallAttrs, PredId, ProcId,
-            ForeignArgVars, [], no, ForeignCode),
+            ForeignArgVars, [], no, ForeignProc),
         NonLocals = set_of_var.list_to_set(ArgVars),
         InstMapDelta = instmap_delta_from_assoc_list([]),
         CallGoalInfo = impure_init_goal_info(NonLocals, InstMapDelta,
@@ -1221,12 +1226,12 @@ coverage_points_map_list(Map, List) :-
 pred_proc_id(CoverageInfo, PredId, ProcId) :-
     MaybeRecInfo = CoverageInfo ^ ci_maybe_rec_info,
     PredProcId = CoverageInfo ^ ci_pred_proc_id,
-    (
+    ( if
         MaybeRecInfo = yes(RecInfo),
         RecInfo ^ dri_role = deep_prof_inner_proc(OuterPredProcId)
-    ->
+    then
         OuterPredProcId = proc(PredId, ProcId)
-    ;
+    else
         PredProcId = proc(PredId, ProcId)
     ).
 
@@ -1242,9 +1247,9 @@ proc_static_cons_id(CoverageInfo, ProcStaticConsId) :-
     % Returns a string containing the Low Level C code for a coverage point.
     %
 :- pred coverage_point_ll_code(coverage_data_type::in,
-    pragma_foreign_proc_attributes::out, pragma_foreign_code_impl::out) is det.
+    pragma_foreign_proc_attributes::out, pragma_foreign_proc_impl::out) is det.
 
-coverage_point_ll_code(CoverageDataType, ForeignProcAttrs, ForeignCodeImpl) :-
+coverage_point_ll_code(CoverageDataType, ForeignProcAttrs, ForeignProcImpl) :-
     some [!ForeignProcAttrs] (
         % XXX When running this code in a parallel grade, the contention for
         % the foreign code mutex may be very expensive. To improve this, we
@@ -1261,7 +1266,7 @@ coverage_point_ll_code(CoverageDataType, ForeignProcAttrs, ForeignCodeImpl) :-
             !ForeignProcAttrs),
         ForeignProcAttrs = !.ForeignProcAttrs
     ),
-    ForeignCodeImpl = fc_impl_ordinary(Code, no),
+    ForeignProcImpl = fp_impl_ordinary(Code, no),
     Code = coverage_point_ll_code(CoverageDataType).
 
 :- func coverage_point_ll_code(coverage_data_type) = string.
@@ -1365,34 +1370,7 @@ goal_info_get_dp_info(GoalInfo) = DPInfo :-
         MaybeDPInfo = yes(DPInfo)
     ;
         MaybeDPInfo = no,
-        unexpected($module, $pred, "MaybeDPInfo = no")
-    ).
-
-:- func goal_info_get_maybe_dp_coverage_info(hlds_goal_info) =
-    maybe(dp_coverage_goal_info).
-
-goal_info_get_maybe_dp_coverage_info(GoalInfo) = MaybeCoverageInfo :-
-    MaybeDPInfo = goal_info_get_maybe_dp_info(GoalInfo),
-    (
-        MaybeDPInfo = yes(DPInfo),
-        DPInfo = dp_goal_info(_, MaybeCoverageInfo)
-    ;
-        MaybeDPInfo = no,
-        MaybeCoverageInfo = no
-    ).
-
-:- func goal_get_maybe_dp_port_counts_coverage(hlds_goal) =
-    port_counts_give_coverage_after.
-
-goal_get_maybe_dp_port_counts_coverage(Goal) = PortCountsGiveCoverageAfter :-
-    Goal = hlds_goal(_, GoalInfo),
-    MaybeCoverageInfo = goal_info_get_maybe_dp_coverage_info(GoalInfo),
-    (
-        MaybeCoverageInfo =
-            yes(dp_coverage_goal_info(_, PortCountsGiveCoverageAfter))
-    ;
-        MaybeCoverageInfo = no,
-        PortCountsGiveCoverageAfter = no_port_counts_give_coverage_after
+        unexpected($pred, "MaybeDPInfo = no")
     ).
 
 %-----------------------------------------------------------------------------%
