@@ -1,28 +1,31 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2005-2006,2008-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: ctgc.selector.m.
 % Main author: nancy.
 %
 % Definition of predicates and functions for the manipulation of selectors.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module transform_hlds.ctgc.selector.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_pragma.
 
 :- import_module io.
 :- import_module list.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % An exception of this type is thrown a procedure would need to know the
     % type of a existentially typed node to proceed.
@@ -57,6 +60,12 @@
 :- pred selector_subsumed_by(module_info::in, normalization::in,
     selector::in, selector::in, mer_type::in, selector::out) is semidet.
 
+    % As above but fail if the subtype would be an existential type instead of
+    % aborting.
+    %
+:- pred type_of_node(module_info::in, mer_type::in, selector::in,
+    mer_type::out) is semidet.
+
     % Using the type information of the variable to which the given selector
     % belongs, normalize that selector.
     % S2 is the normalized form of S1 iff:
@@ -66,12 +75,6 @@
 :- pred normalize_selector_with_type_information(module_info::in, mer_type::in,
     selector::in, selector::out) is det.
 
-    % As above but fail if the subtype would be an existential type instead of
-    % aborting.
-    %
-:- pred type_of_node(module_info::in, mer_type::in, selector::in,
-    mer_type::out) is semidet.
-
     % Abbreviate a selector to the type of the node it selects.
     %
 :- pred selector_apply_widening(module_info::in, mer_type::in,
@@ -81,11 +84,12 @@
     %
 :- pred reset_tables(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
 :- import_module parse_tree.prog_type.
 
@@ -98,9 +102,8 @@
 :- import_module require.
 :- import_module set.
 :- import_module solutions.
-:- import_module string.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 top_selector = [].
 
@@ -112,7 +115,7 @@ selector_init(ConsId, Index) = [TermSel] :-
         TermSel = termsel(ConsId, Index)
     ;
         ( ConsId = closure_cons(_, _)
-        ; ConsId = int_const(_)
+        ; ConsId = some_int_const(_)
         ; ConsId = float_const(_)
         ; ConsId = char_const(_)
         ; ConsId = string_const(_)
@@ -125,10 +128,10 @@ selector_init(ConsId, Index) = [TermSel] :-
         ; ConsId = typeclass_info_const(_)
         ; ConsId = ground_term_const(_, _)
         ; ConsId = tabling_info_const(_)
-        ; ConsId = table_io_decl(_)
+        ; ConsId = table_io_entry_desc(_)
         ; ConsId = deep_profiling_proc_layout(_)
         ),
-        unexpected($module, $pred, "cannot handle cons_id")
+        unexpected($pred, "cannot handle cons_id")
     ).
 
 selector_init_from_list(Types)
@@ -150,13 +153,13 @@ selector_subsumed_by(ModuleInfo, Normalization, S1, S2, MainType, Extension) :-
         normalize_selector_with_type_information(ModuleInfo, MainType,
             S2, NormS2)
     ),
-    (
+    ( if
         only_term_selectors(NormS1),
         only_term_selectors(NormS2)
-    ->
+    then
         % Easy case.
         term_selector_subsumed_by(NormS1, NormS2, Extension)
-    ;
+    else
         selector_subsumed_by_2(ModuleInfo, NormS1, NormS2, MainType, Extension)
     ).
 
@@ -188,20 +191,20 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
     ;
         A = [AH | AT],
         B = [BH | BT],
-        (
+        ( if
             AH = termsel(ConsIdA, IndexA),
             BH = termsel(ConsIdB, IndexB)
-        ->
+        then
             ConsIdA = ConsIdB,
             IndexA = IndexB,
-            (
+            ( if
                 Type = type_variable(_, _),
                 AT = BT
-            ->
+            then
                 % XXX Avoid an assertion failure when trying to index arguments
                 % of a type variable.  This is probably a hack.
                 Extension = []
-            ;
+            else
                 % If both selectors begin with term selectors, clearly
                 % they must agree on the node to select for the selectors
                 % to be comparable.
@@ -209,7 +212,7 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
                     IndexA),
                 selector_subsumed_by_2(ModuleInfo, AT, BT, SubType, Extension)
             )
-        ;
+        else
             % If one selector has a term selector at the current position but
             % the other has a type selector, we select the node dictated by the
             % term selector.  We also verify that the type of that node
@@ -217,10 +220,10 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
             AH = termsel(ConsIdA, IndexA),
             BH = typesel(SubTypeB),
             SubTypeA = det_select_subtype(ModuleInfo, Type, ConsIdA, IndexA),
-            ( SubTypeA = SubTypeB ->
+            ( if SubTypeA = SubTypeB then
                 % Both selectors agree on the subtype to select.
                 selector_subsumed_by_2(ModuleInfo, AT, BT, SubTypeA, Extension)
-            ;
+            else
                 type_contains_subtype(ModuleInfo, SubTypeA, SubTypeB),
                 selector_subsumed_by_2(ModuleInfo, AT, B, SubTypeA, Extension)
             )
@@ -229,28 +232,28 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
             AH = typesel(SubTypeA),
             BH = termsel(ConsIdB, IndexB),
             SubTypeB = det_select_subtype(ModuleInfo, Type, ConsIdB, IndexB),
-            ( SubTypeA = SubTypeB ->
+            ( if SubTypeA = SubTypeB then
                 % Both selectors agree on the subtype to select.
                 selector_subsumed_by_2(ModuleInfo, AT, BT, SubTypeB, Extension)
-            ;
+            else
                 type_contains_subtype(ModuleInfo, SubTypeB, SubTypeA),
                 selector_subsumed_by_2(ModuleInfo, A, BT, SubTypeB, Extension)
             )
         ;
             AH = typesel(SubTypeA),
             BH = typesel(SubTypeB),
-            (
+            ( if
                 SubTypeA = SubTypeB
-            ->
+            then
                 % Both selectors begin with type selectors and agree on the
                 % subtype to select.
                 selector_subsumed_by_2(ModuleInfo, AT, BT, SubTypeB, Extension)
-            ;
+            else if
                 % Assume we select node according to the B selector, then check
                 % that the rest of B subsumes A.
                 type_contains_subtype(ModuleInfo, SubTypeB, SubTypeA),
                 selector_subsumed_by_2(ModuleInfo, A, BT, SubTypeB, Extension0)
-            ->
+            then
                 % Don't succeed for something like:
                 %   A   = [typesel(foo)],
                 %   B   = [typesel(bar)],
@@ -259,7 +262,7 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
                 %   [typesel(bar), typesel(foo)] = [typesel(bar)]
                 Extension0 \= A,
                 Extension = Extension0
-            ;
+            else
                 % Assume we select node according to the A selector, then check
                 % that B subsumes the rest of A.
                 type_contains_subtype(ModuleInfo, SubTypeA, SubTypeB),
@@ -278,9 +281,9 @@ selector_subsumed_by_2(ModuleInfo, A, B, Type, Extension) :-
     is semidet.
 
 type_contains_subtype(ModuleInfo, FromType, ToType) :-
-    ( FromType = ToType ->
+    ( if FromType = ToType then
         true
-    ;
+    else
         type_contains_subtype_1(ModuleInfo, FromType, ToType, Contains),
         Contains = yes
     ).
@@ -291,8 +294,9 @@ type_contains_subtype(ModuleInfo, FromType, ToType) :-
     % We assume that type definitions for a module don't change for the
     % duration of the analysis.
     %
-:- pragma memo(type_contains_subtype_1/4,
-    [allow_reset, specified([promise_implied, value, value, output])]).
+:- pragma memo(pred(type_contains_subtype_1/4),
+    [allow_reset, disable_warning_if_ignored,
+    specified([promise_implied, value, value, output])]).
 
 type_contains_subtype_1(ModuleInfo, FromType, ToType, Contains) :-
     queue.put(FromType, queue.init, Queue0),
@@ -308,34 +312,39 @@ type_contains_subtype_1(ModuleInfo, FromType, ToType, Contains) :-
     set(mer_type)::in, set(mer_type)::out, bool::out) is det.
 
 type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes, Contains) :-
-    ( queue.get(FromType, !Queue) ->
-        ( set.contains(!.SeenTypes, FromType) ->
-            type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes,
-                Contains)
-        ;
+    ( if queue.get(FromType, !Queue) then
+        ( if set.contains(!.SeenTypes, FromType) then
+            disable_warning [suspicious_recursion] (
+                type_contains_subtype_2(ModuleInfo, ToType,
+                    !Queue, !SeenTypes, Contains)
+            )
+        else
             set.insert(FromType, !SeenTypes),
             type_arg_types(ModuleInfo, FromType, ArgTypes),
-            ( list.member(ToType, ArgTypes) ->
+            ( if list.member(ToType, ArgTypes) then
                 Contains = yes
-            ;
+            else
                 queue.put_list(ArgTypes, !Queue),
-                type_contains_subtype_2(ModuleInfo, ToType, !Queue, !SeenTypes,
-                    Contains)
+                disable_warning [suspicious_recursion] (
+                    type_contains_subtype_2(ModuleInfo, ToType,
+                        !Queue, !SeenTypes, Contains)
+                )
             )
         )
-    ;
+    else
         Contains = no
     ).
 
 :- pred type_arg_types(module_info::in, mer_type::in, list(mer_type)::out)
     is det.
 
-:- pragma memo(type_arg_types/3,
-    [allow_reset, specified([promise_implied, value, output])]).
+:- pragma memo(pred(type_arg_types/3),
+    [allow_reset, disable_warning_if_ignored,
+    specified([promise_implied, value, output])]).
 
 type_arg_types(ModuleInfo, Type, ArgTypes) :-
     solutions(
-        (pred(ConsIdArgTypes::out) is nondet :-
+        ( pred(ConsIdArgTypes::out) is nondet :-
             cons_id_arg_types(ModuleInfo, Type, _ConsId, ConsIdArgTypes)
         ), ArgTypesLists),
     list.condense(ArgTypesLists, ArgTypes).
@@ -363,9 +372,9 @@ type_of_node(ModuleInfo, StartType, Selector, SubType) :-
 :- func det_select_subtype(module_info, mer_type, cons_id, int) = mer_type.
 
 det_select_subtype(ModuleInfo, Type, ConsID, Position) = SubType :-
-    ( select_subtype(ModuleInfo, Type, ConsID, Position, SubType0) ->
+    ( if select_subtype(ModuleInfo, Type, ConsID, Position, SubType0) then
         SubType = SubType0
-    ;
+    else
         throw(encounter_existential_subtype)
     ).
 
@@ -373,33 +382,41 @@ det_select_subtype(ModuleInfo, Type, ConsID, Position) = SubType :-
     mer_type::out) is semidet.
 
 select_subtype(ModuleInfo, Type, ConsID, Position, SubType) :-
-    (
+    ( if
         get_cons_id_non_existential_arg_types(ModuleInfo, Type, ConsID,
             ArgTypes)
-    ->
+    then
         SubType = list.det_index1(ArgTypes, Position)
-    ;
+    else if
         get_existq_cons_defn(ModuleInfo, Type, ConsID, CtorDefn)
-    ->
-        CtorDefn = ctor_defn(_TVarSet, ExistQVars, _KindMap, _Constraints,
+    then
+        CtorDefn = ctor_defn(_TVarSet, _KindMap, MaybeExistConstraints, 
             ArgTypes, _RetType),
+        (
+            MaybeExistConstraints = no_exist_constraints,
+            % XXX Should be able to optimise following test.
+            ExistQVars = []
+        ;
+            MaybeExistConstraints = exist_constraints(
+                cons_exist_constraints(ExistQVars, _, _, _))
+        ),
         SubType = list.det_index1(ArgTypes, Position),
         not (
             SubType = type_variable(TVar, _),
             list.member(TVar, ExistQVars)
         )
-    ;
-        unexpected($module, $pred,
-            "type is both existential and non-existential")
+    else
+        unexpected($pred, "type is both existential and non-existential")
     ).
 
-:- pragma memo(normalize_selector_with_type_information/4,
-    [allow_reset, specified([promise_implied, value, value, output])]).
+:- pragma memo(pred(normalize_selector_with_type_information/4),
+    [allow_reset, disable_warning_if_ignored,
+    specified([promise_implied, value, value, output])]).
 
 normalize_selector_with_type_information(ModuleInfo, Type, !Selector) :-
-    ( is_introduced_type_info_type(Type) ->
+    ( if is_introduced_type_info_type(Type) then
         true
-    ;
+    else
         branch_map_init(BranchMap0),
         branch_map_insert(Type, top_selector, BranchMap0, BranchMap1),
         do_normalize_selector(ModuleInfo, Type, BranchMap1, top_selector,
@@ -414,41 +431,45 @@ do_normalize_selector(ModuleInfo, VarType, BranchMap0,
     (
         !.Selector = [UnitSelector | SelRest],
         CtorCat = classify_type(ModuleInfo, VarType),
-        ( CtorCat = ctor_cat_user(CatUser) ->
+        ( if CtorCat = ctor_cat_user(CatUser) then
             (
-                CatUser = cat_user_general
-            ;
-                CatUser = cat_user_notag
+                ( CatUser = cat_user_general
+                ; CatUser = cat_user_notag
+                ; CatUser = cat_user_abstract_notag
+                )
             ;
                 CatUser = cat_user_direct_dummy,
                 % We should not be producing selectors for dummy types.
-                unexpected($module, $pred, "cat_user_direct_dummy")
+                unexpected($pred, "cat_user_direct_dummy")
+            ;
+                CatUser = cat_user_abstract_dummy,
+                % We should not be producing selectors for dummy types.
+                unexpected($pred, "cat_user_abstract_dummy")
             ),
 
             % If it is either a term-selector of a non-existentially typed
             % functor or is a type-selector, construct the branch map and
             % proceed with normalization. If it is a term-selector of an
             % existentially typed functor, then normalization stops.
-            (
+            ( if
                 (
                     UnitSelector = termsel(ConsId, Index),
                     get_cons_id_non_existential_arg_types(ModuleInfo,
                         VarType, ConsId, ArgTypes),
-                    ( list.index1(ArgTypes, Index, SubType) ->
+                    ( if list.index1(ArgTypes, Index, SubType) then
                         CType = SubType
-                    ;
-                        unexpected($module, $pred,
-                            "accessing nonexistent index")
+                    else
+                        unexpected($pred, "accessing nonexistent index")
                     )
                 ;
                     UnitSelector = typesel(CType)
                 )
-            ->
+            then
                 !:Selector = SelRest,
-                ( branch_map_search(BranchMap0, CType, BranchSelector) ->
+                ( if branch_map_search(BranchMap0, CType, BranchSelector) then
                     do_normalize_selector(ModuleInfo, CType, BranchMap0,
                         BranchSelector, !Selector)
-                ;
+                else
                     selector_termshift(SelectorAcc0, [UnitSelector],
                         SelectorAcc1),
                     branch_map_insert(CType, SelectorAcc1,
@@ -456,13 +477,13 @@ do_normalize_selector(ModuleInfo, VarType, BranchMap0,
                     do_normalize_selector(ModuleInfo, CType, BranchMap1,
                         SelectorAcc1, !Selector)
                 )
-            ;
+            else
                 % Existentially typed functor.
                 % XXX awaiting confirmation on this from Nancy but it seems
                 % right to me --pw
                 append(SelectorAcc0, [UnitSelector], !:Selector)
             )
-        ;
+        else
             % If it is not a user type, SelRest is empty anyhow, and
             % normalization stops.
             % Resulting selector = accumulator.sel0
@@ -479,16 +500,16 @@ selector_apply_widening(ModuleInfo, MainType, Selector0, Selector) :-
         Selector = []
     ;
         Selector0 = [_ | _],
-        ( type_of_node(ModuleInfo, MainType, Selector0, SubType) ->
+        ( if type_of_node(ModuleInfo, MainType, Selector0, SubType) then
             Selector = [typesel(SubType)]
-        ;
+        else
             % The node is existentially typed.  Let's just leave the selector
             % as-is.
             Selector = Selector0
         )
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % BRANCH_MAP : copy/pasted from wimvh/bta_reduce.m
 %
@@ -511,22 +532,22 @@ branch_map_search([Type - Sel | TypeSels], KeyType, ValueSel):-
     map.init(Empty),
     % The two types are considered equal if they unify under an
     % empty substitution.
-    (
+    ( if
         type_unify(Type, KeyType, [], Empty, Subst),
         map.is_empty(Subst)
-    ->
+    then
         ValueSel = Sel
-    ;
+    else
         branch_map_search(TypeSels, KeyType, ValueSel)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 reset_tables(!IO) :-
     table_reset_for_type_contains_subtype_1_4(!IO),
-    table_reset_for_type_arg_types_3(!IO).
+    table_reset_for_type_arg_types_3(!IO),
     table_reset_for_normalize_selector_with_type_information_4(!IO).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module transform_hlds.ctgc.selector.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
