@@ -14,6 +14,7 @@
 :- interface.
 
 :- import_module ll_backend.llds.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
@@ -27,6 +28,9 @@
     %
 :- pred const_is_constant(rval_const::in, exprn_opts::in, bool::out) is det.
 
+:- pred vars_in_rval(rval::in, list(prog_var)::out) is det.
+:- pred vars_in_lval(lval::in, list(prog_var)::out) is det.
+
     % transform_lval_in_instr(Transform, !Instr, !Acc):
     %
     % Transform all lvals in !.Instr with the predicate Transform.
@@ -68,10 +72,6 @@
 :- pred substitute_rvals_in_rval(assoc_list(rval, rval)::in,
     rval::in, rval::out) is det.
 
-:- pred vars_in_lval(lval::in, list(prog_var)::out) is det.
-
-:- pred vars_in_rval(rval::in, list(prog_var)::out) is det.
-
 :- pred simplify_rval(rval::in, rval::out) is det.
 
     % The following predicates take an lval/rval (list)
@@ -79,15 +79,11 @@
     %
 :- pred rval_list_addrs(list(rval)::in,
     list(code_addr)::out, list(data_id)::out) is det.
-
 :- pred lval_list_addrs(list(lval)::in,
     list(code_addr)::out, list(data_id)::out) is det.
 
-:- pred rval_addrs(rval::in, list(code_addr)::out, list(data_id)::out)
-    is det.
-
-:- pred lval_addrs(lval::in, list(code_addr)::out, list(data_id)::out)
-    is det.
+:- pred rval_addrs(rval::in, list(code_addr)::out, list(data_id)::out) is det.
+:- pred lval_addrs(lval::in, list(code_addr)::out, list(data_id)::out) is det.
 
 :- func var_lval_to_rval(prog_var, lval) = rval.
 
@@ -110,12 +106,31 @@ const_is_constant(Const, ExprnOpts, IsConst) :-
         ( Const = llconst_true
         ; Const = llconst_false
         ; Const = llconst_int(_)
+        ; Const = llconst_uint(_)
+        ; Const = llconst_int8(_)
+        ; Const = llconst_uint8(_)
+        ; Const = llconst_int16(_)
+        ; Const = llconst_uint16(_)
+        ; Const = llconst_int32(_)
+        ; Const = llconst_uint32(_)
         ; Const = llconst_foreign(_, _)
         ; Const = llconst_string(_)
         ; Const = llconst_multi_string(_)
         ; Const = llconst_data_addr(_, _)
         ),
         IsConst = yes
+    ;
+        ( Const = llconst_int64(_)
+        ; Const = llconst_uint64(_)
+        ),
+        SGInt64s = ExprnOpts ^ static_ground_int64s,
+        (
+            SGInt64s = have_static_ground_int64s,
+            IsConst = yes
+        ;
+            SGInt64s = do_not_have_static_ground_int64s,
+            IsConst = no
+        )
     ;
         Const = llconst_float(_),
         SGFloats = ExprnOpts ^ static_ground_floats,
@@ -197,6 +212,8 @@ vars_in_rval(mkword(_, Rval), Vars) :-
     vars_in_rval(Rval, Vars).
 vars_in_rval(mkword_hole(_Tag), []).
 vars_in_rval(const(_Conts), []).
+vars_in_rval(cast(_Type, Rval), Vars) :-
+    vars_in_rval(Rval, Vars).
 vars_in_rval(unop(_Unop, Rval), Vars) :-
     vars_in_rval(Rval, Vars).
 vars_in_rval(binop(_Binop, RvalA, RvalB), Vars) :-
@@ -522,6 +539,10 @@ transform_lval_in_rval(Transform, Rval0, Rval, !Acc) :-
     ;
         Rval0 = const(_Const),
         Rval = Rval0
+    ;
+        Rval0 = cast(Type, Rval1),
+        transform_lval_in_rval(Transform, Rval1, Rval2, !Acc),
+        Rval = cast(Type, Rval2)
     ;
         Rval0 = unop(Unop, Rval1),
         transform_lval_in_rval(Transform, Rval1, Rval2, !Acc),
@@ -574,10 +595,10 @@ substitute_lval_in_rval(OldLval, NewLval, Rval0, Rval) :-
     lval::in, lval::out, int::in, int::out) is det.
 
 substitute_lval_in_lval_count(OldLval, NewLval, Lval0, Lval, !N) :-
-    ( Lval0 = OldLval ->
+    ( if Lval0 = OldLval then
         Lval = NewLval,
         !:N = !.N + 1
-    ;
+    else
         substitute_lval_in_lval_count_2(OldLval, NewLval, Lval0, Lval, !N)
     ).
 
@@ -635,9 +656,9 @@ substitute_lval_in_lval_count_2(OldLval, NewLval, Lval0, Lval, !N) :-
     ).
 
 substitute_rval_in_rval(OldRval, NewRval, Rval0, Rval) :-
-    ( Rval0 = OldRval ->
+    ( if Rval0 = OldRval then
         Rval = NewRval
-    ;
+    else
         (
             Rval0 = lval(Lval0),
             substitute_rval_in_lval(OldRval, NewRval, Lval0, Lval),
@@ -655,6 +676,10 @@ substitute_rval_in_rval(OldRval, NewRval, Rval0, Rval) :-
         ;
             Rval0 = const(_Const),
             Rval = Rval0
+        ;
+            Rval0 = cast(Type, Rval1),
+            substitute_rval_in_rval(OldRval, NewRval, Rval1, Rval2),
+            Rval = cast(Type, Rval2)
         ;
             Rval0 = unop(Unop, Rval1),
             substitute_rval_in_rval(OldRval, NewRval, Rval1, Rval2),
@@ -778,119 +803,41 @@ substitute_rvals_in_rval_2([Left - Right | Rest], !Rval) :-
 %---------------------------------------------------------------------------%
 
 simplify_rval(Rval0, Rval) :-
-    ( simplify_rval_2(Rval0, Rval1) ->
+    ( if simplify_rval_2(Rval0, Rval1) then
         simplify_rval(Rval1, Rval)
-    ;
+    else
         Rval = Rval0
     ).
 
 :- pred simplify_rval_2(rval::in, rval::out) is semidet.
 
 simplify_rval_2(Rval0, Rval) :-
-    (
+    ( if
         Rval0 = lval(field(MaybeTag, Rval1, Num)),
         simplify_rval_2(Rval1, Rval2)
-    ->
+    then
         Rval = lval(field(MaybeTag, Rval2, Num))
-    ;
+    else if
         Rval0 = unop(UnOp, Rval1),
         simplify_rval_2(Rval1, Rval2)
-    ->
+    then
         Rval = unop(UnOp, Rval2)
-    ;
+    else if
         Rval0 = binop(BinOp, Rval1, Rval2),
         simplify_rval_2(Rval1, Rval3)
-    ->
+    then
         Rval = binop(BinOp, Rval3, Rval2)
-    ;
+    else if
         Rval0 = binop(BinOp, Rval1, Rval2),
         simplify_rval_2(Rval2, Rval3)
-    ->
+    then
         Rval = binop(BinOp, Rval1, Rval3)
-    ;
+    else
         fail
     ).
 
-:- pred simplify_args(list(maybe(rval))::in, list(maybe(rval))::out) is det.
-
-simplify_args([], []).
-simplify_args([MR0 | Ms0], [MR | Ms]) :-
-    simplify_args(Ms0, Ms),
-    simplify_arg(MR0, MR).
-
-:- pred simplify_arg(maybe(rval)::in, maybe(rval)::out) is det.
-
-simplify_arg(MaybeRval0, MaybeRval) :-
-    (
-        MaybeRval0 = yes(Rval0),
-        simplify_rval_2(Rval0, Rval)
-    ->
-        MaybeRval = yes(Rval)
-    ;
-        MaybeRval = MaybeRval0
-    ).
-
 %-----------------------------------------------------------------------------%
 
-rval_addrs(lval(Lval), CodeAddrs, DataIds) :-
-    lval_addrs(Lval, CodeAddrs, DataIds).
-rval_addrs(var(_Var), [], []).
-rval_addrs(mkword(_Tag, Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-rval_addrs(mkword_hole(_Tag), [], []).
-rval_addrs(const(Const), CodeAddrs, DataIds) :-
-    ( Const = llconst_code_addr(CodeAddress) ->
-        CodeAddrs = [CodeAddress],
-        DataIds = []
-    ; Const = llconst_data_addr(DataId, _) ->
-        CodeAddrs = [],
-        DataIds = [DataId]
-    ;
-        CodeAddrs = [],
-        DataIds = []
-    ).
-rval_addrs(unop(_Unop, Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-rval_addrs(binop(_Binop, RvalA, RvalB), CodeAddrs, DataIds) :-
-    rval_addrs(RvalA, CodeAddrsA, DataIdsA),
-    rval_addrs(RvalB, CodeAddrsB, DataIdsB),
-    CodeAddrs = CodeAddrsA ++ CodeAddrsB,
-    DataIds = DataIdsA ++ DataIdsB.
-rval_addrs(mem_addr(Rval), CodeAddrs, DataIds) :-
-    mem_ref_addrs(Rval, CodeAddrs, DataIds).
-
-lval_addrs(reg(_Type, _RegNum), [], []).
-lval_addrs(stackvar(_SlotNum), [], []).
-lval_addrs(parent_stackvar(_SlotNum), [], []).
-lval_addrs(framevar(_SlotNum), [], []).
-lval_addrs(double_stackvar(_Type, _SlotNum), [], []).
-lval_addrs(succip, [], []).
-lval_addrs(maxfr, [], []).
-lval_addrs(curfr, [], []).
-lval_addrs(prevfr_slot(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(succfr_slot(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(redofr_slot(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(redoip_slot(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(succip_slot(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(hp, [], []).
-lval_addrs(sp, [], []).
-lval_addrs(parent_sp, [], []).
-lval_addrs(field(_Tag, RvalA, RvalB), CodeAddrs, DataIds) :-
-    rval_addrs(RvalA, CodeAddrsA, DataIdsA),
-    rval_addrs(RvalB, CodeAddrsB, DataIdsB),
-    CodeAddrs = CodeAddrsA ++ CodeAddrsB,
-    DataIds = DataIdsA ++ DataIdsB.
-lval_addrs(lvar(_Var), [], []).
-lval_addrs(temp(_Type, _TmpNum), [], []).
-lval_addrs(mem_ref(Rval), CodeAddrs, DataIds) :-
-    rval_addrs(Rval, CodeAddrs, DataIds).
-lval_addrs(global_var_ref(_), [], []).
-
 rval_list_addrs([], [], []).
 rval_list_addrs([Rval | Rvals], CodeAddrs, DataIds) :-
     rval_addrs(Rval, HeadCodeAddrs, HeadDataIds),
@@ -905,6 +852,81 @@ lval_list_addrs([Lval | Lvals], CodeAddrs, DataIds) :-
     CodeAddrs = HeadCodeAddrs ++ TailCodeAddrs,
     DataIds = HeadDataIds ++ TailDataIds.
 
+rval_addrs(Rval, CodeAddrs, DataIds) :-
+    (
+        Rval = lval(Lval),
+        lval_addrs(Lval, CodeAddrs, DataIds)
+    ;
+        ( Rval = var(_Var)
+        ; Rval = mkword_hole(_Tag)
+        ),
+        CodeAddrs = [],
+        DataIds = []
+    ;
+        Rval = const(Const),
+        ( if Const = llconst_code_addr(CodeAddress) then
+            CodeAddrs = [CodeAddress],
+            DataIds = []
+        else if Const = llconst_data_addr(DataId, _) then
+            CodeAddrs = [],
+            DataIds = [DataId]
+        else
+            CodeAddrs = [],
+            DataIds = []
+        )
+    ;
+        ( Rval = mkword(_Tag, SubRval)
+        ; Rval = cast(_Type, SubRval)
+        ; Rval = unop(_Unop, SubRval)
+        ),
+        rval_addrs(SubRval, CodeAddrs, DataIds)
+    ;
+        Rval = binop(_Binop, SubRvalA, SubRvalB),
+        rval_addrs(SubRvalA, CodeAddrsA, DataIdsA),
+        rval_addrs(SubRvalB, CodeAddrsB, DataIdsB),
+        CodeAddrs = CodeAddrsA ++ CodeAddrsB,
+        DataIds = DataIdsA ++ DataIdsB
+    ;
+        Rval = mem_addr(SubRval),
+        mem_ref_addrs(SubRval, CodeAddrs, DataIds)
+    ).
+
+lval_addrs(Lval, CodeAddrs, DataIds) :-
+    (
+        ( Lval = reg(_Type, _RegNum)
+        ; Lval = stackvar(_SlotNum)
+        ; Lval = parent_stackvar(_SlotNum)
+        ; Lval = framevar(_SlotNum)
+        ; Lval = double_stackvar(_Type, _SlotNum)
+        ; Lval = succip
+        ; Lval = maxfr
+        ; Lval = curfr
+        ; Lval = hp
+        ; Lval = sp
+        ; Lval = parent_sp
+        ; Lval = temp(_Type, _TmpNum)
+        ; Lval = lvar(_Var)
+        ; Lval = global_var_ref(_)
+        ),
+        CodeAddrs = [],
+        DataIds = []
+    ;
+        ( Lval = prevfr_slot(Rval)
+        ; Lval = succfr_slot(Rval)
+        ; Lval = redofr_slot(Rval)
+        ; Lval = redoip_slot(Rval)
+        ; Lval = succip_slot(Rval)
+        ; Lval = mem_ref(Rval)
+        ),
+        rval_addrs(Rval, CodeAddrs, DataIds)
+    ;
+        Lval = field(_Tag, SubRvalA, SubRvalB),
+        rval_addrs(SubRvalA, CodeAddrsA, DataIdsA),
+        rval_addrs(SubRvalB, CodeAddrsB, DataIdsB),
+        CodeAddrs = CodeAddrsA ++ CodeAddrsB,
+        DataIds = DataIdsA ++ DataIdsB
+    ).
+
 :- pred mem_ref_addrs(mem_ref::in,
     list(code_addr)::out, list(data_id)::out) is det.
 
@@ -913,25 +935,6 @@ mem_ref_addrs(framevar_ref(_SlotNum), [], []).
 mem_ref_addrs(heap_ref(Rval, _MaybeTag, _FieldNum), CodeAddrs, DataIds) :-
     rval_addrs(Rval, CodeAddrs, DataIds).
 
-    % Give a list of maybe(rval), return a list of the code and data
-    % addresses that are referenced by that list.
-    %
-:- pred maybe_rval_list_addrs(list(maybe(rval))::in,
-    list(code_addr)::out, list(data_id)::out) is det.
-
-maybe_rval_list_addrs([], [], []).
-maybe_rval_list_addrs([MaybeRval | MaybeRvals], CodeAddrs, DataIds) :-
-    (
-        MaybeRval = yes(Rval),
-        rval_addrs(Rval, HeadCodeAddrs, HeadDataIds),
-        maybe_rval_list_addrs(MaybeRvals, TailCodeAddrs, TailDataIds),
-        CodeAddrs = HeadCodeAddrs ++ TailCodeAddrs,
-        DataIds = HeadDataIds ++ TailDataIds
-    ;
-        MaybeRval = no,
-        maybe_rval_list_addrs(MaybeRvals, CodeAddrs, DataIds)
-    ).
-
 var_lval_to_rval(_Var, Lval) = lval(Lval).
 
 lval_to_rval(Lval) = lval(Lval).
