@@ -5,11 +5,11 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-% 
+%
 % File: follow_code.m.
 % Main author: conway.
 % Extensive modifications by zs.
-% 
+%
 % The problem attacked by this module is that sometimes the code generator
 % doesn't know where it should put the values of live variables at the end
 % of a branched control structure. All branches must put each live variable
@@ -25,12 +25,13 @@
 % where every live variable must be. If a branched control structure is
 % followed by builtin goals such as unifications, we push those goals into
 % each branch.
-% 
+%
 %-----------------------------------------------------------------------------%
 
 :- module ll_backend.follow_code.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -44,20 +45,22 @@
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module hlds.code_model.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
 :- import_module hlds.quantification.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_detism.
 :- import_module parse_tree.set_of_var.
 
 :- import_module bool.
 :- import_module list.
 :- import_module require.
-:- import_module set.
 
 %-----------------------------------------------------------------------------%
 
@@ -77,7 +80,7 @@ move_follow_code_in_proc(_PredProcId, !ProcInfo, !ModuleInfo) :-
             HeadVars, _Warnings, Goal1, Goal2,
             Varset0, Varset, VarTypes0, VarTypes,
             RttiVarMaps0, RttiVarMaps),
-        proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InstMap0),
+        proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InstMap0),
         proc_info_get_inst_varset(!.ProcInfo, InstVarSet),
         recompute_instmap_delta(do_not_recompute_atomic_instmap_deltas,
             Goal2, Goal, VarTypes, InstVarSet, InstMap0, !ModuleInfo),
@@ -136,14 +139,14 @@ move_follow_code_in_goal(Goal0, Goal, RttiVarMaps, !Changed) :-
         Goal = hlds_goal(GoalExpr, GoalInfo)
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             SubGoal = SubGoal0
-        ;
+        else
             move_follow_code_in_goal(SubGoal0, SubGoal, RttiVarMaps, !Changed)
         ),
         GoalExpr = scope(Reason, SubGoal),
@@ -158,7 +161,7 @@ move_follow_code_in_goal(Goal0, Goal, RttiVarMaps, !Changed) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -173,8 +176,8 @@ move_follow_code_in_independent_goals([], [], _, !Changed).
 move_follow_code_in_independent_goals([Goal0 | Goals0], [Goal | Goals],
         RttiVarMaps, !Changed) :-
     move_follow_code_in_goal(Goal0, Goal, RttiVarMaps, !Changed),
-    move_follow_code_in_independent_goals(Goals0, Goals, RttiVarMaps,
-        !Changed).
+    move_follow_code_in_independent_goals(Goals0, Goals,
+        RttiVarMaps, !Changed).
 
 %-----------------------------------------------------------------------------%
 
@@ -210,7 +213,7 @@ move_follow_code_in_conj_2([], _ConjPurity, _RttiVarMaps, !RevPrevGoals,
         !Changed).
 move_follow_code_in_conj_2([Goal0 | Goals0], ConjPurity, RttiVarMaps,
         !RevPrevGoals, !Changed) :-
-    (
+    ( if
         Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
         goal_util.goal_is_branched(GoalExpr0),
         % A goal that has the mode_check_clauses marker on it will not
@@ -236,11 +239,11 @@ move_follow_code_in_conj_2([Goal0 | Goals0], ConjPurity, RttiVarMaps,
         ),
         move_follow_code_move_goals(Goal0, FollowGoals, WorstPurity,
             Goal1Prime)
-    ->
+    then
         !:Changed = yes,
         Goal1 = Goal1Prime,
         RestGoals = RestGoalsPrime
-    ;
+    else
         Goal1 = Goal0,
         RestGoals = Goals0
     ),
@@ -271,7 +274,7 @@ move_follow_code_select([], _, [], [], !Purity).
 move_follow_code_select([Goal | Goals], RttiVarMaps, FollowGoals, RestGoals,
         !Purity) :-
     Goal = hlds_goal(GoalExpr, GoalInfo),
-    (
+    ( if
         move_follow_code_is_builtin(GoalExpr),
 
         % Don't attempt to move existentially typed deconstructions
@@ -284,20 +287,20 @@ move_follow_code_select([Goal | Goals], RttiVarMaps, FollowGoals, RestGoals,
         % which would allow separate type variables for each branch and
         % avoid the above confusion.
         %
-        \+ (
+        not (
             GoalExpr = unify(_, _, _, Unification, _),
             Unification = deconstruct(_, _, Args, _, _, _),
             list.member(Arg, Args),
             rtti_varmaps_var_info(RttiVarMaps, Arg, RttiVarInfo),
             RttiVarInfo \= non_rtti_var
         )
-    ->
+    then
         GoalPurity = goal_info_get_purity(GoalInfo),
         !:Purity = worst_purity(!.Purity, GoalPurity),
         move_follow_code_select(Goals, RttiVarMaps, FollowGoals0, RestGoals,
             !Purity),
         FollowGoals = [Goal | FollowGoals0]
-    ;
+    else
         FollowGoals = [],
         RestGoals = [Goal | Goals]
     ).
@@ -382,9 +385,9 @@ follow_code_conjoin_goal_and_goal_list(Goal0, FollowGoals, FollowPurity,
         ; MaxSolns0 = at_most_many_cc
         ),
         check_follow_code_detism(FollowGoals, Detism0),
-        ( GoalExpr0 = conj(plain_conj, Conjuncts0) ->
+        ( if GoalExpr0 = conj(plain_conj, Conjuncts0) then
             GoalExpr = conj(plain_conj, Conjuncts0 ++ FollowGoals)
-        ;
+        else
             GoalExpr = conj(plain_conj, [Goal0 | FollowGoals])
         ),
         OldPurity = goal_info_get_purity(GoalInfo0),
