@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2003-2012 The University of Melbourne.
+% Copyright (C) 2013, 2015-2018 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -17,14 +18,18 @@
 :- module ll_backend.global_data.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_pred.
+:- import_module libs.
+:- import_module libs.optimization_options.
 :- import_module ll_backend.continuation_info.
 :- import_module ll_backend.layout.
 :- import_module ll_backend.llds.
-:- import_module mdbcomp.prim_data.     % for module_name
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.     % for module_name
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
-:- import_module bool.
 :- import_module list.
 :- import_module map.
 :- import_module set_tree234.
@@ -78,17 +83,6 @@
 :- pred global_data_set_static_cell_info(static_cell_info::in,
     global_data::in, global_data::out) is det.
 
-:- type static_cell_info.
-
-:- func init_static_cell_info(module_name, have_unboxed_floats, bool)
-    = static_cell_info.
-
-:- pred add_scalar_static_cell(list(typed_rval)::in, data_id::out,
-    static_cell_info::in, static_cell_info::out) is det.
-
-:- pred add_scalar_static_cell_natural_types(list(rval)::in, data_id::out,
-    static_cell_info::in, static_cell_info::out) is det.
-
 :- pred global_data_add_new_alloc_sites(set_tree234(alloc_site_info)::in,
     global_data::in, global_data::out) is det.
 
@@ -96,27 +90,54 @@
     list(alloc_site_info)::out, map(alloc_site_id, layout_slot_name)::out)
     is det.
 
-:- pred find_general_llds_types(have_unboxed_floats::in, list(mer_type)::in,
-    list(list(rval))::in, list(llds_type)::out) is semidet.
+%-----------------------------------------------------------------------------%
 
-:- pred add_vector_static_cell(list(llds_type)::in,
-    list(list(rval))::in, data_id::out,
+:- type static_cell_info.
+
+:- func init_static_cell_info(module_name, have_unboxed_floats,
+    have_unboxed_int64s, maybe_use_common_data) = static_cell_info.
+
+:- pred add_scalar_static_cell_natural_types(list(rval)::in, data_id::out,
+    static_cell_info::in, static_cell_info::out) is det.
+
+:- pred add_scalar_static_cell(list(typed_rval)::in, data_id::out,
     static_cell_info::in, static_cell_info::out) is det.
 
 :- pred search_scalar_static_cell_offset(static_cell_info::in, data_id::in,
     int::in, rval::out) is semidet.
 
+:- pred find_general_llds_types(have_unboxed_floats::in,
+    have_unboxed_int64s::in, list(mer_type)::in, list(list(rval))::in,
+    list(llds_type)::out) is semidet.
+
+:- pred add_vector_static_cell(list(llds_type)::in,
+    list(list(rval))::in, data_id::out,
+    static_cell_info::in, static_cell_info::out) is det.
+
 :- pred get_static_cells(static_cell_info::in,
     list(scalar_common_data_array)::out, list(vector_common_data_array)::out)
     is det.
 
-    % Given an rval, the value of the --unboxed_float option and the width of
-    % the constructor argument, figure out the type the rval would have as an
-    % argument. Normally that's the same as its usual type; the exception is
-    % that for boxed floats, the type is data_ptr (i.e. the type of the boxed
-    % value) rather than float (the type of the unboxed value).
+%-----------------------------------------------------------------------------%
+
+    % The arguments of functors are stored in memory cells, either on the heap
+    % or in static data. While the arg_width type represents the space taken up
+    % by a single argument of such a functor, the num_words type represents
+    % the space taken up by one *or more* functors, *after* they have been
+    % packed together into words.
+:- type num_words
+    --->    one_word
+    ;       two_words.
+
+    % Given an rval, the value of the --unboxed-float option, the value of the
+    % --unboxed-int64s and the width of the constructor argument, figure out
+    % the type the rval would have as an argument. Normally that is the same as
+    % its usual type; the exception is that for boxed floats, boxed int64s and
+    % boxed uint64s, the type is data_ptr (i.e. the type of the boxed value)
+    % rather than float, int64, uint64 (the type of the unboxed value).
     %
-:- func rval_type_as_arg(have_unboxed_floats, arg_width, rval) = llds_type.
+:- func rval_type_as_arg(have_unboxed_floats, have_unboxed_int64s, num_words,
+    rval) = llds_type.
 
 %-----------------------------------------------------------------------------%
 
@@ -134,8 +155,8 @@
     % merge_global_datas(GlobalDataA, GlobalDataB, GlobalData, Remap)
     %
     % Merge two global data structures, where static cell information from
-    % GlobalDataA takes precedence over GlobalDataB.  The type numbers of the
-    % two global_data structures must be distinct.  Remap contains the
+    % GlobalDataA takes precedence over GlobalDataB. The type numbers of the
+    % two global_data structures must be distinct. Remap contains the
     % information necessary for remap_static_cell_references/3.
     %
 :- pred merge_global_datas(global_data::in, global_data::in, global_data::out,
@@ -153,8 +174,6 @@
 
 :- implementation.
 
-:- import_module ll_backend.layout.
-
 :- import_module assoc_list.
 :- import_module bimap.
 :- import_module counter.
@@ -185,11 +204,11 @@
                 gd_closure_layouts          :: list(closure_proc_id_data),
 
                 % A table for allocating and maintaining slots where string IDs
-                % will be placed at runtime for threadscope profiling.  The
-                % actual string IDs are allocated at runtime and their IDs are
-                % placed in an array slot which can be referred to statically.
-                % The size of the table is maintained for allocating offsets
-                % into it.
+                % will be placed at runtime for threadscope profiling.
+                % The actual string IDs are allocated at runtime and their IDs
+                % are placed in an array slot which can be referred to
+                % statically. The size of the table is maintained for
+                % allocating offsets into it.
                 gd_ts_string_table_size     :: int,
                 gd_ts_rev_string_table      :: list(string),
 
@@ -330,7 +349,8 @@ make_alloc_id_map(AllocSite, Slot, Slot + 1, !Map) :-
     --->    static_cell_sub_info(
                 scsi_module_name            :: module_name, % base file name
                 scsi_unbox_float            :: have_unboxed_floats,
-                scsi_common_data            :: bool
+                scsi_unbox_int64s           :: have_unboxed_int64s,
+                scsi_common_data            :: maybe_use_common_data
             ).
 
 :- type cell_type_bimap == bimap(common_cell_type, type_num).
@@ -354,8 +374,9 @@ make_alloc_id_map(AllocSite, Slot, Slot + 1, !Map) :-
                 sci_vector_cell_group_map   :: vector_type_cell_map
             ).
 
-init_static_cell_info(BaseName, UnboxFloat, CommonData) = Info0 :-
-    SubInfo0 = static_cell_sub_info(BaseName, UnboxFloat, CommonData),
+init_static_cell_info(BaseName, UnboxFloat, UnboxInt64s, CommonData) = Info0 :-
+    SubInfo0 = static_cell_sub_info(BaseName, UnboxFloat, UnboxInt64s,
+        CommonData),
     Info0 = static_cell_info(SubInfo0, counter.init(0), bimap.init,
         map.init, map.init).
 
@@ -363,16 +384,17 @@ init_static_cell_info(BaseName, UnboxFloat, CommonData) = Info0 :-
 
 add_scalar_static_cell_natural_types(Args, DataId, !Info) :-
     UnboxFloat = !.Info ^ sci_sub_info ^ scsi_unbox_float,
-    ArgWidth = full_word,
-    list.map(associate_natural_type(UnboxFloat, ArgWidth), Args, TypedArgs),
+    UnboxInt64s = !.Info ^ sci_sub_info ^ scsi_unbox_int64s,
+    list.map(associate_natural_type(UnboxFloat, UnboxInt64s, one_word),
+        Args, TypedArgs),
     add_scalar_static_cell(TypedArgs, DataId, !Info).
 
 add_scalar_static_cell(TypedArgs0, DataId, !Info) :-
     % If we have an empty cell, place a dummy field in it,
-    % so that the generated C structure isn't empty.
+    % so that the generated C structure is not empty.
     (
         TypedArgs0 = [],
-        TypedArgs = [typed_rval(const(llconst_int(-1)), lt_integer)]
+        TypedArgs = [typed_rval(const(llconst_int(-1)), lt_int(int_type_int))]
     ;
         TypedArgs0 = [_ | _],
         TypedArgs = TypedArgs0
@@ -392,14 +414,14 @@ do_add_scalar_static_cell(TypedArgs, CellType, CellValue, DataId, !Info) :-
         CellGroupMap0 = !.Info ^ sci_scalar_cell_group_map,
         % We do not want to use bimap.search_insert here, since this search
         % usually succeeds.
-        ( bimap.search(TypeNumMap0, CellType, OldTypeNum) ->
+        ( if bimap.search(TypeNumMap0, CellType, OldTypeNum) then
             TypeNum = OldTypeNum,
-            ( map.search(CellGroupMap0, TypeNum, !:CellGroup) ->
+            ( if map.search(CellGroupMap0, TypeNum, !:CellGroup) then
                 true
-            ;
+            else
                 !:CellGroup = init_scalar_cell_group
             )
-        ;
+        else
             TypeNumCounter0 = !.Info ^ sci_type_counter,
             counter.allocate(TypeRawNum, TypeNumCounter0, TypeNumCounter),
             TypeNum = type_num(TypeRawNum),
@@ -413,7 +435,7 @@ do_add_scalar_static_cell(TypedArgs, CellType, CellValue, DataId, !Info) :-
 
         InsertCommonData = !.Info ^ sci_sub_info ^ scsi_common_data,
         (
-            InsertCommonData = yes,
+            InsertCommonData = use_common_data,
             MembersMap0 = !.CellGroup ^ scalar_cell_group_members,
             CellNumCounter0 = !.CellGroup ^ scalar_cell_counter,
             counter.allocate(CellNum, CellNumCounter0, CellNumCounter),
@@ -438,11 +460,11 @@ do_add_scalar_static_cell(TypedArgs, CellType, CellValue, DataId, !Info) :-
                 !Info ^ sci_scalar_cell_group_map := CellGroupMap
             )
         ;
-            InsertCommonData = no,
+            InsertCommonData = do_not_use_common_data,
             MembersMap0 = !.CellGroup ^ scalar_cell_group_members,
-            ( bimap.search(MembersMap0, Args, DataIdPrime) ->
+            ( if bimap.search(MembersMap0, Args, DataIdPrime) then
                 DataId = DataIdPrime
-            ;
+            else
                 CellNumCounter0 = !.CellGroup ^ scalar_cell_counter,
                 counter.allocate(CellNum, CellNumCounter0, CellNumCounter),
                 !CellGroup ^ scalar_cell_counter := CellNumCounter,
@@ -473,30 +495,35 @@ search_scalar_static_cell_offset(Info, DataId, Offset, Rval) :-
 
 %-----------------------------------------------------------------------------%
 
-find_general_llds_types(UnboxFloat, Types, [Vector | Vectors], LLDSTypes) :-
-    ArgWidth = full_word,
-    list.map(natural_type(UnboxFloat, ArgWidth), Vector, LLDSTypes0),
-    find_general_llds_types_2(UnboxFloat, Types, Vectors,
-        LLDSTypes0, LLDSTypes).
+find_general_llds_types(UnboxFloat, UnboxInt64s, Types,
+        [Vector | Vectors], !:LLDSTypes) :-
+    list.map(natural_type(UnboxFloat, UnboxInt64s, one_word),
+        Vector, !:LLDSTypes),
+    find_general_llds_types_loop(UnboxFloat, UnboxInt64s, Types,
+        Vectors, !LLDSTypes).
 
-:- pred find_general_llds_types_2(have_unboxed_floats::in, list(mer_type)::in,
-    list(list(rval))::in, list(llds_type)::in, list(llds_type)::out)
-    is semidet.
+:- pred find_general_llds_types_loop(have_unboxed_floats::in,
+    have_unboxed_int64s::in, list(mer_type)::in, list(list(rval))::in,
+    list(llds_type)::in, list(llds_type)::out) is semidet.
 
-find_general_llds_types_2(_UnboxFloat, _Types, [], !LLDSTypes).
-find_general_llds_types_2(UnboxFloat, Types, [Vector | Vectors], !LLDSTypes) :-
-    find_general_llds_types_in_cell(UnboxFloat, Types, Vector, !LLDSTypes),
-    find_general_llds_types_2(UnboxFloat, Types, Vectors, !LLDSTypes).
+find_general_llds_types_loop(_UnboxFloat, _UnboxInt64s, _Types,
+        [], !LLDSTypes).
+find_general_llds_types_loop(UnboxFloat, UnboxInt64s, Types,
+        [Vector | Vectors], !LLDSTypes) :-
+    find_general_llds_types_in_cell(UnboxFloat, UnboxInt64s, Types,
+        Vector, !LLDSTypes),
+    find_general_llds_types_loop(UnboxFloat, UnboxInt64s, Types,
+        Vectors, !LLDSTypes).
 
 :- pred find_general_llds_types_in_cell(have_unboxed_floats::in,
-    list(mer_type)::in, list(rval)::in, list(llds_type)::in,
-    list(llds_type)::out) is semidet.
+    have_unboxed_int64s::in, list(mer_type)::in, list(rval)::in,
+    list(llds_type)::in, list(llds_type)::out) is semidet.
 
-find_general_llds_types_in_cell(_UnboxFloat, [], [], [], []).
-find_general_llds_types_in_cell(UnboxFloat, [_Type | Types], [Rval | Rvals],
-        [LLDSType0 | LLDSTypes0], [LLDSType | LLDSTypes]) :-
-    ArgWidth = full_word,
-    natural_type(UnboxFloat, ArgWidth, Rval, NaturalType),
+find_general_llds_types_in_cell(_UnboxFloat, _UnboxInt64s, [], [], [], []).
+find_general_llds_types_in_cell(UnboxFloat, UnboxInt64s, [_Type | Types],
+        [Rval | Rvals], [LLDSType0 | LLDSTypes0], [LLDSType | LLDSTypes]) :-
+    NumWords = one_word,
+    natural_type(UnboxFloat, UnboxInt64s, NumWords, Rval, NaturalType),
     % For user-defined types, some function symbols may be constants
     % (whose representations yield integer rvals) while others may be
     % non-constants (whose representations yield data_ptr rvals).
@@ -507,31 +534,31 @@ find_general_llds_types_in_cell(UnboxFloat, [_Type | Types], [Rval | Rvals],
     %
     % If there are any other similar cases, they should be added here.
     % The value of Type may be useful in such code.
-    (
+    ( if
         NaturalType = LLDSType0
-    ->
+    then
         LLDSType = LLDSType0
-    ;
-        NaturalType = lt_integer,
+    else if
+        NaturalType = lt_int(int_type_int),
         LLDSType0 = lt_data_ptr
-    ->
+    then
         LLDSType = lt_data_ptr
-    ;
+    else if
         NaturalType = lt_data_ptr,
-        LLDSType0 = lt_integer
-    ->
+        LLDSType0 = lt_int(int_type_int)
+    then
         LLDSType = lt_data_ptr
-    ;
+    else
         fail
     ),
-    find_general_llds_types_in_cell(UnboxFloat, Types, Rvals,
+    find_general_llds_types_in_cell(UnboxFloat, UnboxInt64s, Types, Rvals,
         LLDSTypes0, LLDSTypes).
 
 %-----------------------------------------------------------------------------%
 
 add_vector_static_cell(LLDSTypes, VectorData, DataId, !Info) :-
-    require(list.is_not_empty(LLDSTypes), "add_vector_static_cell: no types"),
-    require(list.is_not_empty(VectorData), "add_vector_static_cell: no data"),
+    expect(list.is_not_empty(LLDSTypes), $pred, "no types"),
+    expect(list.is_not_empty(VectorData), $pred, "no data"),
 
     % We don't to use grouped_args_type, since that would (a) make the code
     % below significantly more complex, and (b) the type declaration can be
@@ -543,14 +570,14 @@ add_vector_static_cell(LLDSTypes, VectorData, DataId, !Info) :-
     some [!CellGroup] (
         TypeNumMap0 = !.Info ^ sci_cell_type_num_map,
         CellGroupMap0 = !.Info ^ sci_vector_cell_group_map,
-        ( bimap.search(TypeNumMap0, CellType, TypeNumPrime) ->
+        ( if bimap.search(TypeNumMap0, CellType, TypeNumPrime) then
             TypeNum = TypeNumPrime,
-            ( map.search(CellGroupMap0, TypeNum, !:CellGroup) ->
+            ( if map.search(CellGroupMap0, TypeNum, !:CellGroup) then
                 true
-            ;
+            else
                 !:CellGroup = init_vector_cell_group
             )
-        ;
+        else
             TypeNumCounter0 = !.Info ^ sci_type_counter,
             counter.allocate(TypeNum0, TypeNumCounter0, TypeNumCounter),
             TypeNum = type_num(TypeNum0),
@@ -633,17 +660,17 @@ add_one_vector_static_cell(TypeNum, CellType, CellNum,
     common_cell_type::out, common_cell_value::out) is det.
 
 compute_cell_type(TypedArgs, CellType, CellValue) :-
-    (
+    ( if
         TypedArgs = [typed_rval(FirstArg, FirstArgType) | LaterTypedArgs],
         threshold_group_types(FirstArgType, [FirstArg], LaterTypedArgs,
             TypeGroups, TypeAndArgGroups),
         OldLength = list.length(TypedArgs),
         NewLength = list.length(TypeAndArgGroups),
         OldLength >= NewLength * 2
-    ->
+    then
         CellType = grouped_args_type(TypeGroups),
         CellValue = grouped_args_value(TypeAndArgGroups)
-    ;
+    else
         CellType = plain_type(typed_rvals_project_types(TypedArgs)),
         CellValue = plain_value(TypedArgs)
     ).
@@ -661,10 +688,10 @@ threshold_group_types(CurType, RevArgsSoFar, LaterArgsTypes, TypeGroups,
         TypeAndArgGroups = [TypeAndArgGroup]
     ;
         LaterArgsTypes = [typed_rval(NextArg, NextType) | MoreArgsTypes],
-        ( CurType = NextType ->
+        ( if CurType = NextType then
             threshold_group_types(CurType, [NextArg | RevArgsSoFar],
                 MoreArgsTypes, TypeGroups, TypeAndArgGroups)
-        ;
+        else
             threshold_group_types(NextType, [NextArg], MoreArgsTypes,
                 TypeGroupsTail, TypeAndArgGroupsTail),
             make_arg_groups(CurType, RevArgsSoFar, TypeGroup, TypeAndArgGroup),
@@ -677,10 +704,10 @@ threshold_group_types(CurType, RevArgsSoFar, LaterArgsTypes, TypeGroups,
     pair(llds_type, int)::out, common_cell_arg_group::out) is det.
 
 make_arg_groups(Type, RevArgs, TypeGroup, TypeAndArgGroup) :-
-    ( RevArgs = [Arg] ->
+    ( if RevArgs = [Arg] then
         TypeGroup = Type - 1,
         TypeAndArgGroup = common_cell_ungrouped_arg(Type, Arg)
-    ;
+    else
         list.length(RevArgs, NumArgs),
         list.reverse(RevArgs, Args),
         TypeGroup = Type - NumArgs,
@@ -689,30 +716,77 @@ make_arg_groups(Type, RevArgs, TypeGroup, TypeAndArgGroup) :-
 
 %-----------------------------------------------------------------------------%
 
-rval_type_as_arg(UnboxedFloat, ArgWidth, Rval) = Type :-
-    natural_type(UnboxedFloat, ArgWidth, Rval, Type).
+rval_type_as_arg(UnboxedFloat, UnboxedInt64s, NumWords, Rval) = Type :-
+    natural_type(UnboxedFloat, UnboxedInt64s, NumWords, Rval, Type).
 
-:- pred natural_type(have_unboxed_floats::in, arg_width::in, rval::in,
-    llds_type::out) is det.
+:- pred associate_natural_type(have_unboxed_floats::in,
+    have_unboxed_int64s::in, num_words::in, rval::in, typed_rval::out) is det.
 
-natural_type(UnboxFloat, ArgWidth, Rval, Type) :-
+associate_natural_type(UnboxFloat, UnboxInt64s, NumWords, Rval, TypedRval) :-
+    natural_type(UnboxFloat, UnboxInt64s, NumWords, Rval, Type),
+    TypedRval = typed_rval(Rval, Type).
+
+:- pred natural_type(have_unboxed_floats::in, have_unboxed_int64s::in,
+    num_words::in, rval::in, llds_type::out) is det.
+
+natural_type(UnboxFloat, UnboxInt64s, NumWords, Rval, Type) :-
     llds.rval_type(Rval, Type0),
     (
         Type0 = lt_float,
-        UnboxFloat = do_not_have_unboxed_floats,
-        ArgWidth \= double_word
-    ->
-        Type = lt_data_ptr
+        ( if
+            UnboxFloat = do_not_have_unboxed_floats,
+            NumWords = one_word
+        then
+            Type = lt_data_ptr
+        else
+            Type = Type0
+        )
     ;
+        ( Type0 = lt_int(int_type_int64)
+        ; Type0 = lt_int(int_type_uint64)
+        ),
+        ( if
+            UnboxInt64s = do_not_have_unboxed_int64s,
+            NumWords = one_word
+        then
+            Type = lt_data_ptr
+        else
+            Type = Type0
+        )
+    ;
+        ( Type0 = lt_int(int_type_int)
+        ; Type0 = lt_int(int_type_int32)
+        ; Type0 = lt_int(int_type_int16)
+        ; Type0 = lt_int(int_type_int8)
+        ),
+        Type = lt_int(int_type_int)
+    ;
+        ( Type0 = lt_int(int_type_uint)
+        ; Type0 = lt_int(int_type_uint32)
+        ; Type0 = lt_int(int_type_uint16)
+        ; Type0 = lt_int(int_type_uint8)
+        ),
+        Type = lt_int(int_type_uint)
+    ;
+        ( Type0 = lt_bool
+        ; Type0 = lt_code_ptr
+        ; Type0 = lt_data_ptr
+        ; Type0 = lt_string
+        ; Type0 = lt_word
+        ),
         Type = Type0
+    ;
+        Type0 = lt_int_least(_),
+        % These LLDS types do not correspond to any Mercury type;
+        % they are intended to be used by the compiler when generating
+        % RTTI data structures, especially layout structures for tools
+        % such as the debugger and the profilers. When creating these
+        % structures, the compiler will *know* exactly what C types
+        % describe the values it wants to generate, so it should not need
+        % to use this predicate to find that out.
+        unexpected($pred, "least type")
     ).
 
-:- pred associate_natural_type(have_unboxed_floats::in, arg_width::in,
-    rval::in, typed_rval::out) is det.
-
-associate_natural_type(UnboxFloat, ArgWidth, Rval, typed_rval(Rval, Type)) :-
-    natural_type(UnboxFloat, ArgWidth, Rval, Type).
-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -789,7 +863,7 @@ merge_static_cell_infos(SCIa, SCIb, SCI, Remap) :-
         CellTypeNumMapA, ScalarCellGroupMapA, VectorCellGroupMapA),
     SCIb = static_cell_info(SubInfoB, _TypeCounterB,
         CellTypeNumMapB, ScalarCellGroupMapB, VectorCellGroupMapB),
-    expect(unify(SubInfoA, SubInfoB), $module, $pred, "mismatch"),
+    expect(unify(SubInfoA, SubInfoB), $pred, "mismatch"),
 
     % Merge cell type number maps.
     bimap.foldl3(merge_cell_type_num_maps, CellTypeNumMapB,
@@ -817,10 +891,10 @@ merge_static_cell_infos(SCIa, SCIb, SCI, Remap) :-
 
 merge_cell_type_num_maps(CellType, BTypeNum,
         !TypeCounter, !CellTypeNumMap, !TypeNumRemap) :-
-    ( bimap.search(!.CellTypeNumMap, CellType, ATypeNum) ->
+    ( if bimap.search(!.CellTypeNumMap, CellType, ATypeNum) then
         % A type also in GlobalDataA.
         map.det_insert(BTypeNum, ATypeNum, !TypeNumRemap)
-    ;
+    else
         % A type not in GlobalDataA.
         counter.allocate(N, !TypeCounter),
         NewTypeNum = type_num(N),
@@ -851,9 +925,9 @@ merge_scalar_cell_group_maps(TypeNumRemap,
 merge_scalar_cell_group_maps_2(TypeNumRemap, BTypeNum, BScalarCellGroup,
         !ScalarCellGroupMap, !Remap) :-
     map.lookup(TypeNumRemap, BTypeNum, TypeNum),
-    ( map.search(!.ScalarCellGroupMap, TypeNum, ScalarCellGroupPrime) ->
+    ( if map.search(!.ScalarCellGroupMap, TypeNum, ScalarCellGroupPrime) then
         ScalarCellGroup0 = ScalarCellGroupPrime
-    ;
+    else
         % Could do this more efficiently.
         ScalarCellGroup0 = scalar_cell_group(counter.init(0), bimap.init, [])
     ),
@@ -890,17 +964,18 @@ merge_scalar_cell_groups(TypeNum, GroupA, GroupB, GroupAB, GroupRemap) :-
 
 merge_scalar_cell_groups_2(TypeNum, ArrayB, ArrayAB,
         Rvals, BDataId, !GroupMembers, !GroupRemap) :-
-    ( bimap.search(!.GroupMembers, Rvals, DataId) ->
+    ( if bimap.search(!.GroupMembers, Rvals, DataId) then
         % Seen this list of rvals before in the group.
         map.det_insert(BDataId, DataId, !GroupRemap)
-    ;
+    else
         % Not seen this list of rvals before in the group.
         (
             BDataId = scalar_common_data_id(_, BCellNum),
             % Look up what value this cell number referred to in the B array.
             % Find the cell number of the same value in the combined A+B array.
             CommonCellValue = list.det_index0(ArrayB, BCellNum),
-            CellNum = nth_member_lookup0(ArrayAB, CommonCellValue),
+            CellNum =
+                list.det_index0_of_first_occurrence(ArrayAB, CommonCellValue),
             % Add the new data name.
             DataId = scalar_common_data_id(TypeNum, CellNum),
             bimap.det_insert(Rvals, DataId, !GroupMembers),
@@ -912,7 +987,7 @@ merge_scalar_cell_groups_2(TypeNum, ArrayB, ArrayAB,
             ; BDataId = layout_id(_)
             ; BDataId = layout_slot_id(_, _)
             ),
-            unexpected($module, $pred, "unexpected BDataId")
+            unexpected($pred, "unexpected BDataId")
         )
     ).
 
@@ -935,11 +1010,6 @@ merge_vector_cell_group_maps_2(TypeNumRemap, OldTypeNum, VectorCellGroup,
     map.lookup(TypeNumRemap, OldTypeNum, NewTypeNum),
     map.det_insert(NewTypeNum, VectorCellGroup, !VectorCellGroupMap).
 
-:- func nth_member_lookup0(list(T), T) = int.
-
-nth_member_lookup0(List, Elem) = Pos-1 :-
-    list.nth_member_lookup(List, Elem, Pos).
-
 %-----------------------------------------------------------------------------%
 
     % The scalar cell group and vector cell group contents themselves
@@ -1056,9 +1126,13 @@ remap_instr(GlobalDataRemap, Instr0, Instr) :-
         remap_rval(StaticCellRemap, Rval0, Rval),
         Instr  = if_val(Rval, CodeAddr)
     ;
-        Instr0 = foreign_proc_code(A, Comps0, B, C, D, E, F, G, H, I),
+        Instr0 = foreign_proc_code(Decls, Comps0, MayCallMerc,
+            FixNoLayout, FixLayout, FixOnlyLayout, NoFix,
+            HashDefnLabel, StackSlotRef, MaybeDup),
         list.map(remap_foreign_proc_component(StaticCellRemap), Comps0, Comps),
-        Instr  = foreign_proc_code(A, Comps,  B, C, D, E, F, G, H, I)
+        Instr  = foreign_proc_code(Decls, Comps,  MayCallMerc,
+            FixNoLayout, FixLayout, FixOnlyLayout, NoFix,
+            HashDefnLabel, StackSlotRef, MaybeDup)
     ;
         Instr0 = computed_goto(Rval0, MaybeLabels),
         remap_rval(StaticCellRemap, Rval0, Rval),
@@ -1294,6 +1368,10 @@ remap_rval(Remap, Rval0, Rval) :-
         Rval0 = const(Const0),
         remap_rval_const(Remap, Const0, Const),
         Rval = const(Const)
+    ;
+        Rval0 = cast(Type, A0),
+        remap_rval(Remap, A0, A),
+        Rval = cast(Type, A)
     ;
         Rval0 = unop(Unop, A0),
         remap_rval(Remap, A0, A),
@@ -1318,18 +1396,18 @@ remap_rval_const(Remap, Const0, Const) :-
         (
             DataId0 = scalar_common_data_id(TypeNum0, _CellNum),
             Remap = static_cell_remap_info(TypeNumRemap, ScalarCellGroupRemap),
-            ( map.contains(TypeNumRemap, TypeNum0) ->
+            ( if map.contains(TypeNumRemap, TypeNum0) then
                 map.lookup(ScalarCellGroupRemap, TypeNum0, ScalarCellGroup),
                 map.lookup(ScalarCellGroup, DataId0, DataId)
-            ;
+            else
                 DataId = DataId0
             )
         ;
             DataId0 = vector_common_data_id(TypeNum0, CellNum),
             Remap = static_cell_remap_info(TypeNumRemap, _),
-            ( map.search(TypeNumRemap, TypeNum0, TypeNum) ->
+            ( if map.search(TypeNumRemap, TypeNum0, TypeNum) then
                 DataId = vector_common_data_id(TypeNum, CellNum)
-            ;
+            else
                 DataId = DataId0
             )
         ;
@@ -1345,6 +1423,15 @@ remap_rval_const(Remap, Const0, Const) :-
         ( Const0 = llconst_true
         ; Const0 = llconst_false
         ; Const0 = llconst_int(_)
+        ; Const0 = llconst_uint(_)
+        ; Const0 = llconst_int8(_)
+        ; Const0 = llconst_uint8(_)
+        ; Const0 = llconst_int16(_)
+        ; Const0 = llconst_uint16(_)
+        ; Const0 = llconst_int32(_)
+        ; Const0 = llconst_uint32(_)
+        ; Const0 = llconst_int64(_)
+        ; Const0 = llconst_uint64(_)
         ; Const0 = llconst_foreign(_, _)
         ; Const0 = llconst_float(_)
         ; Const0 = llconst_string(_)
