@@ -21,6 +21,8 @@
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module parse_tree.
+:- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
 
 :- import_module bool.
@@ -71,7 +73,7 @@
     % `goal_cannot_throw' otherwise.
     %
     % This version differs from the ones below in that it can use results from
-    % the intermodule-analysis framework (if they are available).  The HLDS
+    % the intermodule-analysis framework (if they are available). The HLDS
     % and I/O state need to be threaded through in case analysis files need to
     % be read and in case IMDGs need to be updated.
     %
@@ -132,43 +134,40 @@
 :- func goal_is_flat(hlds_goal) = bool.
 
     % Determine whether a goal might allocate some heap space, i.e.
-    % whether it contains any construction unifications or predicate
-    % calls.  BEWARE that this predicate is only an approximation,
-    % used to decide whether or not to try to reclaim the heap
-    % space; currently it fails even for some goals which do
-    % allocate heap space, such as construction of boxed constants.
+    % whether it contains any construction unifications or predicate calls.
+    % BEWARE that this predicate is only an approximation,
+    % used to decide whether or not to try to reclaim the heap space;
+    % currently it fails even for some goals which do allocate heap space,
+    % such as construction of boxed constants.
     %
 :- pred goal_may_allocate_heap(hlds_goal::in) is semidet.
 :- pred goal_list_may_allocate_heap(hlds_goals::in) is semidet.
 
     % Succeed if execution of the given goal cannot encounter a context
-    % that causes any variable to be flushed to its stack slot.  If such a
-    % goal needs a resume point, and that resume point cannot be
-    % backtracked to once control leaves the goal, then the only entry
-    % point we need for the resume point is the one with the resume
-    % variables in their original locations.
+    % that causes any variable to be flushed to its stack slot. If such a goal
+    % needs a resume point, and that resume point cannot be backtracked to
+    % once control leaves the goal, then the only entry point we need
+    % for the resume point is the one with the resume variables in their
+    % original locations.
     %
 :- pred cannot_stack_flush(hlds_goal::in) is semidet.
 
     % Succeed if the given goal cannot fail before encountering a
-    % context that forces all variables to be flushed to their stack
-    % slots.  If such a goal needs a resume point, the only entry
-    % point we need is the stack entry point.
+    % context that forces all variables to be flushed to their stack slots.
+    % If such a goal needs a resume point, the only entry point we need
+    % is the stack entry point.
     %
 :- pred cannot_fail_before_stack_flush(hlds_goal::in) is semidet.
 
-    % count_recursive_calls(Goal, PredId, ProcId, Min, Max). Given
-    % that we are in predicate PredId and procedure ProcId, return
-    % the minimum and maximum number of recursive calls that an
-    % execution of Goal may encounter.
+    % count_recursive_calls(Goal, PredId, ProcId, Min, Max). Given that
+    % we are in predicate PredId and procedure ProcId, return the minimum
+    % and maximum number of recursive calls that an execution of Goal
+    % may encounter.
     %
 :- pred count_recursive_calls(hlds_goal::in, pred_id::in, proc_id::in,
     int::out, int::out) is det.
 
 %-----------------------------------------------------------------------------%
-%
-% Trail usage
-%
 
     % Returns `yes' if the goal does not modify the trail.
     %
@@ -178,20 +177,77 @@
     %
 :- func goal_may_modify_trail(hlds_goal_info) = bool.
 
+%-----------------------------------------------------------------------------%
+
+    % Returns yes if the goal, or subgoal contained within, contains
+    % any foreign code.
+    %
+:- func goal_has_foreign(hlds_goal) = bool.
+
+:- type has_subgoals
+    --->    has_subgoals
+    ;       does_not_have_subgoals.
+
+    % A goal is primitive iff it doesn't contain any sub-goals
+    % (except possibly goals inside lambda expressions --
+    % but lambda expressions will get transformed into separate
+    % predicates by lambda.m).
+    %
+:- func goal_expr_has_subgoals(hlds_goal_expr) = has_subgoals.
+
+%-----------------------------------------------------------------------------%
+
+    % Insist on both given lists having exactly one element,
+    % and return those elements. The lists are intended to contain
+    % the arguments and unify modes respectively of a function symbol
+    % whose representation is no_tag or direct_arg_tag, which means
+    % that its arity must be exactly one.
+    %
+:- pred get_notag_or_direct_arg_arg_mode(list(Arg)::in, list(UM)::in,
+    Arg::out, UM::out) is det.
+
+    % A version of the above without the modes.
+    %
+:- pred get_notag_or_direct_arg_arg(list(Arg)::in, Arg::out) is det.
+
+%-----------------------------------------------------------------------------%
+
+:- type is_termvar_needed
+    --->    termvar_is_not_needed
+    ;       termvar_is_needed.
+
+    % Given the TermVar and the subgoal of a from_ground_term_construct scope,
+    % return an indication of whether the termvar is needed outside the scope,
+    % the conjuncts inside the scope, and the conjunction's goal_info.
+    % The conjuncts should all be processable (without an abort) by
+    % get_from_ground_term_construct_conjunct_info.
+    %
+:- pred get_from_ground_term_construct_info(prog_var::in, hlds_goal::in,
+    is_termvar_needed::out, list(hlds_goal)::out, hlds_goal_info::out) is det.
+
+    % get_from_ground_term_construct_conjunct_info(Goal,
+    %   LHSVar, ConsId, RHSVars, GoalInfo):
+    %
+    % A conjunct inside a from_ground_term_construct scope *must* be a
+    % construction unification; return its details.
+    % 
+:- pred get_from_ground_term_construct_conjunct_info(hlds_goal::in,
+    prog_var::out, cons_id::out, list(prog_var)::out, hlds_goal_info::out)
+    is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module hlds.code_model.
-:- import_module hlds.hlds_goal.
-:- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
+:- import_module parse_tree.prog_data_pragma.
+:- import_module transform_hlds.
 :- import_module transform_hlds.exception_analysis.
-:- import_module transform_hlds.term_constr_main.
+:- import_module transform_hlds.term_constr_main_types.
 
-:- import_module bool.
 :- import_module int.
-:- import_module map.
 :- import_module maybe.
 :- import_module require.
 
@@ -237,10 +293,10 @@ only_constant_goals([Goal | Goals], !ToAssignVars) :-
 
 goal_can_throw(hlds_goal(GoalExpr, GoalInfo), Result, !ModuleInfo) :-
     Determinism = goal_info_get_determinism(GoalInfo),
-    ( Determinism \= detism_erroneous ->
-        goal_can_throw_2(GoalExpr, GoalInfo, Result, !ModuleInfo)
-    ;
+    ( if Determinism = detism_erroneous then
         Result = can_throw
+    else
+        goal_can_throw_2(GoalExpr, GoalInfo, Result, !ModuleInfo)
     ).
 
 :- pred goal_can_throw_2(hlds_goal_expr::in, hlds_goal_info::in,
@@ -296,31 +352,31 @@ goal_can_throw_2(GoalExpr, _GoalInfo, Result, !ModuleInfo) :-
         goal_can_throw(SubGoal, Result, !ModuleInfo)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % These scopes contain only construction/deconstruction
             % unifications.
             Result = cannot_throw
-        ;
+        else
             goal_can_throw(SubGoal, Result, !ModuleInfo)
         )
     ;
         GoalExpr = call_foreign_proc(Attributes, _, _, _, _, _, _),
         ExceptionStatus = get_may_throw_exception(Attributes),
-        (
+        ( if
             (
                 ExceptionStatus = proc_will_not_throw_exception
             ;
                 ExceptionStatus = default_exception_behaviour,
                 get_may_call_mercury(Attributes) = proc_will_not_call_mercury
             )
-        ->
+        then
             Result = cannot_throw
-        ;
+        else
             Result = can_throw
         )
     ;
@@ -372,7 +428,7 @@ cases_can_throw([Case | Cases], Result, !ModuleInfo) :-
 goal_can_loop_or_throw(Goal, Result, !ModuleInfo) :-
     % XXX This will need to change after the termination analyses are converted
     % to use the intermodule-analysis framework.
-    ( goal_cannot_loop(!.ModuleInfo, Goal) ->
+    ( if goal_cannot_loop(!.ModuleInfo, Goal) then
         goal_can_throw(Goal, ThrowResult, !ModuleInfo),
         (
             ThrowResult = can_throw,
@@ -381,7 +437,7 @@ goal_can_loop_or_throw(Goal, Result, !ModuleInfo) :-
             ThrowResult = cannot_throw,
             Result = cannot_loop_or_throw
         )
-    ;
+    else
         Result = can_loop_or_throw
     ).
 
@@ -434,7 +490,7 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
         )
     ;
         GoalExpr = plain_call(PredId, ProcId, _, _, _, _),
-        (
+        ( if
             MaybeModuleInfo = yes(ModuleInfo),
             module_info_pred_proc_info(ModuleInfo, PredId, ProcId, _,
                 ProcInfo),
@@ -443,11 +499,11 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
                 MaybeTermInfo = yes(cannot_loop(_))
             ;
                 proc_info_get_termination2_info(ProcInfo, Term2Info),
-                Term2Info ^ term_status = yes(cannot_loop(_))
+                term2_info_get_term_status(Term2Info) = yes(cannot_loop(_))
             )
-        ->
+        then
             CanLoop = no
-        ;
+        else
             CanLoop = yes
         )
     ;
@@ -457,17 +513,21 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
         CanLoop = yes
     ;
         GoalExpr = call_foreign_proc(Attributes, _, _, _, _, _, _),
-        (
+        ( if
             Terminates = get_terminates(Attributes),
+            require_complete_switch [Terminates]
             (
                 Terminates = proc_terminates
             ;
                 Terminates = depends_on_mercury_calls,
                 get_may_call_mercury(Attributes) = proc_will_not_call_mercury
+            ;
+                Terminates = proc_does_not_terminate,
+                fail
             )
-        ->
+        then
             CanLoop = no
-        ;
+        else
             CanLoop = yes
         )
     ;
@@ -487,11 +547,11 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
         CanLoop = case_list_can_loop(MaybeModuleInfo, Cases)
     ;
         GoalExpr = if_then_else(_Vars, Cond, Then, Else),
-        ( goal_can_loop_func(MaybeModuleInfo, Cond) = yes ->
+        ( if goal_can_loop_func(MaybeModuleInfo, Cond) = yes then
             CanLoop = yes
-        ; goal_can_loop_func(MaybeModuleInfo, Then) = yes ->
+        else if goal_can_loop_func(MaybeModuleInfo, Then) = yes then
             CanLoop = yes
-        ;
+        else
             CanLoop = goal_can_loop_func(MaybeModuleInfo, Else)
         )
     ;
@@ -499,16 +559,16 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
         CanLoop = goal_can_loop_func(MaybeModuleInfo, SubGoal)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % These scopes contain only construction/deconstruction
             % unifications.
             CanLoop = no
-        ;
+        else
             CanLoop = goal_can_loop_func(MaybeModuleInfo, SubGoal)
         )
     ;
@@ -523,7 +583,7 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
             CanLoop = goal_can_loop_func(MaybeModuleInfo, SubGoal)
         ;
             ShortHand = bi_implication(_, _),
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
@@ -531,9 +591,9 @@ goal_can_loop_func(MaybeModuleInfo, Goal) = CanLoop :-
 
 goal_list_can_loop(_, []) = no.
 goal_list_can_loop(MaybeModuleInfo, [Goal | Goals]) =
-    ( goal_can_loop_func(MaybeModuleInfo, Goal) = yes ->
+    ( if goal_can_loop_func(MaybeModuleInfo, Goal) = yes then
         yes
-    ;
+    else
         goal_list_can_loop(MaybeModuleInfo, Goals)
     ).
 
@@ -541,9 +601,9 @@ goal_list_can_loop(MaybeModuleInfo, [Goal | Goals]) =
 
 case_list_can_loop(_, []) = no.
 case_list_can_loop(MaybeModuleInfo, [case(_, _, Goal) | Cases]) =
-    ( goal_can_loop_func(MaybeModuleInfo, Goal) = yes ->
+    ( if goal_can_loop_func(MaybeModuleInfo, Goal) = yes then
         yes
-    ;
+    else
         case_list_can_loop(MaybeModuleInfo, Cases)
     ).
 
@@ -554,9 +614,9 @@ case_list_can_loop(MaybeModuleInfo, [case(_, _, Goal) | Cases]) =
 goal_can_throw_func(MaybeModuleInfo, hlds_goal(GoalExpr, GoalInfo))
         = CanThrow :-
     Determinism = goal_info_get_determinism(GoalInfo),
-    ( Determinism = detism_erroneous ->
+    ( if Determinism = detism_erroneous then
         CanThrow = yes
-    ;
+    else
         CanThrow = goal_expr_can_throw(MaybeModuleInfo, GoalExpr)
     ).
 
@@ -580,29 +640,31 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
         )
     ;
         GoalExpr = plain_call(PredId, ProcId, _, _, _, _),
-        (
+        ( if
             MaybeModuleInfo = yes(ModuleInfo),
-            module_info_get_exception_info(ModuleInfo, ExceptionInfo),
-            map.search(ExceptionInfo, proc(PredId, ProcId), ProcExceptionInfo),
-            ProcExceptionInfo = proc_exception_info(will_not_throw, _)
-        ->
+            module_info_pred_proc_info(ModuleInfo, PredId, ProcId,
+                _PredInfo, ProcInfo),
+            proc_info_get_exception_info(ProcInfo, MaybeExceptionInfo),
+            MaybeExceptionInfo = yes(ExceptionInfo),
+            ExceptionInfo = proc_exception_info(will_not_throw, _)
+        then
             CanThrow = no
-        ;
+        else
             CanThrow = yes
         )
     ;
         GoalExpr = call_foreign_proc(Attributes, _, _, _, _, _, _),
         ExceptionStatus = get_may_throw_exception(Attributes),
-        (
+        ( if
             (
                 ExceptionStatus = proc_will_not_throw_exception
             ;
                 ExceptionStatus = default_exception_behaviour,
                 get_may_call_mercury(Attributes) = proc_will_not_call_mercury
             )
-        ->
+        then
             CanThrow = no
-        ;
+        else
             CanThrow = yes
         )
     ;
@@ -620,11 +682,11 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
         CanThrow = case_list_can_throw(MaybeModuleInfo, Cases)
     ;
         GoalExpr = if_then_else(_, Cond, Then, Else),
-        ( goal_can_throw_func(MaybeModuleInfo, Cond) = yes ->
+        ( if goal_can_throw_func(MaybeModuleInfo, Cond) = yes then
             CanThrow = yes
-        ; goal_can_throw_func(MaybeModuleInfo, Then) = yes ->
+        else if goal_can_throw_func(MaybeModuleInfo, Then) = yes then
             CanThrow = yes
-        ;
+        else
             CanThrow = goal_can_throw_func(MaybeModuleInfo, Else)
         )
     ;
@@ -632,16 +694,16 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
         CanThrow = goal_can_throw_func(MaybeModuleInfo, SubGoal)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % These scopes contain only construction/deconstruction
             % unifications.
             CanThrow = no
-        ;
+        else
             CanThrow = goal_can_throw_func(MaybeModuleInfo, SubGoal)
         )
     ;
@@ -654,7 +716,7 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
             CanThrow = yes
         ;
             ShortHand = bi_implication(_, _),
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
@@ -662,9 +724,9 @@ goal_expr_can_throw(MaybeModuleInfo, GoalExpr) = CanThrow :-
 
 goal_list_can_throw(_, []) = no.
 goal_list_can_throw(MaybeModuleInfo, [Goal | Goals]) =
-    ( goal_can_throw_func(MaybeModuleInfo, Goal) = yes ->
+    ( if goal_can_throw_func(MaybeModuleInfo, Goal) = yes then
         yes
-    ;
+    else
         goal_list_can_throw(MaybeModuleInfo, Goals)
     ).
 
@@ -672,9 +734,9 @@ goal_list_can_throw(MaybeModuleInfo, [Goal | Goals]) =
 
 case_list_can_throw(_, []) = no.
 case_list_can_throw(MaybeModuleInfo, [case(_, _, Goal) | Cases]) =
-    ( goal_can_throw_func(MaybeModuleInfo, Goal) = yes ->
+    ( if goal_can_throw_func(MaybeModuleInfo, Goal) = yes then
         yes
-    ;
+    else
         case_list_can_throw(MaybeModuleInfo, Cases)
     ).
 
@@ -713,14 +775,14 @@ goal_is_flat_expr(GoalExpr) = IsFlat :-
         IsFlat = goal_is_flat(SubGoal)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             IsFlat = yes
-        ;
+        else
             IsFlat = goal_is_flat(SubGoal)
         )
     ).
@@ -729,9 +791,9 @@ goal_is_flat_expr(GoalExpr) = IsFlat :-
 
 goal_is_flat_list([]) = yes.
 goal_is_flat_list([Goal | Goals]) = IsFlat :-
-    ( goal_is_flat(Goal) = yes ->
+    ( if goal_is_flat(Goal) = yes then
         IsFlat = goal_is_flat_list(Goals)
-    ;
+    else
         IsFlat = no
     ).
 
@@ -753,12 +815,12 @@ goal_may_allocate_heap(hlds_goal(GoalExpr, _GoalInfo), May) :-
 goal_may_allocate_heap_2(GoalExpr, May) :-
     (
         GoalExpr = unify(_, _, _, Unification, _),
-        (
+        ( if
             Unification = construct(_, _, Args, _, _, _, _),
             Args = [_ | _]
-        ->
+        then
             May = yes
-        ;
+        else
             May = no
         )
     ;
@@ -767,9 +829,7 @@ goal_may_allocate_heap_2(GoalExpr, May) :-
             Builtin = inline_builtin,
             May = no
         ;
-            ( Builtin = out_of_line_builtin
-            ; Builtin = not_builtin
-            ),
+            Builtin = not_builtin,
             May = yes
         )
     ;
@@ -800,11 +860,11 @@ goal_may_allocate_heap_2(GoalExpr, May) :-
         cases_may_allocate_heap(Cases, May)
     ;
         GoalExpr = if_then_else(_Vars, Cond, Then, Else),
-        ( goal_may_allocate_heap(Cond, yes) ->
+        ( if goal_may_allocate_heap(Cond, yes) then
             May = yes
-        ; goal_may_allocate_heap(Then, yes) ->
+        else if goal_may_allocate_heap(Then, yes) then
             May = yes
-        ;
+        else
             goal_may_allocate_heap(Else, May)
         )
     ;
@@ -812,19 +872,19 @@ goal_may_allocate_heap_2(GoalExpr, May) :-
         goal_may_allocate_heap(SubGoal, May)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % Construct scopes construct ground terms, but they construct them
             % statically, so if we modify the code above to check the
             % construct_how field of construction unifications, we could
             % return May = no for them.
             % Deconstruct scopes do not construct new ground terms.
             May = yes
-        ;
+        else
             goal_may_allocate_heap(SubGoal, May)
         )
     ;
@@ -836,9 +896,9 @@ goal_may_allocate_heap_2(GoalExpr, May) :-
             May = yes
         ;
             ShortHand = bi_implication(GoalA, GoalB),
-            ( goal_may_allocate_heap(GoalA, yes) ->
+            ( if goal_may_allocate_heap(GoalA, yes) then
                 May = yes
-            ;
+            else
                 goal_may_allocate_heap(GoalB, May)
             )
         )
@@ -848,9 +908,9 @@ goal_may_allocate_heap_2(GoalExpr, May) :-
 
 goal_list_may_allocate_heap([], no).
 goal_list_may_allocate_heap([Goal | Goals], May) :-
-    ( goal_may_allocate_heap(Goal, yes) ->
+    ( if goal_may_allocate_heap(Goal, yes) then
         May = yes
-    ;
+    else
         goal_list_may_allocate_heap(Goals, May)
     ).
 
@@ -858,9 +918,9 @@ goal_list_may_allocate_heap([Goal | Goals], May) :-
 
 cases_may_allocate_heap([], no).
 cases_may_allocate_heap([case(_, _, Goal) | Cases], May) :-
-    ( goal_may_allocate_heap(Goal, yes) ->
+    ( if goal_may_allocate_heap(Goal, yes) then
         May = yes
-    ;
+    else
         cases_may_allocate_heap(Cases, May)
     ).
 
@@ -920,21 +980,21 @@ cannot_fail_before_stack_flush_2(conj(ConjType, Goals)) :-
 cannot_fail_before_stack_flush_conj([]).
 cannot_fail_before_stack_flush_conj([Goal | Goals]) :-
     Goal = hlds_goal(GoalExpr, GoalInfo),
-    (
+    ( if
         (
             GoalExpr = plain_call(_, _, _, BuiltinState, _, _),
             BuiltinState \= inline_builtin
         ;
             GoalExpr = generic_call(_, _, _, _, _)
         )
-    ->
+    then
         true
-    ;
+    else if
         Detism = goal_info_get_determinism(GoalInfo),
         determinism_components(Detism, cannot_fail, _)
-    ->
+    then
         cannot_fail_before_stack_flush_conj(Goals)
-    ;
+    else
         fail
     ).
 
@@ -951,12 +1011,12 @@ count_recursive_calls(Goal, PredId, ProcId, Min, Max) :-
         Max = 0
     ;
         GoalExpr = plain_call(CallPredId, CallProcId, _, _, _, _),
-        (
+        ( if
             PredId = CallPredId,
             ProcId = CallProcId
-        ->
+        then
             Count = 1
-        ;
+        else
             Count = 0
         ),
         Min = Count,
@@ -984,17 +1044,17 @@ count_recursive_calls(Goal, PredId, ProcId, Min, Max) :-
         count_recursive_calls(SubGoal, PredId, ProcId, Min, Max)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % These scopes contain only construction/deconstruction
             % unifications.
             Min = 0,
             Max = 0
-        ;
+        else
             count_recursive_calls(SubGoal, PredId, ProcId, Min, Max)
         )
     ;
@@ -1009,7 +1069,7 @@ count_recursive_calls(Goal, PredId, ProcId, Min, Max) :-
         ;
             ShortHand = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
@@ -1045,7 +1105,7 @@ count_recursive_calls_disj([Goal | Goals], PredId, ProcId, Min, Max) :-
     int::out, int::out) is det.
 
 count_recursive_calls_cases([], _, _, _, _) :-
-    unexpected($module, $pred, "[]").
+    unexpected($pred, "[]").
 count_recursive_calls_cases([case(_, _, Goal) | Cases], PredId, ProcId,
         Min, Max) :-
     (
@@ -1065,14 +1125,201 @@ count_recursive_calls_cases([case(_, _, Goal) | Cases], PredId, ProcId,
 %
 
 goal_cannot_modify_trail(GoalInfo) =
-    ( goal_info_has_feature(GoalInfo, feature_will_not_modify_trail) ->
+    ( if goal_info_has_feature(GoalInfo, feature_will_not_modify_trail) then
         yes
-    ;
+    else
         no
     ).
 
 goal_may_modify_trail(GoalInfo) = bool.not(goal_cannot_modify_trail(GoalInfo)).
 
+%-----------------------------------------------------------------------------%
+
+goal_has_foreign(Goal) = HasForeign :-
+    Goal = hlds_goal(GoalExpr, _),
+    (
+        ( GoalExpr = plain_call(_, _, _, _, _, _)
+        ; GoalExpr = generic_call(_, _, _, _, _)
+        ; GoalExpr = unify(_, _, _, _, _)
+        ),
+        HasForeign = no
+    ;
+        GoalExpr = conj(_, Goals),
+        HasForeign = goal_list_has_foreign(Goals)
+    ;
+        GoalExpr = disj(Goals),
+        HasForeign = goal_list_has_foreign(Goals)
+    ;
+        GoalExpr = switch(_, _, Cases),
+        HasForeign = case_list_has_foreign(Cases)
+    ;
+        GoalExpr = negation(SubGoal),
+        HasForeign = goal_has_foreign(SubGoal)
+    ;
+        GoalExpr = scope(Reason, SubGoal),
+        ( if
+            Reason = from_ground_term(_, FGT),
+            ( FGT = from_ground_term_construct
+            ; FGT = from_ground_term_deconstruct
+            )
+        then
+            HasForeign = no
+        else
+            HasForeign = goal_has_foreign(SubGoal)
+        )
+    ;
+        GoalExpr = if_then_else(_, Cond, Then, Else),
+        ( if
+            ( goal_has_foreign(Cond) = yes
+            ; goal_has_foreign(Then) = yes
+            ; goal_has_foreign(Else) = yes
+            )
+        then
+            HasForeign = yes
+        else
+            HasForeign = no
+        )
+    ;
+        GoalExpr = call_foreign_proc(_, _, _, _, _, _, _),
+        HasForeign = yes
+    ;
+        GoalExpr = shorthand(ShortHand),
+        (
+            ShortHand = atomic_goal(_, _, _, _, _, _, _),
+            HasForeign = yes
+        ;
+            ShortHand = try_goal(_, _, SubGoal),
+            HasForeign = goal_has_foreign(SubGoal)
+        ;
+            ShortHand = bi_implication(GoalA, GoalB),
+            HasForeign = bool.or(goal_has_foreign(GoalA),
+                goal_has_foreign(GoalB))
+        )
+    ).
+
+:- func goal_list_has_foreign(list(hlds_goal)) = bool.
+
+goal_list_has_foreign([]) = no.
+goal_list_has_foreign([Goal | Goals]) = HasForeign :-
+    ( if goal_has_foreign(Goal) = yes then
+        HasForeign = yes
+    else
+        HasForeign = goal_list_has_foreign(Goals)
+    ).
+
+:- func case_list_has_foreign(list(case)) = bool.
+
+case_list_has_foreign([]) = no.
+case_list_has_foreign([Case | Cases]) = HasForeign :-
+    Case = case(_, _, Goal),
+    ( if goal_has_foreign(Goal) = yes then
+        HasForeign = yes
+    else
+        HasForeign = case_list_has_foreign(Cases)
+    ).
+
+%-----------------------------------------------------------------------------%
+
+goal_expr_has_subgoals(GoalExpr) = HasSubGoals :-
+    (
+        ( GoalExpr = unify(_, _, _, _, _)
+        ; GoalExpr = generic_call(_, _, _, _, _)
+        ; GoalExpr = plain_call(_, _, _, _, _, _)
+        ; GoalExpr = call_foreign_proc(_, _, _, _, _, _,  _)
+        ),
+        HasSubGoals = does_not_have_subgoals
+    ;
+        ( GoalExpr = conj(_, SubGoals)
+        ; GoalExpr = disj(SubGoals)
+        ),
+        (
+            SubGoals = [],
+            HasSubGoals = does_not_have_subgoals
+        ;
+            SubGoals = [_ | _],
+            HasSubGoals = has_subgoals
+        )
+    ;
+        ( GoalExpr = if_then_else(_, _, _, _)
+        ; GoalExpr = negation(_)
+        ; GoalExpr = switch(_, _, _)
+        ; GoalExpr = scope(_, _)
+        ),
+        HasSubGoals = has_subgoals
+    ;
+        GoalExpr = shorthand(ShortHand),
+        ( ShortHand = atomic_goal(_, _, _, _, _, _, _)
+        ; ShortHand = try_goal(_, _, _)
+        ; ShortHand = bi_implication(_, _)
+        ),
+        HasSubGoals = has_subgoals
+    ).
+
+%-----------------------------------------------------------------------------%
+
+get_notag_or_direct_arg_arg_mode(RHSVars, ArgModes, RHSVar, ArgMode) :-
+    ( if
+        RHSVars = [RHSVarPrime],
+        ArgModes = [ArgModePrime]
+    then
+        RHSVar = RHSVarPrime,
+        ArgMode = ArgModePrime
+    else
+        unexpected($pred, "arity != 1)")
+    ).
+
+get_notag_or_direct_arg_arg(RHSVars, RHSVar) :-
+    ( if RHSVars = [RHSVarPrime] then
+        RHSVar = RHSVarPrime
+    else
+        unexpected($pred, "arity != 1)")
+    ).
+
+%-----------------------------------------------------------------------------%
+
+get_from_ground_term_construct_info(TermVar, Goal,
+         TermVarIsNeeded, Conjuncts, GoalInfo) :-
+    Goal = hlds_goal(GoalExpr, GoalInfo),
+    NonLocals = goal_info_get_nonlocals(GoalInfo),
+    set_of_var.to_sorted_list(NonLocals, NonLocalList),
+    (
+        (
+            NonLocalList = [],
+            TermVarIsNeeded = termvar_is_not_needed
+        ;
+            NonLocalList = [NonLocal],
+            ( if NonLocal = TermVar then
+                TermVarIsNeeded = termvar_is_needed
+            else
+                unexpected($pred, "unexpected nonlocal")
+            )
+        ),
+        ( if GoalExpr = conj(plain_conj, ConjunctsPrime) then
+            Conjuncts = ConjunctsPrime
+        else
+            unexpected($pred, "unexpected nonlocal")
+        )
+    ;
+        NonLocalList = [_, _ | _],
+        unexpected($pred, "unexpected nonlocals")
+    ).
+
+get_from_ground_term_construct_conjunct_info(Goal, LHSVar, ConsId, RHSVars,
+        GoalInfo) :-
+    Goal = hlds_goal(GoalExpr, GoalInfo),
+    ( if
+        GoalExpr = unify(_, _, _, Unify, _),
+        Unify = construct(LHSVarPrime, ConsIdPrime, RHSVarsPrime,
+            _, _, _, SubInfo),
+        SubInfo = no_construct_sub_info
+    then
+        LHSVar = LHSVarPrime,
+        ConsId = ConsIdPrime,
+        RHSVars = RHSVarsPrime
+    else
+        unexpected($pred, "unexpected goal as fgt_construct conjunct")
+    ).
+
 %-----------------------------------------------------------------------------%
 :- end_module hlds.goal_form.
 %-----------------------------------------------------------------------------%
