@@ -10,7 +10,7 @@
 % Main authors: juliensf.
 %
 % This module defines the part of the HLDS that deals with procedure and call
-% site arguments.  (See comments at the head of polymorphism.m for further
+% site arguments. (See comments at the head of polymorphism.m for further
 % details.)
 %
 %-----------------------------------------------------------------------------%
@@ -18,6 +18,7 @@
 :- module hlds.hlds_args.
 :- interface.
 
+:- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
 
 :- import_module list.
@@ -27,15 +28,15 @@
 
 %-----------------------------------------------------------------------------%
 
-	% This type represents the arguments to a predicate symbol, or the
-	% arguments and result of a function symbol.  The arguments may be
-	% variables, types, modes, etc, depending on the context.
-	%
-	% Rather than keep all arguments in a single list, we retain
-	% information about the origin of each argument (such as whether
-	% it was introduced by polymorphism.m to hold a type_info).  This
-	% simplifies the processing in polymorphism.m, and also abstracts
-	% away the specific calling convention that we use.
+    % This type represents the arguments to a predicate symbol, or
+    % the arguments and result of a function symbol. The arguments may be
+    % variables, types, modes, etc, depending on the context.
+    %
+    % Rather than keep all arguments in a single list, we retain information
+    % about the origin of each argument (such as whether it was introduced
+    % by polymorphism.m to hold a type_info). This simplifies the processing
+    % in polymorphism.m, and also abstracts away the specific calling
+    % convention that we use.
     %
 :- type proc_arg_vector(T).
 
@@ -83,13 +84,13 @@
 
 %-----------------------------------------------------------------------------%
 %
-% Utility predicates that operate on proc_arg_vectors
+% Utility predicates that operate on proc_arg_vectors.
 %
 
-    % Return a list of the items in a arg_vector.  The order of the
-    % list corresponds to that required by the calling conventions.
-    % See comments at the head of polymorphism.m. for details.  If the
-    % arg_vector is for a function then the last element in the list
+    % Return a list of the items in a arg_vector. The order of the list
+    % corresponds to that required by the calling conventions.
+    % See comments at the head of polymorphism.m for details.
+    % If the arg_vector is for a function, then the last element in the list
     % will correspond to the function return value.
     %
 :- func proc_arg_vector_to_list(proc_arg_vector(T)) = list(T).
@@ -119,7 +120,7 @@
 :- pred proc_arg_vector_member(proc_arg_vector(T)::in, T::in) is semidet.
 
     % Partition the given arg_vector into a list of arguments and
-    % a function return value.  Throws an exception if the arg_vector does
+    % a function return value. Throws an exception if the arg_vector does
     % not correspond to a function.
     %
 :- pred proc_arg_vector_to_func_args(proc_arg_vector(T)::in,
@@ -127,12 +128,12 @@
 
 %-----------------------------------------------------------------------------%
 %
-% Higher-order operations on proc_arg_vectors
+% Higher-order operations on proc_arg_vectors.
 %
 
 %
-% NOTE: these higher-order operations all work in a similar fashion
-%       to their counterparts in library/list.m.
+% NOTE these higher-order operations all work in a similar fashion
+% to their counterparts in library/list.m.
 
 :- func proc_arg_vector_map(func(T) = U, proc_arg_vector(T)) =
     proc_arg_vector(U).
@@ -179,7 +180,7 @@
 
 %-----------------------------------------------------------------------------%
 %
-% Stuff related to the polymorphism pass
+% Stuff related to the polymorphism pass.
 %
 
     % This type represents those arguments of a predicate or function
@@ -209,7 +210,7 @@
 
     % Convert a poly_arg_vector into a list.
     % XXX ARGVEC - this is only temporary until the proc_info structure use
-    % proc_arg_vectors.  We should then provide a predicate that merges a
+    % proc_arg_vectors. We should then provide a predicate that merges a
     % poly_arg_vector with a proc_arg_vector.
     %
 :- func poly_arg_vector_to_list(poly_arg_vector(T)) = list(T).
@@ -219,6 +220,7 @@
 
 :- implementation.
 
+:- import_module parse_tree.
 :- import_module parse_tree.prog_type.
     % Required for apply_partial_map_to_list.
     % XXX That should really live in a different module.
@@ -348,7 +350,7 @@ proc_arg_vector_to_list(ArgVec) = List :-
 
 proc_arg_vector_to_set(ArgVec) = Set :-
     List = proc_arg_vector_to_list(ArgVec),
-    Set  = set.from_list(List).
+    Set  = set.list_to_set(List).
 
 apply_renaming_to_proc_arg_vector(Renaming, ArgVec0, ArgVec) :-
     ArgVec0 = proc_arg_vector(InstanceTypeInfos0, InstanceTypeClassInfos0,
@@ -366,9 +368,9 @@ apply_renaming_to_proc_arg_vector(Renaming, ArgVec0, ArgVec) :-
     apply_partial_map_to_list(Renaming, OrigArgs0, OrigArgs),
     (
         MaybeRetValue0 = yes(Value0),
-        ( map.search(Renaming, Value0, Value) ->
+        ( if map.search(Renaming, Value0, Value) then
             MaybeRetValue = yes(Value)
-        ;
+        else
             MaybeRetValue = yes(Value0)
         )
     ;
@@ -387,17 +389,26 @@ proc_arg_vector_partition_poly_args(ArgVec, PolyArgs, NonPolyArgs) :-
         UnivTypeInfos, ExistTypeInfos, UnivTypeClassInfos,
         ExistTypeClassInfos], PolyArgs),
     (
-        MaybeRetValue = yes(Value),
-        RetValue = [Value]
+        MaybeRetValue = yes(RetValue),
+        NonPolyArgs = OrigArgs ++ [RetValue]
     ;
         MaybeRetValue = no,
-        RetValue = []
-    ),
-    NonPolyArgs = OrigArgs ++ RetValue.
+        NonPolyArgs = OrigArgs
+    ).
 
-proc_arg_vector_member(Vec, V) :-
-    List = proc_arg_vector_to_list(Vec),
-    list.member(V, List).
+proc_arg_vector_member(ArgVec, Var) :-
+    ArgVec = proc_arg_vector(InstanceTypeInfos, InstanceTypeClassInfos,
+        UnivTypeInfos, ExistTypeInfos, UnivTypeClassInfos,
+        ExistTypeClassInfos, OrigArgs, MaybeRetValue),
+    ( list.member(Var, OrigArgs)
+    ; MaybeRetValue = yes(Var)
+    ; list.member(Var, InstanceTypeInfos)
+    ; list.member(Var, InstanceTypeClassInfos)
+    ; list.member(Var, UnivTypeInfos)
+    ; list.member(Var, ExistTypeInfos)
+    ; list.member(Var, UnivTypeClassInfos)
+    ; list.member(Var, ExistTypeClassInfos)
+    ).
 
 proc_arg_vector_to_func_args(Vector, FuncArgs, FuncRetVal) :-
     Vector = proc_arg_vector(InstanceTypeInfos, InstanceTypeClassInfos,
@@ -410,7 +421,7 @@ proc_arg_vector_to_func_args(Vector, FuncArgs, FuncRetVal) :-
         MaybeRetValue = yes(FuncRetVal)
     ;
         MaybeRetValue = no,
-        unexpected($module, $pred, "predicate")
+        unexpected($pred, "predicate")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -471,17 +482,21 @@ proc_arg_vector_map_corresponding(P, A, B, C) :-
     list.map_corresponding(P, ArgsA, ArgsB, ArgsC),
     (
         MaybeRetValA = yes(RetValA),
-        MaybeRetValB = yes(RetValB)
-    ->
+        MaybeRetValB = yes(RetValB),
         P(RetValA, RetValB, RetValC),
         MaybeRetValC = yes(RetValC)
     ;
-        MaybeRetValA = no,
-        MaybeRetValB = no
-    ->
-        MaybeRetValC = no
+        MaybeRetValA = yes(_),
+        MaybeRetValB = no,
+        unexpected($pred, "mismatched proc_arg_vectors")
     ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
+        MaybeRetValA = no,
+        MaybeRetValB = yes(_),
+        unexpected($pred, "mismatched proc_arg_vectors")
+    ;
+        MaybeRetValA = no,
+        MaybeRetValB = no,
+        MaybeRetValC = no
     ),
     C = proc_arg_vector(ITIC, ITCIC, UTIC, ETIC, UTCIC, ETCIC, ArgsC,
         MaybeRetValC).
@@ -516,17 +531,21 @@ proc_arg_vector_map_corresponding_foldl2(P, A, B, C, !Acc1, !Acc2) :-
     list.map_corresponding_foldl2(P, ArgsA, ArgsB, ArgsC, !Acc1, !Acc2),
     (
         MaybeRetValA = yes(RetValA),
-        MaybeRetValB = yes(RetValB)
-    ->
+        MaybeRetValB = yes(RetValB),
         P(RetValA, RetValB, RetValC, !Acc1, !Acc2),
         MaybeRetValC = yes(RetValC)
     ;
-        MaybeRetValA = no,
-        MaybeRetValB = no
-    ->
-        MaybeRetValC = no
+        MaybeRetValA = yes(_),
+        MaybeRetValB = no,
+        unexpected($pred, "mismatched proc_arg_vectors")
     ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
+        MaybeRetValA = no,
+        MaybeRetValB = yes(_),
+        unexpected($pred, "mismatched proc_arg_vectors")
+    ;
+        MaybeRetValA = no,
+        MaybeRetValB = no,
+        MaybeRetValC = no
     ),
     C = proc_arg_vector(ITIC, ITCIC, UTIC, ETIC, UTCIC, ETCIC, ArgsC,
         MaybeRetValC).
@@ -545,16 +564,19 @@ proc_arg_vector_foldl3_corresponding(P, A, B, !Acc1, !Acc2, !Acc3) :-
     list.foldl3_corresponding(P, ArgsA, ArgsB, !Acc1, !Acc2, !Acc3),
     (
         MaybeRetValA = yes(RetValA),
-        MaybeRetValB = yes(RetValB)
-    ->
+        MaybeRetValB = yes(RetValB),
         P(RetValA, RetValB, !Acc1, !Acc2, !Acc3)
+    ;
+        MaybeRetValA = yes(_),
+        MaybeRetValB = no,
+        unexpected($pred, "mismatched proc_arg_vectors")
+    ;
+        MaybeRetValA = no,
+        MaybeRetValB = yes(_),
+        unexpected($pred, "mismatched proc_arg_vectors")
     ;
         MaybeRetValA = no,
         MaybeRetValB = no
-    ->
-        true
-    ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
     ).
 
 proc_arg_vector_foldl2_corresponding3(P, A, B, C, !Acc1, !Acc2) :-
@@ -571,20 +593,20 @@ proc_arg_vector_foldl2_corresponding3(P, A, B, C, !Acc1, !Acc2) :-
     list.foldl2_corresponding3(P, UTCIA, UTCIB, UTCIC, !Acc1, !Acc2),
     list.foldl2_corresponding3(P, ETCIA, ETCIB, ETCIC, !Acc1, !Acc2),
     list.foldl2_corresponding3(P, ArgsA, ArgsB, ArgsC, !Acc1, !Acc2),
-    (
+    ( if
         MaybeRetValA = yes(RetValA),
         MaybeRetValB = yes(RetValB),
         MaybeRetValC = yes(RetValC)
-    ->
+    then
         P(RetValA, RetValB, RetValC, !Acc1, !Acc2)
-    ;
+    else if
         MaybeRetValA = no,
         MaybeRetValB = no,
         MaybeRetValC = no
-    ->
+    then
         true
-    ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
+    else
+        unexpected($pred, "mismatched proc_arg_vectors")
     ).
 
 proc_arg_vector_foldl3_corresponding3(P, A, B, C, !Acc1, !Acc2, !Acc3) :-
@@ -601,20 +623,20 @@ proc_arg_vector_foldl3_corresponding3(P, A, B, C, !Acc1, !Acc2, !Acc3) :-
     list.foldl3_corresponding3(P, UTCIA, UTCIB, UTCIC, !Acc1, !Acc2, !Acc3),
     list.foldl3_corresponding3(P, ETCIA, ETCIB, ETCIC, !Acc1, !Acc2, !Acc3),
     list.foldl3_corresponding3(P, ArgsA, ArgsB, ArgsC, !Acc1, !Acc2, !Acc3),
-    (
+    ( if
         MaybeRetValA = yes(RetValA),
         MaybeRetValB = yes(RetValB),
         MaybeRetValC = yes(RetValC)
-    ->
+    then
         P(RetValA, RetValB, RetValC, !Acc1, !Acc2, !Acc3)
-    ;
+    else if
         MaybeRetValA = no,
         MaybeRetValB = no,
         MaybeRetValC = no
-    ->
+    then
         true
-    ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
+    else
+        unexpected($pred, "mismatched proc_arg_vectors")
     ).
 
 proc_arg_vector_foldl4_corresponding3(P, A, B, C, !Acc1, !Acc2, !Acc3,
@@ -639,33 +661,33 @@ proc_arg_vector_foldl4_corresponding3(P, A, B, C, !Acc1, !Acc2, !Acc3,
         !Acc4),
     list.foldl4_corresponding3(P, ArgsA, ArgsB, ArgsC, !Acc1, !Acc2, !Acc3,
         !Acc4),
-    (
+    ( if
         MaybeRetValA = yes(RetValA),
         MaybeRetValB = yes(RetValB),
         MaybeRetValC = yes(RetValC)
-    ->
+    then
         P(RetValA, RetValB, RetValC, !Acc1, !Acc2, !Acc3, !Acc4)
-    ;
+    else if
         MaybeRetValA = no,
         MaybeRetValB = no,
         MaybeRetValC = no
-    ->
+    then
         true
-    ;
-        unexpected($module, $pred, "mismatched proc_arg_vectors")
+    else
+        unexpected($pred, "mismatched proc_arg_vectors")
     ).
 
 %-----------------------------------------------------------------------------%
 %
-% Stuff related to the polymorphism transformation
+% Stuff related to the polymorphism transformation.
 %
 
     % Internally we represent a poly_arg_vector as a proc_arg_vector.
     % This ensures that poly_arg_vectors obey the same calling convention
     % w.r.t introduced type_info and typeclass_info arguments that
-    % proc_arg_vectors do.  For the proc_arg_vectors that are
-    % used to represent poly_arg_vectors we insist that the the last
-    % two fields are the empty list and `no' respectively.
+    % proc_arg_vectors do. For the proc_arg_vectors that are used to represent
+    % poly_arg_vectors, we insist that the last two fields are
+    % the empty list and `no' respectively.
     %
 :- type poly_arg_vector(T) == proc_arg_vector(T).
 
