@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2002-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -11,14 +12,15 @@
 % Various utilities routines for use during HLDS generation.
 %
 %-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
 
 :- module hlds.hlds_code_util.
 :- interface.
 
+:- import_module hlds.hlds_class.
 :- import_module hlds.hlds_data.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_module.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
@@ -46,11 +48,6 @@
 :- func base_typeclass_info_cons_id(instance_table,
     prog_constraint, int, list(mer_type)) = cons_id.
 
-    % Succeeds iff this inst is one that can be used in a valid
-    % mutable declaration.
-    %
-:- pred is_valid_mutable_inst(module_info::in, mer_inst::in) is semidet.
-
 %-----------------------------------------------------------------------------%
 
     % Find the procedure with argmodes which match the ones we want.
@@ -62,24 +59,27 @@
     % If there was no mode declaration, then use the inferred argmodes.
     % Allow for a renaming between the inst vars.
     %
-:- pred get_procedure_matching_declmodes_with_renaming(
-    assoc_list(proc_id, proc_info)::in, list(mer_mode)::in,
-    module_info::in, proc_id::out) is semidet.
+:- pred get_procedure_matching_declmodes_with_renaming(module_info::in,
+    assoc_list(proc_id, proc_info)::in, list(mer_mode)::in, proc_id::out)
+    is semidet.
 
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.mode_util.
 :- import_module check_hlds.type_util.
-:- import_module hlds.hlds_pred.
+:- import_module libs.
 :- import_module libs.globals.
-:- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_type.
 
 :- import_module char.
 :- import_module map.
+:- import_module maybe.
 :- import_module pair.
 :- import_module require.
 :- import_module set.
@@ -87,61 +87,91 @@
 
 %-----------------------------------------------------------------------------%
 
-cons_id_to_tag(ModuleInfo, ConsId) = Tag:-
+cons_id_to_tag(ModuleInfo, ConsId) = ConsTag:-
     (
-        ConsId = int_const(Int),
-        Tag = int_tag(Int)
+        ConsId = some_int_const(IntConst),
+        (
+            IntConst = int_const(Int),
+            ConsTag = int_tag(int_tag_int(Int))
+        ;
+            IntConst = uint_const(UInt),
+            ConsTag = int_tag(int_tag_uint(UInt))
+        ;
+            IntConst = int8_const(Int8),
+            ConsTag = int_tag(int_tag_int8(Int8))
+        ;
+            IntConst = uint8_const(UInt8),
+            ConsTag = int_tag(int_tag_uint8(UInt8))
+        ;
+            IntConst = int16_const(Int16),
+            ConsTag = int_tag(int_tag_int16(Int16))
+        ;
+            IntConst = uint16_const(UInt16),
+            ConsTag = int_tag(int_tag_uint16(UInt16))
+        ;
+            IntConst = int32_const(Int32),
+            ConsTag = int_tag(int_tag_int32(Int32))
+        ;
+            IntConst = uint32_const(UInt32),
+            ConsTag = int_tag(int_tag_uint32(UInt32))
+        ;
+            IntConst = int64_const(Int64),
+            ConsTag = int_tag(int_tag_int64(Int64))
+        ;
+            IntConst = uint64_const(UInt64),
+            ConsTag = int_tag(int_tag_uint64(UInt64))
+        )
     ;
         ConsId = float_const(Float),
-        Tag = float_tag(Float)
+        ConsTag = float_tag(Float)
     ;
         ConsId = char_const(Char),
         char.to_int(Char, CharCode),
-        Tag = int_tag(CharCode)
+        ConsTag = int_tag(int_tag_int(CharCode))
     ;
         ConsId = string_const(String),
-        Tag = string_tag(String)
+        ConsTag = string_tag(String)
     ;
         ConsId = impl_defined_const(_),
-        unexpected($module, $pred, "implementation_defined_const")
+        unexpected($pred, "implementation_defined_const")
     ;
         ConsId = closure_cons(ShroudedPredProcId, EvalMethod),
         proc(PredId, ProcId) = unshroud_pred_proc_id(ShroudedPredProcId),
-        Tag = closure_tag(PredId, ProcId, EvalMethod)
+        ConsTag = closure_tag(PredId, ProcId, EvalMethod)
     ;
         ConsId = type_ctor_info_const(ModuleName, TypeName, Arity),
-        Tag = type_ctor_info_tag(ModuleName, TypeName, Arity)
+        ConsTag = type_ctor_info_tag(ModuleName, TypeName, Arity)
     ;
         ConsId = base_typeclass_info_const(ModuleName, ClassName,
             _Instance, EncodedArgs),
-        Tag = base_typeclass_info_tag(ModuleName, ClassName, EncodedArgs)
+        ConsTag = base_typeclass_info_tag(ModuleName, ClassName, EncodedArgs)
     ;
         ( ConsId = type_info_cell_constructor(_)
         ; ConsId = typeclass_info_cell_constructor
         ),
-        Tag = unshared_tag(0)
+        ConsTag = remote_args_tag(remote_args_unshared(ptag(0u8)))
     ;
         ConsId = type_info_const(TIConstNum),
-        Tag = type_info_const_tag(TIConstNum)
+        ConsTag = type_info_const_tag(TIConstNum)
     ;
         ConsId = typeclass_info_const(TCIConstNum),
-        Tag = typeclass_info_const_tag(TCIConstNum)
+        ConsTag = typeclass_info_const_tag(TCIConstNum)
     ;
         ConsId = ground_term_const(ConstNum, SubConsId),
         SubConsTag = cons_id_to_tag(ModuleInfo, SubConsId),
-        Tag = ground_term_const_tag(ConstNum, SubConsTag)
+        ConsTag = ground_term_const_tag(ConstNum, SubConsTag)
     ;
         ConsId = tabling_info_const(ShroudedPredProcId),
         proc(PredId, ProcId) = unshroud_pred_proc_id(ShroudedPredProcId),
-        Tag = tabling_info_tag(PredId, ProcId)
+        ConsTag = tabling_info_tag(PredId, ProcId)
     ;
         ConsId = deep_profiling_proc_layout(ShroudedPredProcId),
         proc(PredId, ProcId) = unshroud_pred_proc_id(ShroudedPredProcId),
-        Tag = deep_profiling_proc_layout_tag(PredId, ProcId)
+        ConsTag = deep_profiling_proc_layout_tag(PredId, ProcId)
     ;
-        ConsId = table_io_decl(ShroudedPredProcId),
+        ConsId = table_io_entry_desc(ShroudedPredProcId),
         proc(PredId, ProcId) = unshroud_pred_proc_id(ShroudedPredProcId),
-        Tag = table_io_decl_tag(PredId, ProcId)
+        ConsTag = table_io_entry_tag(PredId, ProcId)
     ;
         ConsId = tuple_cons(Arity),
         % Tuples do not need a tag. Note that unary tuples are not treated
@@ -150,40 +180,24 @@ cons_id_to_tag(ModuleInfo, ConsId) = Tag:-
         module_info_get_globals(ModuleInfo, Globals),
         globals.get_target(Globals, TargetLang),
         (
-            ( TargetLang = target_c
-            ; TargetLang = target_x86_64
-            ; TargetLang = target_erlang
-            ), 
-            ( Arity = 0 ->
-                Tag = int_tag(0)
-            ;
-                Tag = single_functor_tag
+            TargetLang = target_c,
+            ( if Arity = 0 then
+                ConsTag = int_tag(int_tag_int(0))
+            else
+                ConsTag = remote_args_tag(remote_args_only_functor)
             )
         ;
             % For these target languages, converting arity-zero tuples into
             % dummy integer tags results in invalid code being generated.
-            ( TargetLang = target_il
-            ; TargetLang = target_csharp
+            ( TargetLang = target_csharp
             ; TargetLang = target_java
             ),
-            Tag = single_functor_tag
+            ConsTag = remote_args_tag(remote_args_only_functor)
         )
     ;
-        ConsId = cons(_Name, _Arity, TypeCtor),
-        module_info_get_type_table(ModuleInfo, TypeTable),
-        lookup_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
-        hlds_data.get_type_defn_body(TypeDefn, TypeBody),
-        (
-            TypeBody = hlds_du_type(_, ConsTagTable, _, _, _, _, _, _, _),
-            map.lookup(ConsTagTable, ConsId, Tag)
-        ;
-            ( TypeBody = hlds_eqv_type(_)
-            ; TypeBody = hlds_foreign_type(_)
-            ; TypeBody = hlds_solver_type(_, _)
-            ; TypeBody = hlds_abstract_type(_)
-            ),
-            unexpected($module, $pred, "type is not d.u. type")
-        )
+        ConsId = cons(_Name, _Arity, _TypeCtor),
+        get_cons_repn_defn_det(ModuleInfo, ConsId, ConsRepn),
+        ConsTag = ConsRepn ^ cr_tag
     ).
 
 %-----------------------------------------------------------------------------%
@@ -192,12 +206,12 @@ make_instance_string(InstanceTypes, InstanceString) :-
     % Note that for historical reasons, builtin types are treated as being
     % unqualified (`int') rather than being qualified (`builtin.int')
     % at this point.
-    list.map(type_to_string, InstanceTypes, InstanceStrings),
+    list.map(instance_type_ctor_to_string, InstanceTypes, InstanceStrings),
     string.append_list(InstanceStrings, InstanceString).
 
-:- pred type_to_string(mer_type::in, string::out) is det.
+:- pred instance_type_ctor_to_string(mer_type::in, string::out) is det.
 
-type_to_string(Type, String) :-
+instance_type_ctor_to_string(Type, String) :-
     type_to_ctor_det(Type, TypeCtor),
     TypeCtor = type_ctor(TypeName, TypeArity),
     TypeNameString = sym_name_to_string_sep(TypeName, "__"),
@@ -216,60 +230,13 @@ base_typeclass_info_cons_id(InstanceTable, Constraint, InstanceNum,
     ClassId = class_id(ClassName, list.length(ConstraintArgTypes)),
     map.lookup(InstanceTable, ClassId, InstanceList),
     list.det_index1(InstanceList, InstanceNum, InstanceDefn),
-    InstanceModuleName = InstanceDefn ^ instance_module,
+    InstanceModuleName = InstanceDefn ^ instdefn_module,
     make_instance_string(InstanceTypes, InstanceString),
     ConsId = base_typeclass_info_const(InstanceModuleName, ClassId,
         InstanceNum, InstanceString).
 
 %----------------------------------------------------------------------------%
 
-is_valid_mutable_inst(ModuleInfo, Inst) :-
-    set.init(Expansions),
-    is_valid_mutable_inst_2(ModuleInfo, Inst, Expansions).
-
-:- pred is_valid_mutable_inst_2(module_info::in, mer_inst::in,
-    set(inst_name)::in) is semidet.
-
-is_valid_mutable_inst_2(ModuleInfo, Inst, Expansions0) :-
-    (
-        ( Inst = any(Uniq, _)
-        ; Inst = ground(Uniq, _)
-        ),
-        Uniq = shared
-    ;
-        Inst = bound(shared, _, BoundInsts),
-        are_valid_mutable_bound_insts(ModuleInfo, BoundInsts, Expansions0)
-    ;
-        Inst = defined_inst(InstName),
-        ( if not set.member(InstName, Expansions0) then
-            set.insert(InstName, Expansions0, Expansions),
-            inst_lookup(ModuleInfo, InstName, SubInst),
-            is_valid_mutable_inst_2(ModuleInfo, SubInst, Expansions)
-        else
-            true
-        )
-    ).
-
-:- pred are_valid_mutable_bound_insts(module_info::in, list(bound_inst)::in,
-    set(inst_name)::in) is semidet.
-
-are_valid_mutable_bound_insts(_ModuleInfo, [], _Expansions0).
-are_valid_mutable_bound_insts(ModuleInfo, [BoundInst | BoundInsts],
-        Expansions0) :-
-    BoundInst = bound_functor(_ConsId, ArgInsts),
-    are_valid_mutable_insts(ModuleInfo, ArgInsts, Expansions0),
-    are_valid_mutable_bound_insts(ModuleInfo, BoundInsts, Expansions0).
-
-:- pred are_valid_mutable_insts(module_info::in, list(mer_inst)::in,
-    set(inst_name)::in) is semidet.
-
-are_valid_mutable_insts(_ModuleInfo, [], _Expansions0).
-are_valid_mutable_insts(ModuleInfo, [Inst | Insts], Expansions0) :-
-    is_valid_mutable_inst_2(ModuleInfo, Inst, Expansions0),
-    are_valid_mutable_insts(ModuleInfo, Insts, Expansions0).
-
-%----------------------------------------------------------------------------%
-
 get_procedure_matching_argmodes(Procs, Modes0, ModuleInfo, ProcId) :-
     list.map(constrain_inst_vars_in_mode, Modes0, Modes),
     get_procedure_matching_argmodes_2(Procs, Modes, ModuleInfo, ProcId).
@@ -280,9 +247,9 @@ get_procedure_matching_argmodes(Procs, Modes0, ModuleInfo, ProcId) :-
 get_procedure_matching_argmodes_2([P | Procs], Modes, ModuleInfo, OurProcId) :-
     P = ProcId - ProcInfo,
     proc_info_get_argmodes(ProcInfo, ArgModes),
-    ( mode_list_matches(Modes, ArgModes, ModuleInfo) ->
+    ( if mode_list_matches(Modes, ArgModes, ModuleInfo) then
         OurProcId = ProcId
-    ;
+    else
         get_procedure_matching_argmodes_2(Procs, Modes, ModuleInfo, OurProcId)
     ).
 
@@ -301,25 +268,25 @@ mode_list_matches([Mode1 | Modes1], [Mode2 | Modes2], ModuleInfo) :-
 %----------------------------------------------------------------------------%
 %----------------------------------------------------------------------------%
 
-get_procedure_matching_declmodes_with_renaming(Procs, Modes0,
-        ModuleInfo, ProcId) :-
+get_procedure_matching_declmodes_with_renaming(ModuleInfo, Procs, Modes0,
+        ProcId) :-
     list.map(constrain_inst_vars_in_mode, Modes0, Modes),
-    get_procedure_matching_declmodes_with_renaming_2(Procs, Modes,
-        ModuleInfo, ProcId).
+    get_procedure_matching_declmodes_with_renaming_2(ModuleInfo, Procs, Modes,
+        ProcId).
 
-:- pred get_procedure_matching_declmodes_with_renaming_2(
-    assoc_list(proc_id, proc_info)::in, list(mer_mode)::in,
-    module_info::in, proc_id::out) is semidet.
+:- pred get_procedure_matching_declmodes_with_renaming_2(module_info::in,
+    assoc_list(proc_id, proc_info)::in, list(mer_mode)::in, proc_id::out)
+    is semidet.
 
-get_procedure_matching_declmodes_with_renaming_2([P | Procs], Modes,
-        ModuleInfo, OurProcId) :-
-    P = ProcId - ProcInfo,
+get_procedure_matching_declmodes_with_renaming_2(ModuleInfo, [Proc | Procs],
+        Modes, MatchingProcId) :-
+    Proc = ProcId - ProcInfo,
     proc_info_declared_argmodes(ProcInfo, ArgModes),
-    ( mode_list_matches_with_renaming(Modes, ArgModes, ModuleInfo) ->
-        OurProcId = ProcId
-    ;
-        get_procedure_matching_declmodes_with_renaming_2(Procs, Modes,
-            ModuleInfo, OurProcId)
+    ( if mode_list_matches_with_renaming(Modes, ArgModes, ModuleInfo) then
+        MatchingProcId = ProcId
+    else
+        get_procedure_matching_declmodes_with_renaming_2(ModuleInfo, Procs,
+            Modes, MatchingProcId)
     ).
 
 :- type inst_var_renaming == map(inst_var, inst_var).
@@ -434,22 +401,22 @@ match_insts_with_renaming(ModuleInfo, InstA, InstB, Renaming) :-
         match_insts_with_renaming(ModuleInfo, SpecInstA, SpecInstB, Renaming0),
         ListVarA = set.to_sorted_list(InstVarSetA),
         ListVarB = set.to_sorted_list(InstVarSetB),
-        (
+        ( if
             ListVarA = [VarA0],
             ListVarB = [VarB0]
-        ->
+        then
             VarA = VarA0,
             VarB = VarB0
-        ;
-            unexpected($module, $pred, "non-singleton sets")
+        else
+            unexpected($pred, "non-singleton sets")
         ),
-        ( map.search(Renaming0, VarA, SpecVarB) ->
+        ( if map.search(Renaming0, VarA, SpecVarB) then
             % If VarA was already in the renaming then check that it is
             % consistent with the renaming from the set of inst vars.
             VarB = SpecVarB,
             Renaming = Renaming0
-        ;
-            map.insert(VarA, VarB, Renaming0, Renaming)
+        else
+            map.det_insert(VarA, VarB, Renaming0, Renaming)
         )
     ;
         InstA = defined_inst(InstNameA),
@@ -469,8 +436,8 @@ match_insts_with_renaming(ModuleInfo, InstA, InstB, Renaming) :-
 match_ho_inst_infos_with_renaming(ModuleInfo, HOInstInfoA, HOInstInfoB,
         Renaming) :-
     (
-        HOInstInfoA = none,
-        HOInstInfoB = none,
+        HOInstInfoA = none_or_default_func,
+        HOInstInfoB = none_or_default_func,
         Renaming = map.init
     ;
         HOInstInfoA = higher_order(PredInstInfoA),
@@ -492,8 +459,8 @@ match_inst_names_with_renaming(ModuleInfo, InstNameA, InstNameB, Renaming) :-
     ;
         % XXX The rest of these are introduced by the compiler, it doesn't
         % look like they need any special treatment.
-        ( InstNameA = merge_inst(_, _)
-        ; InstNameA = unify_inst(_, _, _, _)
+        ( InstNameA = unify_inst(_, _, _, _)
+        ; InstNameA = merge_inst(_, _)
         ; InstNameA = ground_inst(_, _, _, _)
         ; InstNameA = any_inst(_, _, _, _)
         ; InstNameA = shared_inst(_)
@@ -514,7 +481,6 @@ merge_inst_var_renamings(RenamingA, RenamingB, Result) :-
 
 merge_common_inst_vars(A, A, A).
 
-
 %----------------------------------------------------------------------------%
-:- end_module hlds_code_util.
+:- end_module hlds.hlds_code_util.
 %----------------------------------------------------------------------------%
