@@ -5,13 +5,13 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-% 
+%
 % File: hlds_llds.m.
 % Author: zs.
-% 
+%
 % This module defines annotations on HLDS goals that are used by the LLDS
 % back end.
-% 
+%
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -19,7 +19,9 @@
 :- interface.
 
 :- import_module hlds.hlds_goal.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_rename.
 :- import_module parse_tree.set_of_var.
 
 :- import_module bool.
@@ -47,7 +49,7 @@
 
     % Maps variables to their stack slots.
     %
-:- type stack_slots ==  map(prog_var, stack_slot).
+:- type stack_slots == map(prog_var, stack_slot).
 
 :- func explain_stack_slots(stack_slots, prog_varset) = string.
 
@@ -77,7 +79,7 @@
 
     % Authoritative information about where variables must be put
     % at the ends of branches of branched control structures.
-    % However, between the follow_vars and and store_alloc passes,
+    % However, between the follow_vars and store_alloc passes,
     % these fields temporarily hold follow_vars information.
     % The final value is not allowed to map any variable to any_reg.
     %
@@ -91,8 +93,8 @@
 :- type resume_locs
     --->    resume_locs_orig_only
     ;       resume_locs_stack_only
-    ;       resume_locs_orig_and_stack
-    ;       resume_locs_stack_and_orig.
+    ;       resume_locs_orig_then_stack
+    ;       resume_locs_stack_then_orig.
 
     % When code in live_vars.m finds out what variables need to be saved
     % across calls (including generic calls and foreign_procs that can call
@@ -306,7 +308,6 @@
 :- implementation.
 
 :- import_module assoc_list.
-:- import_module list.
 :- import_module pair.
 :- import_module require.
 :- import_module string.
@@ -430,63 +431,63 @@ explain_stack_slots_2([Var - Slot | Rest], VarSet, !Explanation) :-
 
 goal_info_get_pre_births(GoalInfo, PreBirths) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( PreBirthsPrime = CodeGenInfo ^ llds_code_gen ^ pre_births ->
+    ( if PreBirthsPrime = CodeGenInfo ^ llds_code_gen ^ pre_births then
         PreBirths = PreBirthsPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_post_births(GoalInfo, PostBirths) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( PostBirthsPrime = CodeGenInfo ^ llds_code_gen ^ post_births ->
+    ( if PostBirthsPrime = CodeGenInfo ^ llds_code_gen ^ post_births then
         PostBirths = PostBirthsPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_pre_deaths(GoalInfo, PreDeaths) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( PreDeathsPrime = CodeGenInfo ^ llds_code_gen ^ pre_deaths ->
+    ( if PreDeathsPrime = CodeGenInfo ^ llds_code_gen ^ pre_deaths then
         PreDeaths = PreDeathsPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_post_deaths(GoalInfo, PostDeaths) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( PostDeathsPrime = CodeGenInfo ^ llds_code_gen ^ post_deaths ->
+    ( if PostDeathsPrime = CodeGenInfo ^ llds_code_gen ^ post_deaths then
         PostDeaths = PostDeathsPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_follow_vars(GoalInfo, FollowVars) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( FollowVarsPrime = CodeGenInfo ^ llds_code_gen ^ follow_vars ->
+    ( if FollowVarsPrime = CodeGenInfo ^ llds_code_gen ^ follow_vars then
         FollowVars = FollowVarsPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_store_map(GoalInfo, StoreMap) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( StoreMapPrime = CodeGenInfo ^ llds_code_gen ^ store_map ->
+    ( if StoreMapPrime = CodeGenInfo ^ llds_code_gen ^ store_map then
         StoreMap = StoreMapPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_resume_point(GoalInfo, ResumePoint) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( ResumePointPrime = CodeGenInfo ^ llds_code_gen ^ resume_point ->
+    ( if ResumePointPrime = CodeGenInfo ^ llds_code_gen ^ resume_point then
         ResumePoint = ResumePointPrime
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_maybe_need_across_call(GoalInfo, MaybeNeedAtCall) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need ->
+    ( if MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need then
         (
             MaybeNeed = need_call(NeedAtCall),
             MaybeNeedAtCall = yes(NeedAtCall)
@@ -497,13 +498,13 @@ goal_info_get_maybe_need_across_call(GoalInfo, MaybeNeedAtCall) :-
             ),
             MaybeNeedAtCall = no
         )
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_maybe_need_in_resume(GoalInfo, MaybeNeedInResume) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need ->
+    ( if MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need then
         (
             MaybeNeed = need_resume(NeedInResume),
             MaybeNeedInResume = yes(NeedInResume)
@@ -514,13 +515,13 @@ goal_info_get_maybe_need_in_resume(GoalInfo, MaybeNeedInResume) :-
             ),
             MaybeNeedInResume = no
         )
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 goal_info_get_maybe_need_in_par_conj(GoalInfo, MaybeNeedInParConj) :-
     CodeGenInfo = goal_info_get_code_gen_info(GoalInfo),
-    ( MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need ->
+    ( if MaybeNeed = CodeGenInfo ^ llds_code_gen ^ maybe_need then
         (
             MaybeNeed = need_par_conj(NeedInParConj),
             MaybeNeedInParConj = yes(NeedInParConj)
@@ -531,8 +532,8 @@ goal_info_get_maybe_need_in_par_conj(GoalInfo, MaybeNeedInParConj) :-
             ),
             MaybeNeedInParConj = no
         )
-    ;
-        unexpected($module, $pred, "no code_gen_info")
+    else
+        unexpected($pred, "no code_gen_info")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -684,7 +685,7 @@ goal_info_resume_vars_and_loc(Resume, Vars, Locs) :-
         Resume = resume_point(Vars, Locs)
     ;
         Resume = no_resume_point,
-        unexpected($module, $pred, "no resume point")
+        unexpected($pred, "no resume point")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -801,7 +802,7 @@ stack_slot_num(StackSlot) = N :-
         Width = single_width
     ;
         Width = double_width,
-        unexpected($module, $pred, "double_width")
+        unexpected($pred, "double_width")
     ).
 
 stack_slot_to_abs_locn(StackSlot) = AbsLocn :-
