@@ -18,6 +18,7 @@
 :- interface.
 
 :- import_module hlds.hlds_module.
+
 :- import_module io.
 
     % Write out size statistics about each procedure in the module.
@@ -35,10 +36,13 @@
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_error_util.
-:- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
 
+:- import_module assoc_list.
 :- import_module int.
 :- import_module list.
 :- import_module map.
@@ -65,14 +69,14 @@ write_proc_stats_for_module(OutStream, Msg, ModuleInfo, !IO) :-
 
 write_proc_stats_for_pred(OutStream, Msg, ModuleInfo, PredId - PredInfo,
         !IO) :-
-    (
+    ( if
         ( pred_info_is_imported(PredInfo)
-        ; is_unify_or_compare_pred(PredInfo)
+        ; is_unify_index_or_compare_pred(PredInfo)
         )
-    ->
+    then
         true
-    ;
-        pred_info_get_procedures(PredInfo, ProcTable),
+    else
+        pred_info_get_proc_table(PredInfo, ProcTable),
         map.to_assoc_list(ProcTable, Procs),
         list.foldl(
             write_proc_stats_for_proc(OutStream, Msg, ModuleInfo, PredId),
@@ -143,10 +147,11 @@ accumulate_proc_stats_in_goal(Goal, !UsedVars, !Stats) :-
                 !Stats ^ ps_unify_complicateds :=
                     !.Stats ^ ps_unify_complicateds + 1
             ;
-                RHS = rhs_lambda_goal(_, _, _, _, NonLocals, QuantVars, _, _,
-                    LambdaGoal),
+                RHS = rhs_lambda_goal(_, _, _, _, NonLocals, ArgVarsModes,
+                    _, LambdaGoal),
+                assoc_list.keys(ArgVarsModes, ArgVars),
                 set_tree234.insert_list(NonLocals, !UsedVars),
-                set_tree234.insert_list(QuantVars, !UsedVars),
+                set_tree234.insert_list(ArgVars, !UsedVars),
                 !Stats ^ ps_unify_complicateds :=
                     !.Stats ^ ps_unify_complicateds + 1,
                 accumulate_proc_stats_in_goal(LambdaGoal, !UsedVars, !Stats)
@@ -433,10 +438,10 @@ do_write_proc_stats(OutStream, Msg, Name, PredId, ProcId,
 
 output_proc_stat_component(OutStream, _Msg, _Name, _PredId, _ProcId,
         ComponentName, ComponentCount, !IO) :-
-    ( ComponentCount > 0 ->
+    ( if ComponentCount > 0 then
         io.format(OutStream, "GOAL %s: %d\n",
             [s(ComponentName), i(ComponentCount)], !IO)
-    ;
+    else
         true
     ).
 
