@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2002-2012 The University of Melbourne.
+% Copyright (C) 2015-2021 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -11,9 +12,9 @@
 %
 % This module contains a predicate to build up interval information for a
 % procedure; in particular the start and end points of intervals and the set
-% of variables needed in that interval.  It also contains a procedure to
+% of variables needed in that interval. It also contains a procedure to
 % insert deconstruction unifications into a goal, given a map of insertions
-% to make after particular anchors.  More detailed information is in
+% to make after particular anchors. More detailed information is in
 % stack_opt.m, from where this code was extracted.
 %
 % A description of intervals is in the paper "Using the heap to eliminate
@@ -23,8 +24,8 @@
 %    left-right pair of anchors, satisfying the property that if forward
 %    execution starts at the left anchor and continues without encountering
 %    failure (which would initiate backtracking, i.e. backward execution),
-%    the next anchor it reaches is the right anchor of the pair.  We
-%    consider a call to be part of the atomic goals of the interval only if
+%    the next anchor it reaches is the right anchor of the pair. We consider
+%    a call to be part of the atomic goals of the interval only if
 %    the call site is the right anchor of the interval, not the left anchor.
 %
 %-----------------------------------------------------------------------------%
@@ -35,6 +36,7 @@
 :- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
+:- import_module hlds.vartypes.
 :- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 :- import_module parse_tree.
@@ -175,9 +177,10 @@
 :- pred apply_headvar_correction(set_of_progvar::in, rename_map::in,
     hlds_goal::in, hlds_goal::out) is det.
 
-:- pred dump_interval_info(interval_info::in, io::di, io::uo) is det.
+:- pred dump_interval_info(io.text_output_stream::in, interval_info::in,
+    io::di, io::uo) is det.
 
-:- pred write_int_list(list(int)::in, io::di, io::uo) is det.
+:- func int_list_to_string(list(int)) = string.
 
 :- func interval_id_to_int(interval_id) = int.
 
@@ -186,19 +189,21 @@
 
 :- implementation.
 
-:- import_module check_hlds.        % needed for type_util, mode_util
-:- import_module check_hlds.inst_match.
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.inst_test.
+:- import_module check_hlds.mode_test.
 :- import_module hlds.arg_info.
 :- import_module hlds.code_model.
 :- import_module hlds.hlds_llds.
 :- import_module hlds.instmap.
 :- import_module ll_backend.
 :- import_module ll_backend.call_gen.
+:- import_module parse_tree.prog_rename.
 
 :- import_module assoc_list.
 :- import_module pair.
 :- import_module require.
+:- import_module string.
 :- import_module term.
 :- import_module varset.
 
@@ -274,12 +279,14 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             CondOpenIntervals, !IntervalInfo)
     ;
         GoalExpr = scope(Reason, SubGoal),
-        ( Reason = from_ground_term(TermVar, from_ground_term_construct) ->
+        ( if
+            Reason = from_ground_term(TermVar, from_ground_term_construct)
+        then
             % We treat this scope as a construction unification that unifies
             % TermVar with a single big variable-free term, since this is what
             % the generated code will do.
             require_access([TermVar], !IntervalInfo)
-        ;
+        else
             % XXX We could treat from_ground_term_deconstruct scopes specially
             % as well.
             build_interval_info_in_goal(SubGoal, !IntervalInfo, !Acc)
@@ -299,9 +306,9 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             _OutputArgsR, _OutputArgsF),
         list.append(InputArgsR, InputArgsF, InputArgs),
 
-        % Casts are generated inline.
         (
             GenericCall = cast(_),
+            % Casts are generated inline.
             require_in_regs(InputArgs, !IntervalInfo),
             require_access(InputArgs, !IntervalInfo)
         ;
@@ -333,9 +340,7 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             require_in_regs(Inputs, !IntervalInfo),
             require_access(Inputs, !IntervalInfo)
         ;
-            ( Builtin = out_of_line_builtin
-            ; Builtin = not_builtin
-            ),
+            Builtin = not_builtin,
             goal_info_get_maybe_need_across_call(GoalInfo,
                 MaybeNeedAcrossCall),
             build_interval_info_at_call(Inputs, MaybeNeedAcrossCall, GoalInfo,
@@ -355,14 +360,14 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             ModuleInfo, ArgVars, InputArgVarSet, _, _),
         set.to_sorted_list(InputArgVarSet, InputArgVars),
         list.append(InputArgVars, ExtraVars, InputVars),
-        (
+        ( if
             goal_info_maybe_get_maybe_need_across_call(GoalInfo,
                 MaybeNeedAcrossCall),
             MaybeNeedAcrossCall = yes(_)
-        ->
+        then
             build_interval_info_at_call(InputVars, MaybeNeedAcrossCall,
                 GoalInfo, !IntervalInfo, !Acc)
-        ;
+        else
             require_in_regs(InputVars, !IntervalInfo),
             require_access(InputVars, !IntervalInfo)
         )
@@ -373,13 +378,13 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
                 HowToConstruct, _, _),
             (
                 HowToConstruct = reuse_cell(_),
-                unexpected($module, $pred, "reuse")
+                unexpected($pred, "reuse")
             ;
                 % XXX Temporary for the time being.
                 HowToConstruct = construct_in_region(_),
-                unexpected($module, $pred, "construct in region")
+                unexpected($pred, "construct in region")
             ;
-                ( HowToConstruct = construct_statically
+                ( HowToConstruct = construct_statically(_)
                 ; HowToConstruct = construct_dynamically
                 )
             ),
@@ -394,10 +399,10 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
                 _, _),
             IntParams = !.IntervalInfo ^ ii_interval_params,
             ModuleInfo = IntParams ^ ip_module_info,
-            ( shared_left_to_right_deconstruct(ModuleInfo, ArgModes) ->
+            ( if shared_left_to_right_deconstruct(ModuleInfo, ArgModes) then
                 Goal = hlds_goal(GoalExpr, GoalInfo),
                 use_cell(CellVar, ArgVars, ConsId, Goal, !IntervalInfo, !Acc)
-            ;
+            else
                 true
             ),
             require_in_regs([CellVar], !IntervalInfo),
@@ -412,24 +417,25 @@ build_interval_info_in_goal(hlds_goal(GoalExpr, GoalInfo), !IntervalInfo,
             require_access([Var1, Var2], !IntervalInfo)
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected($module, $pred, "complicated_unify")
+            unexpected($pred, "complicated_unify")
         )
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
-:- pred shared_left_to_right_deconstruct(module_info::in, list(uni_mode)::in)
-    is semidet.
+:- pred shared_left_to_right_deconstruct(module_info::in,
+    list(unify_mode)::in) is semidet.
 
 shared_left_to_right_deconstruct(_, []).
 shared_left_to_right_deconstruct(ModuleInfo, [ArgMode | ArgsModes]) :-
-    ArgMode = ((InitCell - InitArg) -> (FinalCell - FinalArg)),
-    mode_is_fully_input(ModuleInfo, InitCell -> FinalCell),
-    mode_is_output(ModuleInfo, InitArg -> FinalArg),
-    inst_is_not_partly_unique(ModuleInfo, FinalCell),
-    inst_is_not_partly_unique(ModuleInfo, FinalArg),
+    ArgMode = unify_modes_li_lf_ri_rf(CellInitInst, CellFinalInst,
+        ArgInitInst, ArgFinalInst),
+    init_inst_is_fully_input(ModuleInfo, CellInitInst),
+    init_final_insts_is_output(ModuleInfo, ArgInitInst, ArgFinalInst),
+    inst_is_not_partly_unique(ModuleInfo, CellFinalInst),
+    inst_is_not_partly_unique(ModuleInfo, ArgFinalInst),
     shared_left_to_right_deconstruct(ModuleInfo, ArgsModes).
 
 %-----------------------------------------------------------------------------%
@@ -455,14 +461,14 @@ build_interval_info_at_call(Inputs, MaybeNeedAcrossCall, GoalInfo,
         record_interval_end(BeforeCallId, CallAnchor, !IntervalInfo),
         InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
         IntParams = !.IntervalInfo ^ ii_interval_params,
-        (
+        ( if
             ( instmap_delta_is_reachable(InstMapDelta)
             ; IntParams ^ ip_at_most_zero_calls = no
             )
-        ->
+        then
             record_interval_succ(BeforeCallId, AfterCallId, !IntervalInfo),
             VarsOnStack = VarsOnStack0
-        ;
+        else
             % If the call cannot succeed, then execution cannot
             % get from BeforeCallId to AfterCallId.
             record_interval_no_succ(BeforeCallId, !IntervalInfo),
@@ -485,7 +491,7 @@ build_interval_info_at_call(Inputs, MaybeNeedAcrossCall, GoalInfo,
         require_access(Inputs, !IntervalInfo)
     ;
         MaybeNeedAcrossCall = no,
-        unexpected($module, $pred, "no need across call")
+        unexpected($pred, "no need across call")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -550,25 +556,25 @@ reached_branch_end(GoalInfo, MaybeResumeGoal, Construct,
         MaybeResumeVars, !IntervalInfo, !Acc) :-
     GoalId = goal_info_get_goal_id(GoalInfo),
     record_branch_end_info(GoalId, !IntervalInfo),
-    (
+    ( if
         MaybeResumeGoal = yes(hlds_goal(_ResumeGoalExpr, ResumeGoalInfo)),
         goal_info_maybe_get_resume_point(ResumeGoalInfo, ResumePoint),
         ResumePoint = resume_point(ResumeVars, ResumeLocs),
         ResumeLocs \= resume_locs_orig_only
-    ->
+    then
         HasResumeSave = has_resume_save,
         MaybeResumeVars = yes(ResumeVars)
-    ;
+    else
         HasResumeSave = has_no_resume_save,
         MaybeResumeVars = no
     ),
     record_branch_resume(GoalId, HasResumeSave, !IntervalInfo),
-    ( goal_info_maybe_get_store_map(GoalInfo, StoreMap) ->
+    ( if goal_info_maybe_get_store_map(GoalInfo, StoreMap) then
         map.sorted_keys(StoreMap, StoreMapVarList),
         StoreMapVars = set_of_var.sorted_list_to_set(StoreMapVarList),
         require_flushed(StoreMapVars, !IntervalInfo)
-    ;
-        unexpected($module, $pred, "no store map")
+    else
+        unexpected($pred, "no store map")
     ),
     EndAnchor = anchor_branch_end(Construct, GoalId),
     StartAnchor = anchor_branch_start(Construct, GoalId),
@@ -678,7 +684,7 @@ assign_open_intervals_to_anchor(Anchor, !IntervalInfo) :-
     CurOpenIntervals = !.IntervalInfo ^ ii_open_intervals,
     set.fold(gather_interval_vars(IntervalVarMap), CurOpenIntervals,
         set_of_var.init, CurOpenIntervalVars),
-    ( map.search(AnchorFollowMap0, Anchor, AnchorFollowInfo0) ->
+    ( if map.search(AnchorFollowMap0, Anchor, AnchorFollowInfo0) then
         AnchorFollowInfo0 =
             anchor_follow_info(OpenIntervalVars0, OpenIntervals0),
         OpenIntervalVars =
@@ -688,7 +694,7 @@ assign_open_intervals_to_anchor(Anchor, !IntervalInfo) :-
             anchor_follow_info(OpenIntervalVars, OpenIntervals),
         map.det_update(Anchor, AnchorFollowInfo,
             AnchorFollowMap0, AnchorFollowMap)
-    ;
+    else
         AnchorFollowInfo =
             anchor_follow_info(CurOpenIntervalVars, CurOpenIntervals),
         map.det_insert(Anchor, AnchorFollowInfo,
@@ -770,9 +776,9 @@ record_interval_start(Id, Start, !IntervalInfo) :-
 
 record_interval_succ(Id, Succ, !IntervalInfo) :-
     SuccMap0 = !.IntervalInfo ^ ii_interval_succ,
-    ( map.search(SuccMap0, Id, Succ0) ->
+    ( if map.search(SuccMap0, Id, Succ0) then
         map.det_update(Id, [Succ | Succ0], SuccMap0, SuccMap)
-    ;
+    else
         map.det_insert(Id, [Succ], SuccMap0, SuccMap)
     ),
     !IntervalInfo ^ ii_interval_succ := SuccMap.
@@ -782,19 +788,19 @@ record_interval_succ(Id, Succ, !IntervalInfo) :-
 
 record_interval_no_succ(Id, !IntervalInfo) :-
     SuccMap0 = !.IntervalInfo ^ ii_interval_succ,
-    ( map.search(SuccMap0, Id, _Succ0) ->
-        unexpected($module, $pred, "already in succ map")
-    ;
+    ( if map.search(SuccMap0, Id, _Succ0) then
+        unexpected($pred, "already in succ map")
+    else
         map.det_insert(Id, [], SuccMap0, SuccMap)
     ),
     !IntervalInfo ^ ii_interval_succ := SuccMap.
 
 record_interval_vars(Id, NewVars, !IntervalInfo) :-
     VarsMap0 = !.IntervalInfo ^ ii_interval_vars,
-    ( map.search(VarsMap0, Id, Vars0) ->
+    ( if map.search(VarsMap0, Id, Vars0) then
         set_of_var.insert_list(NewVars, Vars0, Vars),
         map.det_update(Id, Vars, VarsMap0, VarsMap)
-    ;
+    else
         Vars = set_of_var.list_to_set(NewVars),
         map.det_insert(Id, Vars, VarsMap0, VarsMap)
     ),
@@ -811,10 +817,10 @@ delete_interval_vars(Id, ToDeleteVars, DeletedVars, !IntervalInfo) :-
     % The deletions are recorded only for debugging. The algorithm itself
     % does not need this information to be recorded.
     DeleteMap0 = !.IntervalInfo ^ ii_interval_delvars,
-    ( map.search(DeleteMap0, Id, Deletions0) ->
+    ( if map.search(DeleteMap0, Id, Deletions0) then
         Deletions = [DeletedVars | Deletions0],
         map.det_update(Id, Deletions, DeleteMap0, DeleteMap)
-    ;
+    else
         Deletions = [DeletedVars],
         map.det_insert(Id, Deletions, DeleteMap0, DeleteMap)
     ),
@@ -961,10 +967,12 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
             rename_var(need_not_rename, !.VarRename, Var0, Var),
             Reason = from_ground_term(Var, Kind)
         ;
-            ( Reason0 = promise_purity(_)
+            ( Reason0 = disable_warnings(_, _)
+            ; Reason0 = promise_purity(_)
             ; Reason0 = promise_solutions(_, _)
             ; Reason0 = require_detism(_)
             ; Reason0 = require_complete_switch(_)
+            ; Reason0 = require_switch_arms_detism(_, _)
             ; Reason0 = commit(_)
             ; Reason0 = barrier(_)
             ; Reason0 = trace_goal(_, _, _, _, _)
@@ -972,10 +980,10 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
             ),
             Reason = Reason0
         ),
-        ( Reason = from_ground_term(_, from_ground_term_construct) ->
+        ( if Reason = from_ground_term(_, from_ground_term_construct) then
             % There won't be any decisions to record.
             Goal = Goal0
-        ;
+        else
             % XXX We could treat from_ground_term_deconstruct scopes specially
             % as well.
             record_decisions_in_goal(SubGoal0, SubGoal, !VarInfo, !VarRename,
@@ -1004,9 +1012,7 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
             Builtin = inline_builtin,
             MustHaveMap = no
         ;
-            ( Builtin = out_of_line_builtin
-            ; Builtin = not_builtin
-            ),
+            Builtin = not_builtin,
             MustHaveMap = yes
         ),
         record_decisions_at_call_site(Goal0, Goal, !VarInfo, !VarRename,
@@ -1020,7 +1026,7 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
         rename_some_vars_in_goal(!.VarRename, Goal0, Goal)
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -1029,9 +1035,9 @@ record_decisions_in_goal(Goal0, Goal, !VarInfo, !VarRename, InsertMap,
     is det.
 
 lookup_inserts(InsertMap, Anchor, Inserts) :-
-    ( map.search(InsertMap, Anchor, InsertsPrime) ->
+    ( if map.search(InsertMap, Anchor, InsertsPrime) then
         Inserts = InsertsPrime
-    ;
+    else
         Inserts = []
     ).
 
@@ -1063,10 +1069,10 @@ make_inserted_goals(!VarInfo, !VarRename, [Spec | Specs], MaybeFeature,
 make_inserted_goal(!VarInfo, !VarRename, Spec, MaybeFeature, Goal) :-
     Spec = insert_spec(Goal0, VarsToExtract),
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
-    (
+    ( if
         GoalExpr0 = unify(_, _, _, Unification0, _),
         Unification0 = deconstruct(_, _, ArgVars, _, _, _)
-    ->
+    then
         Unification1 = Unification0 ^ deconstruct_can_fail := cannot_fail,
         GoalExpr1 = GoalExpr0 ^ unify_kind := Unification1,
         goal_info_set_determinism(detism_det, GoalInfo0, GoalInfo1),
@@ -1086,8 +1092,8 @@ make_inserted_goal(!VarInfo, !VarRename, Spec, MaybeFeature, Goal) :-
         % We rename the original goal.
         rename_some_vars_in_goal(!.VarRename, Goal2, Goal3),
         rename_some_vars_in_goal(VoidRename, Goal3, Goal)
-    ;
-        unexpected($module, $pred, "not a deconstruct")
+    else
+        unexpected($pred, "not a deconstruct")
     ).
 
 make_inserted_goal(VarSet0, VarSet, VarTypes0, VarTypes, !RenameMap,
@@ -1120,9 +1126,9 @@ create_shadow_var(Arg, VarsToExtract, !VarSet, !VarTypes,
     varset.new_named_var(Name, Shadow, !VarSet),
     lookup_var_type(!.VarTypes, Arg, Type),
     add_var_type(Shadow, Type, !VarTypes),
-    ( set_of_var.member(VarsToExtract, Arg) ->
+    ( if set_of_var.member(VarsToExtract, Arg) then
         map.det_insert(Arg, Shadow, !VarRename)
-    ;
+    else
         map.det_insert(Arg, Shadow, !VoidRename)
     ).
 
@@ -1137,23 +1143,23 @@ record_decisions_at_call_site(Goal0, Goal, !VarInfo, !VarRename,
         MustHaveMap, InsertMap, MaybeFeature) :-
     Goal0 = hlds_goal(_, GoalInfo0),
     rename_some_vars_in_goal(!.VarRename, Goal0, Goal1),
-    (
+    ( if
         goal_info_maybe_get_maybe_need_across_call(GoalInfo0,
             MaybeNeedAcrossCall),
         MaybeNeedAcrossCall = yes(_NeedAcrossCall)
-    ->
+    then
         GoalId = goal_info_get_goal_id(GoalInfo0),
         Anchor = anchor_call_site(GoalId),
         lookup_inserts(InsertMap, Anchor, Inserts),
         insert_goals_after(Goal1, Goal, !VarInfo, !:VarRename, Inserts,
             MaybeFeature)
-    ;
+    else
         (
             MustHaveMap = no,
             Goal = Goal1
         ;
             MustHaveMap = yes,
-            unexpected($module, $pred, "no save map")
+            unexpected($pred, "no save map")
         )
     ).
 
@@ -1171,12 +1177,12 @@ record_decisions_in_conj([Goal0 | Goals0], Goals, !VarInfo, !VarRename,
         InsertMap, MaybeFeature),
     record_decisions_in_conj(Goals0, TailGoals, !VarInfo, !VarRename,
         ConjType, InsertMap, MaybeFeature),
-    (
+    ( if
         Goal = hlds_goal(conj(InnerConjType, SubGoals), _),
         ConjType = InnerConjType
-    ->
+    then
         Goals = SubGoals ++ TailGoals
-    ;
+    else
         Goals = [Goal | TailGoals]
     ).
 
@@ -1216,9 +1222,9 @@ record_decisions_in_cases([Case0 | Cases0], [Case | Cases],
 apply_headvar_correction(HeadVarSet, RenameMap, Goal0, Goal) :-
     HeadVars = set_of_var.to_sorted_list(HeadVarSet),
     build_headvar_subst(HeadVars, RenameMap, map.init, Subst),
-    ( map.is_empty(Subst) ->
+    ( if map.is_empty(Subst) then
         Goal = Goal0
-    ;
+    else
         rename_some_vars_in_goal(Subst, Goal0, Goal)
     ).
 
@@ -1227,10 +1233,10 @@ apply_headvar_correction(HeadVarSet, RenameMap, Goal0, Goal) :-
 
 build_headvar_subst([], _RenameMap, !Subst).
 build_headvar_subst([HeadVar | HeadVars], RenameMap, !Subst) :-
-    ( map.search(RenameMap, HeadVar, Replacement) ->
+    ( if map.search(RenameMap, HeadVar, Replacement) then
         map.det_insert(Replacement, HeadVar, !Subst),
         map.det_insert(HeadVar, Replacement, !Subst)
-    ;
+    else
         true
     ),
     build_headvar_subst(HeadVars, RenameMap, !Subst).
@@ -1250,91 +1256,88 @@ construct_anchors(Construct, Goal, StartAnchor, EndAnchor) :-
 
 % For debugging purposes.
 
-dump_interval_info(IntervalInfo, !IO) :-
+dump_interval_info(Stream, IntervalInfo, !IO) :-
     map.keys(IntervalInfo ^ ii_interval_start, StartIds),
     map.keys(IntervalInfo ^ ii_interval_end, EndIds),
     map.keys(IntervalInfo ^ ii_interval_vars, VarsIds),
     map.keys(IntervalInfo ^ ii_interval_succ, SuccIds),
     list.condense([StartIds, EndIds, VarsIds, SuccIds], IntervalIds0),
     list.sort_and_remove_dups(IntervalIds0, IntervalIds),
-    io.write_string("INTERVALS:\n", !IO),
-    list.foldl(dump_interval_info_id(IntervalInfo), IntervalIds, !IO),
+    io.write_string(Stream, "INTERVALS:\n", !IO),
+    list.foldl(dump_interval_info_id(Stream, IntervalInfo), IntervalIds, !IO),
 
     map.to_assoc_list(IntervalInfo ^ ii_anchor_follow_map, AnchorFollows),
-    io.write_string("\nANCHOR FOLLOW:\n", !IO),
-    list.foldl(dump_anchor_follow, AnchorFollows, !IO).
+    io.write_string(Stream, "\nANCHOR FOLLOW:\n", !IO),
+    list.foldl(dump_anchor_follow(Stream), AnchorFollows, !IO).
 
-:- pred dump_interval_info_id(interval_info::in, interval_id::in,
-    io::di, io::uo) is det.
+:- pred dump_interval_info_id(io.text_output_stream::in, interval_info::in,
+    interval_id::in, io::di, io::uo) is det.
 
-dump_interval_info_id(IntervalInfo, IntervalId, !IO) :-
-    io.write_string("\ninterval ", !IO),
-    io.write_int(interval_id_to_int(IntervalId), !IO),
-    io.write_string(": ", !IO),
-    ( map.search(IntervalInfo ^ ii_interval_succ, IntervalId, SuccIds) ->
+dump_interval_info_id(Stream, IntervalInfo, IntervalId, !IO) :-
+    io.format(Stream, "\ninterval %d:",
+        [i(interval_id_to_int(IntervalId))], !IO),
+    ( if map.search(IntervalInfo ^ ii_interval_succ, IntervalId, SuccIds) then
         SuccNums = list.map(interval_id_to_int, SuccIds),
-        io.write_string("succ [", !IO),
-        write_int_list(SuccNums, !IO),
-        io.write_string("]\n", !IO)
-    ;
-        io.write_string("no succ\n", !IO)
+        io.format(Stream, "succ [%s]\n",
+            [s(int_list_to_string(SuccNums))], !IO)
+    else
+        io.write_string(Stream, "no succ\n", !IO)
     ),
-    ( map.search(IntervalInfo ^ ii_interval_start, IntervalId, Start) ->
-        io.write_string("start ", !IO),
-        io.write(Start, !IO),
-        io.write_string("\n", !IO)
-    ;
-        io.write_string("no start\n", !IO)
+    ( if map.search(IntervalInfo ^ ii_interval_start, IntervalId, Start) then
+        io.write_string(Stream, "start ", !IO),
+        io.write_line(Stream, Start, !IO)
+    else
+        io.write_string(Stream, "no start\n", !IO)
     ),
-    ( map.search(IntervalInfo ^ ii_interval_end, IntervalId, End) ->
-        io.write_string("end ", !IO),
-        io.write(End, !IO),
-        io.write_string("\n", !IO)
-    ;
-        io.write_string("no end\n", !IO)
+    ( if map.search(IntervalInfo ^ ii_interval_end, IntervalId, End) then
+        io.write_string(Stream, "end ", !IO),
+        io.write_line(Stream, End, !IO)
+    else
+        io.write_string(Stream, "no end\n", !IO)
     ),
-    ( map.search(IntervalInfo ^ ii_interval_vars, IntervalId, Vars) ->
+    ( if map.search(IntervalInfo ^ ii_interval_vars, IntervalId, Vars) then
         list.map(term.var_to_int, set_of_var.to_sorted_list(Vars), VarNums),
-        io.write_string("vars [", !IO),
-        write_int_list(VarNums, !IO),
-        io.write_string("]\n", !IO)
-    ;
-        io.write_string("no vars\n", !IO)
+        io.format(Stream, "vars [%s]\n",
+            [s(int_list_to_string(VarNums))], !IO)
+    else
+        io.write_string(Stream, "no vars\n", !IO)
     ),
-    ( map.search(IntervalInfo ^ ii_interval_delvars, IntervalId, Deletions) ->
-        io.write_string("deletions", !IO),
-        list.foldl(dump_deletion, Deletions, !IO),
-        io.write_string("\n", !IO)
-    ;
+    ( if
+        map.search(IntervalInfo ^ ii_interval_delvars, IntervalId, Deletions)
+    then
+        io.write_string(Stream, "deletions", !IO),
+        list.foldl(dump_deletion(Stream), Deletions, !IO),
+        io.write_string(Stream, "\n", !IO)
+    else
         true
     ).
 
-:- pred dump_deletion(set_of_progvar::in, io::di, io::uo) is det.
-
-dump_deletion(Vars, !IO) :-
-    list.map(term.var_to_int, set_of_var.to_sorted_list(Vars), VarNums),
-    io.write_string(" [", !IO),
-    write_int_list(VarNums, !IO),
-    io.write_string("]", !IO).
-
-:- pred dump_anchor_follow(pair(anchor, anchor_follow_info)::in,
+:- pred dump_deletion(io.text_output_stream::in, set_of_progvar::in,
     io::di, io::uo) is det.
 
-dump_anchor_follow(Anchor - AnchorFollowInfo, !IO) :-
-    AnchorFollowInfo = anchor_follow_info(Vars, Intervals),
-    io.write_string("\n", !IO),
-    io.write(Anchor, !IO),
-    io.write_string(" =>\n", !IO),
+dump_deletion(Stream, Vars, !IO) :-
     list.map(term.var_to_int, set_of_var.to_sorted_list(Vars), VarNums),
-    io.write_string("vars [", !IO),
-    write_int_list(VarNums, !IO),
-    io.write_string("]\nintervals: ", !IO),
-    set.to_sorted_list(Intervals, IntervalList),
-    write_int_list(list.map(interval_id_to_int, IntervalList), !IO),
-    io.write_string("\n", !IO).
+    io.format(Stream, " [%s]", [s(int_list_to_string(VarNums))], !IO).
 
-write_int_list(List, !IO) :-
-    io.write_list(List, ", ", io.write_int, !IO).
+:- pred dump_anchor_follow(io.text_output_stream::in,
+    pair(anchor, anchor_follow_info)::in, io::di, io::uo) is det.
+
+dump_anchor_follow(Stream, Anchor - AnchorFollowInfo, !IO) :-
+    AnchorFollowInfo = anchor_follow_info(Vars, Intervals),
+    list.map(term.var_to_int, set_of_var.to_sorted_list(Vars), VarNums),
+    set.to_sorted_list(Intervals, IntervalList),
+    IntervalIntList = list.map(interval_id_to_int, IntervalList),
+    io.write_string(Stream, "\n", !IO),
+    io.write(Stream, Anchor, !IO),
+    io.write_string(Stream, " =>\n", !IO),
+    io.format(Stream, "vars [%s]\n",
+        [s(int_list_to_string(VarNums))], !IO),
+    io.format(Stream, "intervals: %s\n",
+        [s(int_list_to_string(IntervalIntList))], !IO).
+
+int_list_to_string(Ints) = IntsStr :-
+    IntStrs = list.map(string.int_to_string, Ints),
+    IntsStr = string.join_list(", ", IntStrs).
 
 interval_id_to_int(interval_id(Num)) = Num.
 
