@@ -10,11 +10,11 @@
 % Main authors: conway, zs.
 %
 % This module finds out what variables need to be saved across calls,
-% across goals that may fail, and in parallel conjunctions. It then does those
+% across goals that may fail, and in parallel conjunctions. It then does two
 % things with that information. First, it attaches that information to the
 % relevant goal as a LLDS-backend-specific annotation. Second, it invokes
 % the relevant type class method of the allocator-specific data structure
-% it is passed; the basic stack slot allocator and the optimizing stack slot
+% it is passed. The basic stack slot allocator and the optimizing stack slot
 % allocator pass different instances of this type class.
 %
 %-----------------------------------------------------------------------------%
@@ -22,11 +22,15 @@
 :- module ll_backend.live_vars.
 :- interface.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_llds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module hlds.vartypes.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
 
@@ -82,16 +86,14 @@
 
 :- import_module hlds.arg_info.
 :- import_module hlds.code_model.
-:- import_module hlds.hlds_goal.
-:- import_module hlds.hlds_llds.
+:- import_module hlds.goal_form.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
-:- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
 
 :- import_module assoc_list.
 :- import_module enum.
 :- import_module int.
-:- import_module map.
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
@@ -99,7 +101,7 @@
 %-----------------------------------------------------------------------------%
 
 build_dummy_type_array(ModuleInfo, VarTypes, DummyTypeArray, DummyVars) :-
-    vartypes_to_assoc_list(VarTypes, VarsTypes),
+    vartypes_to_sorted_assoc_list(VarTypes, VarsTypes),
     list.foldl(max_var_num, VarsTypes, 0, MaxVarNum),
     % We want to index the array with variable numbers, which will be from
     % 1 to MaxVarNum.
@@ -122,7 +124,7 @@ set_dummy_array_elements(_, [], !DummyTypeArray, !DummyVars).
 set_dummy_array_elements(ModuleInfo, [VarType | VarsTypes],
         !DummyTypeArray, !DummyVars) :-
     VarType = Var - Type,
-    IsDummyType = check_dummy_type(ModuleInfo, Type),
+    IsDummyType = is_type_a_dummy(ModuleInfo, Type),
     (
         IsDummyType = is_dummy_type,
         array.set(to_int(Var), IsDummyType, !DummyTypeArray),
@@ -162,7 +164,7 @@ build_live_sets_in_goal(Goal0, Goal, ResumeVars0,
     goal_info_get_post_deaths(GoalInfo0, PostDeaths),
     goal_info_get_post_births(GoalInfo0, PostBirths),
 
-    % note: we must be careful to apply deaths before births
+    % NOTE We must be careful to apply deaths before births.
     set_of_var.difference(!.Liveness, PreDeaths, !:Liveness),
     set_of_var.union(!.Liveness, PreBirths, !:Liveness),
 
@@ -184,10 +186,13 @@ build_live_sets_in_goal(Goal0, Goal, ResumeVars0,
         GoalInfo1 = GoalInfo0
     ;
         ResumePoint = resume_point(ResumePointVars, Locs),
-        ( resume_locs_include_stack(Locs, yes) ->
+        resume_locs_include_stack(Locs, InclStack),
+        (
+            InclStack = yes,
             set_of_var.union(ResumeVars0, ResumePointVars, ResumeVars1),
             ResumeOnStack = yes
         ;
+            InclStack = no,
             ResumeVars1 = ResumeVars0,
             ResumeOnStack = no
         ),
@@ -197,7 +202,7 @@ build_live_sets_in_goal(Goal0, Goal, ResumeVars0,
             GoalInfo0, GoalInfo1, !StackAlloc)
     ),
 
-    build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo1, GoalInfo,
+    build_live_sets_in_goal_expr(GoalExpr0, GoalExpr, GoalInfo1, GoalInfo,
         ResumeVars1, AllocData, !StackAlloc, !Liveness, !NondetLiveness,
         !ParStackVars),
 
@@ -215,8 +220,8 @@ build_live_sets_in_goal(Goal0, Goal, ResumeVars0,
 
 resume_locs_include_stack(resume_locs_orig_only, no).
 resume_locs_include_stack(resume_locs_stack_only, yes).
-resume_locs_include_stack(resume_locs_orig_and_stack, yes).
-resume_locs_include_stack(resume_locs_stack_and_orig, yes).
+resume_locs_include_stack(resume_locs_orig_then_stack, yes).
+resume_locs_include_stack(resume_locs_stack_then_orig, yes).
 
 %-----------------------------------------------------------------------------%
 
@@ -228,7 +233,7 @@ resume_locs_include_stack(resume_locs_stack_and_orig, yes).
     % interference graph, i.e. the set of sets of variables which need to be
     % on the stack at the same time.
     %
-:- pred build_live_sets_in_goal_2(hlds_goal_expr::in, hlds_goal_expr::out,
+:- pred build_live_sets_in_goal_expr(hlds_goal_expr::in, hlds_goal_expr::out,
     hlds_goal_info::in, hlds_goal_info::out,
     set_of_progvar::in, alloc_data::in, T::in, T::out,
     set_of_progvar::in, set_of_progvar::out,
@@ -236,7 +241,7 @@ resume_locs_include_stack(resume_locs_stack_and_orig, yes).
     parallel_stackvars::in, parallel_stackvars::out)
     is det <= stack_alloc_info(T).
 
-build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
+build_live_sets_in_goal_expr(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
         ResumeVars0, AllocData, !StackAlloc, !Liveness, !NondetLiveness,
         !ParStackVars) :-
     (
@@ -306,7 +311,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
                 % Note that we must check the disjunction's code model, not any
                 % disjuncts'; the disjunction as a whole can be model_non
                 % without any disjunct being model_non.
-                (
+                ( if
                     goal_info_get_code_model(GoalInfo0) = model_non,
                     some [Disjunct] (
                         list.member(Disjunct, Goals),
@@ -316,10 +321,10 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
                         DisjunctResumePoint = resume_point(_, Locs),
                         resume_locs_include_stack(Locs, yes)
                     )
-                ->
+                then
                     set_of_var.union(!.NondetLiveness, ResumeVars,
                         !:NondetLiveness)
-                ;
+                else
                     true
                 )
             ;
@@ -369,7 +374,9 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
         GoalInfo = GoalInfo0
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        ( Reason = from_ground_term(TermVar, from_ground_term_construct) ->
+        ( if
+            Reason = from_ground_term(TermVar, from_ground_term_construct)
+        then
             % We do not modify construct unifications or conjunctions,
             % so we do not modify these scopes, which contain only a
             % conjunction of construct unifications.
@@ -382,16 +389,17 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
 
             % XXX We could treat from_ground_term_deconstruct scopes specially
             % as well.
-        ; Reason = loop_control(LCVar, LCSVar, _) ->
+        else if
+            Reason = loop_control(LCVar, LCSVar, _)
+        then
             % We must handle loop control scopes specially, see the comment for
-            % parallel conjunctions above.  Like parallel conjunctions, we need
+            % parallel conjunctions above. Like parallel conjunctions, we need
             % stack slots for the NonLocals, but we also need non-overlapping
             % slots for LC and LCS.
-            %
 
             % XXX: When we use a frame on the child's stack rather than the
             % parent's we may be able to save fewer variables to the stack at
-            % this time.  This is an optimization for later, not doing it now
+            % this time. This is an optimization for later, not doing it now
             % will make it easier to generate the code in the spawned off
             % computation (since it can have the same layout as the parent).
 
@@ -401,12 +409,12 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
             % Include ResumeVars0 because this goal may be backtracked over,
             % (Except that loop control is only applied to deterministic
             % predicates and it would mean entering a recursive call more than
-            % once.  So I think that ResumeVars0 will always be empty.
+            % once. So I think that ResumeVars0 will always be empty.
             % The loop control variables must also be allocated stack slots.
-            % Inclusion of !.Liveness is a conservative approximation.  Values
+            % Inclusion of !.Liveness is a conservative approximation. Values
             % in !.Liveness don't need to have stack slots but if they already
             % have stack slots then those slots most be distinct from others by
-            % the spawn off scope.  So what we want here is the intersection of
+            % the spawn off scope. So what we want here is the intersection of
             % !.Liveness and AlreadyHasStackSlot.
             OuterParStackVars = !.ParStackVars,
             LCStackVars =
@@ -435,7 +443,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
                 !ParStackVars),
 
             GoalExpr = scope(Reason, SubGoal)
-        ;
+        else
             NondetLiveness0 = !.NondetLiveness,
             build_live_sets_in_goal(SubGoal0, SubGoal, ResumeVars0, AllocData,
                 !StackAlloc, !Liveness, !NondetLiveness, !ParStackVars),
@@ -489,15 +497,13 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
             Builtin = inline_builtin,
             GoalInfo = GoalInfo0
         ;
-            ( Builtin = out_of_line_builtin
-            ; Builtin = not_builtin
-            ),
+            Builtin = not_builtin,
             build_live_sets_in_call(set_to_bitset(OutVars),
                 GoalInfo0, GoalInfo, ResumeVars0, AllocData,
                 !StackAlloc, !.Liveness, !NondetLiveness, !ParStackVars)
         ),
         CalleePredProcId = AllocData ^ ad_pred_proc_id,
-        ( CalleePredProcId = proc(PredId, ProcId) ->
+        ( if CalleePredProcId = proc(PredId, ProcId) then
             % If a call is recursive and a loop control scope has been seen
             % then the recursive call is a barrier for loop control, we have to
             % ensure that spawned off computations use distinct stack slots
@@ -512,7 +518,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
                 MaybeNeedLC = no
             ),
             !:Liveness = set_of_var.difference(!.Liveness, DelayDeathSet)
-        ;
+        else
             true
         )
     ;
@@ -527,7 +533,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
             )
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected($module, $pred, "complicated_unify")
+            unexpected($pred, "complicated_unify")
         )
     ;
         GoalExpr0 = call_foreign_proc(Attributes, PredId, ProcId, Args,
@@ -541,7 +547,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
         arg_info.partition_proc_call_args(ProcInfo, VarTypes, ModuleInfo,
             ArgVars, _InVars, OutVars, _UnusedVars),
         CodeModel = goal_info_get_code_model(GoalInfo0),
-        (
+        ( if
             % We don't need to save any variables onto the stack before a
             % foreign_proc if we know that it can't succeed more than once
             % and that it is not going to call back Mercury code, because
@@ -549,9 +555,9 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
 
             CodeModel \= model_non,
             get_may_call_mercury(Attributes) = proc_will_not_call_mercury
-        ->
+        then
             GoalInfo = GoalInfo0
-        ;
+        else
             % The variables which need to be saved onto the stack before
             % the call are all the variables that are live after the call
             % (except for the output arguments produced by the call), plus
@@ -564,7 +570,7 @@ build_live_sets_in_goal_2(GoalExpr0, GoalExpr, GoalInfo0, GoalInfo,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -592,13 +598,13 @@ build_live_sets_in_call(OutVars, GoalInfo0, GoalInfo, ResumeVars0, AllocData,
         AllocData ^ ad_typeinfo_liveness, OutVars, ForwardVars0, ForwardVars),
 
     Detism = goal_info_get_determinism(GoalInfo0),
-    (
+    ( if
         Detism = detism_erroneous,
         AllocData ^ ad_opt_no_return_calls = yes
-    ->
+    then
         NeedAcrossCall = need_across_call(set_of_var.init, set_of_var.init,
             set_of_var.init)
-    ;
+    else
         NeedAcrossCall = need_across_call(ForwardVars, ResumeVars0,
             !.NondetLiveness)
     ),
@@ -620,7 +626,7 @@ build_live_sets_in_call(OutVars, GoalInfo0, GoalInfo, ResumeVars0, AllocData,
     ),
 
     % In a parallel conjunction all the stack slots we need must not be reused
-    % in other parallel conjuncts.  We keep track of which variables have been
+    % in other parallel conjuncts. We keep track of which variables have been
     % allocated stack slots in each conjunct.
     par_stack_vars_accumulate_stack_vars(ForwardVars, !ParStackVars).
 
@@ -637,15 +643,15 @@ build_live_sets_in_conj([], [], _, _, !StackAlloc, !Liveness, !NondetLiveness,
         !ParStackVars).
 build_live_sets_in_conj([Goal0 | Goals0], [Goal | Goals], ResumeVars0,
         AllocData, !StackAlloc, !Liveness, !NondetLiveness, !ParStackVars) :-
-    (
+    ( if
         Goal0 = hlds_goal(_, GoalInfo),
         InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
         instmap_delta_is_unreachable(InstMapDelta)
-    ->
+    then
         build_live_sets_in_goal(Goal0, Goal, ResumeVars0, AllocData,
             !StackAlloc, !Liveness, !NondetLiveness, !ParStackVars),
         Goals = [] % XXX was Goals = Goal0
-    ;
+    else
         build_live_sets_in_goal(Goal0, Goal, ResumeVars0, AllocData,
             !StackAlloc, !Liveness, !NondetLiveness, !ParStackVars),
         build_live_sets_in_conj(Goals0, Goals, ResumeVars0, AllocData,
@@ -701,12 +707,12 @@ build_live_sets_in_disj([Goal0 | Goals0], [Goal | Goals],
         % just keep them in this set of sets.
         set_of_var.union(NondetLiveness1, NondetLiveness2, NondetLiveness3),
         goal_info_get_resume_point(GoalInfo, Resume),
-        (
+        ( if
             Resume = resume_point(ResumePointVars, Locs),
             resume_locs_include_stack(Locs, yes)
-        ->
+        then
             set_of_var.union(NondetLiveness3, ResumePointVars, NondetLiveness)
-        ;
+        else
             NondetLiveness = NondetLiveness3
         )
     ;
@@ -826,7 +832,7 @@ record_par_conj(NeedInParConj, AllocData, !GoalInfo, !StackAlloc) :-
     ;       loop_control_scope(
                 % Variables nonlocal to the scope, these all need stack slots.
                 % This field may be unnecessary since it's used to remove items
-                % from sets when adding them to the set below.  And then a
+                % from sets when adding them to the set below. And then a
                 % union of it and the set below is calculated to set
                 % need_in_par_conj.
                 set_of_progvar,
@@ -842,7 +848,7 @@ record_par_conj(NeedInParConj, AllocData, !GoalInfo, !StackAlloc) :-
                 list(set_of_progvar),
 
                 % The set of variables whose death we must delay until after
-                % the recursive call.  they may still be using their slots on
+                % the recursive call. they may still be using their slots on
                 % our stack frame.
                 set_of_progvar,
 
@@ -879,7 +885,7 @@ par_stack_vars_end_parallel_conjunction(LiveSet, OuterParStackVars,
             OuterLocalStackVars, OuterAccStackVars0),
         % All the local variables which needed stack slots in the parallel
         % conjuncts (InnerStackVars) become part of the accumulating set of
-        % variables that have stack slots.  Variables which are not local to
+        % variables that have stack slots. Variables which are not local to
         % but are needed in the parallel conjunctions also become part of the
         % accumulating set.
         OuterAccStackVars = OuterAccStackVars0
@@ -892,7 +898,7 @@ par_stack_vars_end_parallel_conjunction(LiveSet, OuterParStackVars,
         OuterParStackVars = loop_control_scope(OuterNonLocals, StackVars0),
         % The loop control scope must ensure that any stackvars needed by a
         % parallel conjunction are distinct from those already needed by loop
-        % control.  The same is true in the case for
+        % control. The same is true in the case for
         % after_loop_control_scope/2 below.
         StackVars = StackVars0 `set_of_var.union` InnerStackVars
             `set_of_var.union`
@@ -922,8 +928,7 @@ par_stack_vars_start_loop_control(NonLocals, ParStackVars0,
         ( ParStackVars0 = parallel_conjunction(_, _, _)
         ; ParStackVars0 = loop_control_scope(_, _)
         ),
-        unexpected($module, $pred,
-            "Loop control scope found in other parallel context")
+        unexpected($pred, "Loop control scope found in other parallel context")
     ).
 
 :- pred par_stack_vars_end_loop_control(set_of_progvar::in,
@@ -948,8 +953,7 @@ par_stack_vars_end_loop_control(NewWouldDieSet, OldParStackVars, ParStackVars0,
         ( OldParStackVars = parallel_conjunction(_, _, _)
         ; OldParStackVars = loop_control_scope(_, _)
         ),
-        unexpected($module, $pred,
-            "Loop control scope found in other parallel context")
+        unexpected($pred, "Loop control scope found in other parallel context")
     ).
 
 
@@ -996,12 +1000,14 @@ par_stack_vars_get_nonlocals(after_loop_control_scope(_, _, _),
     parallel_stackvars::in, parallel_stackvars::out) is det.
 
 par_stack_vars_next_par_conjunct(!ParStackVars) :-
-    ( !.ParStackVars = parallel_conjunction(Nonlocals, PrevSets, CurSet) ->
+    ( if
+        !.ParStackVars = parallel_conjunction(Nonlocals, PrevSets, CurSet)
+    then
         !:ParStackVars =
             parallel_conjunction(Nonlocals, [CurSet | PrevSets],
                 set_of_var.init)
-    ;
-        unexpected($module, $pred, "expected parallel_conjunction/3")
+    else
+        unexpected($pred, "expected parallel_conjunction/3")
     ).
 
 :- pred par_stack_vars_recursive_call(maybe(need_for_loop_control)::out,
@@ -1017,7 +1023,7 @@ par_stack_vars_recursive_call(MaybeNeedLC, DelayDeathSet, !ParStackVars) :-
         DelayDeathSet = set_of_var.init
     ;
         !.ParStackVars = loop_control_scope(_, _),
-        unexpected($module, $pred, "recursive call in loop control scope")
+        unexpected($pred, "recursive call in loop control scope")
     ;
         !.ParStackVars = after_loop_control_scope(StackVarsList0,
             DelayDeathSet, StackVars),
