@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 1993-2012 The University of Melbourne.
+% Copyright (C) 2014-2018 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -18,19 +19,27 @@
 :- module ll_backend.llds.
 :- interface.
 
+:- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
 :- import_module backend_libs.rtti.
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
+:- import_module hlds.
 :- import_module hlds.code_model.
 :- import_module hlds.hlds_data.
 :- import_module hlds.hlds_llds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 :- import_module ll_backend.layout.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
 :- import_module mdbcomp.program_representation.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_foreign.
+:- import_module parse_tree.prog_data_pragma.
 :- import_module parse_tree.prog_foreign.
 
 :- import_module bool.
@@ -43,25 +52,6 @@
 :- import_module counter.
 :- import_module term.
 
-%-----------------------------------------------------------------------------%
-
-    % Foreign_interface_info holds information used when generating
-    % code that uses the foreign language interface.
-    %
-:- type foreign_interface_info
-    --->    foreign_interface_info(
-                module_name,
-
-                % Info about stuff imported from C:
-                foreign_decl_info,
-                foreign_import_module_info_list,
-                foreign_body_info,
-
-                % Info about stuff exported to C:
-                foreign_export_decls,
-                foreign_export_defns
-            ).
-
 %-----------------------------------------------------------------------------%
 
     % The type `c_file' is the actual LLDS.
@@ -69,9 +59,9 @@
 :- type c_file
     --->    c_file(
                 cfile_modulename            :: module_name,
-                cfile_foreign_decl          :: foreign_decl_info,
-                cfile_foreign_code          :: list(user_foreign_code),
-                cfile_foreign_export        :: list(foreign_export),
+                cfile_foreign_decl_codes    :: list(foreign_decl_code),
+                cfile_foreign_body_codes    :: list(foreign_body_code),
+                cfile_foreign_export_defns  :: list(foreign_export_defn),
                 cfile_vars                  :: list(tabling_info_struct),
                 cfile_scalar_common_data    :: list(scalar_common_data_array),
                 cfile_vector_common_data    :: list(vector_common_data_array),
@@ -94,8 +84,8 @@
                 cfile_proc_var_names        :: list(int),
                 cfile_proc_body_bytecodes   :: list(int),
                 cfile_ts_string_table       :: list(string),
-                cfile_table_io_decls        :: list(table_io_decl_data),
-                cfile_table_io_decl_map     :: map(pred_proc_id,
+                cfile_table_io_entries      :: list(table_io_entry_data),
+                cfile_table_io_entry_map    :: map(pred_proc_id,
                                                 layout_slot_name),
                 cfile_proc_event_layouts    :: list(layout_slot_name),
                 cfile_exec_traces           :: list(proc_layout_exec_trace),
@@ -116,7 +106,7 @@
     --->    tabling_info_struct(
                 % The id of the procedure whose table this structure is.
                 tis_proc_label              :: proc_label,
-                tis_eval_method             :: eval_method,
+                tis_eval_method             :: tabled_eval_method,
 
                 tis_num_inputs              :: int,
                 tis_num_outputs             :: int,
@@ -141,7 +131,7 @@
 :- pred build_typed_rvals(list(rval)::in, list(llds_type)::in,
     list(typed_rval)::out) is det.
 
-:- type common_cell_type 
+:- type common_cell_type
     --->    plain_type(list(llds_type))
             % The type is a structure with one field for each one
             % of the cell's arguments.
@@ -212,10 +202,11 @@
 
 :- type comp_gen_c_module
     --->    comp_gen_c_module(
+                % The name of this C module.
                 cgcm_name               :: string,
-                                        % The name of this C module.
+
+                % The code.
                 cgcm_procs              :: list(c_procedure)
-                                        % The code.
             ).
 
 :- type c_procedure
@@ -229,15 +220,15 @@
                 % The pred_proc_id of this code.
                 cproc_id                :: pred_proc_id,
 
+                % Proc_label of this procedure.
+                cproc_proc_label        :: proc_label,
+
                 % The code model of the procedure.
                 cproc_code_model        :: code_model,
 
                 % The code for this procedure.
                 cproc_code              :: list(instruction),
 
-                % Proc_label of this procedure.
-                cproc_proc_label        :: proc_label,
-
                 % Source for new label numbers.
                 cproc_label_nums        :: counter,
 
@@ -266,6 +257,10 @@
                 llds_comment    :: string
             ).
 
+:- type maybe_auto_comments
+    --->    no_auto_comments
+    ;       auto_comments.
+
 :- type nondet_tail_call
     --->    no_tail_call
             % At the point of the call, the procedure has more alternatives.
@@ -389,7 +384,7 @@
             % Restore maxfr from the saved copy in the given lval. Assumes the
             % lval was saved with save_maxfr.
 
-    ;       incr_hp(lval, maybe(tag), maybe(int), rval, maybe(alloc_site_id),
+    ;       incr_hp(lval, maybe(ptag), maybe(int), rval, maybe(alloc_site_id),
                 may_use_atomic_alloc, maybe(rval), llds_reuse)
             % incr_hp(Target, MaybeTag, MaybeOffset, SizeRval, MaybeAllocId,
             %   MayUseAtomicAlloc, MaybeRegionId, MaybeReuse)
@@ -660,7 +655,7 @@
             % conjunct see runtime/mercury_context.{c,h}.
             % The synchronisation term is specified by the given lval.
             % The label gives the address of the code following the parallel
-            % conjunction. 
+            % conjunction.
 
     ;       lc_create_loop_control(int, lval)
             % Create a loop control structure with the given number of slots,
@@ -693,13 +688,13 @@
             % the same as the two rval arguments in the original lc_spawn_off
             % instruction.
 
-:- inst instr_llcall
+:- inst instr_llcall for instr/0
     --->    llcall(ground, ground, ground, ground, ground, ground).
-:- inst instr_goto
+:- inst instr_goto for instr/0
     --->    goto(ground).
-:- inst instr_if_val
+:- inst instr_if_val for instr/0
     --->    if_val(ground, ground).
-:- inst instr_foreign_proc_code
+:- inst instr_foreign_proc_code for instr/0
     --->    foreign_proc_code(ground, ground, ground, ground, ground,
                 ground, ground, ground, ground, ground).
 
@@ -846,7 +841,7 @@
 :- type foreign_proc_type
     --->    foreign_proc_type(
                 string,         % The C type name.
-                list(foreign_type_assertion)
+                foreign_type_assertions
                                 % The assertions on the foreign_type
                                 % declarations that the C type name came from.
             ).
@@ -1042,7 +1037,7 @@
     % as the target of an assignment.
     %
 :- type lval
-    
+
     % Virtual machine registers.
 
     --->    reg(reg_type, int)
@@ -1136,13 +1131,14 @@
 
     % Values on the heap.
 
-    ;       field(maybe(tag), rval, rval)
-            % field(Tag, Address, FieldNum) selects a field of a compound term.
-            % Address is a tagged pointer to a cell on the heap; the offset
-            % into the cell is FieldNum words. If Tag is yes, the arg gives
-            % the value of the tag; if it is no, the tag bits will have to be
-            % masked off. The value of the tag should be given if it is known,
-            % since this will lead to faster code.
+    ;       field(maybe(ptag), rval, rval)
+            % field(MaybePtag, Address, FieldNum) selects one field of a
+            % compound term. Address is a tagged pointer to a cell on the heap;
+            % the offset into the cell is FieldNum words. If MaybePtag is yes,
+            % its argument gives the value of the primary tag to be subtracted
+            % from Address; if it is no, the primary tag bits will have to be
+            % masked off. The value of the primary tag should be given
+            % if it is known, since this will lead to faster code.
 
     % Values somewhere in memory.
 
@@ -1179,14 +1175,16 @@
             % should not be present in the LLDS at any stage after code
             % generation.
 
-    ;       mkword(tag, rval)
-            % Given a pointer and a tag, mkword returns a tagged pointer.
+    ;       mkword(ptag, rval)
+            % Given a pointer and a ptag, mkword returns a tagged pointer.
 
-    ;       mkword_hole(tag)
+    ;       mkword_hole(ptag)
             % Make a tagged pointer to an address which is not yet known.
 
     ;       const(rval_const)
 
+    ;       cast(llds_type, rval)
+
     ;       unop(unary_op, rval)
 
     ;       binop(binary_op, rval, rval)
@@ -1202,9 +1200,9 @@
     ;       framevar_ref(rval)
             % Stack slot number.
 
-    ;       heap_ref(rval, maybe(int), rval).
-            % The cell pointer, the tag to subtract (if unknown, all the tag
-            % bits must be masked off), and the field number.
+    ;       heap_ref(rval, maybe(ptag), rval).
+            % The cell pointer, the ptag to subtract (if unknown, all the
+            % primary tag bits must be masked off), and the field number.
 
 :- type c_global_var_ref
     --->    env_var_ref(string).
@@ -1212,12 +1210,27 @@
 :- type rval_const
     --->    llconst_true
     ;       llconst_false
+
     ;       llconst_int(int)
+    ;       llconst_uint(uint)
+
+    ;       llconst_int8(int8)
+    ;       llconst_uint8(uint8)
+
+    ;       llconst_int16(int16)
+    ;       llconst_uint16(uint16)
+
+    ;       llconst_int32(int32)
+    ;       llconst_uint32(uint32)
+
+    ;       llconst_int64(int64)
+    ;       llconst_uint64(uint64)
+
     ;       llconst_foreign(string, llds_type)
             % A constant in the target language.
             % It may be a #defined constant in C which is why
             % it is represented as string.
-            
+
     ;       llconst_float(float)
     ;       llconst_string(string)
     ;       llconst_multi_string(list(string))
@@ -1261,11 +1274,11 @@
             % by the layout_name.
 
     ;       layout_slot_id(layout_slot_id_kind, pred_proc_id).
-            % The slot reserved for the given pred_proc_id in in the array
+            % The slot reserved for the given pred_proc_id in the array
             % identified by the layout_slot_id_kind.
 
 :- type layout_slot_id_kind
-    --->    table_io_decl_id.
+    --->    table_io_entry_id.
 
     % There are two kinds of labels: entry labels and internal labels.
     % Entry labels are the entry points of procedures; internal labels are not.
@@ -1335,10 +1348,18 @@
             % visible regular register input arguments, and zero visible float
             % register input arguments.
 
-    % A tag (used in mkword, create and field expressions and in incr_hp
-    % instructions) is a small integer.
+    % Each function symbol of this type corresponds to one of the C types
+    % {int,uint}_least{8,16,32}.
     %
-:- type tag ==  int.
+    % Values of these types are intended for use in static data declarations,
+    % not for data that gets stored in registers, stack slots etc.
+:- type int_least_type
+    --->    int_least8
+    ;       uint_least8
+    ;       int_least16
+    ;       uint_least16
+    ;       int_least32
+    ;       uint_least32.
 
     % We categorize the data types used in the LLDS into a small number of
     % categories, for purposes such as choosing the right sort of register
@@ -1348,41 +1369,12 @@
     --->    lt_bool
             % A boolean value represented using the C type `MR_Integer'.
 
-    ;       lt_int_least8
-            % A signed value that fits that contains at least eight bits,
-            % represented using the C type MR_int_least8_t. Intended for use
-            % in static data declarations, not for data that gets stored in
-            % registers, stack slots etc.
+    ;       lt_int_least(int_least_type)
+            % As documented above.
 
-    ;       lt_uint_least8
-            % An unsigned version of int_least8, represented using the C type
-            % MR_uint_least8_t.
-
-    ;       lt_int_least16
-            % A signed value that fits that contains at least sixteen bits,
-            % represented using the C type MR_int_least16_t. Intended for use
-            % in static data declarations, not for data that gets stored in
-            % registers, stack slots etc.
-
-    ;       lt_uint_least16
-            % An unsigned version of int_least16, represented using the C type
-            % MR_uint_least16_t.
-
-    ;       lt_int_least32
-            % A signed value that fits that contains at least 32 bits,
-            % represented using the C type MR_int_least32_t. Intended for use
-            % in static data declarations, not for data that gets stored in
-            % registers, stack slots etc.
-
-    ;       lt_uint_least32
-            % An unsigned version of intleast_32, represented using the C type
-            % uint_least32_t.
-
-    ;       lt_integer
+    ;       lt_int(int_type)
             % A Mercury `int', represented in C as a value of type `MR_Integer'
             % (which is a signed integral type of the same size as a pointer).
-
-    ;       lt_unsigned
             % Something whose C type is `MR_Unsigned' (the unsigned equivalent
             % of `MR_Integer').
 
@@ -1420,12 +1412,15 @@
             % Fill two words of a cell with the given rval, which must be a
             % double precision float.
 
-    ;       cell_arg_skip
-            % Leave a single word of a cell unfilled.
+    ;       cell_arg_skip_one_word
+    ;       cell_arg_skip_two_words
+            % Leave one or two words of a cell unfilled.
 
-    ;       cell_arg_take_addr(prog_var, maybe(rval)).
-            % Take the address of a field. If the second argument is
-            % `yes(Rval)' then the field is set to Rval beforehand.
+    ;       cell_arg_take_addr_one_word(prog_var, maybe(rval))
+    ;       cell_arg_take_addr_two_words(prog_var, maybe({rval, rval})).
+            % Take the address of a one- or two-word field.
+            % If the second argument is `yes(Rval/Rvals)',
+            % then set the field to Rval/Rvals beforehand.
 
 :- type completeness
     --->    complete
@@ -1435,14 +1430,14 @@
 
 :- pred break_up_local_label(label::in, proc_label::out, int::out) is det.
 
-    % Given a non-var rval, figure out its type.
-    %
-:- pred rval_type(rval::in, llds_type::out) is det.
-
     % Given a non-var lval, figure out its type.
     %
 :- pred lval_type(lval::in, llds_type::out) is det.
 
+    % Given a non-var rval, figure out its type.
+    %
+:- pred rval_type(rval::in, llds_type::out) is det.
+
     % Given a constant, figure out its type.
     %
 :- pred const_type(rval_const::in, llds_type::out) is det.
@@ -1483,6 +1478,10 @@
     --->    use_float_registers
     ;       do_not_use_float_registers.
 
+:- type have_unboxed_int64s
+    --->    have_unboxed_int64s
+    ;       do_not_have_unboxed_int64s.
+
 :- type have_static_ground_cells
     --->    have_static_ground_cells
     ;       do_not_have_static_ground_cells.
@@ -1491,6 +1490,10 @@
     --->    have_static_ground_floats
     ;       do_not_have_static_ground_floats.
 
+:- type have_static_ground_int64s
+    --->    have_static_ground_int64s
+    ;       do_not_have_static_ground_int64s.
+
 :- type have_static_code_addresses
     --->    have_static_code_addresses
     ;       do_not_have_static_code_addresses.
@@ -1502,8 +1505,10 @@
                 unboxed_floats          :: have_unboxed_floats,
                 float_registers         :: use_float_registers,
                 det_stack_float_width   :: stack_slot_width,
+                unboxed_int64s          :: have_unboxed_int64s,
                 static_ground_cells     :: have_static_ground_cells,
                 static_ground_floats    :: have_static_ground_floats,
+                static_ground_int64s    :: have_static_ground_int64s,
                 static_code_addresses   :: have_static_code_addresses
             ).
 
@@ -1512,8 +1517,10 @@
 :- func get_unboxed_floats(exprn_opts) = have_unboxed_floats.
 :- func get_float_registers(exprn_opts) = use_float_registers.
 :- func get_det_stack_float_width(exprn_opts) = stack_slot_width.
+:- func get_unboxed_int64s(exprn_opts) = have_unboxed_int64s.
 :- func get_static_ground_cells(exprn_opts) = have_static_ground_cells.
 :- func get_static_ground_floats(exprn_opts) = have_static_ground_floats.
+:- func get_static_ground_int64s(exprn_opts) = have_static_ground_int64s.
 :- func get_static_code_addresses(exprn_opts) = have_static_code_addresses.
 
 %-----------------------------------------------------------------------------%
@@ -1536,9 +1543,9 @@ typed_rvals_project_types([typed_rval(_Rval, Type) | TypedRvals]) =
 
 build_typed_rvals([], [], []).
 build_typed_rvals([_|_], [], _) :-
-    unexpected($module, $pred, "length mismatch").
+    unexpected($pred, "length mismatch").
 build_typed_rvals([], [_|_], _) :-
-    unexpected($module, $pred, "length mismatch").
+    unexpected($pred, "length mismatch").
 build_typed_rvals([Rval | Rvals], [Type | Types], [TypedRval | TypedRvals]) :-
     TypedRval = typed_rval(Rval, Type),
     build_typed_rvals(Rvals, Types, TypedRvals).
@@ -1591,7 +1598,7 @@ abs_locn_to_lval_or_any_reg(abs_framevar(N)) =
     loa_lval(stack_slot_to_lval(nondet_slot(N))).
 
 abs_locn_to_lval(any_reg) = _ :-
-    unexpected($module, $pred, "any_reg").
+    unexpected($pred, "any_reg").
 abs_locn_to_lval(abs_reg(Type, N)) = reg(Type, N).
 abs_locn_to_lval(abs_stackvar(N, Width)) =
     stack_slot_to_lval(det_slot(N, Width)).
@@ -1624,7 +1631,7 @@ break_up_local_label(Label, ProcLabel, LabelNum) :-
         Label = internal_label(LabelNum, ProcLabel)
     ;
         Label = entry_label(_, _),
-        unexpected($module, $pred, "entry label")
+        unexpected($pred, "entry label")
     ).
 
 lval_type(reg(RegType, _), Type) :-
@@ -1648,14 +1655,14 @@ lval_type(succfr_slot(_), lt_data_ptr).
 lval_type(prevfr_slot(_), lt_data_ptr).
 lval_type(field(_, _, _), lt_word).
 lval_type(lvar(_), _) :-
-    unexpected($module, $pred, "lvar").
+    unexpected($pred, "lvar").
 lval_type(mem_ref(_), lt_word).
 lval_type(global_var_ref(_), lt_word).
 
 rval_type(lval(Lval), Type) :-
     lval_type(Lval, Type).
 rval_type(var(_), _) :-
-    unexpected($module, $pred, "var").
+    unexpected($pred, "var").
     %
     % Note that mkword and data_addr consts must be of type data_ptr,
     % not of type word, to ensure that static consts containing them
@@ -1671,6 +1678,7 @@ rval_type(mkword(_, _), lt_data_ptr).
 rval_type(mkword_hole(_), lt_data_ptr).
 rval_type(const(Const), Type) :-
     const_type(Const, Type).
+rval_type(cast(Type, _), Type).
 rval_type(unop(UnOp, _), Type) :-
     unop_return_type(UnOp, Type).
 rval_type(binop(BinOp, _, _), Type) :-
@@ -1679,7 +1687,16 @@ rval_type(mem_addr(_), lt_data_ptr).
 
 const_type(llconst_true, lt_bool).
 const_type(llconst_false, lt_bool).
-const_type(llconst_int(_), lt_integer).
+const_type(llconst_int(_), lt_int(int_type_int)).
+const_type(llconst_uint(_), lt_int(int_type_uint)).
+const_type(llconst_int8(_), lt_int(int_type_int8)).
+const_type(llconst_uint8(_), lt_int(int_type_uint8)).
+const_type(llconst_int16(_), lt_int(int_type_int16)).
+const_type(llconst_uint16(_), lt_int(int_type_uint16)).
+const_type(llconst_int32(_), lt_int(int_type_int32)).
+const_type(llconst_uint32(_), lt_int(int_type_uint32)).
+const_type(llconst_int64(_), lt_int(int_type_int64)).
+const_type(llconst_uint64(_), lt_int(int_type_uint64)).
 const_type(llconst_foreign(_, Type), Type).
 const_type(llconst_float(_), lt_float).
 const_type(llconst_string(_), lt_string).
@@ -1687,72 +1704,91 @@ const_type(llconst_multi_string(_), lt_string).
 const_type(llconst_code_addr(_), lt_code_ptr).
 const_type(llconst_data_addr(_, _), lt_data_ptr).
 
-unop_return_type(mktag, lt_word).
 unop_return_type(tag, lt_word).
-unop_return_type(unmktag, lt_word).
 unop_return_type(strip_tag, lt_word).
 unop_return_type(mkbody, lt_word).
 unop_return_type(unmkbody, lt_word).
-unop_return_type(bitwise_complement, lt_integer).
+unop_return_type(bitwise_complement(IntType), lt_int(IntType)).
 unop_return_type(logical_not, lt_bool).
-unop_return_type(hash_string, lt_integer).
-unop_return_type(hash_string2, lt_integer).
-unop_return_type(hash_string3, lt_integer).
+unop_return_type(hash_string, lt_int(int_type_int)).
+unop_return_type(hash_string2, lt_int(int_type_int)).
+unop_return_type(hash_string3, lt_int(int_type_int)).
+unop_return_type(hash_string4, lt_int(int_type_int)).
+unop_return_type(hash_string5, lt_int(int_type_int)).
+unop_return_type(hash_string6, lt_int(int_type_int)).
+unop_return_type(dword_float_get_word0,  lt_word).
+unop_return_type(dword_float_get_word1,  lt_word).
+unop_return_type(dword_int64_get_word0,  lt_word).
+unop_return_type(dword_int64_get_word1,  lt_word).
+unop_return_type(dword_uint64_get_word0, lt_word).
+unop_return_type(dword_uint64_get_word1, lt_word).
 
-unop_arg_type(mktag, lt_word).
 unop_arg_type(tag, lt_word).
-unop_arg_type(unmktag, lt_word).
 unop_arg_type(strip_tag, lt_word).
 unop_arg_type(mkbody, lt_word).
 unop_arg_type(unmkbody, lt_word).
-unop_arg_type(bitwise_complement, lt_integer).
+unop_arg_type(bitwise_complement(IntType), lt_int(IntType)).
 unop_arg_type(logical_not, lt_bool).
 unop_arg_type(hash_string, lt_string).
 unop_arg_type(hash_string2, lt_string).
 unop_arg_type(hash_string3, lt_string).
+unop_arg_type(hash_string4, lt_string).
+unop_arg_type(hash_string5, lt_string).
+unop_arg_type(hash_string6, lt_string).
+unop_arg_type(dword_float_get_word0,  lt_float).
+unop_arg_type(dword_float_get_word1,  lt_float).
+unop_arg_type(dword_int64_get_word0,  lt_int(int_type_int64)).
+unop_arg_type(dword_int64_get_word1,  lt_int(int_type_int64)).
+unop_arg_type(dword_uint64_get_word0, lt_int(int_type_uint64)).
+unop_arg_type(dword_uint64_get_word1, lt_int(int_type_uint64)).
 
-binop_return_type(int_add, lt_integer).
-binop_return_type(int_sub, lt_integer).
-binop_return_type(int_mul, lt_integer).
-binop_return_type(int_div, lt_integer).
-binop_return_type(int_mod, lt_integer).
-binop_return_type(unchecked_left_shift, lt_integer).
-binop_return_type(unchecked_right_shift, lt_integer).
-binop_return_type(bitwise_and, lt_integer).
-binop_return_type(bitwise_or, lt_integer).
-binop_return_type(bitwise_xor, lt_integer).
+binop_return_type(int_add(IntType), lt_int(IntType)).
+binop_return_type(int_sub(IntType), lt_int(IntType)).
+binop_return_type(int_mul(IntType), lt_int(IntType)).
+binop_return_type(int_div(IntType), lt_int(IntType)).
+binop_return_type(int_mod(IntType), lt_int(IntType)).
+binop_return_type(unchecked_left_shift(IntType, _), lt_int(IntType)).
+binop_return_type(unchecked_right_shift(IntType, _), lt_int(IntType)).
+binop_return_type(bitwise_and(IntType), lt_int(IntType)).
+binop_return_type(bitwise_or(IntType), lt_int(IntType)).
+binop_return_type(bitwise_xor(IntType), lt_int(IntType)).
 binop_return_type(logical_and, lt_bool).
 binop_return_type(logical_or, lt_bool).
-binop_return_type(eq, lt_bool).
-binop_return_type(ne, lt_bool).
+binop_return_type(eq(_), lt_bool).
+binop_return_type(ne(_), lt_bool).
 binop_return_type(array_index(_Type), lt_word).
+binop_return_type(string_unsafe_index_code_unit, lt_int(int_type_int)).
+binop_return_type(offset_str_eq(_), lt_bool).
 binop_return_type(str_eq, lt_bool).
 binop_return_type(str_ne, lt_bool).
 binop_return_type(str_lt, lt_bool).
 binop_return_type(str_gt, lt_bool).
 binop_return_type(str_le, lt_bool).
 binop_return_type(str_ge, lt_bool).
-binop_return_type(str_cmp, lt_integer).
-binop_return_type(int_lt, lt_bool).
-binop_return_type(int_gt, lt_bool).
-binop_return_type(int_le, lt_bool).
-binop_return_type(int_ge, lt_bool).
+binop_return_type(str_cmp, lt_int(int_type_int)).
+binop_return_type(int_lt(_), lt_bool).
+binop_return_type(int_gt(_), lt_bool).
+binop_return_type(int_le(_), lt_bool).
+binop_return_type(int_ge(_), lt_bool).
+binop_return_type(unsigned_lt, lt_bool).
 binop_return_type(unsigned_le, lt_bool).
-binop_return_type(float_plus, lt_float).
-binop_return_type(float_minus, lt_float).
-binop_return_type(float_times, lt_float).
-binop_return_type(float_divide, lt_float).
+binop_return_type(float_add, lt_float).
+binop_return_type(float_sub, lt_float).
+binop_return_type(float_mul, lt_float).
+binop_return_type(float_div, lt_float).
 binop_return_type(float_eq, lt_bool).
 binop_return_type(float_ne, lt_bool).
 binop_return_type(float_lt, lt_bool).
 binop_return_type(float_gt, lt_bool).
 binop_return_type(float_le, lt_bool).
 binop_return_type(float_ge, lt_bool).
-binop_return_type(float_word_bits, lt_word).
-binop_return_type(float_from_dword, lt_float).
+binop_return_type(float_from_dword,  lt_float).
+binop_return_type(int64_from_dword,  lt_int(int_type_int64)).
+binop_return_type(uint64_from_dword, lt_int(int_type_uint64)).
 binop_return_type(body, lt_word).
 binop_return_type(compound_eq, lt_bool).
 binop_return_type(compound_lt, lt_bool).
+binop_return_type(pointer_equal_conservative, lt_bool).
 
 register_type(reg_r, lt_word).
 register_type(reg_f, lt_float).
@@ -1767,11 +1803,13 @@ get_defining_module_name(special_proc_label(ModuleName, _, _, _, _, _))
 
 get_nonlocal_gotos(ExprnOpts) = ExprnOpts ^ non_local_gotos.
 get_asm_labels(ExprnOpts) = ExprnOpts ^ asm_labels.
-get_static_ground_cells(ExprnOpts) = ExprnOpts ^ static_ground_cells.
 get_unboxed_floats(ExprnOpts) = ExprnOpts ^ unboxed_floats.
 get_float_registers(ExprnOpts) = ExprnOpts ^ float_registers.
 get_det_stack_float_width(ExprnOpts) = ExprnOpts ^ det_stack_float_width.
+get_unboxed_int64s(ExprnOpts) = ExprnOpts ^ unboxed_int64s.
+get_static_ground_cells(ExprnOpts) = ExprnOpts ^ static_ground_cells.
 get_static_ground_floats(ExprnOpts) = ExprnOpts ^ static_ground_floats.
+get_static_ground_int64s(ExprnOpts) = ExprnOpts ^ static_ground_int64s.
 get_static_code_addresses(ExprnOpts) = ExprnOpts ^ static_code_addresses.
 
 %-----------------------------------------------------------------------------%
