@@ -5,20 +5,25 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-% 
+%
 % File: matching.m.
 % Authors: pjs, zs.
-% 
+%
 % Module `matching' - performs bipartite graph maximal matching computation
 % specialized for the stack slot optimization. The structure of the graph
 % on which the algorithm operates is documented in the paper "Using the heap
 % to eliminate stack accesses" by Zoltan Somogyi and Peter Stuckey.
-% 
+%
 %-----------------------------------------------------------------------------%
 
 :- module backend_libs.matching.
 :- interface.
 
+:- import_module libs.
+:- import_module libs.globals.
+:- import_module libs.optimization_options.
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.
 :- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
@@ -55,14 +60,15 @@
                 field_var_load_cost     :: int,
                 one_path_op_ratio       :: int,
                 one_path_node_ratio     :: int,
-                include_all_candidates  :: bool
+                include_all_candidates  :: maybe_opt_svcell_all_candidates
             ).
 
 :- type benefit_node.
 :- type cost_node.
 
-    % find_via_cell_vars(CellVar, CandidateFieldVars, CellVarFlushedLater,
-    %   BeforeFlush, AfterFlush, MatchingParams,
+    % find_via_cell_vars(Globals, ModuleName, MatchingParams,
+    %   CellVar, CandidateFieldVars, CellVarFlushedLater,
+    %   BeforeFlush, AfterFlush,
     %   RealizedBenefitNodes, RealizedCostNodes, ViaCellVars):
     %
     % CellVar gives a variable that corresponds to a memory cell, while
@@ -77,10 +83,13 @@
     % and RealizedCostNodes give the benefit and cost nodes realized
     % by this choice.
     %
-:- pred find_via_cell_vars(prog_var::in, set_of_progvar::in, bool::in,
-    set_of_progvar::in, list(set_of_progvar)::in, matching_params::in,
-    set(benefit_node)::out, set(cost_node)::out, set_of_progvar::out)
-    is det.
+    % The Globals and ModuleName arguments are needed only so that
+    % we can get the right debug output stream.
+    %
+:- pred find_via_cell_vars(globals::in, module_name::in, matching_params::in,
+    prog_var::in, set_of_progvar::in, bool::in, set_of_progvar::in,
+    list(set_of_progvar)::in,
+    set(benefit_node)::out, set(cost_node)::out, set_of_progvar::out) is det.
 
 %-----------------------------------------------------------------------------%
 
@@ -147,22 +156,19 @@
 
 %-----------------------------------------------------------------------------%
 
-% Uncomment if you want to dump performance information into the .err file.
-% :- pragma promise_pure(find_via_cell_vars/15).
-
-find_via_cell_vars(CellVar, CandidateFieldVars, CellVarFlushedLater,
-        BeforeFlush, AfterFlush, MatchingParams,
+find_via_cell_vars(Globals, ModuleName, MatchingParams, CellVar,
+        CandidateFieldVars, CellVarFlushedLater, BeforeFlush, AfterFlush,
         RealizedBenefitNodes, RealizedCostNodes, ViaCellVars) :-
     InclAllCand = MatchingParams ^ include_all_candidates,
     (
-        InclAllCand = no,
+        InclAllCand = do_not_opt_svcell_all_candidates,
         AllSegmentVars = set_of_var.union_list([BeforeFlush | AfterFlush]),
         set_of_var.intersect(CandidateFieldVars, AllSegmentVars,
             OccurringCandidateFieldVars),
         set_of_var.difference(CandidateFieldVars, OccurringCandidateFieldVars,
             NonOccurringCandidateFieldVars)
     ;
-        InclAllCand = yes,
+        InclAllCand = opt_svcell_all_candidates,
         OccurringCandidateFieldVars = CandidateFieldVars,
         NonOccurringCandidateFieldVars = set_of_var.init
     ),
@@ -206,29 +212,27 @@ find_via_cell_vars(CellVar, CandidateFieldVars, CellVarFlushedLater,
     list.length(RealizedCostOpList, RealizedCostOpCount),
     OpRatio = MatchingParams ^ one_path_op_ratio,
     NodeRatio = MatchingParams ^ one_path_node_ratio,
-    (
+    ( if
         RealizedBenefitOpCount * 100 >= RealizedCostOpCount * OpRatio,
         RealizedBenefitNodeCount * 100 >= RealizedCostNodeCount * NodeRatio
-    ->
-        ViaCellOccurringVars = ViaCellOccurringVars0
-        % Uncomment if you want to dump performance information into
-        % the .err file.
-        % Nullified = no
-    ;
-        ViaCellOccurringVars = set_of_var.init
-        % Uncomment if you want to dump performance information into
-        % the .err file.
-        % Nullified = yes
+    then
+        ViaCellOccurringVars = ViaCellOccurringVars0,
+        Nullified = no
+    else
+        ViaCellOccurringVars = set_of_var.init,
+        Nullified = yes
     ),
     ViaCellVars = set_of_var.union(ViaCellOccurringVars,
-        NonOccurringCandidateFieldVars).
-    % Uncomment if you want to dump performance information into
-    % the .err file.
-    % impure unsafe_perform_io(dump_results(CellVar, CandidateFieldVars,
-    %   OccurringCandidateFieldVarList, ViaCellOccurringVars0,
-    %   Nullified, BeforeFlush, NumberedAfterFlush,
-    %   RealizedBenefitNodeList, RealizedBenefitOpList,
-    %   RealizedCostNodeList, RealizedCostOpList)).
+        NonOccurringCandidateFieldVars),
+    % Enable if you want to dump performance information into the .err file.
+    trace [compile_time(flag("debug_matching")), io(!IO)] (
+        get_debug_output_stream(Globals, ModuleName, DebugStream, !IO),
+        dump_results(DebugStream, CellVar, CandidateFieldVars,
+            OccurringCandidateFieldVarList, ViaCellOccurringVars0,
+            Nullified, BeforeFlush, NumberedAfterFlush,
+            RealizedBenefitNodeList, RealizedBenefitOpList,
+            RealizedCostNodeList, RealizedCostOpList, !IO)
+    ).
 
 %-----------------------------------------------------------------------------%
 
@@ -242,7 +246,7 @@ find_via_cell_vars(CellVar, CandidateFieldVars, CellVarFlushedLater,
     set_of_progvar::out) is semidet.
 
 simplify_segment(CellVar, CandidateArgVars, SegmentVars0, SegmentVars) :-
-    \+ set_of_var.member(SegmentVars0, CellVar),
+    not set_of_var.member(SegmentVars0, CellVar),
     SegmentVars = set_of_var.intersect(SegmentVars0, CandidateArgVars).
 
 :- func number_segments(int, list(set_of_progvar)) =
@@ -272,9 +276,9 @@ find_costs_benefits(CellVar, BeforeFlush, AfterFlush, CellVarFlushedLater,
         CostOps = [cell_var_store | CostOps0]
     ),
     BenefitOps0 = [field_var_store(FieldVar)],
-    ( set_of_var.member(BeforeFlush, CellVar) ->
+    ( if set_of_var.member(BeforeFlush, CellVar) then
         BenefitOps = BenefitOps0
-    ;
+    else
         BenefitOps = [field_var_load(FieldVar) | BenefitOps0]
     ),
 
@@ -301,9 +305,9 @@ find_costs_benefits(CellVar, BeforeFlush, AfterFlush, CellVarFlushedLater,
 find_cell_var_loads_for_field([], _, !CostOps).
 find_cell_var_loads_for_field([SegmentNum - SegmentVars | AfterFlush],
         FieldVar, !CostOps) :-
-    ( set_of_var.member(SegmentVars, FieldVar) ->
+    ( if set_of_var.member(SegmentVars, FieldVar) then
         !:CostOps = [cell_var_load(SegmentNum) | !.CostOps]
-    ;
+    else
         true
     ),
     find_cell_var_loads_for_field(AfterFlush, FieldVar, !CostOps).
@@ -320,9 +324,9 @@ replicate_cost_op(StoreCost, _LoadCost, cell_var_store) =
 :- func make_cost_op_copies(int, cost_operation) = list(cost_node).
 
 make_cost_op_copies(Cur, Op) =
-    ( Cur > 0 ->
+    ( if Cur > 0 then
         [cost_node(Op, Cur) | make_cost_op_copies(Cur - 1, Op)]
-    ;
+    else
         []
     ).
 
@@ -336,9 +340,9 @@ replicate_benefit_op(StoreCost, _LoadCost, field_var_store(FieldVar)) =
 :- func make_benefit_op_copies(int, benefit_operation) = list(benefit_node).
 
 make_benefit_op_copies(Cur, Op) =
-    ( Cur > 0 ->
+    ( if Cur > 0 then
         [benefit_node(Op, Cur) | make_benefit_op_copies(Cur - 1, Op)]
-    ;
+    else
         []
     ).
 
@@ -389,10 +393,10 @@ create_graph_links(field_costs_benefits(_FieldVar, Costs, Benefits),
     map(cost_node, set(benefit_node))::out) is det.
 
 add_cost_benefit_links(Benefits, Cost, !CostToBenefitsMap) :-
-    ( map.search(!.CostToBenefitsMap, Cost, CostBenefits0) ->
+    ( if map.search(!.CostToBenefitsMap, Cost, CostBenefits0) then
         set.union(CostBenefits0, Benefits, CostBenefits),
         map.det_update(Cost, CostBenefits, !CostToBenefitsMap)
-    ;
+    else
         map.det_insert(Cost, Benefits, !CostToBenefitsMap)
     ).
 
@@ -401,10 +405,10 @@ add_cost_benefit_links(Benefits, Cost, !CostToBenefitsMap) :-
     map(benefit_node, set(cost_node))::out) is det.
 
 add_benefit_cost_links(Costs, Benefit, !BenefitToCostsMap) :-
-    ( map.search(!.BenefitToCostsMap, Benefit, BenefitCosts0) ->
+    ( if map.search(!.BenefitToCostsMap, Benefit, BenefitCosts0) then
         set.union(BenefitCosts0, Costs, BenefitCosts),
         map.det_update(Benefit, BenefitCosts, !BenefitToCostsMap)
-    ;
+    else
         map.det_insert(Benefit, Costs, !BenefitToCostsMap)
     ).
 
@@ -422,10 +426,12 @@ maximal_matching(BenefitNodes, Graph) = Matching :-
     matching::in, matching::out) is det.
 
 maximize_matching(BenefitNodes, Graph, !Matching) :-
-    ( Path = find_augmenting_path(BenefitNodes, Graph, !.Matching) ->
+    ( if Path = find_augmenting_path(BenefitNodes, Graph, !.Matching) then
         !:Matching = update_matches(Path, !.Matching),
-        maximize_matching(BenefitNodes, Graph, !Matching)
-    ;
+        disable_warning [suspicious_recursion] (
+            maximize_matching(BenefitNodes, Graph, !Matching)
+        )
+    else
         true
     ).
 
@@ -491,9 +497,9 @@ augpath_bf(Queue0, Seen0, Graph, Matching) = Path :-
     Matching = matching(CostToBenefitMap, _),
     CostMatches = map_adjs_to_matched_cost(
         set.to_sorted_list(AdjCostNodes), CostToBenefitMap),
-    ( find_unmatched_cost(CostMatches) = UnmatchedCostNode ->
+    ( if find_unmatched_cost(CostMatches) = UnmatchedCostNode then
         Path = [BenefitNode - UnmatchedCostNode | Path0]
-    ;
+    else
         add_alternates(CostMatches, Seen0, Seen, BenefitNode, Path0,
             Queue1, Queue2),
         Path = augpath_bf(Queue2, Seen, Graph, Matching)
@@ -526,16 +532,16 @@ find_unmatched_cost([CostNode - MaybeBenefitNode | Matches]) = Unmatched :-
 add_alternates([], !Seen, _, _, !Queue).
 add_alternates([CostMatch | CostMatches], !Seen, BenefitNode, Path, !Queue) :-
     CostMatch = CostNode - MaybeAdjBenefitNode,
-    (
+    ( if
         MaybeAdjBenefitNode = yes(AdjBenefitNode),
         not list.member(AdjBenefitNode, !.Seen)
-    ->
+    then
         !:Seen = [AdjBenefitNode | !.Seen],
         NewPath = [BenefitNode - CostNode | Path],
         BenefitNodeAndEdgeList =
             benefit_node_and_edge_list(AdjBenefitNode, NewPath),
         queue.put(BenefitNodeAndEdgeList, !Queue)
-    ;
+    else
         true
     ),
     add_alternates(CostMatches, !Seen, BenefitNode, Path, !Queue).
@@ -607,9 +613,9 @@ map_adjs_to_matched_cost(AdjCostNodes, CostToBenefitMap) = CostMatches :-
     pair(cost_node, maybe(benefit_node)).
 
 adj_to_matched_cost(CostToBenefitMap, CostNode) = Match :-
-    ( map.search(CostToBenefitMap, CostNode, BenefitNode) ->
+    ( if map.search(CostToBenefitMap, CostNode, BenefitNode) then
         Match = CostNode - yes(BenefitNode)
-    ;
+    else
         Match = CostNode - no
     ).
 
@@ -624,12 +630,12 @@ compute_via_cell_vars([FieldCostsBenefits | FieldsCostsBenefits],
     ViaCellVars1 = compute_via_cell_vars(FieldsCostsBenefits, MarkedBenefits),
     FieldCostsBenefits = field_costs_benefits(FieldVar, _, FieldBenefits),
     set.intersect(FieldBenefits, MarkedBenefits, MarkedFieldBenefits),
-    ( set.is_empty(MarkedFieldBenefits) ->
+    ( if set.is_empty(MarkedFieldBenefits) then
         set_of_var.insert(FieldVar, ViaCellVars1, ViaCellVars)
-    ; set.equal(MarkedFieldBenefits, FieldBenefits) ->
+    else if set.equal(MarkedFieldBenefits, FieldBenefits) then
         ViaCellVars = ViaCellVars1
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "theorem violation: intersection neither empty nor full")
     ).
 
@@ -644,9 +650,9 @@ compute_via_cell_vars([FieldCostsBenefits | FieldsCostsBenefits],
 get_unmatched_benefit_nodes([], _) = [].
 get_unmatched_benefit_nodes([Node | Nodes], MatchingBC) = UnmatchedNodes :-
     UnmatchedNodes1 = get_unmatched_benefit_nodes(Nodes, MatchingBC),
-    ( map.search(MatchingBC, Node, _Match) ->
+    ( if map.search(MatchingBC, Node, _Match) then
         UnmatchedNodes = UnmatchedNodes1
-    ;
+    else
         UnmatchedNodes = [Node | UnmatchedNodes1]
     ).
 
@@ -659,9 +665,9 @@ get_unmatched_benefit_nodes([Node | Nodes], MatchingBC) = UnmatchedNodes :-
 get_unmatched_cost_nodes([], _) = [].
 get_unmatched_cost_nodes([Node | Nodes], MatchingCB) = UnmatchedNodes :-
     UnmatchedNodes1 = get_unmatched_cost_nodes(Nodes, MatchingCB),
-    ( map.search(MatchingCB, Node, _Match) ->
+    ( if map.search(MatchingCB, Node, _Match) then
         UnmatchedNodes = UnmatchedNodes1
-    ;
+    else
         UnmatchedNodes = [Node | UnmatchedNodes1]
     ).
 
@@ -673,13 +679,15 @@ get_unmatched_cost_nodes([Node | Nodes], MatchingCB) = UnmatchedNodes :-
     % the call to dump_results, and two lines computing one of the arguments of
     % that call.
     %
-:- pred dump_results(prog_var::in, set_of_progvar::in, list(prog_var)::in,
+:- pred dump_results(io.text_output_stream::in,
+    prog_var::in, set_of_progvar::in, list(prog_var)::in,
     set_of_progvar::in, bool::in, set_of_progvar::in,
     assoc_list(int, set_of_progvar)::in,
     list(benefit_node)::in, list(benefit_operation)::in,
     list(cost_node)::in, list(cost_operation)::in, io::di, io::uo) is det.
 
-dump_results(CellVar, CandidateFieldVars, OccurringCandidateFieldVarList,
+dump_results(Stream, CellVar,
+        CandidateFieldVars, OccurringCandidateFieldVarList,
         ViaCellOccurringVars, Nullified, BeforeFlush, AfterFlush,
         BenefitNodes, BenefitOps, CostNodes, CostOps, !IO) :-
     term.var_to_int(CellVar, CellVarInt),
@@ -692,56 +700,52 @@ dump_results(CellVar, CandidateFieldVars, OccurringCandidateFieldVarList,
         OccurringCandidateFieldVarInts),
     list.map(term.var_to_int, ViaCellVarList, ViaCellVarInts),
     list.map(term.var_to_int, BeforeFlushList, BeforeFlushInts),
-    io.write_string("%\n% FIND_VIA_CELL_VARS ", !IO),
-    io.write_int(CellVarInt, !IO),
-    io.write_string(" => f(", !IO),
-    io.write_list(CandidateFieldVarInts, ", ", io.write_int, !IO),
-    io.write_string(")\n", !IO),
-    io.write_string("% occurring [", !IO),
-    io.write_list(OccurringCandidateFieldVarInts, ", ", io.write_int, !IO),
-    io.write_string("]\n", !IO),
-    io.write_string("% via cell [", !IO),
-    io.write_list(ViaCellVarInts, ", ", io.write_int, !IO),
-    io.write_string("]", !IO),
+    CandidateFieldVarsStr = string.join_list(", ",
+        list.map(string.int_to_string, CandidateFieldVarInts)),
+    OccurringCandidateFieldVarsStr = string.join_list(", ",
+        list.map(string.int_to_string, OccurringCandidateFieldVarInts)),
+    ViaCellVarsStr = string.join_list(", ",
+        list.map(string.int_to_string, ViaCellVarInts)),
     (
         Nullified = no,
-        io.write_string("\n", !IO)
+        NullifiedSuffix = ""
     ;
         Nullified = yes,
-        io.write_string(" nullified\n", !IO)
+        NullifiedSuffix = " nullified"
     ),
-    io.write_string("% before flush, segment 1: [", !IO),
-    io.write_list(BeforeFlushInts, ", ", io.write_int, !IO),
-    io.write_string("]\n", !IO),
-    list.foldl(dump_after_flush, AfterFlush, !IO),
-    io.write_string("% realized benefits: ", !IO),
-    io.write_int(list.length(BenefitOps), !IO),
-    io.write_string(" ops, ", !IO),
-    io.write_int(list.length(BenefitNodes), !IO),
-    io.write_string(" nodes:\n", !IO),
-    io.write_string("% ", !IO),
-    io.write(BenefitOps, !IO),
-    io.write_string("\n", !IO),
-    io.write_string("% realized costs: ", !IO),
-    io.write_int(list.length(CostOps), !IO),
-    io.write_string(" ops, ", !IO),
-    io.write_int(list.length(CostNodes), !IO),
-    io.write_string(" nodes:\n", !IO),
-    io.write_string("% ", !IO),
-    io.write(CostOps, !IO),
-    io.write_string("\n%\n", !IO).
+    BeforeFlushIntsStr = string.join_list(", ",
+        list.map(string.int_to_string, BeforeFlushInts)),
 
-:- pred dump_after_flush(pair(int, set_of_progvar)::in,
-    io::di, io::uo) is det.
+    io.format(Stream, "%%\n%% FIND_VIA_CELL_VARS %d => f(%s)\n",
+        [i(CellVarInt), s(CandidateFieldVarsStr)], !IO),
+    io.format(Stream, "%% occurring [%s]\n",
+        [s(OccurringCandidateFieldVarsStr)], !IO),
+    io.format(Stream, "%% via cell [%s]%s\n",
+        [s(ViaCellVarsStr), s(NullifiedSuffix)], !IO),
+    io.format(Stream, "%% before flush, segment 1: [%s]\n",
+        [s(BeforeFlushIntsStr)], !IO),
+    list.foldl(dump_after_flush(Stream), AfterFlush, !IO),
+    io.format(Stream, "%% realized benefits: %d ops, %d nodes:\n",
+        [i(list.length(BenefitOps)), i(list.length(BenefitNodes))], !IO),
+    io.write_string(Stream, "% ", !IO),
+    io.write_line(Stream, BenefitOps, !IO),
 
-dump_after_flush(SegmentNum - SegmentVars, !IO) :-
+    io.format(Stream, "%% realized costs: %d ops, %d nodes:",
+        [i(list.length(CostOps)), i(list.length(CostNodes))], !IO),
+    io.write_string(Stream, "% ", !IO),
+    io.write_line(Stream, CostOps, !IO),
+    io.write_string(Stream, "%\n", !IO).
+
+:- pred dump_after_flush(io.text_output_stream::in,
+    pair(int, set_of_progvar)::in, io::di, io::uo) is det.
+
+dump_after_flush(Stream, SegmentNum - SegmentVars, !IO) :-
     SegmentVarList = set_of_var.to_sorted_list(SegmentVars),
     list.map(term.var_to_int, SegmentVarList, SegmentVarInts),
-    io.write_string("% after flush, segment ", !IO),
-    io.write_int(SegmentNum, !IO),
-    io.write_string(": [", !IO),
-    io.write_list(SegmentVarInts, ", ", io.write_int, !IO),
-    io.write_string("]\n", !IO).
+    list.map(string.int_to_string, SegmentVarInts, SegmentVarStrs),
+    SegmentVarsStr = string.join_list(", ", SegmentVarStrs),
+    io.format(Stream, "%% after flush, segment %d: [%s]\n",
+        [i(SegmentNum), s(SegmentVarsStr)], !IO).
 
 %-----------------------------------------------------------------------------%
 
