@@ -1,22 +1,21 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2009, 2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: ml_commit_gen.m.
 % Main author: fjh.
 %
 % This module handles code generation for commits.
 %
-% There's several different ways of handling commits:
+% There are several different ways of handling commits:
 %   - using catch/throw
 %   - using setjmp/longjmp
 %   - using GCC's __builtin_setjmp/__builtin_longjmp
-%   - exiting nested functions via gotos to
-%     their containing functions
+%   - exiting nested functions via gotos to their containing functions
 %
 % The MLDS data structure abstracts away these differences using the
 % `try_commit' and `do_commit' instructions. The comments below show
@@ -25,7 +24,7 @@
 % This shows how the MLDS->target back-end can map mlds_commit_type,
 % do_commit and try_commit into target language constructs.
 %
-% Note that if we're using GCC's __builtin_longjmp(), then it is important
+% Note that if we are using GCC's __builtin_longjmp(), then it is important
 % that the call to __builtin_longjmp() be put in its own function, to ensure
 % that it is not in the same function as the __builtin_setjmp(). The code
 % generation schema below does that automatically. We will need to be careful
@@ -157,10 +156,12 @@
 :- module ml_backend.ml_commit_gen.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.code_model.
 :- import_module hlds.hlds_goal.
 :- import_module ml_backend.ml_gen_info.
 :- import_module ml_backend.mlds.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module list.
@@ -168,17 +169,16 @@
     % Generate code for a commit.
     %
 :- pred ml_gen_commit(hlds_goal::in, code_model::in, prog_context::in,
-    list(mlds_defn)::out, list(statement)::out,
-    ml_gen_info::in, ml_gen_info::out) is det.
+    list(mlds_local_var_defn)::out, list(mlds_function_defn)::out,
+    list(mlds_stmt)::out, ml_gen_info::in, ml_gen_info::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
-:- import_module libs.globals.
-:- import_module libs.options.
 :- import_module ml_backend.ml_accurate_gc.
 :- import_module ml_backend.ml_code_gen.
 :- import_module ml_backend.ml_code_util.
@@ -187,21 +187,20 @@
 :- import_module bool.
 :- import_module map.
 :- import_module maybe.
-:- import_module set.
-:- import_module string.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-ml_gen_commit(Goal, CodeModel, Context, Decls, Statements, !Info) :-
+ml_gen_commit(Goal, CodeModel, Context, LocalVarDefns, FuncDefns, Stmts,
+        !Info) :-
     Goal = hlds_goal(_, GoalInfo),
     GoalCodeModel = goal_info_get_code_model(GoalInfo),
     GoalContext = goal_info_get_context(GoalInfo),
 
-    (
+    ( if
         GoalCodeModel = model_non,
         CodeModel = model_semi
-    ->
+    then
 
         %   model_non in semi context: (using try_commit/do_commit)
         %       <succeeded = Goal>
@@ -239,49 +238,60 @@ ml_gen_commit(Goal, CodeModel, Context, Decls, Statements, !Info) :-
         %       commit_func();
         %   #endif
 
-        ml_gen_maybe_make_locals_for_output_args(GoalInfo, LocalVarDecls,
-            CopyLocalsToOutputArgs, OrigVarLvalMap, !Info),
+        ml_gen_info_get_var_lvals(!.Info, OrigVarLvalMap),
+        ml_gen_maybe_make_locals_for_output_args(GoalInfo,
+            OutputArgLocalVarDefns, CopyLocalsToOutputArgs, !Info),
 
         % Generate the `success()' function.
         ml_gen_new_func_label(no, SuccessFuncLabel, SuccessFuncLabelRval,
             !Info),
-        % push nesting level
-        MLDS_Context = mlds_make_context(Context),
-        ml_gen_info_new_aux_var_name("commit", CommitRef, !Info),
-        ml_gen_var_lval(!.Info, CommitRef, mlds_commit_type, CommitRefLval),
-        CommitRefDecl = ml_gen_commit_var_decl(MLDS_Context, CommitRef),
-        DoCommitStmt = ml_stmt_do_commit(ml_lval(CommitRefLval)),
-        DoCommitStatement = statement(DoCommitStmt, MLDS_Context),
+        % Push nesting level.
+        ml_gen_info_new_aux_var_name(mcav_commit, CommitRef, !Info),
+        CommitRefLval = ml_local_var(CommitRef, mlds_commit_type),
+        CommitRefDefn = ml_gen_commit_var_decl(Context, CommitRef),
+        DoCommitStmt = ml_stmt_do_commit(ml_lval(CommitRefLval), Context),
         % Pop nesting level.
         ml_gen_nondet_label_func(!.Info, SuccessFuncLabel, Context,
-            DoCommitStatement, SuccessFunc),
+            DoCommitStmt, SuccessFuncDefn),
 
-        ml_get_env_ptr(!.Info, EnvPtrRval),
-        SuccessCont = success_cont(SuccessFuncLabelRval, EnvPtrRval, [], []),
+        ml_get_env_ptr(EnvPtrRval),
+        SuccessCont = success_cont(SuccessFuncLabelRval, EnvPtrRval, []),
         ml_gen_info_push_success_cont(SuccessCont, !Info),
-        ml_gen_goal(model_non, Goal, GoalDecls, GoalStatements, !Info),
-        GoalStatement = ml_gen_block(GoalDecls, GoalStatements, GoalContext),
+        % Do not take any information about packed args into the new function
+        % depth, since that may cause dangling cross-function references
+        % when the new function depth is flattened out.
+        ml_gen_info_set_packed_word_map(map.init, !Info),
+        ml_gen_info_increment_func_nest_depth(!Info),
+        ml_gen_goal(model_non, Goal, GoalLocalVarDefns, GoalFuncDefns,
+            GoalStmts, !Info),
+        GoalStmt = ml_gen_block(GoalLocalVarDefns, GoalFuncDefns, GoalStmts,
+            GoalContext),
+        ml_gen_info_decrement_func_nest_depth(!Info),
+        % Do not take any information about packed args out of the new function
+        % depth, for the same reason.
+        ml_gen_info_set_packed_word_map(map.init, !Info),
         ml_gen_info_pop_success_cont(!Info),
-        ml_gen_set_success(!.Info, ml_const(mlconst_false), Context,
-            SetSuccessFalse),
-        ml_gen_set_success(!.Info, ml_const(mlconst_true), Context,
-            SetSuccessTrue),
+        ml_gen_set_success(ml_const(mlconst_false), Context, SetSuccessFalse,
+            !Info),
+        ml_gen_set_success(ml_const(mlconst_true), Context, SetSuccessTrue,
+            !Info),
         TryCommitStmt = ml_stmt_try_commit(CommitRefLval,
-            ml_gen_block([], [GoalStatement, SetSuccessFalse], Context),
-            ml_gen_block([], CopyLocalsToOutputArgs ++ [SetSuccessTrue],
-                Context)
+            ml_gen_block([], [], [GoalStmt, SetSuccessFalse], Context),
+            ml_gen_block([], [], CopyLocalsToOutputArgs ++ [SetSuccessTrue],
+                Context),
+            Context
         ),
-        TryCommitStatement = statement(TryCommitStmt, MLDS_Context),
-        CommitFuncLocalDecls = [CommitRefDecl, SuccessFunc],
-        maybe_put_commit_in_own_func(CommitFuncLocalDecls,
-            [TryCommitStatement], Context, CommitFuncDecls, Statements, !Info),
-        Decls = LocalVarDecls ++ CommitFuncDecls,
+        maybe_put_commit_in_own_func(CommitRefDefn, SuccessFuncDefn,
+            [TryCommitStmt], Context,
+            CommitFuncLocalVarDefns, CommitFuncFuncDefns, Stmts, !Info),
+        LocalVarDefns = OutputArgLocalVarDefns ++ CommitFuncLocalVarDefns,
+        FuncDefns = CommitFuncFuncDefns,
 
         ml_gen_info_set_var_lvals(OrigVarLvalMap, !Info)
-    ;
+    else if
         GoalCodeModel = model_non,
         CodeModel = model_det
-    ->
+    then
         %   model_non in det context: (using try_commit/do_commit)
         %       <do Goal>
         %   ===>
@@ -313,45 +323,49 @@ ml_gen_commit(Goal, CodeModel, Context, Decls, Statements, !Info) :-
         %       commit_func();
         %       #endif
 
-        ml_gen_maybe_make_locals_for_output_args(GoalInfo, LocalVarDecls,
-            CopyLocalsToOutputArgs, OrigVarLvalMap, !Info),
+        ml_gen_info_get_var_lvals(!.Info, OrigVarLvalMap),
+        ml_gen_maybe_make_locals_for_output_args(GoalInfo,
+            OutputArgLocalVarDefns, CopyLocalsToOutputArgs, !Info),
 
         % Generate the `success()' function.
         ml_gen_new_func_label(no, SuccessFuncLabel, SuccessFuncLabelRval,
             !Info),
         % push nesting level
-        MLDS_Context = mlds_make_context(Context),
-        ml_gen_info_new_aux_var_name("commit", CommitRef, !Info),
-        ml_gen_var_lval(!.Info, CommitRef, mlds_commit_type, CommitRefLval),
-        CommitRefDecl = ml_gen_commit_var_decl(MLDS_Context, CommitRef),
-        DoCommitStmt = ml_stmt_do_commit(ml_lval(CommitRefLval)),
-        DoCommitStatement = statement(DoCommitStmt, MLDS_Context),
+        ml_gen_info_new_aux_var_name(mcav_commit, CommitRef, !Info),
+        CommitRefLval = ml_local_var(CommitRef, mlds_commit_type),
+        CommitRefDefn = ml_gen_commit_var_decl(Context, CommitRef),
+        DoCommitStmt = ml_stmt_do_commit(ml_lval(CommitRefLval), Context),
         % pop nesting level
         ml_gen_nondet_label_func(!.Info, SuccessFuncLabel, Context,
-            DoCommitStatement, SuccessFunc),
+            DoCommitStmt, SuccessFuncDefn),
 
-        ml_get_env_ptr(!.Info, EnvPtrRval),
-        SuccessCont = success_cont(SuccessFuncLabelRval, EnvPtrRval, [], []),
+        ml_get_env_ptr(EnvPtrRval),
+        SuccessCont = success_cont(SuccessFuncLabelRval, EnvPtrRval, []),
         ml_gen_info_push_success_cont(SuccessCont, !Info),
-        ml_gen_goal(model_non, Goal, GoalDecls, GoalStatements, !Info),
+        ml_gen_info_increment_func_nest_depth(!Info),
+        ml_gen_goal(model_non, Goal, GoalLocalVarDefns, GoalFuncDefns,
+            GoalStmts, !Info),
+        ml_gen_info_decrement_func_nest_depth(!Info),
         % Hoist any static constant declarations for Goal out to the top level.
-        GoalStatement = ml_gen_block(GoalDecls, GoalStatements, GoalContext),
+        GoalStmt = ml_gen_block(GoalLocalVarDefns, GoalFuncDefns, GoalStmts,
+            GoalContext),
         ml_gen_info_pop_success_cont(!Info),
 
-        TryCommitStmt = ml_stmt_try_commit(CommitRefLval, GoalStatement,
-            ml_gen_block([], CopyLocalsToOutputArgs, Context)),
-        TryCommitStatement = statement(TryCommitStmt, MLDS_Context),
-        CommitFuncLocalDecls = [CommitRefDecl, SuccessFunc],
-        maybe_put_commit_in_own_func(CommitFuncLocalDecls,
-            [TryCommitStatement], Context, CommitFuncDecls, Statements, !Info),
-        Decls = LocalVarDecls ++ CommitFuncDecls,
+        TryCommitStmt = ml_stmt_try_commit(CommitRefLval, GoalStmt,
+            ml_gen_block([], [], CopyLocalsToOutputArgs, Context), Context),
+        maybe_put_commit_in_own_func(CommitRefDefn, SuccessFuncDefn,
+            [TryCommitStmt], Context,
+            CommitFuncLocalVarDefns, CommitFuncFuncDefns, Stmts, !Info),
+        LocalVarDefns = OutputArgLocalVarDefns ++ CommitFuncLocalVarDefns,
+        FuncDefns = CommitFuncFuncDefns,
         ml_gen_info_set_var_lvals(OrigVarLvalMap, !Info)
-    ;
+    else
         % No commit required.
-        ml_gen_goal(CodeModel, Goal, Decls, Statements, !Info)
+        ml_gen_goal(CodeModel, Goal, LocalVarDefns, FuncDefns, Stmts, !Info)
     ).
 
-    % maybe_put_commit_in_own_func(Defns0, Stmts0, Defns, Stmts):
+    % maybe_put_commit_in_own_func(LocalVarDefs0, FuncDefn0,
+    %   Stmts0, Defns, Stmts):
     %
     % If the --put-commit-in-own-func option is set, put the commit in its
     % own function. This is needed for the high-level C back-end, to handle
@@ -367,7 +381,7 @@ ml_gen_commit(Goal, CodeModel, Context, Decls, Statements, !Info) :-
     %
     % To avoid this, whenever we generate a commit, we put it in its own
     % nested function, with the local variables (e.g. `succeeded', plus any
-    % outputs from the goal that we're committing over) remaining in the
+    % outputs from the goal that we are committing over) remaining in the
     % containing function. This ensures that none of the variables which
     % get modified between the setjmp() and the longjmp() and which get
     % referenced after the longjmp() are local variables in the function
@@ -376,58 +390,53 @@ ml_gen_commit(Goal, CodeModel, Context, Decls, Statements, !Info) :-
     % [The obvious alternative of declaring the local variables in the function
     % containing setjmp() as `volatile' doesn't work, since the assignments
     % to those output variables may be deep in some function called indirectly
-    % from the goal that we're committing across, and assigning to a
+    % from the goal that we are committing across, and assigning to a
     % volatile-qualified variable via a non-volatile pointer is undefined
     % behaviour. The only way to make it work would be to be to declare
     % *every* output argument that we pass by reference as `volatile T *'.
     % But that would impose distributed fat and would make interoperability
     % difficult.]
     %
-:- pred maybe_put_commit_in_own_func(list(mlds_defn)::in, list(statement)::in,
-    prog_context::in, list(mlds_defn)::out, list(statement)::out,
-    ml_gen_info::in, ml_gen_info::out) is det.
+:- pred maybe_put_commit_in_own_func(
+    mlds_local_var_defn::in, mlds_function_defn::in, list(mlds_stmt)::in,
+    prog_context::in,
+    list(mlds_local_var_defn)::out, list(mlds_function_defn)::out,
+    list(mlds_stmt)::out, ml_gen_info::in, ml_gen_info::out) is det.
 
-maybe_put_commit_in_own_func(CommitFuncLocalDecls, TryCommitStatements,
-        Context, Decls, Statements, !Info) :-
+maybe_put_commit_in_own_func(LocalVarDefn0, FuncDefn0, TryCommitStmts,
+        Context, LocalVarDefns, FuncDefns, Stmts, !Info) :-
     ml_gen_info_put_commit_in_own_func(!.Info, PutCommitInOwnFunc),
     (
         PutCommitInOwnFunc = yes,
 
         % Generate the `void commit_func() { ... }' wrapper
-        % around the main body that we generated above
+        % around the main body that we generated above.
         ml_gen_new_func_label(no, CommitFuncLabel, CommitFuncLabelRval, !Info),
-        % push nesting level
-        CommitFuncBody = ml_gen_block(CommitFuncLocalDecls,
-            TryCommitStatements, Context),
-        % pop nesting level
+        % Push nesting level.
+        CommitFuncBody = ml_gen_block([LocalVarDefn0], [FuncDefn0],
+            TryCommitStmts, Context),
+        % Pop nesting level.
         ml_gen_nondet_label_func(!.Info, CommitFuncLabel, Context,
-            CommitFuncBody, CommitFunc),
+            CommitFuncBody, CommitFuncDefn),
 
         % Generate the call to `commit_func();'
-        ml_gen_info_use_gcc_nested_functions(!.Info, UseNestedFuncs),
-        (
-            UseNestedFuncs = yes,
-            ArgRvals = [],
-            ArgTypes = []
-        ;
-            UseNestedFuncs = no,
-            ml_get_env_ptr(!.Info, EnvPtrRval),
-            ArgRvals = [EnvPtrRval],
-            ArgTypes = [mlds_generic_env_ptr_type]
-        ),
+        ml_get_env_ptr(EnvPtrRval),
+        ArgRvals = [EnvPtrRval],
+        ArgTypes = [mlds_generic_env_ptr_type],
         RetTypes = [],
         Signature = mlds_func_signature(ArgTypes, RetTypes),
         CallKind = ordinary_call,
-        CallStmt = ml_stmt_call(Signature, CommitFuncLabelRval, no, ArgRvals,
-            [], CallKind),
-        CallStatement = statement(CallStmt, mlds_make_context(Context)),
+        CallStmt = ml_stmt_call(Signature, CommitFuncLabelRval, ArgRvals,
+            [], CallKind, Context),
         % Package it all up.
-        Statements = [CallStatement],
-        Decls = [CommitFunc]
+        LocalVarDefns = [],
+        FuncDefns = [CommitFuncDefn],
+        Stmts = [CallStmt]
     ;
         PutCommitInOwnFunc = no,
-        Statements = TryCommitStatements,
-        Decls = CommitFuncLocalDecls
+        LocalVarDefns = [LocalVarDefn0],
+        FuncDefns = [FuncDefn0],
+        Stmts = TryCommitStmts
     ).
 
     % In commits, you have model_non code called from a model_det or model_semi
@@ -435,25 +444,21 @@ maybe_put_commit_in_own_func(CommitFuncLocalDecls, TryCommitStatements,
     % if the context is a model_det or model_semi procedure with output
     % arguments passed by reference, then we need to introduce local variables
     % corresponding to those output arguments, and at the end of the commit
-    % we'll copy the local variables into the output arguments.
+    % we will copy the local variables into the output arguments.
     %
 :- pred ml_gen_maybe_make_locals_for_output_args(hlds_goal_info::in,
-    list(mlds_defn)::out, list(statement)::out,
-    map(prog_var, mlds_lval)::out,
+    list(mlds_local_var_defn)::out, list(mlds_stmt)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
 ml_gen_maybe_make_locals_for_output_args(GoalInfo, LocalVarDecls,
-        CopyLocalsToOutputArgs, OrigVarLvalMap, !Info) :-
-    ml_gen_info_get_var_lvals(!.Info, OrigVarLvalMap),
-    ml_gen_info_get_globals(!.Info, Globals),
-    globals.lookup_bool_option(Globals, nondet_copy_out, NondetCopyOut),
+        CopyLocalsToOutputArgs, !Info) :-
+    ml_gen_info_get_nondet_copy_out(!.Info, NondetCopyOut),
     (
         NondetCopyOut = yes,
         Context = goal_info_get_context(GoalInfo),
         NonLocals = goal_info_get_nonlocals(GoalInfo),
         ml_gen_info_get_byref_output_vars(!.Info, ByRefOutputVars),
-        VarsToCopy = set_of_var.intersect(
-            set_of_var.list_to_set(ByRefOutputVars), NonLocals),
+        VarsToCopy = set_of_var.intersect(ByRefOutputVars, NonLocals),
         ml_gen_make_locals_for_output_args(
             set_of_var.to_sorted_list(VarsToCopy),
             Context, LocalVarDecls, CopyLocalsToOutputArgs, !Info)
@@ -464,7 +469,7 @@ ml_gen_maybe_make_locals_for_output_args(GoalInfo, LocalVarDecls,
     ).
 
 :- pred ml_gen_make_locals_for_output_args(list(prog_var)::in,
-    prog_context::in, list(mlds_defn)::out, list(statement)::out,
+    prog_context::in, list(mlds_local_var_defn)::out, list(mlds_stmt)::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
 ml_gen_make_locals_for_output_args([], _, [], [], !Info).
@@ -474,7 +479,7 @@ ml_gen_make_locals_for_output_args([Var | Vars], Context,
         !Info),
     ml_gen_info_get_module_info(!.Info, ModuleInfo),
     ml_variable_type(!.Info, Var, Type),
-    IsDummy = check_dummy_type(ModuleInfo, Type),
+    IsDummy = is_type_a_dummy(ModuleInfo, Type),
     (
         IsDummy = is_dummy_type,
         LocalDefns = LocalDefns0,
@@ -488,28 +493,26 @@ ml_gen_make_locals_for_output_args([Var | Vars], Context,
     ).
 
 :- pred ml_gen_make_local_for_output_arg(prog_var::in, mer_type::in,
-    prog_context::in, mlds_defn::out, statement::out,
+    prog_context::in, mlds_local_var_defn::out, mlds_stmt::out,
     ml_gen_info::in, ml_gen_info::out) is det.
 
 ml_gen_make_local_for_output_arg(OutputVar, Type, Context,
         LocalVarDefn, Assign, !Info) :-
     % Look up the name of the output variable.
     ml_gen_info_get_varset(!.Info, VarSet),
-    OutputVarName = ml_gen_var_name(VarSet, OutputVar),
+    OutputVarName = ml_gen_local_var_name(VarSet, OutputVar),
 
     % Generate a declaration for a corresponding local variable.
-    OutputVarName = mlds_var_name(OutputVarNameStr, MaybeNum),
-    LocalVarName = mlds_var_name(
-        string.append("local_", OutputVarNameStr), MaybeNum),
+    OutputVarName = lvn_prog_var(OutputVarNameStr, MaybeNum),
+    LocalVarName = lvn_local_var(OutputVarNameStr, MaybeNum),
     ml_gen_type(!.Info, Type, MLDS_Type),
-    ml_gen_gc_statement(LocalVarName, Type, Context, GCStatement,
-        !Info),
-    LocalVarDefn = ml_gen_mlds_var_decl(mlds_data_var(LocalVarName), MLDS_Type,
-        GCStatement, mlds_make_context(Context)),
+    ml_gen_gc_statement(LocalVarName, Type, Context, GCStmt, !Info),
+    LocalVarDefn = ml_gen_mlds_var_decl(LocalVarName, MLDS_Type,
+        GCStmt, Context),
 
     % Generate code to assign from the local var to the output var.
     ml_gen_var(!.Info, OutputVar, OutputVarLval),
-    ml_gen_var_lval(!.Info, LocalVarName, MLDS_Type, LocalVarLval),
+    LocalVarLval = ml_local_var(LocalVarName, MLDS_Type),
     Assign = ml_gen_assign(OutputVarLval, ml_lval(LocalVarLval), Context),
 
     % Update the lval for this variable so that any references to it inside
@@ -520,12 +523,12 @@ ml_gen_make_local_for_output_arg(OutputVar, Type, Context,
 
     % Generate the declaration for the `commit' variable.
     %
-:- func ml_gen_commit_var_decl(mlds_context, mlds_var_name) = mlds_defn.
+:- func ml_gen_commit_var_decl(prog_context, mlds_local_var_name)
+    = mlds_local_var_defn.
 
 ml_gen_commit_var_decl(Context, VarName) =
-    ml_gen_mlds_var_decl(mlds_data_var(VarName), mlds_commit_type, gc_no_stmt,
-        Context).
+    ml_gen_mlds_var_decl(VarName, mlds_commit_type, gc_no_stmt, Context).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module ml_backend.ml_commit_gen.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
