@@ -23,6 +23,7 @@
 :- import_module hlds.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_module.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 
 :- import_module io.
@@ -69,16 +70,6 @@
 :- pred mode_reordering(pred_constraints_map::in, mc_var_map::in,
     list(list(pred_id))::in, module_info::in, module_info::out) is det.
 
-    % dump_goal_paths(ModuleInfo, PredIds, !IO)
-    %
-    % Dumps the goal paths of each goal in the order they appear for each
-    % predicate in PredIds for the purposes of visually checking re-ordering.
-    %
-:- pred dump_goal_paths(module_info::in, list(pred_id)::in, io::di, io::uo)
-    is det.
-
-%-----------------------------------------------------------------------------%
-
     % add_ordering_constraint(Constraint, !OCI) adds Constraint
     % to the ordering constraints store. It fails if it immediately
     % detects a contradiction (at the moment, this means it has
@@ -98,6 +89,16 @@
 :- pred add_lt_constraint(conjunct_id::in, conjunct_id::in,
     ordering_constraints_info::in, ordering_constraints_info::out) is semidet.
 
+%-----------------------------------------------------------------------------%
+
+    % dump_goal_paths(ModuleInfo, PredIds, !IO)
+    %
+    % Dumps the goal paths of each goal in the order they appear for each
+    % predicate in PredIds for the purposes of visually checking re-ordering.
+    %
+:- pred dump_goal_paths(module_info::in, list(pred_id)::in, io::di, io::uo)
+    is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
@@ -111,7 +112,6 @@
 :- import_module hlds.hlds_goal.
 :- import_module libs.
 :- import_module libs.globals.
-:- import_module mdbcomp.
 :- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
@@ -185,14 +185,14 @@ scc_reordering(PredConstraintsMap, VarMap, SCC0, !ModuleInfo) :-
         SCC0, _, SCC),
 
     list.filter(
-        (pred(PredId::in) is semidet :-
+        ( pred(PredId::in) is semidet :-
             module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
             pred_info_infer_modes(PredInfo)
         ), SCC, PredsToInfer, PredsToCheck),
 
     (
         PredsToInfer = [_ | _],
-        sorry($module, $pred, "NYI: mode inference")
+        sorry($pred, "NYI: mode inference")
     ;
         PredsToInfer = []
     ),
@@ -209,15 +209,17 @@ scc_reordering(PredConstraintsMap, VarMap, SCC0, !ModuleInfo) :-
 
 pred_reordering(PredConstraintsMap, VarMap, PredId, !ModuleInfo) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo0),
-    ( pred_info_infer_modes(PredInfo0) ->
+    ( if pred_info_infer_modes(PredInfo0) then
         % XXX GIVE UP FOR NOW!!!! In reality, execution shouldn't reach here
         % if the pred is to be mode inferred, should it?
-        sorry($module, $pred, "mode inference constraints")
-    ;
+        sorry($pred, "mode inference constraints")
+    else
         % XXX Maybe move this outside of this predicate - then
         % the predicate can assume that the correct procedures
         % have been created and that they have the correct bodies.
-        copy_module_clauses_to_procs([PredId], !ModuleInfo),
+        % XXX Indeed, this job *should* have been done *before* this pass
+        % was invoked.
+        copy_clauses_to_procs_for_pred_in_module_info(PredId, !ModuleInfo),
 
         module_info_pred_info(!.ModuleInfo, PredId, PredInfo1),
 
@@ -237,7 +239,7 @@ pred_reordering(PredConstraintsMap, VarMap, PredId, !ModuleInfo) :-
             % XXX Deal with mode errors here!
             % This is a placeholder error message.
             ErrorsString = string.string(Errors),
-            sorry($module, $pred, "mode checking failure: " ++ ErrorsString)
+            sorry($pred, "mode checking failure: " ++ ErrorsString)
         )
     ).
 
@@ -302,15 +304,15 @@ proc_reordering(ContainingGoalMap, PredConstraints, VarMap, PredId, ProcId,
 
 solve_proc_reordering(ContainingGoalMap, VarMap, PredId, ProcId,
         SolverConstraints, !Errors, !Goal) :-
-    (
+    ( if
         mcsolver.solve(SolverConstraints, Bindings),
         goal_reordering(ContainingGoalMap, PredId, VarMap, Bindings, !Goal)
-    ->
+    then
         true
-    ;
-        ( mcsolver.solve(SolverConstraints, _) ->
+    else
+        ( if mcsolver.solve(SolverConstraints, _) then
             list.cons(conjunct_ordering_failed(proc(PredId, ProcId)), !Errors)
-        ;
+        else
             list.cons(no_producer_consumer_sols(proc(PredId, ProcId)), !Errors)
         )
     ).
@@ -372,7 +374,7 @@ goal_reordering(ContainingGoalMap, PredId, VarMap, Bindings, Goal0, Goal) :-
     ;
         GoalExpr0 = switch(_, _, _),
         % We haven't yet even tried to turn disjunctions into switches.
-        unexpected($module, $pred, "switch")
+        unexpected($pred, "switch")
     ;
         GoalExpr0 = if_then_else(Vars, Cond0, Then0, Else0),
         goal_reordering(ContainingGoalMap, PredId, VarMap, Bindings,
@@ -398,7 +400,7 @@ goal_reordering(ContainingGoalMap, PredId, VarMap, Bindings, Goal0, Goal) :-
         GoalExpr0 = shorthand(_),
         % XXX We need to handle atomic goals.
         % XXX We need to handle try goals.
-        unexpected($module, $pred, "NYI: shorthand")
+        unexpected($pred, "NYI: shorthand")
     ),
     Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -415,13 +417,14 @@ ordering_init(ContainingGoalMap, N) =
 %-----------------------------------------------------------------------------%
 
 add_ordering_constraint(Constraint, !OCI) :-
-    ( set.member(Constraint, !.OCI ^ oci_constraints) ->
+    ( if set.member(Constraint, !.OCI ^ oci_constraints) then
         true
-    ;
+    else
         constraint_transitive_closure(!.OCI, Constraint, NewConstraints),
 
         % No cycles. (lt(X, X) is a contradiction)
-        set.empty(set.filter(pred(lt(X, X)::in) is semidet, NewConstraints)),
+        set.is_empty(
+            set.filter(pred(lt(X, X)::in) is semidet, NewConstraints)),
 
         !OCI ^ oci_constraints :=
             set.union(NewConstraints, !.OCI ^ oci_constraints)
@@ -497,12 +500,12 @@ make_conjuncts_nonlocal_repvars(PredId, Goals, RepvarMap) :-
     prog_var_at_conjuncts_map::in, prog_var_at_conjuncts_map::out) is det.
 
 make_conjunct_nonlocal_repvars(PredId, Goal, !RepvarMap) :-
-    GoalInfo = Goal ^ hlds_goal_info,
+    Goal = hlds_goal(_, GoalInfo),
     Nonlocals = goal_info_get_nonlocals(GoalInfo),
     GoalId = goal_info_get_goal_id(GoalInfo),
 
     set_of_var.fold(
-        (pred(NL::in, RMap0::in, RMap::out) is det :-
+        ( pred(NL::in, RMap0::in, RMap::out) is det :-
             multi_map.set(NL, NL `in` PredId `at` GoalId, RMap0, RMap)
         ),
         Nonlocals, !RepvarMap).
@@ -602,7 +605,7 @@ minimum_reordering(OCI, Order) :-
 %    original_order_constraints(OCI ^ num_conjuncts, OriginalOrderConstraints),
 %    constrain_if_possible(OriginalOrderConstraints, OCI0, OCI1),
 
-    Conjuncts = set.from_sorted_list(1 `..` OCI ^ oci_num_conjuncts),
+    Conjuncts = set.sorted_list_to_set(1 `..` OCI ^ oci_num_conjuncts),
     topological_sort_min_reordering(OCI ^ oci_constraints, Conjuncts, Order).
 
     % original_order_constraints(N, MOCs) produces a list of constraints MOCs
@@ -611,6 +614,7 @@ minimum_reordering(OCI, Order) :-
     %
 :- pred original_order_constraints(int::in,
     mode_ordering_constraints::out) is det.
+:- pragma consider_used(pred(original_order_constraints/2)).
 
 original_order_constraints(N, MOCs) :-
     complete_order_constraints(1 `..` N, MOCs).
@@ -645,12 +649,14 @@ add_complete_order_constraints([Conjunct | Conjuncts], !MOCs) :-
     %
 :- pred constrain_if_possible(mode_ordering_constraints::in,
     ordering_constraints_info::in, ordering_constraints_info::out) is det.
+:- pragma consider_used(pred(constrain_if_possible/3)).
 
 constrain_if_possible([], !OCI).
 constrain_if_possible([Constraint | Constraints], !OCI) :-
-    ( add_ordering_constraint(Constraint, !OCI) ->
+    ( if add_ordering_constraint(Constraint, !OCI) then
         constrain_if_possible(Constraints, !OCI)
-    ;
+    else
+        % XXX This action contradicts the comment above.
         constrain_if_possible(Constraints, !OCI)
     ).
 
@@ -668,7 +674,7 @@ topological_sort_min_reordering(Constraints0, Conjuncts0, Ordering) :-
     NotFirst = set.map(func(lt(_From, To)) = To, Constraints0),
     CandidatesForFirst = set.difference(Conjuncts0, NotFirst),
 
-    ( set.remove_least(First, CandidatesForFirst, _) ->
+    ( if set.remove_least(First, CandidatesForFirst, _) then
         % Remove First from the system.
         set.remove(First, Conjuncts0, Conjuncts),
         Constraints = set.filter(
@@ -678,10 +684,10 @@ topological_sort_min_reordering(Constraints0, Conjuncts0, Ordering) :-
         % Order the rest, then put First at the head.
         topological_sort_min_reordering(Constraints, Conjuncts, Ordering0),
         Ordering = [First | Ordering0]
-    ;
+    else
         % No cantidates for First, so we are only done if there were
         % no nodes (conjuncts) left to begin with.
-        set.empty(Conjuncts0),
+        set.is_empty(Conjuncts0),
         Ordering = []
     ).
 
@@ -703,7 +709,7 @@ dump_goal_paths(ModuleInfo, PredIds0, !IO) :-
 
 dump_pred_goal_paths(ModuleInfo, PredId, !IO) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    pred_info_get_procedures(PredInfo, ProcTable),
+    pred_info_get_proc_table(PredInfo, ProcTable),
     ProcIds = map.keys(ProcTable),
 
     % Start with a blank line.
@@ -720,7 +726,7 @@ dump_pred_goal_paths(ModuleInfo, PredId, !IO) :-
         ProcIds = [],
         pred_info_get_clauses_info(PredInfo, ClausesInfo),
         clauses_info_get_clauses_rep(ClausesInfo, ClausesRep, _ItemNumbers),
-        get_clause_list(ClausesRep, Clauses),
+        get_clause_list_maybe_repeated(ClausesRep, Clauses),
         Goals = list.map(func(Clause) = clause_body(Clause), Clauses),
         Indent = 0,
         list.foldl(dump_goal_goal_paths(Globals, Indent), Goals, !IO)
@@ -783,7 +789,7 @@ dump_goal_goal_paths(Globals, Indent, Goal, !IO) :-
         list.foldl(dump_goal_goal_paths(Globals, SubGoalIndent), Goals, !IO)
     ;
         GoalExpr = switch(_, _, _),
-        unexpected($module, $pred, "switch")
+        unexpected($pred, "switch")
     ;
         GoalExpr = if_then_else(_, CondGoal, ThenGoal, ElseGoal),
         Goals = [CondGoal, ThenGoal, ElseGoal],
@@ -803,13 +809,13 @@ dump_goal_goal_paths(Globals, Indent, Goal, !IO) :-
                 !IO)
         ;
             ShortHand = try_goal(_, _, _),
-            unexpected($module, $pred, "try_goal")
+            unexpected($pred, "try_goal")
         ;
             ShortHand = bi_implication(_, _),
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
 %-----------------------------------------------------------------------------%
-:- end_module ordering_mode_constraints.
+:- end_module check_hlds.ordering_mode_constraints.
 %-----------------------------------------------------------------------------%
