@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2011-2012 The University of Melbourne.
+% Copyright (C) 2015, 2017 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -48,15 +49,15 @@
 %   map(LC, _, [], []) :-
 %       finish_loop_control(LC).
 %   map(LC, M, [X | Xs], [Y | Ys) :-
-%       wait_free_slot(LC, LCS) ->
+%       wait_free_slot(LC, LCS),
 %       spawn_off(LCS, (
 %           M(X, Y),
 %           join_and_terminate(LC, LCS)
-%       ),
+%       )),
 %       map(LC, M, Xs, Ys). % May not use tail recursion.
 %
 % The parallel conjunction is replaced with a wait_free_slot goal and a
-% spawn_off goal for each conjunct except for the last. The last is re-written
+% spawn_off goal for each conjunct except for the last. The last is rewritten
 % to call the loop control version of the predicate.
 %
 % Rules:
@@ -84,6 +85,7 @@
 :- module transform_hlds.par_loop_control.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 
 %----------------------------------------------------------------------------%
@@ -98,20 +100,27 @@
 
 :- import_module hlds.goal_path.
 :- import_module hlds.goal_util.
+:- import_module hlds.hlds_dependency_graph.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.instmap.
 :- import_module hlds.passes_aux.
 :- import_module hlds.pred_table.
+:- import_module hlds.status.
+:- import_module hlds.vartypes.
+:- import_module libs.
+:- import_module libs.dependency_graph.
 :- import_module libs.globals.
 :- import_module libs.options.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
 :- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_util.
 :- import_module parse_tree.set_of_var.
-:- import_module transform_hlds.dependency_graph.
 
 :- import_module bool.
 :- import_module digraph.
@@ -128,15 +137,16 @@
 
 maybe_par_loop_control_module(!ModuleInfo) :-
     module_info_rebuild_dependency_info(!ModuleInfo, DepInfo),
-    process_all_nonimported_procs(
+    process_valid_nonimported_procs(
         update_module(maybe_par_loop_control_proc(DepInfo)),
         !ModuleInfo).
 
-:- pred maybe_par_loop_control_proc(dependency_info::in, pred_proc_id::in,
-    proc_info::in, proc_info::out, module_info::in, module_info::out) is det.
+:- pred maybe_par_loop_control_proc(hlds_dependency_info::in,
+    pred_proc_id::in, proc_info::in, proc_info::out,
+    module_info::in, module_info::out) is det.
 
 maybe_par_loop_control_proc(DepInfo, PredProcId, !ProcInfo, !ModuleInfo) :-
-    ( loop_control_is_applicable(DepInfo, PredProcId, !.ProcInfo) ->
+    ( if loop_control_is_applicable(DepInfo, PredProcId, !.ProcInfo) then
         proc_info_get_goal(!.ProcInfo, Body0),
 
         % Re-calculate goal ids.
@@ -161,7 +171,7 @@ maybe_par_loop_control_proc(DepInfo, PredProcId, !ProcInfo, !ModuleInfo) :-
             update_outer_proc(PredProcId, InnerPredProcId, InnerPredName,
                 !.ModuleInfo, !ProcInfo)
         )
-    ;
+    else
         true
     ).
 
@@ -171,11 +181,12 @@ maybe_par_loop_control_proc(DepInfo, PredProcId, !ProcInfo, !ModuleInfo) :-
     % conjunction with exactly two conjuncts whose right conjunct contains a
     % recursive call.
     %
-:- pred loop_control_is_applicable(dependency_info::in, pred_proc_id::in,
-    proc_info::in) is semidet.
+:- pred loop_control_is_applicable(hlds_dependency_info::in,
+    pred_proc_id::in, proc_info::in) is semidet.
 
 loop_control_is_applicable(DepInfo, PredProcId, ProcInfo) :-
-    proc_info_get_has_parallel_conj(ProcInfo, yes),
+    proc_info_get_has_parallel_conj(ProcInfo, HasParallelConj),
+    HasParallelConj = has_parallel_conj,
     proc_info_get_inferred_determinism(ProcInfo, Detism),
     % If the predicate itself is not deterministic then its recursive call
     % will not be deterministic and therefore will not be found in a parallel
@@ -185,11 +196,11 @@ loop_control_is_applicable(DepInfo, PredProcId, ProcInfo) :-
     ),
     proc_is_self_recursive(DepInfo, PredProcId).
 
-:- pred proc_is_self_recursive(dependency_info::in, pred_proc_id::in)
+:- pred proc_is_self_recursive(hlds_dependency_info::in, pred_proc_id::in)
     is semidet.
 
 proc_is_self_recursive(DepInfo, PredProcId) :-
-    hlds_dependency_info_get_dependency_graph(DepInfo, DepGraph),
+    DepGraph = dependency_info_get_graph(DepInfo),
 
     % There must be a directly recursive call.
     digraph.lookup_key(DepGraph, PredProcId, SelfKey),
@@ -231,7 +242,7 @@ proc_is_self_recursive(DepInfo, PredProcId) :-
     % This subtype of seen usable recursion is the set of values for which we
     % should keep searching.
     %
-:- inst seen_usable_recursion_continue
+:- inst seen_usable_recursion_continue for seen_usable_recursion/0
     --->    have_not_seen_recursive_call
     ;       seen_one_recursive_call_on_every_branch
     ;       seen_usable_recursion_in_par_conj(ground).
@@ -243,16 +254,16 @@ goal_get_loop_control_par_conjs(Goal, SelfPredProcId, SeenUsableRecursion) :-
     Goal = hlds_goal(GoalExpr, GoalInfo),
     Detism = goal_info_get_determinism(GoalInfo),
     InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
-    ( instmap_delta_is_reachable(InstmapDelta) ->
+    ( if instmap_delta_is_reachable(InstmapDelta) then
         (
             GoalExpr = unify(_, _, _, _, _),
             SeenUsableRecursion0 = have_not_seen_recursive_call
         ;
             GoalExpr = plain_call(PredId, ProcId, _, _, _, _),
-            ( SelfPredProcId = proc(PredId, ProcId) ->
+            ( if SelfPredProcId = proc(PredId, ProcId) then
                 SeenUsableRecursion0 =
                     seen_one_recursive_call_on_every_branch
-            ;
+            else
                 SeenUsableRecursion0 = have_not_seen_recursive_call
             )
         ;
@@ -280,9 +291,9 @@ goal_get_loop_control_par_conjs(Goal, SelfPredProcId, SeenUsableRecursion) :-
             GoalExpr = disj(_),
             % If the disjunction contains a recursive call at all, then the
             % recursive call is in an unusable context.
-            ( goal_calls(Goal, SelfPredProcId) ->
+            ( if goal_calls(Goal, SelfPredProcId) then
                 SeenUsableRecursion0 = seen_unusable_recursion
-            ;
+            else
                 SeenUsableRecursion0 = have_not_seen_recursive_call
             )
         ;
@@ -335,7 +346,7 @@ goal_get_loop_control_par_conjs(Goal, SelfPredProcId, SeenUsableRecursion) :-
             )
         ;
             GoalExpr = shorthand(_),
-            unexpected($module, $pred, "shorthand")
+            unexpected($pred, "shorthand")
         ),
 
         % If the goal might fail or might succeed more than once, then any
@@ -365,7 +376,7 @@ goal_get_loop_control_par_conjs(Goal, SelfPredProcId, SeenUsableRecursion) :-
                 SeenUsableRecursion = seen_unusable_recursion
             )
         )
-    ;
+    else
         % InstmapDelta is unreachable.
         SeenUsableRecursion = have_not_seen_recursive_call
     ).
@@ -383,7 +394,7 @@ par_conj_get_loop_control_par_conjs(Conjs, SelfPredProcId,
         GoalId, SeenUsableRecursion) :-
     (
         Conjs = [],
-        unexpected($module, $pred, "Empty parallel conjunction")
+        unexpected($pred, "Empty parallel conjunction")
     ;
         Conjs = [Head | Tail],
         par_conj_get_loop_control_par_conjs_lag(Head, Tail, SelfPredProcId,
@@ -595,10 +606,11 @@ create_inner_proc(RecParConjIds, OldPredProcId, OldProcInfo,
     some [!PredInfo, !Body, !VarSet, !VarTypes] (
         % Construct the pred info structure. We initially construct it with
         % the old proc info which will be replaced below.
+        GoalType = goal_not_for_promise(np_goal_type_none),
         pred_info_create(ModuleName, PredSym, PredOrFunc, Context, Origin,
-            status_local, Markers, ArgTypes0, TypeVarSet, ExistQVars,
-            ClassConstraints, set.init, map.init, OldProcInfo, ProcId,
-            !:PredInfo),
+            pred_status(status_local), Markers, ArgTypes0, TypeVarSet,
+            ExistQVars, ClassConstraints, set.init, map.init, GoalType,
+            OldProcInfo, ProcId, !:PredInfo),
 
         % Add the new predicate to the module.
         some [!PredTable] (
@@ -609,7 +621,7 @@ create_inner_proc(RecParConjIds, OldPredProcId, OldProcInfo,
         PredProcId = proc(PredId, ProcId),
 
         % Now transform the predicate. This could not be done earlier because
-        % we needed to know the new PredProcId to re-write the recursive calls
+        % we needed to know the new PredProcId to rewrite the recursive calls
         % in the body.
         proc_info_get_argmodes(OldProcInfo, ArgModes0),
         proc_info_get_headvars(OldProcInfo, HeadVars0),
@@ -635,20 +647,22 @@ create_inner_proc(RecParConjIds, OldPredProcId, OldProcInfo,
         goal_update_non_loop_control_paths(Info, RecParConjIds, _, !Body),
 
         % Now create the new proc_info structure.
+        SeqNum = item_no_seq_num,
         HeadVars = [LCVar | HeadVars0],
         ArgTypes = [loop_control_var_type | ArgTypes0],
-        Ground = ground(shared, none),
-        In = (Ground -> Ground),
+        Ground = ground(shared, none_or_default_func),
+        In = from_to_mode(Ground, Ground),
         ArgModes = [In | ArgModes0],
 
         proc_info_get_inst_varset(OldProcInfo, InstVarSet),
         proc_info_get_rtti_varmaps(OldProcInfo, RttiVarMaps),
         proc_info_get_inferred_determinism(OldProcInfo, Detism),
-        proc_info_create(Context, !.VarSet, !.VarTypes, HeadVars, InstVarSet,
-            ArgModes, detism_decl_none, Detism, !.Body, RttiVarMaps,
-            address_is_not_taken, map.init, ProcInfo),
+        proc_info_create(Context, SeqNum, !.VarSet, !.VarTypes, HeadVars,
+            InstVarSet, ArgModes, detism_decl_none, Detism, !.Body,
+            RttiVarMaps, address_is_not_taken, has_parallel_conj, map.init,
+            ProcInfo),
 
-        % Update the other structures
+        % Update the other structures.
         pred_info_set_arg_types(TypeVarSet, ExistQVars, ArgTypes, !PredInfo),
         pred_info_set_proc_info(ProcId, ProcInfo, !PredInfo),
         module_info_set_pred_info(PredId, !.PredInfo, !ModuleInfo)
@@ -688,7 +702,7 @@ should_preserve_tail_recursion(ModuleInfo, PreserveTailRecursion) :-
     ;       do_not_preserve_tail_recursion.
 
     % Is the current goal the last goal on an execution path through the
-    % procedure.  In other words, can the last goal within the current goal use
+    % procedure. In other words, can the last goal within the current goal use
     % a tailcall?
     %
 :- type goal_is_last_goal_on_path
@@ -714,20 +728,20 @@ goal_loop_control_all_recursive_paths(Info, GoalIds, ContainingGoalMap, !Goal,
 goal_loop_control_one_recursive_path(Info, IsLastGoal, GoalPath0, !Goal,
         !VarSet, !VarTypes) :-
     !.Goal = hlds_goal(GoalExpr0, GoalInfo),
-    ( goal_path_remove_first(GoalPath0, GoalPath, Step) ->
+    ( if goal_path_remove_first(GoalPath0, GoalPath, Step) then
         format("Couldn't follow goal path step: \"%s\"", [s(string(Step))],
             ErrorString),
         (
             Step = step_conj(N),
-            (
+            ( if
                 GoalExpr0 = conj(plain_conj, Conjs0),
                 list.index1(Conjs0, N, Conj0)
-            ->
+            then
                 (
                     IsLastGoal = goal_is_last_goal_on_path,
-                    ( N = length(Conjs0) ->
+                    ( if N = length(Conjs0) then
                         IsLastGoalConj = goal_is_last_goal_on_path
-                    ;
+                    else
                         IsLastGoalConj = goal_is_not_last_goal_on_path
                     )
                 ;
@@ -738,50 +752,50 @@ goal_loop_control_one_recursive_path(Info, IsLastGoal, GoalPath0, !Goal,
                     GoalPath, Conj0, Conj, !VarSet, !VarTypes),
                 det_replace_nth(Conjs0, N, Conj, Conjs),
                 GoalExpr = conj(plain_conj, Conjs)
-            ;
-                unexpected($module, $pred, ErrorString)
+            else
+                unexpected($pred, ErrorString)
             )
         ;
             Step = step_switch(N, _),
-            (
+            ( if
                 GoalExpr0 = switch(Var, CanFail, Cases0),
                 list.index1(Cases0, N, Case0)
-            ->
+            then
                 Goal0 = Case0 ^ case_goal,
                 goal_loop_control_one_recursive_path(Info, IsLastGoal,
                     GoalPath, Goal0, Goal, !VarSet, !VarTypes),
                 Case = Case0 ^ case_goal := Goal,
                 det_replace_nth(Cases0, N, Case, Cases),
                 GoalExpr = switch(Var, CanFail, Cases)
-            ;
-                unexpected($module, $pred, ErrorString)
+            else
+                unexpected($pred, ErrorString)
             )
         ;
             Step = step_ite_then,
-            ( GoalExpr0 = if_then_else(Vars, Cond, Then0, Else) ->
+            ( if GoalExpr0 = if_then_else(Vars, Cond, Then0, Else) then
                 goal_loop_control_one_recursive_path(Info, IsLastGoal,
                     GoalPath, Then0, Then, !VarSet, !VarTypes),
                 GoalExpr = if_then_else(Vars, Cond, Then, Else)
-            ;
-                unexpected($module, $pred, ErrorString)
+            else
+                unexpected($pred, ErrorString)
             )
         ;
             Step = step_ite_else,
-            ( GoalExpr0 = if_then_else(Vars, Cond, Then, Else0) ->
+            ( if GoalExpr0 = if_then_else(Vars, Cond, Then, Else0) then
                 goal_loop_control_one_recursive_path(Info, IsLastGoal,
                     GoalPath, Else0, Else, !VarSet, !VarTypes),
                 GoalExpr = if_then_else(Vars, Cond, Then, Else)
-            ;
-                unexpected($module, $pred, ErrorString)
+            else
+                unexpected($pred, ErrorString)
             )
         ;
             Step = step_scope(_),
-            ( GoalExpr0 = scope(Reason, SubGoal0) ->
+            ( if GoalExpr0 = scope(Reason, SubGoal0) then
                 goal_loop_control_one_recursive_path(Info, IsLastGoal,
                     GoalPath, SubGoal0, SubGoal, !VarSet, !VarTypes),
                 GoalExpr = scope(Reason, SubGoal)
-            ;
-                unexpected($module, $pred, ErrorString)
+            else
+                unexpected($pred, ErrorString)
             )
         ;
             ( Step = step_ite_cond
@@ -792,18 +806,18 @@ goal_loop_control_one_recursive_path(Info, IsLastGoal, GoalPath0, !Goal,
             ; Step = step_atomic_main
             ; Step = step_atomic_orelse(_)
             ),
-            unexpected($module, $pred,
-                format("Unexpected step in goal path \"%s\"",
+            unexpected($pred,
+                string.format("Unexpected step in goal path \"%s\"",
                 [s(string(Step))]))
         ),
         !:Goal = hlds_goal(GoalExpr, GoalInfo),
         fixup_goal_info(Info, !Goal)
-    ;
-        ( GoalExpr0 = conj(parallel_conj, Conjs) ->
+    else
+        ( if GoalExpr0 = conj(parallel_conj, Conjs) then
             par_conj_loop_control(Info, Conjs, IsLastGoal, GoalInfo, !:Goal,
                 !VarSet, !VarTypes)
-        ;
-            unexpected($module, $pred, "expected parallel conjunction")
+        else
+            unexpected($pred, "expected parallel conjunction")
         )
     ).
 
@@ -827,7 +841,7 @@ par_conj_loop_control(Info, Conjuncts0, IsLastGoal, GoalInfo, Goal, !VarSet,
     % the goal_info of Goal0 appropriately. Why call it if we then immediately
     % overwrite the goal_info?
     create_conj_from_list(Conjuncts, plain_conj, Goal0),
-    Goal1 = Goal0 ^ hlds_goal_info := GoalInfo,
+    Goal1 = Goal0 ^ hg_info := GoalInfo,
     fixup_goal_info(Info, Goal1, Goal).
 
     % Process each of the conjuncts, building the new expression from them.
@@ -839,13 +853,13 @@ par_conj_loop_control(Info, Conjuncts0, IsLastGoal, GoalInfo, Goal, !VarSet,
 rewrite_nonrecursive_par_conjuncts(_, _, [], [], !VarSet, !VarTypes).
 rewrite_nonrecursive_par_conjuncts(Info, UseParentStack,
         [Conjunct0 | Conjuncts0], Goals, !VarSet, !VarTypes) :-
-    % Create the "get free slot" call..
+    % Create the "get free slot" call.
     create_get_free_slot_goal(Info, LCSVar, GetFreeSlotGoal,
         !VarSet, !VarTypes),
 
-    % Add a join_and_terminate goal to the end of Conj0 forming Conj.
+    % Add a join_and_terminate goal to the end of Conjunct0 forming Conjunct.
     create_join_and_terminate_goal(Info, LCVar, LCSVar, JoinAndTerminateGoal),
-    Conjunct0GoalInfo = Conjunct0 ^ hlds_goal_info,
+    Conjunct0GoalInfo = Conjunct0 ^ hg_info,
     goal_to_conj_list(Conjunct0, Conjunct0Goals),
     ConjunctGoals = Conjunct0Goals ++ [JoinAndTerminateGoal],
     some [!NonLocals] (
@@ -868,7 +882,7 @@ rewrite_nonrecursive_par_conjuncts(Info, UseParentStack,
         TailGoals, !VarSet, !VarTypes),
     Goals = [GetFreeSlotGoal, ScopeGoal | TailGoals].
 
-    % Re-write any recursive calls in this goal.
+    % Rewrite any recursive calls in this goal.
     %
     % This predicate's argument order does not conform to the Mercury coding
     % standards, this is deliberate as it makes it easier to call from
@@ -876,8 +890,9 @@ rewrite_nonrecursive_par_conjuncts(Info, UseParentStack,
     %
     % UseParentStack is lc_use_parent_stack_frame if, from this goal's
     % perspective it is save to use the parent stack in any spawned off code
-    % running in parallel with this goal.  Otherwise it is
+    % running in parallel with this goal. Otherwise it is
     % lc_create_frame_on_child_stack.
+    %
 :- pred goal_rewrite_recursive_call(loop_control_info::in,
     goal_is_last_goal_on_path::in, hlds_goal::in, hlds_goal::out,
     lc_use_parent_stack::out, fixup_goal_info::out) is det.
@@ -889,7 +904,7 @@ goal_rewrite_recursive_call(Info, IsLastGoal, !Goal, UseParentStack,
         GoalExpr0 = plain_call(CallPredId0, CallProcId0, Args0, Builtin,
             MaybeUnify, _Name0),
         RecPredProcId = Info ^ lci_rec_pred_proc_id,
-        ( RecPredProcId = proc(CallPredId0, CallProcId0) ->
+        ( if RecPredProcId = proc(CallPredId0, CallProcId0) then
             NewPredProcId = Info ^ lci_inner_pred_proc_id,
             proc(CallPredId, CallProcId) = NewPredProcId,
             LCVar = Info ^ lci_lc_var,
@@ -899,14 +914,14 @@ goal_rewrite_recursive_call(Info, IsLastGoal, !Goal, UseParentStack,
                 MaybeUnify, Name),
             PreserveTailRecursion = Info ^ lci_preserve_tail_recursion,
             !:Goal = hlds_goal(GoalExpr, GoalInfo),
-            (
+            ( if
                 IsLastGoal = goal_is_last_goal_on_path,
                 PreserveTailRecursion = preserve_tail_recursion
-            ->
+            then
                 % Create a frame on the child's stack so that the parent can
                 % tail-call.
                 UseParentStack = lc_create_frame_on_child_stack
-            ;
+            else
                 UseParentStack = lc_use_parent_stack_frame,
                 % Inform the code generator that this call may not be a tail
                 % call.
@@ -914,7 +929,7 @@ goal_rewrite_recursive_call(Info, IsLastGoal, !Goal, UseParentStack,
             ),
             fixup_goal_info(Info, !Goal),
             FixupGoalInfo = fixup_goal_info
-        ;
+        else
             UseParentStack = lc_use_parent_stack_frame,
             FixupGoalInfo = do_not_fixup_goal_info
         )
@@ -1006,7 +1021,7 @@ goal_rewrite_recursive_call(Info, IsLastGoal, !Goal, UseParentStack,
         )
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 :- pred case_rewrite_recursive_call(loop_control_info::in,
@@ -1026,9 +1041,9 @@ case_rewrite_recursive_call(Info, IsLastGoal, !Case, UseParentStack,
     is det.
 
 goals_fixup_goal_info(List, Fixup) :-
-    ( list.contains(List, fixup_goal_info) ->
+    ( if list.contains(List, fixup_goal_info) then
         Fixup = fixup_goal_info
-    ;
+    else
         Fixup = do_not_fixup_goal_info
     ).
 
@@ -1055,11 +1070,13 @@ combine_use_parent_stack(lc_create_frame_on_child_stack,
 %----------------------------------------------------------------------------%
 
     % This predicate does two things:
-    %   + It inserts a loop control barrier into the base case(s) of the
-    %     predicate.
-    %   + It re-writes the recursive calls that aren't part of parallel
-    %     conjunctions so that they call the inner procedure and pass the loop
-    %     control variable.
+    %
+    % 1 It inserts a loop control barrier into the base case(s) of the
+    %   predicate.
+    %
+    % 2 It rewrites the recursive calls that aren't part of parallel
+    %   conjunctions so that they call the inner procedure and pass the loop
+    %   control variable.
     %
 :- pred goal_update_non_loop_control_paths(loop_control_info::in,
     list(goal_id)::in, fixup_goal_info::out,
@@ -1067,9 +1084,9 @@ combine_use_parent_stack(lc_create_frame_on_child_stack,
 
 goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
         !Goal) :-
-    GoalInfo0 = !.Goal ^ hlds_goal_info,
+    GoalInfo0 = !.Goal ^ hg_info,
     GoalId = goal_info_get_goal_id(GoalInfo0),
-    (
+    ( if
         % This goal is one of the transformed parallel conjunctions,
         % nothing needs to be done.
         % The last conjunct always recurses, this is inforced by
@@ -1077,26 +1094,28 @@ goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
         % to see how often this happens and if we should handle it.
         % XXX This may not work, I don't know if the goal ID is maintained.
         list.member(GoalId, RecParConjIds)
-    ->
+    then
         FixupGoalInfo = do_not_fixup_goal_info
-    ;
+    else if
         % This goal is a base case, insert the barrier.
-        not ( some [Callee] (
-            goal_calls(!.Goal, Callee),
-            (
-                Callee = Info ^ lci_rec_pred_proc_id
-            ;
-                Callee = Info ^ lci_inner_pred_proc_id
+        not (
+            some [Callee] (
+                goal_calls(!.Goal, Callee),
+                (
+                    Callee = Info ^ lci_rec_pred_proc_id
+                ;
+                    Callee = Info ^ lci_inner_pred_proc_id
+                )
             )
-        ) )
-    ->
+        )
+    then
         goal_to_conj_list(!.Goal, Conjs0),
         create_finish_loop_control_goal(Info, FinishLCGoal),
         Conjs = Conjs0 ++ [FinishLCGoal],
         conj_list_to_goal(Conjs, GoalInfo0, !:Goal),
         fixup_goal_info(Info, !Goal),
         FixupGoalInfo = fixup_goal_info
-    ;
+    else
         !.Goal = hlds_goal(GoalExpr0, _),
         (
             ( GoalExpr0 = unify(_, _, _, _, _)
@@ -1105,13 +1124,13 @@ goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
             ),
             % These cannot be a recursive call and they cannot be a base case
             % since base cases are detected above.
-            unexpected($module, $pred, "Non-recursive atomic goal")
+            unexpected($pred, "Non-recursive atomic goal")
         ;
             GoalExpr0 = plain_call(PredId, ProcId, Args0, Builtin,
                 MaybeContext, _SymName0),
-            % This can only be a recursive call, it must be re-written
+            % This can only be a recursive call, it must be rewritten
             RecPredProcId = Info ^ lci_rec_pred_proc_id,
-            expect(unify(RecPredProcId, proc(PredId, ProcId)), $module, $pred,
+            expect(unify(RecPredProcId, proc(PredId, ProcId)), $pred,
                 "Expected recursive call"),
             proc(InnerPredId, InnerProcId) = Info ^ lci_inner_pred_proc_id,
             LCVar = Info ^ lci_lc_var,
@@ -1122,14 +1141,13 @@ goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
             FixupGoalInfo = fixup_goal_info
         ;
             GoalExpr0 = conj(ConjType, Conjs0),
-            expect(unify(ConjType, plain_conj), $module, $pred,
-                "parallel conjunction"),
+            expect(unify(ConjType, plain_conj), $pred, "parallel conjunction"),
             conj_update_non_loop_control_paths(Info, RecParConjIds,
                 FixupGoalInfo, Conjs0, Conjs),
             GoalExpr = conj(ConjType, Conjs)
         ;
             GoalExpr0 = disj(_),
-            sorry($module, $pred, "disjunction")
+            sorry($pred, "disjunction")
         ;
             GoalExpr0 = switch(Var, CanFail, Cases0),
             list.map2(case_update_non_loop_control_paths(Info, RecParConjIds),
@@ -1138,7 +1156,7 @@ goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
             GoalExpr = switch(Var, CanFail, Cases)
         ;
             GoalExpr0 = negation(_),
-            sorry($module, $pred, "negation")
+            sorry($pred, "negation")
         ;
             GoalExpr0 = scope(Reason, SubGoal0),
             goal_update_non_loop_control_paths(Info, RecParConjIds,
@@ -1157,18 +1175,18 @@ goal_update_non_loop_control_paths(Info, RecParConjIds, FixupGoalInfo,
             GoalExpr = if_then_else(ExistVars, Cond, Then, Else)
         ;
             GoalExpr0 = shorthand(_),
-            unexpected($module, $pred, "shorthand")
+            unexpected($pred, "shorthand")
         ),
-        !Goal ^ hlds_goal_expr := GoalExpr,
+        !Goal ^ hg_expr := GoalExpr,
         (
             FixupGoalInfo = fixup_goal_info,
             some [!NonLocals, !GoalInfo] (
-                !:GoalInfo = !.Goal ^ hlds_goal_info,
+                !:GoalInfo = !.Goal ^ hg_info,
                 !:NonLocals = goal_info_get_nonlocals(!.GoalInfo),
                 set_of_var.insert(Info ^ lci_lc_var, !NonLocals),
                 goal_info_set_nonlocals(!.NonLocals, !GoalInfo),
                 goal_info_set_purity(purity_impure, !GoalInfo),
-                !Goal ^ hlds_goal_info := !.GoalInfo
+                !Goal ^ hg_info := !.GoalInfo
             )
         ;
             FixupGoalInfo = do_not_fixup_goal_info
@@ -1185,7 +1203,7 @@ conj_update_non_loop_control_paths(_Info, _RecGoalIds, do_not_fixup_goal_info,
         [], []).
 conj_update_non_loop_control_paths(Info, RecGoalIds, FixupGoalInfo,
         [Conj0 | Conjs0], [Conj | Conjs]) :-
-    (
+    ( if
         not goal_calls(Conj0, Callee),
         (
             % XXX At the moment, we require that all recursive calls be
@@ -1196,13 +1214,13 @@ conj_update_non_loop_control_paths(Info, RecGoalIds, FixupGoalInfo,
         ;
             Callee = Info ^ lci_inner_pred_proc_id
         )
-    ->
+    then
         % Conj0 does not make a recursive call or contain a recursive
         % parallel conjunction. We don't need to transform it.
         Conj = Conj0,
         conj_update_non_loop_control_paths(Info, RecGoalIds, FixupGoalInfo,
             Conjs0, Conjs)
-    ;
+    else
         % This Conj has something that needs to be transformed.
         goal_update_non_loop_control_paths(Info, RecGoalIds, FixupGoalInfo,
             Conj0, Conj),
@@ -1232,17 +1250,16 @@ case_update_non_loop_control_paths(Info, RecParConjIds, !Case,
     hlds_goal::out, prog_varset::in, prog_varset::out,
     vartypes::in, vartypes::out) is det.
 
-create_get_free_slot_goal(Info, LCSVar, Goal, !VarSet,
-        !VarTypes) :-
+create_get_free_slot_goal(Info, LCSVar, Goal, !VarSet, !VarTypes) :-
     varset.new_named_var("LCS", LCSVar, !VarSet),
     add_var_type(LCSVar, loop_control_slot_var_type, !VarTypes),
     LCVar = Info ^ lci_lc_var,
     proc(PredId, ProcId) = Info ^ lci_wait_free_slot_proc,
     SymName = Info ^ lci_wait_free_slot_proc_name,
 
-    GoalExpr = plain_call(PredId, ProcId, [LCVar, LCSVar], not_builtin, no,
-        SymName),
-    NonLocals = list_to_set([LCVar, LCSVar]),
+    GoalExpr = plain_call(PredId, ProcId, [LCVar, LCSVar], not_builtin,
+        maybe.no, SymName),
+    NonLocals = set_of_var.list_to_set([LCVar, LCSVar]),
     InstmapDelta = instmap_delta_bind_var(LCSVar),
     GoalInfo = impure_init_goal_info(NonLocals, InstmapDelta, detism_det),
     Goal = hlds_goal(GoalExpr, GoalInfo).
@@ -1273,9 +1290,9 @@ create_join_and_terminate_goal(Info, LCVar, LCSVar, Goal) :-
     proc(PredId, ProcId) = Info ^ lci_join_and_terminate_proc,
     SymName = Info ^ lci_join_and_terminate_proc_name,
 
-    GoalExpr = plain_call(PredId, ProcId, [LCVar, LCSVar], not_builtin, no,
-        SymName),
-    NonLocals = list_to_set([LCVar, LCSVar]),
+    GoalExpr = plain_call(PredId, ProcId, [LCVar, LCSVar], not_builtin,
+        maybe.no, SymName),
+    NonLocals = set_of_var.list_to_set([LCVar, LCSVar]),
     instmap_delta_init_reachable(InstmapDelta),
     GoalInfo = impure_init_goal_info(NonLocals, InstmapDelta, detism_det),
     Goal = hlds_goal(GoalExpr, GoalInfo).
@@ -1289,9 +1306,9 @@ create_finish_loop_control_goal(Info, Goal) :-
     get_lc_finish_loop_control_proc(Info ^ lci_module_info, PredId, ProcId),
     LCVar = Info ^ lci_lc_var,
 
-    GoalExpr = plain_call(PredId, ProcId, [LCVar], not_builtin, no,
+    GoalExpr = plain_call(PredId, ProcId, [LCVar], not_builtin, maybe.no,
         lc_finish_loop_control_name),
-    NonLocals = list_to_set([LCVar]),
+    NonLocals = set_of_var.list_to_set([LCVar]),
     instmap_delta_init_reachable(InstmapDelta),
     GoalInfo = impure_init_goal_info(NonLocals, InstmapDelta, detism_det),
     Goal = hlds_goal(GoalExpr, GoalInfo).
@@ -1334,15 +1351,15 @@ update_outer_proc(PredProcId, InnerPredProcId, InnerPredName, ModuleInfo,
     proc_info_get_headvars(!.ProcInfo, HeadVars0),
     proc_info_get_inferred_determinism(!.ProcInfo, Detism),
     proc_info_get_goal(!.ProcInfo, OrigGoal),
-    OrigInstmapDelta = goal_info_get_instmap_delta(OrigGoal ^ hlds_goal_info),
+    OrigInstmapDelta = goal_info_get_instmap_delta(OrigGoal ^ hg_info),
     some [!VarSet, !VarTypes] (
         % Re-build the variables in the procedure with smaller sets.
         varset.init(!:VarSet),
         init_vartypes(!:VarTypes),
         proc_info_get_varset(!.ProcInfo, OldVarSet),
-        foldl3_corresponding(add_old_var_to_sets(OldVarSet), HeadVars0,
+        list.foldl3_corresponding(add_old_var_to_sets(OldVarSet), HeadVars0,
             HeadVarTypes, !VarSet, !VarTypes, map.init, Remap),
-        map(map.lookup(Remap), HeadVars0, HeadVars),
+        list.map(map.lookup(Remap), HeadVars0, HeadVars),
         proc_info_set_headvars(HeadVars, !ProcInfo),
 
         % Fix rtti varmaps.
@@ -1356,11 +1373,11 @@ update_outer_proc(PredProcId, InnerPredProcId, InnerPredName, ModuleInfo,
         % (for auto-parallelisation), but for now we just set it using
         % a runtime call so that it can be tuned.
         varset.new_named_var("NumContexts", NumContextsVar, !VarSet),
-        add_var_type(NumContextsVar, builtin_type(builtin_type_int),
-            !VarTypes),
+        add_var_type(NumContextsVar,
+            builtin_type(builtin_type_int(int_type_int)), !VarTypes),
         get_lc_default_num_contexts_proc(ModuleInfo,
             LCDefaultNumContextsPredId, LCDefaultNumContextsProcId),
-        goal_info_init(list_to_set([NumContextsVar]),
+        goal_info_init(set_of_var.list_to_set([NumContextsVar]),
             instmap_delta_bind_var(NumContextsVar),
             detism_det, purity_pure, GetNumContextsGoalInfo),
         GetNumContextsGoal = hlds_goal(plain_call(LCDefaultNumContextsPredId,
@@ -1374,7 +1391,7 @@ update_outer_proc(PredProcId, InnerPredProcId, InnerPredName, ModuleInfo,
 
         % Create the inner call.
         InnerCallArgs = [LCVar | HeadVars],
-        NonLocals = list_to_set(InnerCallArgs),
+        NonLocals = set_of_var.list_to_set(InnerCallArgs),
         % The instmap of the call to the transformed body has the same instmap
         % delta as the original body.
         remap_instmap(Remap, OrigInstmapDelta, InstmapDelta),
@@ -1386,13 +1403,13 @@ update_outer_proc(PredProcId, InnerPredProcId, InnerPredName, ModuleInfo,
             InnerProcCallGoalInfo),
 
         % Build a conjunction of these goals.
-        goal_info_init(list_to_set(HeadVars), InstmapDelta, Detism,
+        goal_info_init(set_of_var.list_to_set(HeadVars), InstmapDelta, Detism,
             purity_impure, ConjGoalInfo),
         ConjGoal = hlds_goal(conj(plain_conj,
                 [GetNumContextsGoal, LCCreateGoal, InnerProcCallGoal]),
             ConjGoalInfo),
 
-        OrigPurity = goal_info_get_purity(OrigGoal ^ hlds_goal_info),
+        OrigPurity = goal_info_get_purity(OrigGoal ^ hg_info),
         (
             OrigPurity = purity_impure,
             % The impurity introduced by this transformation does not need
@@ -1419,9 +1436,9 @@ update_outer_proc(PredProcId, InnerPredProcId, InnerPredName, ModuleInfo,
 
 add_old_var_to_sets(OldVarSet, OldVar, VarType, !VarSet, !VarTypes,
         !Remap) :-
-    ( varset.search_name(OldVarSet, OldVar, Name) ->
+    ( if varset.search_name(OldVarSet, OldVar, Name) then
         varset.new_named_var(Name, Var, !VarSet)
-    ;
+    else
         varset.new_var(Var, !VarSet)
     ),
     add_var_type(Var, VarType, !VarTypes),
@@ -1433,7 +1450,8 @@ add_old_var_to_sets(OldVarSet, OldVar, VarType, !VarSet, !VarTypes,
 remap_instmap(Remap, OldInstmapDelta, !:InstmapDelta) :-
     instmap_delta_to_assoc_list(OldInstmapDelta, VarInsts),
     instmap_delta_init_reachable(!:InstmapDelta),
-    foldl((pred((OldVar - Inst)::in, IMD0::in, IMD::out) is det :-
+    list.foldl(
+        (pred((OldVar - Inst)::in, IMD0::in, IMD::out) is det :-
             map.lookup(Remap, OldVar, Var),
             instmap_delta_set_var(Var, Inst, IMD0, IMD)
         ), VarInsts, !InstmapDelta).
@@ -1447,7 +1465,7 @@ loop_control_var_type = defined_type(Sym, [], kind_star) :-
 
 :- func loop_control_slot_var_type = mer_type.
 
-loop_control_slot_var_type = builtin_type(builtin_type_int).
+loop_control_slot_var_type = builtin_type(builtin_type_int(int_type_int)).
 
 %----------------------------------------------------------------------------%
 
