@@ -1,25 +1,27 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 1998-2001, 2003-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: pd_info.m.
 % Main author: stayl.
 %
 % Types for deforestation.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module transform_hlds.pd_info.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.instmap.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module transform_hlds.pd_term.
 
@@ -31,7 +33,7 @@
 :- import_module pair.
 :- import_module set.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type pd_info
     --->    pd_info(
@@ -85,6 +87,8 @@
     pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_unfold_info(unfold_info::in,
     pd_info::in, pd_info::out) is det.
+:- pred pd_info_unset_unfold_info(
+    pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_goal_version_index(goal_version_index::in,
     pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_versions(version_index::in,
@@ -109,29 +113,27 @@
 :- pred pd_info_bind_var_to_functors(prog_var::in,
     cons_id::in, list(cons_id)::in, pd_info::in, pd_info::out) is det.
 
-:- pred pd_info_unset_unfold_info(pd_info::in, pd_info::out) is det.
-
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.inst_match.
-:- import_module hlds.hlds_goal.
-:- import_module hlds.hlds_pred.
+:- import_module check_hlds.modecheck_util.
+:- import_module hlds.vartypes.
+:- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
-:- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_util.
 :- import_module parse_tree.set_of_var.
 :- import_module transform_hlds.pd_debug.
 :- import_module transform_hlds.pd_util.
 
-:- import_module bool.
 :- import_module int.
 :- import_module require.
 :- import_module term.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_info_init(ModuleInfo, ProcArgInfos, PDInfo) :-
     map.init(GoalVersionIndex),
@@ -146,12 +148,14 @@ pd_info_init(ModuleInfo, ProcArgInfos, PDInfo) :-
 
 pd_info_init_unfold_info(PredProcId, PredInfo, ProcInfo, !PDInfo) :-
     pd_info_get_module_info(!.PDInfo, ModuleInfo),
-    proc_info_get_initial_instmap(ProcInfo, ModuleInfo, InstMap),
+    proc_info_get_argmodes(ProcInfo, ArgModes),
+    get_constrained_inst_vars(ModuleInfo, ArgModes, HeadInstVars),
+    proc_info_get_initial_instmap(ModuleInfo, ProcInfo, InstMap),
     CostDelta = 0,
     pd_term.local_term_info_init(LocalTermInfo),
     Parents = set.make_singleton_set(PredProcId),
-    UnfoldInfo = unfold_info(ProcInfo, InstMap, CostDelta, LocalTermInfo,
-        PredInfo, Parents, PredProcId, no, 0, no),
+    UnfoldInfo = unfold_info(ProcInfo, HeadInstVars, InstMap, CostDelta,
+        LocalTermInfo, PredInfo, Parents, PredProcId, 0, no, no),
     pd_info_set_unfold_info(UnfoldInfo, !PDInfo).
 
 pd_info_get_module_info(PDInfo, PDInfo ^ pdi_module_info).
@@ -161,7 +165,7 @@ pd_info_get_unfold_info(PDInfo, UnfoldInfo) :-
         MaybeUnfoldInfo = yes(UnfoldInfo)
     ;
         MaybeUnfoldInfo = no,
-        unexpected($module, $pred, "unfold_info not set")
+        unexpected($pred, "unfold_info not set")
     ).
 pd_info_get_goal_version_index(PDInfo, PDInfo ^ pdi_goal_version_index).
 pd_info_get_versions(PDInfo, PDInfo ^ pdi_versions).
@@ -201,7 +205,7 @@ pd_info_set_useless_versions(Versions, !PDInfo) :-
 pd_info_update_goal(hlds_goal(_, GoalInfo), !PDInfo) :-
     pd_info_get_instmap(!.PDInfo, InstMap0),
     Delta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(InstMap0, Delta, InstMap),
+    apply_instmap_delta(Delta, InstMap0, InstMap),
     pd_info_set_instmap(InstMap, !PDInfo).
 
 pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
@@ -215,7 +219,7 @@ pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
     pd_info_set_instmap(InstMap, !PDInfo),
     pd_info_set_module_info(ModuleInfo, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- interface.
 
@@ -224,6 +228,7 @@ pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
 :- type unfold_info
     --->    unfold_info(
                 ufi_proc_info       :: proc_info,
+                ufi_head_inst_vars  :: map(inst_var, mer_inst),
                 ufi_instmap         :: instmap,
 
                 % Improvement in cost measured while processing this procedure.
@@ -239,12 +244,12 @@ pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
                 % Current pred_proc_id.
                 ufi_pred_proc_id    :: pred_proc_id,
 
-                % Has anything changed?
-                ufi_changed         :: bool,
-
                 % Increase in size measured while processing this procedure.
                 ufi_size_delta      :: int,
 
+                % Has anything changed?
+                ufi_changed         :: bool,
+
                 % Does determinism analysis need to be rerun.
                 ufi_rerun_det       :: bool
             ).
@@ -275,14 +280,16 @@ pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
 :- type branch_info_map(T)  ==  map(T, set(int)).
 
 :- pred pd_info_get_proc_info(pd_info::in, proc_info::out) is det.
+:- pred pd_info_get_head_inst_vars(pd_info::in, map(inst_var, mer_inst)::out)
+    is det.
 :- pred pd_info_get_instmap(pd_info::in, instmap::out) is det.
 :- pred pd_info_get_cost_delta(pd_info::in, int::out) is det.
 :- pred pd_info_get_local_term_info(pd_info::in, local_term_info::out) is det.
 :- pred pd_info_get_pred_info(pd_info::in, pred_info::out) is det.
 :- pred pd_info_get_parents(pd_info::in, set(pred_proc_id)::out) is det.
 :- pred pd_info_get_pred_proc_id(pd_info::in, pred_proc_id::out) is det.
-:- pred pd_info_get_changed(pd_info::in, bool::out) is det.
 :- pred pd_info_get_size_delta(pd_info::in, int::out) is det.
+:- pred pd_info_get_changed(pd_info::in, bool::out) is det.
 :- pred pd_info_get_rerun_det(pd_info::in, bool::out) is det.
 
 :- pred pd_info_set_proc_info(proc_info::in,
@@ -299,23 +306,26 @@ pd_info_bind_var_to_functors(Var, MainConsId, OtherConsIds, !PDInfo) :-
     pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_pred_proc_id(pred_proc_id::in,
     pd_info::in, pd_info::out) is det.
-:- pred pd_info_set_changed(bool::in,
-    pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_size_delta(int::in,
     pd_info::in, pd_info::out) is det.
+:- pred pd_info_set_changed(bool::in,
+    pd_info::in, pd_info::out) is det.
 :- pred pd_info_set_rerun_det(bool::in,
     pd_info::in, pd_info::out) is det.
+
 :- pred pd_info_incr_cost_delta(int::in,
     pd_info::in, pd_info::out) is det.
 :- pred pd_info_incr_size_delta(int::in,
     pd_info::in, pd_info::out) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 pd_info_get_proc_info(PDInfo, UnfoldInfo ^ ufi_proc_info) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
+pd_info_get_head_inst_vars(PDInfo, UnfoldInfo ^ ufi_head_inst_vars) :-
+    pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 pd_info_get_instmap(PDInfo, UnfoldInfo ^ ufi_instmap) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 pd_info_get_cost_delta(PDInfo, UnfoldInfo ^ ufi_cost_delta) :-
@@ -328,10 +338,10 @@ pd_info_get_parents(PDInfo, UnfoldInfo ^ ufi_parents) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 pd_info_get_pred_proc_id(PDInfo, UnfoldInfo ^ ufi_pred_proc_id) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
-pd_info_get_changed(PDInfo, UnfoldInfo ^ ufi_changed) :-
-    pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 pd_info_get_size_delta(PDInfo, UnfoldInfo ^ ufi_size_delta) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
+pd_info_get_changed(PDInfo, UnfoldInfo ^ ufi_changed) :-
+    pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 pd_info_get_rerun_det(PDInfo, UnfoldInfo ^ ufi_rerun_det) :-
     pd_info_get_unfold_info(PDInfo, UnfoldInfo).
 
@@ -363,14 +373,14 @@ pd_info_set_pred_proc_id(PredProcId, !PDInfo) :-
     pd_info_get_unfold_info(!.PDInfo, UnfoldInfo0),
     UnfoldInfo = UnfoldInfo0 ^ ufi_pred_proc_id := PredProcId,
     pd_info_set_unfold_info(UnfoldInfo, !PDInfo).
-pd_info_set_changed(Changed, !PDInfo) :-
-    pd_info_get_unfold_info(!.PDInfo, UnfoldInfo0),
-    UnfoldInfo = UnfoldInfo0 ^ ufi_changed := Changed,
-    pd_info_set_unfold_info(UnfoldInfo, !PDInfo).
 pd_info_set_size_delta(SizeDelta, !PDInfo) :-
     pd_info_get_unfold_info(!.PDInfo, UnfoldInfo0),
     UnfoldInfo = UnfoldInfo0 ^ ufi_size_delta := SizeDelta,
     pd_info_set_unfold_info(UnfoldInfo, !PDInfo).
+pd_info_set_changed(Changed, !PDInfo) :-
+    pd_info_get_unfold_info(!.PDInfo, UnfoldInfo0),
+    UnfoldInfo = UnfoldInfo0 ^ ufi_changed := Changed,
+    pd_info_set_unfold_info(UnfoldInfo, !PDInfo).
 pd_info_set_rerun_det(Rerun, !PDInfo) :-
     pd_info_get_unfold_info(!.PDInfo, UnfoldInfo0),
     UnfoldInfo = UnfoldInfo0 ^ ufi_rerun_det := Rerun,
@@ -386,8 +396,8 @@ pd_info_incr_size_delta(Delta1, !PDInfo) :-
     Delta = Delta0 + Delta1,
     pd_info_set_size_delta(Delta, !PDInfo).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- interface.
 
@@ -468,7 +478,7 @@ pd_info_incr_size_delta(Delta1, !PDInfo) :-
                 version_source      :: maybe(pred_proc_id)
             ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -483,20 +493,20 @@ pd_info.search_version(PDInfo, Goal, MaybeVersion) :-
     pd_info_get_proc_info(PDInfo, ProcInfo),
     pd_info_get_instmap(PDInfo, InstMap),
     proc_info_get_vartypes(ProcInfo, VarTypes),
-    (
+    ( if
         map.search(GoalVersionIndex, CalledPreds, VersionIds),
         pd_info.get_matching_version(ModuleInfo, Goal, InstMap,
             VarTypes, VersionIds, Versions, MaybeVersion0)
-    ->
+    then
         MaybeVersion = MaybeVersion0
-    ;
+    else
         MaybeVersion = no_version
     ),
     trace [io(!IO)] (
         pd_debug_search_version_result(PDInfo, MaybeVersion, !IO)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred pd_info.get_matching_version(module_info::in, hlds_goal::in,
     instmap::in, vartypes::in, list(pred_proc_id)::in,
@@ -508,11 +518,11 @@ pd_info.get_matching_version(ModuleInfo, ThisGoal, ThisInstMap, VarTypes,
     map.lookup(Versions, VersionId, Version),
     Version = version_info(OldGoal, _, OldArgs, OldArgTypes,
         OldInstMap, _, _, _, _),
-    (
+    ( if
         pd_info.goal_is_more_general(ModuleInfo, OldGoal, OldInstMap, OldArgs,
             OldArgTypes, ThisGoal, ThisInstMap, VarTypes, VersionId, Version,
             MaybeVersion1)
-    ->
+    then
         (
             MaybeVersion1 = no_version,
             pd_info.get_matching_version(ModuleInfo, ThisGoal, ThisInstMap,
@@ -529,12 +539,12 @@ pd_info.get_matching_version(ModuleInfo, ThisGoal, ThisInstMap, VarTypes,
             pd_info.pick_version(ModuleInfo, PredProcId, Renaming,
                 TypeSubn, MoreGeneralVersion, MaybeVersion2, MaybeVersion)
         )
-    ;
+    else
         pd_info.get_matching_version(ModuleInfo, ThisGoal, ThisInstMap,
             VarTypes, VersionIds, Versions, MaybeVersion)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Choose between two versions.
     %
@@ -553,15 +563,15 @@ pd_info.pick_version(_ModuleInfo, PredProcId1, Renaming1, TSubn1, Version1,
     Version1 = version_info(_, _, _, _, _, _, CostDelta1, _, _),
     Version2 = version_info(_, _, _, _, _, _, CostDelta2, _, _),
     % Select the version with the biggest decrease in cost.
-    ( CostDelta1 > CostDelta2 ->
+    ( if CostDelta1 > CostDelta2 then
         MaybeVersion = version(more_general, PredProcId1,
             Version1, Renaming1, TSubn1)
-    ;
+    else
         MaybeVersion = version(more_general, PredProcId2,
             Version2, Renaming2, TSubn2)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % The aim of this is to check whether the first goal can be used
     % instead of the second if specialisation on the second goal does
@@ -598,7 +608,7 @@ pd_info.goal_is_more_general(ModuleInfo, OldGoal, OldInstMap, OldArgs,
     MaybeVersion = version(Exact, PredProcId, Version,
         OldNewRenaming, TypeRenaming).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Check that all the insts in the old version are at least as
     % general as the insts in the new version.
@@ -614,15 +624,17 @@ pd_info.check_insts(ModuleInfo, [OldVar | Vars], VarRenaming, OldInstMap,
     map.lookup(VarRenaming, OldVar, NewVar),
     instmap_lookup_var(NewInstMap, NewVar, NewVarInst),
     lookup_var_type(VarTypes, NewVar, Type),
-    inst_matches_initial(NewVarInst, OldVarInst, Type, ModuleInfo),
+    inst_matches_initial(ModuleInfo, Type, NewVarInst, OldVarInst),
     (
         !.ExactSoFar = exact,
         % Does inst_matches_initial(Inst1, Inst2, M) and
         % inst_matches_initial(Inst2, Inst1, M) imply that Inst1
         % and Inst2 are interchangable?
-        ( inst_matches_initial(OldVarInst, NewVarInst, Type, ModuleInfo) ->
+        ( if
+            inst_matches_initial(ModuleInfo, Type, OldVarInst, NewVarInst)
+        then
             !:ExactSoFar = exact
-        ;
+        else
             !:ExactSoFar = more_general
         )
     ;
@@ -631,7 +643,7 @@ pd_info.check_insts(ModuleInfo, [OldVar | Vars], VarRenaming, OldInstMap,
     pd_info.check_insts(ModuleInfo, Vars, VarRenaming, OldInstMap,
         NewInstMap, VarTypes, !ExactSoFar).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_info.define_new_pred(Origin, Goal, PredProcId, CallGoal, !PDInfo) :-
     pd_info_get_instmap(!.PDInfo, InstMap),
@@ -658,16 +670,17 @@ pd_info.define_new_pred(Origin, Goal, PredProcId, CallGoal, !PDInfo) :-
     proc_info_get_vartypes(ProcInfo, VarTypes),
     proc_info_get_rtti_varmaps(ProcInfo, RttiVarMaps),
     proc_info_get_inst_varset(ProcInfo, InstVarSet),
+    proc_info_get_has_parallel_conj(ProcInfo, HasParallelConj),
     proc_info_get_var_name_remap(ProcInfo, VarNameRemap),
     % XXX handle the extra typeinfo arguments for
     % --typeinfo-liveness properly.
     hlds_pred.define_new_pred(Origin, Goal, CallGoal, Args, _ExtraArgs,
         InstMap, SymName, TVarSet, VarTypes, ClassContext, RttiVarMaps,
-        VarSet, InstVarSet, Markers, address_is_not_taken, VarNameRemap,
-        ModuleInfo0, ModuleInfo, PredProcId),
+        VarSet, InstVarSet, Markers, address_is_not_taken, HasParallelConj,
+        VarNameRemap, ModuleInfo0, ModuleInfo, PredProcId),
     pd_info_set_module_info(ModuleInfo, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_info.register_version(PredProcId, Version, !PDInfo) :-
     trace [io(!IO)] (
@@ -676,10 +689,10 @@ pd_info.register_version(PredProcId, Version, !PDInfo) :-
     pd_info_get_goal_version_index(!.PDInfo, GoalVersionIndex0),
     Goal = Version ^ version_orig_goal,
     pd_util.goal_get_calls(Goal, Calls),
-    ( map.search(GoalVersionIndex0, Calls, VersionList0) ->
+    ( if map.search(GoalVersionIndex0, Calls, VersionList0) then
         VersionList = [PredProcId | VersionList0],
         map.det_update(Calls, VersionList, GoalVersionIndex0, GoalVersionIndex)
-    ;
+    else
         VersionList = [PredProcId],
         map.det_insert(Calls, VersionList, GoalVersionIndex0, GoalVersionIndex)
     ),
@@ -691,23 +704,23 @@ pd_info.register_version(PredProcId, Version, !PDInfo) :-
     set.insert(PredProcId, CreatedVersions0, CreatedVersions),
     pd_info_set_created_versions(CreatedVersions, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_info.invalidate_version(PredProcId, !PDInfo) :-
     pd_info_get_versions(!.PDInfo, Versions0),
     map.lookup(Versions0, PredProcId, Version),
     Goal = Version ^ version_orig_goal,
     pd_util.goal_get_calls(Goal, Calls),
-    (
+    ( if
         Calls = [FirstCall | _],
         list.last(Calls, LastCall)
-    ->
+    then
         % Make sure we never create another version to deforest
         % this pair of calls.
         pd_info_get_useless_versions(!.PDInfo, Useless0),
         set.insert(FirstCall - LastCall, Useless0, Useless),
         pd_info_set_useless_versions(Useless, !PDInfo)
-    ;
+    else
         true
     ),
     pd_info.remove_version(PredProcId, !PDInfo).
@@ -721,11 +734,11 @@ pd_info.remove_version(PredProcId, !PDInfo) :-
     pd_info_set_versions(Versions, !PDInfo),
 
     pd_info_get_goal_version_index(!.PDInfo, GoalIndex0),
-    ( map.search(GoalIndex0, Calls, GoalVersions0) ->
+    ( if map.search(GoalIndex0, Calls, GoalVersions0) then
         list.delete_all(GoalVersions0, PredProcId, GoalVersions),
         map.det_update(Calls, GoalVersions, GoalIndex0, GoalIndex),
         pd_info_set_goal_version_index(GoalIndex, !PDInfo)
-    ;
+    else
         true
     ),
 
@@ -738,4 +751,6 @@ pd_info.remove_version(PredProcId, !PDInfo) :-
     module_info_remove_predicate(PredId, ModuleInfo0, ModuleInfo),
     pd_info_set_module_info(ModuleInfo, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module transform_hlds.pd_info.
+%---------------------------------------------------------------------------%
