@@ -1,26 +1,32 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 1998-2012 University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File pd_util.m.
 % Main author: stayl.
 %
 % Utility predicates for deforestation and partial evaluation.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module transform_hlds.pd_util.
 :- interface.
 
+:- import_module check_hlds.
 :- import_module check_hlds.mode_errors.
 :- import_module check_hlds.simplify.
+:- import_module check_hlds.simplify.simplify_tasks.
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module hlds.vartypes.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
 :- import_module transform_hlds.pd_info.
@@ -30,7 +36,7 @@
 :- import_module map.
 :- import_module maybe.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Pick out the pred_proc_ids of the calls in a list of atomic goals.
     %
@@ -44,7 +50,7 @@
 
     % Apply simplify.m to the goal.
     %
-:- pred pd_simplify_goal(simplifications::in, hlds_goal::in,
+:- pred pd_simplify_goal(simplify_tasks::in, hlds_goal::in,
     hlds_goal::out, pd_info::in, pd_info::out) is det.
 
     % Apply unique_modes.m to the goal.
@@ -148,27 +154,32 @@
 :- pred pd_can_reorder_goals(module_info::in, bool::in,
     hlds_goal::in, hlds_goal::in) is semidet.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module check_hlds.det_analysis.
 :- import_module check_hlds.det_report.
 :- import_module check_hlds.det_util.
+:- import_module check_hlds.inst_lookup.
 :- import_module check_hlds.inst_match.
-:- import_module check_hlds.inst_util.
+:- import_module check_hlds.inst_test.
 :- import_module check_hlds.mode_info.
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.mode_test.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module check_hlds.unique_modes.
+:- import_module check_hlds.simplify.simplify_proc.
+:- import_module hlds.goal_form.
 :- import_module hlds.goal_util.
 :- import_module hlds.instmap.
 :- import_module hlds.quantification.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_type.
-:- import_module parse_tree.set_of_var.
 :- import_module transform_hlds.constraint.
 :- import_module transform_hlds.pd_debug.
 
@@ -181,22 +192,23 @@
 
 goal_get_calls(Goal0, CalledPreds) :-
     goal_to_conj_list(Goal0, GoalList),
-    GetCalls = (pred(Goal::in, CalledPred::out) is semidet :-
-        Goal = hlds_goal(plain_call(PredId, ProcId, _, _, _, _), _),
-        CalledPred = proc(PredId, ProcId)
-    ),
+    GetCalls =
+        ( pred(Goal::in, CalledPred::out) is semidet :-
+            Goal = hlds_goal(plain_call(PredId, ProcId, _, _, _, _), _),
+            CalledPred = proc(PredId, ProcId)
+        ),
     list.filter_map(GetCalls, GoalList, CalledPreds).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 propagate_constraints(!Goal, !PDInfo) :-
     pd_info_get_module_info(!.PDInfo, ModuleInfo0),
     module_info_get_globals(ModuleInfo0, Globals),
-    globals.lookup_bool_option(Globals, local_constraint_propagation,
-        ConstraintProp),
+    globals.get_opt_tuple(Globals, OptTuple),
+    ConstraintProp = OptTuple ^ ot_prop_local_constraints,
     globals.lookup_bool_option(Globals, debug_pd, DebugPD),
     (
-        ConstraintProp = yes,
+        ConstraintProp = prop_local_constraints,
         Goal0 = !.Goal,
         trace [io(!IO)] (
             pd_debug_message(DebugPD, "%% Propagating constraints\n", [], !IO),
@@ -225,8 +237,9 @@ propagate_constraints(!Goal, !PDInfo) :-
             pd_requantify_goal(NonLocals, !Goal, !PDInfo),
             pd_recompute_instmap_delta(!Goal, !PDInfo),
             rerun_det_analysis(!Goal, !PDInfo),
-            simplify.find_simplifications(no, Globals, Simplifications),
-            pd_simplify_goal(Simplifications, !Goal, !PDInfo)
+            find_simplify_tasks(Globals, do_not_generate_warnings,
+                SimplifyTasks),
+            pd_simplify_goal(SimplifyTasks, !Goal, !PDInfo)
         ;
             % Use Goal0 rather than the output of propagate_constraints_in_goal
             % because constraint propagation can make the quantification
@@ -236,38 +249,25 @@ propagate_constraints(!Goal, !PDInfo) :-
             !:Goal = Goal0
         )
     ;
-        ConstraintProp = no
+        ConstraintProp = do_not_prop_local_constraints
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-pd_simplify_goal(Simplifications, Goal0, Goal, !PDInfo) :-
-    % Construct a simplify_info.
+pd_simplify_goal(SimplifyTasks, !Goal, !PDInfo) :-
     pd_info_get_module_info(!.PDInfo, ModuleInfo0),
     pd_info_get_pred_proc_id(!.PDInfo, proc(PredId, ProcId)),
     pd_info_get_proc_info(!.PDInfo, ProcInfo0),
     pd_info_get_instmap(!.PDInfo, InstMap0),
-    simplify_info_init(ModuleInfo0, PredId, ProcId, ProcInfo0,
-        InstMap0, Simplifications, SimplifyInfo0),
 
-    simplify_process_clause_body_goal(Goal0, Goal,
-        SimplifyInfo0, SimplifyInfo),
+    simplify_goal_update_vars_in_proc(SimplifyTasks, ModuleInfo0, ModuleInfo,
+        PredId, ProcId, ProcInfo0, ProcInfo, InstMap0, !Goal, CostDelta),
 
-    % Deconstruct the simplify_info.
-    simplify_info_get_module_info(SimplifyInfo, ModuleInfo),
-    simplify_info_get_varset(SimplifyInfo, VarSet),
-    simplify_info_get_var_types(SimplifyInfo, VarTypes),
-    simplify_info_get_cost_delta(SimplifyInfo, CostDelta),
-    simplify_info_get_rtti_varmaps(SimplifyInfo, RttiVarMaps),
-    pd_info_get_proc_info(!.PDInfo, ProcInfo1),
-    proc_info_set_varset(VarSet, ProcInfo1, ProcInfo2),
-    proc_info_set_vartypes(VarTypes, ProcInfo2, ProcInfo3),
-    proc_info_set_rtti_varmaps(RttiVarMaps, ProcInfo3, ProcInfo),
+    pd_info_set_module_info(ModuleInfo, !PDInfo),
     pd_info_set_proc_info(ProcInfo, !PDInfo),
-    pd_info_incr_cost_delta(CostDelta, !PDInfo),
-    pd_info_set_module_info(ModuleInfo, !PDInfo).
+    pd_info_incr_cost_delta(CostDelta, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 unique_modecheck_goal(Goal0, Goal, Errors, !PDInfo) :-
     get_goal_live_vars(!.PDInfo, Goal0, LiveVars),
@@ -282,14 +282,16 @@ unique_modecheck_goal_live_vars(LiveVars, Goal0, Goal, Errors, !PDInfo) :-
     term.context_init(Context),
     pd_info_get_pred_info(!.PDInfo, PredInfo0),
     pd_info_get_proc_info(!.PDInfo, ProcInfo0),
+    pd_info_get_head_inst_vars(!.PDInfo, HeadInstVars),
     module_info_set_pred_proc_info(PredId, ProcId, PredInfo0, ProcInfo0,
         ModuleInfo0, ModuleInfo1),
 
     % If we perform generalisation, we shouldn't change any called procedures,
     % since that could cause a less efficient version to be chosen.
     MayChangeCalledProc = may_not_change_called_proc,
-    mode_info_init(ModuleInfo1, PredId, ProcId, Context, LiveVars, InstMap0,
-        check_unique_modes, MayChangeCalledProc, ModeInfo0),
+    mode_info_init(ModuleInfo1, PredId, ProcId, Context, LiveVars,
+        HeadInstVars, InstMap0, check_unique_modes, MayChangeCalledProc,
+        ModeInfo0),
 
     unique_modes_check_goal(Goal0, Goal, ModeInfo0, ModeInfo),
     mode_info_get_module_info(ModeInfo, ModuleInfo),
@@ -300,8 +302,7 @@ unique_modecheck_goal_live_vars(LiveVars, Goal0, Goal, Errors, !PDInfo) :-
         Debug = yes,
         trace [io(!IO)] (
             ErrorSpecs = list.map(mode_error_info_to_spec(ModeInfo), Errors),
-            write_error_specs(ErrorSpecs, Globals,
-                0, _NumWarnings, 0, _NumErrors, !IO)
+            write_error_specs(Globals, ErrorSpecs, !IO)
         )
     ;
         Debug = no
@@ -341,19 +342,19 @@ get_goal_live_vars(PDInfo, hlds_goal(_, GoalInfo), !:Vars) :-
 get_goal_live_vars_2(_, [], _, _, !Vars).
 get_goal_live_vars_2(ModuleInfo, [NonLocal | NonLocals],
         InstMap, InstMapDelta, !Vars) :-
-    ( instmap_delta_search_var(InstMapDelta, NonLocal, FinalInst0) ->
+    ( if instmap_delta_search_var(InstMapDelta, NonLocal, FinalInst0) then
         FinalInst = FinalInst0
-    ;
+    else
         instmap_lookup_var(InstMap, NonLocal, FinalInst)
     ),
-    ( inst_is_clobbered(ModuleInfo, FinalInst) ->
+    ( if inst_is_clobbered(ModuleInfo, FinalInst) then
         true
-    ;
+    else
         set_of_var.insert(NonLocal, !Vars)
     ),
     get_goal_live_vars_2(ModuleInfo, NonLocals, InstMap, InstMapDelta, !Vars).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred rerun_det_analysis(hlds_goal::in, hlds_goal::out,
     pd_info::in, pd_info::out) is det.
@@ -366,15 +367,16 @@ rerun_det_analysis(Goal0, Goal, !PDInfo) :-
 
     % det_infer_goal looks up the proc_info in the module_info for the
     % vartypes, so we'd better stick them back in the module_info.
-    pd_info_get_pred_proc_id(!.PDInfo, proc(PredId, ProcId)),
+    pd_info_get_pred_proc_id(!.PDInfo, PredProcId),
     pd_info_get_pred_info(!.PDInfo, PredInfo),
     pd_info_get_proc_info(!.PDInfo, ProcInfo),
     pd_info_get_module_info(!.PDInfo, ModuleInfo0),
-    module_info_set_pred_proc_info(PredId, ProcId, PredInfo, ProcInfo,
+    module_info_set_pred_proc_info(PredProcId, PredInfo, ProcInfo,
         ModuleInfo0, ModuleInfo1),
 
+    proc_info_get_varset(ProcInfo, VarSet),
     proc_info_get_vartypes(ProcInfo, VarTypes),
-    det_info_init(ModuleInfo1, VarTypes, PredId, ProcId,
+    det_info_init(ModuleInfo1, PredProcId, VarSet, VarTypes,
         pess_extra_vars_ignore, [], DetInfo0),
     pd_info_get_instmap(!.PDInfo, InstMap),
     det_infer_goal(Goal0, Goal, InstMap, SolnContext, [], no, _, _,
@@ -387,36 +389,9 @@ rerun_det_analysis(Goal0, Goal, !PDInfo) :-
     module_info_get_globals(ModuleInfo, Globals),
     disable_det_warnings(_OptionsToRestore, Globals, GlobalsToUse),
     ContainsErrors = contains_errors(GlobalsToUse, Specs),
-    expect(unify(ContainsErrors, no), $module, $pred, "determinism errors").
+    expect(unify(ContainsErrors, no), $pred, "determinism errors").
 
-%-----------------------------------------------------------------------------%
-
-convert_branch_info(ArgInfo, Args, VarInfo) :-
-    ArgInfo = pd_branch_info(ArgMap, LeftArgs, OpaqueArgs),
-    map.to_assoc_list(ArgMap, ArgList),
-    map.init(BranchVarMap0),
-    convert_branch_info_2(ArgList, Args, BranchVarMap0, BranchVarMap),
-
-    set.to_sorted_list(LeftArgs, LeftArgNos),
-    list.map(list.det_index1(Args), LeftArgNos, LeftVars0),
-    set.list_to_set(LeftVars0, LeftVars),
-
-    set.to_sorted_list(OpaqueArgs, OpaqueArgNos),
-    list.map(list.det_index1(Args), OpaqueArgNos, OpaqueVars0),
-    set.list_to_set(OpaqueVars0, OpaqueVars),
-
-    VarInfo = pd_branch_info(BranchVarMap, LeftVars, OpaqueVars).
-
-:- pred convert_branch_info_2(assoc_list(int, set(int))::in,
-    prog_vars::in, pd_var_info::in, pd_var_info::out) is det.
-
-convert_branch_info_2([], _, !VarInfo).
-convert_branch_info_2([ArgNo - Branches | ArgInfos], Args, !VarInfo) :-
-    list.det_index1(Args, ArgNo, Arg),
-    map.set(Arg, Branches, !VarInfo),
-    convert_branch_info_2(ArgInfos, Args, !VarInfo).
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type pd_var_info ==  branch_info_map(prog_var).
 
@@ -427,10 +402,10 @@ get_branch_vars_proc(PredProcId, ProcInfo, !ModuleInfo, !ArgInfo) :-
     map.init(Vars0),
     set_of_var.init(LeftVars0),
     goal_to_conj_list(Goal, GoalList),
-    (
+    ( if
         get_branch_vars_goal_2(!.ModuleInfo, GoalList, no,
             VarTypes, InstMap0, LeftVars0, LeftVars, Vars0, Vars)
-    ->
+    then
         proc_info_get_headvars(ProcInfo, HeadVars),
         map.init(ThisProcArgMap0),
         set.init(ThisProcLeftArgs0),
@@ -456,7 +431,7 @@ get_branch_vars_proc(PredProcId, ProcInfo, !ModuleInfo, !ArgInfo) :-
         BranchInfo = pd_branch_info(ThisProcArgMap, ThisProcLeftArgs,
             OpaqueArgs),
         map.set(PredProcId, BranchInfo, !ArgInfo)
-    ;
+    else
         true
     ).
 
@@ -471,12 +446,12 @@ get_branch_vars_proc(PredProcId, ProcInfo, !ModuleInfo, !ArgInfo) :-
 get_opaque_args(_, _, [], _, !OpaqueArgs).
 get_opaque_args(ModuleInfo, ArgNo, [ArgMode | ArgModes],
         ExtraInfoArgs, !OpaqueArgs) :-
-    (
+    ( if
         mode_is_output(ModuleInfo, ArgMode),
-        \+ map.contains(ExtraInfoArgs, ArgNo)
-    ->
+        not map.contains(ExtraInfoArgs, ArgNo)
+    then
         set.insert(ArgNo, !OpaqueArgs)
-    ;
+    else
         true
     ),
     NextArg = ArgNo + 1,
@@ -495,21 +470,21 @@ get_opaque_args(ModuleInfo, ArgNo, [ArgMode | ArgModes],
 get_extra_info_headvars([], _, _, _, !Args, !LeftArgs).
 get_extra_info_headvars([HeadVar | HeadVars], ArgNo,
         LeftVars, VarInfo, !ThisProcArgs, !ThisProcLeftVars) :-
-    ( map.search(VarInfo, HeadVar, ThisVarInfo) ->
+    ( if map.search(VarInfo, HeadVar, ThisVarInfo) then
         map.det_insert(ArgNo, ThisVarInfo, !ThisProcArgs)
-    ;
+    else
         true
     ),
-    ( set_of_var.member(LeftVars, HeadVar) ->
+    ( if set_of_var.member(LeftVars, HeadVar) then
         set.insert(ArgNo, !ThisProcLeftVars)
-    ;
+    else
         true
     ),
     NextArgNo = ArgNo + 1,
     get_extra_info_headvars(HeadVars, NextArgNo,
         LeftVars, VarInfo, !ThisProcArgs, !ThisProcLeftVars).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 get_branch_vars_goal(Goal, MaybeBranchInfo, !PDInfo) :-
     pd_info_get_module_info(!.PDInfo, ModuleInfo0),
@@ -519,10 +494,10 @@ get_branch_vars_goal(Goal, MaybeBranchInfo, !PDInfo) :-
     proc_info_get_vartypes(ProcInfo, VarTypes),
     set_of_var.init(LeftVars0),
     map.init(Vars0),
-    (
+    ( if
         get_branch_vars_goal_2(ModuleInfo0, [Goal], no,
             VarTypes, InstMap0, LeftVars0, LeftVars, Vars0, Vars1)
-    ->
+    then
         get_sub_branch_vars_goal(ProcArgInfo, [Goal],
             VarTypes, InstMap0, Vars1, Vars,
             ModuleInfo0, ModuleInfo),
@@ -532,7 +507,7 @@ get_branch_vars_goal(Goal, MaybeBranchInfo, !PDInfo) :-
         set.init(OpaqueVars),
         MaybeBranchInfo = yes(pd_branch_info(Vars,
             set_of_var.bitset_to_set(LeftVars), OpaqueVars))
-    ;
+    else
         MaybeBranchInfo = no
     ).
 
@@ -546,8 +521,8 @@ get_branch_vars_goal_2(ModuleInfo, [Goal | Goals], !.FoundBranch,
         VarTypes, InstMap0, !LeftVars, !Vars) :-
     Goal = hlds_goal(_, GoalInfo),
     InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(InstMap0, InstMapDelta, InstMap),
-    ( get_branch_instmap_deltas(Goal, InstMapDeltas) ->
+    apply_instmap_delta(InstMapDelta, InstMap0, InstMap),
+    ( if get_branch_instmap_deltas(Goal, InstMapDeltas) then
         % Only look for goals with one top-level branched goal,
         % since deforestation of goals with more than one is
         % likely to be less productive.
@@ -556,7 +531,7 @@ get_branch_vars_goal_2(ModuleInfo, [Goal | Goals], !.FoundBranch,
             1, !Vars),
         get_left_vars(Goal, !LeftVars),
         !:FoundBranch = yes
-    ;
+    else
         Goal = hlds_goal(GoalExpr, _),
         goal_expr_has_subgoals(GoalExpr) = does_not_have_subgoals
     ),
@@ -580,7 +555,7 @@ get_branch_instmap_deltas(Goal, InstMapDeltas) :-
     ;
         GoalExpr = switch(_, _, Cases),
         GetCaseInstMapDelta =
-            (pred(Case::in, InstMapDelta::out) is det :-
+            ( pred(Case::in, InstMapDelta::out) is det :-
                 Case = case(_, _, hlds_goal(_, CaseInfo)),
                 InstMapDelta = goal_info_get_instmap_delta(CaseInfo)
             ),
@@ -588,7 +563,7 @@ get_branch_instmap_deltas(Goal, InstMapDeltas) :-
     ;
         GoalExpr = disj(Disjuncts),
         GetDisjunctInstMapDelta =
-            (pred(Disjunct::in, InstMapDelta::out) is det :-
+            ( pred(Disjunct::in, InstMapDelta::out) is det :-
                 Disjunct = hlds_goal(_, DisjInfo),
                 InstMapDelta = goal_info_get_instmap_delta(DisjInfo)
             ),
@@ -603,9 +578,9 @@ get_branch_instmap_deltas(Goal, InstMapDeltas) :-
     set_of_progvar::in, set_of_progvar::out) is det.
 
 get_left_vars(Goal, Vars0, Vars) :-
-    ( Goal = hlds_goal(switch(Var, _, _), _) ->
+    ( if Goal = hlds_goal(switch(Var, _, _), _) then
         set_of_var.insert(Var, Vars0, Vars)
-    ;
+    else
         Vars = Vars0
     ).
 
@@ -617,21 +592,21 @@ get_branch_vars(_, _, [], _, _, !ExtraVars).
 get_branch_vars(ModuleInfo, Goal, [InstMapDelta | InstMapDeltas],
         InstMap, BranchNo, !ExtraVars) :-
     AddExtraInfoVars =
-        (pred(ChangedVar::in, Vars0::in, Vars::out) is det :-
-            (
+        ( pred(ChangedVar::in, Vars0::in, Vars::out) is det :-
+            ( if
                 instmap_lookup_var(InstMap, ChangedVar, VarInst),
                 instmap_delta_search_var(InstMapDelta, ChangedVar,
                     DeltaVarInst),
                 inst_is_bound_to_functors(ModuleInfo, DeltaVarInst, [_]),
-                \+ inst_is_bound_to_functors(ModuleInfo, VarInst, [_])
-            ->
-                ( map.search(Vars0, ChangedVar, Set0) ->
+                not inst_is_bound_to_functors(ModuleInfo, VarInst, [_])
+            then
+                ( if map.search(Vars0, ChangedVar, Set0) then
                     set.insert(BranchNo, Set0, Set)
-                ;
+                else
                     Set = set.make_singleton_set(BranchNo)
                 ),
                 map.set(ChangedVar, Set, Vars0, Vars)
-            ;
+            else
                 Vars = Vars0
             )
         ),
@@ -641,14 +616,14 @@ get_branch_vars(ModuleInfo, Goal, [InstMapDelta | InstMapDeltas],
 
     % We have extra information about a switched-on variable
     % at the end of each branch.
-    ( Goal = hlds_goal(switch(SwitchVar, _, _), _) ->
-        ( map.search(!.ExtraVars, SwitchVar, SwitchVarSet0) ->
+    ( if Goal = hlds_goal(switch(SwitchVar, _, _), _) then
+        ( if map.search(!.ExtraVars, SwitchVar, SwitchVarSet0) then
             set.insert(BranchNo, SwitchVarSet0, SwitchVarSet)
-        ;
+        else
             SwitchVarSet = set.make_singleton_set(BranchNo)
         ),
         map.set(SwitchVar, SwitchVarSet, !ExtraVars)
-    ;
+    else
         true
     ),
     NextBranch = BranchNo + 1,
@@ -670,7 +645,7 @@ get_sub_branch_vars_goal(ProcArgInfo, [Goal | GoalList],
         GoalExpr = if_then_else(_, Cond, Then, Else),
         Cond = hlds_goal(_, CondInfo),
         CondDelta = goal_info_get_instmap_delta(CondInfo),
-        instmap.apply_instmap_delta(InstMap0, CondDelta, InstMap1),
+        apply_instmap_delta(CondDelta, InstMap0, InstMap1),
         goal_to_conj_list(Then, ThenList),
         examine_branch(!.ModuleInfo, ProcArgInfo, 1, ThenList,
             VarTypes, InstMap1, !Vars),
@@ -696,10 +671,10 @@ get_sub_branch_vars_goal(ProcArgInfo, [Goal | GoalList],
         )
     ;
         GoalExpr = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ),
     InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(InstMap0, InstMapDelta, InstMap),
+    apply_instmap_delta(InstMapDelta, InstMap0, InstMap),
     get_sub_branch_vars_goal(ProcArgInfo, GoalList,
         VarTypes, InstMap, !.Vars, SubVars, !ModuleInfo).
 
@@ -743,31 +718,33 @@ examine_case_list(ProcArgInfo, BranchNo, Var, [Case | Cases],
 examine_branch(_, _, _, [], _, _, !Vars).
 examine_branch(ModuleInfo, ProcArgInfo, BranchNo, [Goal | Goals],
         VarTypes, InstMap, !Vars) :-
-    (
+    ( if
         Goal = hlds_goal(plain_call(PredId, ProcId, Args, _, _, _), _)
-    ->
-        ( map.search(ProcArgInfo, proc(PredId, ProcId), ThisProcArgInfo) ->
+    then
+        ( if
+            map.search(ProcArgInfo, proc(PredId, ProcId), ThisProcArgInfo)
+        then
             convert_branch_info(ThisProcArgInfo, Args, BranchInfo),
             BranchInfo = pd_branch_info(!:Vars, _, _),
             map.keys(!.Vars, ExtraVars1),
             combine_vars(BranchNo, ExtraVars1, !Vars)
-        ;
+        else
             true
         )
-    ;
+    else if
         set_of_var.init(LeftVars0),
         map.init(!:Vars),
         get_branch_vars_goal_2(ModuleInfo, [Goal], no,
             VarTypes, InstMap, LeftVars0, _, !Vars)
-    ->
+    then
         map.keys(!.Vars, ExtraVars2),
         combine_vars(BranchNo, ExtraVars2, !Vars)
-    ;
+    else
         true
     ),
     Goal = hlds_goal(_, GoalInfo),
     InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
-    instmap.apply_instmap_delta(InstMap, InstMapDelta, InstMap1),
+    apply_instmap_delta(InstMapDelta, InstMap, InstMap1),
     examine_branch(ModuleInfo, ProcArgInfo, BranchNo,
         Goals, VarTypes, InstMap1, !Vars).
 
@@ -776,16 +753,16 @@ examine_branch(ModuleInfo, ProcArgInfo, BranchNo, [Goal | Goals],
 
 combine_vars(_, [], !Vars).
 combine_vars(BranchNo, [ExtraVar | ExtraVars], !Vars) :-
-    ( map.search(!.Vars, ExtraVar, Branches0) ->
+    ( if map.search(!.Vars, ExtraVar, Branches0) then
         set.insert(BranchNo, Branches0, Branches),
         map.det_update(ExtraVar, Branches, !Vars)
-    ;
+    else
         Branches = set.make_singleton_set(BranchNo),
         map.det_insert(ExtraVar, Branches, !Vars)
     ),
     combine_vars(BranchNo, ExtraVars, !Vars).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_requantify_goal(NonLocals, Goal0, Goal, !PDInfo) :-
     some [!ProcInfo] (
@@ -812,7 +789,34 @@ pd_recompute_instmap_delta(Goal0, Goal, !PDInfo) :-
         Goal0, Goal, VarTypes, InstVarSet, InstMap, ModuleInfo0, ModuleInfo),
     pd_info_set_module_info(ModuleInfo, !PDInfo).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
+convert_branch_info(ArgInfo, Args, VarInfo) :-
+    ArgInfo = pd_branch_info(ArgMap, LeftArgs, OpaqueArgs),
+    map.to_assoc_list(ArgMap, ArgList),
+    map.init(BranchVarMap0),
+    convert_branch_info_2(ArgList, Args, BranchVarMap0, BranchVarMap),
+
+    set.to_sorted_list(LeftArgs, LeftArgNos),
+    list.map(list.det_index1(Args), LeftArgNos, LeftVars0),
+    set.list_to_set(LeftVars0, LeftVars),
+
+    set.to_sorted_list(OpaqueArgs, OpaqueArgNos),
+    list.map(list.det_index1(Args), OpaqueArgNos, OpaqueVars0),
+    set.list_to_set(OpaqueVars0, OpaqueVars),
+
+    VarInfo = pd_branch_info(BranchVarMap, LeftVars, OpaqueVars).
+
+:- pred convert_branch_info_2(assoc_list(int, set(int))::in,
+    prog_vars::in, pd_var_info::in, pd_var_info::out) is det.
+
+convert_branch_info_2([], _, !VarInfo).
+convert_branch_info_2([ArgNo - Branches | ArgInfos], Args, !VarInfo) :-
+    list.det_index1(Args, ArgNo, Arg),
+    map.set(Arg, Branches, !VarInfo),
+    convert_branch_info_2(ArgInfos, Args, !VarInfo).
+
+%---------------------------------------------------------------------------%
 
 inst_MSG(InstA, InstB, ModuleInfo, Inst) :-
     set.init(Expansions),
@@ -824,18 +828,18 @@ inst_MSG(InstA, InstB, ModuleInfo, Inst) :-
     mer_inst::out) is semidet.
 
 inst_MSG_1(InstA, InstB, !.Expansions, ModuleInfo, Inst) :-
-    ( InstA = InstB ->
+    ( if InstA = InstB then
         Inst = InstA
-    ;
+    else
         % We don't do recursive MSGs
         % (we could, but it's probably not worth it).
-        \+ set.member(InstA - InstB, !.Expansions),
+        not set.member(InstA - InstB, !.Expansions),
         inst_expand(ModuleInfo, InstA, InstA2),
         inst_expand(ModuleInfo, InstB, InstB2),
         set.insert(InstA - InstB, !Expansions),
-        ( InstB2 = not_reached ->
+        ( if InstB2 = not_reached then
             Inst = InstA2
-        ;
+        else
             inst_MSG_2(InstA2, InstB2, !.Expansions, ModuleInfo, Inst)
         )
     ).
@@ -909,27 +913,27 @@ inst_list_MSG([ArgA | ArgsA], [ArgB | ArgsB], Expansions,
     list(bound_inst)::in, mer_inst::out) is semidet.
 
 bound_inst_list_MSG(Xs, Ys, Expansions, ModuleInfo, Uniq, BoundInsts, Inst) :-
-    (
+    ( if
         Xs = [],
         Ys = []
-    ->
+    then
         Inst = bound(Uniq, inst_test_no_results, [])
-    ;
+    else if
         Xs = [X | Xs1],
         Ys = [Y | Ys1],
         X = bound_functor(ConsId, ArgsX),
         Y = bound_functor(ConsId, ArgsY)
-    ->
+    then
         inst_list_MSG(ArgsX, ArgsY, Expansions, ModuleInfo, Args),
         Z = bound_functor(ConsId, Args),
         bound_inst_list_MSG(Xs1, Ys1, Expansions,
             ModuleInfo, Uniq, BoundInsts, Inst1),
-        ( Inst1 = bound(Uniq, _, Zs) ->
+        ( if Inst1 = bound(Uniq, _, Zs) then
             Inst = bound(Uniq, inst_test_no_results, [Z | Zs])
-        ;
+        else
             Inst = Inst1
         )
-    ;
+    else
         % Check that it's OK to round off the uniqueness information.
         (
             Uniq = shared,
@@ -941,12 +945,14 @@ bound_inst_list_MSG(Xs, Ys, Expansions, ModuleInfo, Uniq, BoundInsts, Inst) :-
             NewInst = bound(unique, inst_test_no_results, BoundInsts),
             inst_is_unique(ModuleInfo, NewInst)
         ),
-        \+ inst_contains_nonstandard_func_mode(ModuleInfo,
-            bound(shared, inst_test_no_results, BoundInsts)),
-        Inst = ground(Uniq, none)
+        not (
+            inst_contains_nondefault_func_mode(ModuleInfo,
+                bound(shared, inst_test_no_results, BoundInsts))
+        ),
+        Inst = ground(Uniq, none_or_default_func)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 inst_size(ModuleInfo, Inst, Size) :-
     set.init(Expansions),
@@ -971,9 +977,9 @@ inst_size_2(ModuleInfo, Inst, !.Expansions, Size) :-
         inst_size_2(ModuleInfo, SubInst, !.Expansions, Size)
     ;
         Inst = defined_inst(InstName),
-        ( set.member(InstName, !.Expansions) ->
+        ( if set.member(InstName, !.Expansions) then
             Size = 1
-        ;
+        else
             set.insert(InstName, !Expansions),
             inst_lookup(ModuleInfo, InstName, SubInst),
             inst_size_2(ModuleInfo, SubInst, !.Expansions, Size)
@@ -1006,7 +1012,7 @@ inst_list_size(ModuleInfo, [Inst | Insts], Expansions, !Size) :-
     !:Size = !.Size + InstSize,
     inst_list_size(ModuleInfo, Insts, Expansions, !Size).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 goals_match(_ModuleInfo, OldGoal, OldArgs, OldArgTypes,
         NewGoal, NewVarTypes, OldNewRenaming, TypeSubn) :-
@@ -1018,9 +1024,10 @@ goals_match(_ModuleInfo, OldGoal, OldArgs, OldArgTypes,
 
     % Check that the goal produces a superset of the outputs of the
     % version we are searching for.
-    Search = (pred(K1::in, V1::out) is semidet :-
-        map.search(OldNewRenaming, K1, V1)
-    ),
+    Search =
+        ( pred(K1::in, V1::out) is semidet :-
+            map.search(OldNewRenaming, K1, V1)
+        ),
     list.map(Search, OldArgs, NewArgs),
     NewGoal = hlds_goal(_, NewGoalInfo),
     NewNonLocals = goal_info_get_nonlocals(NewGoalInfo),
@@ -1040,14 +1047,14 @@ goals_match(_ModuleInfo, OldGoal, OldArgs, OldArgTypes,
 collect_matching_arg_types([], [], _, !MatchingTypes) :-
     list.reverse(!MatchingTypes).
 collect_matching_arg_types([_ | _], [], _, !MatchingTypes) :-
-    unexpected($module, $pred, "list length mismatch").
+    unexpected($pred, "list length mismatch").
 collect_matching_arg_types([], [_ | _], _, !MatchingTypes) :-
-    unexpected($module, $pred, "list length mismatch").
+    unexpected($pred, "list length mismatch").
 collect_matching_arg_types([Arg | Args], [Type | Types],
         Renaming, !MatchingTypes) :-
-    ( map.contains(Renaming, Arg) ->
+    ( if map.contains(Renaming, Arg) then
         !:MatchingTypes = [Type | !.MatchingTypes]
-    ;
+    else
         true
     ),
     collect_matching_arg_types(Args, Types, Renaming, !MatchingTypes).
@@ -1060,12 +1067,14 @@ collect_matching_arg_types([Arg | Args], [Type | Types],
 
 goals_match_2([], [], !ONRenaming).
 goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
+    % ON here is short for OldNew.
     OldGoal = hlds_goal(OldGoalExpr, _),
     NewGoal = hlds_goal(NewGoalExpr, _),
-    (
+    ( if
         (
             OldGoalExpr = unify(_, _, _, OldUnification, _),
             NewGoalExpr = unify(_, _, _, NewUnification, _),
+            require_complete_switch [OldUnification]
             (
                 OldUnification = simple_test(OldVar1, OldVar2),
                 NewUnification = simple_test(NewVar1, NewVar2),
@@ -1090,6 +1099,10 @@ goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
                     _, _, _),
                 OldArgs = [OldVar | OldArgs1],
                 NewArgs = [NewVar | NewArgs1]
+            ;
+                OldUnification = complicated_unify(_, _, _),
+                unexpected($pred,
+                    "complicated_unify should have been expanded by now")
             )
         ;
             OldGoalExpr = plain_call(PredId, ProcId, OldArgs, _, _, _),
@@ -1103,23 +1116,23 @@ goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
             match_generic_call(OldGenericCall, NewGenericCall),
             goal_util.generic_call_vars(OldGenericCall, OldArgs0),
             goal_util.generic_call_vars(NewGenericCall, NewArgs0),
-            list.append(OldArgs0, OldArgs1, OldArgs),
-            list.append(NewArgs0, NewArgs1, NewArgs)
+            OldArgs = OldArgs0 ++ OldArgs1,
+            NewArgs = NewArgs0 ++ NewArgs1
         )
-    ->
+    then
         assoc_list.from_corresponding_lists(OldArgs, NewArgs, ONArgsList),
         MapInsert =
-            (pred(KeyValue::in, Map0::in, Map::out) is semidet :-
+            ( pred(KeyValue::in, Map0::in, Map::out) is semidet :-
                 KeyValue = Key - Value,
-                ( map.search(Map0, Key, Value0) ->
+                ( if map.search(Map0, Key, Value0) then
                     Value = Value0,
                     Map = Map0
-                ;
+                else
                     map.det_insert(Key, Value, Map0, Map)
                 )
             ),
         list.foldl(MapInsert, ONArgsList, !ONRenaming)
-    ;
+    else if
         (
             OldGoalExpr = negation(OldSubGoal),
             NewGoalExpr = negation(NewSubGoal)
@@ -1127,11 +1140,11 @@ goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
             OldGoalExpr = scope(_, OldSubGoal),
             NewGoalExpr = scope(_, NewSubGoal)
         )
-    ->
+    then
         goal_to_conj_list(OldSubGoal, OldSubGoalList),
         goal_to_conj_list(NewSubGoal, NewSubGoalList),
         goals_match_2(OldSubGoalList, NewSubGoalList, !ONRenaming)
-    ;
+    else
         fail
     ),
     goals_match_2(OldGoals, NewGoals, !ONRenaming).
@@ -1140,12 +1153,16 @@ goals_match_2([OldGoal | OldGoals], [NewGoal | NewGoals], !ONRenaming) :-
     %
 :- pred match_generic_call(generic_call::in, generic_call::in) is semidet.
 
-match_generic_call(higher_order(_, Purity, PredOrFunc, Arity),
-        higher_order(_, Purity, PredOrFunc, Arity)).
-match_generic_call(class_method(_, MethodNum, ClassId, CallId),
-        class_method(_, MethodNum, ClassId, CallId)).
+match_generic_call(GenericCallA, GenericCallB) :-
+    (
+        GenericCallA = higher_order(_, Purity, PredOrFunc, Arity),
+        GenericCallB = higher_order(_, Purity, PredOrFunc, Arity)
+    ;
+        GenericCallA = class_method(_, MethodNum, ClassId, CallId),
+        GenericCallB = class_method(_, MethodNum, ClassId, CallId)
+    ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 pd_can_reorder_goals(ModuleInfo, FullyStrict, EarlierGoal, LaterGoal) :-
     % The logic here is mostly duplicated in can_reorder_goals and
@@ -1163,7 +1180,7 @@ pd_can_reorder_goals(ModuleInfo, FullyStrict, EarlierGoal, LaterGoal) :-
     (
         determinism_components(EarlierDetism, can_fail, _)
     =>
-        \+ determinism_components(LaterDetism, _, at_most_many_cc)
+        not determinism_components(LaterDetism, _, at_most_many_cc)
     ),
 
     % Impure goals and trace goals cannot be reordered.
@@ -1179,13 +1196,13 @@ pd_can_reorder_goals(ModuleInfo, FullyStrict, EarlierGoal, LaterGoal) :-
 
     % Don't reorder the goals if the later goal depends on the outputs
     % of the current goal.
-    \+ goal_depends_on_goal(EarlierGoal, LaterGoal),
+    not goal_depends_on_goal(EarlierGoal, LaterGoal),
 
     % Don't reorder the goals if the later goal changes the instantiatedness
     % of any of the non-locals of the earlier goal. This is necessary if the
     % later goal clobbers any of the non-locals of the earlier goal, and
     % avoids rerunning full mode analysis in other cases.
-    \+ goal_depends_on_goal(LaterGoal, EarlierGoal).
+    not goal_depends_on_goal(LaterGoal, EarlierGoal).
 
 :- pred goal_depends_on_goal(hlds_goal::in, hlds_goal::in) is semidet.
 
@@ -1196,6 +1213,6 @@ goal_depends_on_goal(hlds_goal(_, GoalInfo1), hlds_goal(_, GoalInfo2)) :-
     set_of_var.intersect(ChangedVars1, NonLocals2, Intersection),
     set_of_var.is_non_empty(Intersection).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module transform_hlds.pd_util.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
