@@ -16,16 +16,18 @@
 :- module ll_backend.peephole.
 :- interface.
 
-:- import_module ll_backend.llds.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
+:- import_module ll_backend.llds.
 
 :- import_module bool.
 :- import_module list.
 
     % Peephole optimize a list of instructions.
     %
-:- pred peephole_optimize(gc_method::in, bool::in, list(instruction)::in,
-    list(instruction)::out, bool::out) is det.
+:- pred peephole_optimize(gc_method::in, maybe_opt_peep_mkword::in,
+    list(instruction)::in, list(instruction)::out, bool::out) is det.
 
 :- pred combine_decr_sp(list(instruction)::in, list(instruction)::out) is det.
 
@@ -34,14 +36,20 @@
 
 :- implementation.
 
+:- import_module hlds.
+:- import_module hlds.hlds_data.
+:- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
 :- import_module ll_backend.opt_util.
+:- import_module parse_tree.
+:- import_module parse_tree.prog_data.
 
 :- import_module int.
 :- import_module map.
 :- import_module maybe.
 :- import_module pair.
 :- import_module string.
+:- import_module uint8.
 
 %-----------------------------------------------------------------------------%
 
@@ -68,9 +76,9 @@ peephole_optimize_2(_, [], [], no).
 peephole_optimize_2(InvalidPatterns, [Instr0 | Instrs0], Instrs, Mod) :-
     peephole_optimize_2(InvalidPatterns, Instrs0, Instrs1, Mod0),
     peephole_opt_instr(Instr0, Instrs1, InvalidPatterns, Instrs, Mod1),
-    ( Mod0 = no, Mod1 = no ->
+    ( if Mod0 = no, Mod1 = no then
         Mod = no
-    ;
+    else
         Mod = yes
     ).
 
@@ -82,7 +90,9 @@ peephole_optimize_2(InvalidPatterns, [Instr0 | Instrs0], Instrs, Mod) :-
 
 peephole_opt_instr(Instr0, Instrs0, InvalidPatterns, Instrs, Mod) :-
     opt_util.skip_comments(Instrs0, Instrs1),
-    ( peephole_match(Instr0, Instrs1, InvalidPatterns, Instrs2) ->
+    ( if
+        peephole_match(Instr0, Instrs1, InvalidPatterns, Instrs2)
+    then
         (
             Instrs2 = [Instr2 | Instrs3],
             peephole_opt_instr(Instr2, Instrs3, InvalidPatterns, Instrs, _)
@@ -91,10 +101,12 @@ peephole_opt_instr(Instr0, Instrs0, InvalidPatterns, Instrs, Mod) :-
             Instrs = Instrs2
         ),
         Mod = yes
-    ; peephole_match_norepeat(Instr0, Instrs1, InvalidPatterns, Instrs2) ->
+    else if
+        peephole_match_norepeat(Instr0, Instrs1, InvalidPatterns, Instrs2)
+    then
         Instrs = Instrs2,
         Mod = yes
-    ;
+    else
         Instrs = [Instr0 | Instrs0],
         Mod = no
     ).
@@ -110,9 +122,9 @@ peephole_opt_instr(Instr0, Instrs0, InvalidPatterns, Instrs, Mod) :-
 build_peephole_jump_label_map([], _, !LabelMap).
 build_peephole_jump_label_map([MaybeLabel | MaybeLabels], Val, !LabelMap) :-
     MaybeLabel = yes(Label),
-    ( map.search(!.LabelMap, Label, Vals0) ->
+    ( if map.search(!.LabelMap, Label, Vals0) then
         map.det_update(Label, [Val | Vals0], !LabelMap)
-    ;
+    else
         map.det_insert(Label, [Val], !LabelMap)
     ),
     build_peephole_jump_label_map(MaybeLabels, Val + 1, !LabelMap).
@@ -128,15 +140,15 @@ peephole_pick_one_val_label(LabelVals1, LabelVals2, OneValLabel, Val,
         OtherLabel) :-
     LabelVals1 = Label1 - Vals1,
     LabelVals2 = Label2 - Vals2,
-    ( Vals1 = [Val1] ->
+    ( if Vals1 = [Val1] then
         OneValLabel = Label1,
         Val = Val1,
         OtherLabel = Label2
-    ; Vals2 = [Val2] ->
+    else if Vals2 = [Val2] then
         OneValLabel = Label2,
         Val = Val2,
         OtherLabel = Label1
-    ;
+    else
         fail
     ).
 
@@ -164,23 +176,24 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
     Uinstr0 = computed_goto(SelectorRval, Labels),
     build_peephole_jump_label_map(Labels, 0, map.init, LabelMap),
     map.to_assoc_list(LabelMap, LabelValsList),
-    (
+    ( if
         LabelValsList = [Label - _]
-    ->
+    then
         GotoInstr = llds_instr(goto(code_label(Label)), Comment0),
         Instrs = [GotoInstr | Instrs0]
-    ;
+    else if
         LabelValsList = [LabelVals1, LabelVals2],
         peephole_pick_one_val_label(LabelVals1, LabelVals2, OneValLabel,
             Val, OtherLabel)
-    ->
-        CondRval = binop(eq, SelectorRval, const(llconst_int(Val))),
+    then
+        CondRval = binop(eq(int_type_int), SelectorRval,
+            const(llconst_int(Val))),
         CommentInstr = llds_instr(comment(Comment0), ""),
         BranchInstr = llds_instr(if_val(CondRval, code_label(OneValLabel)),
             ""),
         GotoInstr = llds_instr(goto(code_label(OtherLabel)), Comment0),
         Instrs = [CommentInstr, BranchInstr, GotoInstr | Instrs0]
-    ;
+    else
         fail
     ).
 
@@ -196,9 +209,9 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
 peephole_match(Instr0, Instrs0, _, Instrs) :-
     Instr0 = llds_instr(Uinstr0, Comment0),
     Uinstr0 = if_val(Rval, CodeAddr),
-    (
+    ( if
         opt_util.is_const_condition(Rval, Taken)
-    ->
+    then
         (
             Taken = yes,
             Instrs = [llds_instr(goto(CodeAddr), Comment0) | Instrs0]
@@ -206,18 +219,18 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
             Taken = no,
             Instrs = Instrs0
         )
-    ;
+    else if
         opt_util.skip_comments(Instrs0, Instrs1),
         Instrs1 = [Instr1 | _],
         Instr1 = llds_instr(goto(CodeAddr), _)
-    ->
+    then
         Instrs = Instrs0
-    ;
+    else if
         CodeAddr = code_label(Label),
         opt_util.is_this_label_next(Label, Instrs0, _)
-    ->
+    then
         Instrs = Instrs0
-    ;
+    else
         fail
     ).
 
@@ -277,7 +290,7 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
 peephole_match(Instr0, Instrs0, _, Instrs) :-
     Instr0 = llds_instr(Uinstr0, Comment0),
     Uinstr0 = mkframe(NondetFrameInfo, yes(Redoip0)),
-    (
+    ( if
         % A mkframe sets curfr to point to the new frame
         % only for ordinary frames, not temp frames.
         (
@@ -290,76 +303,76 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
         opt_util.next_assign_to_redoip(Instrs0, AllowedBases, [], Redoip1,
             Skipped, Rest),
         opt_util.touches_nondet_ctrl(Skipped) = no
-    ->
+    then
         Instrs1 = Skipped ++ Rest,
         NewInstr = llds_instr(mkframe(NondetFrameInfo, yes(Redoip1)),
             Comment0),
         Instrs = [NewInstr | Instrs1]
-    ;
+    else if
         opt_util.skip_comments_livevals(Instrs0, Instrs1),
         Instrs1 = [Instr1 | Instrs2],
         Instr1 = llds_instr(if_val(Test, Target), Comment1),
-        (
+        ( if
             Redoip0 = do_fail,
             ( Target = do_redo ; Target = do_fail)
-        ->
+        then
             InstrsPrime = [
                 llds_instr(if_val(Test, do_redo), Comment1),
                 llds_instr(mkframe(NondetFrameInfo, yes(do_fail)), Comment0)
                 | Instrs2
             ]
-        ;
+        else if
             Redoip0 = code_label(_)
-        ->
-            (
+        then
+            ( if
                 Target = do_fail
-            ->
+            then
                 InstrsPrime = [
                     llds_instr(if_val(Test, do_redo), Comment1),
                     Instr0
                     | Instrs2
                 ]
-            ;
+            else if
                 Target = do_redo
-            ->
+            then
                 InstrsPrime = [
                     Instr0,
                     llds_instr(if_val(Test, Redoip0), Comment1)
                     | Instrs2
                 ]
-            ;
+            else
                 fail
             )
-        ;
+        else
             fail
         )
-    ->
+    then
         Instrs = InstrsPrime
-    ;
+    else if
         opt_util.skip_comments_livevals(Instrs0, Instrs1),
         Instrs1 = [Instr1 | Instrs2],
         Instr1 = llds_instr(goto(do_fail), Comment2)
-    ->
+    then
         Instrs = [llds_instr(goto(do_redo), Comment2) | Instrs2]
-    ;
+    else if
         Redoip0 = do_fail,
         no_stack_straight_line(Instrs0, Straight, Instrs1),
         Instrs1 = [Instr1 | Instrs2],
         Instr1 = llds_instr(goto(do_succeed(_)), _)
-    ->
+    then
         GotoSuccip = llds_instr(goto(code_succip),
             "return from optimized away mkframe"),
         Instrs = Straight ++ [GotoSuccip | Instrs2]
-    ;
+    else if
         Redoip0 = do_fail,
         may_replace_succeed_with_succeed_discard(Instrs0, UntilSucceed,
             SucceedComment, Instrs2)
-    ->
+    then
         DiscardUinstr = goto(do_succeed(no)),
         DiscardComment = SucceedComment ++ " (added discard)",
         DiscardInstr = llds_instr(DiscardUinstr, DiscardComment),
         Instrs = [Instr0 | UntilSucceed] ++ [DiscardInstr | Instrs2]
-    ;
+    else
         fail
     ).
 
@@ -397,25 +410,25 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
 peephole_match(Instr0, Instrs0, _, Instrs) :-
     Instr0 = llds_instr(Uinstr0, Comment0),
     Uinstr0 = assign(redoip_slot(lval(Base)), Redoip0),
-    (
+    ( if
         opt_util.next_assign_to_redoip(Instrs0, [Base], [], Redoip1,
             Skipped, Rest),
         opt_util.touches_nondet_ctrl(Skipped) = no
-    ->
+    then
         Instrs1 = Skipped ++ Rest,
         RedoipInstr = llds_instr(assign(redoip_slot(lval(Base)),
             const(llconst_code_addr(Redoip1))), Comment0),
         Instrs = [RedoipInstr | Instrs1]
-    ;
+    else if
         Base = curfr,
         Redoip0 = const(llconst_code_addr(do_fail)),
         opt_util.straight_alternative(Instrs0, Between, After),
         opt_util.touches_nondet_ctrl(Between) = no,
         string.sub_string_search(Comment0, "curfr==maxfr", _)
-    ->
+    then
         SucceedInstr = llds_instr(goto(do_succeed(yes)), "early discard"),
         Instrs = Between ++ [SucceedInstr] ++ After
-    ;
+    else
         fail
     ).
 
@@ -436,10 +449,10 @@ peephole_match(Instr0, Instrs0, _, Instrs) :-
 peephole_match(Instr0, Instrs0, InvalidPatterns, Instrs) :-
     Instr0 = llds_instr(Uinstr0, _Comment0),
     Uinstr0 = incr_sp(N, _, _),
-    \+ list.member(pattern_incr_sp, InvalidPatterns),
-    ( opt_util.no_stackvars_til_decr_sp(Instrs0, N, Between, Remain) ->
+    not list.member(pattern_incr_sp, InvalidPatterns),
+    ( if opt_util.no_stackvars_til_decr_sp(Instrs0, N, Between, Remain) then
         Instrs = Between ++ Remain
-    ;
+    else
         fail
     ).
 
@@ -453,55 +466,56 @@ peephole_match(Instr0, Instrs0, InvalidPatterns, Instrs) :-
     list(pattern)::in, list(instruction)::out) is semidet.
 
     % If none of the instructions in brackets can affect Lval, then
-    % we can transform references to tag(Lval) to Tag and body(Lval, Tag)
+    % we can transform references to tag(Lval) to Ptag and body(Lval, Ptag)
     % to Base.
     %
-    %   Lval = mkword(Tag, Base)        Lval = mkword(Tag, Base)
+    %   Lval = mkword(Ptag, Base)       Lval = mkword(Ptag, Base)
     %   <...>                       =>  <...>
-    %   ... tag(Lval) ...               ... Tag ...
-    %   ... body(Lval, Tag) ...         ... Base ...
+    %   ... tag(Lval) ...               ... Ptag ...
+    %   ... body(Lval, Ptag) ...        ... Base ...
     %
 peephole_match_norepeat(Instr0, Instrs0, InvalidPatterns, Instrs) :-
     Instr0 = llds_instr(Uinstr0, _),
-    Uinstr0 = assign(Lval, mkword(Tag, Base)),
-    \+ list.member(pattern_mkword, InvalidPatterns),
-    replace_tagged_ptr_components_in_instrs(Lval, Tag, Base, Instrs0, Instrs1),
+    Uinstr0 = assign(Lval, mkword(Ptag, Base)),
+    not list.member(pattern_mkword, InvalidPatterns),
+    replace_tagged_ptr_components_in_instrs(Lval, Ptag, Base,
+        Instrs0, Instrs1),
     Instrs = [Instr0 | Instrs1].
 
 %-----------------------------------------------------------------------------%
 
-:- pred replace_tagged_ptr_components_in_instrs(lval::in, tag::in, rval::in,
+:- pred replace_tagged_ptr_components_in_instrs(lval::in, ptag::in, rval::in,
     list(instruction)::in, list(instruction)::out) is det.
 
 replace_tagged_ptr_components_in_instrs(_, _, _, [], []).
-replace_tagged_ptr_components_in_instrs(Lval, Tag, Base, Instrs0, Instrs) :-
+replace_tagged_ptr_components_in_instrs(Lval, Ptag, Base, Instrs0, Instrs) :-
     Instrs0 = [HeadInstr0 | TailInstrs0],
-    replace_tagged_ptr_components_in_instr(Lval, Tag, Base,
+    replace_tagged_ptr_components_in_instr(Lval, Ptag, Base,
         HeadInstr0, MaybeHeadInstr),
     (
         MaybeHeadInstr = no,
         Instrs = Instrs0
     ;
         MaybeHeadInstr = yes(HeadInstr),
-        replace_tagged_ptr_components_in_instrs(Lval, Tag, Base,
+        replace_tagged_ptr_components_in_instrs(Lval, Ptag, Base,
             TailInstrs0, TailInstrs),
         Instrs = [HeadInstr | TailInstrs]
     ).
 
-:- pred replace_tagged_ptr_components_in_instr(lval::in, tag::in, rval::in,
+:- pred replace_tagged_ptr_components_in_instr(lval::in, ptag::in, rval::in,
     instruction::in, maybe(instruction)::out) is det.
 
-replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
+replace_tagged_ptr_components_in_instr(OldLval, OldPtag, OldBase,
         Instr0, MaybeInstr) :-
     Instr0 = llds_instr(Uinstr0, Comment),
     (
         Uinstr0 = assign(Lval, Rval0),
-        ( Lval = OldLval ->
+        ( if Lval = OldLval then
             MaybeInstr = no
-        ; Lval = mem_ref(_) ->
+        else if Lval = mem_ref(_) then
             MaybeInstr = no
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 Rval0, Rval),
             Uinstr = assign(Lval, Rval),
             Instr = llds_instr(Uinstr, Comment),
@@ -509,12 +523,12 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         )
     ;
         Uinstr0 = keep_assign(Lval, Rval0),
-        ( Lval = OldLval ->
+        ( if Lval = OldLval then
             MaybeInstr = no
-        ; Lval = mem_ref(_) ->
+        else if Lval = mem_ref(_) then
             MaybeInstr = no
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 Rval0, Rval),
             Uinstr = keep_assign(Lval, Rval),
             Instr = llds_instr(Uinstr, Comment),
@@ -522,69 +536,69 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         )
     ;
         Uinstr0 = computed_goto(Rval0, Targets),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = computed_goto(Rval, Targets),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = if_val(Rval0, Target),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = if_val(Rval, Target),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
-        Uinstr0 = incr_hp(Target, MaybeTag, MaybeOffset, SizeRval0,
+        Uinstr0 = incr_hp(Target, MaybePtag, MaybeOffset, SizeRval0,
             TypeMsg, MayUseAtomicAlloc, MaybeRegionId, MaybeReuse),
-        ( Target = OldLval ->
+        ( if Target = OldLval then
             MaybeInstr = no
-        ; Target = mem_ref(_) ->
+        else if Target = mem_ref(_) then
             MaybeInstr = no
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 SizeRval0, SizeRval),
-            Uinstr = incr_hp(Target, MaybeTag, MaybeOffset, SizeRval,
+            Uinstr = incr_hp(Target, MaybePtag, MaybeOffset, SizeRval,
                 TypeMsg, MayUseAtomicAlloc, MaybeRegionId, MaybeReuse),
             Instr = llds_instr(Uinstr, Comment),
             MaybeInstr = yes(Instr)
         )
     ;
         Uinstr0 = restore_hp(Rval0),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = restore_hp(Rval),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = free_heap(Rval0),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = free_heap(Rval),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = reset_ticket(Rval0, Reason),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = reset_ticket(Rval, Reason),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = prune_tickets_to(Rval0),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             Rval0, Rval),
         Uinstr = prune_tickets_to(Rval),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = lc_wait_free_slot(Rval0, Lval0, Label),
-        ( Lval0 = OldLval ->
+        ( if Lval0 = OldLval then
             MaybeInstr = no
-        ; Lval0 = mem_ref(_) ->
+        else if Lval0 = mem_ref(_) then
             MaybeInstr = no
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 Rval0, Rval),
             Uinstr = lc_wait_free_slot(Rval, Lval0, Label),
             Instr = llds_instr(Uinstr, Comment),
@@ -592,18 +606,18 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         )
     ;
         Uinstr0 = lc_spawn_off(LCRval0, LCSRval0, Label),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             LCRval0, LCRval),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             LCSRval0, LCSRval),
         Uinstr = lc_spawn_off(LCRval, LCSRval, Label),
         Instr = llds_instr(Uinstr, Comment),
         MaybeInstr = yes(Instr)
     ;
         Uinstr0 = lc_join_and_terminate(LCRval0, LCSRval0),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             LCRval0, LCRval),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             LCSRval0, LCSRval),
         Uinstr = lc_join_and_terminate(LCRval, LCSRval),
         Instr = llds_instr(Uinstr, Comment),
@@ -615,11 +629,11 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         ; Uinstr0 = mark_ticket_stack(Lval0)
         ; Uinstr0 = lc_create_loop_control(_NumSlots, Lval0)
         ),
-        ( Lval0 = OldLval ->
+        ( if Lval0 = OldLval then
             MaybeInstr = no
-        ; Lval0 = mem_ref(_) ->
+        else if Lval0 = mem_ref(_) then
             MaybeInstr = no
-        ;
+        else
             MaybeInstr = yes(Instr0)
         )
     ;
@@ -652,43 +666,52 @@ replace_tagged_ptr_components_in_instr(OldLval, OldTag, OldBase,
         MaybeInstr = no
     ).
 
-:- pred replace_tagged_ptr_components_in_rval(lval::in, tag::in, rval::in,
+:- pred replace_tagged_ptr_components_in_rval(lval::in, ptag::in, rval::in,
     rval::in, rval::out) is det.
 
-replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase, Rval0, Rval) :-
+replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
+        Rval0, Rval) :-
     (
         Rval0 = unop(UnOp, RvalA0),
-        (
+        ( if
             UnOp = tag,
             RvalA0 = lval(OldLval)
-        ->
-            Rval = unop(mktag, const(llconst_int(OldTag)))
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        then
+            OldPtag = ptag(OldPtagUint8),
+            Rval = const(llconst_int(uint8.cast_to_int(OldPtagUint8)))
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 RvalA0, RvalA),
             Rval = unop(UnOp, RvalA)
         )
     ;
         Rval0 = binop(BinOp, RvalA0, RvalB0),
-        (
+        ( if
             BinOp = body,
             RvalA0 = lval(OldLval),
-            RvalB0 = unop(mktag, const(llconst_int(OldTag))),
+            RvalB0 = const(llconst_int(RvalB0Int)),
+            OldPtag = ptag(OldPtagUint8),
+            RvalB0Int = uint8.cast_to_int(OldPtagUint8),
             OldBase = const(_)
-        ->
+        then
             Rval = OldBase
-        ;
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        else
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 RvalA0, RvalA),
-            replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+            replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
                 RvalB0, RvalB),
             Rval = binop(BinOp, RvalA, RvalB)
         )
     ;
-        Rval0 = mkword(Tag, BaseRval0),
-        replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase,
+        Rval0 = mkword(Ptag, BaseRval0),
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
             BaseRval0, BaseRval),
-        Rval = mkword(Tag, BaseRval)
+        Rval = mkword(Ptag, BaseRval)
+    ;
+        Rval0 = cast(Type, BaseRval0),
+        replace_tagged_ptr_components_in_rval(OldLval, OldPtag, OldBase,
+            BaseRval0, BaseRval),
+        Rval = cast(Type, BaseRval)
     ;
         ( Rval0 = lval(_)
         ; Rval0 = var(_)
@@ -703,8 +726,8 @@ replace_tagged_ptr_components_in_rval(OldLval, OldTag, OldBase, Rval0, Rval) :-
 
     % Given a GC method, return the list of invalid peephole optimizations.
     %
-:- pred invalid_peephole_opts(gc_method::in, bool::in, list(pattern)::out)
-    is det.
+:- pred invalid_peephole_opts(gc_method::in, maybe_opt_peep_mkword::in,
+    list(pattern)::out) is det.
 
 invalid_peephole_opts(GC_Method, OptPeepMkword, InvalidPatterns) :-
     (
@@ -715,16 +738,15 @@ invalid_peephole_opts(GC_Method, OptPeepMkword, InvalidPatterns) :-
         ; GC_Method = gc_none
         ; GC_Method = gc_boehm
         ; GC_Method = gc_boehm_debug
-        ; GC_Method = gc_mps
         ; GC_Method = gc_hgc
         ),
         InvalidPatterns0 = []
     ),
     (
-        OptPeepMkword = yes,
+        OptPeepMkword = opt_peep_mkword,
         InvalidPatterns = InvalidPatterns0
     ;
-        OptPeepMkword = no,
+        OptPeepMkword = do_not_opt_peep_mkword,
         InvalidPatterns = [pattern_mkword | InvalidPatterns0]
     ).
 
@@ -733,7 +755,7 @@ invalid_peephole_opts(GC_Method, OptPeepMkword, InvalidPatterns) :-
 combine_decr_sp([], []).
 combine_decr_sp([Instr0 | Instrs0], Instrs) :-
     combine_decr_sp(Instrs0, Instrs1),
-    (
+    ( if
         Instr0 = llds_instr(assign(succip, lval(stackvar(N))), _),
         opt_util.skip_comments_livevals(Instrs1, Instrs2),
         Instrs2 = [Instr2 | Instrs3],
@@ -741,13 +763,13 @@ combine_decr_sp([Instr0 | Instrs0], Instrs) :-
         opt_util.skip_comments_livevals(Instrs3, Instrs4),
         Instrs4 = [Instr4 | Instrs5],
         Instr4 = llds_instr(goto(code_succip), Comment)
-    ->
+    then
         NewInstr = llds_instr(decr_sp_and_return(N), Comment),
         Instrs = [NewInstr | Instrs5]
-    ;
+    else
         Instrs = [Instr0 | Instrs1]
     ).
 
 %-----------------------------------------------------------------------------%
-:- end_module peephole.
+:- end_module ll_backend.peephole.
 %-----------------------------------------------------------------------------%
