@@ -19,6 +19,7 @@
 :- interface.
 
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_pragma.
 
 :- import_module bool.
 :- import_module io.
@@ -30,7 +31,7 @@
 
 %-----------------------------------------------------------------------------%
 %
-% Parsing routines
+% Parsing routines.
 %
 
 :- func parse_unit_selector(term(T)) = unit_selector.
@@ -56,19 +57,21 @@
 
 %-----------------------------------------------------------------------------%
 %
-% Printing routines
+% Printing routines.
 %
 
-:- pred print_selector(tvarset::in, selector::in, io::di, io::uo) is det.
-
-:- pred print_datastruct(prog_varset::in, tvarset::in, datastruct::in,
+:- pred print_selector(io.text_output_stream::in, tvarset::in, selector::in,
     io::di, io::uo) is det.
 
-:- pred print_datastructs(prog_varset::in, tvarset::in, list(datastruct)::in,
-    io::di, io::uo) is det.
+:- pred print_datastruct(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, datastruct::in, io::di, io::uo) is det.
 
-:- pred print_structure_sharing_pair(prog_varset::in, tvarset::in,
-    structure_sharing_pair::in, io::di, io::uo) is det.
+:- pred print_datastructs(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, list(datastruct)::in, io::di, io::uo) is det.
+
+:- pred print_structure_sharing_pair(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_sharing_pair::in,
+    io::di, io::uo) is det.
 
     % Print list of structure sharing pairs.
     %
@@ -81,15 +84,17 @@
     % N, is given, then only the first N pairs are printed (and "..."
     % is shown if there are more than N pairs).
     %
-:- pred print_structure_sharing(prog_varset::in, tvarset::in,
-    maybe(int)::in, string::in, string::in, string::in,
-    structure_sharing::in, io::di, io::uo) is det.
+:- pred print_structure_sharing(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, maybe(int)::in,
+    string::in, string::in, string::in, structure_sharing::in,
+    io::di, io::uo) is det.
 
     % Print complete list of structure sharing pairs as a list (using "[",
     % ",", and "]"). This can later be parsed automatically.
     %
-:- pred print_structure_sharing_as_list(prog_varset::in, tvarset::in,
-    structure_sharing::in, io::di, io::uo) is det.
+:- pred print_structure_sharing_as_list(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_sharing::in,
+    io::di, io::uo) is det.
 
     % Print structure sharing domain.
     %
@@ -105,27 +110,35 @@
     % The output can later be parsed again only if VerboseTop = no and
     % MaybeThreshold = no.
     %
-:- pred print_structure_sharing_domain(prog_varset::in, tvarset::in, bool::in,
-    maybe(int)::in, structure_sharing_domain::in, io::di, io::uo) is det.
+:- pred print_structure_sharing_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, bool::in, maybe(int)::in,
+    structure_sharing_domain::in, io::di, io::uo) is det.
 
-    % Print the available structure sharing information as a Mercury comment.
+    % Print structure sharing or reuse information as a Mercury comment.
     % This is used in HLDS dumps.
     %
-:- pred dump_maybe_structure_sharing_domain(prog_varset::in, tvarset::in,
-    maybe(structure_sharing_domain)::in, io::di, io::uo) is det.
+:- pred dump_structure_sharing_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_sharing_domain::in,
+    io::di, io::uo) is det.
 
-:- pred dump_maybe_structure_reuse_domain(prog_varset::in, tvarset::in,
-    maybe(structure_reuse_domain)::in, io::di, io::uo) is det.
+:- pred print_interface_structure_sharing_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, maybe(structure_sharing_domain)::in,
+    io::di, io::uo) is det.
 
-:- pred print_interface_structure_sharing_domain(prog_varset::in,
-    tvarset::in, maybe(structure_sharing_domain)::in, io::di, io::uo) is det.
+    % Print structure sharing or reuse information as a Mercury comment.
+    % This is used in HLDS dumps.
+    %
+:- pred dump_structure_reuse_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_reuse_domain::in,
+    io::di, io::uo) is det.
 
-:- pred print_interface_maybe_structure_reuse_domain(prog_varset::in,
-    tvarset::in, maybe(structure_reuse_domain)::in, io::di, io::uo) is det.
+:- pred print_interface_maybe_structure_reuse_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, maybe(structure_reuse_domain)::in,
+    io::di, io::uo) is det.
 
 %-----------------------------------------------------------------------------%
 %
-% Renaming operations
+% Renaming operations.
 %
 
 :- pred rename_unit_selector(tsubst::in, unit_selector::in,
@@ -172,8 +185,9 @@
 :- implementation.
 
 :- import_module parse_tree.mercury_to_mercury.
-:- import_module parse_tree.prog_io_sym_name.
-:- import_module parse_tree.prog_io_util.
+:- import_module parse_tree.parse_sym_name.
+:- import_module parse_tree.parse_tree_out_info.
+:- import_module parse_tree.parse_type_name.
 :- import_module parse_tree.prog_out.
 :- import_module parse_tree.prog_type.
 :- import_module parse_tree.prog_type_subst.
@@ -184,132 +198,129 @@
 :- import_module require.
 :- import_module set.
 :- import_module string.
-:- import_module varset.
 
 %-----------------------------------------------------------------------------%
 %
-% Parsing routines
+% Parsing routines.
 %
 
 parse_unit_selector(Term) = UnitSelector :-
-    (
-        Term = term.functor(term.atom(Cons), Args, _)
-    ->
+    ( if Term = term.functor(term.atom(Cons), Args, _) then
         % XXX We should include non-dummy type_ctors in cons ConsIds.
         % XXX Why do we parse int, float, and string ConsIds when they
         % never have any arguments and thus cannot select anything?
-        (
+        ( if
             Cons = "sel",
             Args = [ConsTerm, ArityTerm, PosTerm]
-        ->
-            (
+        then
+            ( if
                 try_parse_sym_name_and_no_args(ConsTerm, ConsIdName),
-                ArityTerm = term.functor(term.integer(Arity), _, _),
-                PosTerm = term.functor(term.integer(Pos), _, _)
-            ->
+                decimal_term_to_int(ArityTerm, Arity),
+                decimal_term_to_int(PosTerm, Pos)
+            then
                 ConsId = cons(ConsIdName, Arity, cons_id_dummy_type_ctor),
                 UnitSelector = termsel(ConsId, Pos)
-            ;
-                ConsTerm = term.functor(term.integer(Int), _, _)
-            ->
-                ConsId = int_const(Int),
+            else if
+                % XXX UINT, presuambly we need to handle uints here too.
+                decimal_term_to_int(ConsTerm, Int)
+            then
+                ConsId = some_int_const(int_const(Int)),
                 UnitSelector = termsel(ConsId, 0)
-            ;
+            else if
                 ConsTerm = term.functor(term.float(Float), _, _)
-            ->
+            then
                 ConsId = float_const(Float),
                 UnitSelector = termsel(ConsId, 0)
-            ;
+            else if
                 ConsTerm = term.functor(term.string(Str), _, _)
-            ->
+            then
                 ConsId = string_const(Str),
                 UnitSelector = termsel(ConsId, 0)
-            ;
-                unexpected($module, $pred, "unknown cons_id in unit selector")
+            else
+                unexpected($pred, "unknown cons_id in unit selector")
             )
-        ;
+        else if
             Cons = "typesel",
             Args = [TypeSelectorTerm]
-        ->
-            ( maybe_parse_type(term.coerce(TypeSelectorTerm), TypeSelector) ->
+        then
+            ( if
+                maybe_parse_type(
+                    no_allow_ho_inst_info(wnhii_ctgc_type_selector),
+                    term.coerce(TypeSelectorTerm), TypeSelector)
+            then
                 UnitSelector = typesel(TypeSelector)
-            ;
-                unexpected($module, $pred, "error in parsing type selector")
+            else
+                unexpected($pred, "error in parsing type selector")
             )
-        ;
-            unexpected($module, $pred,
-                "selector is neither sel/3 nor typesel/1.")
+        else
+            unexpected($pred, "selector is neither sel/3 nor typesel/1.")
         )
-    ;
-        unexpected($module, $pred, "term not a functor")
+    else
+        unexpected($pred, "term not a functor")
     ).
 
 parse_selector(Term) = Selector :-
-    ( Term = term.functor(term.atom(Cons), Args, _) ->
-        (
+    ( if Term = term.functor(term.atom(Cons), Args, _) then
+        ( if
             Cons = "[|]",
             Args = [First, Rest]
-        ->
+        then
             Selector = [parse_unit_selector(First) | parse_selector(Rest)]
-        ;
+        else
             Selector = []
         )
-    ;
-        unexpected($module, $pred, "term not a functor")
+    else
+        unexpected($pred, "term not a functor")
     ).
 
 parse_datastruct(Term) = Datastruct :-
-    (
+    ( if
         Term = term.functor(term.atom(Cons), Args, _),
         Cons = "cel",
         Args = [VarTerm, SelectorTerm],
         VarTerm = term.variable(Var, _)
-    ->
+    then
         Datastruct = selected_cel(term.coerce_var(Var),
             parse_selector(SelectorTerm))
-    ;
-        unexpected($module, $pred, "error while parsing datastruct.")
+    else
+        unexpected($pred, "error while parsing datastruct.")
     ).
 
 :- func parse_datastruct_list(term(T)) = list(datastruct).
 
 parse_datastruct_list(Term) = Datastructs :-
-    (
-        Term = term.functor(term.atom(Cons), Args, _)
-    ->
-        (
+    ( if Term = term.functor(term.atom(Cons), Args, _) then
+        ( if
             Cons = "[|]",
             Args = [FirstDataTerm, RestDataTerm]
-        ->
-            Datastructs = [parse_datastruct(FirstDataTerm)|
+        then
+            Datastructs = [parse_datastruct(FirstDataTerm) |
                 parse_datastruct_list(RestDataTerm)]
-        ;
+        else if
             Cons = "[]"
-        ->
+        then
             Datastructs = []
-        ;
-            unexpected($module, $pred,
-                "error while parsing list of datastructs")
+        else
+            unexpected($pred, "error while parsing list of datastructs")
         )
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "error while parsing list of datastructs (term not a functor)")
     ).
 
 parse_structure_sharing_pair(Term) = SharingPair :-
-    (
+    ( if
         Term = term.functor(term.atom(Cons), Args, _),
         Cons = "pair",
         Args = [First, Second]
-    ->
+    then
         SharingPair = parse_datastruct(First) - parse_datastruct(Second)
-    ;
-        unexpected($module, $pred,
-            "error while parsing structure sharing pair")
+    else
+        unexpected($pred, "error while parsing structure sharing pair")
     ).
 
 parse_structure_sharing(Term) = SharingPairs :-
-    (
+    ( if
         Term = term.functor(term.atom(Cons), Args, _),
         (
             Cons = "[|]",
@@ -320,15 +331,15 @@ parse_structure_sharing(Term) = SharingPairs :-
             Cons = "[]",
             SharingPairs0 = []
         )
-    ->
+    then
         SharingPairs = SharingPairs0
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "error while parsing list of structure sharing pairs")
     ).
 
 parse_structure_sharing_domain(Term) = SharingAs :-
-    (
+    ( if
         Term = term.functor(term.atom(Cons), _, _Context),
         (
             Cons = "[|]",
@@ -342,80 +353,76 @@ parse_structure_sharing_domain(Term) = SharingAs :-
                 set.make_singleton_set(
                     top_cannot_improve("from parse_structure_sharing_domain")))
         )
-    ->
+    then
         SharingAs = SharingAs0
-    ;
-        unexpected($module, $pred,
-            "error while parsing structure sharing domain")
+    else
+        unexpected($pred, "error while parsing structure sharing domain")
     ).
 
 parse_structure_reuse_condition(Term) = ReuseCondition :-
-    (
-        Term = term.functor(term.atom(Cons), Args, _)
-    ->
-        (
+    ( if Term = term.functor(term.atom(Cons), Args, _) then
+        ( if
             Cons = "condition",
             Args = [DeadNodesTerm, InUseNodesTerm, SharingTerm]
-        ->
+        then
             DeadNodesList = parse_datastruct_list(DeadNodesTerm),
-            DeadNodes = set.from_list(DeadNodesList),
+            DeadNodes = set.list_to_set(DeadNodesList),
             InUseNodes = parse_datastruct_list(InUseNodesTerm),
             Sharing = parse_structure_sharing_domain(SharingTerm),
             ReuseCondition = structure_reuse_condition(DeadNodes,
                 InUseNodes, Sharing)
-        ;
-            unexpected($module, $pred, "error while parsing reuse condition")
+        else
+            unexpected($pred, "error while parsing reuse condition")
         )
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "error while parsing reuse condition (term not a functor)")
     ).
 
 parse_structure_reuse_conditions(Term) = ReuseConditions :-
-    (
-        Term = term.functor(term.atom(Cons), Args, _)
-    ->
-        (
+    ( if Term = term.functor(term.atom(Cons), Args, _) then
+        ( if
             Cons = "[|]",
             Args = [FirstTupleTerm, RestTuplesTerm]
-        ->
-            ReuseConditions = [parse_structure_reuse_condition(FirstTupleTerm)|
+        then
+            ReuseConditions =
+                [parse_structure_reuse_condition(FirstTupleTerm) |
                 parse_structure_reuse_conditions(RestTuplesTerm)]
-        ;
+        else if
             Cons = "[]"
-        ->
+        then
             ReuseConditions = []
-        ;
-            unexpected($module, $pred, "error while parsing reuse conditions")
+        else
+            unexpected($pred, "error while parsing reuse conditions")
         )
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "error while parsing reuse conditions (term not a functor)")
     ).
 
 parse_structure_reuse_domain(Term) = ReuseDomain :-
-    (
+    ( if
         Term = term.functor(term.atom(Cons), Args, _)
-    ->
-        (
+    then
+        ( if
             Cons = "has_no_reuse"
-        ->
+        then
             ReuseDomain = has_no_reuse
-        ;
+        else if
             Cons = "has_only_unconditional_reuse"
-        ->
+        then
             ReuseDomain = has_only_unconditional_reuse
-        ;
+        else if
             Cons = "has_conditional_reuse",
             Args = [ReuseConditionsTerm]
-        ->
+        then
             ReuseDomain = has_conditional_reuse(
                 parse_structure_reuse_conditions(ReuseConditionsTerm))
-        ;
-            unexpected($module, $pred, "error while parsing reuse domain")
+        else
+            unexpected($pred, "error while parsing reuse domain")
         )
-    ;
-        unexpected($module, $pred,
+    else
+        unexpected($pred,
             "error while parsing reuse domain (term not a functor)")
     ).
 
@@ -437,7 +444,8 @@ parse_user_annotated_sharing(!.Varset, Term, UserSharing) :-
             [TypesTerm, UserSharingTerm], _),
         (
             TypesTerm = term.functor(term.atom("yes"), ListTypeTerms, _),
-            maybe_parse_types(ListTypeTerms, Types),
+            maybe_parse_types(no_allow_ho_inst_info(wnhii_user_struct_sharing),
+                ListTypeTerms, Types),
             term.vars_list(ListTypeTerms, TypeVars),
             varset.select(set.list_to_set(TypeVars), !Varset),
             MaybeUserTypes = yes(user_type_info(Types,
@@ -459,7 +467,7 @@ parse_user_annotated_sharing_term(SharingDomainUserTerm, SharingDomain) :-
         SharingPairTerms = [],
         SharingDomain = structure_sharing_bottom
     ;
-        SharingPairTerms = [_|_],
+        SharingPairTerms = [_ | _],
         list.map(parse_user_annotated_sharing_pair_term, SharingPairTerms,
             SharingPairs),
         SharingDomain = structure_sharing_real(SharingPairs)
@@ -496,7 +504,8 @@ parse_user_annotated_datastruct_term(Term, Datastruct) :-
     VarTerm = term.variable(GenericVar, _),
     term.coerce_var(GenericVar, ProgVar),
     get_list_term_arguments(TypesTerm, TypeTermsList),
-    maybe_parse_types(TypeTermsList, Types),
+    maybe_parse_types(no_allow_ho_inst_info(wnhii_user_struct_sharing),
+        TypeTermsList, Types),
     list.map(mer_type_to_typesel, Types, Selector),
     Datastruct = selected_cel(ProgVar, Selector).
 
@@ -506,7 +515,7 @@ mer_type_to_typesel(Type, typesel(Type)).
 
 %-----------------------------------------------------------------------------%
 %
-% Printing routines
+% Printing routines.
 %
 
 :- func selector_to_string(tvarset, selector) = string.
@@ -527,7 +536,7 @@ selector_to_string(TVarSet, Selector) = String :-
 
 unit_selector_to_string(_, termsel(ConsId, Index)) =
     string.append_list(["sel(",
-        mercury_cons_id_to_string(needs_brackets, ConsId),
+        mercury_cons_id_to_string(output_mercury, needs_brackets, ConsId),
         ",",
         int_to_string(cons_id_arity(ConsId)),
         ",",
@@ -536,40 +545,44 @@ unit_selector_to_string(_, termsel(ConsId, Index)) =
 
 unit_selector_to_string(TVarSet, typesel(TypeSel)) =
     string.append_list(["typesel(",
-        mercury_type_to_string(TVarSet, bool.no, TypeSel),
+        mercury_type_to_string(TVarSet, print_name_only, TypeSel),
         ")"]).
 
-print_selector(TVarSet, Selector, !IO) :-
-    io.write_string(selector_to_string(TVarSet, Selector), !IO).
+print_selector(Stream, TVarSet, Selector, !IO) :-
+    io.write_string(Stream, selector_to_string(TVarSet, Selector), !IO).
 
-print_datastruct(ProgVarSet, TypeVarSet, DataStruct, !IO) :-
+print_datastruct(Stream, ProgVarSet, TypeVarSet, DataStruct, !IO) :-
     varset.lookup_name(ProgVarSet, DataStruct ^ sc_var, VarName),
-    io.write_strings(["cel(", VarName, ", "], !IO),
-    print_selector(TypeVarSet, DataStruct ^ sc_selector, !IO),
-    io.write_string(")", !IO).
+    io.write_strings(Stream, ["cel(", VarName, ", "], !IO),
+    print_selector(Stream, TypeVarSet, DataStruct ^ sc_selector, !IO),
+    io.write_string(Stream, ")", !IO).
 
-print_datastructs(ProgVarSet, TypeVarSet, Datastructs, !IO) :-
-    io.write_string("[", !IO),
-    io.write_list(Datastructs, ", ", print_datastruct(ProgVarSet, TypeVarSet),
-        !IO),
-    io.write_string("]", !IO).
+print_datastructs(Stream, ProgVarSet, TypeVarSet, Datastructs, !IO) :-
+    io.write_string(Stream, "[", !IO),
+    % XXX Specifying the stream to print the items and the separators
+    % is suboptimal, but in the absence of active work on ctgc, we
+    % probably don't care.
+    io.write_list(Stream, Datastructs, ", ",
+        print_datastruct(Stream, ProgVarSet, TypeVarSet), !IO),
+    io.write_string(Stream, "]", !IO).
 
-print_structure_sharing_pair(ProgVarSet, TypeVarSet, SharingPair, !IO) :-
+print_structure_sharing_pair(Stream, ProgVarSet, TypeVarSet,
+        SharingPair, !IO) :-
     SharingPair = D1 - D2,
-    io.write_string("pair(", !IO),
-    print_datastruct(ProgVarSet, TypeVarSet, D1, !IO),
-    io.write_string(", ", !IO),
-    print_datastruct(ProgVarSet, TypeVarSet, D2, !IO),
-    io.write_string(")", !IO).
+    io.write_string(Stream, "pair(", !IO),
+    print_datastruct(Stream, ProgVarSet, TypeVarSet, D1, !IO),
+    io.write_string(Stream, ", ", !IO),
+    print_datastruct(Stream, ProgVarSet, TypeVarSet, D2, !IO),
+    io.write_string(Stream, ")", !IO).
 
-print_structure_sharing(ProgVarSet, TypeVarSet, MaybeLimit, Start, Sep, End,
-        SharingPairs0, !IO) :-
+print_structure_sharing(Stream, ProgVarSet, TypeVarSet, MaybeLimit,
+        Start, Sep, End, SharingPairs0, !IO) :-
     (
         MaybeLimit = yes(Limit),
         list.take_upto(Limit, SharingPairs0, SharingPairs),
-        ( Limit >= list.length(SharingPairs0) ->
+        ( if Limit >= list.length(SharingPairs0) then
             CompleteList = yes
-        ;
+        else
             CompleteList = no
         )
     ;
@@ -577,135 +590,151 @@ print_structure_sharing(ProgVarSet, TypeVarSet, MaybeLimit, Start, Sep, End,
         SharingPairs = SharingPairs0,
         CompleteList = yes
     ),
-    io.write_string(Start, !IO),
-    io.write_list(SharingPairs, Sep,
-        print_structure_sharing_pair(ProgVarSet, TypeVarSet), !IO),
+    io.write_string(Stream, Start, !IO),
+    % XXX Specifying the stream to print the items and the separators
+    % is suboptimal, but in the absence of active work on ctgc, we
+    % probably don't care.
+    io.write_list(Stream, SharingPairs, Sep,
+        print_structure_sharing_pair(Stream, ProgVarSet, TypeVarSet), !IO),
     (
         CompleteList = no,
-        io.write_string(Sep, !IO),
-        io.write_string("...", !IO)
+        io.write_string(Stream, Sep, !IO),
+        io.write_string(Stream, "...", !IO)
     ;
         CompleteList = yes
     ),
-    io.write_string(End, !IO).
+    io.write_string(Stream, End, !IO).
 
-print_structure_sharing_as_list(ProgVarSet, TypeVarSet, SharingPairs, !IO) :-
-    print_structure_sharing(ProgVarSet, TypeVarSet, no,
+print_structure_sharing_as_list(Stream, ProgVarSet, TypeVarSet,
+        SharingPairs, !IO) :-
+    print_structure_sharing(Stream, ProgVarSet, TypeVarSet, no,
         "[", ",", "]", SharingPairs, !IO).
 
-print_structure_sharing_domain(ProgVarSet, TypeVarSet, VerboseTop,
+print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet, VerboseTop,
         MaybeThreshold, SharingAs, !IO) :-
-    do_print_structure_sharing_domain(ProgVarSet, TypeVarSet, VerboseTop,
-        MaybeThreshold, "", ", ", "", SharingAs, !IO).
+    do_print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet,
+        VerboseTop, MaybeThreshold, "", ", ", "", SharingAs, !IO).
 
-:- pred do_print_structure_sharing_domain(prog_varset::in, tvarset::in,
-    bool::in, maybe(int)::in, string::in, string::in, string::in,
-    structure_sharing_domain::in, io::di, io::uo) is det.
+:- pred do_print_structure_sharing_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, bool::in, maybe(int)::in,
+    string::in, string::in, string::in, structure_sharing_domain::in,
+    io::di, io::uo) is det.
 
-do_print_structure_sharing_domain(ProgVarSet, TypeVarSet, VerboseTop,
+do_print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet, VerboseTop,
         MaybeThreshold, Start, Separator, End, SharingAs, !IO) :-
     io.write_string(Start, !IO),
     (
         SharingAs = structure_sharing_top(Msgs),
         (
             VerboseTop = no,
-            io.write_string("top", !IO)
+            io.write_string(Stream, "top", !IO)
         ;
             VerboseTop = yes,
-            io.write_string("top([", !IO),
-            io.write_list(set.to_sorted_list(Msgs), Separator, io.write, !IO),
-            io.write_string("])", !IO)
+            io.write_string(Stream, "top([", !IO),
+            % XXX Specifying the stream to print the items and the separators
+            % is suboptimal, but in the absence of active work on ctgc, we
+            % probably don't care.
+            io.write_list(Stream, set.to_sorted_list(Msgs), Separator,
+                io.write(Stream), !IO),
+            io.write_string(Stream, "])", !IO)
         )
     ;
         SharingAs = structure_sharing_bottom,
-        io.write_string("bottom", !IO)
+        io.write_string(Stream, "bottom", !IO)
     ;
         SharingAs = structure_sharing_real(SharingPairs),
-        print_structure_sharing(ProgVarSet, TypeVarSet,
+        print_structure_sharing(Stream, ProgVarSet, TypeVarSet,
             MaybeThreshold, "[", Separator, "]", SharingPairs, !IO)
     ),
-    io.write_string(End, !IO).
+    io.write_string(Stream, End, !IO).
 
-dump_maybe_structure_sharing_domain(_, _, no, !IO) :-
-    io.write_string("% no sharing information available.\n", !IO).
-dump_maybe_structure_sharing_domain(ProgVarSet, TypeVarSet, yes(SharingAs),
-        !IO) :-
-    do_print_structure_sharing_domain(ProgVarSet, TypeVarSet, yes,
+dump_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet, SharingAs, !IO) :-
+    do_print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet, yes,
         no, "%\t ", "\n%\t", "\n", SharingAs, !IO).
 
-print_interface_structure_sharing_domain(_, _, no, !IO) :-
-    io.write_string("not_available", !IO).
-print_interface_structure_sharing_domain(ProgVarSet, TypeVarSet,
-        yes(SharingAs), !IO) :-
-    io.write_string("yes(", !IO),
-    print_structure_sharing_domain(ProgVarSet, TypeVarSet, no, no, SharingAs,
-        !IO),
-    io.write_string(")", !IO).
+print_interface_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet,
+        MaybeSharingAs, !IO) :-
+    (
+        MaybeSharingAs = no,
+        io.write_string(Stream, "not_available", !IO)
+    ;
+        MaybeSharingAs = yes(SharingAs),
+        io.write_string(Stream, "yes(", !IO),
+        print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet,
+            no, no, SharingAs, !IO),
+        io.write_string(Stream, ")", !IO)
+    ).
 
-dump_maybe_structure_reuse_domain(_, _, no, !IO) :-
-    io.write_string("% no reuse information available.\n", !IO).
-dump_maybe_structure_reuse_domain(ProgVarSet, TypeVarSet, yes(ReuseAs), !IO) :-
-    print_structure_reuse_domain(ProgVarSet, TypeVarSet, ReuseAs,
+dump_structure_reuse_domain(Stream, ProgVarSet, TypeVarSet, ReuseAs, !IO) :-
+    print_structure_reuse_domain(Stream, ProgVarSet, TypeVarSet, ReuseAs,
         "%\t ", ", \n%\t ", "\n", !IO).
 
-:- pred print_structure_reuse_condition(prog_varset::in, tvarset::in,
-    structure_reuse_condition::in, io::di, io::uo) is det.
-
-print_structure_reuse_condition(ProgVarSet, TypeVarSet, ReuseCond, !IO) :-
-    ReuseCond = structure_reuse_condition(DeadNodes, InUseNodes, Sharing),
-    DeadNodesList = set.to_sorted_list(DeadNodes),
-    io.write_string("condition(", !IO),
-    print_datastructs(ProgVarSet, TypeVarSet, DeadNodesList, !IO),
-    io.write_string(", ", !IO),
-    print_datastructs(ProgVarSet, TypeVarSet, InUseNodes, !IO),
-    io.write_string(", ", !IO),
-    print_structure_sharing_domain(ProgVarSet, TypeVarSet, no, no,
-        Sharing, !IO),
-    io.write_string(")", !IO).
-
-:- pred print_structure_reuse_conditions(prog_varset::in, tvarset::in,
-    string::in, structure_reuse_conditions::in, io::di, io::uo) is det.
-
-print_structure_reuse_conditions(ProgVarSet, TypeVarSet, Separator, ReuseConds,
-        !IO) :-
-    io.write_list(ReuseConds, Separator,
-        print_structure_reuse_condition(ProgVarSet, TypeVarSet), !IO).
-
-:- pred print_structure_reuse_domain(prog_varset::in, tvarset::in,
-    structure_reuse_domain::in, string::in, string::in, string::in,
+:- pred print_structure_reuse_condition(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_reuse_condition::in,
     io::di, io::uo) is det.
 
-print_structure_reuse_domain(ProgVarSet, TypeVarSet, ReuseDomain,
+print_structure_reuse_condition(Stream, ProgVarSet, TypeVarSet, ReuseCond, !IO) :-
+    ReuseCond = structure_reuse_condition(DeadNodes, InUseNodes, Sharing),
+    DeadNodesList = set.to_sorted_list(DeadNodes),
+    io.write_string(Stream, "condition(", !IO),
+    print_datastructs(Stream, ProgVarSet, TypeVarSet, DeadNodesList, !IO),
+    io.write_string(Stream, ", ", !IO),
+    print_datastructs(Stream, ProgVarSet, TypeVarSet, InUseNodes, !IO),
+    io.write_string(Stream, ", ", !IO),
+    print_structure_sharing_domain(Stream, ProgVarSet, TypeVarSet, no, no,
+        Sharing, !IO),
+    io.write_string(Stream, ")", !IO).
+
+:- pred print_structure_reuse_conditions(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, string::in, structure_reuse_conditions::in,
+    io::di, io::uo) is det.
+
+print_structure_reuse_conditions(Stream, ProgVarSet, TypeVarSet, Separator,
+        ReuseConds, !IO) :-
+    % XXX Specifying the stream to print the items and the separators
+    % is suboptimal, but in the absence of active work on ctgc, we
+    % probably don't care.
+    io.write_list(Stream, ReuseConds, Separator,
+        print_structure_reuse_condition(Stream, ProgVarSet, TypeVarSet), !IO).
+
+:- pred print_structure_reuse_domain(io.text_output_stream::in,
+    prog_varset::in, tvarset::in, structure_reuse_domain::in,
+    string::in, string::in, string::in, io::di, io::uo) is det.
+
+print_structure_reuse_domain(Stream, ProgVarSet, TypeVarSet, ReuseDomain,
         Start, Separator, End, !IO) :-
-    io.write_string(Start, !IO),
+    io.write_string(Stream, Start, !IO),
     (
         ReuseDomain = has_no_reuse,
-        io.write_string("has_no_reuse", !IO)
+        io.write_string(Stream, "has_no_reuse", !IO)
     ;
         ReuseDomain = has_only_unconditional_reuse,
-        io.write_string("has_only_unconditional_reuse", !IO)
+        io.write_string(Stream, "has_only_unconditional_reuse", !IO)
     ;
         ReuseDomain = has_conditional_reuse(ReuseConditions),
-        io.write_string("has_conditional_reuse([", !IO),
-        print_structure_reuse_conditions(ProgVarSet, TypeVarSet, Separator,
-            ReuseConditions, !IO),
-        io.write_string("])", !IO)
+        io.write_string(Stream, "has_conditional_reuse([", !IO),
+        print_structure_reuse_conditions(Stream, ProgVarSet, TypeVarSet,
+            Separator, ReuseConditions, !IO),
+        io.write_string(Stream, "])", !IO)
     ),
     io.write_string(End, !IO).
 
-print_interface_maybe_structure_reuse_domain(_, _, no, !IO) :-
-    io.write_string("not_available", !IO).
-
-print_interface_maybe_structure_reuse_domain(ProgVarSet, TypeVarSet,
-        yes(ReuseDomain), !IO) :-
-    io.write_string("yes(", !IO),
-    print_structure_reuse_domain(ProgVarSet, TypeVarSet, ReuseDomain,
-        "", ", ", "", !IO),
-    io.write_string(")", !IO).
+print_interface_maybe_structure_reuse_domain(Stream, ProgVarSet, TypeVarSet,
+        MaybeReuseDomain, !IO) :-
+    (
+        MaybeReuseDomain = no,
+        io.write_string(Stream, "not_available", !IO)
+    ;
+        MaybeReuseDomain = yes(ReuseDomain),
+        io.write_string(Stream, "yes(", !IO),
+        print_structure_reuse_domain(Stream, ProgVarSet, TypeVarSet,
+            ReuseDomain, "", ", ", "", !IO),
+        io.write_string(Stream, ")", !IO)
+    ).
 
 %-----------------------------------------------------------------------------%
 %
-% Renaming operations
+% Renaming operations.
 %
 
 rename_unit_selector(Subst, !UnitSelector) :-
@@ -760,13 +789,13 @@ rename_user_annotated_sharing(HeadVars, NewHeadVars, NewTypes,
                 !.SharingDomain = structure_sharing_real(SharingPairs),
                 map.from_corresponding_lists(HeadVars, NewHeadVars,
                     VarRenaming),
-                (
+                ( if
                     MaybeTypes = yes(user_type_info(UserSharingTypes,
                         _UserSharingTVarSet))
-                ->
+                then
                     type_list_subsumes_det(UserSharingTypes, NewTypes,
                         TypeSubst)
-                ;
+                else
                     TypeSubst = map.init
                 ),
                 rename_structure_sharing(VarRenaming, TypeSubst,
