@@ -85,6 +85,9 @@
 :- import_module hlds.hlds_clauses.
 :- import_module hlds.hlds_error_util.
 :- import_module hlds.hlds_goal.
+:- import_module hlds.make_goal.
+:- import_module hlds.status.
+:- import_module hlds.vartypes.
 :- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
@@ -171,10 +174,10 @@ do_prop_mode_constraints_in_pred(ModuleInfo, PredId, !PredInfo, !VarInfo,
     % If mode inference requested, just add constraints for the clause body,
     % otherwise, process the predicate for each of the procedures.
 
-    ( pred_info_infer_modes(!.PredInfo) ->
+    ( if pred_info_infer_modes(!.PredInfo) then
         add_clauses_constraints(ModuleInfo, PredId, !.PredInfo, !VarInfo,
             init_pred_p_c_constraints, BodyConstraints)
-    ;
+    else
         prop_mode_constraints_in_mode_declared_pred(ModuleInfo, PredId,
             !.PredInfo, !VarInfo, BodyConstraints)
     ),
@@ -236,7 +239,7 @@ ensure_unique_arguments(PredId, !ModuleInfo) :-
     clauses_info_get_headvars(ClausesInfo0, HeadVars),
 
     SeenSoFar = set_of_var.list_to_set(proc_arg_vector_to_list(HeadVars)),
-    get_clause_list(ClausesRep0, Clauses0),
+    get_clause_list_for_replacement(ClausesRep0, Clauses0),
     BodyGoals0 = list.map(func(X) = clause_body(X), Clauses0),
     list.map_foldl3(ensure_unique_arguments_in_goal, BodyGoals0, BodyGoals,
         SeenSoFar, _, VarSet0, VarSet, VarTypes0, VarTypes),
@@ -324,7 +327,7 @@ ensure_unique_arguments_in_goal(!Goal, !SeenSoFar, !VarSet, !VarTypes) :-
             )
         ;
             !.GoalExpr = switch(_SwitchVar, _CanFail, _Cases0),
-            unexpected($module, $pred, "switch")
+            unexpected($pred, "switch")
         ;
             !.GoalExpr = unify(_, _, _, _, _)
         ;
@@ -375,7 +378,7 @@ ensure_unique_arguments_in_goal(!Goal, !SeenSoFar, !VarSet, !VarTypes) :-
                 !:GoalExpr = shorthand(ShortHand)
             ;
                 ShortHand0 = bi_implication(_, _),
-                unexpected($module, $pred, "bi_implication")
+                unexpected($pred, "bi_implication")
             )
         ),
         !:Goal = hlds_goal(!.GoalExpr, !.GoalInfo)
@@ -397,9 +400,9 @@ flatten_conjunction(!Goals) :-
     list(hlds_goal)::in, list(hlds_goal)::out) is det.
 
 add_to_before_conjunction(Goal, !Goals) :-
-    ( Goal = hlds_goal(conj(plain_conj, SubGoals), _) ->
+    ( if Goal = hlds_goal(conj(plain_conj, SubGoals), _) then
         !:Goals = SubGoals ++ !.Goals
-    ;
+    else
         !:Goals = [Goal | !.Goals]
     ).
 
@@ -433,7 +436,7 @@ make_unifications(Context, Unifications, !Args, !SeenSoFar,
 
 make_unification(Context, Var0, Var, !Unifications, !SeenSoFar, !VarSet,
         !VarTypes) :-
-    ( set_of_var.contains(!.SeenSoFar, Var0) ->
+    ( if set_of_var.contains(!.SeenSoFar, Var0) then
         % Make new variable.
         OldVarName = varset.lookup_name(!.VarSet, Var0),
         lookup_var_type(!.VarTypes, Var0, OldVarType),
@@ -452,7 +455,7 @@ make_unification(Context, Var0, Var, !Unifications, !SeenSoFar, !VarSet,
         UnificationGoal =
             hlds_goal(UnificationGoalExpr, UnificationGoalInfo),
         !:Unifications = [UnificationGoal | !.Unifications]
-    ;
+    else
         Var = Var0
     ),
     set_of_var.insert(Var, !SeenSoFar).
@@ -494,8 +497,8 @@ module_info_pred_status_is_imported(ModuleInfo, PredId) :-
 
     % The following used because pred_info_is_imported/2 is not
     % as comprehensive as status_is_imported/2.
-    pred_info_get_import_status(PredInfo, Status),
-    status_is_imported(Status) = yes.
+    pred_info_get_status(PredInfo, Status),
+    pred_status_is_imported(Status) = yes.
 
 %----------------------------------------------------------------------------%
 
