@@ -19,8 +19,10 @@
 :- module transform_hlds.implicit_parallelism.push_goals_together.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module mdbcomp.
 :- import_module mdbcomp.feedback.
 :- import_module mdbcomp.feedback.automatic_parallelism.
 
@@ -49,26 +51,28 @@
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_goal.
 :- import_module hlds.hlds_out.hlds_out_util.
-:- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.quantification.
-:- import_module mdbcomp.feedback.
-:- import_module mdbcomp.feedback.automatic_parallelism.
+:- import_module libs.
+:- import_module libs.globals.
 :- import_module mdbcomp.goal_path.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
+:- import_module parse_tree.parse_tree_out_info.
 :- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
-:- import_module bool.
 :- import_module int.
 :- import_module io.
-:- import_module list.
 :- import_module map.
+:- import_module one_or_more.
 :- import_module pair.
 :- import_module require.
 :- import_module term.
@@ -77,31 +81,36 @@
 
 :- type push_info
     --->    push_info(
-                pi_rtti_varmaps         ::  rtti_varmaps
+                pi_globals              :: globals,
+                pi_module_name          :: module_name,
+                pi_rtti_varmaps         :: rtti_varmaps
             ).
 
 push_goals_in_proc(PushGoals, OverallResult, !ProcInfo, !ModuleInfo) :-
+    module_info_get_globals(!.ModuleInfo, Globals),
+    module_info_get_name(!.ModuleInfo, ModuleName),
     proc_info_get_goal(!.ProcInfo, Goal0),
     proc_info_get_varset(!.ProcInfo, VarSet0),
     proc_info_get_vartypes(!.ProcInfo, VarTypes0),
     proc_info_get_rtti_varmaps(!.ProcInfo, RttiVarMaps0),
-    PushInfo = push_info(RttiVarMaps0),
-    module_info_get_globals(!.ModuleInfo, Globals),
-    OutInfo = init_hlds_out_info(Globals),
+    PushInfo = push_info(Globals, ModuleName, RttiVarMaps0),
+    OutInfo = init_hlds_out_info(Globals, output_debug),
     trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-        io.write_string("Goal before pushes:\n", !IO),
-        write_goal(OutInfo, Goal0, !.ModuleInfo, VarSet0, yes, 0, "", !IO),
-        io.nl(!IO)
+        get_debug_output_stream(Globals, ModuleName, DebugStream, !IO),
+        io.write_string(DebugStream, "Goal before pushes:\n", !IO),
+        write_goal_nl(OutInfo, DebugStream, !.ModuleInfo, VarSet0,
+            print_name_and_num, 0, "", Goal0, !IO)
     ),
-    do_push_list(PushGoals, PushInfo, OverallResult, Goal0, Goal1),
+    do_push_list(PushInfo, PushGoals, OverallResult, Goal0, Goal1),
     (
         OverallResult = push_failed
     ;
         OverallResult = push_succeeded,
         trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-            io.write_string("Goal after pushes:\n", !IO),
-            write_goal(OutInfo, Goal1, !.ModuleInfo, VarSet0, yes, 0, "", !IO),
-            io.nl(!IO)
+            get_debug_output_stream(Globals, ModuleName, DebugStream, !IO),
+            io.write_string(DebugStream, "Goal after pushes:\n", !IO),
+            write_goal_nl(OutInfo, DebugStream, !.ModuleInfo, VarSet0,
+                print_name_and_num, 0, "", Goal1, !IO)
         ),
 
         % We need to fix up the goal_infos of the goals touched directly or
@@ -124,7 +133,7 @@ push_goals_in_proc(PushGoals, OverallResult, !ProcInfo, !ModuleInfo) :-
             HeadVars, _Warnings, Goal1, Goal2,
             VarSet0, VarSet, VarTypes0, VarTypes,
             RttiVarMaps0, RttiVarMaps),
-        proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InstMap0),
+        proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InstMap0),
         proc_info_get_inst_varset(!.ProcInfo, InstVarSet),
         recompute_instmap_delta(do_not_recompute_atomic_instmap_deltas,
             Goal2, Goal, VarTypes, InstVarSet, InstMap0, !ModuleInfo),
@@ -133,151 +142,166 @@ push_goals_in_proc(PushGoals, OverallResult, !ProcInfo, !ModuleInfo) :-
         proc_info_set_vartypes(VarTypes, !ProcInfo),
         proc_info_set_rtti_varmaps(RttiVarMaps, !ProcInfo),
         trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-            io.write_string("Goal after fixups:\n", !IO),
-            write_goal(OutInfo, Goal, !.ModuleInfo, VarSet, yes, 0, "", !IO),
-            io.nl(!IO)
+            get_debug_output_stream(Globals, ModuleName, DebugStream, !IO),
+            io.write_string(DebugStream, "Goal after fixups:\n", !IO),
+            write_goal_nl(OutInfo, DebugStream, !.ModuleInfo, VarSet,
+                print_name_and_num, 0, "", Goal, !IO)
         )
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred do_push_list(list(push_goal)::in, push_info::in,
+:- pred do_push_list(push_info::in, list(push_goal)::in,
     push_result::out, hlds_goal::in, hlds_goal::out) is det.
 
-do_push_list([], _, push_succeeded, !Goal).
-do_push_list([PushGoal | PushGoals], PushInfo, OverallResult, !Goal) :-
-    do_one_push(PushGoal, PushInfo, Result, !Goal),
+do_push_list(_, [], push_succeeded, !Goal).
+do_push_list(PushInfo, [PushGoal | PushGoals], OverallResult, !Goal) :-
+    do_one_push(PushInfo, PushGoal, Result, !Goal),
     (
         Result = push_succeeded,
-        do_push_list(PushGoals, PushInfo, OverallResult, !Goal)
+        do_push_list(PushInfo, PushGoals, OverallResult, !Goal)
     ;
         Result = push_failed,
         OverallResult = push_failed
     ).
 
-:- pred do_one_push(push_goal::in, push_info::in,
+:- pred do_one_push(push_info::in, push_goal::in,
     push_result::out, hlds_goal::in, hlds_goal::out) is det.
 
-do_one_push(PushGoal, PushInfo, Result, !Goal) :-
+do_one_push(PushInfo, PushGoal, Result, !Goal) :-
     PushGoal = push_goal(GoalPathStr, _Lo, _Hi, _PushedInto),
-    ( goal_path_from_string(GoalPathStr, GoalPath) ->
-        do_push_in_goal(GoalPath, PushGoal, PushInfo, Result, !Goal)
-    ;
+    ( if goal_path_from_string(GoalPathStr, GoalPath) then
+        do_push_in_goal(PushInfo, GoalPath, PushGoal, Result, !Goal)
+    else
         Result = push_failed,
         trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-            io.write_string("push_failed: cannot translate goal path\n", !IO)
+            get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+            io.write_string(DebugStream,
+                "push_failed: cannot translate goal path\n", !IO)
         )
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred do_push_in_goal(forward_goal_path::in, push_goal::in, push_info::in,
+:- pred do_push_in_goal(push_info::in, forward_goal_path::in, push_goal::in,
     push_result::out, hlds_goal::in, hlds_goal::out) is det.
 
-do_push_in_goal(fgp_nil, PushGoal, PushInfo, Result, !Goal) :-
+do_push_in_goal(PushInfo, fgp_nil, PushGoal, Result, !Goal) :-
     % We have arrives at the goal in which the push should take place.
-    perform_push_transform(PushGoal, PushInfo, Result, !Goal).
-do_push_in_goal(fgp_cons(Step, Path), PushGoal, PushInfo, Result, !Goal) :-
+    perform_push_transform(PushInfo, PushGoal, Result, !Goal).
+do_push_in_goal(PushInfo, fgp_cons(Step, Path), PushGoal, Result, !Goal) :-
     !.Goal = hlds_goal(GoalExpr0, GoalInfo0),
     (
         Step = step_conj(N),
-        ( GoalExpr0 = conj(ConjType, Goals0) ->
-            do_push_in_goals(N, Path, PushGoal, PushInfo, Result,
+        ( if GoalExpr0 = conj(ConjType, Goals0) then
+            do_push_in_goals(PushInfo, N, Path, PushGoal, Result,
                 Goals0, Goals),
             GoalExpr = conj(ConjType, Goals),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not conj\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream, "push_failed: not conj\n", !IO)
             )
         )
     ;
         Step = step_disj(N),
-        ( GoalExpr0 = disj(Goals0) ->
-            do_push_in_goals(N, Path, PushGoal, PushInfo, Result,
+        ( if GoalExpr0 = disj(Goals0) then
+            do_push_in_goals(PushInfo, N, Path, PushGoal, Result,
                 Goals0, Goals),
             GoalExpr = disj(Goals),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not disj\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream, "push_failed: not disj\n", !IO)
             )
         )
     ;
         Step = step_switch(N, _),
-        ( GoalExpr0 = switch(Var, CanFail, Cases0) ->
-            do_push_in_cases(N, Path, PushGoal, PushInfo, Result,
+        ( if GoalExpr0 = switch(Var, CanFail, Cases0) then
+            do_push_in_cases(PushInfo, N, Path, PushGoal, Result,
                 Cases0, Cases),
             GoalExpr = switch(Var, CanFail, Cases),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not switch\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream, "push_failed: not switch\n", !IO)
             )
         )
     ;
         Step = step_ite_cond,
-        ( GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) ->
-            do_push_in_goal(Path, PushGoal, PushInfo, Result, Cond0, Cond),
+        ( if GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) then
+            do_push_in_goal(PushInfo, Path, PushGoal, Result, Cond0, Cond),
             GoalExpr = if_then_else(Vars0, Cond, Then0, Else0),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not if_then_else\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream,
+                    "push_failed: not if_then_else\n", !IO)
             )
         )
     ;
         Step = step_ite_then,
-        ( GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) ->
-            do_push_in_goal(Path, PushGoal, PushInfo, Result, Then0, Then),
+        ( if GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) then
+            do_push_in_goal(PushInfo, Path, PushGoal, Result, Then0, Then),
             GoalExpr = if_then_else(Vars0, Cond0, Then, Else0),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not if_then_else\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream,
+                    "push_failed: not if_then_else\n", !IO)
             )
         )
     ;
         Step = step_ite_else,
-        ( GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) ->
-            do_push_in_goal(Path, PushGoal, PushInfo, Result, Else0, Else),
+        ( if GoalExpr0 = if_then_else(Vars0, Cond0, Then0, Else0) then
+            do_push_in_goal(PushInfo, Path, PushGoal, Result, Else0, Else),
             GoalExpr = if_then_else(Vars0, Cond0, Then0, Else),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not if_then_else\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream,
+                    "push_failed: not if_then_else\n", !IO)
             )
         )
     ;
         Step = step_neg,
-        ( GoalExpr0 = negation(SubGoal0) ->
-            do_push_in_goal(Path, PushGoal, PushInfo, Result,
+        ( if GoalExpr0 = negation(SubGoal0) then
+            do_push_in_goal(PushInfo, Path, PushGoal, Result,
                 SubGoal0, SubGoal),
             GoalExpr = negation(SubGoal),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not negation\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream,
+                    "push_failed: not negation\n", !IO)
             )
         )
     ;
         Step = step_scope(_),
-        ( GoalExpr0 = scope(Reason, SubGoal0) ->
-            do_push_in_goal(Path, PushGoal, PushInfo, Result,
+        ( if GoalExpr0 = scope(Reason, SubGoal0) then
+            do_push_in_goal(PushInfo, Path, PushGoal, Result,
                 SubGoal0, SubGoal),
             GoalExpr = scope(Reason, SubGoal),
             !:Goal = hlds_goal(GoalExpr, GoalInfo0)
-        ;
+        else
             Result = push_failed,
             trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-                io.write_string("push_failed: not scope\n", !IO)
+                get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+                io.write_string(DebugStream, "push_failed: not scope\n", !IO)
             )
         )
     ;
@@ -290,59 +314,65 @@ do_push_in_goal(fgp_cons(Step, Path), PushGoal, PushInfo, Result, !Goal) :-
         % expanded out by now.
         Result = push_failed,
         trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-            io.write_string("push_failed: unexpected goal path step\n", !IO)
+            get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+            io.write_string(DebugStream,
+                "push_failed: unexpected goal path step\n", !IO)
         )
     ).
 
-:- pred do_push_in_goals(int::in, forward_goal_path::in, push_goal::in,
-    push_info::in, push_result::out,
+:- pred do_push_in_goals(push_info::in, int::in, forward_goal_path::in,
+    push_goal::in, push_result::out,
     list(hlds_goal)::in, list(hlds_goal)::out) is det.
 
-do_push_in_goals(_N, _Path, _PushGoal, _PushInfo, push_failed, [], []) :-
+do_push_in_goals(PushInfo, _N, _Path, _PushGoal, push_failed, [], []) :-
     trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-        io.write_string("push_failed: couldn't find indicated disjunct\n", !IO)
+        get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+        io.write_string(DebugStream,
+            "push_failed: couldn't find indicated disjunct\n", !IO)
     ).
-do_push_in_goals(N, Path, PushGoal, PushInfo, Result,
+do_push_in_goals(PushInfo, N, Path, PushGoal, Result,
         [Goal0 | Goals0], [Goal | Goals]) :-
-    ( N = 1 ->
-        do_push_in_goal(Path, PushGoal, PushInfo, Result, Goal0, Goal),
+    ( if N = 1 then
+        do_push_in_goal(PushInfo, Path, PushGoal, Result, Goal0, Goal),
         Goals = Goals0
-    ;
+    else
         Goal = Goal0,
-        do_push_in_goals(N - 1, Path, PushGoal, PushInfo, Result,
+        do_push_in_goals(PushInfo, N - 1, Path, PushGoal, Result,
             Goals0, Goals)
     ).
 
-:- pred do_push_in_cases(int::in, forward_goal_path::in, push_goal::in,
-    push_info::in, push_result::out, list(case)::in, list(case)::out) is det.
+:- pred do_push_in_cases(push_info::in, int::in, forward_goal_path::in,
+    push_goal::in, push_result::out, list(case)::in, list(case)::out) is det.
 
-do_push_in_cases(_N, _Path, _PushGoal, _PushInfo, push_failed, [], []) :-
+do_push_in_cases(PushInfo, _N, _Path, _PushGoal, push_failed, [], []) :-
     trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-        io.write_string("push_failed: couldn't find indicated case\n", !IO)
+        get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+        io.write_string(DebugStream,
+            "push_failed: couldn't find indicated case\n", !IO)
     ).
-do_push_in_cases(N, Path, PushGoal, PushInfo, Result,
+do_push_in_cases(PushInfo, N, Path, PushGoal, Result,
         [Case0 | Cases0], [Case | Cases]) :-
-    ( N = 1 ->
+    ( if N = 1 then
         Case0 = case(MainConsId, OtherConsIds, Goal0),
-        do_push_in_goal(Path, PushGoal, PushInfo, Result, Goal0, Goal),
+        do_push_in_goal(PushInfo, Path, PushGoal, Result, Goal0, Goal),
         Case = case(MainConsId, OtherConsIds, Goal),
         Cases = Cases0
-    ;
+    else
         Case = Case0,
-        do_push_in_cases(N - 1, Path, PushGoal, PushInfo, Result,
+        do_push_in_cases(PushInfo, N - 1, Path, PushGoal, Result,
             Cases0, Cases)
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred perform_push_transform(push_goal::in, push_info::in,
+:- pred perform_push_transform(push_info::in, push_goal::in,
     push_result::out, hlds_goal::in, hlds_goal::out) is det.
 
-perform_push_transform(PushGoal, PushInfo, Result, !Goal) :-
+perform_push_transform(PushInfo, PushGoal, Result, !Goal) :-
     PushGoal = push_goal(GoalPathStr, Lo, Hi, PushedInto),
     goal_path_from_string_det(GoalPathStr, GoalPath),
     !.Goal = hlds_goal(GoalExpr0, GoalInfo0),
-    (
+    ( if
         GoalExpr0 = conj(plain_conj, Conjuncts),
         find_lo_hi_goals(PushInfo, Conjuncts, Lo, Hi, 1, Before0, LoHi, After,
             pushable),
@@ -358,15 +388,17 @@ perform_push_transform(PushGoal, PushInfo, Result, !Goal) :-
         % before Lo, then this transformation reorders the code.
         % For now, we don't allow that.
         PushConjNum + 1 = Lo
-    ->
+    then
         list.det_replace_nth(Before0, PushConjNum, PushIntoGoal, Before),
         GoalExpr = conj(plain_conj, Before ++ After),
         !:Goal = hlds_goal(GoalExpr, GoalInfo0),
         Result = push_succeeded
-    ;
+    else
         Result = push_failed,
         trace [compiletime(flag("debug_push_goals")), io(!IO)] (
-            io.write_string("push_failed: perform_push_transform\n", !IO)
+            get_push_debug_output_stream(PushInfo, DebugStream, !IO),
+            io.write_string(DebugStream,
+                "push_failed: perform_push_transform\n", !IO)
         )
     ).
 
@@ -391,10 +423,10 @@ perform_push_transform(PushGoal, PushInfo, Result, !Goal) :-
 
 find_lo_hi_goals(PushInfo, Conjuncts, Lo, Hi, Cur, Before, LoHi, After,
         Pushable) :-
-    ( Cur = Lo ->
+    ( if Cur = Lo then
         find_hi_goals(PushInfo, Conjuncts, Hi, Cur, LoHi, After, Pushable),
         Before = []
-    ;
+    else
         (
             Conjuncts = [],
             Before = [],
@@ -428,11 +460,11 @@ find_hi_goals(PushInfo, [Head | Tail], Hi, Cur, LoHi, After, Pushable) :-
     is_pushable_goal(PushInfo, Head, HeadPushable),
     (
         HeadPushable = pushable,
-        ( Cur = Hi ->
+        ( if Cur = Hi then
             LoHi = [Head],
             After = Tail,
             Pushable = pushable
-        ;
+        else
             find_hi_goals(PushInfo, Tail, Hi, Cur + 1, LoHiTail, After,
                 Pushable),
             LoHi = [Head | LoHiTail]
@@ -462,10 +494,10 @@ is_pushable_goal(PushInfo, Goal, Pushable) :-
     Goal = hlds_goal(GoalExpr, GoalInfo),
     Purity = goal_info_get_purity(GoalInfo),
     Detism = goal_info_get_determinism(GoalInfo),
-    (
+    ( if
         Purity = purity_pure,
         Detism = detism_det
-    ->
+    then
         (
             GoalExpr = unify(_, _, _, Unification, _),
             (
@@ -477,16 +509,16 @@ is_pushable_goal(PushInfo, Goal, Pushable) :-
             ;
                 Unification = deconstruct(_, _, Args, _, _, _),
                 RttiVarMaps = PushInfo ^ pi_rtti_varmaps,
-                % See the comment in move_follow_code_select in follow_code.m 
+                % See the comment in move_follow_code_select in follow_code.m
                 % for the reason for this test.
-                ( list.all_true(is_non_rtti_var(RttiVarMaps), Args) ->
+                ( if list.all_true(is_non_rtti_var(RttiVarMaps), Args) then
                     Pushable = pushable
-                ;
+                else
                     Pushable = not_pushable
                 )
             ;
                 Unification = complicated_unify(_, _, _),
-                unexpected($module, $pred, "complicated_unify")
+                unexpected($pred, "complicated_unify")
             )
         ;
             ( GoalExpr = plain_call(_, _, _, _, _, _)
@@ -520,10 +552,10 @@ is_pushable_goal(PushInfo, Goal, Pushable) :-
                 Pushable = not_pushable
             ;
                 Shorthand = bi_implication(_, _),
-                unexpected($module, $pred, "bi_implication")
+                unexpected($pred, "bi_implication")
             )
         )
-    ;
+    else
         Pushable = not_pushable
     ).
 
@@ -580,7 +612,7 @@ is_non_rtti_var(RttiVarMaps, Arg) :-
     % For example, if HeadPath and TailPaths together specified the two
     % expensive goals in the original goal below,
     %
-    %   ( Cond ->
+    %   ( if Cond then
     %       (
     %           X = f,
     %           EXPENSIVE GOAL 1,
@@ -589,13 +621,13 @@ is_non_rtti_var(RttiVarMaps, Arg) :-
     %           X = g,
     %           cheap goal 3
     %       )
-    %   ;
+    %   else
     %       EXPENSIVE GOAL 4
     %   )
     %
     % this predicate should return this transformed goal:
     %
-    %   ( Cond ->
+    %   ( if Cond then
     %       (
     %           X = f,
     %           EXPENSIVE GOAL 1,
@@ -606,7 +638,7 @@ is_non_rtti_var(RttiVarMaps, Arg) :-
     %           cheap goal 3,
     %           LoHi
     %       )
-    %   ;
+    %   else
     %       EXPENSIVE GOAL 4,
     %       LoHi
     %   )
@@ -619,7 +651,7 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
     (
         HeadPath = fgp_nil,
-        expect(unify(TailPaths, []), $module, "TailSteps != []"),
+        expect(unify(TailPaths, []), $pred, "TailSteps != []"),
         add_goals_at_end(LoHi, Goal0, Goal),
         Pushable = pushable
     ;
@@ -634,16 +666,16 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
             Pushable = not_pushable
         ;
             GoalExpr0 = conj(ConjType, Conjuncts0),
-            (
+            ( if
                 % If the expensive goal is a conjunct in this conjunction,
                 % then put LoHi at the end of this conjunction.
                 FirstHeadStep = step_conj(_),
                 LaterHeadPath = fgp_nil
-            ->
-                expect(unify(TailPaths, []), $module, "TailSteps != []"),
+            then
+                expect(unify(TailPaths, []), $pred, "TailSteps != []"),
                 add_goals_at_end(LoHi, Goal0, Goal),
                 Pushable = pushable
-            ;
+            else if
                 % If the expensive goal or goals are INSIDE a conjunct
                 % in this conjunction, push LoHi into the selected conjunct.
                 % We insist on all expensive goals being inside the SAME
@@ -664,52 +696,52 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
                 % and moving them to the front of LoHi.
                 list.length(Conjuncts0, Length),
                 ConjNum = Length
-            ->
+            then
                 push_into_goal(LoHi, LaterHeadPath, LaterTailPaths,
                     SelectedConjunct0, SelectedConjunct, Pushable),
                 list.det_replace_nth(Conjuncts0, ConjNum, SelectedConjunct,
                     Conjuncts),
                 GoalExpr = conj(ConjType, Conjuncts),
                 Goal = hlds_goal(GoalExpr, GoalInfo0)
-            ;
+            else
                 Goal = Goal0,
                 Pushable = not_pushable
             )
         ;
             GoalExpr0 = disj(Disjuncts0),
-            (
+            ( if
                 build_disj_paths_map([HeadPath | TailPaths],
                     map.init, PathsMap)
-            ->
+            then
                 map.to_assoc_list(PathsMap, PathsList),
                 push_into_disjuncts(LoHi, PathsList, 1, Disjuncts0, Disjuncts,
                     Pushable),
                 GoalExpr = disj(Disjuncts),
                 Goal = hlds_goal(GoalExpr, GoalInfo0)
-            ;
+            else
                 Goal = Goal0,
                 Pushable = not_pushable
             )
         ;
             GoalExpr0 = switch(Var, CanFail, Cases0),
-            (
+            ( if
                 build_switch_paths_map([HeadPath | TailPaths],
                     map.init, PathsMap)
-            ->
+            then
                 map.to_assoc_list(PathsMap, PathsList),
                 push_into_cases(LoHi, PathsList, 1, Cases0, Cases, Pushable),
                 GoalExpr = switch(Var, CanFail, Cases),
                 Goal = hlds_goal(GoalExpr, GoalInfo0)
-            ;
+            else
                 Goal = Goal0,
                 Pushable = not_pushable
             )
         ;
             GoalExpr0 = if_then_else(Vars, Cond, Then0, Else0),
-            (
+            ( if
                 build_ite_paths_map([HeadPath | TailPaths],
                     ThenPaths, ElsePaths)
-            ->
+            then
                 (
                     ThenPaths = [],
                     add_goals_at_end(LoHi, Then0, Then),
@@ -728,18 +760,18 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
                     push_into_goal(LoHi, ElsePathsHead, ElsePathsTail,
                         Else0, Else, ElsePushable)
                 ),
-                (
+                ( if
                     ThenPushable = pushable,
                     ElsePushable = pushable
-                ->
+                then
                     GoalExpr = if_then_else(Vars, Cond, Then, Else),
                     Goal = hlds_goal(GoalExpr, GoalInfo0),
                     Pushable = pushable
-                ;
+                else
                     Goal = Goal0,
                     Pushable = not_pushable
                 )
-            ;
+            else
                 Goal = Goal0,
                 Pushable = not_pushable
             )
@@ -754,18 +786,18 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
             SubGoal0 = hlds_goal(_SubGoalExpr0, SubGoalInfo0),
             Detism = goal_info_get_determinism(GoalInfo0),
             SubDetism = goal_info_get_determinism(SubGoalInfo0),
-            (
+            ( if
                 Detism = SubDetism,
                 maybe_steps_after(step_scope(scope_is_no_cut),
                     HeadPath, HeadPathAfter),
                 list.map(maybe_steps_after(step_scope(scope_is_no_cut)),
                     TailPaths, TailPathsAfter)
-            ->
+            then
                 push_into_goal(LoHi, HeadPathAfter, TailPathsAfter,
                     SubGoal0, SubGoal, Pushable),
                 GoalExpr = scope(Reason, SubGoal),
                 Goal = hlds_goal(GoalExpr, GoalInfo0)
-            ;
+            else
                 Goal = Goal0,
                 Pushable = not_pushable
             )
@@ -779,7 +811,7 @@ push_into_goal(LoHi, HeadPath, TailPaths, Goal0, Goal, Pushable) :-
                 Pushable = not_pushable
             ;
                 Shorthand = bi_implication(_, _),
-                unexpected($module, $pred, "bi_implication")
+                unexpected($pred, "bi_implication")
             )
         )
     ).
@@ -817,7 +849,7 @@ push_into_disjuncts(LoHi, PathList, Cur, [HeadDisjunct0 | TailDisjuncts0],
         PathList = [PathListHead | PathListTail],
         (
             PathListHead = PathListHeadNum - one_or_more(One, More),
-            ( PathListHeadNum = Cur ->
+            ( if PathListHeadNum = Cur then
                 push_into_goal(LoHi, One, More, HeadDisjunct0, HeadDisjunct,
                     GoalPushable),
                 (
@@ -829,7 +861,7 @@ push_into_disjuncts(LoHi, PathList, Cur, [HeadDisjunct0 | TailDisjuncts0],
                     TailDisjuncts = TailDisjuncts0,
                     Pushable = not_pushable
                 )
-            ;
+            else
                 add_goals_at_end(LoHi, HeadDisjunct0, HeadDisjunct),
                 push_into_disjuncts(LoHi, PathList, Cur + 1,
                     TailDisjuncts0, TailDisjuncts, Pushable)
@@ -860,7 +892,7 @@ push_into_cases(LoHi, PathList, Cur, [HeadCase0 | TailCases0],
         PathList = [PathListHead | PathListTail],
         (
             PathListHead = PathListHeadNum - one_or_more(One, More),
-            ( PathListHeadNum = Cur ->
+            ( if PathListHeadNum = Cur then
                 push_into_case(LoHi, One, More, HeadCase0, HeadCase,
                     GoalPushable),
                 (
@@ -872,7 +904,7 @@ push_into_cases(LoHi, PathList, Cur, [HeadCase0 | TailCases0],
                     TailCases = TailCases0,
                     Pushable = not_pushable
                 )
-            ;
+            else
                 add_goals_at_end_of_case(LoHi, HeadCase0, HeadCase),
                 push_into_cases(LoHi, PathList, Cur + 1,
                     TailCases0, TailCases, Pushable)
@@ -889,10 +921,10 @@ push_into_cases(LoHi, PathList, Cur, [HeadCase0 | TailCases0],
 build_disj_paths_map([], !DisjPathsMap).
 build_disj_paths_map([Head | Tail], !DisjPathsMap) :-
     Head = fgp_cons(step_disj(N), HeadLaterPath),
-    ( map.search(!.DisjPathsMap, N, one_or_more(One, More)) ->
+    ( if map.search(!.DisjPathsMap, N, one_or_more(One, More)) then
         map.det_update(N, one_or_more(HeadLaterPath, [One | More]),
             !DisjPathsMap)
-    ;
+    else
         map.det_insert(N, one_or_more(HeadLaterPath, []), !DisjPathsMap)
     ),
     build_disj_paths_map(Tail, !DisjPathsMap).
@@ -904,10 +936,10 @@ build_disj_paths_map([Head | Tail], !DisjPathsMap) :-
 build_switch_paths_map([], !DisjPathsMap).
 build_switch_paths_map([Head | Tail], !DisjPathsMap) :-
     Head = fgp_cons(step_switch(N, _), HeadLaterPath),
-    ( map.search(!.DisjPathsMap, N, one_or_more(One, More)) ->
+    ( if map.search(!.DisjPathsMap, N, one_or_more(One, More)) then
         map.det_update(N, one_or_more(HeadLaterPath, [One | More]),
             !DisjPathsMap)
-    ;
+    else
         map.det_insert(N, one_or_more(HeadLaterPath, []), !DisjPathsMap)
     ),
     build_switch_paths_map(Tail, !DisjPathsMap).
@@ -919,13 +951,13 @@ build_ite_paths_map([], [], []).
 build_ite_paths_map([Head | Tail], ThenPaths, ElsePaths) :-
     build_ite_paths_map(Tail, ThenPathsTail, ElsePathsTail),
     Head = fgp_cons(HeadFirstStep, HeadLaterPath),
-    ( HeadFirstStep = step_ite_then ->
+    ( if HeadFirstStep = step_ite_then then
         ThenPaths = [HeadLaterPath | ThenPathsTail],
         ElsePaths = ElsePathsTail
-    ; HeadFirstStep = step_ite_then ->
+    else if HeadFirstStep = step_ite_then then
         ThenPaths = ThenPathsTail,
         ElsePaths = [HeadLaterPath | ElsePathsTail]
-    ;
+    else
         fail
     ).
 
@@ -936,9 +968,9 @@ build_ite_paths_map([Head | Tail], ThenPaths, ElsePaths) :-
 
 add_goals_at_end(AddedGoals, Goal0, Goal) :-
     Goal0 = hlds_goal(GoalExpr0, _GoalInfo0),
-    ( GoalExpr0 = conj(plain_conj, Conjuncts0) ->
+    ( if GoalExpr0 = conj(plain_conj, Conjuncts0) then
         create_conj_from_list(Conjuncts0 ++ AddedGoals, plain_conj, Goal)
-    ;
+    else
         create_conj_from_list([Goal0 | AddedGoals], plain_conj, Goal)
     ).
 
@@ -969,8 +1001,12 @@ maybe_steps_after(Step, fgp_cons(Step, Tail), Tail).
 
 %-----------------------------------------------------------------------------%
 
-:- type one_or_more(T)
-    --->    one_or_more(T, list(T)).
+:- pred get_push_debug_output_stream(push_info::in, io.text_output_stream::out,
+    io::di, io::uo) is det.
+
+get_push_debug_output_stream(PushInfo, DebugStream, !IO) :-
+    get_debug_output_stream(PushInfo ^ pi_globals, PushInfo ^ pi_module_name,
+        DebugStream, !IO).
 
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.implicit_parallelism.push_goals_together.
