@@ -74,8 +74,8 @@
     %
 :- pred collect_non_local_and_in_cond_regions(module_info::in,
     rpta_info_table::in, proc_pp_region_set_table::in,
-    proc_pp_region_set_table::in, renaming_table::in,
-    renaming_annotation_table::in, proc_goal_path_regions_table::out,
+    proc_pp_region_set_table::in, rbmm_renaming_table::in,
+    rbmm_renaming_annotation_table::in, proc_goal_path_regions_table::out,
     proc_goal_path_regions_table::out) is det.
 
     % After having the 2 pieces of information calculated above, this step
@@ -99,7 +99,7 @@
     % to which the program point belongs.
     %
 :- pred collect_ite_renaming(module_info::in, rpta_info_table::in,
-    proc_goal_path_regions_table::in, renaming_table::out) is det.
+    proc_goal_path_regions_table::in, rbmm_renaming_table::out) is det.
 
     % In the then branch of an if-then-else in which renaming happens we
     % need to introduce reverse renaming annotation in the form of
@@ -114,7 +114,7 @@
     % This predicate will also traverse only procedures in which renaming
     % happens. For each Condition where renaming happens,
     % it finds the first program point in the corresponding Then and
-    % introduces the reverse renaming annotation before that ponit.
+    % introduces the reverse renaming annotation before that point.
     % Note that that first program point must not be in the condition of
     % an if-then-else. E.g.,
     % if % a renaming exists here: R -> R_ite_1
@@ -128,14 +128,15 @@
     %    ...
 :- pred collect_ite_annotation(proc_goal_path_regions_table::in,
     execution_path_table::in, rpta_info_table::in,
-    renaming_table::in, renaming_table::out,
-    renaming_annotation_table::out) is det.
+    rbmm_renaming_table::in, rbmm_renaming_table::out,
+    rbmm_renaming_annotation_table::out) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module hlds.goal_form.
 :- import_module hlds.goal_path.
 :- import_module hlds.hlds_goal.
 :- import_module transform_hlds.rbmm.points_to_graph.
@@ -145,7 +146,6 @@
 :- import_module list.
 :- import_module pair.
 :- import_module require.
-:- import_module set.
 :- import_module string.
 
 %-----------------------------------------------------------------------------%
@@ -153,7 +153,7 @@
 collect_non_local_and_in_cond_regions(ModuleInfo, RptaInfoTable,
         LRBeforeTable, LRAfterTable, ResurRenamingTable,
         ResurRenamingAnnoTable, NonLocalRegionsTable, InCondRegionsTable) :-
-    module_info_get_valid_predids(PredIds, ModuleInfo, _),
+    module_info_get_valid_pred_ids(ModuleInfo, PredIds),
     list.foldl2(collect_non_local_and_in_cond_regions_pred(ModuleInfo,
         RptaInfoTable, LRBeforeTable, LRAfterTable, ResurRenamingTable,
         ResurRenamingAnnoTable), PredIds,
@@ -161,8 +161,8 @@ collect_non_local_and_in_cond_regions(ModuleInfo, RptaInfoTable,
 
 :- pred collect_non_local_and_in_cond_regions_pred(module_info::in,
     rpta_info_table::in, proc_pp_region_set_table::in,
-    proc_pp_region_set_table::in, renaming_table::in,
-    renaming_annotation_table::in, pred_id::in,
+    proc_pp_region_set_table::in, rbmm_renaming_table::in,
+    rbmm_renaming_annotation_table::in, pred_id::in,
     proc_goal_path_regions_table::in, proc_goal_path_regions_table::out,
     proc_goal_path_regions_table::in, proc_goal_path_regions_table::out)
     is det.
@@ -172,7 +172,7 @@ collect_non_local_and_in_cond_regions_pred(ModuleInfo, RptaInfoTable,
         ResurRenamingAnnoTable, PredId, !NonLocalRegionsTable,
         !InCondRegionsTable) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl2(collect_non_local_and_in_cond_regions_proc(ModuleInfo,
         PredId, RptaInfoTable, LRBeforeTable, LRAfterTable,
         ResurRenamingTable, ResurRenamingAnnoTable), ProcIds,
@@ -180,8 +180,8 @@ collect_non_local_and_in_cond_regions_pred(ModuleInfo, RptaInfoTable,
 
 :- pred collect_non_local_and_in_cond_regions_proc(module_info::in,
     pred_id::in, rpta_info_table::in, proc_pp_region_set_table::in,
-    proc_pp_region_set_table::in, renaming_table::in,
-    renaming_annotation_table::in, proc_id::in,
+    proc_pp_region_set_table::in, rbmm_renaming_table::in,
+    rbmm_renaming_annotation_table::in, proc_id::in,
     proc_goal_path_regions_table::in, proc_goal_path_regions_table::out,
     proc_goal_path_regions_table::in, proc_goal_path_regions_table::out)
     is det.
@@ -191,23 +191,23 @@ collect_non_local_and_in_cond_regions_proc(ModuleInfo, PredId,
         ResurRenamingAnnoTable, ProcId,
         !NonLocalRegionsTable, !InCondRegionsTable) :-
     PPId = proc(PredId, ProcId),
-    ( some_are_special_preds([PPId], ModuleInfo) ->
+    ( if some_are_special_preds([PPId], ModuleInfo) then
         true
-    ;
+    else
         module_info_proc_info(ModuleInfo, PPId, ProcInfo0),
         fill_goal_path_slots_in_proc(ModuleInfo, ProcInfo0, ProcInfo),
         proc_info_get_goal(ProcInfo, Goal),
         map.lookup(RptaInfoTable, PPId, rpta_info(Graph, _)),
         map.lookup(LRBeforeTable, PPId, LRBeforeProc),
         map.lookup(LRAfterTable, PPId, LRAfterProc),
-        ( map.search(ResurRenamingTable, PPId, ResurRenamingProc0) ->
+        ( if map.search(ResurRenamingTable, PPId, ResurRenamingProc0) then
             ResurRenamingProc = ResurRenamingProc0
-        ;
+        else
             ResurRenamingProc = map.init
         ),
-        ( map.search(ResurRenamingAnnoTable, PPId, ResurRenamingAnnoProc0) ->
+        ( if map.search(ResurRenamingAnnoTable, PPId, ResurRenamingAnnoProc0) then
             ResurRenamingAnnoProc = ResurRenamingAnnoProc0
-        ;
+        else
             ResurRenamingAnnoProc = map.init
         ),
         collect_non_local_and_in_cond_regions_goal(Graph,
@@ -215,21 +215,21 @@ collect_non_local_and_in_cond_regions_proc(ModuleInfo, PredId,
             ResurRenamingAnnoProc, Goal,
             map.init, NonLocalRegionsProc,
             map.init, InCondRegionsProc),
-        ( map.count(NonLocalRegionsProc) = 0 ->
+        ( if map.count(NonLocalRegionsProc) = 0 then
             true
-        ;
+        else
             map.set(PPId, NonLocalRegionsProc, !NonLocalRegionsTable)
         ),
-        ( map.count(InCondRegionsProc) = 0 ->
+        ( if map.count(InCondRegionsProc) = 0 then
             true
-        ;
+        else
             map.set(PPId, InCondRegionsProc, !InCondRegionsTable)
         )
     ).
 
 :- pred collect_non_local_and_in_cond_regions_goal(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, hlds_goal::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, hlds_goal::in,
     goal_path_regions_table::in, goal_path_regions_table::out,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
@@ -243,7 +243,8 @@ collect_non_local_and_in_cond_regions_goal(Graph, LRBeforeProc, LRAfterProc,
 
 :- pred collect_non_local_and_in_cond_regions_expr(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, hlds_goal_expr::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in,
+    hlds_goal_expr::in,
     goal_path_regions_table::in, goal_path_regions_table::out,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
@@ -321,11 +322,11 @@ collect_non_local_and_in_cond_regions_expr(Graph, LRBeforeProc, LRAfterProc,
 collect_non_local_and_in_cond_regions_expr(_, _, _, _, _, shorthand(_),
         !NonLocalRegionProc, !InCondRegionsProc) :-
     % These should have been expanded out by now.
-    unexpected($module, $pred, "shorthand").
+    unexpected($pred, "shorthand").
 
 :- pred collect_non_local_and_in_cond_regions_case(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, case::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, case::in,
     goal_path_regions_table::in, goal_path_regions_table::out,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
@@ -338,8 +339,8 @@ collect_non_local_and_in_cond_regions_case(Graph, LRBeforeProc, LRAfterProc,
         Goal, !NonLocalRegionProc, !InCondRegionsProc).
 
 :- pred collect_non_local_regions_in_ite(rpt_graph::in,
-    pp_region_set_table::in, pp_region_set_table::in, renaming_proc::in,
-    renaming_annotation_proc::in, hlds_goal::in,
+    pp_region_set_table::in, pp_region_set_table::in, rbmm_renaming_proc::in,
+    rbmm_renaming_annotation_proc::in, hlds_goal::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_non_local_regions_in_ite(Graph, LRBeforeProc, LRAfterProc,
@@ -363,9 +364,9 @@ collect_non_local_regions_in_ite(Graph, LRBeforeProc, LRAfterProc,
         % Apply resurrection renaming to those sets.
         % For each renaming annotation, the left one is put into CreatedBefore,
         % and the right one is put into RemovedAfter.
-        ( map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) ->
+        ( if map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) then
             ResurRenaming = ResurRenaming0
-        ;
+        else
             ResurRenaming = map.init
         ),
         set.fold(apply_region_renaming(Graph, ResurRenaming),
@@ -373,9 +374,11 @@ collect_non_local_regions_in_ite(Graph, LRBeforeProc, LRAfterProc,
         set.fold(apply_region_renaming(Graph, ResurRenaming),
             CreatedBeforeNodes, set.init, CreatedBeforeRegions0),
 
-        ( map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos0) ->
+        ( if
+            map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos0)
+        then
             ResurRenamingAnnos = ResurRenamingAnnos0
-        ;
+        else
             ResurRenamingAnnos = []
         ),
         list.foldl2(renaming_annotation_to_regions,
@@ -392,15 +395,15 @@ collect_non_local_regions_in_ite(Graph, LRBeforeProc, LRAfterProc,
             GoalInIte, !NonLocalRegionProc)
     ).
 
-:- pred apply_region_renaming(rpt_graph::in, renaming::in, rptg_node::in,
+:- pred apply_region_renaming(rpt_graph::in, rbmm_renaming::in, rptg_node::in,
     set(string)::in, set(string)::out) is det.
 
 apply_region_renaming(Graph, Renaming, Node, !Regions) :-
     RegionName = rptg_lookup_region_name(Graph, Node),
-    ( map.search(Renaming, RegionName, RenamedRegionNameList) ->
+    ( if map.search(Renaming, RegionName, RenamedRegionNameList) then
         RenamedRegionName = list.det_last(RenamedRegionNameList),
         set.insert(RenamedRegionName, !Regions)
-    ;
+    else
         set.insert(RegionName, !Regions)
     ).
 
@@ -414,7 +417,7 @@ renaming_annotation_to_regions(RenameAnnotation, !LeftRegions,
         ( RenameAnnotation = create_region(_)
         ; RenameAnnotation = remove_region(_)
         ),
-        unexpected($module, $pred, "annotation is not assignment")
+        unexpected($pred, "annotation is not assignment")
     ;
         RenameAnnotation = rename_region(RightRegion, LeftRegion),
         set.insert(LeftRegion, !LeftRegions),
@@ -448,26 +451,26 @@ renaming_annotation_to_regions(RenameAnnotation, !LeftRegions,
 record_non_local_regions(RevPath, Created, Removed, !NonLocalRegionProc) :-
     (
         RevPath = rgp_cons(RevInitialPath, LastStep),
-        ( LastStep = step_ite_else ->
+        ( if LastStep = step_ite_else then
             % The current NonLocalRegions are attached to the goal path
             % to the corresponding condition.
             RevPathToCond = rgp_cons(RevInitialPath, step_ite_cond),
-            (
+            ( if
                 map.search(!.NonLocalRegionProc, RevPathToCond,
                     NonLocalRegions0)
-            ->
+            then
                 set.union(NonLocalRegions0, Created, NonLocalRegions1),
                 set.difference(NonLocalRegions1, Removed, NonLocalRegions)
-            ;
+            else
                 set.difference(Created, Removed, NonLocalRegions)
             ),
             % Only record if some non-local region(s) exist.
-            ( set.empty(NonLocalRegions) ->
+            ( if set.is_empty(NonLocalRegions) then
                 true
-            ;
+            else
                 map.set(RevPathToCond, NonLocalRegions, !NonLocalRegionProc)
             )
-        ;
+        else
             true
         ),
 
@@ -481,7 +484,7 @@ record_non_local_regions(RevPath, Created, Removed, !NonLocalRegionProc) :-
 
 :- pred collect_non_local_regions_in_ite_compound_goal(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, hlds_goal::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, hlds_goal::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_non_local_regions_in_ite_compound_goal(Graph, LRBeforeProc,
@@ -546,12 +549,12 @@ collect_non_local_regions_in_ite_compound_goal(Graph, LRBeforeProc,
         ; Expr = generic_call(_, _, _, _, _)
         ; Expr = shorthand(_)
         ),
-        unexpected($module, $pred, "atomic or unsupported goal")
+        unexpected($pred, "atomic or unsupported goal")
     ).
 
 :- pred collect_non_local_regions_in_ite_case(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, case::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, case::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_non_local_regions_in_ite_case(Graph, LRBeforeProc, LRAfterProc,
@@ -575,7 +578,7 @@ collect_non_local_regions_in_ite_case(Graph, LRBeforeProc, LRAfterProc,
     %
 :- pred collect_regions_created_in_condition(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, hlds_goal::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, hlds_goal::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_regions_created_in_condition(Graph, LRBeforeProc, LRAfterProc,
@@ -593,17 +596,19 @@ collect_regions_created_in_condition(Graph, LRBeforeProc, LRAfterProc,
         % We need to apply renaming to this CreatedNodes set and look up
         % the renaming annotations after this program point. For each renaming
         % annotation the left one is created and the right is removed.
-        ( map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) ->
+        ( if map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) then
             ResurRenaming = ResurRenaming0
-        ;
+        else
             ResurRenaming = map.init
         ),
         set.fold(apply_region_renaming(Graph, ResurRenaming),
             CreatedNodes, set.init, CreatedRegions0),
 
-        ( map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos0) ->
+        ( if
+            map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos0)
+        then
             ResurRenamingAnnos = ResurRenamingAnnos0
-        ;
+        else
             ResurRenamingAnnos = []
         ),
         list.foldl2(renaming_annotation_to_regions, ResurRenamingAnnos,
@@ -634,20 +639,20 @@ collect_regions_created_in_condition(Graph, LRBeforeProc, LRAfterProc,
 record_regions_created_in_condition(RevPath, Created, !InCondRegionsProc) :-
     (
         RevPath = rgp_cons(RevInitialPath, LastStep),
-        ( LastStep = step_ite_cond  ->
-            ( map.search(!.InCondRegionsProc, RevPath, InCondRegions0) ->
+        ( if LastStep = step_ite_cond then
+            ( if map.search(!.InCondRegionsProc, RevPath, InCondRegions0) then
                 set.union(InCondRegions0, Created, InCondRegions)
-            ;
+            else
                 InCondRegions = Created
             ),
             % Only record if some regions are actually created inside
             % the condition.
-            ( set.empty(InCondRegions) ->
+            ( if set.is_empty(InCondRegions) then
                 true
-            ;
+            else
                 map.set(RevPath, InCondRegions, !InCondRegionsProc)
             )
-        ;
+        else
             true
         ),
         record_regions_created_in_condition(RevInitialPath, Created,
@@ -658,7 +663,7 @@ record_regions_created_in_condition(RevPath, Created, !InCondRegionsProc) :-
 
 :- pred collect_regions_created_in_condition_compound_goal(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, hlds_goal::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, hlds_goal::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_regions_created_in_condition_compound_goal(Graph,
@@ -723,12 +728,12 @@ collect_regions_created_in_condition_compound_goal(Graph,
         ; Expr = generic_call(_, _, _, _, _)
         ; Expr = shorthand(_)
         ),
-        unexpected($module, $pred, "atomic or unsupported goal")
+        unexpected($pred, "atomic or unsupported goal")
     ).
 
 :- pred collect_regions_created_in_condition_case(rpt_graph::in,
     pp_region_set_table::in, pp_region_set_table::in,
-    renaming_proc::in, renaming_annotation_proc::in, case::in,
+    rbmm_renaming_proc::in, rbmm_renaming_annotation_proc::in, case::in,
     goal_path_regions_table::in, goal_path_regions_table::out) is det.
 
 collect_regions_created_in_condition_case(Graph,
@@ -759,15 +764,15 @@ collect_ite_renamed_regions(InCondRegionsTable, NonLocalRegionsTable,
 
 collect_ite_renamed_regions_proc(NonLocalRegionsTable, PPId,
         InCondRegionsProc, !IteRenamedRegionTable) :-
-    ( map.search(NonLocalRegionsTable, PPId, NonLocalRegionsProc) ->
+    ( if map.search(NonLocalRegionsTable, PPId, NonLocalRegionsProc) then
         map.foldl(collect_ite_renamed_regions_ite(NonLocalRegionsProc),
             InCondRegionsProc, map.init, IteRenamedRegionProc),
-        ( map.count(IteRenamedRegionProc) = 0 ->
+        ( if map.count(IteRenamedRegionProc) = 0 then
             true
-        ;
+        else
             map.set(PPId, IteRenamedRegionProc, !IteRenamedRegionTable)
         )
-    ;
+    else
         true
     ).
 
@@ -777,14 +782,14 @@ collect_ite_renamed_regions_proc(NonLocalRegionsTable, PPId,
 
 collect_ite_renamed_regions_ite(NonLocalRegionsProc, PathToCond,
         InCondRegions, !IteRenamedRegionProc) :-
-    ( map.search(NonLocalRegionsProc, PathToCond, NonLocalRegions) ->
+    ( if map.search(NonLocalRegionsProc, PathToCond, NonLocalRegions) then
         set.intersect(NonLocalRegions, InCondRegions, RenamedRegions),
-        ( set.empty(RenamedRegions) ->
+        ( if set.is_empty(RenamedRegions) then
             true
-        ;
+        else
             map.set(PathToCond, RenamedRegions, !IteRenamedRegionProc)
         )
-    ;
+    else
         true
     ).
 
@@ -800,7 +805,7 @@ collect_ite_renaming(ModuleInfo, RptaInfoTable, IteRenamedRegionTable,
 
 :- pred collect_ite_renaming_proc(module_info::in, rpta_info_table::in,
     pred_proc_id::in, goal_path_regions_table::in,
-    renaming_table::in, renaming_table::out) is det.
+    rbmm_renaming_table::in, rbmm_renaming_table::out) is det.
 
 collect_ite_renaming_proc(ModuleInfo, RptaInfoTable,
         PPId, IteRenamedRegionProc, !IteRenamingTable) :-
@@ -814,7 +819,7 @@ collect_ite_renaming_proc(ModuleInfo, RptaInfoTable,
     map.set(PPId, IteRenamingProc, !IteRenamingTable).
 
 :- pred collect_ite_renaming_goal(goal_path_regions_table::in, rpt_graph::in,
-    hlds_goal::in, renaming_proc::in, renaming_proc::out) is det.
+    hlds_goal::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_ite_renaming_goal(IteRenamedRegionProc, Graph, Goal,
         !IteRenamingProc) :-
@@ -859,11 +864,11 @@ collect_ite_renaming_goal(IteRenamedRegionProc, Graph, Goal,
             Else, !IteRenamingProc)
     ;
         GoalExpr = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 :- pred collect_ite_renaming_case(goal_path_regions_table::in, rpt_graph::in,
-    case::in, renaming_proc::in, renaming_proc::out) is det.
+    case::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_ite_renaming_case(IteRenamedRegionProc, Graph,
         Case, !IteRenamingProc) :-
@@ -875,7 +880,7 @@ collect_ite_renaming_case(IteRenamedRegionProc, Graph,
     %
 :- pred collect_ite_renaming_in_condition(goal_path_regions_table::in,
     rpt_graph::in, hlds_goal::in,
-    renaming_proc::in, renaming_proc::out) is det.
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_ite_renaming_in_condition(IteRenamedRegionProc, Graph, Cond,
         !IteRenamingProc) :-
@@ -891,13 +896,13 @@ collect_ite_renaming_in_condition(IteRenamedRegionProc, Graph, Cond,
         ProgPoint = pp(_, RevGoalPath),
         get_closest_condition_in_goal_path(RevGoalPath, RevPathToClosestCond,
             0, HowMany),
-        (
+        ( if
             map.search(IteRenamedRegionProc, RevPathToClosestCond,
                 RenamedRegions)
-        ->
+        then
             set.fold(record_ite_renaming(ProgPoint, HowMany, Graph),
                 RenamedRegions, !IteRenamingProc)
-        ;
+        else
             % No region needs to be renamed due to if-then-else covering
             % this program point.
             true
@@ -911,20 +916,20 @@ collect_ite_renaming_in_condition(IteRenamedRegionProc, Graph, Cond,
     % A renaming is of the form: R --> R_ite_HowMany.
     %
 :- pred record_ite_renaming(program_point::in, int::in, rpt_graph::in,
-    string::in, renaming_proc::in, renaming_proc::out) is det.
+    string::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 record_ite_renaming(ProgPoint, HowMany, _Graph, RegName, !IteRenamingProc) :-
     NewName = RegName ++ "_ite_" ++ string.int_to_string(HowMany),
-    ( map.search(!.IteRenamingProc, ProgPoint, IteRenaming0) ->
+    ( if map.search(!.IteRenamingProc, ProgPoint, IteRenaming0) then
         map.set(RegName, [NewName], IteRenaming0, IteRenaming)
-    ;
+    else
         map.set(RegName, [NewName], map.init, IteRenaming)
     ),
     map.set(ProgPoint, IteRenaming, !IteRenamingProc).
 
 :- pred collect_ite_renaming_in_condition_compound_goal(
     goal_path_regions_table::in, rpt_graph::in,
-    hlds_goal::in, renaming_proc::in, renaming_proc::out) is det.
+    hlds_goal::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_ite_renaming_in_condition_compound_goal(IteRenamedRegionProc,
         Graph, GoalInCond, !IteRenamingProc) :-
@@ -972,12 +977,12 @@ collect_ite_renaming_in_condition_compound_goal(IteRenamedRegionProc,
         ; Expr = generic_call(_, _, _, _, _)
         ; Expr = shorthand(_)
         ),
-        unexpected($module, $pred, "atomic or unsupported goal")
+        unexpected($pred, "atomic or unsupported goal")
     ).
 
 :- pred collect_ite_renaming_in_condition_case(goal_path_regions_table::in,
     rpt_graph::in, case::in,
-    renaming_proc::in, renaming_proc::out) is det.
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_ite_renaming_in_condition_case(IteRenamedRegionProc, Graph, Case,
         !IteRenamingProc) :-
@@ -987,7 +992,7 @@ collect_ite_renaming_in_condition_case(IteRenamedRegionProc, Graph, Case,
 
     % This predicate receives a goal path (to some goal) and returns
     % the subpath to the closest condition containing the goal (if any);
-    % if the goal is not in in any condition, the output path is empty.
+    % if the goal is not in any condition, the output path is empty.
     % It also returns the number of conditions that contain the goal.
     % e.g., ( if
     %           ( if
@@ -1003,11 +1008,11 @@ collect_ite_renaming_in_condition_case(IteRenamedRegionProc, Graph, Case,
 get_closest_condition_in_goal_path(RevPath, RevPathToCond, !HowMany) :-
     (
         RevPath = rgp_cons(RevInitialPath, LastStep),
-        ( LastStep = step_ite_cond ->
+        ( if LastStep = step_ite_cond then
             RevPathToCond = RevPath,
             get_closest_condition_in_goal_path(RevInitialPath, _, !HowMany),
             !:HowMany = !.HowMany + 1
-        ;
+        else
             get_closest_condition_in_goal_path(RevInitialPath, RevPathToCond,
                 !HowMany)
         )
@@ -1029,8 +1034,9 @@ collect_ite_annotation(IteRenamedRegionTable, ExecPathTable, RptaInfoTable,
 
 :- pred collect_ite_annotation_proc(execution_path_table::in,
     rpta_info_table::in, pred_proc_id::in, goal_path_regions_table::in,
-    renaming_table::in, renaming_table::out,
-    renaming_annotation_table::in, renaming_annotation_table::out) is det.
+    rbmm_renaming_table::in, rbmm_renaming_table::out,
+    rbmm_renaming_annotation_table::in, rbmm_renaming_annotation_table::out)
+    is det.
 
 collect_ite_annotation_proc(ExecPathTable, RptaInfoTable, PPId,
         IteRenamedRegionProc, !IteRenamingTable, !IteAnnotationTable) :-
@@ -1046,15 +1052,15 @@ collect_ite_annotation_proc(ExecPathTable, RptaInfoTable, PPId,
 
 :- pred collect_ite_annotation_region_names(list(execution_path)::in,
     rpt_graph::in, reverse_goal_path::in, set(string)::in,
-    renaming_proc::in, renaming_proc::out,
-    renaming_annotation_proc::in, renaming_annotation_proc::out) is det.
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out)
+    is det.
 
 collect_ite_annotation_region_names(ExecPaths, Graph, RevPathToCond,
         RenamedRegions, !IteRenamingProc, !IteAnnotationProc) :-
     (
         RevPathToCond = rgp_cons(RevInitialPath, LastStep),
-        expect(unify(LastStep, step_ite_cond), $module, $pred,
-            "not step_ite_cond"),
+        expect(unify(LastStep, step_ite_cond), $pred, "not step_ite_cond"),
         RevPathToThen = rgp_cons(RevInitialPath, step_ite_then),
         get_closest_condition_in_goal_path(RevPathToCond, _, 0, HowMany),
         list.foldl2(
@@ -1063,13 +1069,14 @@ collect_ite_annotation_region_names(ExecPaths, Graph, RevPathToCond,
             ExecPaths, !IteRenamingProc, !IteAnnotationProc)
     ;
         RevPathToCond = rgp_nil,
-        unexpected($module, $pred, "empty path to condition")
+        unexpected($pred, "empty path to condition")
     ).
 
 :- pred collect_ite_annotation_exec_path(rpt_graph::in, reverse_goal_path::in,
     set(string)::in, int::in, execution_path::in,
-    renaming_proc::in, renaming_proc::out,
-    renaming_annotation_proc::in, renaming_annotation_proc::out) is det.
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out)
+    is det.
 
 collect_ite_annotation_exec_path(_, _, _, _, [], !IteRenamingProc,
         !IteAnnotationProc).
@@ -1086,8 +1093,9 @@ collect_ite_annotation_exec_path(Graph, PathToThen,
     %
 :- pred collect_ite_annotation_exec_path_2(rpt_graph::in,
     reverse_goal_path::in, set(string)::in, int::in, program_point::in,
-    execution_path::in, renaming_proc::in, renaming_proc::out,
-    renaming_annotation_proc::in, renaming_annotation_proc::out) is det.
+    execution_path::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out)
+    is det.
 
 collect_ite_annotation_exec_path_2(_, _, _, _, _, [], !IteRenamingProc,
         !IteAnnotationProc).
@@ -1099,20 +1107,20 @@ collect_ite_annotation_exec_path_2(Graph, RevPathToThen,
     list.reverse(RevGoalPathSteps, GoalPathSteps),
     reverse_goal_path_to_steps(RevPathToThen, RevPathToThenSteps),
     list.reverse(RevPathToThenSteps, PathToThenSteps),
-    ( list.append(PathToThenSteps, FromThenSteps, GoalPathSteps) ->
-        ( list.member(step_ite_cond, FromThenSteps) ->
+    ( if list.append(PathToThenSteps, FromThenSteps, GoalPathSteps) then
+        ( if list.member(step_ite_cond, FromThenSteps) then
             % We cannot introduce reverse renaming in the condition of
             % an if-then-else. So we need to maintain the ite renaming
             % from the previous point to this point.
-            ( map.search(!.IteRenamingProc, PrevPoint, PrevIteRenaming) ->
+            ( if map.search(!.IteRenamingProc, PrevPoint, PrevIteRenaming) then
                 map.set(ProgPoint, PrevIteRenaming, !IteRenamingProc)
-            ;
+            else
                 true
             ),
             collect_ite_annotation_exec_path_2(Graph, RevPathToThen,
                 RenamedRegions, HowMany, ProgPoint, ProgPointGoals,
                 !IteRenamingProc, !IteAnnotationProc)
-        ;
+        else
             % This is the first point in the corresponding then branch, which
             % is not in the condition of another if-then-else, we need
             % to introduce reverse renaming at this point.
@@ -1121,7 +1129,7 @@ collect_ite_annotation_exec_path_2(Graph, RevPathToThen,
                     HowMany),
                 RenamedRegions, !IteAnnotationProc)
         )
-    ;
+    else
         collect_ite_annotation_exec_path_2(Graph, RevPathToThen,
             RenamedRegions, HowMany, ProgPoint, ProgPointGoals,
             !IteRenamingProc, !IteAnnotationProc)
@@ -1142,24 +1150,25 @@ reverse_goal_path_to_steps(rgp_cons(EarlierPath, LaterStep),
     % R --> R_1, then the annotation is R_1 = R_ite_HowMany.
     %
 :- pred introduce_reverse_renaming(program_point::in,
-    renaming_proc::in, int::in, string::in,
-    renaming_annotation_proc::in, renaming_annotation_proc::out) is det.
+    rbmm_renaming_proc::in, int::in, string::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out)
+    is det.
 
 introduce_reverse_renaming(ProgPoint, IteRenamingProc, HowMany, RegName,
         !IteAnnotationProc) :-
     CurrentName = RegName ++ "_ite_" ++ string.int_to_string(HowMany),
-    ( map.search(IteRenamingProc, ProgPoint, Renaming) ->
-        ( map.search(Renaming, RegName, RenameToList) ->
-            ( list.length(RenameToList) = 1 ->
+    ( if map.search(IteRenamingProc, ProgPoint, Renaming) then
+        ( if map.search(Renaming, RegName, RenameToList) then
+            ( if list.length(RenameToList) = 1 then
                 RenameTo = list.det_last(RenameToList),
                 make_renaming_instruction(CurrentName, RenameTo, Annotation)
-            ;
-                unexpected($module, $pred, "more than one renaming")
+            else
+                unexpected($pred, "more than one renaming")
             )
-        ;
+        else
             make_renaming_instruction(CurrentName, RegName, Annotation)
         )
-    ;
+    else
         % No renaming exists at this program point.
         make_renaming_instruction(CurrentName, RegName, Annotation)
     ),
