@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2005-2012 The University of Melbourne.
+% Copyright (C) 2017 The Mercury Team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -48,11 +49,14 @@
 :- implementation.
 
 :- import_module hlds.goal_path.
+:- import_module hlds.hlds_dependency_graph.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
+:- import_module libs.
+:- import_module libs.dependency_graph.
 :- import_module parse_tree.
 :- import_module parse_tree.prog_data.
-:- import_module transform_hlds.dependency_graph.
+:- import_module parse_tree.prog_data_pragma.
 :- import_module transform_hlds.rbmm.points_to_graph.
 :- import_module transform_hlds.smm_common.
 :- import_module transform_hlds.ctgc.
@@ -84,7 +88,7 @@ region_points_to_analysis(InfoTable, !ModuleInfo) :-
     rpta_info_table::in, rpta_info_table::out) is det.
 
 intra_proc_rpta(ModuleInfo, !InfoTable) :-
-    module_info_get_valid_predids(PredIds, ModuleInfo, _),
+    module_info_get_valid_pred_ids(ModuleInfo, PredIds),
     list.foldl(intra_proc_rpta_pred(ModuleInfo), PredIds, !InfoTable).
 
 :- pred intra_proc_rpta_pred(module_info::in, pred_id::in,
@@ -92,7 +96,7 @@ intra_proc_rpta(ModuleInfo, !InfoTable) :-
 
 intra_proc_rpta_pred(ModuleInfo, PredId, !InfoTable) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl(intra_proc_rpta_proc(ModuleInfo, PredId), ProcIds, !InfoTable).
 
 :- pred intra_proc_rpta_proc(module_info::in, pred_id::in, proc_id::in,
@@ -100,9 +104,9 @@ intra_proc_rpta_pred(ModuleInfo, PredId, !InfoTable) :-
 
 intra_proc_rpta_proc(ModuleInfo, PredId, ProcId, !InfoTable) :-
     PPId = proc(PredId, ProcId),
-    ( some_are_special_preds([PPId], ModuleInfo) ->
+    ( if some_are_special_preds([PPId], ModuleInfo) then
         true
-    ;
+    else
         module_info_proc_info(ModuleInfo, PPId, ProcInfo),
         RptaInfo0 = rpta_info_init(ProcInfo),
         proc_info_get_goal(ProcInfo, Goal),
@@ -156,7 +160,7 @@ intra_analyse_goal_expr(scope(_Reason, Goal), !RptaInfo) :-
 %    ;
 %        Msg = "intra_analyse_goal_expr: Scope's reason of from_ground_term "
 %            ++ "not handled",
-%        unexpected($module, $pred, Msg)
+%        unexpected($pred, Msg)
 %    ).
 
 intra_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), !RptaInfo) :-
@@ -166,7 +170,7 @@ intra_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), !RptaInfo) :-
 
 intra_analyse_goal_expr(shorthand(_), _, _) :-
     % These should have been expanded out by now.
-    unexpected($module, $pred, "shorthand").
+    unexpected($pred, "shorthand").
 
 :- pred intra_analyse_case(case::in, rpta_info::in, rpta_info::out) is det.
 
@@ -179,7 +183,7 @@ intra_analyse_case(Case, !RptaInfo) :-
     % For construction and deconstruction unifications, add an edge from
     % the node of the variable on the LHS to that of each variable on the RHS.
     % For construction, also mark the node of lhs variable allocated.
-    % 
+    %
     % For assignment unifications we merge the nodes corresponding to
     % the variables on either side.
     %
@@ -207,9 +211,9 @@ intra_analyse_unification(assign(ToVar, FromVar),
     rpta_info(!.Graph, AlphaMapping), rpta_info(!:Graph, AlphaMapping)) :-
     rptg_get_node_by_variable(!.Graph, ToVar, ToNode),
     rptg_get_node_by_variable(!.Graph, FromVar, FromNode),
-    ( ToNode = FromNode ->
+    ( if ToNode = FromNode then
         true
-    ;
+    else
         unify_operator(ToNode, FromNode, !Graph),
         % After merging the two nodes, apply rule P1 to restore the
         % RPTG's invariants.
@@ -217,7 +221,7 @@ intra_analyse_unification(assign(ToVar, FromVar),
     ).
 intra_analyse_unification(simple_test(_, _), !RptaInfo).
 intra_analyse_unification(complicated_unify(_, _, _), _, _) :-
-    unexpected($module, $pred, "complicated_unify").
+    unexpected($pred, "complicated_unify").
 
 :- pred process_cons_and_decons(prog_var::in, cons_id::in, prog_var::in,
     int::in, int::out, rpt_graph::in, rpt_graph::out) is det.
@@ -234,9 +238,9 @@ process_cons_and_decons(LVar, ConsId, RVar, !Component, !Graph) :-
     % is actually added or not so it is convenient to check the edge's
     % existence outside edge_operator. Otherwise we can extend edge_operator
     % with one more argument to indicate that.
-    ( rptg_edge_in_graph(LNode, EdgeLabel, RNode, !.Graph) ->
+    ( if rptg_edge_in_graph(LNode, EdgeLabel, RNode, !.Graph) then
         true
-    ;
+    else
         edge_operator(LNode, RNode, EdgeLabel, !Graph),
 
         % After an edge is added, rules P2 and P3 are applied to ensure
@@ -262,50 +266,44 @@ process_cons_and_decons(LVar, ConsId, RVar, !Component, !Graph) :-
     rpta_info_table::in, rpta_info_table::out) is det.
 
 inter_proc_rpta(ModuleInfo0, !InfoTable) :-
-    module_info_ensure_dependency_info(ModuleInfo0, ModuleInfo),
-    module_info_get_maybe_dependency_info(ModuleInfo, MaybeDepInfo),
-    (
-        MaybeDepInfo = yes(DepInfo),
-        hlds_dependency_info_get_dependency_ordering(DepInfo, DepOrdering),
-        run_with_dependencies(DepOrdering, ModuleInfo, !InfoTable)
-    ;
-        MaybeDepInfo = no,
-        unexpected($module, $pred, "no dependency information")
-    ).
+    module_info_ensure_dependency_info(ModuleInfo0, ModuleInfo, DepInfo),
+    BottomUpSCCs = dependency_info_get_bottom_up_sccs(DepInfo),
+    run_with_dependencies(BottomUpSCCs, ModuleInfo, !InfoTable).
 
-:- pred run_with_dependencies(dependency_ordering::in, module_info::in,
-    rpta_info_table::in, rpta_info_table::out) is det.
+:- pred run_with_dependencies(hlds_bottom_up_dependency_sccs::in,
+    module_info::in, rpta_info_table::in, rpta_info_table::out) is det.
 
-run_with_dependencies(Deps, ModuleInfo, !InfoTable) :-
-    list.foldl(run_with_dependency(ModuleInfo), Deps, !InfoTable).
+run_with_dependencies(BottomUpSCCs, ModuleInfo, !InfoTable) :-
+    list.foldl(run_with_dependency(ModuleInfo), BottomUpSCCs, !InfoTable).
 
-:- pred run_with_dependency(module_info::in, list(pred_proc_id)::in,
+:- pred run_with_dependency(module_info::in, set(pred_proc_id)::in,
     rpta_info_table::in, rpta_info_table::out) is det.
 
 run_with_dependency(ModuleInfo, SCC, !InfoTable) :-
-    ( some_are_special_preds(SCC, ModuleInfo) ->
+    set.to_sorted_list(SCC, SCCProcs),
+    ( if some_are_special_preds(SCCProcs, ModuleInfo) then
         % Analysis ignores special predicates.
         true
-    ;
+    else
         % Run the fixpoint computation on the SCC.
-        FPTable = init_rpta_fixpoint_table(SCC, !.InfoTable),
+        FPTable = init_rpta_fixpoint_table(SCCProcs, !.InfoTable),
         run_with_dependency_until_fixpoint(SCC, FPTable, ModuleInfo,
             !InfoTable)
     ).
 
-:- pred run_with_dependency_until_fixpoint(list(pred_proc_id)::in,
+:- pred run_with_dependency_until_fixpoint(scc::in,
     rpta_fixpoint_table::in, module_info::in, rpta_info_table::in,
     rpta_info_table::out) is det.
 
 run_with_dependency_until_fixpoint(SCC, FPTable0, ModuleInfo, !InfoTable) :-
-    list.foldl(inter_analyse_proc(ModuleInfo, !.InfoTable), SCC,
+    set.foldl(inter_analyse_proc(ModuleInfo, !.InfoTable), SCC,
         FPTable0, FPTable1),
-    ( fixpoint_reached(FPTable1) ->
+    ( if fixpoint_reached(FPTable1) then
         % If we have reached a fixpoint for this SCC then update the
         % RPTA info table.
-        list.foldl(update_rpta_info_in_rpta_info_table(FPTable1), SCC,
+        set.foldl(update_rpta_info_in_rpta_info_table(FPTable1), SCC,
             !InfoTable)
-    ;
+    else
         % Otherwise, begin the next iteration.
         new_run(FPTable1, FPTable),
         run_with_dependency_until_fixpoint(SCC, FPTable, ModuleInfo,
@@ -423,7 +421,7 @@ inter_analyse_goal_expr(Goal, GoalInfo, ModuleInfo, InfoTable,
 
 inter_analyse_goal_expr(generic_call(_, _, _, _, _), _, _, _, !FPTable,
         !RptaInfo) :-
-    sorry($module, $pred, "generic_call not handled").
+    sorry($pred, "generic_call not handled").
 
 inter_analyse_goal_expr(switch(_, _, Cases), _, ModuleInfo, InfoTable,
         !FPTable, !RptaInfo) :-
@@ -461,7 +459,7 @@ inter_analyse_goal_expr(scope(_Reason, Goal), _, ModuleInfo, InfoTable,
 %    ;
 %        Msg = "inter_analyse_goal_expr: Scope's reason of from_ground_term "
 %            ++ "not handled",
-%        unexpected($module, $pred, Msg)
+%        unexpected($pred, Msg)
 %    ).
 
 inter_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), _, ModuleInfo,
@@ -472,10 +470,10 @@ inter_analyse_goal_expr(if_then_else(_Vars, If, Then, Else), _, ModuleInfo,
 
 inter_analyse_goal_expr(GoalExpr, _, _, _, !FPTable, !RptaInfo) :-
     GoalExpr = call_foreign_proc(_, _, _, _, _, _, _),
-    sorry($module, $pred, "foreign code").
+    sorry($pred, "foreign code").
 
 inter_analyse_goal_expr(shorthand(_), _, _, _, !FPTable, !RptaInfo) :-
-    unexpected($module, $pred, "shorthand").
+    unexpected($pred, "shorthand").
 
 :- pred inter_analyse_case(module_info::in,
     rpta_info_table::in, case::in, rpta_fixpoint_table::in,
@@ -499,20 +497,20 @@ inter_analyse_case(ModuleInfo, InfoTable, Case, !FPtable, !RptaInfo) :-
 
 lookup_rpta_info(PPId, InfoTable, !FPtable, RptaInfo, Init) :-
     ( if
-        % First look up in the current fixpoint table,
+        % First look up in the current fixpoint table, ...
         get_from_fixpoint_table(PPId, RptaInfo0, !.FPtable, FPtable1)
-      then
+    then
         RptaInfo  = RptaInfo0,
         !:FPtable = FPtable1,
         Init = bool.no
-      else
+    else
         % ... second look up among already recorded rpta_info.
         ( if
             RptaInfo0 = rpta_info_table_search_rpta_info(PPId, InfoTable)
-          then
+        then
             RptaInfo = RptaInfo0,
             Init = bool.no
-          else
+        else
             % Initialize a dummy.
             RptaInfo = rpta_info(rpt_graph_init, map.init),
             Init = bool.yes
@@ -579,7 +577,7 @@ rule_1(Node, !Graph) :-
     % the predicate will not look further in the list.
     % The unification of nodes, if happends (Happened = yes),
     % will be propagated by calling rule_1 predicate mutually recursively.
-    % 
+    %
     % The loop in this predicate is similar to
     % for i = ... to N - 1
     %    for j = i+1 to N ...
@@ -609,14 +607,14 @@ merge_nodes_reached_by_same_labelled_edges(Edge, [Ed | Eds], Rest, !Graph,
     rptg_get_edge_contents(!.Graph, Ed, _S, E, EdC),
     ( if
         EdgeContent = EdC
-      then
+    then
         % Unify the two end nodes.
         unify_operator(End, E, !.Graph, Graph1),
 
         % Apply rule 1 after the above unification.
         rule_1(End, Graph1, !:Graph),
         Happened = bool.yes
-      else
+    else
         % Still not found an edge with the same label, continue the
         % inner loop.
         merge_nodes_reached_by_same_labelled_edges(Edge, Eds, Rest, !Graph,
@@ -657,10 +655,10 @@ merge_nodes_reached_by_same_labelled_edge(Sel, M, [Ed | Eds], !Graph) :-
         EdgeContent = rptg_edge_content(Selector),
         Selector = Sel,
         MPrime \= M
-      then
+    then
         unify_operator(M, MPrime, !Graph),
         rule_1(M, !Graph)
-      else
+    else
         % Still not found an edge with the same label, continue the loop.
         merge_nodes_reached_by_same_labelled_edge(Sel, M, Eds, !Graph)
     ).
@@ -705,7 +703,9 @@ rule_3(Node, !Graph) :-
             % call. So this predicate can terminate at some point (due
             % to the fact that the "size" of !.Graph is smaller than that
             % of !:Graph).
-            rule_3(Node, !Graph)
+            disable_warning [suspicious_recursion] (
+                rule_3(Node, !Graph)
+            )
           ;
             % No node in Nodes has been unified with NY, which means that
             % no more nodes need to be unified, so just quit.
@@ -714,7 +714,7 @@ rule_3(Node, !Graph) :-
     ;
         Nodes = [],
         % No node in the graph, impossible.
-        unexpected($module, $pred, "impossible having no node in graph")
+        unexpected($pred, "impossible having no node in graph")
     ).
 
     % Check each node in the list to see if it satisfies the condition of
@@ -732,13 +732,13 @@ rule_3_2([], _, !Graph, bool.no).
 rule_3_2([NZ | NZs], NY, !Graph, Happened) :-
     ( if
         rule_3_condition(NZ, NY, !.Graph, NZ1)
-      then
+    then
         unify_operator(NZ, NZ1, !Graph),
 
         % Apply rule 1.
         rule_1(NZ, !Graph),
         Happened = bool.yes
-      else
+    else
         % Try with the rest, namely NS.
         rule_3_2(NZs, NY, !Graph, Happened)
     ).
@@ -768,34 +768,34 @@ rule_3_condition(NZ, NY, Graph, NZ1) :-
 
 alpha_mapping_at_call_site([], [], _, !CallerGraph, !AlphaMap).
 alpha_mapping_at_call_site([], [_ | _], _, _, _, _, _) :-
-    unexpected($module, $pred, "mismatched lists").
+    unexpected($pred, "mismatched lists").
 alpha_mapping_at_call_site([_ | _], [], _, _, _, _, _) :-
-    unexpected($module, $pred, "mismatched lists").
+    unexpected($pred, "mismatched lists").
 alpha_mapping_at_call_site([Xi | Xs], [Yi | Ys], CalleeGraph,
         !CallerGraph, !AlphaMap) :-
     % Xi's are formal arguments, Yi's are actual arguments at the call site.
     rptg_get_node_by_variable(CalleeGraph, Xi, N_Xi),
     rptg_get_node_by_variable(!.CallerGraph, Yi, N_Yi),
-    ( map.search(!.AlphaMap, N_Xi, N_Y) ->
+    ( if map.search(!.AlphaMap, N_Xi, N_Y) then
         % alpha(N_Xi) = N_Y, alpha(N_Xi) = N_Yi, N_Y != N_Yi.
         %
-        ( N_Y = N_Yi ->
+        ( if N_Y = N_Yi then
             true
-        ;
+        else
             % Apply rule P4.
             unify_operator(N_Y, N_Yi, !CallerGraph),
 
             % Apply rule P1 after some nodes are unified.
             rule_1(N_Y, !CallerGraph)
         )
-    ;
+    else
         map.set(N_Xi, N_Yi, !AlphaMap),
 
         % If N_Xi's is_allocated then N_Yi is also allocated.
         % Otherwise leave N_Yi alone.
-        ( rptg_is_allocated_node(CalleeGraph, N_Xi) ->
+        ( if rptg_is_allocated_node(CalleeGraph, N_Xi) then
             rptg_set_node_is_allocated(N_Yi, bool.yes, !CallerGraph)
-        ;
+        else
            true
         )
     ),
@@ -868,7 +868,7 @@ apply_rules_outedges([Edge | Edges], CallerNode, CallSite, CalleeRptaInfo,
 :- pred rule_5(rptg_edge::in, program_point::in, rpta_info::in,
     rptg_node::in, rpta_info::in, rpta_info::out) is det.
 
-rule_5(Edge, CallSite, CalleeRptaInfo, CallerNode, 
+rule_5(Edge, CallSite, CalleeRptaInfo, CallerNode,
         rpta_info(!.CallerGraph, CallerAlphaMapping),
         rpta_info(!:CallerGraph, CallerAlphaMapping)) :-
     % Find an out-edge in the caller's graph that has a same label
@@ -884,21 +884,21 @@ rule_5(Edge, CallSite, CalleeRptaInfo, CallerNode,
         map.search(AlphaAtCallSite, CalleeM, CallerM),
         rptg_get_node_by_node(!.CallerGraph, CallerM, RealCallerM),
         CallerMPrime \= RealCallerM
-      then
+    then
         % When the conditions of rule P5 are satisfied, nodes are unified and
         % rule P1 applied to ensure invariants.
         unify_operator(RealCallerM, CallerMPrime, !CallerGraph),
 
         % Apply rule P1 after the unification.
         rule_1(RealCallerM, !CallerGraph)
-      else
+    else
         true
     ).
 
 :- pred rule_6(rptg_edge::in, program_point::in, rpta_info::in,
     rptg_node::in, rpta_info::in, rpta_info::out) is det.
 
-rule_6(Edge, CallSite, CalleeRptaInfo, CallerNode, 
+rule_6(Edge, CallSite, CalleeRptaInfo, CallerNode,
         rpta_info(!.CallerGraph, !.CallerAlphaMapping),
         rpta_info(!:CallerGraph, !:CallerAlphaMapping)) :-
     % Find an out-edge in the caller's graph that has a same label
@@ -909,28 +909,27 @@ rule_6(Edge, CallSite, CalleeRptaInfo, CallerNode,
     ( if
         rptg_find_edge_from_node_with_same_content(RealCallerNode, Label,
             !.CallerGraph, CallerM)
-      then
+    then
         % (CallerNode, sel, CallerM) in the graph.
         map.lookup(!.CallerAlphaMapping, CallSite, AlphaAtCallSite0),
-        ( if
-            map.search(AlphaAtCallSite0, CalleeM, _)
-          then
+        ( if map.search(AlphaAtCallSite0, CalleeM, _) then
             % alpha(CalleeM) = CallerM so ignore.
             true
-          else
+        else
             % Apply rule P6: when its conditions are satisfied
             % record alpha(CalleeM) = CallerM.
             map.set(CalleeM, CallerM, AlphaAtCallSite0, AlphaAtCallSite1),
             map.set(CallSite, AlphaAtCallSite1, !CallerAlphaMapping),
 
-            % If CalleeM's is_allocated then CallerM is also allocated. 
+            % If CalleeM's is_allocated then CallerM is also allocated.
             % Otherwise leave CallerM alone.
-            ( rptg_is_allocated_node(CalleeGraph, CalleeM) ->
+            ( if rptg_is_allocated_node(CalleeGraph, CalleeM) then
                 rptg_set_node_is_allocated(CallerM, bool.yes, !CallerGraph)
-            ;   true
+            else
+                true
             )
         )
-      else
+    else
         true
     ).
 
@@ -948,14 +947,14 @@ rule_7(Edge, CallSite, CalleeRptaInfo, CallerNode,
     ( if
         rptg_find_edge_from_node_with_same_content(RealCallerNode, Label,
             !.CallerGraph, _)
-      then
+    then
         true
-      else
+    else
         % No edge from CallerNode with the label exists.
         ( if
             map.lookup(CallerAlphaMapping, CallSite, AlphaAtCallSite),
             map.search(AlphaAtCallSite, CalleeM, CallerM)
-          then
+        then
             % Reach here means all the conditions of rule P7 are satisfied,
             % add (CallerNode, sel, CallerM).
             rptg_get_node_by_node(!.CallerGraph, CallerM, RealCallerM),
@@ -963,7 +962,7 @@ rule_7(Edge, CallSite, CalleeRptaInfo, CallerNode,
 
             % Need to apply rule 3.
             rule_3(RealCallerM, !CallerGraph)
-          else
+        else
             true
         )
     ).
@@ -982,16 +981,16 @@ rule_8(Edge, CallSite, CalleeRptaInfo, CallerNode,
     ( if
         rptg_find_edge_from_node_with_same_content(RealCallerNode, Label,
             !.CallerGraph, _)
-      then
+    then
         true
-      else
+    else
         % No edge from CallerNode with the label exists.
         ( if
             map.lookup(!.CallerAlphaMapping, CallSite, AlphaAtCallSite0),
             map.search(AlphaAtCallSite0, CalleeM, _)
-          then
+        then
             true
-          else
+        else
             % rule 8: add node CallerM, alpha(CalleeM) = CallerM,
             % edge(CallerNode, sel, CallerM)
             %
@@ -1041,22 +1040,23 @@ init_rpta_fixpoint_table(Keys, InfoTable) = Table :-
     rpta_fixpoint_table::in, rpta_fixpoint_table::out) is det.
 
 rpta_fixpoint_table_new_rpta_info(PPId, RptaInfo, !Table) :-
-    EqualityTest = (pred(TabledElem::in, Elem::in) is semidet :-
-        rpta_info_equal(Elem, TabledElem)
-    ),
+    EqualityTest =
+        ( pred(TabledElem::in, Elem::in) is semidet :-
+            rpta_info_equal(Elem, TabledElem)
+        ),
     add_to_fixpoint_table(EqualityTest, PPId, RptaInfo, !Table).
 
 :- func wrapped_init(rpta_info_table, pred_proc_id) = rpta_info.
 
 wrapped_init(InfoTable, PPId) = Entry :-
-    ( Entry0 = rpta_info_table_search_rpta_info(PPId, InfoTable) ->
+    ( if Entry0 = rpta_info_table_search_rpta_info(PPId, InfoTable) then
         Entry = Entry0
-    ;
+    else
         % The information we are looking for should be there after the
         % intraprocedural analysis.
-        unexpected($module, $pred, "no rpta_info")
+        unexpected($pred, "no rpta_info")
     ).
 
 %-----------------------------------------------------------------------------%
-:- end_module rbmm.points_to_analysis.
+:- end_module transform_hlds.rbmm.points_to_analysis.
 %-----------------------------------------------------------------------------%
