@@ -35,11 +35,11 @@
 
 %-----------------------------------------------------------------------------%
 
-:- type renaming_table ==
-    map(pred_proc_id, renaming_proc).
+:- type rbmm_renaming_table ==
+    map(pred_proc_id, rbmm_renaming_proc).
 
-:- type renaming_proc ==
-    map(program_point, renaming).
+:- type rbmm_renaming_proc ==
+    map(program_point, rbmm_renaming).
 
     % Most of the time, at a program point there is only one renaming for a
     % region variable. But at some resurrection points, two renamings exists
@@ -49,12 +49,12 @@
     %       create(R), % R -> R_Resur_2
     %   (i) goal
     % That's why multi_map is used.
-:- type renaming == multi_map(string, string).
+:- type rbmm_renaming == multi_map(string, string).
 
-:- type renaming_annotation_table ==
-    map(pred_proc_id, renaming_annotation_proc).
+:- type rbmm_renaming_annotation_table ==
+    map(pred_proc_id, rbmm_renaming_annotation_proc).
 
-:- type renaming_annotation_proc ==
+:- type rbmm_renaming_annotation_proc ==
     map(program_point, list(region_instr)).
 
 :- type proc_resurrection_path_table ==
@@ -120,25 +120,25 @@
     %
 :- pred collect_region_resurrection_renaming(proc_pp_region_set_table::in,
     proc_region_set_table::in, rpta_info_table::in,
-    proc_resurrection_path_table::in,
-    renaming_table::out) is det.
+    proc_resurrection_path_table::in, rbmm_renaming_table::out) is det.
 
     % This predicate collects *renaming* along the execution paths in
     % procedures where region resurrection happens. It also computes
     % the reversed renaming *annotations* to ensure the integrity use of
     % regions.
     %
-:- pred collect_renaming_and_annotation(renaming_table::in,
+:- pred collect_renaming_and_annotation(rbmm_renaming_table::in,
     join_point_region_name_table::in, proc_pp_region_set_table::in,
     proc_pp_region_set_table::in, proc_region_set_table::in,
     rpta_info_table::in, proc_resurrection_path_table::in,
-    execution_path_table::in, renaming_annotation_table::out,
-    renaming_table::out) is det.
+    execution_path_table::in, rbmm_renaming_annotation_table::out,
+    rbmm_renaming_table::out) is det.
 
     % Record the annotation for a procedure.
     %
 :- pred record_annotation(program_point::in, region_instr::in,
-    renaming_annotation_proc::in, renaming_annotation_proc::out) is det.
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out)
+    is det.
 
     % Make a region renaming instruction.
     %
@@ -203,9 +203,9 @@ compute_resurrection_paths_proc(LRBeforeTable, LRAfterTable,
         map.init, PathContainsResurrectionProc),
     % We only want to include procedures in which resurrection happens in this
     % map.
-    ( map.count(PathContainsResurrectionProc) = 0 ->
+    ( if map.count(PathContainsResurrectionProc) = 0 then
         true
-    ;
+    else
         map.set(PPId, PathContainsResurrectionProc,
             !PathContainsResurrectionTable)
     ).
@@ -223,9 +223,9 @@ compute_resurrection_paths_exec_path(LRBeforeProc, LRAfterProc,
         BecomeDeadAfterProc), ExecPath,
         set.init, _, set.init, _, set.init, ResurrectedRegionsInExecPath),
     % We want to record only execution paths in which resurrections happen.
-    ( set.empty(ResurrectedRegionsInExecPath) ->
+    ( if set.is_empty(ResurrectedRegionsInExecPath) then
         true
-    ;
+    else
         map.set(ExecPath, ResurrectedRegionsInExecPath,
             !ResurrectedRegionProc)
     ).
@@ -311,28 +311,28 @@ collect_join_points_path(Paths, Path, !JP2Name, !Counter, !JoinPoints,
     list.delete_all(Paths, Path, TheOtherPaths),
     % We ignore the first program point in each path because
     % it cannot be a join point.
-    ( Path = [PrevPoint, ProgPoint | ProgPoints] ->
-        ( set.member(ProgPoint, !.JoinPoints) ->
+    ( if Path = [PrevPoint, ProgPoint | ProgPoints] then
+        ( if set.member(ProgPoint, !.JoinPoints) then
             true
-        ;
-            ( is_join_point(ProgPoint, PrevPoint, TheOtherPaths) ->
+        else
+            ( if is_join_point(ProgPoint, PrevPoint, TheOtherPaths)then
                 % Try to lookup the postfix at this jp.
-                ( map.search(!.JP2Name, PrevPoint, JPName0) ->
+                ( if map.search(!.JP2Name, PrevPoint, JPName0)then
                     JPName = JPName0
-                ;
+                else
                     counter.allocate(N, !Counter),
                     JPName = "_jp_" ++ string.int_to_string(N),
                     map.set(PrevPoint, JPName, !JP2Name)
                 ),
                 map.set(ProgPoint, JPName, !JoinPointProc),
                 set.insert(ProgPoint, !JoinPoints)
-            ;
+            else
                 true
             )
         ),
         collect_join_points_path(Paths, [ProgPoint | ProgPoints],
             !JP2Name, !Counter, !JoinPoints, !JoinPointProc)
-    ;
+    else
         true
     ).
 
@@ -344,9 +344,9 @@ collect_join_points_path(Paths, Path, !JP2Name, !Counter, !JoinPoints,
     list(list(program_point))::in) is semidet.
 
 is_join_point(ProgPoint, PrevProgPoint, [Path | Paths]) :-
-    ( is_join_point_2(ProgPoint, PrevProgPoint, Path) ->
+    ( if is_join_point_2(ProgPoint, PrevProgPoint, Path)then
         true
-    ;
+    else
         is_join_point(ProgPoint, PrevProgPoint, Paths)
     ).
 
@@ -354,9 +354,9 @@ is_join_point(ProgPoint, PrevProgPoint, [Path | Paths]) :-
     list(program_point)::in) is semidet.
 
 is_join_point_2(ProgPoint, PrevProgPoint, [P1, P2 | Ps]) :-
-    ( P2 = ProgPoint ->
+    ( if P2 = ProgPoint then
         P1 \= PrevProgPoint
-    ;
+    else
         is_join_point_2(ProgPoint, PrevProgPoint, [P2 | Ps])
     ).
 
@@ -380,17 +380,16 @@ collect_paths_containing_join_points_proc(ExecPathTable, JoinPointTable, PPId,
     map.lookup(ExecPathTable, PPId, ExecPaths),
     map.values(PathContainsResurrectionProc, ResurrectedRegionsInPaths),
     list.foldl(
-        pred(ResurRegions::in, R0::in, R::out) is det :- (
+        ( pred(ResurRegions::in, R0::in, R::out) is det :-
             set.union(R0, ResurRegions, R)
-        ),
-        ResurrectedRegionsInPaths, set.init, ResurrectedRegionsProc),
+        ), ResurrectedRegionsInPaths, set.init, ResurrectedRegionsProc),
     map.keys(PathContainsResurrectionProc, PathsContainResurrection),
     list.delete_elems(ExecPaths, PathsContainResurrection, NonResurPaths),
-    ( map.search(JoinPointTable, PPId, JoinPointProc) ->
+    ( if map.search(JoinPointTable, PPId, JoinPointProc) then
         list.foldl(path_containing_join_point(JoinPointProc, PPId,
             ResurrectedRegionsProc),
             NonResurPaths, !PathContainsResurrectionTable)
-    ;
+    else
         true
     ).
 
@@ -404,9 +403,9 @@ path_containing_join_point(JoinPointProc, PPId, ResurrectedRegionsProc,
     assoc_list.keys(NonResurPath, ProgPointsInPath),
     map.foldl(find_join_points_in_path(ProgPointsInPath), JoinPointProc,
         set.init, JoinPointsInThisPath),
-    ( set.empty(JoinPointsInThisPath) ->
+    ( if set.is_empty(JoinPointsInThisPath) then
         true
-    ;
+    else
         map.lookup(!.PathContainsResurrectionTable, PPId,
             PathContainsResurrectionProc0),
         map.det_insert(NonResurPath, ResurrectedRegionsProc,
@@ -420,9 +419,9 @@ path_containing_join_point(JoinPointProc, PPId, ResurrectedRegionsProc,
     set(program_point)::out) is det.
 
 find_join_points_in_path(ProgPointsInPath, JoinPoint, _, !JoinPoints) :-
-    ( list.member(JoinPoint, ProgPointsInPath) ->
+    ( if list.member(JoinPoint, ProgPointsInPath) then
         set.insert(JoinPoint, !JoinPoints)
-    ;
+    else
         true
     ).
 
@@ -438,9 +437,8 @@ collect_region_resurrection_renaming(BecomeLiveTable, LocalRTable,
 
 :- pred collect_region_resurrection_renaming_proc(
     proc_pp_region_set_table::in, proc_region_set_table::in,
-    rpta_info_table::in, pred_proc_id::in,
-    map(execution_path, region_set)::in, renaming_table::in,
-    renaming_table::out) is det.
+    rpta_info_table::in, pred_proc_id::in, map(execution_path, region_set)::in,
+    rbmm_renaming_table::in, rbmm_renaming_table::out) is det.
 
 collect_region_resurrection_renaming_proc(BecomeLiveTable, _LocalRTable,
         RptaInfoTable, PPId, PathsContainResurrection,
@@ -455,9 +453,8 @@ collect_region_resurrection_renaming_proc(BecomeLiveTable, _LocalRTable,
     map.set(PPId, ResurrectionRenameProc, !ResurrectionRenameTable).
 
 :- pred collect_region_resurrection_renaming_exec_path(rpt_graph::in,
-    pp_region_set_table::in, execution_path::in,
-    region_set::in, renaming_proc::in,
-    renaming_proc::out) is det.
+    pp_region_set_table::in, execution_path::in, region_set::in,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_region_resurrection_renaming_exec_path(Graph, BecomeLiveProc,
         ExecPath, ResurrectedRegions, !ResurrectionRenameProc) :-
@@ -469,7 +466,7 @@ collect_region_resurrection_renaming_exec_path(Graph, BecomeLiveProc,
 :- pred collect_region_resurrection_renaming_prog_point(rpt_graph::in,
     pp_region_set_table::in, region_set::in,
     pair(program_point, hlds_goal)::in, counter::in, counter::out,
-    renaming_proc::in, renaming_proc::out) is det.
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_region_resurrection_renaming_prog_point(Graph, BecomeLiveProc,
         ResurrectedRegions, ProgPoint - _, !RenamingCounter,
@@ -478,9 +475,9 @@ collect_region_resurrection_renaming_prog_point(Graph, BecomeLiveProc,
     set.intersect(ResurrectedRegions, BecomeLiveProgPoint,
         ToBeRenamedRegions),
     % We only record the program points where resurrection renaming exists.
-    ( set.empty(ToBeRenamedRegions) ->
+    ( if set.is_empty(ToBeRenamedRegions) then
         true
-    ;
+    else
         counter.allocate(N, !RenamingCounter),
         set.fold(
             record_renaming_prog_point(Graph, ProgPoint, N),
@@ -488,17 +485,18 @@ collect_region_resurrection_renaming_prog_point(Graph, BecomeLiveProc,
     ).
 
 :- pred record_renaming_prog_point(rpt_graph::in, program_point::in, int::in,
-    rptg_node::in, renaming_proc::in, renaming_proc::out) is det.
+    rptg_node::in, rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 record_renaming_prog_point(Graph, ProgPoint, RenamingCounter, Region,
         !ResurrectionRenameProc) :-
     RegionName = rptg_lookup_region_name(Graph, Region),
     Renamed = RegionName ++ "_Resur_"
         ++ string.int_to_string(RenamingCounter),
-
-    ( map.search(!.ResurrectionRenameProc, ProgPoint, RenamingProgPoint0) ->
+    ( if
+        map.search(!.ResurrectionRenameProc, ProgPoint, RenamingProgPoint0)
+    then
         map.set(RegionName, [Renamed], RenamingProgPoint0, RenamingProgPoint)
-    ;
+    else
         RenamingProgPoint = map.singleton(RegionName, [Renamed])
     ),
     map.set(ProgPoint, RenamingProgPoint, !ResurrectionRenameProc).
@@ -529,9 +527,9 @@ collect_renaming_and_annotation(ResurrectionRenameTable, JoinPointTable,
     join_point_region_name_table::in, proc_pp_region_set_table::in,
     proc_pp_region_set_table::in, proc_region_set_table::in,
     rpta_info_table::in, proc_resurrection_path_table::in, pred_proc_id::in,
-    renaming_proc::in,
-    renaming_annotation_table::in, renaming_annotation_table::out,
-    renaming_table::in, renaming_table::out) is det.
+    rbmm_renaming_proc::in,
+    rbmm_renaming_annotation_table::in, rbmm_renaming_annotation_table::out,
+    rbmm_renaming_table::in, rbmm_renaming_table::out) is det.
 
 collect_renaming_and_annotation_proc(ExecPathTable, JoinPointTable,
         LRBeforeTable, LRAfterTable, BornRTable, RptaInfoTable,
@@ -568,28 +566,27 @@ collect_renaming_and_annotation_proc(ExecPathTable, JoinPointTable,
     % (3) R1 --> R1_2, R2 --> R2_1 //R1 becomes live again, needs a new name.
     % ...
     %
-:- pred collect_renaming_and_annotation_exec_path(renaming_proc::in,
+:- pred collect_renaming_and_annotation_exec_path(rbmm_renaming_proc::in,
     map(program_point, string)::in, pp_region_set_table::in,
     pp_region_set_table::in, region_set::in, rpt_graph::in, region_set::in,
     execution_path::in,
-    renaming_annotation_proc::in, renaming_annotation_proc::out,
-    renaming_proc::in, renaming_proc::out) is det.
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
 collect_renaming_and_annotation_exec_path(_, _, _, _, _, _, _, [],
         !AnnotationProc, !RenamingProc) :-
-    unexpected($module, $pred, "empty execution path").
-
-    % This is the first program point in an execution path.
-    % It cannot be a join point. Renaming is needed at this point only
-    % when it is a resurrection point.
-    %
+    unexpected($pred, "empty execution path").
 collect_renaming_and_annotation_exec_path(ResurrectionRenameProc,
         JoinPointProc, LRBeforeProc, LRAfterProc, BornR, Graph,
         ResurrectedRegions, [ProgPoint - _ | ProgPoint_Goals],
         !AnnotationProc, !RenamingProc) :-
-    ( map.search(ResurrectionRenameProc, ProgPoint, ResurRename) ->
+    % This is the first program point in an execution path.
+    % It cannot be a join point. Renaming is needed at this point only
+    % when it is a resurrection point.
+    %
+    ( if map.search(ResurrectionRenameProc, ProgPoint, ResurRename) then
         map.set(ProgPoint, ResurRename, !RenamingProc)
-    ;
+    else
         map.set(ProgPoint, map.init, !RenamingProc)
     ),
     collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
@@ -597,20 +594,22 @@ collect_renaming_and_annotation_exec_path(ResurrectionRenameProc,
         ResurrectedRegions, ProgPoint, ProgPoint_Goals, !AnnotationProc,
         !RenamingProc).
 
-:- pred collect_renaming_and_annotation_exec_path_2(renaming_proc::in,
+:- pred collect_renaming_and_annotation_exec_path_2(rbmm_renaming_proc::in,
     map(program_point, string)::in, pp_region_set_table::in,
     pp_region_set_table::in,
     region_set::in, rpt_graph::in, region_set::in, program_point::in,
     execution_path::in,
-    renaming_annotation_proc::in, renaming_annotation_proc::out,
-    renaming_proc::in, renaming_proc::out) is det.
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::out) is det.
 
-    % This means the first program point is also the last.
-    % We do not need to do anything more.
-    %
 collect_renaming_and_annotation_exec_path_2(_, _, _, _, _, _, _, _, [],
         !AnnotationProc, !RenamingProc).
-
+    % This means the first program point is also the last.
+    % We do not need to do anything more.
+collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
+        JoinPointProc, LRBeforeProc, LRAfterProc, BornR, Graph,
+        ResurrectedRegions, PrevProgPoint, [ProgPoint - _ | ProgPoint_Goals],
+        !AnnotationProc, !RenamingProc) :-
     % This is a program point which is not the first.
     %
     % A program point can belong to different execution paths, therefore
@@ -630,24 +629,20 @@ collect_renaming_and_annotation_exec_path_2(_, _, _, _, _, _, _, _, [],
     % At the last program point, we need to add annotations for any region
     % parameters which resurrect.
     %
-collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
-        JoinPointProc, LRBeforeProc, LRAfterProc, BornR, Graph,
-        ResurrectedRegions, PrevProgPoint, [ProgPoint - _ | ProgPoint_Goals],
-        !AnnotationProc, !RenamingProc) :-
     map.lookup(!.RenamingProc, PrevProgPoint, PrevRenaming),
-    ( map.search(ResurrectionRenameProc, ProgPoint, ResurRenaming) ->
+    ( if map.search(ResurrectionRenameProc, ProgPoint, ResurRenaming) then
         % This is a resurrection point of some region(s). We need to merge
         % the existing renaming at the previous point with the resurrection
         % renaming here. When two renamings have the same key, i.e.,
         % the related region resurrects, we will keep both renamings.
         multi_map.merge(PrevRenaming, ResurRenaming, Renaming0),
         map.set(ProgPoint, Renaming0, !RenamingProc)
-    ;
+    else
         % This is not a resurrection point (of any regions).
         % Renaming at this point is the same as at its previous point.
         map.set(ProgPoint, PrevRenaming, !RenamingProc)
     ),
-    ( map.search(JoinPointProc, ProgPoint, JoinPointName) ->
+    ( if map.search(JoinPointProc, ProgPoint, JoinPointName) then
         % This is a join point.
         % Add annotations to the previous point.
         map.lookup(LRBeforeProc, ProgPoint, LRBeforeProgPoint),
@@ -664,7 +659,7 @@ collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
         % We will just overwrite any existing renaming information
         % at this point.
         map.set(ProgPoint, Renaming, !RenamingProc)
-    ;
+    else
         true
     ),
     (
@@ -673,8 +668,9 @@ collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
         % Add reversed renaming for regions in bornR.
         set.intersect(ResurrectedRegions, BornR, ResurrectedAndBornRegions),
         map.lookup(!.RenamingProc, ProgPoint, LastRenaming),
-        set.fold(add_annotation_at_last_prog_point(ProgPoint, Graph,
-                    LastRenaming),
+        set.fold(
+            add_annotation_at_last_prog_point(ProgPoint, Graph,
+                LastRenaming),
             ResurrectedAndBornRegions, !AnnotationProc)
     ;
         ProgPoint_Goals = [_ | _],
@@ -688,9 +684,9 @@ collect_renaming_and_annotation_exec_path_2(ResurrectionRenameProc,
     % point and records renaming from existing region name.
     %
 :- pred add_annotation_and_renaming_at_join_point(program_point::in,
-    rpt_graph::in, string::in, renaming::in, rptg_node::in,
-    renaming_annotation_proc::in, renaming_annotation_proc::out,
-    renaming::in, renaming::out) is det.
+    rpt_graph::in, string::in, rbmm_renaming::in, rptg_node::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::out,
+    rbmm_renaming::in, rbmm_renaming::out) is det.
 
 add_annotation_and_renaming_at_join_point(PrevProgPoint, Graph, JoinPointName,
         PrevRenaming, Region, !AnnotationProc, !Renaming) :-
@@ -707,18 +703,18 @@ add_annotation_and_renaming_at_join_point(PrevProgPoint, Graph, JoinPointName,
     % It seems that we have to add annotations (reverse renaming) for ones that
     % have not been renamed as implemented below too. The only difference is
     % that the reverse renaming is between the new name and the original name.
-    ( map.search(PrevRenaming, RegionName, RenamedNames) ->
+    ( if map.search(PrevRenaming, RegionName, RenamedNames) then
         list.det_last(RenamedNames, CurrentName),
         make_renaming_instruction(CurrentName, NewName, Annotation),
         record_annotation(PrevProgPoint, Annotation, !AnnotationProc)
-    ;
+    else
         make_renaming_instruction(RegionName, NewName, Annotation),
         record_annotation(PrevProgPoint, Annotation, !AnnotationProc)
     ).
 
 :- pred add_annotation_at_last_prog_point(program_point::in, rpt_graph::in,
-    renaming::in, rptg_node::in, renaming_annotation_proc::in,
-    renaming_annotation_proc::out) is det.
+    rbmm_renaming::in, rptg_node::in, rbmm_renaming_annotation_proc::in,
+    rbmm_renaming_annotation_proc::out) is det.
 
 add_annotation_at_last_prog_point(ProgPoint, Graph, Renaming, Region,
         !AnnotationProc) :-
@@ -727,22 +723,22 @@ add_annotation_at_last_prog_point(ProgPoint, Graph, Renaming, Region,
     % Add annotation to (after) the program point.
     % Annotations are only added for resurrected regions that have been
     % renamed in this execution path.
-    ( map.search(Renaming, RegionName, CurrentNameList) ->
+    ( if map.search(Renaming, RegionName, CurrentNameList) then
         CurrentName = list.det_last(CurrentNameList),
         make_renaming_instruction(CurrentName, RegionName, Annotation),
         record_annotation(ProgPoint, Annotation, !AnnotationProc)
-    ;
+    else
         true
     ).
 
 record_annotation(ProgPoint, Annotation, !AnnotationProc) :-
-    ( map.search(!.AnnotationProc, ProgPoint, Annotations0) ->
-        ( list.member(Annotation, Annotations0) ->
+    ( if map.search(!.AnnotationProc, ProgPoint, Annotations0) then
+        ( if list.member(Annotation, Annotations0) then
             Annotations = Annotations0
-        ;
+        else
             Annotations = [Annotation | Annotations0]
         )
-    ;
+    else
         % No annotation exists at this program point yet.
         Annotations = [Annotation]
     ),
