@@ -61,8 +61,8 @@
     %
 :- pred region_transform(rpta_info_table::in,
     proc_formal_region_args_table::in, proc_pp_actual_region_args_table::in,
-    renaming_table::in, renaming_table::in, region_instr_table::in,
-    renaming_annotation_table::in, renaming_annotation_table::in,
+    rbmm_renaming_table::in, rbmm_renaming_table::in, region_instr_table::in,
+    rbmm_renaming_annotation_table::in, rbmm_renaming_annotation_table::in,
     name_to_prog_var_table::in, name_to_prog_var_table::out,
     module_info::in, module_info::out) is det.
 
@@ -72,15 +72,19 @@
 :- implementation.
 
 :- import_module check_hlds.
-:- import_module check_hlds.mode_util.
 :- import_module check_hlds.purity.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module check_hlds.type_util.
+:- import_module hlds.goal_form.
 :- import_module hlds.goal_path.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.instmap.
 :- import_module hlds.pred_table.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
+:- import_module mdbcomp.
+:- import_module mdbcomp.builtin_modules.
 :- import_module mdbcomp.prim_data.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_mode.
@@ -90,10 +94,7 @@
 
 :- import_module int.
 :- import_module list.
-:- import_module pair.
 :- import_module require.
-:- import_module set.
-:- import_module string.
 :- import_module term.
 :- import_module varset.
 
@@ -133,9 +134,9 @@ region_transform(RptaInfoTable, FormalRegionArgTable, ActualRegionArgTable,
 
 annotate_pred(PPId, FormalRegionArgs, !Processed, !ModuleInfo) :-
     PPId = proc(PredId, _),
-    ( list.member(PredId, !.Processed) ->
+    ( if list.member(PredId, !.Processed) then
         true
-    ;
+    else
         some [!PredInfo] (
             module_info_pred_info(!.ModuleInfo, PredId, !:PredInfo),
             FormalRegionArgs = region_args(Constants, Deads, Borns),
@@ -169,8 +170,8 @@ annotate_pred(PPId, FormalRegionArgs, !Processed, !ModuleInfo) :-
 :- pred region_transform_pred(rpta_info_table::in,
     proc_formal_region_args_table::in,
     proc_pp_actual_region_args_table::in,
-    renaming_table::in, renaming_table::in, region_instr_table::in,
-    renaming_annotation_table::in, renaming_annotation_table::in,
+    rbmm_renaming_table::in, rbmm_renaming_table::in, region_instr_table::in,
+    rbmm_renaming_annotation_table::in, rbmm_renaming_annotation_table::in,
     pred_id::in, name_to_prog_var_table::in, name_to_prog_var_table::out,
     module_info::in, module_info::out) is det.
 
@@ -179,12 +180,14 @@ region_transform_pred(RptaInfoTable, FormalRegionArgTable,
         RegionInstructionTable, ResurRenamingAnnoTable, IteRenamingAnnoTable,
         PredId, !NameToVarTable, !ModuleInfo) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
-    list.foldl2(region_transform_proc(RptaInfoTable,
-        FormalRegionArgTable, ActualRegionArgTable,
-        ResurRenamingTable, IteRenamingTable,
-        RegionInstructionTable, ResurRenamingAnnoTable, IteRenamingAnnoTable,
-        PredId), ProcIds, !NameToVarTable, !ModuleInfo).
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
+    list.foldl2(
+        region_transform_proc(RptaInfoTable,
+            FormalRegionArgTable, ActualRegionArgTable,
+            ResurRenamingTable, IteRenamingTable,
+            RegionInstructionTable, ResurRenamingAnnoTable,
+            IteRenamingAnnoTable, PredId),
+        ProcIds, !NameToVarTable, !ModuleInfo).
 
     % This predicate updates the proc_info data structure, representing
     % a procedure.
@@ -202,8 +205,8 @@ region_transform_pred(RptaInfoTable, FormalRegionArgTable,
     %
 :- pred region_transform_proc(rpta_info_table::in,
     proc_formal_region_args_table::in, proc_pp_actual_region_args_table::in,
-    renaming_table::in, renaming_table::in, region_instr_table::in,
-    renaming_annotation_table::in, renaming_annotation_table::in,
+    rbmm_renaming_table::in, rbmm_renaming_table::in, region_instr_table::in,
+    rbmm_renaming_annotation_table::in, rbmm_renaming_annotation_table::in,
     pred_id::in, proc_id::in, name_to_prog_var_table::in,
     name_to_prog_var_table::out, module_info::in, module_info::out) is det.
 
@@ -222,17 +225,17 @@ region_transform_proc(RptaInfoTable, FormalRegionArgTable,
     map.lookup(RptaInfoTable, PPId, rpta_info(Graph, _)),
     map.lookup(FormalRegionArgTable, PPId, FormalRegionArgProc),
     map.lookup(ActualRegionArgTable, PPId, ActualRegionArgProc),
-    ( map.search(ResurRenamingTable, PPId, ResurRenamingProc0) ->
+    ( if map.search(ResurRenamingTable, PPId, ResurRenamingProc0) then
         ResurRenamingProc = ResurRenamingProc0,
         map.lookup(ResurRenamingAnnoTable, PPId, ResurRenamingAnnoProc)
-    ;
+    else
         ResurRenamingProc = map.init,
         ResurRenamingAnnoProc = map.init
     ),
-    ( map.search(IteRenamingTable, PPId, IteRenamingProc0) ->
+    ( if map.search(IteRenamingTable, PPId, IteRenamingProc0) then
         IteRenamingProc = IteRenamingProc0,
         map.lookup(IteRenamingAnnoTable, PPId, IteRenamingAnnoProc)
-    ;
+    else
         IteRenamingProc = map.init,
         IteRenamingAnnoProc = map.init
     ),
@@ -258,9 +261,10 @@ region_transform_proc(RptaInfoTable, FormalRegionArgTable,
     %    + new calls to region instructions
     %
 :- pred annotate_proc(module_info::in, pred_info::in, rpt_graph::in,
-    region_args::in, pp_actual_region_args_table::in, renaming_proc::in,
-    renaming_proc::in, region_instr_proc::in, renaming_annotation_proc::in,
-    renaming_annotation_proc::in, prog_varset::in, prog_varset::out,
+    region_args::in, pp_actual_region_args_table::in, rbmm_renaming_proc::in,
+    rbmm_renaming_proc::in, region_instr_proc::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::in,
+    prog_varset::in, prog_varset::out,
     vartypes::in, vartypes::out, list(prog_var)::in, list(prog_var)::out,
     list(mer_mode)::in, list(mer_mode)::out, hlds_goal::in, hlds_goal::out,
     name_to_prog_var::in, name_to_prog_var::out, proc_info::in,
@@ -337,10 +341,10 @@ annotate_proc(ModuleInfo, PredInfo, Graph, FormalRegionArgProc,
     % caused by renaming and annotations needed for resurrection problem.
     %
 :- pred region_transform_goal(module_info::in, rpt_graph::in,
-    renaming_proc::in, renaming_proc::in, pp_actual_region_args_table::in,
-    region_instr_proc::in, renaming_annotation_proc::in,
-    renaming_annotation_proc::in, hlds_goal::in, hlds_goal::out,
-    name_to_prog_var::in, name_to_prog_var::out,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::in,
+    pp_actual_region_args_table::in, region_instr_proc::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::in,
+    hlds_goal::in, hlds_goal::out, name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_transform_goal(ModuleInfo, Graph, ResurRenamingProc, IteRenamingProc,
@@ -367,10 +371,10 @@ region_transform_goal(ModuleInfo, Graph, ResurRenamingProc, IteRenamingProc,
             !NameToVar, !VarSet, !VarTypes, [], IteRenamingAssignments),
 
         % Region instructions before and after this program point.
-        (
+        ( if
             map.search(RegionInstructionProc, ProgPoint,
                 instrs_before_after(Before, After))
-        ->
+        then
             % Region instructions before this program point.
             list.foldl4(region_instruction_to_conj_before(ModuleInfo, Context,
                 ResurRenaming, IteRenaming), Before, !NameToVar,
@@ -383,7 +387,7 @@ region_transform_goal(ModuleInfo, Graph, ResurRenamingProc, IteRenamingProc,
             list.foldl4(region_instruction_to_conj(ModuleInfo, Context,
                 ResurRenaming, IteRenaming), After, !NameToVar,
                 !VarSet, !VarTypes, Conjs2, Conjs3)
-        ;
+        else
             % The goal at this program point itself.
             Conjs3 = IteRenamingAssignments ++ [hlds_goal(GoalExpr, GoalInfo)]
         ),
@@ -393,9 +397,9 @@ region_transform_goal(ModuleInfo, Graph, ResurRenamingProc, IteRenamingProc,
             ProgPoint, IteRenaming, !NameToVar, !VarSet, !VarTypes,
             Conjs3, Conjs),
 
-        ( Conjs = [_, _ | _] ->
+        ( if Conjs = [_, _ | _] then
             !:Goal = hlds_goal(conj(plain_conj, Conjs), GoalInfo)
-        ;
+        else
             !:Goal = hlds_goal(GoalExpr, GoalInfo)
         )
     ;
@@ -407,7 +411,7 @@ region_transform_goal(ModuleInfo, Graph, ResurRenamingProc, IteRenamingProc,
     ).
 
 :- pred region_transform_goal_expr(module_info::in, rpt_graph::in,
-    renaming::in, renaming::in, pp_actual_region_args_table::in,
+    rbmm_renaming::in, rbmm_renaming::in, pp_actual_region_args_table::in,
     program_point::in, hlds_goal_expr::in, hlds_goal_expr::out,
     hlds_goal_info::in, hlds_goal_info::out, name_to_prog_var::in,
     name_to_prog_var::out, prog_varset::in, prog_varset::out,
@@ -422,9 +426,9 @@ region_transform_goal_expr(ModuleInfo, Graph, ResurRenaming, IteRenaming,
         Context, Name),
     % XXX Callee may be a builtin or an imported procedure that we have
     % not analysed, we just ignore such a call for now.
-    ( map.search(ActualRegionArgProc, ProgPoint, ActualNodes0) ->
+    ( if map.search(ActualRegionArgProc, ProgPoint, ActualNodes0) then
         ActualNodes = ActualNodes0
-    ;
+    else
         ActualNodes = region_args([], [], [])
     ),
     ActualNodes = region_args(Constants, Ins, Outs),
@@ -460,12 +464,12 @@ region_transform_goal_expr(ModuleInfo, Graph, ResurRenaming, IteRenaming,
 region_transform_goal_expr(_, _, _, _, _, _, !GoalExpr, !GoalInfo, !NameToVar,
         !VarSet, !VarTypes) :-
     !.GoalExpr = generic_call(_, _, _, _, _),
-    sorry($module, $pred, "generic call").
+    sorry($pred, "generic call").
 
 region_transform_goal_expr(_, _, _, _, _, _, !GoalExpr, !GoalInfo, !NameToVar,
         !VarSet, !VarTypes) :-
     !.GoalExpr = call_foreign_proc(_, _, _, _, _, _, _),
-    sorry($module, $pred, "call_foreign_proc").
+    sorry($pred, "call_foreign_proc").
 
 region_transform_goal_expr(_, _, _, _, _, _, !GoalExpr, !GoalInfo, !NameToVar,
         !VarSet, !VarTypes) :-
@@ -483,15 +487,15 @@ region_transform_goal_expr(_, _, _, _, _, _, !GoalExpr, !GoalInfo, !NameToVar,
     ; !.GoalExpr = scope(_, _)
     ; !.GoalExpr = shorthand(_)
     ),
-    unexpected($module, $pred, "compound goal").
+    unexpected($pred, "compound goal").
 
     % Because an atomic goal is turned into a conjunction, we need to
     % flatten its compounding conjunction if it is in one.
 :- pred region_transform_compound_goal(module_info::in, rpt_graph::in,
-    renaming_proc::in, renaming_proc::in, pp_actual_region_args_table::in,
-    region_instr_proc::in, renaming_annotation_proc::in,
-    renaming_annotation_proc::in, hlds_goal::in, hlds_goal::out,
-    name_to_prog_var::in, name_to_prog_var::out,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::in,
+    pp_actual_region_args_table::in, region_instr_proc::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::in,
+    hlds_goal::in, hlds_goal::out, name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_transform_compound_goal(ModuleInfo, Graph,
@@ -545,9 +549,9 @@ region_transform_compound_goal(ModuleInfo, Graph,
         % construct STATIC ground terms. Since these are not created
         % dynamically, they cannot possibly refer to any dynamically created
         % region, so RBMM transformations should ignore them.
-        ( Reason0 = from_ground_term(Var, _Kind) ->
+        ( if Reason0 = from_ground_term(Var, _Kind) then
             Reason = from_ground_term(Var, from_ground_term_other)
-        ;
+        else
             Reason = Reason0
         ),
         region_transform_goal(ModuleInfo, Graph, ResurRenamingProc,
@@ -579,7 +583,7 @@ region_transform_compound_goal(ModuleInfo, Graph,
         ; !.GoalExpr = conj(_, [])
         ; !.GoalExpr = disj([])
         ),
-        unexpected($module, $pred, "shorthand or atomic goal")
+        unexpected($pred, "shorthand or atomic goal")
     ).
 
     % This predicate needs to be consistent with what are done in
@@ -589,7 +593,7 @@ region_transform_compound_goal(ModuleInfo, Graph,
     % The current implementation may not be correct.
     %
 :- pred annotate_constructions_unification(module_info::in, rpt_graph::in,
-    renaming::in, renaming::in, unification::in, unification::out,
+    rbmm_renaming::in, rbmm_renaming::in, unification::in, unification::out,
     name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
@@ -599,9 +603,9 @@ annotate_constructions_unification(ModuleInfo, Graph, ResurRenaming,
         IsUnique, SubInfo),
     rptg_get_node_by_variable(Graph, Var, Node),
     NodeType = rptg_lookup_node_type(Graph, Node),
-    ( type_not_stored_in_region(NodeType, ModuleInfo) ->
+    ( if type_not_stored_in_region(NodeType, ModuleInfo) then
         true
-    ;
+    else
         Name = rptg_lookup_region_name(Graph, Node),
         region_name_to_var_with_both_renamings(Name, ResurRenaming,
             IteRenaming, RegVar, !NameToVar, !VarSet, !VarTypes),
@@ -619,7 +623,7 @@ annotate_constructions_unification(_, _, _, _, !Unification, !VarSet,
         )
     ;
         !.Unification = complicated_unify(_, _, _),
-        unexpected($module, $pred, "complicated unify")
+        unexpected($pred, "complicated unify")
     ).
 
     % The process here is related to the way we treat the unifications
@@ -633,9 +637,10 @@ annotate_constructions_unification(_, _, _, _, !Unification, !VarSet,
     % Finally, we try to flatten this new conjunction.
     %
 :- pred region_transform_case(module_info::in, rpt_graph::in,
-    renaming_proc::in, renaming_proc::in, pp_actual_region_args_table::in,
-    region_instr_proc::in, renaming_annotation_proc::in,
-    renaming_annotation_proc::in, hlds_goal::in, case::in, case::out,
+    rbmm_renaming_proc::in, rbmm_renaming_proc::in,
+    pp_actual_region_args_table::in, region_instr_proc::in,
+    rbmm_renaming_annotation_proc::in, rbmm_renaming_annotation_proc::in,
+    hlds_goal::in, case::in, case::out,
     name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
@@ -645,17 +650,16 @@ region_transform_case(ModuleInfo, Graph, ResurRenamingProc,
         case(MainConsId, OtherConsIds, !.Goal),
         case(MainConsId, OtherConsIds, !:Goal),
         !NameToVar, !VarSet, !VarTypes) :-
-    expect(unify(OtherConsIds, []), $module, $pred,
-        "NYI: multi-cons-id cases"),
-    (
+    expect(unify(OtherConsIds, []), $pred, "NYI: multi-cons-id cases"),
+    ( if
         ( MainConsId = cons(_, 0, _)
-        ; MainConsId = int_const(_)
+        ; MainConsId = some_int_const(_)
         ; MainConsId = float_const(_)
         ; MainConsId = char_const(_)
         ; MainConsId = string_const(_)
         ),
         Switch = hlds_goal(switch(_, _, _), Info)
-    ->
+    then
         ProgPoint = program_point_init(Info),
         ProgPoint = pp(Context, _),
         find_renamings_at_prog_point(ResurRenamingProc, IteRenamingProc,
@@ -666,10 +670,10 @@ region_transform_case(ModuleInfo, Graph, ResurRenamingProc,
             !NameToVar, !VarSet, !VarTypes, [], IteRenamingAssignments),
 
         % Region instructions before and after this program point.
-        (
+        ( if
             map.search(RegionInstructionProc, ProgPoint,
                 instrs_before_after(Before, After))
-        ->
+        then
             % Region instructions before this program point.
             list.foldl4(
                 region_instruction_to_conj_before(ModuleInfo, Context,
@@ -682,7 +686,7 @@ region_transform_case(ModuleInfo, Graph, ResurRenamingProc,
                 region_instruction_to_conj(ModuleInfo, Context, ResurRenaming,
                     IteRenaming),
                 After, !NameToVar, !VarSet, !VarTypes, Conjs1, Conjs2)
-        ;
+        else
             Conjs2 = IteRenamingAssignments
         ),
 
@@ -691,7 +695,7 @@ region_transform_case(ModuleInfo, Graph, ResurRenamingProc,
             IteRenaming, !NameToVar, !VarSet, !VarTypes, Conjs2, Conjs),
 
         RemovedGoal = hlds_goal(conj(plain_conj, Conjs), Info)
-    ;
+    else
         Switch = hlds_goal(_, Info),
         RemovedGoal = hlds_goal(conj(plain_conj, []), Info)
     ),
@@ -703,23 +707,24 @@ region_transform_case(ModuleInfo, Graph, ResurRenamingProc,
     Switch = hlds_goal(_, ConjsInfo),
     !:Goal = hlds_goal(conj(plain_conj, FlatConjs), ConjsInfo).
 
-:- pred find_renamings_at_prog_point(renaming_proc::in, renaming_proc::in,
-    program_point::in, renaming::out, renaming::out) is det.
+:- pred find_renamings_at_prog_point(rbmm_renaming_proc::in,
+    rbmm_renaming_proc::in, program_point::in,
+    rbmm_renaming::out, rbmm_renaming::out) is det.
 
 find_renamings_at_prog_point(ResurRenamingProc, IteRenamingProc, ProgPoint,
         ResurRenaming, IteRenaming) :-
-    ( map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) ->
+    ( if map.search(ResurRenamingProc, ProgPoint, ResurRenaming0) then
         ResurRenaming = ResurRenaming0
-    ;
+    else
         ResurRenaming = map.init
     ),
-    ( map.search(IteRenamingProc, ProgPoint, IteRenaming0) ->
+    ( if map.search(IteRenamingProc, ProgPoint, IteRenaming0) then
         IteRenaming = IteRenaming0
-    ;
+    else
         IteRenaming = map.init
     ).
 
-:- pred assignments_from_ite_renaming_anno(renaming_annotation_proc::in,
+:- pred assignments_from_ite_renaming_anno(rbmm_renaming_annotation_proc::in,
     program_point::in, name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out,
     hlds_goals::in, hlds_goals::out) is det.
@@ -727,24 +732,24 @@ find_renamings_at_prog_point(ResurRenamingProc, IteRenamingProc, ProgPoint,
 assignments_from_ite_renaming_anno(IteRenamingAnnoProc, ProgPoint,
         !NameToVar, !VarSet, !VarTypes, !IteRenamingAssignments) :-
     % Assignment unifications due to ite renaming.
-    ( map.search(IteRenamingAnnoProc, ProgPoint, IteRenamingAnnos) ->
+    ( if map.search(IteRenamingAnnoProc, ProgPoint, IteRenamingAnnos) then
         list.foldl4(ite_renaming_annotation_to_assignment, IteRenamingAnnos,
             !NameToVar, !VarSet, !VarTypes, !IteRenamingAssignments)
-    ;
+    else
         true
     ).
 
-:- pred assignments_from_resur_renaming_anno(renaming_annotation_proc::in,
-    program_point::in, renaming::in, name_to_prog_var::in,
+:- pred assignments_from_resur_renaming_anno(rbmm_renaming_annotation_proc::in,
+    program_point::in, rbmm_renaming::in, name_to_prog_var::in,
     name_to_prog_var::out, prog_varset::in, prog_varset::out,
     vartypes::in, vartypes::out, hlds_goals::in, hlds_goals::out) is det.
 
 assignments_from_resur_renaming_anno(ResurRenamingAnnoProc, ProgPoint,
         IteRenaming, !NameToVar, !VarSet, !VarTypes, !Conjs) :-
-    ( map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos) ->
+    ( if map.search(ResurRenamingAnnoProc, ProgPoint, ResurRenamingAnnos) then
         list.foldl4(resur_renaming_annotation_to_assignment(IteRenaming),
             ResurRenamingAnnos, !NameToVar, !VarSet, !VarTypes, !Conjs)
-    ;
+    else
         true
     ).
 
@@ -770,9 +775,9 @@ node_to_var(Graph, Node, RegVar, !NameToVar, !VarSet, !VarTypes) :-
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_name_to_var(Name, RegVar, !NameToVar, !VarSet, !VarTypes) :-
-    ( map.search(!.NameToVar, Name, RegVar0) ->
+    ( if map.search(!.NameToVar, Name, RegVar0) then
         RegVar = RegVar0
-    ;
+    else
         varset.new_named_var(Name, RegVar, !VarSet),
         add_var_type(RegVar, region_type, !VarTypes),
         map.det_insert(Name, RegVar, !NameToVar)
@@ -781,8 +786,8 @@ region_name_to_var(Name, RegVar, !NameToVar, !VarSet, !VarTypes) :-
     % The same as node_to_var, but the corresponding region name is
     % subjected to resurrection and if-then-else renaming beforehand.
     %
-:- pred node_to_var_with_both_renamings(rpt_graph::in, renaming::in,
-    renaming::in, rptg_node::in, prog_var::out,
+:- pred node_to_var_with_both_renamings(rpt_graph::in, rbmm_renaming::in,
+    rbmm_renaming::in, rptg_node::in, prog_var::out,
     name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
@@ -798,17 +803,18 @@ node_to_var_with_both_renamings(Graph, ResurRenaming, IteRenaming,
     % applicable anymore. If more than one renaming exist, then we use
     % the last one.
     %
-:- pred region_name_to_var_with_both_renamings(string::in, renaming::in,
-    renaming::in, prog_var::out, name_to_prog_var::in, name_to_prog_var::out,
+:- pred region_name_to_var_with_both_renamings(string::in,
+    rbmm_renaming::in, rbmm_renaming::in, prog_var::out,
+    name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_name_to_var_with_both_renamings(Name0, ResurRenaming, IteRenaming,
         RegVar, !NameToVar, !VarSet, !VarTypes) :-
-    ( map.search(ResurRenaming, Name0, ResurNameList) ->
-        list.det_last(ResurNameList, Name) 
-    ; map.search(IteRenaming, Name0, IteNameList) ->
-        list.det_last(IteNameList, Name) 
-    ;
+    ( if map.search(ResurRenaming, Name0, ResurNameList) then
+        list.det_last(ResurNameList, Name)
+    else if map.search(IteRenaming, Name0, IteNameList) then
+        list.det_last(IteNameList, Name)
+    else
         Name = Name0
     ),
     region_name_to_var(Name, RegVar, !NameToVar, !VarSet, !VarTypes).
@@ -818,17 +824,18 @@ region_name_to_var_with_both_renamings(Name0, ResurRenaming, IteRenaming,
     % renaming the region in a remove instruction added before a program
     % point.
     %
-:- pred region_name_to_var_with_both_renamings_before(string::in, renaming::in,
-    renaming::in, prog_var::out, name_to_prog_var::in, name_to_prog_var::out,
+:- pred region_name_to_var_with_both_renamings_before(string::in,
+    rbmm_renaming::in, rbmm_renaming::in, prog_var::out,
+    name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_name_to_var_with_both_renamings_before(Name0, ResurRenaming,
         IteRenaming, RegVar, !NameToVar, !VarSet, !VarTypes) :-
-    ( map.search(ResurRenaming, Name0, ResurNameList) ->
+    ( if map.search(ResurRenaming, Name0, ResurNameList) then
         Name = list.det_index0(ResurNameList, 0)
-    ; map.search(IteRenaming, Name0, IteNameList) ->
+    else if map.search(IteRenaming, Name0, IteNameList) then
         Name = list.det_index0(IteNameList, 0)
-    ;
+    else
         Name = Name0
     ),
     region_name_to_var(Name, RegVar, !NameToVar, !VarSet, !VarTypes).
@@ -836,15 +843,15 @@ region_name_to_var_with_both_renamings_before(Name0, ResurRenaming,
     % The same as region_name_to_var, but the region name here is
     % subjected to resurrection renaming in advance.
     %
-:- pred region_name_to_var_with_renaming(string::in, renaming::in,
+:- pred region_name_to_var_with_renaming(string::in, rbmm_renaming::in,
     prog_var::out, name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out) is det.
 
 region_name_to_var_with_renaming(Name0, ResurRenaming, RegVar,
         !NameToVar, !VarSet, !VarTypes) :-
-    ( map.search(ResurRenaming, Name0, ResurNameList) ->
+    ( if map.search(ResurRenaming, Name0, ResurNameList) then
         Name = list.det_last(ResurNameList)
-    ;
+    else
         Name = Name0
     ),
     region_name_to_var(Name, RegVar, !NameToVar, !VarSet, !VarTypes).
@@ -856,7 +863,7 @@ region_name_to_var_with_renaming(Name0, ResurRenaming, RegVar,
     % will recompute nonlocals, instmap delta anyway.
     %
 :- pred region_instruction_to_conj(module_info::in, term.context::in,
-    renaming::in, renaming::in, region_instr::in,
+    rbmm_renaming::in, rbmm_renaming::in, region_instr::in,
     name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out,
     hlds_goals::in, hlds_goals::out) is det.
@@ -867,21 +874,21 @@ region_instruction_to_conj(ModuleInfo, Context, ResurRenaming, IteRenaming,
         RegionInstruction = create_region(RegionName),
         region_name_to_var_with_both_renamings(RegionName, ResurRenaming,
             IteRenaming, RegionVar, !NameToVar, !VarSet, !VarTypes),
-        generate_simple_call(mercury_region_builtin_module,
+        generate_simple_call(ModuleInfo, mercury_region_builtin_module,
             create_region_pred_name, pf_predicate, only_mode, detism_det,
             purity_impure, [RegionVar], [], instmap_delta_bind_no_var,
-            ModuleInfo, Context, CallGoal)
+            Context, CallGoal)
     ;
         RegionInstruction = remove_region(RegionName),
         region_name_to_var_with_both_renamings(RegionName, ResurRenaming,
             IteRenaming, RegionVar, !NameToVar, !VarSet, !VarTypes),
-        generate_simple_call(mercury_region_builtin_module,
+        generate_simple_call(ModuleInfo, mercury_region_builtin_module,
             remove_region_pred_name, pf_predicate, only_mode, detism_det,
             purity_impure, [RegionVar], [], instmap_delta_bind_no_var,
-            ModuleInfo, Context, CallGoal)
+            Context, CallGoal)
     ;
         RegionInstruction = rename_region(_, _),
-        unexpected($module, $pred, "neither create nor remove instruction")
+        unexpected($pred, "neither create nor remove instruction")
     ),
     Conjs = Conjs0 ++ [CallGoal].
 
@@ -889,7 +896,7 @@ region_instruction_to_conj(ModuleInfo, Context, ResurRenaming, IteRenaming,
     % instruction we apply the first resurrection renaming.
     %
 :- pred region_instruction_to_conj_before(module_info::in, term.context::in,
-    renaming::in, renaming::in, region_instr::in,
+    rbmm_renaming::in, rbmm_renaming::in, region_instr::in,
     name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out,
     hlds_goals::in, hlds_goals::out) is det.
@@ -901,22 +908,22 @@ region_instruction_to_conj_before(ModuleInfo, Context, ResurRenaming,
         RegionInstruction = create_region(RegionName),
         region_name_to_var_with_both_renamings(RegionName, ResurRenaming,
             IteRenaming, RegionVar, !NameToVar, !VarSet, !VarTypes),
-        generate_simple_call(mercury_region_builtin_module,
+        generate_simple_call(ModuleInfo, mercury_region_builtin_module,
             create_region_pred_name, pf_predicate, only_mode, detism_det,
             purity_impure, [RegionVar], [], instmap_delta_bind_no_var,
-            ModuleInfo, Context, CallGoal)
+            Context, CallGoal)
     ;
         RegionInstruction = remove_region(RegionName),
         region_name_to_var_with_both_renamings_before(RegionName,
             ResurRenaming, IteRenaming, RegionVar, !NameToVar, !VarSet,
             !VarTypes),
-        generate_simple_call(mercury_region_builtin_module,
+        generate_simple_call(ModuleInfo, mercury_region_builtin_module,
             remove_region_pred_name, pf_predicate, only_mode, detism_det,
             purity_impure, [RegionVar], [], instmap_delta_bind_no_var,
-            ModuleInfo, Context, CallGoal)
+            Context, CallGoal)
     ;
         RegionInstruction = rename_region(_, _),
-        unexpected($module, $pred, "neither create nor remove instruction")
+        unexpected($pred, "neither create nor remove instruction")
     ),
     Conjs = Conjs0 ++ [CallGoal].
 
@@ -927,7 +934,7 @@ region_instruction_to_conj_before(ModuleInfo, Context, ResurRenaming,
     % The original name of the region is subjected to the renaming due to
     % if-then-else, if such a renaming exists at the current program point.
     %
-:- pred resur_renaming_annotation_to_assignment(renaming::in,
+:- pred resur_renaming_annotation_to_assignment(rbmm_renaming::in,
     region_instr::in, name_to_prog_var::in, name_to_prog_var::out,
     prog_varset::in, prog_varset::out, vartypes::in, vartypes::out,
     hlds_goals::in, hlds_goals::out) is det.
@@ -938,7 +945,7 @@ resur_renaming_annotation_to_assignment(IteRenaming, Annotation,
         ( Annotation = create_region(_)
         ; Annotation = remove_region(_)
         ),
-        unexpected($module, $pred, "annotation is not assigment")
+        unexpected($pred, "annotation is not assigment")
     ;
         Annotation = rename_region(Right, Left),
         % Only the left region needs to be renamed. Ite renaming does not
@@ -966,7 +973,7 @@ ite_renaming_annotation_to_assignment(Annotation, !NameToVar,
         ( Annotation = create_region(_)
         ; Annotation = remove_region(_)
         ),
-        unexpected($module, $pred, "annotation is not assignment")
+        unexpected($pred, "annotation is not assignment")
     ;
         Annotation = rename_region(Right, Left),
         region_name_to_var(Left, LeftRegVar, !NameToVar, !VarSet, !VarTypes),
@@ -981,13 +988,9 @@ ite_renaming_annotation_to_assignment(Annotation, !NameToVar,
 
 make_assignment_goal(LeftRegVar, RightRegVar, Context, AssignmentGoal) :-
     AssignmentExpr = unify(LeftRegVar, rhs_var(RightRegVar),
-        out_mode - in_mode,
+        unify_modes_li_lf_ri_rf(free, ground_inst, ground_inst, ground_inst),
         assign(LeftRegVar, RightRegVar),
-        unify_context(
-            umc_implicit(Context),
-            []
-        )
-    ),
+        unify_context(umc_implicit(Context), [])),
     % Nonlocals and instmap delta will be recomputed anyway, so just put
     % dummy values in.
     % XXX  We need to setup the instantiationess for LeftRegVar here.
@@ -1010,7 +1013,7 @@ make_assignment_goal(LeftRegVar, RightRegVar, Context, AssignmentGoal) :-
 
 update_instmap_delta_pred(PredId, !ModuleInfo) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl(update_instmap_delta_proc(PredId), ProcIds, !ModuleInfo).
 
 :- pred update_instmap_delta_proc(pred_id::in, proc_id::in,
@@ -1033,7 +1036,7 @@ update_instmap_delta_proc(PredId, ProcId, !ModuleInfo) :-
 
 recheck_purity_pred(PredId, !ModuleInfo) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl(recheck_purity_proc(PredId), ProcIds, !ModuleInfo).
 
     % Recheck purity of the procedure.
