@@ -27,6 +27,7 @@
 :- module ll_backend.saved_vars.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -40,8 +41,8 @@
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
-:- import_module check_hlds.polymorphism.
+:- import_module check_hlds.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_goal.
@@ -49,6 +50,10 @@
 :- import_module hlds.hlds_rtti.
 :- import_module hlds.passes_aux.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
+:- import_module parse_tree.
+:- import_module parse_tree.builtin_lib_types.
+:- import_module parse_tree.parse_tree_out_info.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
 
@@ -66,8 +71,8 @@
 
 saved_vars_proc(proc(PredId, ProcId), !ProcInfo, !ModuleInfo) :-
     trace [io(!IO)] (
-        write_proc_progress_message("% Minimizing saved vars in ",
-            PredId, ProcId, !.ModuleInfo, !IO)
+        write_proc_progress_message(!.ModuleInfo,
+            "Minimizing saved vars in", PredId, ProcId, !IO)
     ),
 
     module_info_get_globals(!.ModuleInfo, Globals),
@@ -90,22 +95,23 @@ saved_vars_proc(proc(PredId, ProcId), !ProcInfo, !ModuleInfo) :-
     implicitly_quantify_clause_body_general(ordinary_nonlocals_no_lambda,
         HeadVars, _Warnings, Goal1, Goal2,
         Varset1, Varset, VarTypes1, VarTypes, RttiVarMaps1, RttiVarMaps),
-    proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InstMap0),
+    proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InstMap0),
     proc_info_get_inst_varset(!.ProcInfo, InstVarSet),
     recompute_instmap_delta(do_not_recompute_atomic_instmap_deltas,
         Goal2, Goal, VarTypes, InstVarSet, InstMap0, !ModuleInfo),
 
     trace [io(!IO), compile_time(flag("debug_saved_vars"))] (
-        OutInfo = hlds_out_util.init_hlds_out_info(Globals),
-        io.write_string("initial version:\n", !IO),
-        hlds_out_goal.write_goal(OutInfo, Goal0, !.ModuleInfo, Varset0,
-            yes, 0, "\n", !IO),
-        io.write_string("after transformation:\n", !IO),
-        hlds_out_goal.write_goal(OutInfo, Goal1, !.ModuleInfo, Varset1,
-            yes, 0, "\n", !IO),
-        io.write_string("final version:\n", !IO),
-        hlds_out_goal.write_goal(OutInfo, Goal, !.ModuleInfo, Varset,
-            yes, 0, "\n", !IO)
+        OutInfo = hlds_out_util.init_hlds_out_info(Globals, output_debug),
+        io.output_stream(Stream, !IO),
+        io.write_string(Stream, "initial version:\n", !IO),
+        hlds_out_goal.write_goal(OutInfo, Stream, !.ModuleInfo, Varset0,
+            print_name_and_num, 0, "\n", Goal0, !IO),
+        io.write_string(Stream, "after transformation:\n", !IO),
+        hlds_out_goal.write_goal(OutInfo, Stream, !.ModuleInfo, Varset1,
+            print_name_and_num, 0, "\n", Goal1, !IO),
+        io.write_string(Stream, "final version:\n", !IO),
+        hlds_out_goal.write_goal(OutInfo, Stream, !.ModuleInfo, Varset,
+            print_name_and_num, 0, "\n", Goal, !IO)
     ),
 
     proc_info_set_goal(Goal, !ProcInfo),
@@ -156,12 +162,12 @@ saved_vars_in_goal(Goal0, Goal, !SlotInfo) :-
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        ( Reason = from_ground_term(_, from_ground_term_construct) ->
+        ( if Reason = from_ground_term(_, from_ground_term_construct) then
             % Moving unifications around inside these scopes is
             % (a) counterproductive, and (b) incorrect, since it would
             % invalidate the invariants required of such scopes.
             SubGoal = SubGoal0
-        ;
+        else
             saved_vars_in_goal(SubGoal0, SubGoal, !SlotInfo)
         ),
         GoalExpr = scope(Reason, SubGoal),
@@ -176,7 +182,7 @@ saved_vars_in_goal(Goal0, Goal, !SlotInfo) :-
     ;
         GoalExpr0 = shorthand(_),
         % these should have been expanded out by now
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -196,7 +202,7 @@ saved_vars_in_goal(Goal0, Goal, !SlotInfo) :-
 
 saved_vars_in_conj([], [], _, !SlotInfo).
 saved_vars_in_conj([Goal0 | Goals0], Goals, NonLocals, !SlotInfo) :-
-    (
+    ( if
         Goal0 = hlds_goal(unify(_, _, _, Unif, _), GoalInfo),
         Unif = construct(Var, _, [], _, _, _, _),
         Features = goal_info_get_features(GoalInfo),
@@ -207,17 +213,17 @@ saved_vars_in_conj([Goal0 | Goals0], Goals, NonLocals, !SlotInfo) :-
                 ok_to_duplicate(Feature) = yes
             )
         ),
-        \+ slot_info_do_not_duplicate_var(!.SlotInfo, Var),
+        not slot_info_do_not_duplicate_var(!.SlotInfo, Var),
         skip_constant_constructs(Goals0, Constants, OtherGoals),
         OtherGoals = [First | _Rest],
         can_push(Var, First) = yes
-    ->
+    then
         set_of_var.is_member(NonLocals, Var, IsNonLocal),
         saved_vars_delay_goal(OtherGoals, Goals1, Goal0, Var, IsNonLocal,
             !SlotInfo),
         list.append(Constants, Goals1, Goals2),
         saved_vars_in_conj(Goals2, Goals, NonLocals, !SlotInfo)
-    ;
+    else
         saved_vars_in_goal(Goal0, Goal1, !SlotInfo),
         saved_vars_in_conj(Goals0, Goals1, NonLocals, !SlotInfo),
         Goals = [Goal1 | Goals1]
@@ -237,11 +243,14 @@ ok_to_duplicate(feature_tuple_opt) = no.
 ok_to_duplicate(feature_call_table_gen) = no.
 ok_to_duplicate(feature_preserve_backtrack_into) = no.
 ok_to_duplicate(feature_hide_debug_event) = no.
-ok_to_duplicate(feature_deep_tail_rec_call) = no.
-ok_to_duplicate(feature_debug_tail_rec_call) = no.
+ok_to_duplicate(feature_deep_self_tail_rec_call) = no.
+ok_to_duplicate(feature_debug_self_tail_rec_call) = no.
+ok_to_duplicate(feature_self_or_mutual_tail_rec_call) = no.
+ok_to_duplicate(feature_obvious_nontail_rec_call) = no.
 ok_to_duplicate(feature_keep_constant_binding) = no.
 ok_to_duplicate(feature_save_deep_excp_vars) = no.
 ok_to_duplicate(feature_dont_warn_singleton) = yes.
+ok_to_duplicate(feature_state_var_copy) = yes.
 ok_to_duplicate(feature_duplicated_for_switch) = yes.
 ok_to_duplicate(feature_mode_check_clauses_goal) = yes.
 ok_to_duplicate(feature_will_not_modify_trail) = yes.
@@ -252,6 +261,9 @@ ok_to_duplicate(feature_pretest_equality_condition) = yes.
 ok_to_duplicate(feature_lambda_undetermined_mode) = yes.
 ok_to_duplicate(feature_contains_stm_inner_outer) = yes.
 ok_to_duplicate(feature_do_not_tailcall) = no.
+ok_to_duplicate(feature_do_not_warn_implicit_stream) = no.
+ok_to_duplicate(feature_lifted_by_cse) = no.
+ok_to_duplicate(feature_lambda_from_try) = yes.
 
     % Divide a list of goals into an initial subsequence of goals
     % that construct constants, and all other goals.
@@ -261,13 +273,13 @@ ok_to_duplicate(feature_do_not_tailcall) = no.
 
 skip_constant_constructs([], [], []).
 skip_constant_constructs([Goal0 | Goals0], Constants, Others) :-
-    (
+    ( if
         Goal0 = hlds_goal(unify(_, _, _, Unif, _), _),
         Unif = construct(_, _, [], _, _, _, _)
-    ->
+    then
         skip_constant_constructs(Goals0, Constants1, Others),
         Constants = [Goal0 | Constants1]
-    ;
+    else
         Constants = [],
         Others = [Goal0 | Goals0]
     ).
@@ -286,7 +298,7 @@ skip_constant_constructs([Goal0 | Goals0], Constants, Others) :-
 can_push(Var, Goal) = CanPush :-
     Goal = hlds_goal(GoalExpr, GoalInfo),
     NonLocals = goal_info_get_nonlocals(GoalInfo),
-    ( set_of_var.member(NonLocals, Var) ->
+    ( if set_of_var.member(NonLocals, Var) then
         (
             ( GoalExpr = if_then_else(_, _, _, _)
             ; GoalExpr = negation(_)
@@ -305,6 +317,17 @@ can_push(Var, Goal) = CanPush :-
         ;
             GoalExpr = scope(Reason, _),
             (
+                Reason = disable_warnings(_, _),
+                % NOTE: This assumes that compiler passes that generate the
+                % warnings that could be disabled by this scope have all
+                % been run BEFORE program transformations such as saved vars.
+                % If they haven't been, then the transformations can hide
+                % warnings about code by moving them into these scopes,
+                % or can caused them to be generated when the user does
+                % not want them by moving the warned-about code out of
+                % such scopes.
+                CanPush = yes
+            ;
                 ( Reason = exist_quant(_)
                 ; Reason = from_ground_term(_, from_ground_term_deconstruct)
                 ; Reason = from_ground_term(_, from_ground_term_other)
@@ -323,24 +346,25 @@ can_push(Var, Goal) = CanPush :-
             ;
                 ( Reason = require_detism(_)
                 ; Reason = require_complete_switch(_)
+                ; Reason = require_switch_arms_detism(_, _)
                 ; Reason = from_ground_term(_, from_ground_term_initial)
                 ),
                 % These scopes should have been deleted by now.
-                unexpected($module, $pred, "unexpected scope")
+                unexpected($pred, "unexpected scope")
             )
         ;
             GoalExpr = switch(SwitchVar, _, _),
-            ( Var = SwitchVar ->
+            ( if Var = SwitchVar then
                 CanPush = no
-            ;
+            else
                 CanPush = yes
             )
         ;
             GoalExpr = shorthand(_),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "shorthand")
+            unexpected($pred, "shorthand")
         )
-    ;
+    else
         CanPush = yes
     ).
 
@@ -374,7 +398,7 @@ saved_vars_delay_goal([Goal0 | Goals0], Goals, Construct, Var, IsNonLocal,
         !SlotInfo) :-
     Goal0 = hlds_goal(Goal0Expr, Goal0Info),
     Goal0NonLocals = goal_info_get_nonlocals(Goal0Info),
-    ( set_of_var.member(Goal0NonLocals, Var) ->
+    ( if set_of_var.member(Goal0NonLocals, Var) then
         (
             Goal0Expr = unify(_, _, _, _, _),
             saved_vars_rename_var(Var, _NewVar, Subst, !SlotInfo),
@@ -455,11 +479,11 @@ saved_vars_delay_goal([Goal0 | Goals0], Goals, Construct, Var, IsNonLocal,
             Goals = [Goal1 | Goals1]
         ;
             Goal0Expr = switch(SwitchVar, CF, Cases0),
-            ( SwitchVar = Var ->
+            ( if SwitchVar = Var then
                 saved_vars_delay_goal(Goals0, Goals1, Construct, Var,
                     IsNonLocal, !SlotInfo),
                 Goals = [Construct, Goal0 | Goals1]
-            ;
+            else
                 push_into_cases_rename(Cases0, Cases, Construct, Var,
                     !SlotInfo),
                 Goal1 = hlds_goal(switch(SwitchVar, CF, Cases), Goal0Info),
@@ -479,9 +503,9 @@ saved_vars_delay_goal([Goal0 | Goals0], Goals, Construct, Var, IsNonLocal,
         ;
             Goal0Expr = shorthand(_),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "shorthand")
+            unexpected($pred, "shorthand")
         )
-    ;
+    else
         saved_vars_delay_goal(Goals0, Goals1, Construct, Var, IsNonLocal,
             !SlotInfo),
         Goals = [Goal0 | Goals1]
@@ -511,12 +535,12 @@ push_into_goal(Goal0, Goal, Construct, Var, !SlotInfo) :-
 push_into_goal_rename(Goal0, Goal, Construct, Var, !SlotInfo) :-
     Goal0 = hlds_goal(_, GoalInfo0),
     NonLocals = goal_info_get_nonlocals(GoalInfo0),
-    ( set_of_var.member(NonLocals, Var) ->
+    ( if set_of_var.member(NonLocals, Var) then
         saved_vars_rename_var(Var, NewVar, Subst, !SlotInfo),
         rename_some_vars_in_goal(Subst, Construct, NewConstruct),
         rename_some_vars_in_goal(Subst, Goal0, Goal1),
         push_into_goal(Goal1, Goal, NewConstruct, NewVar, !SlotInfo)
-    ;
+    else
         saved_vars_in_goal(Goal0, Goal, !SlotInfo)
     ).
 
@@ -574,12 +598,12 @@ saved_vars_in_switch([Case0 | Cases0], [Case | Cases], !SlotInfo) :-
 %-----------------------------------------------------------------------------%
 
 :- type slot_info
-    ---> slot_info(
-            prog_varset,
-            vartypes,
-            rtti_varmaps,
-            bool            % TypeInfoLiveness
-        ).
+    --->    slot_info(
+                prog_varset,
+                vartypes,
+                rtti_varmaps,
+                bool            % TypeInfoLiveness
+            ).
 
 :- pred init_slot_info(prog_varset::in, vartypes::in,
     rtti_varmaps::in, bool::in, slot_info::out) is det.
@@ -607,14 +631,13 @@ saved_vars_rename_var(Var, NewVar, Substitution, !SlotInfo) :-
         TypeInfoLiveness).
 
     % Check whether it is ok to duplicate a given variable according
-    % to the information in the slot_info.  If TypeInfoLiveness is set,
-    % it is possible that liveness.m will want to refer to the
-    % rtti_varmaps to calculate which type_infos are live (see the
-    % comments at the top of liveness.m).  If we duplicated any
-    % type_info variables here then this could cause problems because
-    % the rtti_varmaps would not be able to be kept consistent.
-    % Therefore we don't allow type_infos to be duplicated when
-    % TypeInfoLiveness is set.
+    % to the information in the slot_info. If TypeInfoLiveness is set,
+    % it is possible that liveness.m will want to refer to the rtti_varmaps
+    % to calculate which type_infos are live (see the comments at the top
+    % of liveness.m). If we duplicated any type_info variables here,
+    % then this could cause problems because the rtti_varmaps would not
+    % be able to be kept consistent. Therefore we don't allow type_infos
+    % to be duplicated when TypeInfoLiveness is set.
     %
 :- pred slot_info_do_not_duplicate_var(slot_info::in, prog_var::in) is semidet.
 
@@ -622,7 +645,7 @@ slot_info_do_not_duplicate_var(SlotInfo, Var) :-
     SlotInfo = slot_info(_, VarTypes, _, TypeInfoLiveness),
     TypeInfoLiveness = yes,
     lookup_var_type(VarTypes, Var, Type),
-    polymorphism.type_is_type_info_or_ctor_type(Type).
+    type_is_type_info_or_ctor_type(Type).
 
 %-----------------------------------------------------------------------------%
 :- end_module ll_backend.saved_vars.
