@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2006-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: structure_reuse.direct.choose_reuse.m.
 % Main authors: nancy.
@@ -80,7 +80,7 @@
 % the cell could potentially be reused.
 %
 % Note that cells being deconstructed in the different branches of a
-% disjunction can now also be reused after the the disjunction.
+% disjunction can now also be reused after the disjunction.
 %
 % e.g.:
 %   (
@@ -93,29 +93,34 @@
 % In this example, it is allowed to reuse X for Y. And it will also be
 % discovered by the analysis.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module transform_hlds.ctgc.structure_reuse.direct.choose_reuse.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_module.
 :- import_module transform_hlds.ctgc.structure_reuse.domain.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred determine_reuse(module_info::in, proc_info::in, dead_cell_table::in,
     hlds_goal::in, hlds_goal::out, reuse_as::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
 :- import_module hlds.hlds_data.
+:- import_module hlds.vartypes.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_pragma.
 
 :- import_module float.
 :- import_module int.
@@ -126,7 +131,7 @@
 :- import_module string.
 :- import_module term.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 determine_reuse(ModuleInfo, ProcInfo, DeadCellTable, !Goal, ReuseAs) :-
     % Check for local reuse:
@@ -145,7 +150,7 @@ determine_reuse(ModuleInfo, ProcInfo, DeadCellTable, !Goal, ReuseAs) :-
         FreeCells = no
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % A type to collect all the background information needed to process
     % each individual goal.
@@ -165,7 +170,7 @@ background_info_init(Strategy, ModuleInfo, ProcInfo) = Background :-
     proc_info_get_vartypes(ProcInfo, VarTypes),
     Background = background(Strategy, ModuleInfo, ProcInfo, VarTypes).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Some types and predicates for the administration of the deconstructions,
 % constructions and the 'matches' we want to derive from them.
 %
@@ -181,7 +186,7 @@ background_info_init(Strategy, ModuleInfo, ProcInfo) = Background :-
 %
 % (1) extend deconstruction_spec with a field of type may_use_atomic_alloc,
 %     indicating whether the potentially reused cell may be atomic or not, and
-% (2) ensure that we reuse atomically-created cells only for connstructions
+% (2) ensure that we reuse atomically-created cells only for constructions
 %     in which all arguments can be put into atomic cells.
 %
 % These will require applying type_may_use_atomic_alloc to the arguments of
@@ -277,6 +282,7 @@ match_init(DS) = match(DS, [], 0.0, 0).
     % match.
     %
 :- pred match_has_no_construction_candidates(match::in) is semidet.
+:- pragma consider_used(pred(match_has_no_construction_candidates/1)).
 
 match_has_no_construction_candidates(match(_, [], _, _)).
 
@@ -295,11 +301,11 @@ match_get_dead_var(Match) = Var :-
             Rest = []
         ;
             Rest = [_ | _],
-            unexpected($module, $pred, "too many dead vars")
+            unexpected($pred, "too many dead vars")
         )
     ;
         DeadVars = [],
-        unexpected($module, $pred, "empty list of vars")
+        unexpected($pred, "empty list of vars")
     ).
 
     % Get the list of cons_ids that the dead variable may have when it
@@ -326,7 +332,7 @@ match_get_condition(Background, Match) = Condition :-
             Rest, First, Condition)
     ;
         Conditions = [],
-        unexpected($module, $pred, "no reuse conditions")
+        unexpected($pred, "no reuse conditions")
     ).
 
     % Add a construction as a potential place for reusing the garbage
@@ -345,7 +351,7 @@ match_add_construction(ConSpec, Match0, Match) :-
     Value = (Value0 * FDegree0 + ConSpec ^ con_reuse ^ reuse_value) / FDegree,
     Match = match(DeconSpecs0, ConSpecs, Value, Degree).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Manipulating the values of matches...
 %
@@ -359,7 +365,7 @@ highest_match_degree_ratio(MatchTable) = Match :-
         Sorted = [Match | _]
     ;
         Sorted = [],
-        unexpected($module, $pred, "empty multi_map")
+        unexpected($pred, "empty multi_map")
     ).
 
 :- pred compare_matches_value_degree(match::in, match::in,
@@ -379,9 +385,9 @@ reverse_compare_matches_value_degree(MatchA, MatchB, Result) :-
 :- func match_value_degree(match) = float.
 
 match_value_degree(Match) =
-    ( Match ^ match_value \= 0.0 ->
+    ( if Match ^ match_value \= 0.0 then
         Match ^ match_value / float(Match ^ match_degree)
-    ;
+    else
         0.0
     ).
 
@@ -415,7 +421,7 @@ highest_match_in_list(Matches, Match0, Match) :-
         Sorted = [Match | _]
     ;
         Sorted = [],
-        unexpected($module, $pred, "empty list of matches")
+        unexpected($pred, "empty list of matches")
     ).
 
     % Given a list of matches concerning the same (list of) deconstruction,
@@ -442,10 +448,10 @@ average_match(List, AverageMatch) :-
             (Match0 ^ match_value / float(Length)))
     ;
         List = [],
-        unexpected($module, $pred, "empty list")
+        unexpected($pred, "empty list")
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Process a single goal:
 %
@@ -470,9 +476,9 @@ choose_reuse_in_goal(Background, !DeadCellTable, !Goal, !ReuseAs) :-
     % highest value, annotate the goal accordingly, and repeat the procedure.
     % If the match table is empty, the work is finished.
 
-    ( multi_map.is_empty(MatchTable) ->
+    ( if multi_map.is_empty(MatchTable) then
         true
-    ;
+    else
         % Select the deconstructions-constructions with highest value.
 
         Match = highest_match_degree_ratio(MatchTable),
@@ -484,8 +490,14 @@ choose_reuse_in_goal(Background, !DeadCellTable, !Goal, !ReuseAs) :-
             % Maybe dump all the matches recorded in the table, highlight the
             % match with the highest value.
             trace [io(!IO)] (
-                maybe_write_string(VeryVerbose, "% Reuse results: \n", !IO),
-                maybe_dump_match_table(VeryVerbose, MatchTable, Match, !IO)
+                (
+                    VeryVerbose = no
+                ;
+                    VeryVerbose = yes,
+                    io.stderr_stream(StdErr, !IO),
+                    io.write_string(StdErr, "% Reuse results: \n", !IO),
+                    dump_match_table(StdErr, MatchTable, Match, !IO)
+                )
             ),
 
             OldGoal = !.Goal,
@@ -507,15 +519,18 @@ choose_reuse_in_goal(Background, !DeadCellTable, !Goal, !ReuseAs) :-
             % by taking the reuse opportunity, just drop it.
             globals.lookup_int_option(Globals, structure_reuse_max_conditions,
                 MaxConditions),
-            ( reuse_as_count_conditions(!.ReuseAs) > MaxConditions ->
+            ( if reuse_as_count_conditions(!.ReuseAs) > MaxConditions then
                 !:Goal = OldGoal,
                 !:ReuseAs = OldReuseAs
-            ;
+            else
                 true
             ),
 
             % Process the goal for further reuse-matches.
-            choose_reuse_in_goal(Background, !DeadCellTable, !Goal, !ReuseAs)
+            disable_warning [suspicious_recursion] (
+                choose_reuse_in_goal(Background, !DeadCellTable,
+                    !Goal, !ReuseAs)
+            )
         )
     ).
 
@@ -533,7 +548,7 @@ remove_deconstructions_from_dead_cell_table(Match, !DeadCellTable) :-
 remove_deconstruction_from_dead_cell_table(DeconSpec, !DeadCellTable) :-
     dead_cell_table_remove(DeconSpec ^ decon_pp, !DeadCellTable).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Compute the match table for a given goal
 %
@@ -571,12 +586,12 @@ compute_match_table_with_continuation(Background, DeadCellTable,
     CurrentGoal = hlds_goal(GoalExpr, GoalInfo),
     (
         GoalExpr = unify(_, _, _, Unification, _),
-        ( Unification = deconstruct(Var, ConsId, Args, _, _, _) ->
+        ( if Unification = deconstruct(Var, ConsId, Args, _, _, _) then
 
             ProgramPoint = program_point_init(GoalInfo),
-            (
+            ( if
                 Condition = dead_cell_table_search(ProgramPoint, DeadCellTable)
-            ->
+            then
                 ReuseAs = reuse_as_init_with_one_condition(Condition),
                 DeconstructionSpec = deconstruction_spec_init(Var,
                     ProgramPoint, ConsId, Args, ReuseAs),
@@ -584,10 +599,10 @@ compute_match_table_with_continuation(Background, DeadCellTable,
                 find_best_match_in_conjunction(Background, Cont,
                     Match0, Match),
                 multi_map.set(Var, Match, !Table)
-            ;
+            else
                 true
             )
-        ;
+        else
             true
         ),
         compute_match_table_goal_list(Background, DeadCellTable, Cont, !Table)
@@ -651,7 +666,7 @@ compute_match_table_with_continuation(Background, DeadCellTable,
     ;
         GoalExpr = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 :- pred compute_match_table_in_disjs(background_info::in, dead_cell_table::in,
@@ -746,7 +761,7 @@ deconstruction_specs_2(DeadVar, Table, !DeconstructionSpecs) :-
 
 match_get_decon_specs(Match) = Match ^ decon_specs.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Find construction unifications for dead cells, compute the values of the
 % matches.
@@ -813,19 +828,19 @@ find_match_in_goal_2(Background, Goal, !Match) :-
         GoalExpr = unify(_, _, _, Unification, _),
         (
             Unification = construct(Var, Cons, Args, _, _, _, _),
-            (
-                lookup_var_type(Background ^ back_vartypes, Var, VarType),
+            lookup_var_type(Background ^ back_vartypes, Var, VarType),
+            ( if
                 top_cell_may_be_reusable(Background ^ back_module_info,
                     VarType),
 
                 % Is the construction still looking for reuse-possibilities...
                 empty_reuse_description(goal_info_get_reuse(GoalInfo))
-            ->
+            then
                 % Is it possible for the construction to reuse the dead cell
                 % specified by the match?
                 verify_match(Background, Var, Cons, Args,
                     program_point_init(GoalInfo), !Match)
-            ;
+            else
                 true
             )
         ;
@@ -835,7 +850,7 @@ find_match_in_goal_2(Background, Goal, !Match) :-
             )
         ;
             Unification = complicated_unify(_, _, _),
-            unexpected($module, $pred, "complicated unify")
+            unexpected($pred, "complicated unify")
         )
     ;
         GoalExpr = plain_call(_, _, _, _, _, _)
@@ -870,7 +885,7 @@ find_match_in_goal_2(Background, Goal, !Match) :-
         find_match_in_goal_2(Background, ScopeGoal, !Match)
     ;
         GoalExpr = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 :- func count_candidates(list(match)) = int.
@@ -885,7 +900,7 @@ add_degree(Match, Degree0) = Degree0 + Match ^ match_degree.
 
 empty_reuse_description(no_reuse_info).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Verify the value of a match for a given construction
 %
@@ -919,17 +934,17 @@ beta_value = 1.
 
 verify_match(Background, NewVar, NewCons, NewArgs, PP, !Match) :-
     DeconSpecs = !.Match ^ decon_specs,
-    (
+    ( if
         % The construction must be compatible with *all* deconstruction specs.
         % Otherwise we may try to reuse a cell which is only compatible through
         % one code path but not another.
         list.map(compute_reuse_type(Background, NewVar, NewCons, NewArgs),
             DeconSpecs, ReuseTypes),
         ReuseType = glb_reuse_types(ReuseTypes) % Can Fail.
-    ->
+    then
         ConSpec = con(PP, ReuseType),
         match_add_construction(ConSpec, !Match)
-    ;
+    else
         true
     ).
 
@@ -946,16 +961,15 @@ verify_match(Background, NewVar, NewCons, NewArgs, PP, !Match) :-
 :- pred compute_reuse_type(background_info::in, prog_var::in, cons_id::in,
     prog_vars::in, deconstruction_spec::in, reuse_type::out) is semidet.
 
-compute_reuse_type(Background, NewVar, NewCons, NewCellArgs, DeconSpec,
+compute_reuse_type(Background, _NewVar, NewCons, NewCellArgs, DeconSpec,
         ReuseType) :-
-    DeconSpec = decon(DeadVar, _, DeadCons, DeadCellArgs, _),
+    DeconSpec = decon(_DeadVar, _, DeadCons, DeadCellArgs, _),
 
     ModuleInfo = Background ^ back_module_info,
-    VarTypes = Background ^ back_vartypes,
 
-    ( NewCons = DeadCons ->
+    ( if NewCons = DeadCons then
         SameCons = yes
-    ;
+    else
         SameCons = no,
         % XXX All the reuse code was written before packed and double word
         % arguments were introduced. For now only allow reuse of cells with
@@ -972,10 +986,10 @@ compute_reuse_type(Background, NewVar, NewCons, NewCellArgs, DeconSpec,
     NewNumArgs \= 0,
 
     % Include the space needed for secondary tags.
-    has_secondary_tag(ModuleInfo, VarTypes, NewVar, NewCons, SecTag),
-    has_secondary_tag(ModuleInfo, VarTypes, DeadVar, DeadCons, DeadSecTag),
-    NewArity = NewNumArgs + (SecTag = yes -> 1 ; 0),
-    DeadArity = DeadNumArgs + (DeadSecTag = yes -> 1 ; 0),
+    has_secondary_tag(ModuleInfo, NewCons, SecTag),
+    has_secondary_tag(ModuleInfo, DeadCons, DeadSecTag),
+    NewArity = NewNumArgs + (if SecTag = yes then 1 else 0),
+    DeadArity = DeadNumArgs + (if DeadSecTag = yes then 1 else 0),
 
     % The new cell must not be bigger than the dead cell.
     NewArity =< DeadArity,
@@ -1000,11 +1014,9 @@ compute_reuse_type(Background, NewVar, NewCons, NewCellArgs, DeconSpec,
         DeadSecTag, DeadCellArgs),
     UpToDateFields = list.length(
         list.delete_all(ReuseFields, needs_update)),
-    %
-    % Finally, compute the value of this reuse-configuration.
-    %
-    ( SameCons = yes -> SameConsV = 0; SameConsV = 1),
 
+    % Finally, compute the value of this reuse-configuration.
+    (if SameCons = yes then SameConsV = 0 else SameConsV = 1),
     Weight = ( (alfa_value + gamma_value) * NewArity + beta_value
         - gamma_value * (NewArity - UpToDateFields)
         - beta_value * SameConsV
@@ -1014,34 +1026,49 @@ compute_reuse_type(Background, NewVar, NewCons, NewCellArgs, DeconSpec,
 
 :- pred cons_has_normal_fields(module_info::in, cons_id::in) is semidet.
 
-cons_has_normal_fields(ModuleInfo, Cons) :-
+cons_has_normal_fields(ModuleInfo, ConsId) :-
+    require_complete_switch [ConsId]
     (
-        Cons = cons(_, _, TypeCtor),
-        get_cons_defn_det(ModuleInfo, TypeCtor, Cons, ConsDefn),
-        ConsArgs = ConsDefn ^ cons_args,
-        all [Arg] (
-            list.member(Arg, ConsArgs)
+        ConsId = cons(_, _, _),
+        get_cons_repn_defn_det(ModuleInfo, ConsId, ConsRepnDefn),
+        ConsArgRepns = ConsRepnDefn ^ cr_args,
+        all [ArgRepn] (
+            list.member(ArgRepn, ConsArgRepns)
         =>
-            Arg = ctor_arg(_, _, full_word, _)
+            ArgRepn = ctor_arg_repn(_, _, apw_full(_, _), _)
         )
     ;
-        Cons = tuple_cons(_)
+        ConsId = tuple_cons(_)
     ;
-        ( Cons = closure_cons(_, _)
-        ; Cons = int_const(_)
-        ; Cons = float_const(_)
-        ; Cons = char_const(_)
-        ; Cons = string_const(_)
-        ; Cons = impl_defined_const(_)
-        ; Cons = type_ctor_info_const(_, _, _)
-        ; Cons = base_typeclass_info_const(_, _, _, _)
-        ; Cons = type_info_cell_constructor(_)
-        ; Cons = typeclass_info_cell_constructor
-        ; Cons = tabling_info_const(_)
-        ; Cons = table_io_decl(_)
-        ; Cons = deep_profiling_proc_layout(_)
+        ConsId = ground_term_const(_, _RepnConsId)
+        % We could test whether _RepnConsId is normal as we test cons/3
+        % cons_ids above, but we should not do so, because ground term
+        % constants are NOT candidates for reuse.
+    ;
+        ( ConsId = type_info_const(_)
+        ; ConsId = typeclass_info_const(_)
         ),
-        unexpected($module, $pred, "unusual cons_id")
+        % XXX Before the require_complete_switch scope was added,
+        % this predicate silently failed for these two cons_ids.
+        % To me (zs), this shows that this module treats these cons_ids
+        % as not having normal fields more by accident than by design.
+        fail
+    ;
+        ( ConsId = closure_cons(_, _)
+        ; ConsId = some_int_const(_)
+        ; ConsId = float_const(_)
+        ; ConsId = char_const(_)
+        ; ConsId = string_const(_)
+        ; ConsId = impl_defined_const(_)
+        ; ConsId = type_ctor_info_const(_, _, _)
+        ; ConsId = base_typeclass_info_const(_, _, _, _)
+        ; ConsId = type_info_cell_constructor(_)
+        ; ConsId = typeclass_info_cell_constructor
+        ; ConsId = tabling_info_const(_)
+        ; ConsId = table_io_entry_desc(_)
+        ; ConsId = deep_profiling_proc_layout(_)
+        ),
+        unexpected($pred, "unusual cons_id")
     ).
 
 :- func glb_reuse_types(list(reuse_type)) = reuse_type is semidet.
@@ -1060,12 +1087,10 @@ glb_reuse_types_2(R1, R2) = R :-
 :- func ands(list(needs_update), list(needs_update)) = list(needs_update).
 
 ands(L1, L2) = L :-
-    (
-        length(L1) =< length(L2)
-    ->
+    ( if length(L1) =< length(L2) then
         L1b = L1,
         L2b = take_upto(length(L1), L2)
-    ;
+    else
         L1b = take_upto(length(L2), L1),
         L2b = L2
     ),
@@ -1079,26 +1104,22 @@ needs_update_and(does_not_need_update, needs_update) = needs_update.
 needs_update_and(does_not_need_update, does_not_need_update) =
     does_not_need_update.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-    % has_secondary_tag(Var, ConsId, ExplicitSecTag) returns `yes' iff the
-    % variable, Var, with cons_id, ConsId, requires a remote
-    % secondary tag to distinguish between its various functors.
+    % has_secondary_tag(ModuleInfo, ConsId, HasSecTag) returns `yes' iff
+    % ConsId requires a remote secondary tag to distinguish between
+    % the various functors of its type.
     %
-:- pred has_secondary_tag(module_info::in, vartypes::in,
-    prog_var::in, cons_id::in, bool::out) is det.
+:- pred has_secondary_tag(module_info::in, cons_id::in, bool::out) is det.
 
-has_secondary_tag(ModuleInfo, VarTypes, Var, ConsId, SecondaryTag) :-
-    (
-        lookup_var_type(VarTypes, Var, Type),
-        type_to_type_defn_body(ModuleInfo, Type, TypeBody),
-        TypeBody = hlds_du_type(_, ConsTagValues, _, _, _, _, _, _, _),
-        map.search(ConsTagValues, ConsId, ConsTag),
-        MaybeSecondaryTag = get_secondary_tag(ConsTag),
-        MaybeSecondaryTag = yes(_)
-    ->
+has_secondary_tag(ModuleInfo, ConsId, SecondaryTag) :-
+    ( if
+        get_cons_repn_defn(ModuleInfo, ConsId, ConsRepn),
+        ConsTag = ConsRepn ^ cr_tag,
+        get_maybe_secondary_tag(ConsTag) = yes(_)
+    then
         SecondaryTag = yes
-    ;
+    else
         SecondaryTag = no
     ).
 
@@ -1146,9 +1167,9 @@ equals([], []) = [].
 equals([], [_|_]) = [].
 equals([_|_], []) = [].
 equals([X | Xs], [Y | Ys]) = [NeedsUpdate | equals(Xs, Ys)] :-
-    ( X = Y ->
+    ( if X = Y then
         NeedsUpdate = does_not_need_update
-    ;
+    else
         NeedsUpdate = needs_update
     ).
 
@@ -1157,8 +1178,8 @@ equals([X | Xs], [Y | Ys]) = [NeedsUpdate | equals(Xs, Ys)] :-
 drop_one([]) = [].
 drop_one([_ | Xs]) = Xs.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
     % Once a match is selected (hence a set of deconstructions and matching
     % constructions), annotate all the involved unifications in the goal.
@@ -1222,7 +1243,7 @@ annotate_reuses_in_goal(Background, Match, !Goal) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand.")
+        unexpected($pred, "shorthand.")
     ),
     !:Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -1241,31 +1262,31 @@ annotate_reuse_for_unification(Background, Match, Unification, !GoalInfo) :-
     CurrentProgramPoint = program_point_init(!.GoalInfo),
     (
         Unification = deconstruct(_, _, _, _, _, _),
-        (
+        ( if
             match_find_deconstruction_spec(Match, CurrentProgramPoint,
                 _DeconSpec)
-        ->
+        then
             goal_info_set_reuse(potential_reuse(cell_died), !GoalInfo)
-        ;
+        else
             true
         )
     ;
         Unification = construct(_, _, _, _, _, _, _),
-        (
+        ( if
             match_find_construction_spec(Match, CurrentProgramPoint, ConSpec)
-        ->
+        then
             DeadVar = match_get_dead_var(Match),
             DeadConsIds = match_get_dead_cons_ids(Match),
             ReuseAs = match_get_condition(Background, Match),
             ReuseFields = ConSpec ^ con_reuse ^ reuse_fields,
 
-            ( reuse_as_conditional_reuses(ReuseAs) ->
+            ( if reuse_as_conditional_reuses(ReuseAs) then
                 Kind = conditional_reuse
-            ; reuse_as_all_unconditional_reuses(ReuseAs) ->
+            else if reuse_as_all_unconditional_reuses(ReuseAs) then
                 Kind = unconditional_reuse
-            ;
+            else
                 % reuse_as_no_reuses(ReuseAs)
-                unexpected($module, $pred, "no reuse conditions")
+                unexpected($pred, "no reuse conditions")
             ),
             CellReused = cell_reused(DeadVar, Kind, DeadConsIds,
                 ReuseFields),
@@ -1279,7 +1300,7 @@ annotate_reuse_for_unification(Background, Match, Unification, !GoalInfo) :-
                 KindReuse = reuse(CellReused)
             ),
             goal_info_set_reuse(KindReuse, !GoalInfo)
-        ;
+        else
             true
         )
     ;
@@ -1288,7 +1309,7 @@ annotate_reuse_for_unification(Background, Match, Unification, !GoalInfo) :-
         Unification = simple_test(_, _)
     ;
         Unification = complicated_unify(_, _, _),
-        unexpected($module, $pred, "complicated_unify")
+        unexpected($pred, "complicated_unify")
     ).
 
 :- pred match_find_deconstruction_spec(match::in, program_point::in,
@@ -1317,7 +1338,7 @@ deconstruction_spec_with_program_point(DeconstructionSpec ^ decon_pp,
 construction_spec_with_program_point(ConstructionSpec ^ con_pp,
     ConstructionSpec).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Predicates to print intermediate results as stored in a match_table.
 %
@@ -1326,91 +1347,73 @@ construction_spec_with_program_point(ConstructionSpec ^ con_pp,
 
 line_length = 79.
 
-:- pred dump_line(string::in, io::di, io::uo) is det.
+:- pred dump_line(io.text_output_stream::in, string::in,
+    io::di, io::uo) is det.
 
-dump_line(Msg, !IO) :-
+dump_line(Stream, Msg, !IO) :-
     Prefix = "%---",
     Start = string.append(Prefix, Msg),
     Remainder = line_length - string.count_codepoints(Start) - 1,
     Line = Start ++ string.duplicate_char('-', Remainder),
-    io.write_string(Line, !IO),
-    io.write_string("%\n", !IO).
+    io.write_string(Stream, Line, !IO),
+    io.write_string(Stream, "%\n", !IO).
 
-:- pred maybe_dump_match_table(bool::in, match_table::in, match::in,
+:- pred dump_match_table(io.text_output_stream::in, match_table::in, match::in,
     io::di, io::uo) is det.
 
-maybe_dump_match_table(VeryVerbose, MatchTable, HighestMatch, !IO) :-
-    (
-        VeryVerbose = yes,
-        dump_line("reuse table", !IO),
-        io.write_string("%\t|\tvar\t|\tvalue\t|\tdegree\n", !IO),
-        dump_match("%-sel- ", HighestMatch, !IO),
-        dump_full_table(MatchTable, !IO),
-        dump_line("", !IO)
-    ;
-        VeryVerbose = no
-    ).
+dump_match_table(Stream, MatchTable, HighestMatch, !IO) :-
+    dump_line(Stream, "reuse table", !IO),
+    io.write_string(Stream, "%\t|\tvar\t|\tvalue\t|\tdegree\n", !IO),
+    dump_match(Stream, "%-sel- ", HighestMatch, !IO),
+    dump_full_table(Stream, MatchTable, !IO),
+    dump_line(Stream, "", !IO).
 
-:- pred dump_match(string::in, match::in, io::di, io::uo) is det.
+:- pred dump_match(io.text_output_stream::in, string::in, match::in,
+    io::di, io::uo) is det.
 
-dump_match(Prefix, Match, !IO) :-
-    io.write_string(Prefix, !IO),
-    io.write_string("\t|\t", !IO),
-    io.write_int(term.var_to_int(match_get_dead_var(Match)), !IO),
-    io.write_string("\t|\t", !IO),
+dump_match(Stream, Prefix, Match, !IO) :-
+    MatchVarInt = term.var_to_int(match_get_dead_var(Match)),
+    io.format(Stream, "%s\t|\t%d\t|\t", [s(Prefix), i(MatchVarInt)], !IO),
     Val = Match ^ match_value,
-    (
-        Val \= 0.0
-    ->
-        io.format("%.2f", [f(Val)], !IO)
-    ;
-        io.write_string("-", !IO)
+    ( if Val = 0.0 then
+        io.write_string(Stream, "-", !IO)
+    else
+        io.format(Stream, "%.2f", [f(Val)], !IO)
     ),
     Degree = Match ^ match_degree,
-    io.write_string("\t|\t", !IO),
-    io.write_int(Degree, !IO),
-    io.write_string("\t", !IO),
-    dump_match_details(Match, !IO),
-    io.nl(!IO).
+    io.format(Stream, "\t|\t%d\t", [i(Degree)], !IO),
+    dump_match_details(Stream, Match, !IO).
 
-:- pred dump_match_details(match::in, io::di, io::uo) is det.
-
-dump_match_details(Match, !IO) :-
-    Conds = list.map((func(DeconSpec) = DeconSpec ^ decon_conds),
-        Match ^ decon_specs),
-    ( list.takewhile(reuse_as_all_unconditional_reuses, Conds, _, []) ->
-        CondsString = "A"
-    ;
-        CondsString = "C"
-    ),
-
-    D = list.length(Match ^ decon_specs),
-    C = list.length(Match ^ con_specs),
-    string.append_list(["d: ", int_to_string(D), ", c: ",
-        int_to_string(C),
-        ", Co: ", CondsString], Details),
-    io.write_string(Details, !IO).
-
-:- pred dump_full_table(match_table::in, io::di, io::uo) is det.
-
-dump_full_table(MatchTable, !IO) :-
-    ( multi_map.is_empty(MatchTable) ->
-        dump_line("empty match table", !IO)
-    ;
-        dump_line("full table (start)", !IO),
-        multi_map.values(MatchTable, Matches),
-        list.foldl(dump_match("%-----"), Matches, !IO),
-        dump_line("full table (end)", !IO)
-    ).
-
-:- pred maybe_dump_full_table(bool::in, match_table::in,
+:- pred dump_match_details(io.text_output_stream::in, match::in,
     io::di, io::uo) is det.
 
-maybe_dump_full_table(no, _M, !IO).
-maybe_dump_full_table(yes, M, !IO) :-
-    dump_full_table(M, !IO).
+dump_match_details(Stream, Match, !IO) :-
+    Conds = list.map((func(DeconSpec) = DeconSpec ^ decon_conds),
+        Match ^ decon_specs),
+    ( if all_true(reuse_as_all_unconditional_reuses, Conds) then
+        CondsString = "A"
+    else
+        CondsString = "C"
+    ),
+    D = list.length(Match ^ decon_specs),
+    C = list.length(Match ^ con_specs),
+    io.format(Stream, "d: %d, c: %d, Co: %s\n",
+        [i(D), i(C), s(CondsString)], !IO).
 
-%-----------------------------------------------------------------------------%
+:- pred dump_full_table(io.text_output_stream::in, match_table::in,
+    io::di, io::uo) is det.
+
+dump_full_table(Stream, MatchTable, !IO) :-
+    ( if multi_map.is_empty(MatchTable) then
+        dump_line(Stream, "empty match table", !IO)
+    else
+        dump_line(Stream, "full table (start)", !IO),
+        multi_map.values(MatchTable, Matches),
+        list.foldl(dump_match(Stream, "%-----"), Matches, !IO),
+        dump_line(Stream, "full table (end)", !IO)
+    ).
+
+%---------------------------------------------------------------------------%
 
     % After determining all local reuses of dead datastructures (a data
     % structure becomes dead and is reused in one and the same procedure), we
@@ -1426,14 +1429,16 @@ maybe_dump_full_table(yes, M, !IO) :-
 
 check_for_cell_caching(VeryVerbose, DeadCellTable0, !Goal) :-
     dead_cell_table_remove_conditionals(DeadCellTable0, DeadCellTable),
-    ( dead_cell_table_is_empty(DeadCellTable) ->
+    ( if dead_cell_table_is_empty(DeadCellTable) then
         trace [io(!IO)] (
-            maybe_write_string(VeryVerbose,
+            io.stderr_stream(StdErr, !IO),
+            maybe_write_string(StdErr, VeryVerbose,
                 "% No cells to be cached/freed.\n", !IO)
         )
-    ;
+    else
         trace [io(!IO)] (
-            maybe_write_string(VeryVerbose,
+            io.stderr_stream(StdErr, !IO),
+            maybe_write_string(StdErr, VeryVerbose,
                 "% Marking cacheable/freeable cells.\n", !IO)
         ),
         check_for_cell_caching_2(DeadCellTable, !Goal)
@@ -1498,7 +1503,7 @@ check_for_cell_caching_2(DeadCellTable, !Goal) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ),
     !:Goal = hlds_goal(GoalExpr, GoalInfo).
 
@@ -1516,21 +1521,21 @@ check_for_cell_caching_in_case(DeadCellTable, !Case) :-
 
 check_for_cell_caching_in_unification(DeadCellTable, !Unification,
         !GoalInfo) :-
-    (
+    ( if
         !.Unification = deconstruct(Var, ConsId, Args, ArgModes, CanFail, _),
         Condition = dead_cell_table_search(program_point_init(!.GoalInfo),
             DeadCellTable),
-        \+ reuse_condition_is_conditional(Condition)
-    ->
+        not reuse_condition_is_conditional(Condition)
+    then
         !:Unification = deconstruct(Var, ConsId, Args, ArgModes, CanFail,
             can_cgc),
-        % XXX Why potential_reuse and not simply "reuse" ?
+        % XXX Why potential_reuse and not simply "reuse"?
         ReuseInfo = potential_reuse(cell_died),
         goal_info_set_reuse(ReuseInfo, !GoalInfo)
-    ;
+    else
         true
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module transform_hlds.ctgc.structure_reuse.direct.choose_reuse.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
