@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2006-2007, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: structure_reuse.domain.m.
 % Main authors: nancy.
@@ -12,16 +12,20 @@
 % Definition of the abstract domain for keeping track of opportunities for
 % structure reuse.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module transform_hlds.ctgc.structure_reuse.domain.
 :- interface.
 
 :- import_module analysis.
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_pragma.
 :- import_module transform_hlds.ctgc.livedata.
+:- import_module transform_hlds.ctgc.structure_sharing.
 :- import_module transform_hlds.ctgc.structure_sharing.domain.
 
 :- import_module bimap.
@@ -30,7 +34,7 @@
 :- import_module map.
 :- import_module list.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % A reuse condition stores all the necessary information to check if
     % a procedure call is safe w.r.t. a structure reuse opportunity within
@@ -43,7 +47,7 @@
     %
 :- type reuse_as.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % reuse_condition
 %
@@ -76,7 +80,7 @@
 :- pred reuse_condition_subsumed_by(module_info::in, proc_info::in,
     reuse_condition::in, reuse_condition::in) is semidet.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % reuse_as
 %
 % XXX The implementation of this type has changed wrt. its counterpart in the
@@ -113,7 +117,7 @@
 :- func reuse_as_count_conditions(reuse_as) = int.
 
     % reuse_as_rename_using_module_info(ModuleInfo, PPId,
-    %   ActualVars, ActualTypes, CallerTypeVarSet, CallerHeadTypeParams,
+    %   ActualVars, ActualTypes, CallerTypeVarSet, CallerExternalTypeParams,
     %   FormalReuse, ActualReuse):
     %
     % Renaming of the formal description of structure reuse conditions to the
@@ -124,7 +128,7 @@
     %
 :- pred reuse_as_rename_using_module_info(module_info::in,
     pred_proc_id::in, prog_vars::in, list(mer_type)::in, tvarset::in,
-    head_type_params::in, reuse_as::in, reuse_as::out) is det.
+    external_type_params::in, reuse_as::in, reuse_as::out) is det.
 
     % Given a variable and type variable mapping, rename the reuses
     % conditions accordingly.
@@ -203,7 +207,7 @@
 :- func from_structure_reuse_domain(structure_reuse_domain) = reuse_as.
 :- func to_structure_reuse_domain(reuse_as) = structure_reuse_domain.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % reuse_as_table
 %
@@ -254,8 +258,8 @@
     no_clobber_args::in, pred_proc_id::in,
     reuse_as_table::in, reuse_as_table::out) is det.
 
-:- pred reuse_as_table_maybe_dump(bool::in, module_info::in,
-    reuse_as_table::in, io::di, io::uo) is det.
+:- pred reuse_as_table_maybe_dump(io.text_output_stream::in, bool::in,
+    module_info::in, reuse_as_table::in, io::di, io::uo) is det.
 
     % Load all the structure reuse information present in the HLDS into
     % a reuse table. This is only for the old intermodule optimisation system
@@ -263,8 +267,8 @@
     %
 :- func load_structure_reuse_table(module_info) = reuse_as_table.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -282,7 +286,7 @@
 :- import_module solutions.
 :- import_module string.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type reuse_condition
     --->    always
@@ -319,7 +323,7 @@
             % = element representing the collection of reuse conditions
             % collected for the reuses detected so far.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Reuse_condition.
 %
@@ -342,9 +346,9 @@ reuse_condition_init(ModuleInfo, ProcInfo, DeadVar, LFU, LBU, Sharing)
     %   obtained datastructures are kept as the nodes for our condition.
 
     TopCell = ctgc.datastruct.datastruct_init(DeadVar),
-    ( list.member(DeadVar, HeadVars) ->
+    ( if list.member(DeadVar, HeadVars) then
         Nodes = [TopCell]
-    ;
+    else
         SharedDatastructs = extend_datastruct(ModuleInfo, ProcInfo,
             Sharing, TopCell),
         Nodes = datastructs_project(HeadVars, SharedDatastructs)
@@ -378,6 +382,22 @@ reuse_condition_is_conditional(condition(_, _, _)).
 
 reuse_condition_reusable_nodes(condition(Nodes, _, _), Nodes).
 
+reuse_condition_rename(MapVar, TypeSubst, Condition, RenamedCondition):-
+    (
+        Condition = always,
+        RenamedCondition = always
+    ;
+        Condition = condition(DeadNodes, InUseNodes, LocalSharing),
+        RenamedDeadNodes = set.map(rename_datastruct(MapVar, TypeSubst),
+            DeadNodes),
+        RenamedInUseNodes = list.map(rename_datastruct(MapVar, TypeSubst),
+            InUseNodes),
+        sharing_as_rename(MapVar, TypeSubst, LocalSharing,
+            RenamedLocalSharing),
+        RenamedCondition = condition(RenamedDeadNodes, RenamedInUseNodes,
+            RenamedLocalSharing)
+    ).
+
 reuse_condition_subsumed_by(ModuleInfo, ProcInfo, Cond1, Cond2) :-
     (
         Cond1 = always
@@ -419,32 +439,16 @@ reuse_condition_subsumed_by_list(ModuleInfo, ProcInfo, Cond, [Cond1|Rest]) :-
 reuse_conditions_subsume_reuse_condition(ModuleInfo, ProcInfo, Conds, Cond):-
     reuse_condition_subsumed_by_list(ModuleInfo, ProcInfo, Cond, Conds).
 
-reuse_condition_rename(MapVar, TypeSubst, Condition, RenamedCondition):-
-    (
-        Condition = always,
-        RenamedCondition = always
-    ;
-        Condition = condition(DeadNodes, InUseNodes, LocalSharing),
-        RenamedDeadNodes = set.map(rename_datastruct(MapVar, TypeSubst),
-            DeadNodes),
-        RenamedInUseNodes = list.map(rename_datastruct(MapVar, TypeSubst),
-            InUseNodes),
-        sharing_as_rename(MapVar, TypeSubst, LocalSharing,
-            RenamedLocalSharing),
-        RenamedCondition = condition(RenamedDeadNodes, RenamedInUseNodes,
-            RenamedLocalSharing)
-    ).
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % reuse_as
 %
 
 reuse_as_init = no_reuse.
 reuse_as_init_with_one_condition(ReuseCondition) = ReuseAs :-
-    ( reuse_condition_is_conditional(ReuseCondition) ->
+    ( if reuse_condition_is_conditional(ReuseCondition) then
         ReuseAs = conditional([ReuseCondition])
-    ;
+    else
         ReuseAs = unconditional
     ).
 
@@ -468,8 +472,8 @@ reuse_as_subsumed_by(ModuleInfo, ProcInfo, FirstReuseAs, SecondReuseAs) :-
     ;
         FirstReuseAs = conditional(ReuseConditionsFirst),
         SecondReuseAs = conditional(ReuseConditionsSecond),
-        list.takewhile(reuse_conditions_subsume_reuse_condition(ModuleInfo,
-            ProcInfo, ReuseConditionsSecond), ReuseConditionsFirst, _,
+        list.drop_while(reuse_conditions_subsume_reuse_condition(ModuleInfo,
+            ProcInfo, ReuseConditionsSecond), ReuseConditionsFirst,
             NotSubsumed),
         NotSubsumed = []
     ).
@@ -490,10 +494,11 @@ reuse_as_count_conditions(unconditional) = 0.
 reuse_as_count_conditions(conditional(Conds)) = list.length(Conds).
 
 reuse_as_rename_using_module_info(ModuleInfo, PPId, ActualArgs, ActualTypes,
-        CallerTypeVarSet, CallerHeadTypeParams, FormalReuse, ActualReuse) :-
+        CallerTypeVarSet, CallerExternalTypeParams,
+        FormalReuse, ActualReuse) :-
     VarRenaming = get_variable_renaming(ModuleInfo, PPId, ActualArgs),
     TypeSubst = get_type_substitution(ModuleInfo, PPId, ActualTypes,
-        CallerTypeVarSet, CallerHeadTypeParams),
+        CallerTypeVarSet, CallerExternalTypeParams),
     reuse_as_rename(VarRenaming, TypeSubst, FormalReuse, ActualReuse).
 
 reuse_as_rename(MapVar, TypeSubst, ReuseAs, RenamedReuseAs) :-
@@ -513,9 +518,9 @@ reuse_as_rename(MapVar, TypeSubst, ReuseAs, RenamedReuseAs) :-
 reuse_as_add_condition(ModuleInfo, ProcInfo, Condition, !ReuseAs) :-
     (
         Condition = always,
-        ( !.ReuseAs = no_reuse ->
+        ( if !.ReuseAs = no_reuse then
             !:ReuseAs = unconditional
-        ;
+        else
             true
         )
     ;
@@ -548,12 +553,12 @@ reuse_as_add_unconditional(!ReuseAs) :-
     reuse_condition::in, reuse_conditions::in, reuse_conditions::out) is det.
 
 reuse_conditions_add_condition(ModuleInfo, ProcInfo, Condition, !Conds):-
-    (
+    ( if
         reuse_condition_subsumed_by_list(ModuleInfo, ProcInfo,
             Condition, !.Conds)
-    ->
+    then
         true
-    ;
+    else
         !:Conds = [Condition | !.Conds]
     ).
 
@@ -575,9 +580,9 @@ reuse_as_least_upper_bound(ModuleInfo, ProcInfo, NewReuseAs, !ReuseAs) :-
         NewReuseAs = no_reuse
     ;
         NewReuseAs = unconditional,
-        ( !.ReuseAs = no_reuse ->
+        ( if !.ReuseAs = no_reuse then
             !:ReuseAs = unconditional
-        ;
+        else
             true
         )
     ;
@@ -604,10 +609,10 @@ reuse_as_from_called_procedure_to_local_reuse_as(ModuleInfo, ProcInfo,
         HeadVars, LuData, SharingAs, CalledReuseAs) = LocalReuseAs :-
     (
         CalledReuseAs = no_reuse,
-        unexpected($module, $pred, "reuse_as does not specify any reuses")
+        unexpected($pred, "reuse_as does not specify any reuses")
     ;
         CalledReuseAs = unconditional,
-        unexpected($module, $pred, "reuse_as is unconditional")
+        unexpected($pred, "reuse_as is unconditional")
     ;
         CalledReuseAs = conditional(ConditionsCaller),
         ConditionsCallee =
@@ -626,7 +631,7 @@ reuse_condition_from_called_proc_to_local_condition(ModuleInfo, ProcInfo,
         HeadVars, LuData, SharingAs, CalledCondition) = LocalCondition :-
     (
         CalledCondition = always,
-        unexpected($module, $pred, "explicit condition expected")
+        unexpected($pred, "explicit condition expected")
     ;
         CalledCondition = condition(CalledDeadNodes,
             CalledInUseNodes, CalledSharingAs),
@@ -653,7 +658,7 @@ reuse_condition_from_called_proc_to_local_condition(ModuleInfo, ProcInfo,
             AllHeadVarLocalSharing = sharing_as_project(HeadVars,
                 AllLocalSharing),
 
-            LocalCondition = condition(set.from_list(AllDeadHeadVarNodes),
+            LocalCondition = condition(set.list_to_set(AllDeadHeadVarNodes),
                 AllInUseHeadVarNodes, AllHeadVarLocalSharing)
         )
     ).
@@ -727,7 +732,7 @@ reuse_as_satisfied_2(ModuleInfo, ProcInfo, LiveData, SharingAs, StaticVars,
             ( Result1 = reuse_not_possible(unknown_livedata)
             ; Result1 = reuse_not_possible(reuse_nodes_have_sharing(_))
             ),
-            unexpected($module, $pred, "unexpected result")
+            unexpected($pred, "unexpected result")
         )
     ;
         Result0 = reuse_not_possible(no_reuse),
@@ -737,7 +742,7 @@ reuse_as_satisfied_2(ModuleInfo, ProcInfo, LiveData, SharingAs, StaticVars,
         Result = Result0
     ;
         Result0 = reuse_not_possible(reuse_nodes_have_sharing(_)),
-        unexpected($module, $pred, "reuse_nodes_have_sharing")
+        unexpected($pred, "reuse_nodes_have_sharing")
     ).
 
 :- pred aliases_between_reuse_nodes(module_info::in, proc_info::in,
@@ -756,7 +761,7 @@ aliases_between_reuse_nodes(ModuleInfo, ProcInfo, SharingAs, Conditions,
         AliasedVars = set.to_sorted_list(AliasedVarsSet)
     ;
         AllNodes = [],
-        unexpected($module, $pred, "no nodes")
+        unexpected($pred, "no nodes")
     ).
 
 :- pred aliases_between_reuse_nodes_2(module_info::in, proc_info::in,
@@ -808,7 +813,7 @@ collect_aliased_vars(DataA - DataB, !Vars) :-
     set.insert(DataA ^ sc_var, !Vars),
     set.insert(DataB ^ sc_var, !Vars).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred reuse_condition_satisfied(module_info::in, proc_info::in,
     livedata::in, sharing_as::in, prog_vars::in, reuse_condition::in,
@@ -860,7 +865,7 @@ reuse_condition_satisfied(ModuleInfo, ProcInfo, LiveData, SharingAs,
         )
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 from_structure_reuse_domain(ReuseDomain) = ReuseAs :-
     (
@@ -917,7 +922,7 @@ to_structure_reuse_condition(Condition) = StructureReuseCondition :-
     StructureReuseCondition = structure_reuse_condition(DeadNodes, LiveNodes,
         to_structure_sharing_domain(SharingAs)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % reuse_as_table
 %
@@ -933,11 +938,8 @@ reuse_as_table_search_reuse_version_proc(Table, PPId, NoClobbers, NewPPId) :-
 
 reuse_as_table_reverse_search_reuse_version_proc(Table, NewPPId,
         OrigPPId, NoClobbers) :-
-    ( bimap.reverse_search(Table ^ reuse_version_map, Key, NewPPId) ->
-        Key = ppid_no_clobbers(OrigPPId, NoClobbers)
-    ;
-        unexpected($module, $pred, "reverse search failed")
-    ).
+    bimap.reverse_lookup(Table ^ reuse_version_map, Key, NewPPId),
+    Key = ppid_no_clobbers(OrigPPId, NoClobbers).
 
 reuse_as_table_set(PPId, ReuseAs_Status, !Table) :-
     T0 = !.Table ^ reuse_info_map,
@@ -949,38 +951,41 @@ reuse_as_table_insert_reuse_version_proc(PPId, NoClobbers, NewPPId, !Table) :-
     bimap.det_insert(ppid_no_clobbers(PPId, NoClobbers), NewPPId, T0, T),
     !Table ^ reuse_version_map := T.
 
-reuse_as_table_maybe_dump(DoDump, ModuleInfo, Table, !IO) :-
+%---------------------------------------------------------------------------%
+
+reuse_as_table_maybe_dump(Stream, DoDump, ModuleInfo, Table, !IO) :-
     (
         DoDump = no
     ;
         DoDump = yes,
-        reuse_as_table_dump(ModuleInfo, Table, !IO)
+        reuse_as_table_dump(Stream, ModuleInfo, Table, !IO)
     ).
 
-:- pred reuse_as_table_dump(module_info::in, reuse_as_table::in,
-    io::di, io::uo) is det.
+:- pred reuse_as_table_dump(io.text_output_stream::in, module_info::in,
+    reuse_as_table::in, io::di, io::uo) is det.
 
-reuse_as_table_dump(ModuleInfo, Table, !IO) :-
+reuse_as_table_dump(Stream, ModuleInfo, Table, !IO) :-
     ReuseInfoMap = Table ^ reuse_info_map,
-    ( map.is_empty(ReuseInfoMap) ->
-        io.write_string("% ReuseTable: Empty\n", !IO)
-    ;
-        io.write_string("% ReuseTable: PPId --> Reuse\n", !IO),
-        map.foldl(dump_entries(ModuleInfo), ReuseInfoMap, !IO)
+    ( if map.is_empty(ReuseInfoMap) then
+        io.write_string(Stream, "% ReuseTable: Empty\n", !IO)
+    else
+        io.write_string(Stream, "% ReuseTable: PPId --> Reuse\n", !IO),
+        map.foldl(dump_entries(Stream, ModuleInfo), ReuseInfoMap, !IO)
     ).
 
-:- pred dump_entries(module_info::in, pred_proc_id::in,
-    reuse_as_and_status::in, io::di, io::uo) is det.
+:- pred dump_entries(io.text_output_stream::in, module_info::in,
+    pred_proc_id::in, reuse_as_and_status::in, io::di, io::uo) is det.
 
-dump_entries(ModuleInfo, PPId, reuse_as_and_status(ReuseAs, _Status), !IO) :-
-    io.write_string("% ", !IO),
-    write_pred_proc_id(ModuleInfo, PPId, !IO),
-    io.write_string("\t--> ", !IO),
-    io.write_string(reuse_as_short_description(ReuseAs), !IO),
-    io.nl(!IO).
+dump_entries(Stream, ModuleInfo, PPId,
+        reuse_as_and_status(ReuseAs, _Status), !IO) :-
+    PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+    io.format(Stream, "%% %s\t--> %s\n",
+        [s(PPIdStr), s(reuse_as_short_description(ReuseAs))], !IO).
+
+%---------------------------------------------------------------------------%
 
 load_structure_reuse_table(ModuleInfo) = ReuseTable :-
-    module_info_get_valid_predids(PredIds, ModuleInfo, _ModuleInfo),
+    module_info_get_valid_pred_ids(ModuleInfo, PredIds),
     list.foldl(load_structure_reuse_table_2(ModuleInfo), PredIds,
         reuse_as_table_init, ReuseTable).
 
@@ -989,7 +994,7 @@ load_structure_reuse_table(ModuleInfo) = ReuseTable :-
 
 load_structure_reuse_table_2(ModuleInfo, PredId, !ReuseTable) :-
     module_info_pred_info(ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_procids(PredInfo),
+    ProcIds = pred_info_valid_procids(PredInfo),
     list.foldl(load_structure_reuse_table_3(ModuleInfo, PredId),
         ProcIds, !ReuseTable).
 
@@ -1010,6 +1015,6 @@ load_structure_reuse_table_3(ModuleInfo, PredId, ProcId, !ReuseTable) :-
         MaybePublicReuse = no
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module transform_hlds.ctgc.structure_reuse.domain.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
