@@ -16,10 +16,13 @@
 :- module ll_backend.tag_switch.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.code_model.
 :- import_module hlds.hlds_goal.
 :- import_module ll_backend.code_info.
+:- import_module ll_backend.code_loc_dep.
 :- import_module ll_backend.llds.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 
 :- import_module list.
@@ -31,22 +34,23 @@
 :- pred generate_tag_switch(list(tagged_case)::in, rval::in, mer_type::in,
     string::in, code_model::in, can_fail::in, hlds_goal_info::in, label::in,
     branch_end::in, branch_end::out, llds_code::out,
-    code_info::in, code_info::out) is det.
+    code_info::in, code_info::out, code_loc_dep::in) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module backend_libs.
 :- import_module backend_libs.builtin_ops.
 :- import_module backend_libs.rtti.
 :- import_module backend_libs.switch_util.
-:- import_module hlds.hlds_data.
 :- import_module hlds.hlds_llds.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
 :- import_module ll_backend.switch_case.
-:- import_module parse_tree.prog_data.
 
 :- import_module assoc_list.
 :- import_module cord.
@@ -56,6 +60,8 @@
 :- import_module pair.
 :- import_module require.
 :- import_module string.
+:- import_module uint8.
+:- import_module unit.
 
 %-----------------------------------------------------------------------------%
 
@@ -192,76 +198,80 @@
 %-----------------------------------------------------------------------------%
 
 generate_tag_switch(TaggedCases, VarRval, VarType, VarName, CodeModel, CanFail,
-        SwitchGoalInfo, EndLabel, !MaybeEnd, Code, !CI) :-
-
+        SwitchGoalInfo, EndLabel, !MaybeEnd, Code, !CI, CLD0) :-
     % We get registers for holding the primary and (if needed) the secondary
-    % tag. The tags needed only by the switch, and no other code gets control
-    % between producing the tag values and all their uses, so we can release
-    % the registers for use by the code of the various cases.
-    %
-    % We forgo using the primary tag register if the primary tag is needed
-    % only once, or if the "register" we get is likely to be slower than
-    % recomputing the tag from scratch.
+    % tag. The tags are needed only by the switch, and no other code gets
+    % control between producing the tag values and all their uses, so
+    % we can immediately release the registers for use by the code of
+    % the various cases.
     %
     % We need to get and release the registers before we generate the code
     % of the switch arms, since the set of free registers will in general be
     % different before and after that action.
-    acquire_reg(reg_r, PtagReg, !CI),
-    acquire_reg(reg_r, StagReg, !CI),
-    release_reg(PtagReg, !CI),
-    release_reg(StagReg, !CI),
+    %
+    % We forgo using the primary tag register if the primary tag is needed
+    % only once, or if the "register" we get is likely to be slower than
+    % recomputing the tag from scratch.
+    some [!CLD] (
+        !:CLD = CLD0,
+        acquire_reg(reg_r, PtagReg, !CLD),
+        acquire_reg(reg_r, StagReg, !CLD),
+        release_reg(PtagReg, !CLD),
+        release_reg(StagReg, !CLD),
+        remember_position(!.CLD, BranchStart)
+    ),
 
     % Group the cases based on primary tag value and find out how many
     % constructors share each primary tag value.
     get_module_info(!.CI, ModuleInfo),
     get_ptag_counts(VarType, ModuleInfo, MaxPrimary, PtagCountMap),
-    remember_position(!.CI, BranchStart),
     Params = represent_params(VarName, SwitchGoalInfo, CodeModel, BranchStart,
         EndLabel),
     group_cases_by_ptag(TaggedCases, represent_tagged_case_for_llds(Params),
-        map.init, CaseLabelMap0, !MaybeEnd, !CI, _, PtagCaseMap),
+        map.init, CaseLabelMap0, !MaybeEnd, !CI, unit, _, _, PtagCaseMap),
 
     map.count(PtagCaseMap, PtagsUsed),
     get_globals(!.CI, Globals),
-    globals.lookup_int_option(Globals, dense_switch_size, DenseSwitchSize),
-    globals.lookup_int_option(Globals, try_switch_size, TrySwitchSize),
-    globals.lookup_int_option(Globals, binary_switch_size, BinarySwitchSize),
-    ( PtagsUsed >= DenseSwitchSize ->
+    globals.get_opt_tuple(Globals, OptTuple),
+    DenseSwitchSize = OptTuple ^ ot_dense_switch_size,
+    TrySwitchSize = OptTuple ^ ot_try_switch_size,
+    BinarySwitchSize = OptTuple ^ ot_binary_switch_size,
+    ( if PtagsUsed >= DenseSwitchSize then
         PrimaryMethod = jump_table
-    ; PtagsUsed >= BinarySwitchSize ->
+    else if PtagsUsed >= BinarySwitchSize then
         PrimaryMethod = binary_search
-    ; PtagsUsed >= TrySwitchSize ->
+    else if PtagsUsed >= TrySwitchSize then
         PrimaryMethod = try_chain
-    ;
+    else
         PrimaryMethod = try_me_else_chain
     ),
 
-    (
+    ( if
         PrimaryMethod \= jump_table,
         PtagsUsed >= 2,
         globals.lookup_int_option(Globals, num_real_r_regs, NumRealRegs),
         (
             NumRealRegs = 0
         ;
-            ( PtagReg = reg(reg_r, PtagRegNo) ->
+            ( if PtagReg = reg(reg_r, PtagRegNo) then
                 PtagRegNo =< NumRealRegs
-            ;
-                unexpected($module, $pred, "improper reg in tag switch")
+            else
+                unexpected($pred, "improper reg in tag switch")
             )
         )
-    ->
+    then
         PtagCode = singleton(
             llds_instr(assign(PtagReg, unop(tag, VarRval)),
                 "compute tag to switch on")
         ),
         PtagRval = lval(PtagReg)
-    ;
+    else
         PtagCode = empty,
         PtagRval = unop(tag, VarRval)
     ),
 
-    % We generate EndCode (and if needed, FailCode) here because the last
-    % case within a primary tag may not be the last case overall.
+    % We generate EndCode (and if needed, FailCode) here because
+    % the last case within a primary tag may not be the last case overall.
     EndCode = singleton(
         llds_instr(label(EndLabel), "end of tag switch")
     ),
@@ -276,23 +286,27 @@ generate_tag_switch(TaggedCases, VarRval, VarType, VarName, CodeModel, CanFail,
         FailLabelCode = singleton(
             llds_instr(label(FailLabel), "switch has failed")
         ),
-        % We must generate the failure code in the context in which none of the
-        % switch arms have been executed yet.
-        reset_to_position(BranchStart, !CI),
-        generate_failure(FailureCode, !CI),
+        % We must generate the failure code in the context in which
+        % none of the switch arms have been executed yet.
+        some [!CLD] (
+            reset_to_position(BranchStart, !.CI, !:CLD),
+            generate_failure(FailureCode, !CI, !.CLD)
+        ),
         FailCode = FailLabelCode ++ FailureCode
     ),
 
     (
         PrimaryMethod = binary_search,
-        order_ptags_by_value(0, MaxPrimary, PtagCaseMap, PtagCaseList),
-        generate_primary_binary_search(PtagCaseList, 0, MaxPrimary, PtagRval,
+        order_ptags_by_value(ptag(0u8), ptag(MaxPrimary),
+            PtagCaseMap, PtagCaseList),
+        generate_primary_binary_search(PtagCaseList, 0u8, MaxPrimary, PtagRval,
             StagReg, VarRval, MaybeFailLabel, PtagCountMap, CasesCode,
             CaseLabelMap0, CaseLabelMap, !CI)
     ;
         PrimaryMethod = jump_table,
-        order_ptags_by_value(0, MaxPrimary, PtagCaseMap, PtagCaseList),
-        generate_primary_jump_table(PtagCaseList, 0, MaxPrimary, StagReg,
+        order_ptags_by_value(ptag(0u8), ptag(MaxPrimary),
+            PtagCaseMap, PtagCaseList),
+        generate_primary_jump_table(PtagCaseList, 0u8, MaxPrimary, StagReg,
             VarRval, MaybeFailLabel, PtagCountMap, Targets, TableCode,
             CaseLabelMap0, CaseLabelMap, !CI),
         SwitchCode = singleton(
@@ -303,12 +317,12 @@ generate_tag_switch(TaggedCases, VarRval, VarType, VarName, CodeModel, CanFail,
     ;
         PrimaryMethod = try_chain,
         order_ptags_by_count(PtagCountMap, PtagCaseMap, PtagCaseList0),
-        (
+        ( if
             CanFail = cannot_fail,
             PtagCaseList0 = [MostFreqCase | OtherCases]
-        ->
+        then
             PtagCaseList = OtherCases ++ [MostFreqCase]
-        ;
+        else
             PtagCaseList = PtagCaseList0
         ),
         generate_primary_try_chain(PtagCaseList, PtagRval, StagReg, VarRval,
@@ -336,14 +350,14 @@ generate_tag_switch(TaggedCases, VarRval, VarType, VarName, CodeModel, CanFail,
 
 generate_primary_try_me_else_chain([], _, _, _, _, _, _,
         !CaseLabelMap, !CI) :-
-    unexpected($module, $pred, "empty switch").
+    unexpected($pred, "empty switch").
 generate_primary_try_me_else_chain([PtagGroup | PtagGroups], PtagRval, StagReg,
         VarRval, MaybeFailLabel, PtagCountMap, Code, !CaseLabelMap, !CI) :-
     PtagGroup = ptag_case_group_entry(MainPtag, OtherPtags, PtagCase),
     PtagCase = ptag_case(StagLoc, StagGoalMap),
     map.lookup(PtagCountMap, MainPtag, CountInfo),
     CountInfo = StagLocPrime - MaxSecondary,
-    expect(unify(StagLoc, StagLocPrime), $module, $pred,
+    expect(unify(StagLoc, StagLocPrime), $pred,
         "secondary tag locations differ"),
     (
         PtagGroups = [_ | _],
@@ -378,7 +392,7 @@ generate_primary_try_me_else_chain([PtagGroup | PtagGroups], PtagRval, StagReg,
     ).
 
 :- pred generate_primary_try_me_else_chain_case(rval::in, lval::in,
-    tag_bits::in, list(tag_bits)::in, ptag_case(label)::in, int::in, rval::in,
+    ptag::in, list(ptag)::in, ptag_case(label)::in, int::in, rval::in,
     maybe(label)::in, llds_code::out, case_label_map::in, case_label_map::out,
     code_info::in, code_info::out) is det.
 
@@ -386,8 +400,9 @@ generate_primary_try_me_else_chain_case(PtagRval, StagReg,
         MainPtag, OtherPtags, PtagCase, MaxSecondary, VarRval, MaybeFailLabel,
         Code, !CaseLabelMap, !CI) :-
     get_next_label(ElseLabel, !CI),
-    TestRval0 = binop(ne, PtagRval,
-        unop(mktag, const(llconst_int(MainPtag)))),
+    MainPtag = ptag(MainPtagUint8),
+    TestRval0 = binop(ne(int_type_int), PtagRval,
+        const(llconst_int(uint8.cast_to_int(MainPtagUint8)))),
     generate_primary_try_me_else_chain_other_ptags(OtherPtags, PtagRval,
         TestRval0, TestRval),
     TestCode = singleton(
@@ -403,14 +418,15 @@ generate_primary_try_me_else_chain_case(PtagRval, StagReg,
     ),
     Code = TestCode ++ TagCode ++ ElseCode.
 
-:- pred generate_primary_try_me_else_chain_other_ptags(list(tag_bits)::in,
+:- pred generate_primary_try_me_else_chain_other_ptags(list(ptag)::in,
     rval::in, rval::in, rval::out) is det.
 
 generate_primary_try_me_else_chain_other_ptags([], _, TestRval, TestRval).
 generate_primary_try_me_else_chain_other_ptags([OtherPtag | OtherPtags],
         PtagRval, TestRval0, TestRval) :-
-    ThisTestRval = binop(ne, PtagRval,
-        unop(mktag, const(llconst_int(OtherPtag)))),
+    OtherPtag = ptag(OtherPtagUint8),
+    ThisTestRval = binop(ne(int_type_int), PtagRval,
+        const(llconst_int(uint8.cast_to_int(OtherPtagUint8)))),
     TestRval1 = binop(logical_and, TestRval0, ThisTestRval),
     generate_primary_try_me_else_chain_other_ptags(OtherPtags,
         PtagRval, TestRval1, TestRval).
@@ -426,7 +442,7 @@ generate_primary_try_me_else_chain_other_ptags([OtherPtag | OtherPtags],
     code_info::in, code_info::out) is det.
 
 generate_primary_try_chain([], _, _, _, _, _, _, _, _, !CaseLabelMap, !CI) :-
-     unexpected($module, $pred, "empty list").
+     unexpected($pred, "empty list").
 generate_primary_try_chain([PtagGroup | PtagGroups], PtagRval, StagReg,
         VarRval, MaybeFailLabel, PtagCountMap, PrevTestsCode0, PrevCasesCode0,
         Code, !CaseLabelMap, !CI) :-
@@ -434,7 +450,7 @@ generate_primary_try_chain([PtagGroup | PtagGroups], PtagRval, StagReg,
     PtagCase = ptag_case(StagLoc, StagGoalMap),
     map.lookup(PtagCountMap, MainPtag, CountInfo),
     CountInfo = StagLocPrime - MaxSecondary,
-    expect(unify(StagLoc, StagLocPrime), $module, $pred,
+    expect(unify(StagLoc, StagLocPrime), $pred,
         "secondary tag locations differ"),
     (
         PtagGroups = [_ | _],
@@ -475,7 +491,7 @@ generate_primary_try_chain([PtagGroup | PtagGroups], PtagRval, StagReg,
     ).
 
 :- pred generate_primary_try_chain_case(rval::in, lval::in,
-    tag_bits::in, list(tag_bits)::in,
+    ptag::in, list(ptag)::in,
     ptag_case(label)::in, int::in, rval::in, maybe(label)::in,
     llds_code::in, llds_code::out, llds_code::in, llds_code::out,
     case_label_map::in, case_label_map::out,
@@ -486,8 +502,9 @@ generate_primary_try_chain_case(PtagRval, StagReg, MainPtag, OtherPtags,
         PrevTestsCode0, PrevTestsCode, PrevCasesCode0, PrevCasesCode,
         !CaseLabelMap, !CI) :-
     get_next_label(ThisPtagLabel, !CI),
-    TestRval0 = binop(eq, PtagRval,
-        unop(mktag, const(llconst_int(MainPtag)))),
+    MainPtag = ptag(MainPtagUint8),
+    TestRval0 = binop(eq(int_type_int), PtagRval,
+        const(llconst_int(uint8.cast_to_int(MainPtagUint8)))),
     generate_primary_try_chain_other_ptags(OtherPtags, PtagRval,
         TestRval0, TestRval),
     TestCode = singleton(
@@ -505,14 +522,15 @@ generate_primary_try_chain_case(PtagRval, StagReg, MainPtag, OtherPtags,
     PrevTestsCode = PrevTestsCode0 ++ TestCode,
     PrevCasesCode = LabelCode ++ TagCode ++ PrevCasesCode0.
 
-:- pred generate_primary_try_chain_other_ptags(list(tag_bits)::in,
+:- pred generate_primary_try_chain_other_ptags(list(ptag)::in,
     rval::in, rval::in, rval::out) is det.
 
 generate_primary_try_chain_other_ptags([], _, TestRval, TestRval).
 generate_primary_try_chain_other_ptags([OtherPtag | OtherPtags],
         PtagRval, TestRval0, TestRval) :-
-    ThisTestRval = binop(eq, PtagRval,
-        unop(mktag, const(llconst_int(OtherPtag)))),
+    OtherPtag = ptag(OtherPtagUint8),
+    ThisTestRval = binop(eq(int_type_int), PtagRval,
+        const(llconst_int(uint8.cast_to_int(OtherPtagUint8)))),
     TestRval1 = binop(logical_or, TestRval0, ThisTestRval),
     generate_primary_try_chain_other_ptags(OtherPtags,
         PtagRval, TestRval1, TestRval).
@@ -522,8 +540,9 @@ generate_primary_try_chain_other_ptags([OtherPtag | OtherPtags],
     % Generate the cases for a primary tag using a dense jump table
     % that has an entry for all possible primary tag values.
     %
-:- pred generate_primary_jump_table(ptag_case_list(label)::in, int::in,
-    int::in, lval::in, rval::in, maybe(label)::in, ptag_count_map::in,
+:- pred generate_primary_jump_table(ptag_case_list(label)::in,
+    uint8::in, uint8::in, lval::in, rval::in,
+    maybe(label)::in, ptag_count_map::in,
     list(maybe(label))::out, llds_code::out,
     case_label_map::in, case_label_map::out,
     code_info::in, code_info::out) is det.
@@ -531,27 +550,27 @@ generate_primary_try_chain_other_ptags([OtherPtag | OtherPtags],
 generate_primary_jump_table(PtagGroups, CurPrimary, MaxPrimary, StagReg,
         VarRval, MaybeFailLabel, PtagCountMap, Targets, Code,
         !CaseLabelMap, !CI) :-
-    ( CurPrimary > MaxPrimary ->
-        expect(unify(PtagGroups, []), $module, $pred,
+    ( if CurPrimary > MaxPrimary then
+        expect(unify(PtagGroups, []), $pred,
             "PtagGroups != [] when Cur > Max"),
         Targets = [],
         Code = empty
-    ;
-        NextPrimary = CurPrimary + 1,
-        (
+    else
+        NextPrimary = CurPrimary + 1u8,
+        ( if
             PtagGroups = [PtagCaseEntry | PtagGroupsTail],
-            PtagCaseEntry = ptag_case_entry(CurPrimary, PrimaryInfo)
-        ->
+            PtagCaseEntry = ptag_case_entry(ptag(CurPrimary), PrimaryInfo)
+        then
             PrimaryInfo = ptag_case(StagLoc, StagGoalMap),
-            map.lookup(PtagCountMap, CurPrimary, CountInfo),
+            map.lookup(PtagCountMap, ptag(CurPrimary), CountInfo),
             CountInfo = StagLocPrime - MaxSecondary,
-            expect(unify(StagLoc, StagLocPrime), $module, $pred,
+            expect(unify(StagLoc, StagLocPrime), $pred,
                 "secondary tag locations differ"),
             get_next_label(NewLabel, !CI),
             Comment = "start of a case in primary tag switch: ptag " ++
-                string.int_to_string(CurPrimary),
+                string.uint8_to_string(CurPrimary),
             LabelCode = singleton(llds_instr(label(NewLabel), Comment)),
-            generate_primary_tag_code(StagGoalMap, CurPrimary, [],
+            generate_primary_tag_code(StagGoalMap, ptag(CurPrimary), [],
                 MaxSecondary, StagReg, StagLoc, VarRval, MaybeFailLabel,
                 ThisTagCode, !CaseLabelMap, !CI),
             generate_primary_jump_table(PtagGroupsTail, NextPrimary,
@@ -559,7 +578,7 @@ generate_primary_jump_table(PtagGroups, CurPrimary, MaxPrimary, StagReg,
                 TailTargets, TailCode, !CaseLabelMap, !CI),
             Targets = [yes(NewLabel) | TailTargets],
             Code = LabelCode ++ ThisTagCode ++ TailCode
-        ;
+        else
             generate_primary_jump_table(PtagGroups, NextPrimary, MaxPrimary,
                 StagReg, VarRval, MaybeFailLabel, PtagCountMap,
                 TailTargets, TailCode, !CaseLabelMap, !CI),
@@ -574,22 +593,22 @@ generate_primary_jump_table(PtagGroups, CurPrimary, MaxPrimary, StagReg,
     % This invocation looks after primary tag values in the range
     % MinPtag to MaxPtag (including both boundary values).
     %
-:- pred generate_primary_binary_search(ptag_case_list(label)::in, int::in,
-    int::in, rval::in, lval::in, rval::in, maybe(label)::in,
+:- pred generate_primary_binary_search(ptag_case_list(label)::in,
+    uint8::in, uint8::in, rval::in, lval::in, rval::in, maybe(label)::in,
     ptag_count_map::in, llds_code::out,
     case_label_map::in, case_label_map::out,
     code_info::in, code_info::out) is det.
 
 generate_primary_binary_search(PtagGroups, MinPtag, MaxPtag, PtagRval, StagReg,
         VarRval, MaybeFailLabel, PtagCountMap, Code, !CaseLabelMap, !CI) :-
-    ( MinPtag = MaxPtag ->
+    ( if MinPtag = MaxPtag then
         CurPrimary = MinPtag,
         (
             PtagGroups = [],
             % There is no code for this tag.
             (
                 MaybeFailLabel = yes(FailLabel),
-                string.int_to_string(CurPrimary, PtagStr),
+                PtagStr = string.uint8_to_string(CurPrimary),
                 Comment = "no code for ptag " ++ PtagStr,
                 Code = singleton(
                     llds_instr(goto(code_label(FailLabel)), Comment)
@@ -601,41 +620,43 @@ generate_primary_binary_search(PtagGroups, MinPtag, MaxPtag, PtagRval, StagReg,
                 Code = empty
             )
         ;
-            PtagGroups = [ptag_case_entry(CurPrimaryPrime, PrimaryInfo)],
-            expect(unify(CurPrimary, CurPrimaryPrime), $module, $pred,
+            PtagGroups = [ptag_case_entry(ptag(CurPrimaryPrime), PrimaryInfo)],
+            expect(unify(CurPrimary, CurPrimaryPrime), $pred,
                 "cur_primary mismatch"),
             PrimaryInfo = ptag_case(StagLoc, StagGoalMap),
-            map.lookup(PtagCountMap, CurPrimary, CountInfo),
+            map.lookup(PtagCountMap, ptag(CurPrimary), CountInfo),
             CountInfo = StagLocPrime - MaxSecondary,
-            expect(unify(StagLoc, StagLocPrime), $module, $pred,
+            expect(unify(StagLoc, StagLocPrime), $pred,
                 "secondary tag locations differ"),
-            generate_primary_tag_code(StagGoalMap, CurPrimary, [],
+            generate_primary_tag_code(StagGoalMap, ptag(CurPrimary), [],
                 MaxSecondary, StagReg, StagLoc, VarRval, MaybeFailLabel, Code,
                 !CaseLabelMap, !CI)
         ;
             PtagGroups = [_, _ | _],
-            unexpected($module, $pred,
+            unexpected($pred,
                 "caselist not singleton or empty when binary search ends")
         )
-    ;
-        LowRangeEnd = (MinPtag + MaxPtag) // 2,
-        HighRangeStart = LowRangeEnd + 1,
-        InLowGroup = (pred(PtagGroup::in) is semidet :-
-            PtagGroup = ptag_case_entry(Ptag, _),
-            Ptag =< LowRangeEnd
-        ),
+    else
+        LowRangeEnd = (MinPtag + MaxPtag) // 2u8,
+        HighRangeStart = LowRangeEnd + 1u8,
+        InLowGroup =
+            ( pred(PtagGroup::in) is semidet :-
+                PtagGroup = ptag_case_entry(ptag(Ptag), _),
+                Ptag =< LowRangeEnd
+            ),
         list.filter(InLowGroup, PtagGroups, LowGroups, HighGroups),
         get_next_label(NewLabel, !CI),
-        string.int_to_string(MinPtag, LowStartStr),
-        string.int_to_string(LowRangeEnd, LowEndStr),
-        string.int_to_string(HighRangeStart, HighStartStr),
-        string.int_to_string(MaxPtag, HighEndStr),
+        string.uint8_to_string(MinPtag) = LowStartStr,
+        string.uint8_to_string(LowRangeEnd) = LowEndStr,
+        string.uint8_to_string(HighRangeStart) = HighStartStr,
+        string.uint8_to_string(MaxPtag) = HighEndStr,
         IfComment = "fallthrough for ptags " ++
             LowStartStr ++ " to " ++ LowEndStr,
         LabelComment = "code for ptags " ++
             HighStartStr ++ " to " ++ HighEndStr,
-        LowRangeEndConst = const(llconst_int(LowRangeEnd)),
-        TestRval = binop(int_gt, PtagRval, LowRangeEndConst),
+        % XXX ARG_PACK We should do the comparison on uint8s, not ints.
+        LowRangeEndConst = const(llconst_int(uint8.cast_to_int(LowRangeEnd))),
+        TestRval = binop(int_gt(int_type_int), PtagRval, LowRangeEndConst),
         IfCode = singleton(
             llds_instr(if_val(TestRval, code_label(NewLabel)), IfComment)
         ),
@@ -659,7 +680,7 @@ generate_primary_binary_search(PtagGroups, MinPtag, MaxPtag, PtagRval, StagReg,
     % use a jump table to implement the secondary switch.
     %
 :- pred generate_primary_tag_code(stag_goal_map(label)::in,
-    tag_bits::in, list(tag_bits)::in, int::in, lval::in, sectag_locn::in,
+    ptag::in, list(ptag)::in, int::in, lval::in, sectag_locn::in,
     rval::in, maybe(label)::in, llds_code::out,
     case_label_map::in, case_label_map::out,
     code_info::in, code_info::out) is det.
@@ -674,85 +695,90 @@ generate_primary_tag_code(StagGoalMap, MainPtag, OtherPtags, MaxSecondary,
         % There is no secondary tag, so there is no switch on it.
         (
             StagGoalList = [],
-            unexpected($module, $pred, "no goal for non-shared tag")
+            unexpected($pred, "no goal for non-shared tag")
         ;
             StagGoalList = [StagGoal],
-            ( StagGoal = -1 - CaseLabel ->
+            ( if StagGoal = -1 - CaseLabel then
                 generate_case_code_or_jump(CaseLabel, Code, !CaseLabelMap)
-            ;
-                unexpected($module, $pred,
-                    "badly formed goal for non-shared tag")
+            else
+                unexpected($pred, "badly formed goal for non-shared tag")
             )
         ;
             StagGoalList = [_, _ | _],
-            unexpected($module, $pred, "more than one goal for non-shared tag")
+            unexpected($pred, "more than one goal for non-shared tag")
         )
     ;
-        ( StagLoc = sectag_local
-        ; StagLoc = sectag_remote
+        (
+            StagLoc = sectag_remote_word,
+            OrigStagRval = lval(field(yes(MainPtag), Rval,
+                const(llconst_int(0)))),
+            Comment = "compute remote word sec tag to switch on"
+        ;
+            StagLoc = sectag_remote_bits(_NumBits, Mask),
+            StagWordRval = lval(field(yes(MainPtag), Rval,
+                const(llconst_int(0)))),
+            OrigStagRval = binop(bitwise_and(int_type_uint),
+                StagWordRval, const(llconst_uint(Mask))),
+            Comment = "compute remote sec tag bits to switch on"
+        ;
+            StagLoc = sectag_local_rest_of_word,
+            OrigStagRval = unop(unmkbody, Rval),
+            Comment = "compute local rest-of-word sec tag to switch on"
+        ;
+            StagLoc = sectag_local_bits(_NumBits, Mask),
+            OrigStagRval = binop(bitwise_and(int_type_uint),
+                unop(unmkbody, Rval), const(llconst_uint(Mask))),
+            Comment = "compute local sec tag bits to switch on"
         ),
-        expect(unify(OtherPtags, []), $module, $pred,
-            ">1 ptag with secondary tag"),
+        expect(unify(OtherPtags, []), $pred, ">1 ptag with secondary tag"),
 
         % There is a secondary tag, so figure out how to switch on it.
         get_globals(!.CI, Globals),
-        globals.lookup_int_option(Globals, dense_switch_size,
-            DenseSwitchSize),
-        globals.lookup_int_option(Globals, binary_switch_size,
-            BinarySwitchSize),
-        globals.lookup_int_option(Globals, try_switch_size, TrySwitchSize),
-        ( MaxSecondary >= DenseSwitchSize ->
+        globals.get_opt_tuple(Globals, OptTuple),
+        DenseSwitchSize = OptTuple ^ ot_dense_switch_size,
+        TrySwitchSize = OptTuple ^ ot_try_switch_size,
+        BinarySwitchSize = OptTuple ^ ot_binary_switch_size,
+        ( if MaxSecondary >= DenseSwitchSize then
             SecondaryMethod = jump_table
-        ; MaxSecondary >= BinarySwitchSize ->
+        else if MaxSecondary >= BinarySwitchSize then
             SecondaryMethod = binary_search
-        ; MaxSecondary >= TrySwitchSize ->
+        else if MaxSecondary >= TrySwitchSize then
             SecondaryMethod = try_chain
-        ;
+        else
             SecondaryMethod = try_me_else_chain
         ),
 
-        (
-            StagLoc = sectag_remote,
-            OrigStagRval = lval(field(yes(MainPtag), Rval,
-                const(llconst_int(0)))),
-            Comment = "compute remote sec tag to switch on"
-        ;
-            StagLoc = sectag_local,
-            OrigStagRval = unop(unmkbody, Rval),
-            Comment = "compute local sec tag to switch on"
-        ),
-
-        (
+        ( if
             SecondaryMethod \= jump_table,
             MaxSecondary >= 2,
             globals.lookup_int_option(Globals, num_real_r_regs, NumRealRegs),
             (
                 NumRealRegs = 0
             ;
-                ( StagReg = reg(reg_r, StagRegNo) ->
+                ( if StagReg = reg(reg_r, StagRegNo) then
                     StagRegNo =< NumRealRegs
-                ;
-                    unexpected($module, $pred, "improper reg in tag switch")
+                else
+                    unexpected($pred, "improper reg in tag switch")
                 )
             )
-        ->
+        then
             StagCode = singleton(
                 llds_instr(assign(StagReg, OrigStagRval), Comment)
             ),
             StagRval = lval(StagReg)
-        ;
+        else
             StagCode = empty,
             StagRval = OrigStagRval
         ),
         (
             MaybeFailLabel = yes(FailLabel),
-            (
+            ( if
                 list.length(StagGoalList, StagGoalCount),
                 FullGoalCount = MaxSecondary + 1,
                 FullGoalCount = StagGoalCount
-            ->
+            then
                 MaybeSecFailLabel = no
-            ;
+            else
                 MaybeSecFailLabel = yes(FailLabel)
             )
         ;
@@ -795,7 +821,7 @@ generate_primary_tag_code(StagGoalMap, MainPtag, OtherPtags, MaxSecondary,
     code_info::in, code_info::out) is det.
 
 generate_secondary_try_me_else_chain([], _, _, _, !CaseLabelMap, !CI) :-
-    unexpected($module, $pred, "empty switch").
+    unexpected($pred, "empty switch").
 generate_secondary_try_me_else_chain([Case | Cases], StagRval,
         MaybeFailLabel, Code, !CaseLabelMap, !CI) :-
     Case = Secondary - CaseLabel,
@@ -834,7 +860,8 @@ generate_secondary_try_me_else_chain_case(CaseLabel, StagRval, Secondary,
     get_next_label(ElseLabel, !CI),
     TestCode = singleton(
         llds_instr(
-            if_val(binop(ne, StagRval, const(llconst_int(Secondary))),
+            if_val(binop(ne(int_type_int), StagRval,
+                    const(llconst_int(Secondary))),
                 code_label(ElseLabel)),
             "test sec tag only")
     ),
@@ -852,7 +879,7 @@ generate_secondary_try_me_else_chain_case(CaseLabel, StagRval, Secondary,
     case_label_map::in, case_label_map::out) is det.
 
 generate_secondary_try_chain([], _, _, _, _, !CaseLabelMap) :-
-    unexpected($module, $pred, "empty switch").
+    unexpected($pred, "empty switch").
 generate_secondary_try_chain([Case | Cases], StagRval, MaybeFailLabel,
         PrevTestsCode0, Code, !CaseLabelMap) :-
     Case = Secondary - CaseLabel,
@@ -889,7 +916,8 @@ generate_secondary_try_chain_case(CaseLabel, StagRval, Secondary,
     CaseInfo0 = case_label_info(Comment, _CaseCode, _CaseGenerated),
     TestCode = singleton(
         llds_instr(
-            if_val(binop(eq, StagRval, const(llconst_int(Secondary))),
+            if_val(binop(eq(int_type_int), StagRval,
+                    const(llconst_int(Secondary))),
                 code_label(CaseLabel)),
             "test sec tag only for " ++ Comment)
     ),
@@ -905,17 +933,17 @@ generate_secondary_try_chain_case(CaseLabel, StagRval, Secondary,
 
 generate_secondary_jump_table(CaseList, CurSecondary, MaxSecondary,
         MaybeFailLabel, Targets) :-
-    ( CurSecondary > MaxSecondary ->
-        expect(unify(CaseList, []), $module, $pred,
+    ( if CurSecondary > MaxSecondary then
+        expect(unify(CaseList, []), $pred,
             "caselist not empty when reaching limiting secondary tag"),
         Targets = []
-    ;
+    else
         NextSecondary = CurSecondary + 1,
-        ( CaseList = [CurSecondary - CaseLabel | CaseListTail] ->
+        ( if CaseList = [CurSecondary - CaseLabel | CaseListTail] then
             generate_secondary_jump_table(CaseListTail, NextSecondary,
                 MaxSecondary, MaybeFailLabel, OtherTargets),
             Targets = [yes(CaseLabel) | OtherTargets]
-        ;
+        else
             generate_secondary_jump_table(CaseList, NextSecondary,
                 MaxSecondary, MaybeFailLabel, OtherTargets),
             Targets = [MaybeFailLabel | OtherTargets]
@@ -935,7 +963,7 @@ generate_secondary_jump_table(CaseList, CurSecondary, MaxSecondary,
 
 generate_secondary_binary_search(StagGoals, MinStag, MaxStag, StagRval,
         MaybeFailLabel, Code, !CaseLabelMap, !CI) :-
-    ( MinStag = MaxStag ->
+    ( if MinStag = MaxStag then
         CurSec = MinStag,
         (
             StagGoals = [],
@@ -953,15 +981,15 @@ generate_secondary_binary_search(StagGoals, MinStag, MaxStag, StagRval,
             )
         ;
             StagGoals = [CurSecPrime - CaseLabel],
-            expect(unify(CurSec, CurSecPrime), $module, $pred,
+            expect(unify(CurSec, CurSecPrime), $pred,
                 "cur_secondary mismatch"),
             generate_case_code_or_jump(CaseLabel, Code, !CaseLabelMap)
         ;
             StagGoals = [_, _ | _],
-            unexpected($module, $pred,
+            unexpected($pred,
                 "goallist not singleton or empty when binary search ends")
         )
-    ;
+    else
         LowRangeEnd = (MinStag + MaxStag) // 2,
         HighRangeStart = LowRangeEnd + 1,
         InLowGroup = (pred(StagGoal::in) is semidet :-
@@ -979,7 +1007,7 @@ generate_secondary_binary_search(StagGoals, MinStag, MaxStag, StagRval,
         LabelComment = "code for stags " ++
             HighStartStr ++ " to " ++ HighEndStr,
         LowRangeEndConst = const(llconst_int(LowRangeEnd)),
-        TestRval = binop(int_gt, StagRval, LowRangeEndConst),
+        TestRval = binop(int_gt(int_type_int), StagRval, LowRangeEndConst),
         IfCode = singleton(
             llds_instr(if_val(TestRval, code_label(NewLabel)), IfComment)
         ),
@@ -997,21 +1025,25 @@ generate_secondary_binary_search(StagGoals, MinStag, MaxStag, StagRval,
 
 %-----------------------------------------------------------------------------%
 
-:- pred make_ptag_comment(string::in, tag_bits::in, list(tag_bits)::in,
+:- pred make_ptag_comment(string::in, ptag::in, list(ptag)::in,
     string::out) is det.
 
 make_ptag_comment(BaseStr, MainPtag, OtherPtags, Comment) :-
     (
         OtherPtags = [],
-        Comment = BaseStr ++ string.int_to_string(MainPtag)
+        Comment = BaseStr ++ ptag_to_string(MainPtag)
     ;
         OtherPtags = [_ | _],
-        Comment = BaseStr ++ string.int_to_string(MainPtag)
+        Comment = BaseStr ++ ptag_to_string(MainPtag)
             ++ "(shared with " ++
-            string.join_list(", ", list.map(string.int_to_string, OtherPtags))
+            string.join_list(", ", list.map(ptag_to_string, OtherPtags))
             ++ ")"
     ).
 
+:- func ptag_to_string(ptag) = string.
+
+ptag_to_string(ptag(Ptag)) = string.uint8_to_string(Ptag).
+
 %-----------------------------------------------------------------------------%
 :- end_module ll_backend.tag_switch.
 %-----------------------------------------------------------------------------%
