@@ -22,25 +22,24 @@
 %
 % The AR has four kinds of goal:
 %
-% * primitives      - a set of primitive constraints representing the
+% * primitives      - A set of primitive constraints representing the
 %                     abstraction variable size relationships in some
 %                     HLDS goal.
 %
-% * conjunction     - a conjunction of AR goals.
+% * conjunction     - A conjunction of AR goals.
 %
-% * disjunction     - a disjunction of AR goals.
+% * disjunction     - A disjunction of AR goals.
 %
-% * calls           - an abstraction of intra-SCC calls.  Calls to
-%                     procedures lower down the call-graph are abstracted
-%                     as primitive AR goals.
+% * calls           - An abstraction of intra-SCC calls. Calls to procedures
+%                     lower down the call-graph are abstracted as primitive
+%                     AR goals.
 %
-% XXX In order to handle higher-order we need to either modify the
+% XXX In order to handle higher-order, we need to either modify the
 % exiting AR call goal or add a new AR goal type.
 %
 %-----------------------------------------------------------------------------%
 %
-% Mapping the HLDS to the AR
-%
+% Mapping the HLDS to the AR.
 %
 % 1. unification
 %
@@ -53,29 +52,29 @@
 %       { |X| = |A| + |B| + |C| + |f| }
 %
 % where |X| represents the size of the variable X (according to whatever
-% measure we are using).  There will also additional non-negativity
-% constraints on any variables that have non-zero size type.  Variables
-% of that have zero size type are not included at all.  Variables that
-% represent polymorphic types are included.  The code in
-% term_constr_fixpoint.m and term_constr_pass2.m that processes calls is
-% responsible for dealing with the situation where a polymorphic
-% procedure is called with zero sized arguments.
+% measure we are using). There will also additional non-negativity
+% constraints on any variables that have non-zero size type. Variables
+% of that have zero size type are not included at all. Variables that
+% represent polymorphic types are included. The code in term_constr_fixpoint.m
+% and term_constr_pass2.m that processes calls is responsible for dealing with
+% the situation where a polymorphic procedure is called with zero sized
+% arguments.
 %
 % 2. conjunction and parallel conjunction
 %
-% A HLDS conjunction (A, B) is converted to an AR conjunction.  Parallel
+% A HLDS conjunction (A, B) is converted to an AR conjunction. Parallel
 % conjunction is treated the same way.
 %
 % 3. disjunction and switches.
 %
-% A HLDS disjunction (A ; B) is converted to an AR disjunction.  Switches
+% A HLDS disjunction (A ; B) is converted to an AR disjunction. Switches
 % are similar although we also have to add any constraints on the variable
 % being switched on.
 %
 % 4. calls
 %
 % A HLDS call to a procedure lower down the call graph is abstracted as
-% an AR primitive.  A call to something in the same SCC becomes an AR call.
+% an AR primitive. A call to something in the same SCC becomes an AR call.
 %
 % 5. negation.
 %
@@ -90,7 +89,7 @@
 %
 % 8. if-then-else.
 %
-% ( Cond -> Then ; Else ) is abstracted as
+% ( if Cond then Then else Else ) is abstracted as
 %
 %  disj(conj(|Cond|, |Then|), conj(neg(|Cond|), |Else|))
 %
@@ -111,10 +110,13 @@
 
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module libs.
 :- import_module libs.lp_rational.
 :- import_module libs.polyhedron.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module transform_hlds.term_constr_errors.
 
@@ -152,13 +154,13 @@
 
     % The result of the argument size analysis.
     %
-    % NOTE: this is just an indication that everything worked, any
-    % argument size constraint derived will be stored in the
+    % NOTE: this is just an indication that everything worked,
+    % any argument size constraint derived will be stored in the
     % termination2_info structure.
     %
 :- type arg_size_result
     --->    arg_size_ok
-    ;       arg_size_error(term2_errors).
+    ;       arg_size_error(list(term2_error)).
 
 %-----------------------------------------------------------------------------%
 %
@@ -170,7 +172,7 @@
 % about the SCC; currently the relevant information is just duplicated
 % amongst the abstract procs.
 
-:- type abstract_scc == list(abstract_proc).
+:- type abstract_scc == set(abstract_proc).
 
     % XXX This will need to be extended in order to handle HO calls and
     % intermodule mutual recursion.
@@ -192,9 +194,6 @@
                 % The procedure that this is an abstraction of.
                 ap_ppid             :: abstract_ppid,
 
-                % Is this procedure called from outside the SCC?
-                ap_is_entry         :: bool,
-
                 % The context of the procedure.
                 ap_context          :: prog_context,
 
@@ -218,6 +217,9 @@
                 % The size_vars that have zero size.
                 ap_zeros            :: zero_vars,
 
+                % Is this procedure called from outside the SCC?
+                ap_is_entry         :: bool,
+
                 % The type of recursion present in the procedure.
                 ap_recursion        :: recursion_type,
 
@@ -230,18 +232,18 @@
                 ap_ho_calls         :: list(abstract_ho_call)
             ).
 
-    % This is like an error message (and is treated as such
-    % at the moment).  It's here because we want to treat information
-    % regarding higher-order constructs differently from other errors.
-    % In particular higher-order constructs will not always be errors
-    % (ie. when we can analyse them properly).
+    % This is like an error message (and is treated as such at the moment).
+    % It is here because we want to treat information regarding higher-order
+    % constructs differently from other errors. In particular, higher-order
+    % constructs will not always be errors (i.e. when we can analyse
+    % them properly).
     %
 :- type abstract_ho_call
     --->    ho_call(prog_context).
 
-    % NOTE: the AR's notion of local/non-local variables may not
-    % correspond directly to that in the HLDS because of various
-    % transformations performed on the the AR.
+    % NOTE: the AR's notion of local/non-local variables may not correspond
+    % directly to that in the HLDS because of various transformations
+    % performed on the AR.
     %
 :- type local_vars == size_vars.
 :- type nonlocal_vars == size_vars.
@@ -260,10 +262,9 @@
 :- type abstract_goal
     --->    term_disj(
                 disj_goals     :: list(abstract_goal),
+                % We keep track of the number of disjuncts for use
+                % in heuristics that may speed up the convex hull calculation.
                 disj_size      :: int,
-                        % We keep track of the number of disjuncts for use
-                        % in heuristics that may speed up the convex hull
-                        % calculation.
 
                 disj_locals    :: local_vars,
                 disj_nonlocals :: nonlocal_vars
@@ -314,25 +315,6 @@
 :- func update_local_and_nonlocal_vars(abstract_goal, local_vars,
     nonlocal_vars) = abstract_goal.
 
-    % For any two goals whose recursion types are known return the
-    % recursion type of the conjunction of the two goals.
-    %
-:- func combine_recursion_types(recursion_type, recursion_type)
-    = recursion_type.
-
-    % Combines the constraints contained in two primitive goals
-    % into a single primitive goal.  It is an error to pass
-    % any other kind of abstract goal as an argument to this
-    % function.
-    %
-:- func combine_primitive_goals(abstract_goal, abstract_goal) = abstract_goal.
-
-    % Take a list of conjoined primitive goals and simplify them
-    % so there is one large block of constraints.
-    %
-:- func simplify_abstract_rep(abstract_goal) = abstract_goal.
-:- func simplify_conjuncts(list(abstract_goal)) = list(abstract_goal).
-
     % Succeeds iff the given SCC contains recursion.
     %
 :- pred scc_contains_recursion(abstract_scc::in) is semidet.
@@ -346,11 +328,29 @@
     %
 :- func size_varset_from_abstract_scc(abstract_scc) = size_varset.
 
-    % Succeeds iff the results of the analysis depend upon the
-    % values of some higher-order variables.
+    % Succeeds iff the results of the analysis depend upon the values
+    % of some higher-order variables.
     %
 :- pred analysis_depends_on_ho(abstract_proc::in) is semidet.
 
+    % For any two goals whose recursion types are known return the
+    % recursion type of the conjunction of the two goals.
+    %
+:- func combine_recursion_types(recursion_type, recursion_type)
+    = recursion_type.
+
+    % Combines the constraints contained in two primitive goals into
+    % a single primitive goal. It is an error to pass any other kind
+    % of abstract goal as an argument to this function.
+    %
+:- func combine_primitive_goals(abstract_goal, abstract_goal) = abstract_goal.
+
+    % Take a list of conjoined primitive goals and simplify them
+    % so there is one large block of constraints.
+    %
+:- func simplify_abstract_rep(abstract_goal) = abstract_goal.
+:- func simplify_conjuncts(list(abstract_goal)) = list(abstract_goal).
+
 %-----------------------------------------------------------------------------%
 %
 % Predicates for printing out debugging traces, etc.
@@ -358,33 +358,31 @@
 
     % Dump a representation of the AR to stdout.
     %
-:- pred dump_abstract_scc(module_info::in, abstract_scc::in, io::di,
-    io::uo) is det.
+:- pred dump_abstract_scc(io.text_output_stream::in, module_info::in,
+    abstract_scc::in, io::di, io::uo) is det.
 
-    % As above.  The extra argument specifies the indentation level.
+    % As above. The extra argument specifies the indentation level.
     %
-:- pred dump_abstract_scc(module_info::in, int::in, abstract_scc::in, io::di,
-    io::uo) is det.
+:- pred dump_abstract_scc(io.text_output_stream::in, module_info::in,
+    int::in, abstract_scc::in, io::di, io::uo) is det.
 
     % Write an abstract_proc to stdout.
     %
-:- pred dump_abstract_proc(module_info::in, int::in, abstract_proc::in,
-    io::di, io::uo) is det.
+:- pred dump_abstract_proc(io.text_output_stream::in, module_info::in,
+    int::in, abstract_proc::in, io::di, io::uo) is det.
 
     % Write an abstract_goal to stdout.
     %
-:- pred dump_abstract_goal(module_info::in, size_varset::in, int::in,
-    abstract_goal::in, io::di, io::uo) is det.
+:- pred dump_abstract_goal(io.text_output_stream::in, module_info::in,
+    size_varset::in, int::in, abstract_goal::in, io::di, io::uo) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
-:- import_module hlds.hlds_pred.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
-:- import_module parse_tree.prog_data.
 
 :- import_module int.
 :- import_module require.
@@ -423,28 +421,67 @@ update_local_and_nonlocal_vars(Goal0, Locals0, NonLocals0) = Goal :-
         Goal      = term_primitive(Polyhedron, Locals, NonLocals)
     ).
 
-scc_contains_recursion([]) :-
-    unexpected($module, $pred, "empty SCC").
-scc_contains_recursion([Proc | _]) :-
-    Proc ^ ap_recursion \= none.
+scc_contains_recursion(SCC) :-
+    ( if set.remove_least(Proc, SCC, _) then
+        Proc ^ ap_recursion \= none
+    else
+        unexpected($pred, "empty SCC")
+    ).
 
 proc_is_recursive(Proc) :-
     not Proc ^ ap_recursion = none.
 
-size_varset_from_abstract_scc([]) = _ :-
-    unexpected($module, $pred, "empty SCC").
-size_varset_from_abstract_scc([Proc | _]) = Proc ^ ap_size_varset.
+size_varset_from_abstract_scc(SCC) = SizeVarSet :-
+    ( if set.remove_least(Proc, SCC, _) then
+        SizeVarSet = Proc ^ ap_size_varset
+    else
+        unexpected($pred, "empty SCC")
+    ).
 
 analysis_depends_on_ho(Proc) :-
     list.is_not_empty(Proc ^ ap_ho_calls).
 
+%-----------------------------------------------------------------------------%
+%
+% Code for dealing with different types of recursion.
+%
+
+combine_recursion_types(none,        none)        = none.
+combine_recursion_types(none,        direct_only) = direct_only.
+combine_recursion_types(none,        mutual_only) = mutual_only.
+combine_recursion_types(none,        both)        = both.
+combine_recursion_types(direct_only, none)        = direct_only.
+combine_recursion_types(direct_only, direct_only) = direct_only.
+combine_recursion_types(direct_only, mutual_only) = both.
+combine_recursion_types(direct_only, both)        = both.
+combine_recursion_types(mutual_only, none)        = mutual_only.
+combine_recursion_types(mutual_only, direct_only) = both.
+combine_recursion_types(mutual_only, mutual_only) = mutual_only.
+combine_recursion_types(mutual_only, both)        = both.
+combine_recursion_types(both,        none)        = both.
+combine_recursion_types(both,        direct_only) = both.
+combine_recursion_types(both,        mutual_only) = both.
+combine_recursion_types(both,        both)        = both.
+
+combine_primitive_goals(GoalA, GoalB) = Goal :-
+    ( if
+        GoalA = term_primitive(PolyA, LocalsA, NonLocalsA),
+        GoalB = term_primitive(PolyB, LocalsB, NonLocalsB)
+    then
+        Poly      = polyhedron.intersection(PolyA, PolyB),
+        Locals    = LocalsA ++ LocalsB,
+        NonLocals = NonLocalsA ++ NonLocalsB,
+        Goal      = term_primitive(Poly, Locals, NonLocals)
+    else
+        unexpected($pred, "non-primitive goals")
+    ).
+
 %-----------------------------------------------------------------------------%
 %
 % Code for simplifying the abstract representation.
 %
 
-% XXX We should keep running the simplifications until we arrive at a
-% fixpoint.
+% XXX We should keep running the simplifications until we arrive at a fixpoint.
 
 simplify_abstract_rep(Goal0) = Goal :-
     simplify_abstract_rep(Goal0, Goal).
@@ -480,12 +517,12 @@ simplify_abstract_rep(Goal0, Goal) :-
             list.filter(isnt(is_empty_conj), !Conjuncts),
             Conjuncts = !.Conjuncts
         ),
-        ( Conjuncts = [Conjunct] ->
+        ( if Conjuncts = [Conjunct] then
             % The local/non-local var sets need to be updated for similar
             % reasons as we do with disjunctions.
             Goal = update_local_and_nonlocal_vars(Conjunct, Locals,
                 NonLocals)
-        ;
+        else
             Goal = term_conj(Conjuncts, Locals, NonLocals)
         )
     ;
@@ -504,16 +541,15 @@ simplify_abstract_rep(Goal0, Goal) :-
     %
     %   where Px is a primitive goal and NPx is a non-primitive
     %
-    %   then simplify this to:
+    % then simplify this to:
     %
     %   [ ( P1 /\ P2 /\ P3), NP1, NP2, ( P4 /\ P5), NP3, (P6 /\ P7) ]
     %
     %   where `/\' is the intersection of the primitive goals.
     %
     % Note: because intersection is commutative we could go further
-    % and take the intersection of all the primitive goals in a
-    % conjunction but that unnecessarily increases the size of the edge
-    % labels in pass 2.
+    % and take the intersection of all the primitive goals in a conjunction,
+    % but that unnecessarily increases the size of the edge labels in pass 2.
     %
 :- pred flatten_conjuncts(list(abstract_goal)::in, list(abstract_goal)::out)
     is det.
@@ -529,8 +565,8 @@ flatten_conjuncts(Goals0 @ [_, _ | _], Goals) :-
 
 flatten_conjuncts_2([], !RevGoals).
 flatten_conjuncts_2([Goal0 | Goals0], !RevGoals) :-
-    ( Goal0 = term_primitive(_, _, _) ->
-        list.takewhile(is_primitive, Goals0, Primitives, NextNonPrimitive),
+    ( if Goal0 = term_primitive(_, _, _) then
+        list.take_while(is_primitive, Goals0, Primitives, NextNonPrimitive),
         (
             Primitives = [],
             NewPrimitive = Goal0
@@ -539,7 +575,7 @@ flatten_conjuncts_2([Goal0 | Goals0], !RevGoals) :-
             NewPrimitive = list.foldl(combine_primitives, Primitives, Goal0)
         ),
         list.cons(NewPrimitive, !RevGoals)
-    ;
+    else
         list.cons(Goal0, !RevGoals),
         NextNonPrimitive = Goals0
     ),
@@ -554,16 +590,16 @@ is_primitive(term_primitive(_, _, _)).
 :- func combine_primitives(abstract_goal, abstract_goal) = abstract_goal.
 
 combine_primitives(GoalA, GoalB) = Goal :-
-    (
+    ( if
         GoalA = term_primitive(PolyA, LocalsA, NonLocalsA),
         GoalB = term_primitive(PolyB, LocalsB, NonLocalsB)
-    ->
+    then
         Poly = polyhedron.intersection(PolyA, PolyB),
         Locals = LocalsA ++ LocalsB,
         NonLocals = NonLocalsA ++ NonLocalsB,
         Goal = term_primitive(Poly, Locals, NonLocals)
-    ;
-        unexpected($module, $pred, "non-primitive goals")
+    else
+        unexpected($pred, "non-primitive goals")
     ).
 
     % We end up with `empty' primitives by abstracting unifications
@@ -581,160 +617,6 @@ is_empty_primitive(term_primitive(Poly, _, _)) :-
 
 is_empty_conj(term_conj([], _, _)).
 
-    % We end up with `empty' disjunctions by abstracting disjunctions
-    % that involve variables that have zero size.
-    %
-:- pred is_empty_disj(abstract_goal::in) is semidet.
-
-is_empty_disj(term_disj([], _, _, _)).
-
-%-----------------------------------------------------------------------------%
-%
-% Code for dealing with different types of recursion.
-%
-
-combine_recursion_types(none,        none)        = none.
-combine_recursion_types(none,        direct_only) = direct_only.
-combine_recursion_types(none,        mutual_only) = mutual_only.
-combine_recursion_types(none,        both)        = both.
-combine_recursion_types(direct_only, none)        = direct_only.
-combine_recursion_types(direct_only, direct_only) = direct_only.
-combine_recursion_types(direct_only, mutual_only) = both.
-combine_recursion_types(direct_only, both)        = both.
-combine_recursion_types(mutual_only, none)        = mutual_only.
-combine_recursion_types(mutual_only, direct_only) = both.
-combine_recursion_types(mutual_only, mutual_only) = mutual_only.
-combine_recursion_types(mutual_only, both)        = both.
-combine_recursion_types(both,        none)        = both.
-combine_recursion_types(both,        direct_only) = both.
-combine_recursion_types(both,        mutual_only) = both.
-combine_recursion_types(both,        both)        = both.
-
-combine_primitive_goals(GoalA, GoalB) = Goal :-
-    (
-        GoalA = term_primitive(PolyA, LocalsA, NonLocalsA),
-        GoalB = term_primitive(PolyB, LocalsB, NonLocalsB)
-    ->
-        Poly      = polyhedron.intersection(PolyA, PolyB),
-        Locals    = LocalsA ++ LocalsB,
-        NonLocals = NonLocalsA ++ NonLocalsB,
-        Goal      = term_primitive(Poly, Locals, NonLocals)
-    ;
-        unexpected($module, $pred, "non-primitive goals")
-    ).
-
-%-----------------------------------------------------------------------------%
-%
-% Predicates for printing out the abstract data structure.
-% (These are for debugging only)
-%
-
-dump_abstract_scc(ModuleInfo, SCC, !IO) :-
-    dump_abstract_scc(ModuleInfo, 0, SCC, !IO).
-
-dump_abstract_scc(ModuleInfo, Indent, SCC, !IO) :-
-    list.foldl(dump_abstract_proc(ModuleInfo, Indent), SCC, !IO).
-
-dump_abstract_proc(ModuleInfo, Indent, Proc, !IO) :-
-    AbstractPPId = Proc ^ ap_ppid,
-    HeadVars = Proc ^ ap_head_vars,
-    Body = Proc ^ ap_body,
-    SizeVarSet = Proc ^ ap_size_varset,
-    indent_line(Indent, !IO),
-    AbstractPPId = real(PPId),
-    write_pred_proc_id(ModuleInfo, PPId, !IO),
-    io.write_string(" : [", !IO),
-    WriteHeadVars = (pred(Var::in, !.IO::di, !:IO::uo) is det :-
-        varset.lookup_name(SizeVarSet, Var, VarName),
-        io.format(VarName ++ "[%d]", [i(term.var_id(Var))], !IO)
-    ),
-    io.write_list(HeadVars, ", ", WriteHeadVars, !IO),
-    io.write_string(" ] :- \n", !IO),
-    dump_abstract_goal(ModuleInfo, SizeVarSet, Indent + 1, Body, !IO).
-
-:- func recursion_type_to_string(recursion_type) = string.
-
-recursion_type_to_string(none)        = "none".
-recursion_type_to_string(direct_only) = "direct recursion only".
-recursion_type_to_string(mutual_only) = "mutual recursion only".
-recursion_type_to_string(both)        = "mutual and direct recursion".
-
-:- pred dump_abstract_disjuncts(module_info::in, size_varset::in, int::in,
-    list(abstract_goal)::in, io::di, io::uo) is det.
-
-dump_abstract_disjuncts(_, _, _, [], !IO).
-dump_abstract_disjuncts(ModuleInfo, VarSet, Indent, [Goal | Goals], !IO) :-
-    dump_abstract_goal(ModuleInfo, VarSet, Indent + 1, Goal, !IO),
-    (
-        Goals = [_ | _],
-        indent_line(Indent, !IO),
-        io.write_string(";\n", !IO)
-    ;
-        Goals = []
-    ),
-    dump_abstract_disjuncts(ModuleInfo, VarSet, Indent, Goals, !IO).
-
-dump_abstract_goal(ModuleInfo, VarSet, Indent, AbstractGoal, !IO) :-
-    (
-        AbstractGoal = term_disj(Goals, Size, Locals, NonLocals),
-        indent_line(Indent, !IO),
-        io.format("disj[%d](\n", [i(Size)], !IO),
-        dump_abstract_disjuncts(ModuleInfo, VarSet, Indent, Goals, !IO),
-        indent_line(Indent, !IO),
-        io.write_string(" Locals: ", !IO),
-        io.write_list(Locals, ", ", dump_var_name(VarSet), !IO),
-        io.nl(!IO),
-        indent_line(Indent, !IO),
-        io.write_string(" Non-Locals: ", !IO),
-        io.write_list(NonLocals, ", ", dump_var_name(VarSet), !IO),
-        io.nl(!IO),
-        indent_line(Indent, !IO),
-        io.write_string(")\n", !IO)
-    ;
-        AbstractGoal = term_conj(Goals, Locals, NonLocals),
-        indent_line(Indent, !IO),
-        io.write_string("conj(\n", !IO),
-        list.foldl(dump_abstract_goal(ModuleInfo, VarSet, Indent + 1), Goals,
-            !IO),
-        indent_line(Indent, !IO),
-        io.write_string(" Locals: ", !IO),
-        io.write_list(Locals, ", ", dump_var_name(VarSet), !IO),
-        io.nl(!IO),
-        indent_line(Indent, !IO),
-        io.write_string(" Non-Locals: ", !IO),
-        io.write_list(NonLocals, ", ", dump_var_name(VarSet), !IO),
-        io.nl(!IO),
-        indent_line(Indent, !IO),
-        io.write_string(")\n", !IO)
-    ;
-        AbstractGoal = term_call(PPId0, _, CallVars, _, _, _, CallPoly),
-        indent_line(Indent, !IO),
-        io.write_string("call: ", !IO),
-        PPId0 = real(PPId),
-        write_pred_proc_id(ModuleInfo, PPId, !IO),
-        io.write_string(" : [", !IO),
-        io.write_list(CallVars, ", ", dump_var_name(VarSet), !IO),
-        io.write_string("]\n", !IO),
-        indent_line(Indent, !IO),
-        io.write_string("Other call constraints:[\n", !IO),
-        polyhedron.write_polyhedron(CallPoly, VarSet, !IO),
-        indent_line(Indent, !IO),
-        io.write_string("]\n", !IO)
-    ;
-        AbstractGoal = term_primitive(Poly, _, _),
-        indent_line(Indent, !IO),
-        io.write_string("[\n", !IO),
-        polyhedron.write_polyhedron(Poly, VarSet, !IO),
-        indent_line(Indent, !IO),
-        io.write_string("]\n", !IO)
-    ).
-
-:- pred dump_var_name(size_varset::in, size_var::in, io::di, io::uo) is det.
-
-dump_var_name(VarSet, Var, !IO) :-
-    varset.lookup_name(VarSet, Var, VarName),
-    io.write_string(VarName, !IO).
-
 %-----------------------------------------------------------------------------%
 %
 % Predicates for simplifying conjuncts.
@@ -759,33 +641,142 @@ simplify_conjuncts(Goals0, Goals) :-
         % If the list of conjuncts starts with two primitives
         % join them together into a single primitive.
         Goals0 = [GoalA, GoalB | OtherGoals],
-        (
+        ( if
             GoalA = term_primitive(PolyA,  LocalsA, NonLocalsA),
             GoalB = term_primitive(PolyB,  LocalsB, NonLocalsB)
-        ->
+        then
             Poly = polyhedron.intersection(PolyA, PolyB),
             Locals = LocalsA ++ LocalsB,
             NonLocals = NonLocalsA ++ NonLocalsB,
             Goal = term_primitive(Poly, Locals, NonLocals),
             Goals1 = [Goal | OtherGoals],
             simplify_conjuncts(Goals1, Goals)
-        ;
+        else
             Goals = Goals0
         )
     ).
 
 %-----------------------------------------------------------------------------%
 %
-% Utility predicates.
+% Predicates for printing out the abstract data structure.
+% (These are for debugging only.)
 %
 
-:- pred indent_line(int::in, io::di, io::uo) is det.
+dump_abstract_scc(Stream, ModuleInfo, SCC, !IO) :-
+    dump_abstract_scc(Stream, ModuleInfo, 0, SCC, !IO).
 
-indent_line(N, !IO) :-
-    ( N > 0 ->
-        io.write_string("  ", !IO),
-        indent_line(N - 1, !IO)
+dump_abstract_scc(Stream, ModuleInfo, Indent, SCC, !IO) :-
+    set.foldl(dump_abstract_proc(Stream, ModuleInfo, Indent), SCC, !IO).
+
+dump_abstract_proc(Stream, ModuleInfo, Indent, Proc, !IO) :-
+    AbstractPPId = Proc ^ ap_ppid,
+    HeadVars = Proc ^ ap_head_vars,
+    Body = Proc ^ ap_body,
+    SizeVarSet = Proc ^ ap_size_varset,
+    AbstractPPId = real(PPId),
+    PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+    HeadVarSizeStrs = list.map(size_var_to_string(SizeVarSet), HeadVars),
+    HeadVarSizesStr = string.join_list(", ", HeadVarSizeStrs),
+    indent_line(Stream, Indent, !IO),
+    io.format(Stream, "%s : [ %s ] :- \n",
+        [s(PPIdStr), s(HeadVarSizesStr)], !IO),
+    dump_abstract_goal(Stream, ModuleInfo, SizeVarSet, Indent + 1, Body, !IO).
+
+:- func size_var_to_string(size_varset, size_var) = string.
+
+size_var_to_string(SizeVarSet, Var) = Str :-
+    varset.lookup_name(SizeVarSet, Var, VarName),
+    Str = string.format("%s[%d]", [s(VarName), i(term.var_to_int(Var))]).
+
+:- func recursion_type_to_string(recursion_type) = string.
+:- pragma consider_used(func(recursion_type_to_string/1)).
+
+recursion_type_to_string(none)        = "none".
+recursion_type_to_string(direct_only) = "direct recursion only".
+recursion_type_to_string(mutual_only) = "mutual recursion only".
+recursion_type_to_string(both)        = "mutual and direct recursion".
+
+:- pred dump_abstract_disjuncts(io.text_output_stream::in, module_info::in,
+    size_varset::in, int::in, list(abstract_goal)::in, io::di, io::uo) is det.
+
+dump_abstract_disjuncts(_, _, _, _, [], !IO).
+dump_abstract_disjuncts(Stream, ModuleInfo, VarSet, Indent,
+        [Goal | Goals], !IO) :-
+    dump_abstract_goal(Stream, ModuleInfo, VarSet, Indent + 1, Goal, !IO),
+    (
+        Goals = [_ | _],
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, ";\n", !IO)
     ;
+        Goals = []
+    ),
+    dump_abstract_disjuncts(Stream, ModuleInfo, VarSet, Indent, Goals, !IO).
+
+dump_abstract_goal(Stream, ModuleInfo, VarSet, Indent, AbstractGoal, !IO) :-
+    (
+        AbstractGoal = term_disj(Goals, Size, Locals, NonLocals),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, "disj[%d](\n", [i(Size)], !IO),
+        dump_abstract_disjuncts(Stream, ModuleInfo, VarSet, Indent,
+            Goals, !IO),
+        LocalVarNamesStr = var_names_to_string(VarSet, Locals),
+        NonLocalVarNamesStr = var_names_to_string(VarSet, NonLocals),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, " Locals: %s\n", [s(LocalVarNamesStr)], !IO),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, " Non-Locals: %s\n", [s(NonLocalVarNamesStr)], !IO),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, ")\n", !IO)
+    ;
+        AbstractGoal = term_conj(Goals, Locals, NonLocals),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, "conj(\n", !IO),
+        list.foldl(
+            dump_abstract_goal(Stream, ModuleInfo, VarSet, Indent + 1),
+            Goals, !IO),
+        LocalVarNamesStr = var_names_to_string(VarSet, Locals),
+        NonLocalVarNamesStr = var_names_to_string(VarSet, NonLocals),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, " Locals: %s\n", [s(LocalVarNamesStr)], !IO),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, " Non-Locals: %s\n", [s(NonLocalVarNamesStr)], !IO),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, ")\n", !IO)
+    ;
+        AbstractGoal = term_call(PPId0, _, CallVars, _, _, _, CallPoly),
+        PPId0 = real(PPId),
+        PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+        CallVarNamesStr = var_names_to_string(VarSet, CallVars),
+        indent_line(Stream, Indent, !IO),
+        io.format(Stream, "call: %s : [%s]\n",
+            [s(PPIdStr), s(CallVarNamesStr)], !IO),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, "Other call constraints:[\n", !IO),
+        polyhedron.write_polyhedron(Stream, VarSet, CallPoly, !IO),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, "]\n", !IO)
+    ;
+        AbstractGoal = term_primitive(Poly, _, _),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, "[\n", !IO),
+        polyhedron.write_polyhedron(Stream, VarSet, Poly, !IO),
+        indent_line(Stream, Indent, !IO),
+        io.write_string(Stream, "]\n", !IO)
+    ).
+
+:- func var_names_to_string(size_varset, list(size_var)) = string.
+
+var_names_to_string(VarSet, Vars) = Str :-
+    list.map(varset.lookup_name(VarSet), Vars, VarNames),
+    Str = string.join_list(", ", VarNames).
+
+:- pred indent_line(io.text_output_stream::in, int::in, io::di, io::uo) is det.
+
+indent_line(Stream, N, !IO) :-
+    ( if N > 0 then
+        io.write_string(Stream, "  ", !IO),
+        indent_line(Stream, N - 1, !IO)
+    else
         true
     ).
 
