@@ -21,19 +21,22 @@
 :- module transform_hlds.term_constr_fixpoint.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 :- import_module transform_hlds.term_constr_data.
 :- import_module transform_hlds.term_constr_errors.
 
 :- import_module list.
+:- import_module set.
 
 %-----------------------------------------------------------------------------%
 
-    % Derive the argument size constraints for the procedures in this SCC.
+    % Derive the argument size constraints for the procedures in this SCC
+    % that need them.
     %
-:- pred do_fixpoint_calculation(fixpoint_options::in, list(pred_proc_id)::in,
-    int::in, term2_errors::out, module_info::in, module_info::out) is det.
+:- pred do_fixpoint_calculation(fixpoint_options::in, set(pred_proc_id)::in,
+    int::in, list(term2_error)::out, module_info::in, module_info::out) is det.
 
     % This structure holds the values of options used to control
     % the fixpoint calculation.
@@ -56,13 +59,13 @@
 
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
+:- import_module hlds.passes_aux.
+:- import_module libs.
 :- import_module libs.globals.
 :- import_module libs.lp_rational.
 :- import_module libs.options.
 :- import_module libs.polyhedron.
-:- import_module parse_tree.prog_data.
-:- import_module transform_hlds.term_constr_data.
-:- import_module transform_hlds.term_constr_main.
+:- import_module transform_hlds.term_constr_main_types.
 :- import_module transform_hlds.term_constr_util.
 
 :- import_module bool.
@@ -70,22 +73,10 @@
 :- import_module io.
 :- import_module maybe.
 :- import_module require.
-:- import_module set.
 :- import_module string.
 :- import_module term.
 :- import_module varset.
 
-%-----------------------------------------------------------------------------%
-
-:- type fixpoint_options
-    --->    fixpoint_options(
-                fo_widening :: widening,
-                fo_max_size :: int
-            ).
-
-fixpoint_options_init(Widening, MaxMatrixSize) =
-    fixpoint_options(Widening, MaxMatrixSize).
-
 %-----------------------------------------------------------------------------%
 %
 % Perform the fixpoint calculation on the AR.
@@ -112,8 +103,8 @@ do_fixpoint_calculation(Options, SCC, Iteration, [], !ModuleInfo) :-
     % the build phase for non-recursive procedures (and in fact used to)
     % but the code ends up being a horrible mess.
     %
-    list.foldl(
-        term_iterate_over_abstract_proc(Iteration, Options, !.ModuleInfo),
+    set.foldl(
+        term_iterate_over_abstract_proc(!.ModuleInfo, Options, Iteration),
         AbstractSCC, [], IterationInfos),
     ChangeFlag = or_flags(IterationInfos),
     (
@@ -129,17 +120,18 @@ do_fixpoint_calculation(Options, SCC, Iteration, [], !ModuleInfo) :-
         % in the SCC to `true'.
         % XXX Should this be happening?
         %
-        (
+        ( if
             list.member(OneInfo, IterationInfos),
             polyhedron.is_empty(OneInfo ^ ii_arg_size_poly)
-        ->
-            ChangePoly = (func(Info0) = Info :-
-                Identity = polyhedron.universe,
-                Info = Info0 ^ ii_arg_size_poly := Identity
-            ),
+        then
+            ChangePoly =
+                ( func(Info0) = Info :-
+                    Identity = polyhedron.universe,
+                    Info = Info0 ^ ii_arg_size_poly := Identity
+                ),
             list.foldl(update_size_info, list.map(ChangePoly, IterationInfos),
                 !ModuleInfo)
-        ;
+        else
             list.foldl(update_size_info, IterationInfos, !ModuleInfo)
         )
     ).
@@ -158,11 +150,11 @@ update_size_info(Info, !ModuleInfo) :-
 
 %-----------------------------------------------------------------------------%
 
-:- pred term_iterate_over_abstract_proc(int::in, fixpoint_options::in,
-    module_info::in, abstract_proc::in,
+:- pred term_iterate_over_abstract_proc(module_info::in, fixpoint_options::in,
+    int::in, abstract_proc::in,
     iteration_infos::in, iteration_infos::out) is det.
 
-term_iterate_over_abstract_proc(Iteration, Options, ModuleInfo, Proc,
+term_iterate_over_abstract_proc(ModuleInfo, Options, Iteration, Proc,
         !IterationInfo) :-
     WideningInfo = Options ^ fo_widening,
     MaxMatrixSize = Options ^ fo_max_size,
@@ -178,13 +170,13 @@ term_iterate_over_abstract_proc(Iteration, Options, ModuleInfo, Proc,
     (
         DebugTerm = yes,
         trace [io(!IO)] (
-            io.write(PPId, !IO),
-            io.write_string(": ", !IO),
-            write_pred_proc_id(ModuleInfo, PPId, !IO),
-            io.write_string(" ", !IO),
-            write_size_vars(SizeVarSet, HeadVars, !IO),
-            io.format("\nIteration %d:\n", [i(Iteration)], !IO),
-            io.flush_output(!IO)
+            get_debug_output_stream(ModuleInfo, DebugStream, !IO),
+            PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+            io.write(DebugStream, PPId, !IO),
+            io.format(DebugStream, ": %s ", [s(PPIdStr)], !IO),
+            write_size_vars(DebugStream, SizeVarSet, HeadVars, !IO),
+            io.format(DebugStream, "\nIteration %d:\n", [i(Iteration)], !IO),
+            io.flush_output(DebugStream, !IO)
         )
     ;
         DebugTerm = no
@@ -206,10 +198,10 @@ term_iterate_over_abstract_proc(Iteration, Options, ModuleInfo, Proc,
         % (See comments in term_constr_build.m).
         BugConstrs0 = polyhedron.constraints(!.Polyhedron),
         ConstrVarsSet = get_vars_from_constraints(BugConstrs0),
-        HeadVarSet = set.from_list(HeadVars),
+        HeadVarSet = set.list_to_set(HeadVars),
         BadVarsSet = set.difference(ConstrVarsSet, HeadVarSet),
         BadVars = set.to_sorted_list(BadVarsSet),
-        !:Polyhedron = polyhedron.project(BadVars, SizeVarSet, !.Polyhedron),
+        polyhedron.project_polyhedron(SizeVarSet, BadVars, !Polyhedron),
         polyhedron.optimize(SizeVarSet, !Polyhedron),
         % XXX End of bug workaround.
 
@@ -217,9 +209,11 @@ term_iterate_over_abstract_proc(Iteration, Options, ModuleInfo, Proc,
         (
             DebugTerm = yes,
             trace [io(!IO)] (
-                polyhedron.write_polyhedron(!.Polyhedron, SizeVarSet, !IO),
-                io.nl(!IO),
-                io.flush_output(!IO)
+                get_debug_output_stream(ModuleInfo, DebugStream, !IO),
+                polyhedron.write_polyhedron(DebugStream, SizeVarSet,
+                    !.Polyhedron, !IO),
+                io.nl(DebugStream, !IO),
+                io.flush_output(DebugStream, !IO)
             )
         ;
             DebugTerm = no
@@ -241,21 +235,21 @@ term_iterate_over_abstract_proc(Iteration, Options, ModuleInfo, Proc,
             ArgSizeInfo = yes(SizeInfo),
             OldPolyhedron = SizeInfo
         ),
-        ( polyhedron.is_empty(!.Polyhedron) ->
-            ( polyhedron.is_empty(OldPolyhedron) ->
+        ( if polyhedron.is_empty(!.Polyhedron) then
+            ( if polyhedron.is_empty(OldPolyhedron) then
                 ChangeFlag = no
-            ;
-                unexpected($module, $pred, "old polyhedron is empty")
+            else
+                unexpected($pred, "old polyhedron is empty")
             )
-        ;
+        else
             % If the procedure is not recursive then we need only perform one
             % pass over the AR - subsequent iterations will yield the same
             % result.
-            ( Proc ^ ap_recursion = none ->
+            ( if Proc ^ ap_recursion = none then
                 ChangeFlag = no
-            ; polyhedron.is_empty(OldPolyhedron) ->
+            else if polyhedron.is_empty(OldPolyhedron) then
                 ChangeFlag = yes
-            ;
+            else
                 test_fixpoint_and_perhaps_widen(WideningInfo, SizeVarSet,
                    Iteration, OldPolyhedron, !Polyhedron, ChangeFlag)
             )
@@ -322,31 +316,29 @@ term_traverse_abstract_goal(Info, Goal, !Polyhedron) :-
         module_info_pred_proc_info(Info ^ tcfi_module_info, CallPPId, _,
             CallProcInfo),
         proc_info_get_termination2_info(CallProcInfo, CallTerm2Info),
-        CallArgSizeInfo = CallTerm2Info ^ success_constrs,
+        CallArgSizeInfo = term2_info_get_success_constrs(CallTerm2Info),
         (
             CallArgSizeInfo = no,
             !:Polyhedron = polyhedron.empty
         ;
             CallArgSizeInfo = yes(SizeInfo),
-            ( polyhedron.is_empty(SizeInfo) ->
+            ( if polyhedron.is_empty(SizeInfo) then
                 !:Polyhedron = polyhedron.empty
-            ;
-                ( polyhedron.is_universe(SizeInfo) ->
-                    true
-                    % Constraint store += true
-                ;
-                    HeadVars = CallTerm2Info ^ head_vars,
-                    SubstMap = create_var_substitution(CallVars, HeadVars),
-                    Polyhedron0 = polyhedron.substitute_vars(SubstMap,
-                        SizeInfo),
-                    Polyhedron1 = intersection(Polyhedron0, CallArgsPoly),
-                    % Set any zero_vars in the constraints to zero
-                    % (i.e. delete the terms). We need to do this
-                    % when polymorphic arguments are zero sized.
-                    Polyhedron2 = polyhedron.zero_vars(CallZeros, Polyhedron1),
-                    post_process_abstract_goal(Locals, Info,
-                        Polyhedron2, !Polyhedron)
-                )
+            else if polyhedron.is_universe(SizeInfo) then
+                % Constraint store += true
+                true
+            else
+                HeadVars = term2_info_get_head_vars(CallTerm2Info),
+                SubstMap = create_var_substitution(CallVars, HeadVars),
+                Polyhedron0 = polyhedron.substitute_vars(SubstMap,
+                    SizeInfo),
+                Polyhedron1 = intersection(Polyhedron0, CallArgsPoly),
+                % Set any zero_vars in the constraints to zero
+                % (i.e. delete the terms). We need to do this
+                % when polymorphic arguments are zero sized.
+                Polyhedron2 = polyhedron.zero_vars(CallZeros, Polyhedron1),
+                post_process_abstract_goal(Locals, Info,
+                    Polyhedron2, !Polyhedron)
             )
         )
     ;
@@ -360,11 +352,11 @@ term_traverse_abstract_goal(Info, Goal, !Polyhedron) :-
     polyhedron::in, polyhedron::in, polyhedron::out) is det.
 
 post_process_abstract_goal(Locals, Info, GoalPolyhedron0, !Polyhedron) :-
-    ( polyhedron.is_empty(GoalPolyhedron0) ->
+    ( if polyhedron.is_empty(GoalPolyhedron0) then
         GoalPolyhedron = polyhedron.empty
-    ;
-        GoalPolyhedron = polyhedron.project(Locals, Info ^ tcfi_varset,
-            GoalPolyhedron0)
+    else
+        project_polyhedron(Info ^ tcfi_varset, Locals,
+            GoalPolyhedron0, GoalPolyhedron)
     ),
     polyhedron.intersection(GoalPolyhedron, !Polyhedron).
 
@@ -392,7 +384,7 @@ term_traverse_abstract_disj_linearly(Goals, Locals, Info, !Polyhedron) :-
 term_traverse_abstract_disj_linearly_2(Info, Locals, Goal, !Polyhedron) :-
     SizeVarSet = Info ^ tcfi_varset,
     term_traverse_abstract_goal(Info, Goal, polyhedron.universe, Polyhedron0),
-    Polyhedron1 = polyhedron.project(Locals, SizeVarSet, Polyhedron0),
+    project_polyhedron(SizeVarSet, Locals, Polyhedron0, Polyhedron1),
     polyhedron.convex_union(SizeVarSet, yes(Info ^ tcfi_max_matrix_size),
         Polyhedron1, !Polyhedron).
 
@@ -403,6 +395,7 @@ term_traverse_abstract_disj_linearly_2(Info, Locals, Goal, !Polyhedron) :-
     %
 :- pred term_traverse_abstract_disj_pairwise(abstract_goals::in, size_vars::in,
     fixpoint_info::in, polyhedron::in, polyhedron::out) is det.
+:- pragma consider_used(pred(term_traverse_abstract_disj_pairwise/5)).
 
 term_traverse_abstract_disj_pairwise(Goals, Locals, Info, !Polyhedron) :-
     SizeVarSet = Info ^ tcfi_varset,
@@ -410,17 +403,19 @@ term_traverse_abstract_disj_pairwise(Goals, Locals, Info, !Polyhedron) :-
     PolyToLeft = polyhedron.universe,
 
     % First convert the list of goals into their corresponding polyhedra.
-    ToPoly = (func(Goal) = Poly :-
-        term_traverse_abstract_goal(Info, Goal, PolyToLeft, Poly0),
-        Poly = polyhedron.project(Locals, SizeVarSet, Poly0)
-    ),
+    ToPoly =
+        ( func(Goal) = Poly :-
+            term_traverse_abstract_goal(Info, Goal, PolyToLeft, Poly0),
+            polyhedron.project_polyhedron(SizeVarSet, Locals, Poly0, Poly)
+        ),
     Polyhedra0 = list.map(ToPoly, Goals),
 
     % Now pairwise convex hull them.
-    HullOp = (func(A, B) = C :-
-        polyhedron.convex_union(SizeVarSet, yes(Info ^ tcfi_max_matrix_size),
-            A, B, C)
-    ),
+    HullOp =
+        ( func(A, B) = C :-
+            polyhedron.convex_union(SizeVarSet,
+                yes(Info ^ tcfi_max_matrix_size), A, B, C)
+        ),
     ConvexUnion = pairwise_map(HullOp, [ polyhedron.empty | Polyhedra0]),
     polyhedron.intersection(ConvexUnion, !Polyhedron).
 
@@ -430,7 +425,7 @@ term_traverse_abstract_disj_pairwise(Goals, Locals, Info, !Polyhedron) :-
  :- func pairwise_map(func(T, T) = T, list(T)) = T.
 
 pairwise_map(_, []) = _ :-
-    unexpected($module, $pred, "empty list").
+    unexpected($pred, "empty list").
 pairwise_map(_, [X]) = X.
 pairwise_map(Op, List @ [_, _ | _]) = X :-
     pairwise_map_2(Op, List, [], X0),
@@ -455,9 +450,9 @@ pairwise_map_2(Op, [X, Y | Rest], !Acc) :-
 
 test_fixpoint_and_perhaps_widen(after_fixed_cutoff(Threshold), SizeVarSet,
         Iteration, OldPoly, NewPoly, ResultPoly, ChangeFlag) :-
-    ( Iteration > Threshold ->
+    ( if Iteration > Threshold then
         ResultPoly = widen(OldPoly, NewPoly, SizeVarSet)
-    ;
+    else
         ResultPoly = NewPoly
     ),
     ChangeFlag = test_fixpoint(NewPoly, OldPoly, SizeVarSet).
@@ -469,17 +464,28 @@ test_fixpoint(NewPoly, OldPoly, SizeVarSet) = ChangeFlag :-
     NewConstraints = polyhedron.non_false_constraints(NewPoly),
     % Constraints from previous iteration.
     OldConstraints = polyhedron.non_false_constraints(OldPoly),
-    (
+    ( if
         some [OldConstraint] (
             list.member(OldConstraint, OldConstraints),
             not entailed(SizeVarSet, NewConstraints, OldConstraint)
         )
-    ->
+    then
         ChangeFlag = yes
-    ;
+    else
         ChangeFlag = no
     ).
 
+%-----------------------------------------------------------------------------%
+
+:- type fixpoint_options
+    --->    fixpoint_options(
+                fo_widening :: widening,
+                fo_max_size :: int
+            ).
+
+fixpoint_options_init(Widening, MaxMatrixSize) =
+    fixpoint_options(Widening, MaxMatrixSize).
+
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.term_constr_fixpoint.
 %-----------------------------------------------------------------------------%
