@@ -16,19 +16,25 @@
 :- module transform_hlds.term_constr_util.
 :- interface.
 
-:- import_module hlds.hlds_pred.
+:- import_module hlds.
 :- import_module hlds.hlds_module.
+:- import_module hlds.hlds_pred.
+:- import_module hlds.vartypes.
+:- import_module libs.
 :- import_module libs.lp_rational.
 :- import_module libs.polyhedron.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
+:- import_module parse_tree.prog_data_pragma.
 :- import_module transform_hlds.term_constr_data.
-:- import_module transform_hlds.term_constr_main.
+:- import_module transform_hlds.term_constr_main_types.
 
 :- import_module bool.
 :- import_module io.
 :- import_module list.
 :- import_module map.
 :- import_module maybe.
+:- import_module set.
 
 %-----------------------------------------------------------------------------%
 %
@@ -46,7 +52,7 @@
 
     % Retrieve the abstraction representation from the module_info.
     %
-:- func get_abstract_scc(module_info, list(pred_proc_id)) = abstract_scc.
+:- func get_abstract_scc(module_info, set(pred_proc_id)) = abstract_scc.
 
 :- func get_abstract_proc(module_info, pred_proc_id) = abstract_proc.
 
@@ -63,8 +69,8 @@
     size_var_map::out) is det.
 
     % Given a list of prog_vars, allocate one size_var per prog_var.
-    % Allocate the size_vars from the provided size_varset.  Return
-    % a map between prog_vars and size_vars.
+    % Allocate the size_vars from the provided size_varset.
+    % Return a map between prog_vars and size_vars.
     %
 :- pred make_size_var_map(list(prog_var)::in,
     size_varset::in, size_varset::out, size_var_map::out) is det.
@@ -84,7 +90,7 @@
     % create_nonneg_constraints(SizeVarMap, Zeros) = Constraints.
     %
     % Returns a list of constraints of the form "x >= 0" for every size_var
-    % x that is is in `SizeVarMap' and is not in the set `Zeros'.
+    % x that is in `SizeVarMap' and is not in the set `Zeros'.
     %
 :- func create_nonneg_constraints(size_var_map, zero_vars) = constraints.
 
@@ -92,7 +98,7 @@
 
     % create_var_substition(FromVars, ToVars) = Substitution.
     % Create a mapping that maps elements of `FromVars' to their
-    % corresponding elements in `ToVars'.  This mapping is many-one.
+    % corresponding elements in `ToVars'. This mapping is many-one.
     % An exception is thrown if `FromVars' contains any duplicate elements.
     %
 :- func create_var_substitution(size_vars, size_vars) = var_substitution.
@@ -117,48 +123,18 @@
 
 %-----------------------------------------------------------------------------%
 
-    % substitute_size_vars: Takes a list of constraints and a
-    % var_substitution.  Returns the constraints with the specified
-    % substitutions made.
+    % substitute_size_vars: Takes a list of constraints and a var_substitution.
+    % Returns the constraints with the specified substitutions made.
     %
 :- func substitute_size_vars(constraints, map(size_var, size_var))
     = constraints.
 
-%-----------------------------------------------------------------------------%
-%
-% Predicates for printing out debugging traces. The first boolean argument
-% of these predicates should be the value of the --debug-term option.
-%
-
-    % Call the specified predicate.
-    %
-:- pred maybe_write_trace(bool::in, pred(io, io)::in(pred(di, uo) is det),
-    io::di, io::uo) is det.
-
-    % As above but if the boolean argument is `yes', print a newline
-    % to stdout before flushing the output.
-    %
-:- pred maybe_write_trace_nl(bool::in, pred(io, io)::in(pred(di, uo) is det),
-    bool::in, io::di, io::uo) is det.
-
-:- pred maybe_write_scc_procs(list(pred_proc_id)::in, module_info::in,
-    int::in, io::di, io::uo) is det.
-
-:- pred maybe_write_proc_name(pred_proc_id::in, string::in, module_info::in,
-    int::in, io::di, io::uo) is det.
-
-:- pred write_size_vars(size_varset::in, size_vars::in, io::di, io::uo) is det.
-
-:- pred dump_size_varset(size_varset::in, io::di, io::uo) is det.
-
-:- pred dump_size_vars(size_vars::in, size_varset::in, io::di, io::uo) is det.
-
 %-----------------------------------------------------------------------------%
 
 :- pred update_arg_size_info(pred_proc_id::in, polyhedron::in, module_info::in,
     module_info::out) is det.
 
-    % change_procs_constr_termination_info(SCC, Override, TermInfo,
+    % change_procs_constr_termination_info(SCC, Override, Term2Info,
     %   !ProcTable).
     %
     % If Override is yes, then this predicate overrides any existing
@@ -174,7 +150,7 @@
     %   !ProcTable).
     %
     % This predicate sets the arg_size_info property of the given
-    % list of procedures.  If Override is yes, then this predicate
+    % list of procedures. If Override is yes, then this predicate
     % overrides any existing arg_size information. If Override is
     % no, then it leaves the proc_info of a procedure unchanged
     % unless the proc_info had no arg_size information (i.e. the
@@ -183,23 +159,61 @@
 :- pred change_procs_constr_arg_size_info(list(proc_id)::in, bool::in,
     constr_arg_size_info::in, proc_table::in, proc_table::out) is det.
 
+%-----------------------------------------------------------------------------%
+%
+% Predicates for printing out debugging traces. The first boolean argument
+% of these predicates should be the value of the --debug-term option.
+%
+
+    % Call the specified predicate.
+    %
+    % XXX This predicate is currently unused.
+    % If it is ever used again, give it a better name.
+    %
+:- pred maybe_write_trace(io.text_output_stream::in, bool::in,
+    pred(io.text_output_stream, io, io)::in(pred(in, di, uo) is det),
+    io::di, io::uo) is det.
+
+    % As above but if the boolean argument is `yes', print a newline
+    % to stdout before flushing the output.
+    %
+    % XXX This predicate is currently unused.
+    % If it is ever used again, give it a better name.
+    %
+:- pred maybe_write_trace_nl(io.text_output_stream::in, bool::in,
+    pred(io.text_output_stream, io, io)::in(pred(in, di, uo) is det),
+    bool::in, io::di, io::uo) is det.
+
+:- pred maybe_write_scc_procs(io.text_output_stream::in, module_info::in,
+    list(pred_proc_id)::in, io::di, io::uo) is det.
+
+:- pred maybe_write_proc_name(io.text_output_stream::in, module_info::in,
+    string::in, pred_proc_id::in, io::di, io::uo) is det.
+
+:- pred write_size_vars(io.text_output_stream::in, size_varset::in,
+    size_vars::in, io::di, io::uo) is det.
+
+:- pred dump_size_vars(io.text_output_stream::in, size_varset::in,
+    size_vars::in, io::di, io::uo) is det.
+
+:- pred dump_size_varset(io.text_output_stream::in, size_varset::in,
+    io::di, io::uo) is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.
 :- import_module check_hlds.type_util.
-:- import_module hlds.hlds_module.
 :- import_module hlds.hlds_out.
 :- import_module hlds.hlds_out.hlds_out_util.
-:- import_module hlds.hlds_pred.
 :- import_module libs.rat.
 :- import_module transform_hlds.term_constr_errors.
 :- import_module transform_hlds.term_norm.
 
 :- import_module pair.
 :- import_module require.
-:- import_module set.
 :- import_module std_util.
 :- import_module string.
 :- import_module term.
@@ -212,13 +226,13 @@ set_pred_proc_ids_constr_arg_size_info([PPId | PPIds], ArgSize, !ModuleInfo) :-
     PPId = proc(PredId, ProcId),
     module_info_get_preds(!.ModuleInfo, PredTable0),
     map.lookup(PredTable0, PredId, PredInfo0),
-    pred_info_get_procedures(PredInfo0, ProcTable0),
+    pred_info_get_proc_table(PredInfo0, ProcTable0),
     map.lookup(ProcTable0, ProcId, ProcInfo0),
-    proc_info_get_termination2_info(ProcInfo0, TermInfo0),
-    TermInfo = TermInfo0 ^ success_constrs := yes(ArgSize),
-    proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
+    proc_info_get_termination2_info(ProcInfo0, Term2Info0),
+    term2_info_set_success_constrs(yes(ArgSize), Term2Info0, Term2Info),
+    proc_info_set_termination2_info(Term2Info, ProcInfo0, ProcInfo),
     map.det_update(ProcId, ProcInfo, ProcTable0, ProcTable),
-    pred_info_set_procedures(ProcTable, PredInfo0, PredInfo),
+    pred_info_set_proc_table(ProcTable, PredInfo0, PredInfo),
     map.det_update(PredId, PredInfo, PredTable0, PredTable),
     module_info_set_preds(PredTable, !ModuleInfo),
     set_pred_proc_ids_constr_arg_size_info(PPIds, ArgSize, !ModuleInfo).
@@ -226,23 +240,39 @@ set_pred_proc_ids_constr_arg_size_info([PPId | PPIds], ArgSize, !ModuleInfo) :-
 lookup_proc_constr_arg_size_info(ModuleInfo, PredProcId) = MaybeArgSizeInfo :-
     PredProcId = proc(PredId, ProcId),
     module_info_pred_proc_info(ModuleInfo, PredId, ProcId, _, ProcInfo),
-    proc_info_get_termination2_info(ProcInfo, TermInfo),
-    MaybeArgSizeInfo = TermInfo ^ success_constrs.
+    proc_info_get_termination2_info(ProcInfo, Term2Info),
+    MaybeArgSizeInfo = term2_info_get_success_constrs(Term2Info).
 
 %-----------------------------------------------------------------------------%
 
-make_size_var_map(ProgVars, SizeVarset, SizeVarMap) :-
-    make_size_var_map(ProgVars, varset.init, SizeVarset, SizeVarMap).
+get_abstract_scc(ModuleInfo, SCC) =
+    set.map(get_abstract_proc(ModuleInfo), SCC).
 
-make_size_var_map(ProgVars, !SizeVarset, SizeVarMap) :-
+get_abstract_proc(ModuleInfo, PPId) = AbstractProc :-
+    module_info_pred_proc_info(ModuleInfo, PPId, _, ProcInfo),
+    proc_info_get_termination2_info(ProcInfo, Term2Info),
+    MaybeAbstractProc = term2_info_get_abstract_rep(Term2Info),
+    (
+        MaybeAbstractProc = yes(AbstractProc)
+    ;
+        MaybeAbstractProc = no,
+        unexpected($pred, "no abstract rep. for proc")
+    ).
+
+%-----------------------------------------------------------------------------%
+
+make_size_var_map(ProgVars, SizeVarSet, SizeVarMap) :-
+    make_size_var_map(ProgVars, varset.init, SizeVarSet, SizeVarMap).
+
+make_size_var_map(ProgVars, !SizeVarSet, SizeVarMap) :-
     list.foldl2(make_size_var_map_2, ProgVars,
-        map.init, SizeVarMap, !SizeVarset).
+        map.init, SizeVarMap, !SizeVarSet).
 
 :- pred make_size_var_map_2(prog_var::in, size_var_map::in, size_var_map::out,
     size_varset::in, size_varset::out) is det.
 
-make_size_var_map_2(ProgVar, !SizeVarMap, !SizeVarset) :-
-    varset.new_var(SizeVar, !SizeVarset),
+make_size_var_map_2(ProgVar, !SizeVarMap, !SizeVarSet) :-
+    varset.new_var(SizeVar, !SizeVarSet),
     map.set(ProgVar, SizeVar, !SizeVarMap).
 
 prog_vars_to_size_vars(SizeVarMap, Vars)
@@ -252,7 +282,7 @@ prog_var_to_size_var(SizeVarMap, Var) = SizeVar :-
     ( if map.search(SizeVarMap, Var, SizeVar0) then
         SizeVar = SizeVar0
     else
-        unexpected($module, $pred, "prog_var not in size_var_map")
+        unexpected($pred, "prog_var not in size_var_map")
     ).
 
 find_zero_size_vars(ModuleInfo, SizeVarMap, VarTypes) = Zeros :-
@@ -262,7 +292,7 @@ find_zero_size_vars(ModuleInfo, SizeVarMap, VarTypes) = Zeros :-
 
     % Build zeros from corresponding size_vars.
     ZerosList = prog_vars_to_size_vars(SizeVarMap, ZeroProgVars),
-    Zeros = set.from_list(ZerosList).
+    Zeros = set.list_to_set(ZerosList).
 
 :- pred is_zero_size_prog_var(module_info::in, vartypes::in,
     prog_var::in) is semidet.
@@ -274,28 +304,9 @@ is_zero_size_prog_var(ModuleInfo, VarTypes, Var) :-
     ;
         % We don't include dummy types in the constraints - they won't tell us
         % anything useful.
-        check_dummy_type(ModuleInfo, Type) = is_dummy_type
+        is_type_a_dummy(ModuleInfo, Type) = is_dummy_type
     ).
 
-add_context_to_constr_termination_info(no, _, no).
-add_context_to_constr_termination_info(yes(cannot_loop(_)), _,
-        yes(cannot_loop(term_reason_import_supplied))).
-add_context_to_constr_termination_info(yes(can_loop(_)), Context,
-        yes(can_loop([Context - imported_pred]))).
-
-%-----------------------------------------------------------------------------%
-
-substitute_size_vars(Constraints0, SubstMap) = Constraints :-
-    SubVarInCoeff = (func(OldVar - Rat) = NewVar - Rat :-
-        map.lookup(SubstMap, OldVar, NewVar)
-    ),
-    SubVarInEqn = (func(Constr0) = Constr :-
-        deconstruct_constraint(Constr0, Coeffs0, Op, Rat),
-        Coeffs = list.map(SubVarInCoeff, Coeffs0),
-        Constr = construct_constraint(Coeffs, Op, Rat)
-    ),
-    Constraints = list.map(SubVarInEqn, Constraints0).
-
 %-----------------------------------------------------------------------------%
 %
 % Utility procedures used by various parts of the IR analysis.
@@ -320,9 +331,9 @@ create_var_substitution(Args, HeadVars) = SubstMap :-
 
 create_var_substitution_2([], [], !Subst).
 create_var_substitution_2([_|_], [], _, _) :-
-    unexpected($module, $pred, "unmatched lists").
+    unexpected($pred, "unmatched lists").
 create_var_substitution_2([], [_|_], _, _) :-
-    unexpected($module, $pred, "unmatched lists").
+    unexpected($pred, "unmatched lists").
 create_var_substitution_2([Arg | Args], [HeadVar | HeadVars],  !Subst) :-
     map.det_insert(HeadVar, Arg, !Subst),
     create_var_substitution_2(Args, HeadVars, !Subst).
@@ -330,84 +341,38 @@ create_var_substitution_2([Arg | Args], [HeadVar | HeadVars],  !Subst) :-
 make_arg_constraints([], _) = [].
 make_arg_constraints([Var | Vars], Zeros) = Constraints :-
     Constraints0 = make_arg_constraints(Vars, Zeros),
-    ( set.member(Var, Zeros) ->
+    ( if set.member(Var, Zeros) then
         Constraints = Constraints0
-    ;
+    else
         NewConstraint = construct_constraint([Var - one], lp_gt_eq, zero),
         Constraints = [NewConstraint | Constraints0]
     ).
 
-is_zero_size_var(Zeros, SizeVar) :- set.member(SizeVar, Zeros).
+is_zero_size_var(Zeros, SizeVar) :-
+    set.member(SizeVar, Zeros).
 
 %-----------------------------------------------------------------------------%
-%
-% Predicates for printing out debugging traces ...
-%
 
-maybe_write_trace(DebugTerm, TracePred, !IO) :-
-    maybe_write_trace_nl(DebugTerm, TracePred, no, !IO).
+add_context_to_constr_termination_info(no, _, no).
+add_context_to_constr_termination_info(yes(cannot_loop(_)), _,
+        yes(cannot_loop(term_reason_import_supplied))).
+add_context_to_constr_termination_info(yes(can_loop(_)), Context,
+        yes(can_loop([term2_error(Context, imported_pred)]))).
 
-maybe_write_trace_nl(DebugTerm, TracePred, NewLine, !IO) :-
-    (
-        DebugTerm = yes,
-        TracePred(!IO),
-        (
-            NewLine = yes,
-            io.nl(!IO)
-        ;
-            NewLine = no
+%-----------------------------------------------------------------------------%
+
+substitute_size_vars(Constraints0, SubstMap) = Constraints :-
+    SubVarInCoeff =
+        ( func(OldVar - Rat) = NewVar - Rat :-
+            map.lookup(SubstMap, OldVar, NewVar)
         ),
-        io.flush_output(!IO)
-    ;
-        DebugTerm = no
-    ).
-
-maybe_write_scc_procs(SCC, ModuleInfo, _, !IO) :-
-    write_scc_procs_2(SCC, ModuleInfo, !IO),
-    io.nl(!IO).
-
-:- pred write_scc_procs_2(list(pred_proc_id)::in, module_info::in,
-    io::di, io::uo) is det.
-
-write_scc_procs_2([], _, !IO).
-write_scc_procs_2([PPId | PPIds], ModuleInfo, !IO) :-
-    io.write_char('\t', !IO),
-    write_pred_proc_id(ModuleInfo, PPId, !IO),
-    io.nl(!IO),
-    write_scc_procs_2(PPIds, ModuleInfo, !IO).
-
-maybe_write_proc_name(PPId, String, ModuleInfo, _, !IO) :-
-    io.write_string(String, !IO),
-    write_pred_proc_id(ModuleInfo, PPId, !IO),
-    io.nl(!IO).
-
-write_size_vars(Varset, Vars, !IO) :-
-    WriteSizeVar = (pred(Var::in, !.IO::di, !:IO::uo) is det :-
-        varset.lookup_name(Varset, Var, Name),
-        io.write_string(Name, !IO)
-    ),
-    io.write_char('[', !IO),
-    io.write_list(Vars, ", ", WriteSizeVar, !IO),
-    io.write_char(']', !IO).
-
-%-----------------------------------------------------------------------------%
-
-dump_size_vars(Vars, Varset, !IO) :-
-    dump_size_varset_2(Vars, Varset, !IO).
-
-dump_size_varset(Varset, !IO) :-
-    Vars = varset.vars(Varset),
-    dump_size_varset_2(Vars, Varset, !IO).
-
-:- pred dump_size_varset_2(size_vars::in, size_varset::in, io::di, io::uo)
-    is det.
-
-dump_size_varset_2([], _, !IO).
-dump_size_varset_2([Var | Vars], Varset, !IO) :-
-    Name = varset.lookup_name(Varset, Var),
-    io.write(Var, !IO),
-    io.format(" = %s\n", [s(Name)], !IO),
-    dump_size_varset_2(Vars, Varset, !IO).
+    SubVarInEqn =
+        ( func(Constr0) = Constr :-
+            deconstruct_constraint(Constr0, Coeffs0, Op, Rat),
+            Coeffs = list.map(SubVarInCoeff, Coeffs0),
+            Constr = construct_constraint(Coeffs, Op, Rat)
+        ),
+    Constraints = list.map(SubVarInEqn, Constraints0).
 
 %-----------------------------------------------------------------------------%
 
@@ -419,17 +384,17 @@ update_arg_size_info(PPID, Polyhedron, !ModuleInfo) :-
 change_procs_constr_termination_info([], _, _, !ProcTable).
 change_procs_constr_termination_info([ProcId | ProcIds], Override, Termination,
         !ProcTable) :-
-    ProcInfo0 = !.ProcTable ^ det_elem(ProcId),
-    proc_info_get_termination2_info(ProcInfo0, TermInfo0),
-    (
+    map.lookup(!.ProcTable, ProcId, ProcInfo0),
+    proc_info_get_termination2_info(ProcInfo0, Term2Info0),
+    ( if
         ( Override = yes
-        ; TermInfo0 ^ term_status = no
+        ; term2_info_get_term_status(Term2Info0) = no
         )
-    ->
-        TermInfo = TermInfo0 ^ term_status := yes(Termination),
-        proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
+    then
+        term2_info_set_term_status(yes(Termination), Term2Info0, Term2Info),
+        proc_info_set_termination2_info(Term2Info, ProcInfo0, ProcInfo),
         map.det_update(ProcId, ProcInfo, !ProcTable)
-    ;
+    else
         true
     ),
     change_procs_constr_termination_info(ProcIds, Override, Termination,
@@ -438,37 +403,84 @@ change_procs_constr_termination_info([ProcId | ProcIds], Override, Termination,
 change_procs_constr_arg_size_info([], _, _, !ProcTable).
 change_procs_constr_arg_size_info([ProcId | ProcIds], Override, ArgSize,
         !ProcTable) :-
-    ProcInfo0 = !.ProcTable ^ det_elem(ProcId),
-    proc_info_get_termination2_info(ProcInfo0, TermInfo0),
-    (
+    map.lookup(!.ProcTable, ProcId, ProcInfo0),
+    proc_info_get_termination2_info(ProcInfo0, Term2Info0),
+    ( if
         ( Override = yes
-        ; TermInfo0 ^ success_constrs = no
+        ; term2_info_get_success_constrs(Term2Info0) = no
         )
-    ->
-        TermInfo = TermInfo0 ^ success_constrs := yes(ArgSize),
-        proc_info_set_termination2_info(TermInfo, ProcInfo0, ProcInfo),
+    then
+        term2_info_set_success_constrs(yes(ArgSize), Term2Info0, Term2Info),
+        proc_info_set_termination2_info(Term2Info, ProcInfo0, ProcInfo),
         map.det_update(ProcId, ProcInfo, !ProcTable)
-    ;
+    else
         true
     ),
     change_procs_constr_arg_size_info(ProcIds, Override, ArgSize, !ProcTable).
 
 %-----------------------------------------------------------------------------%
+%
+% Predicates for printing out debugging traces ...
+%
 
-get_abstract_scc(ModuleInfo, SCC) =
-    list.map(get_abstract_proc(ModuleInfo), SCC).
+maybe_write_trace(Stream, DebugTerm, TracePred, !IO) :-
+    maybe_write_trace_nl(Stream, DebugTerm, TracePred, no, !IO).
 
-get_abstract_proc(ModuleInfo, PPId) = AbstractProc :-
-    module_info_pred_proc_info(ModuleInfo, PPId, _, ProcInfo),
-    proc_info_get_termination2_info(ProcInfo, TermInfo),
-    MaybeAbstractProc = TermInfo ^ abstract_rep,
+maybe_write_trace_nl(Stream, DebugTerm, TracePred, NewLine, !IO) :-
     (
-        MaybeAbstractProc = yes(AbstractProc)
+        DebugTerm = yes,
+        TracePred(Stream, !IO),
+        (
+            NewLine = yes,
+            io.nl(Stream, !IO)
+        ;
+            NewLine = no
+        ),
+        io.flush_output(Stream, !IO)
     ;
-        MaybeAbstractProc = no,
-        unexpected($module, $pred, "no abstract rep. for proc")
+        DebugTerm = no
     ).
 
+maybe_write_scc_procs(Stream, ModuleInfo, SCC, !IO) :-
+    write_scc_procs_loop(Stream, ModuleInfo, SCC, !IO),
+    io.nl(Stream, !IO).
+
+:- pred write_scc_procs_loop(io.text_output_stream::in,  module_info::in,
+    list(pred_proc_id)::in, io::di, io::uo) is det.
+
+write_scc_procs_loop(_, _, [], !IO).
+write_scc_procs_loop(Stream, ModuleInfo, [PPId | PPIds], !IO) :-
+    PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+    io.format(Stream, "\t%s\n", [s(PPIdStr)], !IO),
+    write_scc_procs_loop(Stream, ModuleInfo, PPIds, !IO).
+
+maybe_write_proc_name(Stream, ModuleInfo, Prefix, PPId, !IO) :-
+    PPIdStr = pred_proc_id_to_string(ModuleInfo, PPId),
+    io.format(Stream, "%s%s\n", [s(Prefix), s(PPIdStr)], !IO).
+
+write_size_vars(Stream, VarSet, Vars, !IO) :-
+    list.map(varset.lookup_name(VarSet), Vars, VarNames),
+    io.format(Stream, "[%s]", [s(string.join_list(", ", VarNames))], !IO).
+
+%-----------------------------------------------------------------------------%
+
+dump_size_vars(Stream, VarSet, Vars, !IO) :-
+    dump_size_varset_loop(Stream, VarSet, Vars, !IO).
+
+dump_size_varset(Stream, VarSet, !IO) :-
+    Vars = varset.vars(VarSet),
+    dump_size_varset_loop(Stream, VarSet, Vars, !IO).
+
+:- pred dump_size_varset_loop(io.text_output_stream::in, size_varset::in,
+    size_vars::in, io::di, io::uo) is det.
+
+dump_size_varset_loop(_, _, [], !IO).
+dump_size_varset_loop(Stream, VarSet, [Var | Vars], !IO) :-
+    Name = varset.lookup_name(VarSet, Var),
+    io.write(Stream, Var, !IO),
+    io.format(Stream, " = %s\n", [s(Name)], !IO),
+    dump_size_varset_loop(Stream, VarSet, Vars, !IO).
+
 %-----------------------------------------------------------------------------%
 :- end_module transform_hlds.term_constr_util.
 %-----------------------------------------------------------------------------%
