@@ -5,10 +5,10 @@
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
-% 
+%
 % File: trace_params.m.
 % Author: zs.
-% 
+%
 % This module defines the parameters of execution tracing at various trace
 % levels and with various settings of the --suppress-trace option.
 %
@@ -29,7 +29,7 @@
 % effective trace levels. Most of the other functions defined in this module
 % convert the given (global) trace level into the effective trace level of
 % the relevant procedure before calculating their result.
-% 
+%
 %-----------------------------------------------------------------------------%
 
 :- module libs.trace_params.
@@ -38,6 +38,7 @@
 :- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
+:- import_module mdbcomp.
 :- import_module mdbcomp.prim_data.
 
 :- import_module bool.
@@ -67,9 +68,33 @@
     is semidet.
 :- func default_trace_suppress = trace_suppress_items.
 
+%-----------------------------------------------------------------------------%
+
+:- func eff_trace_level(module_info, pred_info, proc_info, trace_level)
+    = trace_level.
+
+    % Given a trace level for a module, return the trace level we should use
+    % for compiler-generated unify, index and compare predicates.
+    %
+:- func trace_level_for_unify_compare(trace_level) = trace_level.
+
+%-----------------------------------------------------------------------------%
+
+:- func trace_level_none = trace_level.
+
+:- func at_least_at_shallow(trace_level) = bool.
+:- func at_least_at_deep(trace_level) = bool.
+
+%-----------------------------------------------------------------------------%
+
+:- type maybe_exec_trace_enabled
+    --->    exec_trace_is_not_enabled
+    ;       exec_trace_is_enabled.
+
     % These functions check for various properties of the global trace level.
     %
-:- func given_trace_level_is_none(trace_level) = bool.
+:- func is_exec_trace_enabled_at_given_trace_level(trace_level) =
+    maybe_exec_trace_enabled.
 :- func trace_level_allows_delay_death(trace_level) = bool.
 :- func trace_needs_return_info(trace_level, trace_suppress_items) = bool.
 :- func trace_level_allows_tail_rec(trace_level) = bool.
@@ -86,8 +111,8 @@
     % These functions check for various properties of the given procedure's
     % effective trace level.
     %
-:- func eff_trace_level_is_none(module_info, pred_info, proc_info,
-    trace_level) = bool.
+:- func is_exec_trace_enabled_at_eff_trace_level(module_info,
+    pred_info, proc_info, trace_level) = maybe_exec_trace_enabled.
 :- func eff_trace_level_needs_input_vars(module_info, pred_info, proc_info,
     trace_level) = bool.
 :- func eff_trace_level_needs_fail_vars(module_info, pred_info, proc_info,
@@ -103,18 +128,7 @@
 :- func eff_trace_needs_port(module_info, pred_info, proc_info, trace_level,
     trace_suppress_items, trace_port) = bool.
 
-:- func eff_trace_level(module_info, pred_info, proc_info, trace_level)
-    = trace_level.
-
-:- func trace_level_none = trace_level.
-
-:- func at_least_at_shallow(trace_level) = bool.
-:- func at_least_at_deep(trace_level) = bool.
-
-    % Given a trace level for a module, return the trace level we should use
-    % for compiler-generated unify, index and compare predicates.
-    %
-:- func trace_level_for_unify_compare(trace_level) = trace_level.
+%-----------------------------------------------------------------------------%
 
     % This is used to represent the trace level in the module layout
     % and in proc layouts.
@@ -128,13 +142,11 @@
 
 :- implementation.
 
-:- import_module mdbcomp.
-:- import_module mdbcomp.prim_data.
+:- import_module hlds.status.
 
 :- import_module char.
 :- import_module int.
 :- import_module list.
-:- import_module pair.
 :- import_module set.
 :- import_module string.
 
@@ -174,28 +186,7 @@
 
 :- type trace_suppress_items == set(trace_suppress_item).
 
-trace_level_none = none.
-
-trace_level_for_unify_compare(none) = none.
-trace_level_for_unify_compare(basic) = none.
-trace_level_for_unify_compare(basic_user) = none.
-trace_level_for_unify_compare(shallow) = shallow.
-trace_level_for_unify_compare(deep) = shallow.
-trace_level_for_unify_compare(decl_rep) = shallow.
-
-at_least_at_shallow(none) = no.
-at_least_at_shallow(basic) = no.
-at_least_at_shallow(basic_user) = no.
-at_least_at_shallow(shallow) = yes.
-at_least_at_shallow(deep) = yes.
-at_least_at_shallow(decl_rep) = yes.
-
-at_least_at_deep(none) = no.
-at_least_at_deep(basic) = no.
-at_least_at_deep(basic_user) = no.
-at_least_at_deep(shallow) = no.
-at_least_at_deep(deep) = yes.
-at_least_at_deep(decl_rep) = yes.
+%-----------------------------------------------------------------------------%
 
 convert_trace_level("minimum", no,  no,  yes(none)).
 convert_trace_level("minimum", yes, no,  yes(shallow)).
@@ -210,250 +201,6 @@ convert_trace_level("default", no,  no,  yes(none)).
 convert_trace_level("default", yes, no,  yes(deep)).
 convert_trace_level("default", _,   yes, yes(decl_rep)).
 
-eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel) = EffTraceLevel :-
-    ( TraceLevel = none ->
-        EffTraceLevel = none
-    ;
-        pred_info_get_origin(PredInfo, Origin),
-        (
-            Origin = origin_special_pred(SpecialPred - _),
-            % Unify and compare predicates can be called from the generic
-            % unify and compare predicates in builtin.m, so they can be called
-            % from outside this module even if they don't have their address
-            % taken.
-            %
-            % Index predicates can never be called from anywhere except
-            % the compare predicate.
-            %
-            % Initialise predicates invoke user-provided code. Whether that
-            % code has debugging enabled or not, there is no point in
-            % generating events in the initialise predicate itself.
-            (
-                SpecialPred = spec_pred_unify,
-                EffTraceLevel = shallow
-            ;
-                SpecialPred = spec_pred_compare,
-                EffTraceLevel = shallow
-            ;
-                SpecialPred = spec_pred_index,
-                EffTraceLevel = none
-            ;
-                SpecialPred = spec_pred_init,
-                EffTraceLevel = TraceLevel
-            )
-        ;
-            Origin = origin_created(PredCreation),
-            (
-                PredCreation = created_by_io_tabling,
-                % Predicates called by a predicate that is I/O tabled
-                % should not be traced. If such a predicate were allowed
-                % to generate events, then the event numbers of events
-                % after the I/O primitive would be different between
-                % the first and subsequent (idempotent) executions
-                % of the same I/O action.
-                EffTraceLevel = none
-            ;
-                PredCreation = created_by_deforestation,
-                EffTraceLevel = usual_eff_trace_level(ModuleInfo,
-                    PredInfo, ProcInfo, TraceLevel)
-            )
-        ;
-            ( Origin = origin_instance_method(_, _)
-            ; Origin = origin_transformed(_, _, _)
-            ; Origin = origin_assertion(_, _)
-            ; Origin = origin_lambda(_, _, _)
-            ; Origin = origin_user(_)
-            ),
-            EffTraceLevel = usual_eff_trace_level(ModuleInfo,
-                PredInfo, ProcInfo, TraceLevel)
-        )
-    ).
-
-:- func usual_eff_trace_level(module_info, pred_info, proc_info, trace_level)
-    = trace_level.
-
-usual_eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)
-        = EffTraceLevel :-
-    pred_info_get_import_status(PredInfo, Status),
-    (
-        TraceLevel = shallow,
-        status_is_exported(Status) = no,
-        proc_info_get_is_address_taken(ProcInfo, address_is_not_taken)
-    ->
-        proc_info_get_has_user_event(ProcInfo, ProcHasUserEvent),
-        (
-            ProcHasUserEvent = yes,
-            EffTraceLevel = basic_user
-        ;
-            ProcHasUserEvent = no,
-            module_info_get_contains_user_event(ModuleInfo,
-                ModuleHasUserEvent),
-            (
-                ModuleHasUserEvent = yes,
-                EffTraceLevel = basic
-            ;
-                ModuleHasUserEvent = no,
-                EffTraceLevel = none
-            )
-        )
-    ;
-        EffTraceLevel = TraceLevel
-    ).
-
-given_trace_level_is_none(TraceLevel) =
-    trace_level_is_none(TraceLevel).
-
-eff_trace_level_is_none(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
-    trace_level_is_none(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
-eff_trace_level_needs_input_vars(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
-    trace_level_needs_input_vars(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
-eff_trace_level_needs_fail_vars(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
-    trace_level_needs_fail_vars(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
-eff_trace_level_needs_fixed_slots(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
-    trace_level_needs_fixed_slots(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
-eff_trace_level_needs_from_full_slot(ModuleInfo, PredInfo, ProcInfo,
-        TraceLevel) =
-    trace_level_needs_from_full_slot(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
-eff_trace_needs_all_var_names(ModuleInfo, PredInfo, ProcInfo, TraceLevel,
-        SuppressItems) =
-    trace_needs_all_var_names(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
-        SuppressItems).
-eff_trace_needs_proc_body_reps(ModuleInfo, PredInfo, ProcInfo, TraceLevel,
-        SuppressItems) =
-    trace_needs_proc_body_reps(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
-        SuppressItems).
-eff_trace_needs_port(ModuleInfo, PredInfo, ProcInfo, TraceLevel, SuppressItems,
-        Port) =
-    trace_needs_port(
-        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
-        SuppressItems, Port).
-
-:- func trace_level_is_none(trace_level) = bool.
-:- func trace_level_needs_input_vars(trace_level) = bool.
-:- func trace_level_needs_fail_vars(trace_level) = bool.
-:- func trace_level_needs_fixed_slots(trace_level) = bool.
-:- func trace_level_needs_from_full_slot(trace_level) = bool.
-:- func trace_needs_all_var_names(trace_level, trace_suppress_items) = bool.
-:- func trace_needs_port(trace_level, trace_suppress_items, trace_port) = bool.
-
-trace_level_is_none(none) = yes.
-trace_level_is_none(basic) = no.
-trace_level_is_none(basic_user) = no.
-trace_level_is_none(shallow) = no.
-trace_level_is_none(deep) = no.
-trace_level_is_none(decl_rep) = no.
-
-trace_level_needs_input_vars(none) = no.
-trace_level_needs_input_vars(basic) = no.
-trace_level_needs_input_vars(basic_user) = no.
-trace_level_needs_input_vars(shallow) = yes.
-trace_level_needs_input_vars(deep) = yes.
-trace_level_needs_input_vars(decl_rep) = yes.
-
-trace_level_needs_fail_vars(none) = no.
-trace_level_needs_fail_vars(basic) = no.
-trace_level_needs_fail_vars(basic_user) = yes.
-trace_level_needs_fail_vars(shallow) = yes.
-trace_level_needs_fail_vars(deep) = yes.
-trace_level_needs_fail_vars(decl_rep) = yes.
-
-trace_level_needs_fixed_slots(none) = no.
-trace_level_needs_fixed_slots(basic) = yes.
-trace_level_needs_fixed_slots(basic_user) = yes.
-trace_level_needs_fixed_slots(shallow) = yes.
-trace_level_needs_fixed_slots(deep) = yes.
-trace_level_needs_fixed_slots(decl_rep) = yes.
-
-trace_level_needs_from_full_slot(none) = no.
-trace_level_needs_from_full_slot(basic) = no.
-trace_level_needs_from_full_slot(basic_user) = no.
-trace_level_needs_from_full_slot(shallow) = yes.
-trace_level_needs_from_full_slot(deep) = no.
-trace_level_needs_from_full_slot(decl_rep) = no.
-
-trace_level_allows_delay_death(none) = no.
-trace_level_allows_delay_death(basic) = no.
-trace_level_allows_delay_death(basic_user) = yes.
-trace_level_allows_delay_death(shallow) = no.
-trace_level_allows_delay_death(deep) = yes.
-trace_level_allows_delay_death(decl_rep) = yes.
-
-trace_level_needs_meaningful_var_names(none) = no.
-trace_level_needs_meaningful_var_names(basic) = no.
-trace_level_needs_meaningful_var_names(basic_user) = yes.
-trace_level_needs_meaningful_var_names(shallow) = no.
-trace_level_needs_meaningful_var_names(deep) = yes.
-trace_level_needs_meaningful_var_names(decl_rep) = yes.
-
-trace_level_allows_tail_rec(none) = yes.
-trace_level_allows_tail_rec(basic) = yes.
-trace_level_allows_tail_rec(basic_user) = yes.
-trace_level_allows_tail_rec(shallow) = no.
-trace_level_allows_tail_rec(deep) = yes.
-trace_level_allows_tail_rec(decl_rep) = no.
-
-trace_needs_return_info(TraceLevel, TraceSuppressItems) = Need :-
-    (
-        trace_level_has_return_info(TraceLevel) = yes,
-        \+ set.member(suppress_return_info, TraceSuppressItems)
-    ->
-        Need = yes
-    ;
-        Need = no
-    ).
-
-trace_needs_all_var_names(TraceLevel, TraceSuppressItems) = Need :-
-    (
-        trace_level_has_all_var_names(TraceLevel) = yes,
-        \+ set.member(suppress_all_var_names, TraceSuppressItems)
-    ->
-        Need = yes
-    ;
-        Need = no
-    ).
-
-trace_needs_proc_body_reps(TraceLevel, TraceSuppressItems) = Need :-
-    (
-        trace_level_has_proc_body_reps(TraceLevel) = yes,
-        \+ set.member(suppress_proc_body_reps, TraceSuppressItems)
-    ->
-        Need = yes
-    ;
-        Need = no
-    ).
-
-:- func trace_level_has_return_info(trace_level) = bool.
-:- func trace_level_has_all_var_names(trace_level) = bool.
-:- func trace_level_has_proc_body_reps(trace_level) = bool.
-
-trace_level_has_return_info(none) = no.
-trace_level_has_return_info(basic) = yes.
-trace_level_has_return_info(basic_user) = yes.
-trace_level_has_return_info(shallow) = yes.
-trace_level_has_return_info(deep) = yes.
-trace_level_has_return_info(decl_rep) = yes.
-
-trace_level_has_all_var_names(none) = no.
-trace_level_has_all_var_names(basic) = no.
-trace_level_has_all_var_names(basic_user) = no.
-trace_level_has_all_var_names(shallow) = no.
-trace_level_has_all_var_names(deep) = no.
-trace_level_has_all_var_names(decl_rep) = yes.
-
-trace_level_has_proc_body_reps(none) = no.
-trace_level_has_proc_body_reps(basic) = no.
-trace_level_has_proc_body_reps(basic_user) = no.
-trace_level_has_proc_body_reps(shallow) = no.
-trace_level_has_proc_body_reps(deep) = no.
-trace_level_has_proc_body_reps(decl_rep) = yes.
-
 convert_trace_suppress(SuppressString, SuppressItemSet) :-
     SuppressWords = string.words_separator(char_is_comma, SuppressString),
     list.map(convert_item_name, SuppressWords, SuppressItemLists),
@@ -464,7 +211,23 @@ convert_trace_suppress(SuppressString, SuppressItemSet) :-
 
 char_is_comma(',').
 
-default_trace_suppress = set.init.
+:- pred convert_item_name(string::in, list(trace_suppress_item)::out)
+    is semidet.
+
+convert_item_name(String, Names) :-
+    ( if convert_port_name(String) = PortName then
+        Names = [suppress_port(PortName)]
+    else if convert_port_class_name(String) = PortNames then
+        list.map(wrap_port, PortNames, Names)
+    else if convert_other_name(String) = OtherName then
+        Names = [OtherName]
+    else
+        fail
+    ).
+
+:- pred wrap_port(trace_port::in, trace_suppress_item::out) is det.
+
+wrap_port(Port, suppress_port(Port)).
 
 :- func convert_port_name(string) = trace_port is semidet.
 
@@ -518,23 +281,295 @@ convert_other_name("all_var_names") = suppress_all_var_names.
 convert_other_name("bodies") = suppress_proc_body_reps.
 convert_other_name("proc_body_reps") = suppress_proc_body_reps.
 
-:- pred convert_item_name(string::in, list(trace_suppress_item)::out)
-    is semidet.
+default_trace_suppress = set.init.
 
-convert_item_name(String, Names) :-
-    ( convert_port_name(String) = PortName ->
-        Names = [suppress_port(PortName)]
-    ; convert_port_class_name(String) = PortNames ->
-        list.map(wrap_port, PortNames, Names)
-    ; convert_other_name(String) = OtherName ->
-        Names = [OtherName]
-    ;
-        fail
+%-----------------------------------------------------------------------------%
+
+eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel) = EffTraceLevel :-
+    ( if TraceLevel = none then
+        EffTraceLevel = none
+    else
+        pred_info_get_origin(PredInfo, Origin),
+        (
+            Origin = origin_special_pred(SpecialPred, _),
+            % Unify and compare predicates can be called from the generic
+            % unify and compare predicates in builtin.m, so they can be called
+            % from outside this module even if they don't have their address
+            % taken.
+            %
+            % Index predicates can never be called from anywhere except
+            % the compare predicate.
+            %
+            % Initialise predicates invoke user-provided code. Whether that
+            % code has debugging enabled or not, there is no point in
+            % generating events in the initialise predicate itself.
+            (
+                SpecialPred = spec_pred_unify,
+                EffTraceLevel = shallow
+            ;
+                SpecialPred = spec_pred_compare,
+                EffTraceLevel = shallow
+            ;
+                SpecialPred = spec_pred_index,
+                EffTraceLevel = none
+            )
+        ;
+            Origin = origin_created(PredCreation),
+            (
+                PredCreation = created_by_io_tabling,
+                % Predicates called by a predicate that is I/O tabled
+                % should not be traced. If such a predicate were allowed
+                % to generate events, then the event numbers of events
+                % after the I/O primitive would be different between
+                % the first and subsequent (idempotent) executions
+                % of the same I/O action.
+                EffTraceLevel = none
+            ;
+                PredCreation = created_by_deforestation,
+                EffTraceLevel = usual_eff_trace_level(ModuleInfo,
+                    PredInfo, ProcInfo, TraceLevel)
+            )
+        ;
+            ( Origin = origin_instance_method(_, _)
+            ; Origin = origin_class_method(_, _)
+            ; Origin = origin_transformed(_, _, _)
+            ; Origin = origin_assertion(_, _)
+            ; Origin = origin_lambda(_, _, _)
+            ; Origin = origin_solver_type(_, _, _)
+            ; Origin = origin_tabling(_, _)
+            ; Origin = origin_mutable(_, _, _)
+            ; Origin = origin_initialise
+            ; Origin = origin_finalise
+            ; Origin = origin_user(_)
+            ),
+            EffTraceLevel = usual_eff_trace_level(ModuleInfo,
+                PredInfo, ProcInfo, TraceLevel)
+        )
     ).
 
-:- pred wrap_port(trace_port::in, trace_suppress_item::out) is det.
+:- func usual_eff_trace_level(module_info, pred_info, proc_info, trace_level)
+    = trace_level.
 
-wrap_port(Port, suppress_port(Port)).
+usual_eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)
+        = EffTraceLevel :-
+    pred_info_get_status(PredInfo, PredStatus),
+    ( if
+        TraceLevel = shallow,
+        pred_status_is_exported(PredStatus) = no,
+        proc_info_get_is_address_taken(ProcInfo, address_is_not_taken)
+    then
+        proc_info_get_has_user_event(ProcInfo, ProcHasUserEvent),
+        (
+            ProcHasUserEvent = has_user_event,
+            EffTraceLevel = basic_user
+        ;
+            ProcHasUserEvent = has_no_user_event,
+            module_info_get_has_user_event(ModuleInfo, ModuleHasUserEvent),
+            (
+                ModuleHasUserEvent = has_user_event,
+                EffTraceLevel = basic
+            ;
+                ModuleHasUserEvent = has_no_user_event,
+                EffTraceLevel = none
+            )
+        )
+    else
+        EffTraceLevel = TraceLevel
+    ).
+
+trace_level_for_unify_compare(none) = none.
+trace_level_for_unify_compare(basic) = none.
+trace_level_for_unify_compare(basic_user) = none.
+trace_level_for_unify_compare(shallow) = shallow.
+trace_level_for_unify_compare(deep) = shallow.
+trace_level_for_unify_compare(decl_rep) = shallow.
+
+%-----------------------------------------------------------------------------%
+
+trace_level_none = none.
+
+at_least_at_shallow(none) = no.
+at_least_at_shallow(basic) = no.
+at_least_at_shallow(basic_user) = no.
+at_least_at_shallow(shallow) = yes.
+at_least_at_shallow(deep) = yes.
+at_least_at_shallow(decl_rep) = yes.
+
+at_least_at_deep(none) = no.
+at_least_at_deep(basic) = no.
+at_least_at_deep(basic_user) = no.
+at_least_at_deep(shallow) = no.
+at_least_at_deep(deep) = yes.
+at_least_at_deep(decl_rep) = yes.
+
+%-----------------------------------------------------------------------------%
+
+is_exec_trace_enabled_at_given_trace_level(none) = exec_trace_is_not_enabled.
+is_exec_trace_enabled_at_given_trace_level(basic) = exec_trace_is_enabled.
+is_exec_trace_enabled_at_given_trace_level(basic_user) = exec_trace_is_enabled.
+is_exec_trace_enabled_at_given_trace_level(shallow) = exec_trace_is_enabled.
+is_exec_trace_enabled_at_given_trace_level(deep) = exec_trace_is_enabled.
+is_exec_trace_enabled_at_given_trace_level(decl_rep) = exec_trace_is_enabled.
+
+:- func trace_level_needs_input_vars(trace_level) = bool.
+
+trace_level_needs_input_vars(none) = no.
+trace_level_needs_input_vars(basic) = no.
+trace_level_needs_input_vars(basic_user) = no.
+trace_level_needs_input_vars(shallow) = yes.
+trace_level_needs_input_vars(deep) = yes.
+trace_level_needs_input_vars(decl_rep) = yes.
+
+:- func trace_level_needs_fail_vars(trace_level) = bool.
+
+trace_level_needs_fail_vars(none) = no.
+trace_level_needs_fail_vars(basic) = no.
+trace_level_needs_fail_vars(basic_user) = yes.
+trace_level_needs_fail_vars(shallow) = yes.
+trace_level_needs_fail_vars(deep) = yes.
+trace_level_needs_fail_vars(decl_rep) = yes.
+
+:- func trace_level_needs_fixed_slots(trace_level) = bool.
+
+trace_level_needs_fixed_slots(none) = no.
+trace_level_needs_fixed_slots(basic) = yes.
+trace_level_needs_fixed_slots(basic_user) = yes.
+trace_level_needs_fixed_slots(shallow) = yes.
+trace_level_needs_fixed_slots(deep) = yes.
+trace_level_needs_fixed_slots(decl_rep) = yes.
+
+:- func trace_level_needs_from_full_slot(trace_level) = bool.
+
+trace_level_needs_from_full_slot(none) = no.
+trace_level_needs_from_full_slot(basic) = no.
+trace_level_needs_from_full_slot(basic_user) = no.
+trace_level_needs_from_full_slot(shallow) = yes.
+trace_level_needs_from_full_slot(deep) = no.
+trace_level_needs_from_full_slot(decl_rep) = no.
+
+trace_level_allows_delay_death(none) = no.
+trace_level_allows_delay_death(basic) = no.
+trace_level_allows_delay_death(basic_user) = yes.
+trace_level_allows_delay_death(shallow) = no.
+trace_level_allows_delay_death(deep) = yes.
+trace_level_allows_delay_death(decl_rep) = yes.
+
+trace_needs_return_info(TraceLevel, TraceSuppressItems) = Need :-
+    ( if
+        trace_level_has_return_info(TraceLevel) = yes,
+        not set.member(suppress_return_info, TraceSuppressItems)
+    then
+        Need = yes
+    else
+        Need = no
+    ).
+
+trace_level_allows_tail_rec(none) = yes.
+trace_level_allows_tail_rec(basic) = yes.
+trace_level_allows_tail_rec(basic_user) = yes.
+trace_level_allows_tail_rec(shallow) = no.
+trace_level_allows_tail_rec(deep) = yes.
+trace_level_allows_tail_rec(decl_rep) = no.
+
+trace_level_needs_meaningful_var_names(none) = no.
+trace_level_needs_meaningful_var_names(basic) = no.
+trace_level_needs_meaningful_var_names(basic_user) = yes.
+trace_level_needs_meaningful_var_names(shallow) = no.
+trace_level_needs_meaningful_var_names(deep) = yes.
+trace_level_needs_meaningful_var_names(decl_rep) = yes.
+
+:- func trace_needs_all_var_names(trace_level, trace_suppress_items) = bool.
+
+trace_needs_all_var_names(TraceLevel, TraceSuppressItems) = Need :-
+    ( if
+        trace_level_has_all_var_names(TraceLevel) = yes,
+        not set.member(suppress_all_var_names, TraceSuppressItems)
+    then
+        Need = yes
+    else
+        Need = no
+    ).
+
+trace_needs_proc_body_reps(TraceLevel, TraceSuppressItems) = Need :-
+    ( if
+        trace_level_has_proc_body_reps(TraceLevel) = yes,
+        not set.member(suppress_proc_body_reps, TraceSuppressItems)
+    then
+        Need = yes
+    else
+        Need = no
+    ).
+
+%-----------------------------------------------------------------------------%
+
+is_exec_trace_enabled_at_eff_trace_level(ModuleInfo, PredInfo, ProcInfo,
+        TraceLevel) =
+    is_exec_trace_enabled_at_given_trace_level(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
+
+eff_trace_level_needs_input_vars(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
+    trace_level_needs_input_vars(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
+
+eff_trace_level_needs_fail_vars(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
+    trace_level_needs_fail_vars(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
+
+eff_trace_level_needs_fixed_slots(ModuleInfo, PredInfo, ProcInfo, TraceLevel) =
+    trace_level_needs_fixed_slots(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
+
+eff_trace_level_needs_from_full_slot(ModuleInfo, PredInfo, ProcInfo,
+        TraceLevel) =
+    trace_level_needs_from_full_slot(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel)).
+
+eff_trace_needs_all_var_names(ModuleInfo, PredInfo, ProcInfo, TraceLevel,
+        SuppressItems) =
+    trace_needs_all_var_names(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
+        SuppressItems).
+
+eff_trace_needs_proc_body_reps(ModuleInfo, PredInfo, ProcInfo, TraceLevel,
+        SuppressItems) =
+    trace_needs_proc_body_reps(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
+        SuppressItems).
+
+eff_trace_needs_port(ModuleInfo, PredInfo, ProcInfo, TraceLevel, SuppressItems,
+        Port) =
+    trace_needs_port(
+        eff_trace_level(ModuleInfo, PredInfo, ProcInfo, TraceLevel),
+        SuppressItems, Port).
+
+%-----------------------------------------------------------------------------%
+
+:- func trace_level_has_return_info(trace_level) = bool.
+
+trace_level_has_return_info(none) = no.
+trace_level_has_return_info(basic) = yes.
+trace_level_has_return_info(basic_user) = yes.
+trace_level_has_return_info(shallow) = yes.
+trace_level_has_return_info(deep) = yes.
+trace_level_has_return_info(decl_rep) = yes.
+
+:- func trace_level_has_all_var_names(trace_level) = bool.
+
+trace_level_has_all_var_names(none) = no.
+trace_level_has_all_var_names(basic) = no.
+trace_level_has_all_var_names(basic_user) = no.
+trace_level_has_all_var_names(shallow) = no.
+trace_level_has_all_var_names(deep) = no.
+trace_level_has_all_var_names(decl_rep) = yes.
+
+:- func trace_level_has_proc_body_reps(trace_level) = bool.
+
+trace_level_has_proc_body_reps(none) = no.
+trace_level_has_proc_body_reps(basic) = no.
+trace_level_has_proc_body_reps(basic_user) = no.
+trace_level_has_proc_body_reps(shallow) = no.
+trace_level_has_proc_body_reps(deep) = no.
+trace_level_has_proc_body_reps(decl_rep) = yes.
 
     % If this is modified, then the corresponding code in
     % runtime/mercury_stack_layout.h needs to be updated.
@@ -603,17 +638,19 @@ trace_level_allows_port_suppression(shallow) = yes.
 trace_level_allows_port_suppression(deep) = yes.
 trace_level_allows_port_suppression(decl_rep) = no.
 
+:- func trace_needs_port(trace_level, trace_suppress_items, trace_port) = bool.
+
 trace_needs_port(TraceLevel, TraceSuppressItems, Port) = NeedsPort :-
-    (
+    ( if
         trace_port_category(Port) = Category,
         list.member(Category, trace_level_port_categories(TraceLevel)),
-        \+ (
+        not (
             trace_level_allows_port_suppression(TraceLevel) = yes,
             set.member(suppress_port(Port), TraceSuppressItems)
         )
-    ->
+    then
         NeedsPort = yes
-    ;
+    else
         NeedsPort = no
     ).
 
@@ -655,3 +692,7 @@ port_number(port_disj_first) = 12.
 port_number(port_disj_later) = 13.
 port_number(port_switch) = 14.
 port_number(port_user) = 15.
+
+%-----------------------------------------------------------------------------%
+:- end_module libs.trace_params.
+%-----------------------------------------------------------------------------%
