@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2009-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public Licence - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -50,9 +51,9 @@
 %      magic_exception_result(TryResult),  % out(cannot_fail)
 %      (
 %          TryResult = succeeded({}),
-%          ( p(X, Y) ->
+%          ( if p(X, Y) then
 %              q(X, Y)
-%          ;
+%          else
 %              r
 %          )
 %      ;
@@ -72,9 +73,9 @@
 %      magic_exception_result(TryResult),  % out(cannot_fail)
 %      (
 %          TryResult = succeeded({}),
-%          ( <Goal> ->
+%          ( if <Goal> then
 %              <Then>
-%          ;
+%          else
 %              <Else>
 %          )
 %      ;
@@ -167,22 +168,22 @@
 % The <ExcpHandling> parts can be passed through from the pre-transformation.
 % If a `catch_any' is present the exception handling looks like this:
 %
-%      ( exc_univ_to_type(Excp, <CatchPattern1>) ->
+%      ( if exc_univ_to_type(Excp, <CatchPattern1>) then
 %          <CatchGoal1>
-%      ; exc_univ_to_type(Excp, <CatchPattern2>) ->
+%      else if exc_univ_to_type(Excp, <CatchPattern2>) then
 %          <CatchGoal2>
-%      ;
+%      else
 %          CatchAnyVar = exc_univ_value(Excp),
 %          <CatchAnyGoal>
 %      )
 %
 % Otherwise, if `catch_any' is not present:
 %
-%      ( exc_univ_to_type(Excp, <CatchPattern1>) ->
+%      ( if exc_univ_to_type(Excp, <CatchPattern1>) then
 %          <CatchGoal1>
-%      ; exc_univ_to_type(Excp, <CatchPattern2>) ->
+%      else if exc_univ_to_type(Excp, <CatchPattern2>) then
 %          <CatchGoal2>
-%      ;
+%      else
 %          rethrow(TryResult)
 %      )
 %
@@ -193,6 +194,7 @@
 
 :- import_module hlds.
 :- import_module hlds.hlds_module.
+:- import_module parse_tree.
 :- import_module parse_tree.error_util.
 :- import_module parse_tree.prog_data.
 
@@ -218,16 +220,19 @@
 :- import_module check_hlds.det_analysis.
 :- import_module check_hlds.det_report.
 :- import_module check_hlds.modes.
-:- import_module check_hlds.polymorphism.
+:- import_module check_hlds.polymorphism_type_info.
 :- import_module hlds.goal_util.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
 :- import_module hlds.instmap.
+:- import_module hlds.make_goal.
 :- import_module hlds.pred_table.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
 :- import_module mdbcomp.
+:- import_module mdbcomp.builtin_modules.
 :- import_module mdbcomp.prim_data.
-:- import_module parse_tree.
+:- import_module mdbcomp.sym_name.
 :- import_module parse_tree.builtin_lib_types.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.set_of_var.
@@ -237,7 +242,6 @@
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
-:- import_module set.
 :- import_module string.
 :- import_module term.
 
@@ -245,16 +249,16 @@
 
 expand_try_goals_in_module(!ModuleInfo, !Specs) :-
     % The exception module is implicitly imported if any try goals were seen,
-    % so if the exception module is not imported then we know there are no try
-    % goals to be expanded.
-    module_info_get_imported_module_specifiers(!.ModuleInfo, ImportedModules),
-    ( set.contains(ImportedModules, mercury_exception_module) ->
+    % so if the exception module is not imported, then we know there are
+    % no try goals to be expanded.
+    module_info_get_avail_module_map(!.ModuleInfo, AvailModuleMap),
+    ( if map.search(AvailModuleMap, mercury_exception_module, _) then
         some [!Globals] (
             module_info_get_globals(!.ModuleInfo, !:Globals),
             disable_det_warnings(OptionsToRestore, !Globals),
             module_info_set_globals(!.Globals, !ModuleInfo),
 
-            module_info_get_valid_predids(PredIds, !ModuleInfo),
+            module_info_get_valid_pred_ids(!.ModuleInfo, PredIds),
             list.foldl2(expand_try_goals_in_pred, PredIds,
                 !ModuleInfo, !Specs),
 
@@ -262,7 +266,7 @@ expand_try_goals_in_module(!ModuleInfo, !Specs) :-
             restore_det_warnings(OptionsToRestore, !Globals),
             module_info_set_globals(!.Globals, !ModuleInfo)
         )
-    ;
+    else
         true
     ).
 
@@ -272,7 +276,7 @@ expand_try_goals_in_module(!ModuleInfo, !Specs) :-
 
 expand_try_goals_in_pred(PredId, !ModuleInfo, !Specs) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl2(expand_try_goals_in_proc(PredId), ProcIds,
         !ModuleInfo, !Specs).
 
@@ -285,10 +289,10 @@ expand_try_goals_in_proc(PredId, ProcId, !ModuleInfo, !Specs) :-
         module_info_pred_proc_info(!.ModuleInfo, PredId, ProcId,
             !:PredInfo, !:ProcInfo),
         proc_info_get_goal(!.ProcInfo, Goal0),
-        proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InitInstmap),
+        proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InitInstMap),
 
         Info0 = trys_info(!.ModuleInfo, !.PredInfo, !.ProcInfo, no),
-        expand_try_goals_in_goal(InitInstmap, Goal0, Goal, Info0, Info),
+        expand_try_goals_in_goal(InitInstMap, Goal0, Goal, Info0, Info),
         Info = trys_info(!:ModuleInfo, !:PredInfo, !:ProcInfo, Changed),
 
         (
@@ -312,7 +316,7 @@ update_changed_proc(Goal, PredId, ProcId, PredInfo, !.ProcInfo, !ModuleInfo,
     module_info_set_pred_proc_info(PredId, ProcId, PredInfo, !.ProcInfo,
         !ModuleInfo),
 
-    modecheck_proc(ProcId, PredId, !ModuleInfo, ModeSpecs, _Changed),
+    modecheck_proc(PredId, ProcId, !ModuleInfo, _Changed, ModeSpecs),
     module_info_get_globals(!.ModuleInfo, Globals),
     HasModeErrors = contains_errors(Globals, ModeSpecs),
     (
@@ -344,29 +348,29 @@ update_changed_proc(Goal, PredId, ProcId, PredInfo, !.ProcInfo, !ModuleInfo,
 :- pred expand_try_goals_in_goal(instmap::in, hlds_goal::in, hlds_goal::out,
     trys_info::in, trys_info::out) is det.
 
-expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
+expand_try_goals_in_goal(InstMap, Goal0, Goal, !Info) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
     (
         GoalExpr0 = unify(_, _, _, _, _),
         Goal = Goal0
     ;
         GoalExpr0 = conj(ConjType, Conjuncts0),
-        expand_try_goals_in_conj(Instmap, Conjuncts0, Conjuncts, !Info),
+        expand_try_goals_in_conj(InstMap, Conjuncts0, Conjuncts, !Info),
         GoalExpr = conj(ConjType, Conjuncts),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = disj(Disjuncts0),
-        expand_try_goals_in_disj(Instmap, Disjuncts0, Disjuncts, !Info),
+        expand_try_goals_in_disj(InstMap, Disjuncts0, Disjuncts, !Info),
         GoalExpr = disj(Disjuncts),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = negation(SubGoal0),
-        expand_try_goals_in_goal(Instmap, SubGoal0, SubGoal, !Info),
+        expand_try_goals_in_goal(InstMap, SubGoal0, SubGoal, !Info),
         GoalExpr = negation(SubGoal),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = switch(Var, CanFail, Cases0),
-        expand_try_goals_in_cases(Instmap, Cases0, Cases, !Info),
+        expand_try_goals_in_cases(InstMap, Cases0, Cases, !Info),
         GoalExpr = switch(Var, CanFail, Cases),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
@@ -376,11 +380,13 @@ expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
             % There can be no try goals inside this scope.
             Goal = Goal0
         ;
-            ( Reason = exist_quant(_)
+            ( Reason = disable_warnings(_, _)
+            ; Reason = exist_quant(_)
             ; Reason = promise_solutions(_, _)
             ; Reason = promise_purity(_)
             ; Reason = require_detism(_)
             ; Reason = require_complete_switch(_)
+            ; Reason = require_switch_arms_detism(_, _)
             ; Reason = commit(_)
             ; Reason = barrier(_)
             ; Reason = from_ground_term(_, from_ground_term_deconstruct)
@@ -388,16 +394,16 @@ expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
             ; Reason = trace_goal(_, _, _, _, _)
             ; Reason = loop_control(_, _, _)
             ),
-            expand_try_goals_in_goal(Instmap, InnerGoal0, InnerGoal, !Info),
+            expand_try_goals_in_goal(InstMap, InnerGoal0, InnerGoal, !Info),
             GoalExpr = scope(Reason, InnerGoal),
             Goal = hlds_goal(GoalExpr, GoalInfo0)
         ;
             Reason = from_ground_term(_, from_ground_term_initial),
-            unexpected($module, $pred, "from_ground_term_initial")
+            unexpected($pred, "from_ground_term_initial")
         )
     ;
         GoalExpr0 = if_then_else(Vars, Cond0, Then0, Else0),
-        expand_try_goals_in_if_then_else(Instmap, Cond0, Cond, Then0, Then,
+        expand_try_goals_in_if_then_else(InstMap, Cond0, Cond, Then0, Then,
             Else0, Else, !Info),
         GoalExpr = if_then_else(Vars, Cond, Then, Else),
         Goal = hlds_goal(GoalExpr, GoalInfo0)
@@ -412,19 +418,19 @@ expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
         GoalExpr0 = shorthand(ShortHand0),
         (
             ShortHand0 = try_goal(_, _, _),
-            expand_try_goal(Instmap, ShortHand0, Goal, !Info)
+            expand_try_goal(InstMap, ShortHand0, Goal, !Info)
         ;
             ShortHand0 = atomic_goal(AtomicGoalType, Outer, Inner,
                 MaybeOutputVars, MainGoal0, OrElseGoals0, OrElseInners),
-            expand_try_goals_in_goal(Instmap, MainGoal0, MainGoal, !Info),
-            expand_try_goals_in_disj(Instmap, OrElseGoals0, OrElseGoals,
+            expand_try_goals_in_goal(InstMap, MainGoal0, MainGoal, !Info),
+            expand_try_goals_in_disj(InstMap, OrElseGoals0, OrElseGoals,
                 !Info),
             GoalExpr = atomic_goal(AtomicGoalType, Outer, Inner,
                 MaybeOutputVars, MainGoal, OrElseGoals, OrElseInners),
             Goal = hlds_goal(shorthand(GoalExpr), GoalInfo0)
         ;
             ShortHand0 = bi_implication(_, _),
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
@@ -432,29 +438,29 @@ expand_try_goals_in_goal(Instmap, Goal0, Goal, !Info) :-
     list(hlds_goal)::in, list(hlds_goal)::out,
     trys_info::in, trys_info::out) is det.
 
-expand_try_goals_in_conj(_Instmap0, [], [], !Info).
-expand_try_goals_in_conj(Instmap0, [Goal0 | Goals0], [Goal | Goals], !Info) :-
-    expand_try_goals_in_goal(Instmap0, Goal0, Goal, !Info),
+expand_try_goals_in_conj(_InstMap0, [], [], !Info).
+expand_try_goals_in_conj(InstMap0, [Goal0 | Goals0], [Goal | Goals], !Info) :-
+    expand_try_goals_in_goal(InstMap0, Goal0, Goal, !Info),
     Goal0 = hlds_goal(_, GoalInfo),
-    InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
-    apply_instmap_delta(Instmap0, InstmapDelta, Instmap),
-    expand_try_goals_in_conj(Instmap, Goals0, Goals, !Info).
+    InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
+    apply_instmap_delta(InstMapDelta, InstMap0, InstMap),
+    expand_try_goals_in_conj(InstMap, Goals0, Goals, !Info).
 
 :- pred expand_try_goals_in_disj(instmap::in,
     list(hlds_goal)::in, list(hlds_goal)::out,
     trys_info::in, trys_info::out) is det.
 
-expand_try_goals_in_disj(Instmap0, Goals0, Goals, !Info) :-
-    list.map_foldl(expand_try_goals_in_goal(Instmap0), Goals0, Goals, !Info).
+expand_try_goals_in_disj(InstMap0, Goals0, Goals, !Info) :-
+    list.map_foldl(expand_try_goals_in_goal(InstMap0), Goals0, Goals, !Info).
 
 :- pred expand_try_goals_in_cases(instmap::in, list(case)::in, list(case)::out,
     trys_info::in, trys_info::out) is det.
 
-expand_try_goals_in_cases(_Instmap0, [], [], !Info).
-expand_try_goals_in_cases(Instmap0, [Case0 | Cases0], [Case | Cases], !Info) :-
+expand_try_goals_in_cases(_InstMap0, [], [], !Info).
+expand_try_goals_in_cases(InstMap0, [Case0 | Cases0], [Case | Cases], !Info) :-
     Case0 = case(MainConsId, OtherConsIds, Goal0),
-    expand_try_goals_in_goal(Instmap0, Goal0, Goal, !Info),
-    expand_try_goals_in_cases(Instmap0, Cases0, Cases, !Info),
+    expand_try_goals_in_goal(InstMap0, Goal0, Goal, !Info),
+    expand_try_goals_in_cases(InstMap0, Cases0, Cases, !Info),
     Case = case(MainConsId, OtherConsIds, Goal).
 
 :- pred expand_try_goals_in_if_then_else(instmap::in,
@@ -462,47 +468,47 @@ expand_try_goals_in_cases(Instmap0, [Case0 | Cases0], [Case | Cases], !Info) :-
     hlds_goal::out, hlds_goal::in, hlds_goal::out,
     trys_info::in, trys_info::out) is det.
 
-expand_try_goals_in_if_then_else(Instmap0, Cond0, Cond, Then0, Then,
+expand_try_goals_in_if_then_else(InstMap0, Cond0, Cond, Then0, Then,
         Else0, Else, !Info) :-
-    expand_try_goals_in_goal(Instmap0, Cond0, Cond, !Info),
+    expand_try_goals_in_goal(InstMap0, Cond0, Cond, !Info),
 
     Cond0 = hlds_goal(_, CondInfo),
-    CondInstmapDelta = goal_info_get_instmap_delta(CondInfo),
-    apply_instmap_delta(Instmap0, CondInstmapDelta, InstmapAfterCond),
-    expand_try_goals_in_goal(InstmapAfterCond, Then0, Then, !Info),
+    CondInstMapDelta = goal_info_get_instmap_delta(CondInfo),
+    apply_instmap_delta(CondInstMapDelta, InstMap0, InstMapAfterCond),
+    expand_try_goals_in_goal(InstMapAfterCond, Then0, Then, !Info),
 
-    expand_try_goals_in_goal(Instmap0, Else0, Else, !Info).
+    expand_try_goals_in_goal(InstMap0, Else0, Else, !Info).
 
 %-----------------------------------------------------------------------------%
 
-:- inst try_goal
+:- inst try_goal for shorthand_goal_expr/0
     --->    try_goal(ground, ground, ground).
 
 :- pred expand_try_goal(instmap::in, shorthand_goal_expr::in(try_goal),
     hlds_goal::out, trys_info::in, trys_info::out) is det.
 
-expand_try_goal(Instmap, TryGoal, FinalGoal, !Info) :-
+expand_try_goal(InstMap, TryGoal, FinalGoal, !Info) :-
     TryGoal = try_goal(MaybeIO, ResultVar, IntermediateGoal),
     extract_intermediate_goal_parts(!.Info ^ ti_module_info, ResultVar,
         IntermediateGoal, Goal0, Then0, MaybeElse0, ExcpHandling0),
 
     % Handle nested try goals.
-    expand_try_goals_in_goal(Instmap, Goal0, Goal1, !Info),
-    update_instmap(Goal0, Instmap, InstmapAfterGoal),
-    expand_try_goals_in_goal(InstmapAfterGoal, Then0, Then1, !Info),
+    expand_try_goals_in_goal(InstMap, Goal0, Goal1, !Info),
+    update_instmap(Goal0, InstMap, InstMapAfterGoal),
+    expand_try_goals_in_goal(InstMapAfterGoal, Then0, Then1, !Info),
     (
         MaybeElse0 = yes(Else0),
-        expand_try_goals_in_goal(Instmap, Else0, Else1, !Info),
+        expand_try_goals_in_goal(InstMap, Else0, Else1, !Info),
         MaybeElse1 = yes(Else1)
     ;
         MaybeElse0 = no,
         MaybeElse1 = no
     ),
-    expand_try_goals_in_goal(Instmap, ExcpHandling0, ExcpHandling1, !Info),
+    expand_try_goals_in_goal(InstMap, ExcpHandling0, ExcpHandling1, !Info),
 
     % Find the output variables.  Note we use Goal0, not Goal1, as any nested
     % tries would have been transformed will mess up the calculation.
-    bound_nonlocals_in_goal(!.Info ^ ti_module_info, Instmap, Goal0,
+    bound_nonlocals_in_goal(!.Info ^ ti_module_info, InstMap, Goal0,
         GoalOutputVarsSet0),
     (
         MaybeIO = yes(try_io_state_vars(_IOStateVarInitial, IOStateVarFinal)),
@@ -516,7 +522,7 @@ expand_try_goal(Instmap, TryGoal, FinalGoal, !Info) :-
     some [!ModuleInfo, !PredInfo, !ProcInfo, !VarTypes] (
         !.Info = trys_info(!:ModuleInfo, !:PredInfo, !:ProcInfo, _),
         expand_try_goal_2(MaybeIO, ResultVar, Goal1, Then1, MaybeElse1,
-            ExcpHandling1, InstmapAfterGoal, GoalOutputVarsSet, FinalGoal,
+            ExcpHandling1, InstMapAfterGoal, GoalOutputVarsSet, FinalGoal,
             !PredInfo, !ProcInfo, !ModuleInfo),
         !:Info = trys_info(!.ModuleInfo, !.PredInfo, !.ProcInfo, yes)
     ).
@@ -528,7 +534,7 @@ expand_try_goal(Instmap, TryGoal, FinalGoal, !Info) :-
     module_info::in, module_info::out) is det.
 
 expand_try_goal_2(MaybeIO, ResultVar, Goal1, Then1, MaybeElse1, ExcpHandling1,
-        Instmap, GoalOutputVarsSet, FinalGoal,
+        InstMap, GoalOutputVarsSet, FinalGoal,
         !PredInfo, !ProcInfo, !ModuleInfo) :-
     some [!VarTypes] (
         % Get the type of the output tuple.
@@ -608,7 +614,7 @@ expand_try_goal_2(MaybeIO, ResultVar, Goal1, Then1, MaybeElse1, ExcpHandling1,
         TupleVar, !ProcInfo),
     deconstruct_functor(ResultVar, exception_succeeded_functor, [TmpTupleVar],
         DeconstructSucceeded),
-    instmap_lookup_vars(Instmap, GoalOutputVars, TupleArgInsts),
+    instmap_lookup_vars(InstMap, GoalOutputVars, TupleArgInsts),
     make_output_tuple_inst_cast(TmpTupleVar, TupleVar, TupleArgInsts,
         CastOutputTuple),
     deconstruct_tuple(TupleVar, GoalOutputVars, DeconstructOutputs),
@@ -642,16 +648,16 @@ expand_try_goal_2(MaybeIO, ResultVar, Goal1, Then1, MaybeElse1, ExcpHandling1,
 
 extract_intermediate_goal_parts(ModuleInfo, ResultVar, IntermediateGoal,
         Goal, Then, MaybeElse, ExcpHandling) :-
-    (
+    ( if
         extract_intermediate_goal_parts_2(ModuleInfo, ResultVar,
             IntermediateGoal, GoalPrime, ThenPrime, MaybeElsePrime,
             ExcpHandlingPrime)
-    ->
+    then
         Goal = GoalPrime,
         Then = ThenPrime,
         MaybeElse = MaybeElsePrime,
         ExcpHandling = ExcpHandlingPrime
-    ;
+    else if
         % This form should only be encountered if there was an error in the
         % program, when the inner goal may fail, in a context where it must not
         % fail.  Compilation doesn't stop immediately after determinism
@@ -660,13 +666,13 @@ extract_intermediate_goal_parts(ModuleInfo, ResultVar, IntermediateGoal,
         extract_intermediate_goal_parts_2(ModuleInfo, ResultVar,
             ScopedGoal, GoalPrime, ThenPrime, MaybeElsePrime,
             ExcpHandlingPrime)
-    ->
+    then
         Goal = GoalPrime,
         Then = ThenPrime,
         MaybeElse = MaybeElsePrime,
         ExcpHandling = ExcpHandlingPrime
-    ;
-        unexpected($module, $pred, "unexpected goal form")
+    else
+        unexpected($pred, "unexpected goal form")
     ).
 
 :- pred extract_intermediate_goal_parts_2(module_info::in, prog_var::in,
@@ -694,9 +700,9 @@ extract_intermediate_goal_parts_2(ModuleInfo, ResultVar, IntermediateGoal,
     %
     %      TryResult = exception.succeeded(V),
     %      V = {},
-    %      ( Goal ->
+    %      ( if Goal then
     %          Then
-    %      ;
+    %      else
     %          Else
     %      ),
     %      Rest
@@ -718,12 +724,12 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
     Conjuncts0 = [DeconstructResult, TestNullTuple | Conjuncts1],
     DeconstructResult = hlds_goal(unify(_ResultVar, _, _, _, _), _),
     TestNullTuple = hlds_goal(unify(_, TestRHS, _, _, _), _),
-    TestRHS = rhs_functor(tuple_cons(0), no, []),
+    TestRHS = rhs_functor(tuple_cons(0), is_not_exist_constr, []),
 
-    (
+    ( if
         Conjuncts1 = [hlds_goal(IfThenElse, _) | Rest],
         IfThenElse = if_then_else(_, GoalPrime, Then0, Else0)
-    ->
+    then
         Goal = GoalPrime,
 
         % If Goal is erroneous the Then part may have been optimised away to
@@ -747,16 +753,16 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
 
         conjoin_goal_and_goal_list(Else0, Rest, Else),
         MaybeElse = yes(Else)
-    ;
+    else
         Conjuncts1 = [SomeGoal | AfterSomeGoal],
         SomeGoal = hlds_goal(scope(exist_quant([]), Goal), _),
-        (
+        ( if
             AfterSomeGoal = [SomeThen | Rest],
             SomeThen = hlds_goal(scope(exist_quant([]), Then0), _)
-        ->
+        then
             conjoin_goal_and_goal_list(Then0, Rest, Then),
             MaybeElse = no
-        ;
+        else
             % If "some [] Then" is missing then "some [] Goal" must be
             % `erroneous'.  Make the Then part into a call to an erroneous
             % procedure.
@@ -772,7 +778,7 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
                 ; GoalMaxSoln = at_most_many_cc
                 ; GoalMaxSoln = at_most_many
                 ),
-                unexpected($module, $pred, "goal not erroneous")
+                unexpected($pred, "goal not erroneous")
             )
         )
     ).
@@ -780,12 +786,12 @@ extract_from_succeeded_goal(ModuleInfo, SucceededGoal, Goal, Then,
 :- pred lookup_case_goal(list(case)::in, cons_id::in, hlds_goal::out) is det.
 
 lookup_case_goal([], ConsId, _) :-
-    unexpected($module, $pred, "couldn't find " ++ string(ConsId)).
-lookup_case_goal([H | T], ConsId, Goal) :-
-    ( H = case(ConsId, [], GoalPrime) ->
+    unexpected($pred, "couldn't find " ++ string(ConsId)).
+lookup_case_goal([Case | Cases], ConsId, Goal) :-
+    ( if Case = case(ConsId, [], GoalPrime) then
         Goal = GoalPrime
-    ;
-        lookup_case_goal(T, ConsId, Goal)
+    else
+        lookup_case_goal(Cases, ConsId, Goal)
     ).
 
 :- pred bound_nonlocals_in_goal(module_info::in, instmap::in, hlds_goal::in,
@@ -794,9 +800,9 @@ lookup_case_goal([H | T], ConsId, Goal) :-
 bound_nonlocals_in_goal(ModuleInfo, InstMap, Goal, BoundNonLocals) :-
     Goal = hlds_goal(_, GoalInfo),
     NonLocals = goal_info_get_nonlocals(GoalInfo),
-    InstmapDelta = goal_info_get_instmap_delta(GoalInfo),
+    InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
     BoundNonLocals = set_of_var.filter(
-        var_is_bound_in_instmap_delta(ModuleInfo, InstMap, InstmapDelta),
+        var_is_bound_in_instmap_delta(ModuleInfo, InstMap, InstMapDelta),
         NonLocals).
 
 :- pred make_try_lambda(hlds_goal::in, set_of_progvar::in, mer_type::in,
@@ -804,7 +810,7 @@ bound_nonlocals_in_goal(ModuleInfo, InstMap, Goal, BoundNonLocals) :-
     proc_info::in, proc_info::out) is det.
 
 make_try_lambda(Body0, OutputVarsSet, OutputTupleType, MaybeIO,
-        LambdaVar, AssignLambdaVar, !ProcInfo) :-
+        LambdaVar, AssignLambdaVarGoal, !ProcInfo) :-
     Body0 = hlds_goal(_, BodyInfo0),
     NonLocals0 = goal_info_get_nonlocals(BodyInfo0),
     set_of_var.difference(NonLocals0, OutputVarsSet, NonLocals1),
@@ -813,19 +819,18 @@ make_try_lambda(Body0, OutputVarsSet, OutputTupleType, MaybeIO,
         OutputTupleVar, !ProcInfo),
     (
         MaybeIO = yes(try_io_state_vars(IOVarInitial, IOVarFinal)),
-        LambdaParams = [OutputTupleVar, IOVarInitial, IOVarFinal],
+        LambdaParamsModes = [OutputTupleVar - out_mode,
+            IOVarInitial - di_mode, IOVarFinal - uo_mode],
         LambdaParamTypes = [OutputTupleType, io_state_type, io_state_type],
-        LambdaParamModes = [out_mode, di_mode, uo_mode],
         set_of_var.delete(IOVarFinal, NonLocals1, NonLocals)
     ;
         MaybeIO = no,
-        LambdaParams = [OutputTupleVar],
+        LambdaParamsModes = [OutputTupleVar - out_mode],
         LambdaParamTypes = [OutputTupleType],
-        LambdaParamModes = [out_mode],
         NonLocals = NonLocals0
     ),
-    LambdaType = higher_order_type(LambdaParamTypes, no, purity_pure,
-        lambda_normal),
+    LambdaType = higher_order_type(pf_predicate, LambdaParamTypes,
+        none_or_default_func, purity_pure, lambda_normal),
     proc_info_create_var_from_type(LambdaType, yes("TryLambda"), LambdaVar,
         !ProcInfo),
 
@@ -850,11 +855,14 @@ make_try_lambda(Body0, OutputVarsSet, OutputTupleType, MaybeIO,
     detism_to_try_lambda_detism(BodyDetism, LambdaDetism),
 
     % Make the lambda assignment.
-    RHS = rhs_lambda_goal(purity_pure, ho_ground, pf_predicate,
-        lambda_normal, set_of_var.to_sorted_list(NonLocals),
-        LambdaParams, LambdaParamModes, LambdaDetism, LambdaBody),
+    RHS = rhs_lambda_goal(purity_pure, ho_ground, pf_predicate, lambda_normal,
+        set_of_var.to_sorted_list(NonLocals), LambdaParamsModes,
+        LambdaDetism, LambdaBody),
     create_pure_atomic_complicated_unification(LambdaVar, RHS,
-        term.context_init, umc_implicit("try_expand"), [], AssignLambdaVar).
+        term.context_init, umc_implicit("try_expand"), [],
+        AssignLambdaVarGoal0),
+    goal_add_feature(feature_lambda_from_try,
+        AssignLambdaVarGoal0, AssignLambdaVarGoal).
 
     % try* don't cover all possible determinisms so we have generate lambdas
     % with less restrictive determinisms.
@@ -878,18 +886,16 @@ detism_to_try_lambda_detism(detism_failure, detism_semi).
 
 make_try_call(PredName, LambdaVar, ResultVar, ExtraArgs, OutputTupleType,
         GoalPurity, Context, OverallGoal, !PredInfo, !ProcInfo, !ModuleInfo) :-
-    create_poly_info(!.ModuleInfo, !.PredInfo, !.ProcInfo, PolyInfo0),
-    polymorphism_make_type_info_var(OutputTupleType, Context,
-        TypeInfoVar, MakeTypeInfoGoals, PolyInfo0, PolyInfo),
-    poly_info_extract(PolyInfo, !PredInfo, !ProcInfo, !:ModuleInfo),
+    polymorphism_make_type_info_var_raw(OutputTupleType, Context,
+        TypeInfoVar, MakeTypeInfoGoals, !ModuleInfo, !PredInfo, !ProcInfo),
 
     % The mode will be fixed up by a later analysis.
     Mode = mode_no(0),
     Args = [TypeInfoVar, LambdaVar, ResultVar] ++ ExtraArgs,
     Features = [],
-    generate_simple_call(mercury_exception_module, PredName,
+    generate_simple_call(!.ModuleInfo, mercury_exception_module, PredName,
         pf_predicate, Mode, detism_cc_multi, purity_pure, Args, Features,
-        instmap_delta_bind_no_var, !.ModuleInfo, Context, CallGoal0),
+        instmap_delta_bind_no_var, Context, CallGoal0),
 
     goal_info_init(Context, GoalInfo),
 
@@ -912,10 +918,9 @@ make_try_call(PredName, LambdaVar, ResultVar, ExtraArgs, OutputTupleType,
 :- pred make_unreachable_call(module_info::in, hlds_goal::out) is det.
 
 make_unreachable_call(ModuleInfo, Goal) :-
-    generate_simple_call(mercury_exception_module, "unreachable",
+    generate_simple_call(ModuleInfo, mercury_exception_module, "unreachable",
         pf_predicate, only_mode, detism_erroneous, purity_pure,
-        [], [], instmap_delta_bind_no_var, ModuleInfo, term.context_init,
-        Goal).
+        [], [], instmap_delta_bind_no_var, term.context_init, Goal).
 
 :- pred make_output_tuple_inst_cast(prog_var::in, prog_var::in,
     list(mer_inst)::in, hlds_goal::out) is det.
@@ -923,17 +928,17 @@ make_unreachable_call(ModuleInfo, Goal) :-
 make_output_tuple_inst_cast(TmpTupleVar, TupleVar, TupleArgInsts,
         CastOrUnify) :-
     % If all the arguments have inst `ground' then a unification is enough.
-    (
+    ( if
         list.member(ArgInst, TupleArgInsts),
-        ArgInst \= ground(_, none)
-    ->
+        ArgInst \= ground(_, none_or_default_func)
+    then
         TupleArity = list.length(TupleArgInsts),
         TupleInst = bound(shared, inst_test_no_results, [
             bound_functor(tuple_cons(TupleArity), TupleArgInsts)
         ]),
         generate_cast_with_insts(unsafe_type_inst_cast, TmpTupleVar, TupleVar,
             ground_inst, TupleInst, term.context_init, CastOrUnify)
-    ;
+    else
         create_pure_atomic_complicated_unification(TupleVar,
             rhs_var(TmpTupleVar), term.context_init,
             umc_implicit("try_expand"), [], CastOrUnify)
