@@ -56,7 +56,7 @@
 
     % Just check a single procedure.
     %
-:- pred unique_modes_check_proc(proc_id::in, pred_id::in,
+:- pred unique_modes_check_proc(pred_id::in, proc_id::in,
     module_info::in, module_info::out, bool::out, list(error_spec)::out)
     is det.
 
@@ -65,22 +65,23 @@
 :- pred unique_modes_check_goal(hlds_goal::in, hlds_goal::out,
     mode_info::in, mode_info::out) is det.
 
-    % Prepare for checking a disjunct in a disjunction.
-    %
-:- pred prepare_for_disjunct(hlds_goal::in, determinism::in,
-    set_of_progvar::in, mode_info::in, mode_info::out) is det.
-
     % Make all nondet-live variables whose current inst
     % is `unique' become `mostly_unique'.
     %
 :- pred make_all_nondet_live_vars_mostly_uniq(mode_info::in, mode_info::out)
     is det.
 
+    % Prepare for checking a disjunct in a disjunction.
+    %
+:- pred prepare_for_disjunct(hlds_goal::in, determinism::in,
+    set_of_progvar::in, mode_info::in, mode_info::out) is det.
+
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- implementation.
 
+:- import_module check_hlds.inst_lookup.
 :- import_module check_hlds.inst_match.
 :- import_module check_hlds.inst_util.
 :- import_module check_hlds.mode_debug.
@@ -91,13 +92,15 @@
 :- import_module check_hlds.modecheck_util.
 :- import_module check_hlds.modes.
 :- import_module hlds.instmap.
+:- import_module hlds.make_goal.
+:- import_module hlds.vartypes.
 :- import_module mdbcomp.
-:- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.prog_data_foreign.
 :- import_module parse_tree.prog_mode.
 
 :- import_module bag.
 :- import_module int.
-:- import_module map.
 :- import_module maybe.
 :- import_module pair.
 :- import_module require.
@@ -110,32 +113,30 @@ unique_modes_check_module(!ModuleInfo, Specs) :-
     check_pred_modes(check_unique_modes, may_change_called_proc,
         !ModuleInfo, _SafeToContinue, Specs).
 
-unique_modes_check_proc(ProcId, PredId, !ModuleInfo, Changed, Specs) :-
-    modecheck_proc_general(ProcId, PredId, check_unique_modes,
-        may_change_called_proc, !ModuleInfo, Specs, Changed).
+unique_modes_check_proc(PredId, ProcId, !ModuleInfo, Changed, Specs) :-
+    modecheck_proc_general(check_unique_modes, may_change_called_proc,
+        PredId, ProcId, !ModuleInfo, Changed, Specs).
 
 unique_modes_check_goal(Goal0, Goal, !ModeInfo) :-
     Goal0 = hlds_goal(GoalExpr0, GoalInfo0),
     Context = goal_info_get_context(GoalInfo0),
-    term.context_init(EmptyContext),
-    ( Context = EmptyContext ->
+    ( if is_dummy_context(Context) then
         true
-    ;
+    else
         mode_info_set_context(Context, !ModeInfo)
     ),
-    ( goal_info_has_feature(GoalInfo0, feature_duplicated_for_switch) ->
+    ( if goal_info_has_feature(GoalInfo0, feature_duplicated_for_switch) then
         mode_info_get_in_dupl_for_switch(!.ModeInfo, InDuplForSwitch),
         mode_info_set_in_dupl_for_switch(in_dupl_for_switch, !ModeInfo),
         unique_modes_check_goal_2(GoalExpr0, GoalInfo0, Goal, !ModeInfo),
         mode_info_set_in_dupl_for_switch(InDuplForSwitch, !ModeInfo)
-    ;
+    else
         unique_modes_check_goal_2(GoalExpr0, GoalInfo0, Goal, !ModeInfo)
     ).
 
 :- pred unique_modes_check_goal_2(hlds_goal_expr::in, hlds_goal_info::in,
     hlds_goal::out, mode_info::in, mode_info::out) is det.
-
-:- pragma inline(unique_modes_check_goal_2/5).
+:- pragma inline(pred(unique_modes_check_goal_2/5)).
 
 unique_modes_check_goal_2(GoalExpr0, GoalInfo0, Goal, !ModeInfo) :-
     mode_info_get_instmap(!.ModeInfo, InstMap0),
@@ -145,9 +146,9 @@ unique_modes_check_goal_2(GoalExpr0, GoalInfo0, Goal, !ModeInfo) :-
     % If the goal is not nondet, then nothing is nondet-live, so reset the bag
     % of nondet-live vars to be empty.
     Detism = goal_info_get_determinism(GoalInfo0),
-    ( determinism_components(Detism, _, at_most_many) ->
+    ( if determinism_components(Detism, _, at_most_many) then
         true
-    ;
+    else
         mode_info_set_nondet_live_vars(bag.init, !ModeInfo)
     ),
 
@@ -163,11 +164,11 @@ unique_modes_check_goal_2(GoalExpr0, GoalInfo0, Goal, !ModeInfo) :-
 
 make_all_nondet_live_vars_mostly_uniq(ModeInfo0, ModeInfo) :-
     mode_info_get_instmap(ModeInfo0, FullInstMap0),
-    ( instmap_is_reachable(FullInstMap0) ->
+    ( if instmap_is_reachable(FullInstMap0) then
         instmap_vars_list(FullInstMap0, AllVars),
         select_nondet_live_vars(AllVars, ModeInfo0, NondetLiveVars),
         make_var_list_mostly_uniq(NondetLiveVars, ModeInfo0, ModeInfo)
-    ;
+    else
         ModeInfo = ModeInfo0
     ).
 
@@ -176,10 +177,10 @@ make_all_nondet_live_vars_mostly_uniq(ModeInfo0, ModeInfo) :-
 
 select_live_vars([], _, []).
 select_live_vars([Var|Vars], ModeInfo, LiveVars) :-
-    ( mode_info_var_is_live(ModeInfo, Var, is_live) ->
+    ( if mode_info_var_is_live(ModeInfo, Var, is_live) then
         select_live_vars(Vars, ModeInfo, LiveVars1),
         LiveVars = [Var | LiveVars1]
-    ;
+    else
         select_live_vars(Vars, ModeInfo, LiveVars)
     ).
 
@@ -188,10 +189,10 @@ select_live_vars([Var|Vars], ModeInfo, LiveVars) :-
 
 select_nondet_live_vars([], _, []).
 select_nondet_live_vars([Var|Vars], ModeInfo, NondetLiveVars) :-
-    ( mode_info_var_is_nondet_live(ModeInfo, Var, is_live) ->
+    ( if mode_info_var_is_nondet_live(ModeInfo, Var, is_live) then
         select_nondet_live_vars(Vars, ModeInfo, NondetLiveVars1),
         NondetLiveVars = [Var | NondetLiveVars1]
-    ;
+    else
         select_nondet_live_vars(Vars, ModeInfo, NondetLiveVars)
     ).
 
@@ -209,14 +210,14 @@ select_changed_inst_vars([Var | Vars], DeltaInstMap, ModeInfo, ChangedVars) :-
     instmap_lookup_var(InstMap0, Var, Inst0),
     mode_info_get_var_types(ModeInfo, VarTypes),
     lookup_var_type(VarTypes, Var, Type),
-    (
+    ( if
         instmap_delta_is_reachable(DeltaInstMap),
         instmap_delta_search_var(DeltaInstMap, Var, Inst),
-        \+ inst_matches_final_typed(Inst, Inst0, Type, ModuleInfo)
-    ->
+        not inst_matches_final_typed(ModuleInfo, Type, Inst, Inst0)
+    then
         select_changed_inst_vars(Vars, DeltaInstMap, ModeInfo, ChangedVars1),
         ChangedVars = [Var | ChangedVars1]
-    ;
+    else
         select_changed_inst_vars(Vars, DeltaInstMap, ModeInfo, ChangedVars)
     ).
 
@@ -234,7 +235,7 @@ make_var_list_mostly_uniq([Var | Vars], !ModeInfo) :-
 make_var_mostly_uniq(Var, !ModeInfo) :-
     mode_info_get_instmap(!.ModeInfo, InstMap0),
     mode_info_get_module_info(!.ModeInfo, ModuleInfo0),
-    (
+    ( if
         % Only variables which are `unique' need to be changed.
         instmap_is_reachable(InstMap0),
         instmap_vars_list(InstMap0, Vars),
@@ -245,12 +246,12 @@ make_var_mostly_uniq(Var, !ModeInfo) :-
         ; Inst1 = bound(unique, _, _)
         ; Inst1 = any(unique, _)
         )
-    ->
+    then
         make_mostly_uniq_inst(Inst0, Inst, ModuleInfo0, ModuleInfo),
         mode_info_set_module_info(ModuleInfo, !ModeInfo),
         instmap_set_var(Var, Inst, InstMap0, InstMap),
         mode_info_set_instmap(InstMap, !ModeInfo)
-    ;
+    else
         true
     ).
 
@@ -285,12 +286,10 @@ unique_modes_check_goal_expr(GoalExpr0, GoalInfo0, GoalExpr, !ModeInfo) :-
             PragmaCode0, GoalInfo0, GoalExpr, !ModeInfo)
     ;
         GoalExpr0 = conj(GoalType0, Goals0),
-        unique_modes_check_goal_conj(GoalType0, Goals0, GoalExpr,
-            !ModeInfo)
+        unique_modes_check_goal_conj(GoalType0, Goals0, GoalExpr, !ModeInfo)
     ;
         GoalExpr0 = disj(Goals0),
-        unique_modes_check_goal_disj(Goals0, GoalInfo0, GoalExpr,
-            !ModeInfo)
+        unique_modes_check_goal_disj(Goals0, GoalInfo0, GoalExpr, !ModeInfo)
     ;
         GoalExpr0 = switch(Var0, CanFail0, Cases0),
         unique_modes_check_goal_switch(Var0, CanFail0, Cases0, GoalInfo0,
@@ -325,7 +324,7 @@ unique_modes_check_goal_expr(GoalExpr0, GoalInfo0, GoalExpr, !ModeInfo) :-
         ;
             ShortHand0 = bi_implication(_, _),
             % These should have been expanded out by now.
-            unexpected($module, $pred, "bi_implication")
+            unexpected($pred, "bi_implication")
         )
     ).
 
@@ -333,7 +332,7 @@ unique_modes_check_goal_expr(GoalExpr0, GoalInfo0, GoalExpr, !ModeInfo) :-
     hlds_goal_expr::out, mode_info::in, mode_info::out) is det.
 
 unique_modes_check_goal_conj(ConjType, Goals0, GoalExpr, !ModeInfo) :-
-    mode_checkpoint(enter, "*conj", !ModeInfo),
+    mode_checkpoint(enter, "conj", !ModeInfo),
     (
         Goals0 = [],
         % For efficiency, optimize common case.
@@ -344,7 +343,7 @@ unique_modes_check_goal_conj(ConjType, Goals0, GoalExpr, !ModeInfo) :-
         unique_modes_check_conj(ConjType, Goals0, Goals, !ModeInfo)
     ),
     GoalExpr = conj(ConjType, Goals),
-    mode_checkpoint(exit, "*conj", !ModeInfo).
+    mode_checkpoint(exit, "conj", !ModeInfo).
 
 :- pred unique_modes_check_goal_disj(list(hlds_goal)::in,
     hlds_goal_info::in, hlds_goal_expr::out,
@@ -372,11 +371,11 @@ unique_modes_check_goal_disj(Goals0, GoalInfo0, GoalExpr, !ModeInfo) :-
         NonLocals = goal_info_get_nonlocals(GoalInfo0),
         Determinism = goal_info_get_determinism(GoalInfo0),
         % Does this disjunction create a choice point?
-        ( determinism_components(Determinism, _, at_most_many) ->
+        ( if determinism_components(Determinism, _, at_most_many) then
             mode_info_add_live_vars(NonLocals, !ModeInfo),
             make_all_nondet_live_vars_mostly_uniq(!ModeInfo),
             mode_info_remove_live_vars(NonLocals, !ModeInfo)
-        ;
+        else
             true
         ),
 
@@ -444,10 +443,10 @@ unique_modes_check_goal_if_then_else(Vars, Cond0, Then0, Else0, GoalInfo0,
     mode_info_remove_live_vars(ThenVars, !ModeInfo),
     mode_info_unlock_vars(var_lock_if_then_else, NonLocals, !ModeInfo),
     mode_info_get_instmap(!.ModeInfo, InstMapCond),
-    ( instmap_is_reachable(InstMapCond) ->
+    ( if instmap_is_reachable(InstMapCond) then
         unique_modes_check_goal(Then0, Then, !ModeInfo),
         mode_info_get_instmap(!.ModeInfo, InstMapThen)
-    ;
+    else
         % We should not mode-analyse the goal, since it is unreachable.
         % Instead we optimize the goal away, so that later passes
         % won't complain about it not having unique mode information.
@@ -532,15 +531,15 @@ unique_modes_check_goal_scope(Reason, SubGoal0, GoalInfo0, GoalExpr,
                 SubGoal = SubGoal0,
                 SubGoal = hlds_goal(_, SubGoalInfo),
                 InstMapDelta = goal_info_get_instmap_delta(SubGoalInfo),
-                (
+                ( if
                     instmap_delta_search_var(InstMapDelta, TermVar,
                         TermVarInst)
-                ->
+                then
                     mode_info_get_instmap(!.ModeInfo, InstMap0),
                     instmap_set_var(TermVar, TermVarInst, InstMap0, InstMap),
                     mode_info_set_instmap(InstMap, !ModeInfo)
-                ;
-                    unexpected($module, $pred, "bad InstMapDelta")
+                else
+                    unexpected($pred, "bad InstMapDelta")
                 ),
                 GoalExpr = scope(Reason, SubGoal)
             ;
@@ -560,14 +559,16 @@ unique_modes_check_goal_scope(Reason, SubGoal0, GoalInfo0, GoalExpr,
             GoalExpr = scope(Reason, SubGoal)
         ;
             FGT = from_ground_term_initial,
-            unexpected($module, $pred, "from_ground_term_initial")
+            unexpected($pred, "from_ground_term_initial")
         )
     ;
-        ( Reason = exist_quant(_)
+        ( Reason = disable_warnings(_, _)
+        ; Reason = exist_quant(_)
         ; Reason = promise_solutions(_, _)
         ; Reason = promise_purity(_)
         ; Reason = require_detism(_)
         ; Reason = require_complete_switch(_)
+        ; Reason = require_switch_arms_detism(_, _)
         ; Reason = commit(_)
         ; Reason = barrier(_)
         ; Reason = loop_control(_, _, _)
@@ -586,12 +587,13 @@ unique_modes_check_goal_scope(Reason, SubGoal0, GoalInfo0, GoalExpr,
 unique_modes_check_goal_generic_call(GenericCall, ArgVars, Modes,
         MaybeRegTypes, Detism, GoalExpr, !ModeInfo) :-
     mode_checkpoint(enter, "generic_call", !ModeInfo),
-    hlds_goal.generic_call_id(GenericCall, CallId),
+    hlds_goal.generic_call_to_id(GenericCall, GenericCallId),
+    CallId = mode_call_generic(GenericCallId),
     mode_info_set_call_context(call_context_call(CallId), !ModeInfo),
-    ( determinism_components(Detism, _, at_most_zero) ->
-        NeverSucceeds = yes
-    ;
-        NeverSucceeds = no
+    ( if determinism_components(Detism, _, at_most_zero) then
+        CanProcSucceed = proc_cannot_succeed
+    else
+        CanProcSucceed = proc_can_maybe_succeed
     ),
     (
         GenericCall = higher_order(_, _, _, _),
@@ -606,11 +608,12 @@ unique_modes_check_goal_generic_call(GenericCall, ArgVars, Modes,
         ArgOffset = 0
     ;
         % Casts are introduced by the compiler and should be mode correct.
+        % Coercions are mode checked.
         GenericCall = cast(_),
         ArgOffset = 0
     ),
     unique_modes_check_call_modes(ArgVars, Modes, ArgOffset, Detism,
-        NeverSucceeds, !ModeInfo),
+        CanProcSucceed, !ModeInfo),
     GoalExpr = generic_call(GenericCall, ArgVars, Modes, MaybeRegTypes,
         Detism),
     mode_info_unset_call_context(!ModeInfo),
@@ -626,8 +629,7 @@ unique_modes_check_goal_plain_call(PredId, ProcId0, ArgVars, Builtin,
     PredNameString = sym_name_to_string(PredName),
     string.append("call ", PredNameString, CallString),
     mode_checkpoint(enter, CallString, !ModeInfo),
-    mode_info_get_call_id(!.ModeInfo, PredId, CallId),
-    mode_info_set_call_context(call_context_call(plain_call_id(CallId)),
+    mode_info_set_call_context(call_context_call(mode_call_plain(PredId)),
         !ModeInfo),
     unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo0, ProcId,
         !ModeInfo),
@@ -675,7 +677,7 @@ unique_modes_check_goal_switch(Var, CanFail, Cases0, GoalInfo0, GoalExpr,
 :- pred unique_modes_check_goal_call_foreign_proc(
     pragma_foreign_proc_attributes::in, pred_id::in, proc_id::in,
     list(foreign_arg)::in, list(foreign_arg)::in,
-    maybe(trace_expr(trace_runtime))::in, pragma_foreign_code_impl::in,
+    maybe(trace_expr(trace_runtime))::in, pragma_foreign_proc_impl::in,
     hlds_goal_info::in, hlds_goal_expr::out,
     mode_info::in, mode_info::out) is det.
 
@@ -685,8 +687,7 @@ unique_modes_check_goal_call_foreign_proc(Attributes, PredId, ProcId0,
     % To modecheck a pragma_c_code, we just modecheck the proc for
     % which it is the goal.
     mode_checkpoint(enter, "foreign_proc", !ModeInfo),
-    mode_info_get_call_id(!.ModeInfo, PredId, CallId),
-    mode_info_set_call_context(call_context_call(plain_call_id(CallId)),
+    mode_info_set_call_context(call_context_call(mode_call_plain(PredId)),
         !ModeInfo),
     ArgVars = list.map(foreign_arg_var, Args),
     unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo0, ProcId,
@@ -714,16 +715,16 @@ unique_modes_check_goal_atomic_goal(GoalType, Outer, Inner, MaybeOutputVars,
     ;
         OrElseGoals0 = [_ | _],
         % The unique mode check on the or_else goals is very similar
-        % to the unique mode check for disjunctions.  Please see
+        % to the unique mode check for disjunctions. Please see
         % "unique_modes_check_goal_disj" for disjunctions for discussion
         % of this code.
         NonLocals = goal_info_get_nonlocals(GoalInfo0),
         Determinism = goal_info_get_determinism(GoalInfo0),
-        ( determinism_components(Determinism, _, at_most_many) ->
+        ( if determinism_components(Determinism, _, at_most_many) then
             mode_info_add_live_vars(NonLocals, !ModeInfo),
             make_all_nondet_live_vars_mostly_uniq(!ModeInfo),
             mode_info_remove_live_vars(NonLocals, !ModeInfo)
-        ;
+        else
             true
         ),
         Goals0 = [MainGoal0 | OrElseGoals0],
@@ -733,7 +734,7 @@ unique_modes_check_goal_atomic_goal(GoalType, Outer, Inner, MaybeOutputVars,
             Goals = [MainGoal | OrElseGoals]
         ;
             Goals = [],
-            unexpected($module, $pred, "Goals = []")
+            unexpected($pred, "Goals = []")
         ),
         make_arm_instmaps_for_goals(Goals, InstMaps, ArmInstMaps),
         instmap_merge(NonLocals, ArmInstMaps, merge_disj, !ModeInfo)
@@ -763,10 +764,10 @@ unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo, ProcId,
     compute_arg_offset(PredInfo, ArgOffset),
     proc_info_get_argmodes(ProcInfo, ProcArgModes0),
     proc_info_interface_determinism(ProcInfo, InterfaceDeterminism),
-    proc_info_never_succeeds(ProcInfo, NeverSucceeds),
+    proc_info_never_succeeds(ProcInfo, CanSucceed),
     unique_modes_check_call_modes(ArgVars, ProcArgModes0, ArgOffset,
-        InterfaceDeterminism, NeverSucceeds, !ModeInfo),
-    ModeErrors = ProcInfo ^ mode_errors,
+        InterfaceDeterminism, CanSucceed, !ModeInfo),
+    proc_info_get_mode_errors(ProcInfo, ModeErrors),
     (
         ModeErrors = [_ | _],
         % mode error in callee for this mode
@@ -775,7 +776,7 @@ unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo, ProcId,
         instmap_lookup_vars(InstMap, ArgVars, ArgInsts),
         mode_info_error(WaitingVars,
             mode_error_in_callee(ArgVars, ArgInsts, PredId, ProcId0,
-                ProcInfo ^ mode_errors),
+                ModeErrors),
             !ModeInfo)
     ;
         ModeErrors = []
@@ -814,17 +815,17 @@ unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo, ProcId,
             modecheck_call_pred(PredId, yes(Determinism), ProcId0, ProcId,
                 ArgVars, NewArgVars, GoalInfo, ExtraGoals, !ModeInfo),
 
-            (
+            ( if
                 NewArgVars = ArgVars,
                 ExtraGoals = no_extra_goals
-            ->
+            then
                 true
-            ;
+            else
                 % This shouldn't happen, since modes.m should do all the
                 % handling of implied modes.
                 % XXX It might happen, though, if the user writes strange code;
                 % we should report a proper error here.
-                unexpected($module, $pred, "call to implied mode?")
+                unexpected($pred, "call to implied mode?")
             )
         )
     ).
@@ -834,10 +835,11 @@ unique_modes_check_call(PredId, ProcId0, ArgVars, GoalInfo, ProcId,
     % is nondet-live and the required initial inst was unique.
     %
 :- pred unique_modes_check_call_modes(list(prog_var)::in, list(mer_mode)::in,
-    int::in, determinism::in, bool::in, mode_info::in, mode_info::out) is det.
+    int::in, determinism::in, can_proc_succeed::in,
+    mode_info::in, mode_info::out) is det.
 
 unique_modes_check_call_modes(ArgVars, ProcArgModes, ArgOffset, Determinism,
-        NeverSucceeds, !ModeInfo) :-
+        CanProcSucceed, !ModeInfo) :-
     mode_info_get_module_info(!.ModeInfo, ModuleInfo),
     mode_list_get_initial_insts(ModuleInfo, ProcArgModes, InitialInsts),
     modecheck_var_has_inst_list_no_exact_match(ArgVars, InitialInsts,
@@ -846,28 +848,28 @@ unique_modes_check_call_modes(ArgVars, ProcArgModes, ArgOffset, Determinism,
     inst_list_apply_substitution(InstVarSub, FinalInsts0, FinalInsts),
     modecheck_set_var_inst_list(ArgVars, InitialInsts, FinalInsts,
         ArgOffset, NewArgVars, ExtraGoals, !ModeInfo),
-    (
+    ( if
         NewArgVars = ArgVars,
         ExtraGoals = no_extra_goals
-    ->
+    then
         true
-    ;
+    else
         % This shouldn't happen, since modes.m should do all the handling
         % of implied modes.
-        unexpected($module, $pred, "call to implied mode?")
+        unexpected($pred, "call to implied mode?")
     ),
     (
-        NeverSucceeds = yes,
+        CanProcSucceed = proc_cannot_succeed,
         instmap.init_unreachable(InstMap),
         mode_info_set_instmap(InstMap, !ModeInfo)
     ;
-        NeverSucceeds = no,
+        CanProcSucceed = proc_can_maybe_succeed,
         % Check whether we are at a call to a nondet predicate.
         % If so, mark all the currently nondet-live variables
         % whose inst is `unique' as instead being only `mostly_unique'.
-        ( determinism_components(Determinism, _, at_most_many) ->
+        ( if determinism_components(Determinism, _, at_most_many) then
             make_all_nondet_live_vars_mostly_uniq(!ModeInfo)
-        ;
+        else
             true
         )
     ).
@@ -878,15 +880,15 @@ unique_modes_check_call_modes(ArgVars, ProcArgModes, ArgOffset, Determinism,
     list(hlds_goal)::in, list(hlds_goal)::out, mode_info::in, mode_info::out)
     is det.
 
-    % Just process each conjunct in turn.  Note that we don't do any
+    % Just process each conjunct in turn. Note that we don't do any
     % reordering of conjuncts here, although we do flatten conjunctions.
     %
 unique_modes_check_conj(_ConjType, [], [], !ModeInfo).
 unique_modes_check_conj(ConjType, [Goal0 | Goals0], Goals, !ModeInfo) :-
-    ( Goal0 = hlds_goal(conj(ConjType, ConjGoals), _) ->
+    ( if Goal0 = hlds_goal(conj(ConjType, ConjGoals), _) then
         list.append(ConjGoals, Goals0, Goals1),
         unique_modes_check_conj(ConjType, Goals1, Goals, !ModeInfo)
-    ;
+    else
         unique_modes_check_conj_2(ConjType, Goal0, Goals0, Goals, !ModeInfo)
     ).
 
@@ -900,33 +902,18 @@ unique_modes_check_conj_2(ConjType, Goal0, Goals0, [Goal | Goals],
     mode_info_remove_live_vars(NonLocals, !ModeInfo),
     unique_modes_check_goal(Goal0, Goal, !ModeInfo),
     mode_info_get_instmap(!.ModeInfo, InstMap),
-    ( instmap_is_unreachable(InstMap) ->
+    ( if instmap_is_unreachable(InstMap) then
         % We should not mode-analyse the remaining goals, since they are
         % unreachable. Instead we optimize them away, so that later passes
         % won't complain about them not having unique mode information.
         mode_info_remove_goals_live_vars(Goals0, !ModeInfo),
         Goals = []
-    ;
+    else
         unique_modes_check_conj(ConjType, Goals0, Goals, !ModeInfo)
     ).
 
 %-----------------------------------------------------------------------------%
 
-    % Make_par_conj_nonlocal_multiset builds a multiset (bag) of all
-    % the nonlocals of the conjuncts.
-    %
-:- pred make_par_conj_nonlocal_multiset(list(hlds_goal)::in,
-    bag(prog_var)::out) is det.
-
-make_par_conj_nonlocal_multiset([], Empty) :-
-    bag.init(Empty).
-make_par_conj_nonlocal_multiset([Goal | Goals], NonLocalsMultiSet) :-
-    make_par_conj_nonlocal_multiset(Goals, NonLocalsMultiSet0),
-    NonLocals = goal_get_nonlocals(Goal),
-    set_of_var.to_sorted_list(NonLocals, NonLocalsList),
-    bag.from_list(NonLocalsList, NonLocalsMultiSet1),
-    bag.union(NonLocalsMultiSet0, NonLocalsMultiSet1, NonLocalsMultiSet).
-
     % To unique-modecheck a parallel conjunction, we find the variables
     % that are nonlocal to more than one conjunct and make them shared,
     % then we unique-modecheck the conjuncts.
@@ -939,6 +926,7 @@ make_par_conj_nonlocal_multiset([Goal | Goals], NonLocalsMultiSet) :-
 :- pred unique_modes_check_par_conj(list(hlds_goal)::in, bag(prog_var)::in,
     list(hlds_goal)::out, list(pair(instmap, set_of_progvar))::out,
     mode_info::in, mode_info::out) is det.
+:- pragma consider_used(pred(unique_modes_check_par_conj/6)).
 
 unique_modes_check_par_conj(Goals0, NonLocalVarsBag, Goals, Instmaps,
         !ModeInfo) :-
@@ -1010,25 +998,26 @@ unique_modes_check_disj([Goal0 | Goals0], DisjDetism, DisjNonLocals,
         !ModeInfo).
 
 prepare_for_disjunct(Goal0, DisjDetism, DisjNonLocals, !ModeInfo) :-
-    (
+    determinism_components(DisjDetism, _, DisjMaxSolns),
+    Goal0 = hlds_goal(_, GoalInfo0),
+    Determinism = goal_info_get_determinism(GoalInfo0),
+    determinism_components(Determinism, CanFail, _),
+    ( if
         % If the disjunction was model_nondet, then we already marked all the
         % non-locals as only being mostly-unique, so we don't need to do
         % anything special here...
-        \+ determinism_components(DisjDetism, _, at_most_many),
+        DisjMaxSolns \= at_most_many,
 
         % ... but for model_semi or model_det disjunctions, if the _disjunct_
         % can fail, then we still might backtrack to another disjunct, so again
         % in that case we need to mark all the non-locals as being only
         % mostly-unique rather than unique.
-        Goal0 = hlds_goal(_, GoalInfo0),
-        Determinism = goal_info_get_determinism(GoalInfo0),
-        determinism_components(Determinism, CanFail, _),
         CanFail = can_fail
-    ->
+    then
         mode_info_add_live_vars(DisjNonLocals, !ModeInfo),
         make_all_nondet_live_vars_mostly_uniq(!ModeInfo),
         mode_info_remove_live_vars(DisjNonLocals, !ModeInfo)
-    ;
+    else
         true
     ).
 
@@ -1051,9 +1040,9 @@ unique_modes_check_case_list([Case0 | Cases0], Var, [Case | Cases],
     modecheck_functors_test(Var, MainConsId, OtherConsIds, !ModeInfo),
 
     mode_info_get_instmap(!.ModeInfo, InstMap1),
-    ( instmap_is_reachable(InstMap1) ->
+    ( if instmap_is_reachable(InstMap1) then
         unique_modes_check_goal(Goal0, Goal1, !ModeInfo)
-    ;
+    else
         % We should not mode-analyse the goal, since it is unreachable.
         % Instead we optimize the goal away, so that later passes
         % won't complain about it not having unique mode information.
