@@ -2,6 +2,7 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2000-2012 The University of Melbourne.
+% Copyright (C) 2015 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
 %-----------------------------------------------------------------------------%
@@ -59,6 +60,7 @@
 :- module transform_hlds.unneeded_code.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 :- import_module hlds.hlds_pred.
 
@@ -72,8 +74,9 @@
 
 :- implementation.
 
-:- import_module check_hlds.inst_match.
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.inst_test.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module hlds.goal_form.
 :- import_module hlds.goal_path.
 :- import_module hlds.hlds_goal.
@@ -81,11 +84,18 @@
 :- import_module hlds.hlds_out.hlds_out_goal.
 :- import_module hlds.hlds_out.hlds_out_util.
 :- import_module hlds.instmap.
+:- import_module hlds.make_goal.
 :- import_module hlds.passes_aux.
 :- import_module hlds.quantification.
+:- import_module hlds.vartypes.
+:- import_module libs.
 :- import_module libs.globals.
+:- import_module libs.optimization_options.
 :- import_module libs.options.
+:- import_module mdbcomp.
 :- import_module mdbcomp.goal_path.
+:- import_module parse_tree.
+:- import_module parse_tree.parse_tree_out_info.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.set_of_var.
 
@@ -95,7 +105,6 @@
 :- import_module io.
 :- import_module list.
 :- import_module map.
-:- import_module maybe.
 :- import_module pair.
 :- import_module require.
 :- import_module set.
@@ -181,9 +190,9 @@
     %       ... code that needs Y and Z ...
     %   ; % s2
     %       X = b,
-    %       ( Y = f ->
+    %       ( if Y = f then
     %           ... code that needs only Z ...
-    %       ;
+    %       else
     %           ... code that does not need Y or Z ...
     %       )
     %   )
@@ -217,31 +226,19 @@
 %-----------------------------------------------------------------------------%
 
 unneeded_process_proc_msg(PredProcId, !ProcInfo, !ModuleInfo) :-
+    trace [io(!IO)] (
+        write_proc_progress_message(!.ModuleInfo,
+            "Removing dead code in", PredProcId, !IO)
+    ),
     % The transformation considers every nonlocal variable of a goal
     % that is bound on entry to be consumed by that goal. If the nonlocal set
     % contains any such variables that are not actually needed by the goal,
     % then the transformation will not be as effective as it could be.
     % Therefore we preprocess the procedure body to ensure that the nonlocals
     % sets are accurate reflections of the true needs of goals.
-
-    trace [io(!IO)] (
-        write_proc_progress_message("% Removing dead code in ",
-            PredProcId, !.ModuleInfo, !IO)
-    ),
     unneeded_pre_process_proc(!ProcInfo),
     PredProcId = proc(PredId, _),
-    unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, 1, Successful),
-    trace [io(!IO)] (
-        (
-            Successful = yes,
-            write_proc_progress_message("% done.\n",
-                PredProcId, !.ModuleInfo, !IO)
-        ;
-            Successful = no,
-            write_proc_progress_message("% none found.\n",
-                PredProcId, !.ModuleInfo, !IO)
-        )
-    ).
+    unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, 1, _Successful).
 
 :- pred unneeded_pre_process_proc(proc_info::in, proc_info::out) is det.
 
@@ -289,8 +286,8 @@ unneeded_pre_process_proc(!ProcInfo) :-
     --->    uc_option_values(
                 uc_fully_strict         :: bool,
                 uc_reorder_conj         :: bool,
-                uc_copy_limit           :: int,
-                uc_debug                :: bool
+                uc_debug                :: bool,
+                uc_copy_limit           :: int
             ).
 
 :- type unneeded_code_info
@@ -309,46 +306,48 @@ unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, Pass, Successful) :-
     proc_info_get_goal(!.ProcInfo, Goal0),
     proc_info_get_varset(!.ProcInfo, VarSet0),
     proc_info_get_vartypes(!.ProcInfo, VarTypes0),
-    proc_info_get_initial_instmap(!.ProcInfo, !.ModuleInfo, InitInstMap),
+    proc_info_get_initial_instmap(!.ModuleInfo, !.ProcInfo, InitInstMap),
     Goal0 = hlds_goal(_, GoalInfo0),
     InstMapDelta = goal_info_get_instmap_delta(GoalInfo0),
-    instmap.apply_instmap_delta(InitInstMap, InstMapDelta, FinalInstMap),
+    apply_instmap_delta(InstMapDelta, InitInstMap, FinalInstMap),
     proc_info_instantiated_head_vars(!.ModuleInfo, !.ProcInfo, NeededVarsList),
     map.init(WhereNeededMap0),
-    NeededEverywhere = (pred(Var::in, NeededMap0::in, NeededMap::out) is det :-
-        map.det_insert(Var, everywhere, NeededMap0, NeededMap)
-    ),
+    NeededEverywhere =
+        ( pred(Var::in, NeededMap0::in, NeededMap::out) is det :-
+            map.det_insert(Var, everywhere, NeededMap0, NeededMap)
+        ),
     list.foldl(NeededEverywhere, NeededVarsList,
         WhereNeededMap0, WhereNeededMap1),
     module_info_get_globals(!.ModuleInfo, Globals),
     globals.lookup_bool_option(Globals, reorder_conj, ReorderConj),
     globals.lookup_bool_option(Globals, fully_strict, FullyStrict),
-    globals.lookup_int_option(Globals, unneeded_code_copy_limit, Limit),
+    globals.get_opt_tuple(Globals, OptTuple),
+    Limit = OptTuple ^ ot_opt_unneeded_code_copy_limit,
     globals.lookup_bool_option(Globals, unneeded_code_debug, Debug),
-    Options = uc_option_values(FullyStrict, ReorderConj, Limit, Debug),
+    Options = uc_option_values(FullyStrict, ReorderConj, Debug, Limit),
     (
         Debug = no
     ;
         Debug = yes,
         trace [io(!IO)] (
+            io.output_stream(Stream, !IO),
             module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
             PredName = pred_info_name(PredInfo),
             globals.lookup_accumulating_option(Globals,
                 unneeded_code_debug_pred_name, DebugPredNames),
             (
                 DebugPredNames = [],
-                io.format("%% Starting unneededed code pass %d\n",
+                io.format(Stream, "%% Starting unneededed code pass %d\n",
                     [i(Pass)], !IO)
             ;
                 DebugPredNames = [_ | _],
-                ( list.member(PredName, DebugPredNames) ->
-                    io.format("%% Starting unneededed code pass %d\n",
+                ( if list.member(PredName, DebugPredNames) then
+                    io.format(Stream, "%% Starting unneededed code pass %d\n",
                         [i(Pass)], !IO),
-                    AppendVarNums = yes,
-                    OutInfo = init_hlds_out_info(Globals),
-                    write_goal(OutInfo, Goal0, !.ModuleInfo, VarSet0,
-                        AppendVarNums, 0, ".\n", !IO)
-                ;
+                    OutInfo = init_hlds_out_info(Globals, output_debug),
+                    write_goal(OutInfo, Stream, !.ModuleInfo, VarSet0,
+                        print_name_and_num, 0, ".\n", Goal0, !IO)
+                else
                     true
                 )
             )
@@ -360,8 +359,8 @@ unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, Pass, Successful) :-
         InitInstMap, FinalInstMap, WhereNeededMap1, _,
         map.init, RefinedGoals1, no, Changed),
     unneeded_refine_goal(Goal1, Goal2, RefinedGoals1, RefinedGoals),
-    expect(map.is_empty(RefinedGoals),
-        $module, $pred, "goal reattachment unsuccessful"),
+    expect(map.is_empty(RefinedGoals), $pred,
+        "goal reattachment unsuccessful"),
     (
         Changed = yes,
         % We need to fix up the goal_info by recalculating the nonlocal vars
@@ -379,9 +378,9 @@ unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, Pass, Successful) :-
         proc_info_set_varset(VarSet, !ProcInfo),
         proc_info_set_vartypes(VarTypes, !ProcInfo),
         proc_info_set_rtti_varmaps(RttiVarMaps, !ProcInfo),
-        ( Pass > 3 ->
+        ( if Pass > 3 then
             true
-        ;
+        else
             unneeded_process_proc(!ProcInfo, !ModuleInfo, PredId, Pass + 1, _)
         ),
         Successful = yes
@@ -423,14 +422,15 @@ unneeded_process_goal(UnneededInfo, Goal0, Goal, InitInstMap, FinalInstMap,
             Goal0 = hlds_goal(_GoalExpr0, GoalInfo0),
             goal_info_get_goal_id(GoalInfo0) = goal_id(GoalIdNum0),
             trace [io(!IO)] (
-                io.format("unneeded code at goal id %d\n", [i(GoalIdNum0)],
-                    !IO)
+                io.output_stream(Stream, !IO),
+                io.format(Stream, "unneeded code at goal id %d\n",
+                    [i(GoalIdNum0)], !IO)
             )
         )
     ),
     ModuleInfo = UnneededInfo ^ uci_module_info,
     undemand_virgin_outputs(Goal0, ModuleInfo, InitInstMap, !WhereNeededMap),
-    ( goal_get_purity(Goal) = purity_impure ->
+    ( if goal_get_purity(Goal) = purity_impure then
         % By saying that all vars that are live before the impure goal are
         % needed everywhere, we prevent the movement of the goals producing
         % those vars across the impure goal.
@@ -438,7 +438,7 @@ unneeded_process_goal(UnneededInfo, Goal0, Goal, InitInstMap, FinalInstMap,
         % This code requires compound goals containing impure code
         % to also be marked impure.
         map.map_values_only(demand_var_everywhere, !WhereNeededMap)
-    ;
+    else
         true
     ).
 
@@ -459,10 +459,10 @@ insert_branch_into_refined_goals(Goal, BranchPoint - BranchNumSet,
 insert_branch_arm_into_refined_goals(Goal, GoalPath, BranchNum,
         !RefinedGoals) :-
     Key = GoalPath - BranchNum,
-    ( map.search(!.RefinedGoals, Key, Goals0) ->
+    ( if map.search(!.RefinedGoals, Key, Goals0) then
         Goals = [Goal | Goals0],
         map.det_update(Key, Goals, !RefinedGoals)
-    ;
+    else
         map.det_insert(Key, [Goal], !RefinedGoals)
     ).
 
@@ -476,7 +476,7 @@ can_eliminate_or_move(UnneededInfo, Goal, InitInstMap, FinalInstMap,
         WhereNeededMap, !:WhereInfo) :-
     ModuleInfo = UnneededInfo ^ uci_module_info,
     VarTypes = UnneededInfo ^ uci_vartypes,
-    instmap_changed_vars(InitInstMap, FinalInstMap, VarTypes, ModuleInfo,
+    instmap_changed_vars(ModuleInfo, VarTypes, InitInstMap, FinalInstMap,
         ChangedVarSet),
     set_of_var.to_sorted_list(ChangedVarSet, ChangedVars),
     map.init(Empty),
@@ -496,10 +496,10 @@ can_eliminate_or_move(UnneededInfo, Goal, InitInstMap, FinalInstMap,
 
 collect_where_needed(ContainingGoalMap, CurrentId, WhereNeededMap, ChangedVar,
         !WhereInfo) :-
-    ( map.search(WhereNeededMap, ChangedVar, Where) ->
+    ( if map.search(WhereNeededMap, ChangedVar, Where) then
         where_needed_upper_bound(ContainingGoalMap, CurrentId, Where,
             !WhereInfo)
-    ;
+    else
         true
     ).
 
@@ -512,7 +512,7 @@ collect_where_needed(ContainingGoalMap, CurrentId, WhereNeededMap, ChangedVar,
     where_needed::in, where_needed::out) is det.
 
 adjust_where_needed(Goal, Options, !WhereInfo) :-
-    (
+    ( if
         Goal = hlds_goal(GoalExpr, GoalInfo),
         (
             % Do not move goals that can fail, since doing so can cause
@@ -535,7 +535,7 @@ adjust_where_needed(Goal, Options, !WhereInfo) :-
             Options ^ uc_reorder_conj = no,
             goal_can_loop_or_throw(Goal),
             !.WhereInfo = branches(BranchMap),
-            \+ map.is_empty(BranchMap)
+            not map.is_empty(BranchMap)
         ;
             % Do not delete the `true' goal, since deleting it is a no-op,
             % and thus does *not* strictly reduce the number of computation
@@ -557,9 +557,9 @@ adjust_where_needed(Goal, Options, !WhereInfo) :-
             % Due to better instruction cache behavior, not moving Goal
             % may in fact yield faster code after all.
         )
-    ->
+    then
         !:WhereInfo = everywhere
-    ;
+    else
         true
     ).
 
@@ -616,7 +616,7 @@ undemand_virgin_outputs(Goal, ModuleInfo, InstMap, !WhereNeededMap) :-
 
 nonlocal_is_virgin_output(ModuleInfo, InstMap, Var) :-
     instmap_lookup_var(InstMap, Var, Inst),
-    \+ inst_is_bound(ModuleInfo, Inst).
+    not inst_is_bound(ModuleInfo, Inst).
 
 %---------------------------------------------------------------------------%
 
@@ -625,11 +625,11 @@ nonlocal_is_virgin_output(ModuleInfo, InstMap, Var) :-
     where_needed_map::in, where_needed_map::out) is det.
 
 demand_var(ContainingGoalMap, CurrentId, WhereNeeded, Var, !WhereNeededMap) :-
-    ( map.search(!.WhereNeededMap, Var, Where0) ->
+    ( if map.search(!.WhereNeededMap, Var, Where0) then
         where_needed_upper_bound(ContainingGoalMap, CurrentId,
             WhereNeeded, Where0, Where),
         map.det_update(Var, Where, !WhereNeededMap)
-    ;
+    else
         map.det_insert(Var, WhereNeeded, !WhereNeededMap)
     ).
 
@@ -682,7 +682,7 @@ unneeded_process_goal_internal(UnneededInfo, Goal0, Goal,
     ;
         GoalExpr0 = switch(SwitchVar, CanFail, Cases0),
         ContainingGoalMap = UnneededInfo ^ uci_containing_goal_map,
-        (
+        ( if
             Cases0 = [FirstCase0 | _],
             FirstCase0 = case(_, _, FirstCaseGoal0),
             FirstCaseGoal0 = hlds_goal(_, FirstCaseGoalInfo0),
@@ -691,10 +691,10 @@ unneeded_process_goal_internal(UnneededInfo, Goal0, Goal,
             GoalContaining0 = containing_goal(_ContainingGoalId,
                 FirstCaseLastStep),
             FirstCaseLastStep = step_switch(_, MaybeNumAltPrime)
-        ->
+        then
             MaybeNumAlt = MaybeNumAltPrime
-        ;
-            unexpected($module, $pred, "switch count")
+        else
+            unexpected($pred, "switch count")
         ),
         GoalId = goal_info_get_goal_id(GoalInfo0),
         BranchPoint = branch_point(GoalId, alt_switch(MaybeNumAlt)),
@@ -739,9 +739,9 @@ unneeded_process_goal_internal(UnneededInfo, Goal0, Goal,
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = scope(Reason, SomeGoal0),
-        ( Reason = from_ground_term(_, from_ground_term_construct) ->
+        ( if Reason = from_ground_term(_, from_ground_term_construct) then
             Goal = Goal0
-        ;
+        else
             unneeded_process_goal(UnneededInfo, SomeGoal0, SomeGoal,
                 InitInstMap, FinalInstMap,
                 !WhereNeededMap, !RefinedGoals, !Changed),
@@ -751,7 +751,7 @@ unneeded_process_goal_internal(UnneededInfo, Goal0, Goal,
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %---------------------------------------------------------------------------%
@@ -777,12 +777,12 @@ unneeded_process_conj(UnneededInfo, Goals0, Goals, InitInstMap, _FinalInstMap,
 
 build_bracketed_conj([], _, []).
 build_bracketed_conj([Goal | Goals], InitInstMap, BracketedGoals) :-
-    ( instmap_is_unreachable(InitInstMap) ->
+    ( if instmap_is_unreachable(InitInstMap) then
         BracketedGoals = []
-    ;
+    else
         Goal = hlds_goal(_, GoalInfo),
         InstMapDelta = goal_info_get_instmap_delta(GoalInfo),
-        instmap.apply_instmap_delta(InitInstMap, InstMapDelta, FinalInstMap),
+        apply_instmap_delta(InstMapDelta, InitInstMap, FinalInstMap),
         build_bracketed_conj(Goals, FinalInstMap, BracketedTail),
         BracketedGoal = bracketed_goal(Goal, InitInstMap, FinalInstMap),
         BracketedGoals = [BracketedGoal | BracketedTail]
@@ -803,9 +803,9 @@ unneeded_process_rev_bracketed_conj(UnneededInfo,
         InitInstMap, FinalInstMap, !WhereNeededMap, !RefinedGoals, !Changed),
     unneeded_process_rev_bracketed_conj(UnneededInfo, BracketedGoals, Goals1,
         !WhereNeededMap, !RefinedGoals, !Changed),
-    ( Goal1 = hlds_goal(true_goal_expr, _) ->
+    ( if Goal1 = hlds_goal(true_goal_expr, _) then
         Goals = Goals1
-    ;
+    else
         Goals = [Goal1 | Goals1]
     ).
 
@@ -872,7 +872,7 @@ unneeded_process_ite(UnneededInfo, Cond0, Cond, Then0, Then, Else0, Else,
         !WhereNeededMap, !RefinedGoals, !Changed) :-
     Cond0 = hlds_goal(_, CondInfo0),
     InstMapDelta = goal_info_get_instmap_delta(CondInfo0),
-    instmap.apply_instmap_delta(InitInstMap, InstMapDelta, InstMapCond),
+    apply_instmap_delta(InstMapDelta, InitInstMap, InstMapCond),
 
     unneeded_process_goal(UnneededInfo, Else0, Else, InitInstMap, FinalInstMap,
         !.WhereNeededMap, WhereNeededMapElse, !RefinedGoals, !Changed),
@@ -915,11 +915,11 @@ merge_where_needed_maps(ContainingGoalMap, CurrentId,
 add_where_needed_list(_, [], _, !WhereNeededMap).
 add_where_needed_list(ContainingGoalMap, [Var - BranchWhere | WhereNeededList],
         CurrentId, !WhereNeededMap) :-
-    ( map.search(!.WhereNeededMap, Var, OldWhere) ->
+    ( if map.search(!.WhereNeededMap, Var, OldWhere) then
         where_needed_upper_bound(ContainingGoalMap, CurrentId,
             BranchWhere, OldWhere, CombinedWhere),
         map.det_update(Var, CombinedWhere, !WhereNeededMap)
-    ;
+    else
         map.det_insert(Var, BranchWhere, !WhereNeededMap)
     ),
     add_where_needed_list(ContainingGoalMap, WhereNeededList, CurrentId,
@@ -947,11 +947,11 @@ add_alt_start(ContainingGoalMap, [Var - BranchWhere0 | WhereNeededList],
         BranchWhere0 = branches(_),
         BranchWhere = BranchWhere0
     ),
-    ( map.search(!.WhereNeededMap, Var, OldWhere) ->
+    ( if map.search(!.WhereNeededMap, Var, OldWhere) then
         where_needed_upper_bound(ContainingGoalMap, CurrentId,
             BranchWhere, OldWhere, CombinedWhere),
         map.det_update(Var, CombinedWhere, !WhereNeededMap)
-    ;
+    else
         map.det_insert(Var, BranchWhere, !WhereNeededMap)
     ),
     add_alt_start(ContainingGoalMap, WhereNeededList, BranchPoint, BranchNum,
@@ -1009,16 +1009,16 @@ unneeded_refine_goal(Goal0, Goal, !RefinedGoals) :-
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = scope(Reason, SomeGoal0),
-        ( Reason = from_ground_term(_, from_ground_term_construct) ->
+        ( if Reason = from_ground_term(_, from_ground_term_construct) then
             Goal = Goal0
-        ;
+        else
             unneeded_refine_goal(SomeGoal0, SomeGoal, !RefinedGoals),
             GoalExpr = scope(Reason, SomeGoal),
             Goal = hlds_goal(GoalExpr, GoalInfo0)
         )
     ;
         GoalExpr0 = shorthand(_),
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 :- pred unneeded_refine_conj(list(hlds_goal)::in, list(hlds_goal)::out,
@@ -1028,9 +1028,9 @@ unneeded_refine_conj([], [], !RefinedGoals).
 unneeded_refine_conj([Goal0 | Goals0], Goals, !RefinedGoals) :-
     unneeded_refine_goal(Goal0, HeadGoal, !RefinedGoals),
     unneeded_refine_conj(Goals0, TailGoals, !RefinedGoals),
-    ( HeadGoal = hlds_goal(conj(plain_conj, HeadGoals), _) ->
+    ( if HeadGoal = hlds_goal(conj(plain_conj, HeadGoals), _) then
         Goals = HeadGoals ++ TailGoals
-    ;
+    else
         Goals = [HeadGoal | TailGoals]
     ).
 
@@ -1043,10 +1043,10 @@ unneeded_refine_cases([Case0 | Cases0], [Case | Cases], !RefinedGoals,
         GoalId, BranchNum) :-
     Case0 = case(MainConsId, OtherConsIds, Goal0),
     unneeded_refine_goal(Goal0, Goal1, !RefinedGoals),
-    ( map.search(!.RefinedGoals, GoalId - BranchNum, ToInsertGoals) ->
+    ( if map.search(!.RefinedGoals, GoalId - BranchNum, ToInsertGoals) then
         insert_refine_goals(ToInsertGoals, Goal1, Goal),
         map.delete(GoalId - BranchNum, !RefinedGoals)
-    ;
+    else
         Goal = Goal1
     ),
     Case = case(MainConsId, OtherConsIds, Goal),
@@ -1061,10 +1061,10 @@ unneeded_refine_disj([], [], !RefinedGoals, _, _).
 unneeded_refine_disj([Goal0 | Goals0], [Goal | Goals], !RefinedGoals,
         GoalId, BranchNum) :-
     unneeded_refine_goal(Goal0, Goal1, !RefinedGoals),
-    ( map.search(!.RefinedGoals, GoalId - BranchNum, ToInsertGoals) ->
+    ( if map.search(!.RefinedGoals, GoalId - BranchNum, ToInsertGoals) then
         insert_refine_goals(ToInsertGoals, Goal1, Goal),
         map.delete(GoalId - BranchNum, !RefinedGoals)
-    ;
+    else
         Goal = Goal1
     ),
     unneeded_refine_disj(Goals0, Goals, !RefinedGoals,
@@ -1080,16 +1080,16 @@ unneeded_refine_ite(Cond0, Cond, Then0, Then, Else0, Else,
     unneeded_refine_goal(Then0, Then1, !RefinedGoals),
     unneeded_refine_goal(Else0, Else1, !RefinedGoals),
 
-    ( map.search(!.RefinedGoals, GoalId - 1, ToInsertGoalsThen) ->
+    ( if map.search(!.RefinedGoals, GoalId - 1, ToInsertGoalsThen) then
         insert_refine_goals(ToInsertGoalsThen, Then1, Then),
         map.delete(GoalId - 1, !RefinedGoals)
-    ;
+    else
         Then = Then1
     ),
-    ( map.search(!.RefinedGoals, GoalId - 2, ToInsertGoalsElse) ->
+    ( if map.search(!.RefinedGoals, GoalId - 2, ToInsertGoalsElse) then
         insert_refine_goals(ToInsertGoalsElse, Else1, Else),
         map.delete(GoalId - 2, !RefinedGoals)
-    ;
+    else
         Else = Else1
     ).
 
@@ -1107,7 +1107,7 @@ insert_refine_goals(ToInsertGoals, Goal0, Goal) :-
     % Given two sets of requirements about where a goal is needed, return
     % a single requirement that contains all the demands. The main purpose
     % of this predicate is to discover when the union of two sets of
-    % requirements (e.g. branch sets {b1,b2} and {b3} covers all
+    % requirements (e.g. branch sets {b1,b2} and {b3}) covers all
     % computation paths.
     %
 :- pred where_needed_upper_bound(containing_goal_map::in, goal_id::in,
@@ -1150,17 +1150,17 @@ where_needed_branches_upper_bound_2(_, _, [],
 where_needed_branches_upper_bound_2(ContainingGoalMap, CurrentId,
         [First | Rest], Branches0, WhereNeeded) :-
     First = BranchPoint - NewAlts,
-    ( map.search(Branches0, BranchPoint, OldAlts) ->
+    ( if map.search(Branches0, BranchPoint, OldAlts) then
         set.union(OldAlts, NewAlts, Alts),
         BranchPoint = branch_point(BranchGoalId, BranchAlts),
-        ( branch_point_is_complete(BranchAlts, Alts) ->
-            (
+        ( if branch_point_is_complete(BranchAlts, Alts) then
+            ( if
                 get_parent_branch_point(ContainingGoalMap, BranchGoalId,
                     ParentBranchGoalId, ParentBranchArmGoalId,
                     ParentBranchAlt, ParentBranchNum),
-                \+ goal_id_inside(ContainingGoalMap, ParentBranchArmGoalId,
+                not goal_id_inside(ContainingGoalMap, ParentBranchArmGoalId,
                     CurrentId)
-            ->
+            then
                 map.delete(BranchPoint, Branches0, Branches1),
                 ParentBranchPoint = branch_point(ParentBranchGoalId,
                     ParentBranchAlt),
@@ -1168,15 +1168,15 @@ where_needed_branches_upper_bound_2(ContainingGoalMap, CurrentId,
                 where_needed_branches_upper_bound_2(ContainingGoalMap,
                     CurrentId, [ParentBranchPoint - ParentAlts | Rest],
                     Branches1, WhereNeeded)
-            ;
+            else
                 WhereNeeded = everywhere
             )
-        ;
+        else
             map.det_update(BranchPoint, Alts, Branches0, Branches1),
             where_needed_branches_upper_bound_2(ContainingGoalMap, CurrentId,
                 Rest, Branches1, WhereNeeded)
         )
-    ;
+    else
         map.det_insert(BranchPoint, NewAlts, Branches0, Branches1),
         where_needed_branches_upper_bound_2(ContainingGoalMap, CurrentId,
             Rest, Branches1, WhereNeeded)
@@ -1189,6 +1189,7 @@ get_parent_branch_point(ContainingGoalMap, GoalId, BranchGoalId,
         BranchArmGoalId, BranchAlt, BranchNum) :-
     map.lookup(ContainingGoalMap, GoalId, GoalContaining),
     GoalContaining = containing_goal(ContainingGoalId, LastStep),
+    require_complete_switch [LastStep]
     (
         LastStep = step_switch(Arm, MaybeNumAlts),
         BranchGoalId = ContainingGoalId,
@@ -1216,17 +1217,34 @@ get_parent_branch_point(ContainingGoalMap, GoalId, BranchGoalId,
         ),
         get_parent_branch_point(ContainingGoalMap, ContainingGoalId,
             BranchGoalId, BranchArmGoalId, BranchAlt, BranchNum)
+    ;
+        % The optimization has not yet been extended to handle atomic goals.
+        ( LastStep = step_atomic_main
+        ; LastStep = step_atomic_orelse(_)
+        ),
+        fail
+    ;
+        % Lambdas and try scopes should have been expanded out
+        % by the time this optimization is invoked.
+        ( LastStep = step_lambda
+        ; LastStep = step_try
+        ),
+        fail
     ).
 
 :- pred branch_point_is_complete(branch_alts::in, set(int)::in) is semidet.
 
-branch_point_is_complete(alt_ite, Alts) :-
-    set.count(Alts, NumAlts),
-    NumAlts = 2.
-branch_point_is_complete(alt_switch(known_num_functors_in_type(NumFunctors)),
-        Alts) :-
-    set.count(Alts, NumAlts),
-    NumAlts = NumFunctors.
+branch_point_is_complete(BranchAlts, Alts) :-
+    (
+        BranchAlts = alt_ite,
+        set.count(Alts, NumAlts),
+        NumAlts = 2
+    ;
+        BranchAlts = alt_switch(MaybeSwitchNumFunctors),
+        MaybeSwitchNumFunctors = known_num_functors_in_type(NumFunctors),
+        set.count(Alts, NumAlts),
+        NumAlts = NumFunctors
+    ).
 
 %---------------------------------------------------------------------------%
 :- end_module transform_hlds.unneeded_code.
