@@ -10,12 +10,17 @@
 % Author: wangp.
 %
 % This module takes the HLDS and transforms the locally-defined procedures as
-% follows: if the formal parameter of a procedure has a type consisting of a
-% single function symbol then that parameter is expanded into multiple
-% parameters (one for each field of the functor).  Tuple types are also
-% expanded.  The argument lists are expanded as deeply (flatly) as possible.
+% follows. If a formal parameter of a procedure has a type that has only
+% a single function symbol (i.e. it is a kind of tuple), then it replaces
+% the parameter holding the tuple with one parameter for each field of the
+% tuple. If some of those fields also have types that have only one function
+% symbol, it expand them as well. It recurses until it has expanded the
+% argument list as deeply as possible, yielding a parameter list that has
+% *no* arguments of such tuple-like types. We call such parameter lists "flat",
+% because they have no expandable structure (at least no structure that is
+% expandable by this module).
 %
-% e.g. for the following predicate and types,
+% For example, for the following predicate and types,
 %
 %   :- type t ---> t(u).
 %   :- type u ---> u(v, w).
@@ -25,18 +30,23 @@
 %   :- pred f(t::in) is det.
 %   f(T) :- blah.
 %
-% a transformed version of f/1 would be added:
+% we would generate this transformed version of f/1:
 %
 %   :- pred f_untupled(v::in, int::in, string::in) is det.
 %   f_untupled(V, W1, W2) :- blah.
 %
-% After all the procedures have been processed in that way, a second pass is
-% made to update all the calls in the module which refer to the old procedures
-% to call the transformed procedures.  This is done by adding deconstruction
-% and construction unifications as needed, which can later be simplified by a
-% simplification pass (not called from this module).
+% The first pass creates transformed versions for all the procedures
+% whose argument lists weren't already flat already.
+% The second pass then replaces all the calls in the module which refer
+% to the old procedures with calls to their transformed versions.
+% It does this by adding deconstruction and construction unifications
+% as needed, which can later be simplified by a simplification pass.
+% (This module does not itself invoke simplification, because we expect that
+% the HLDS it generates will be subject to further optimization passes;
+% simplification *will* be called by the target backend before it starts
+% code generation.)
 %
-% e.g. a call to the predicate above,
+% For example, we transform this code, which calls the predicate above,
 %
 %   :- pred g(T::in) is det.
 %   g(_) :-
@@ -48,7 +58,7 @@
 %       F = t(E),
 %       f(F).
 %
-% is changed to this:
+% to this:
 %
 %   g(_) :-
 %       A = 1,
@@ -62,7 +72,7 @@
 %       I = w(J, K),
 %       f_untupled(H, J, K).
 %
-% which, after simplication, should become:
+% which, after simplification, should become:
 %
 %   g(_) :-
 %       A = 1,
@@ -73,13 +83,13 @@
 % Limitations:
 %
 % - When a formal parameter is expanded, both the parameter's type and mode
-% have to be expanded.  Currently only arguments with in and out modes can
-% be expanded, as I don't know how to do it for the general case.
-% It should be enough for the majority of code.
+%   have to be expanded. Currently only arguments with in and out modes can
+%   be expanded, as I don't know how to do it for the general case.
+%   It should be enough for the majority of code.
 %
 % - Some predicates may or may not be expandable but won't be right now,
-% because I don't understand the features they use (see expand_args_in_pred
-% below).
+%   because I don't understand the features they use (see expand_args_in_pred
+%   below).
 %
 % Julien says: "it should be possible for this transformation to work across
 % module boundaries by exporting the goal templates [search for CallAux
@@ -90,6 +100,7 @@
 :- module transform_hlds.untupling.
 :- interface.
 
+:- import_module hlds.
 :- import_module hlds.hlds_module.
 
 :- import_module io.
@@ -102,12 +113,18 @@
 
 :- implementation.
 
-:- import_module check_hlds.mode_util.
+:- import_module check_hlds.
+:- import_module check_hlds.recompute_instmap_deltas.
 :- import_module hlds.hlds_data.
 :- import_module hlds.hlds_goal.
 :- import_module hlds.hlds_pred.
+:- import_module hlds.make_goal.
 :- import_module hlds.quantification.
-:- import_module mdbcomp.prim_data.
+:- import_module hlds.status.
+:- import_module hlds.vartypes.
+:- import_module mdbcomp.
+:- import_module mdbcomp.sym_name.
+:- import_module parse_tree.
 :- import_module parse_tree.prog_data.
 :- import_module parse_tree.prog_mode.
 :- import_module parse_tree.prog_type.
@@ -119,6 +136,7 @@
 :- import_module list.
 :- import_module map.
 :- import_module maybe.
+:- import_module one_or_more.
 :- import_module pair.
 :- import_module require.
 :- import_module string.
@@ -134,13 +152,13 @@
 
 :- type transformed_proc
     --->    transformed_proc(
+                % A procedure that was generated by the untupling
+                % transformation.
                 pred_proc_id,
-                    % A procedure that was generated by the
-                    % untupling transformation.
+
+                % A call goal template that is used to update calls
+                % referring to the old procedure to the new procedure.
                 hlds_goal
-                    % A call goal template that is used to update
-                    % calls referring to the old procedure to the
-                    % new procedure.
             ).
 
 untuple_arguments(!ModuleInfo, !IO) :-
@@ -149,19 +167,19 @@ untuple_arguments(!ModuleInfo, !IO) :-
 
 %-----------------------------------------------------------------------------%
 %
-% Pass 1
+% Pass 1.
 %
 
-    % This is the top level of the first pass.  It expands procedure
-    % arguments where possible, adding new versions of the transformed
-    % procedures into the module and recording the mapping between the old
-    % and new procedures in the transform map.
+    % This is the top level of the first pass. It expands procedure arguments
+    % where possible, adding new versions of the transformed procedures
+    % into the module and recording the mapping between the old and new
+    % procedures in the transform map.
     %
 :- pred expand_args_in_module(module_info::in, module_info::out,
     transform_map::out) is det.
 
 expand_args_in_module(!ModuleInfo, TransformMap) :-
-    module_info_get_valid_predids(PredIds, !ModuleInfo),
+    module_info_get_valid_pred_ids(!.ModuleInfo, PredIds),
     list.foldl3(expand_args_in_pred, PredIds,
         !ModuleInfo, map.init, TransformMap, counter.init(0), _).
 
@@ -171,27 +189,28 @@ expand_args_in_module(!ModuleInfo, TransformMap) :-
 expand_args_in_pred(PredId, !ModuleInfo, !TransformMap, !Counter) :-
     module_info_get_type_table(!.ModuleInfo, TypeTable),
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    (
+    ( if
         % Only perform the transformation on predicates which
         % satisfy the following criteria.
-        pred_info_get_import_status(PredInfo, ImportStatus),
-        status_defined_in_this_module(ImportStatus) = yes,
-        pred_info_get_goal_type(PredInfo, goal_type_clause),
+        pred_info_get_status(PredInfo, PredStatus),
+        pred_status_defined_in_this_module(PredStatus) = yes,
+        pred_info_get_goal_type(PredInfo,
+            goal_not_for_promise(np_goal_type_clause)),
         % Some of these limitations may be able to be lifted later.
         % For now, take the safe option and don't touch them.
         pred_info_get_exist_quant_tvars(PredInfo, []),
-        pred_info_get_head_type_params(PredInfo, []),
+        pred_info_get_external_type_params(PredInfo, []),
         pred_info_get_class_context(PredInfo, constraints([], [])),
         pred_info_get_origin(PredInfo, origin_user(_)),
         pred_info_get_arg_types(PredInfo, TypeVarSet, ExistQVars, ArgTypes),
         varset.is_empty(TypeVarSet),
         ExistQVars = [],
         at_least_one_expandable_type(ArgTypes, TypeTable)
-    ->
-        ProcIds = pred_info_non_imported_procids(PredInfo),
+    then
+        ProcIds = pred_info_valid_non_imported_procids(PredInfo),
         list.foldl3(expand_args_in_proc(PredId), ProcIds,
             !ModuleInfo, !TransformMap, !Counter)
-    ;
+    else
         true
     ).
 
@@ -207,7 +226,7 @@ at_least_one_expandable_type([Type | Types], TypeTable) :-
 
     % This structure records the mapping between a head variable of the
     % original procedure, and the list of variables that it was finally
-    % expanded into.  If the head variable expands into some intermediate
+    % expanded into. If the head variable expands into some intermediate
     % variables which are then expanded further, the intermediate
     % variables are not listed in the mapping.
     %
@@ -267,7 +286,7 @@ expand_args_in_proc_2(HeadVars0, ArgModes0, HeadVars, ArgModes,
     expand_args_in_proc_3(HeadVars0, ArgModes0, ListOfHeadVars,
         ListOfArgModes, Goal0, hlds_goal(GoalExpr, GoalInfo1), !VarSet,
         !VarTypes, [], TypeTable),
-    Context = goal_info_get_context(Goal0 ^ hlds_goal_info),
+    Context = goal_info_get_context(Goal0 ^ hg_info),
     goal_info_set_context(Context, GoalInfo1, GoalInfo),
     list.condense(ListOfHeadVars, HeadVars),
     list.condense(ListOfArgModes, ArgModes),
@@ -279,6 +298,10 @@ expand_args_in_proc_2(HeadVars0, ArgModes0, HeadVars, ArgModes,
     vartypes::in, vartypes::out, list(mer_type)::in, type_table::in) is det.
 
 expand_args_in_proc_3([], [], [], [], !_, !_, !_, _, _).
+expand_args_in_proc_3([], [_|_], _, _, !_, !_, !_, _, _) :-
+    unexpected($pred, "length mismatch").
+expand_args_in_proc_3([_|_], [], _, _, !_, !_, !_, _, _) :-
+    unexpected($pred, "length mismatch").
 expand_args_in_proc_3([HeadVar0 | HeadVars0], [ArgMode0 | ArgModes0],
         [HeadVar | HeadVars], [ArgMode | ArgModes],
         !Goal, !VarSet, !VarTypes, ContainerTypes, TypeTable) :-
@@ -286,10 +309,6 @@ expand_args_in_proc_3([HeadVar0 | HeadVars0], [ArgMode0 | ArgModes0],
         !Goal, !VarSet, !VarTypes, ContainerTypes, TypeTable),
     expand_args_in_proc_3(HeadVars0, ArgModes0, HeadVars, ArgModes,
         !Goal, !VarSet, !VarTypes, ContainerTypes, TypeTable).
-expand_args_in_proc_3([], [_|_], _, _, !_, !_, !_, _, _) :-
-    unexpected($module, $pred, "length mismatch").
-expand_args_in_proc_3([_|_], [], _, _, !_, !_, !_, _, _) :-
-    unexpected($module, $pred, "length mismatch").
 
 :- pred expand_one_arg_in_proc(prog_var::in, mer_mode::in, prog_vars::out,
     list(mer_mode)::out, hlds_goal::in, hlds_goal::out, prog_varset::in,
@@ -331,14 +350,14 @@ expand_one_arg_in_proc_2(HeadVar0, ArgMode0, MaybeHeadVarsAndArgModes,
             !VarSet, !VarTypes),
         list.duplicate(list.length(NewHeadVars), ArgMode0, NewArgModes),
         MaybeHeadVarsAndArgModes = yes(NewHeadVars - NewArgModes),
-        ( ArgMode0 = in_mode ->
+        ( if ArgMode0 = in_mode then
             construct_functor(HeadVar0, ConsId, NewHeadVars, UnifGoal),
             conjoin_goals_keep_detism(UnifGoal, !Goal)
-        ; ArgMode0 = out_mode ->
+        else if ArgMode0 = out_mode then
             deconstruct_functor(HeadVar0, ConsId, NewHeadVars, UnifGoal),
             conjoin_goals_keep_detism(!.Goal, UnifGoal, !:Goal)
-        ;
-            unexpected($module, $pred, "unsupported mode")
+        else
+            unexpected($pred, "unsupported mode")
         ),
         ContainerTypes = [Type | ContainerTypes0]
     ;
@@ -375,23 +394,23 @@ conjoin_goals_keep_detism(GoalA, GoalB, Goal) :-
     untuple_map::in, untuple_map::out) is det.
 
 build_untuple_map([], [], !UntupleMap).
+build_untuple_map([], [_| _], !_) :-
+    unexpected($pred, "length mismatch").
+build_untuple_map([_| _], [], !_) :-
+    unexpected($pred, "length mismatch").
 build_untuple_map([OldVar | OldVars], [NewVars | NewVarss], !UntupleMap) :-
-    ( NewVars = [OldVar] ->
+    ( if NewVars = [OldVar] then
         build_untuple_map(OldVars, NewVarss, !UntupleMap)
-    ;
+    else
         map.det_insert(OldVar, NewVars, !UntupleMap),
         build_untuple_map(OldVars, NewVarss, !UntupleMap)
     ).
-build_untuple_map([], [_| _], !_) :-
-    unexpected($module, $pred, "length mismatch").
-build_untuple_map([_| _], [], !_) :-
-    unexpected($module, $pred, "length mismatch").
 
 %-----------------------------------------------------------------------------%
 
-    % This predicate makes a new version of the given procedure in a
-    % module.  Amongst other things the new procedure is given a new
-    % pred_id and proc_id, a new name and a new goal.
+    % This predicate makes a new version of the given procedure in a module.
+    % Amongst other things the new procedure is given a new pred_id and
+    % proc_id, a new name and a new goal.
     %
     % CallAux is an output variable, which is unified with a goal that
     % can be used as a template for constructing calls to the newly
@@ -401,15 +420,14 @@ build_untuple_map([_| _], [], !_) :-
     %
 :- pred create_aux_pred(pred_id::in, proc_id::in, pred_info::in,
     proc_info::in, int::in, pred_id::out, proc_id::out, hlds_goal::out,
-    pred_info::out, proc_info::out, module_info::in, module_info::out)
-    is det.
+    pred_info::out, proc_info::out, module_info::in, module_info::out) is det.
 
 create_aux_pred(PredId, ProcId, PredInfo, ProcInfo, Counter,
         AuxPredId, AuxProcId, CallAux, AuxPredInfo, AuxProcInfo,
         !ModuleInfo) :-
     proc_info_get_headvars(ProcInfo, AuxHeadVars),
     proc_info_get_goal(ProcInfo, Goal @ hlds_goal(_GoalExpr, GoalInfo)),
-    proc_info_get_initial_instmap(ProcInfo, !.ModuleInfo, InitialAuxInstMap),
+    proc_info_get_initial_instmap(!.ModuleInfo, ProcInfo, InitialAuxInstMap),
     pred_info_get_typevarset(PredInfo, TVarSet),
     proc_info_get_vartypes(ProcInfo, VarTypes),
     pred_info_get_class_context(PredInfo, ClassContext),
@@ -418,6 +436,7 @@ create_aux_pred(PredId, ProcId, PredInfo, ProcInfo, Counter,
     proc_info_get_inst_varset(ProcInfo, InstVarSet),
     pred_info_get_markers(PredInfo, Markers),
     pred_info_get_origin(PredInfo, OrigOrigin),
+    proc_info_get_has_parallel_conj(ProcInfo, HasParallelConj),
     pred_info_get_var_name_remap(PredInfo, VarNameRemap),
 
     PredModule = pred_info_module(PredInfo),
@@ -435,18 +454,19 @@ create_aux_pred(PredId, ProcId, PredInfo, ProcInfo, Counter,
     hlds_pred.define_new_pred(Origin, Goal, CallAux, AuxHeadVars, _ExtraArgs,
         InitialAuxInstMap, AuxPredSymName, TVarSet, VarTypes, ClassContext,
         RttiVarMaps, VarSet, InstVarSet, Markers, address_is_not_taken,
-        VarNameRemap, !ModuleInfo, proc(AuxPredId, AuxProcId)),
+        HasParallelConj, VarNameRemap, !ModuleInfo, AuxPredProcId),
+    AuxPredProcId = proc(AuxPredId, AuxProcId),
 
     module_info_pred_proc_info(!.ModuleInfo, AuxPredId, AuxProcId,
         AuxPredInfo, AuxProcInfo).
 
 %-----------------------------------------------------------------------------%
 %
-% Pass 2
+% Pass 2.
 %
 
     % This is the top level of the second pass. It takes the transform map
-    % built during the first pass as input.  For every call to a procedure
+    % built during the first pass as input. For every call to a procedure
     % in the transform map, it rewrites the call to use the new procedure
     % instead, inserting unifications before and after the call as necessary.
     %
@@ -454,7 +474,7 @@ create_aux_pred(PredId, ProcId, PredInfo, ProcInfo, Counter,
     module_info::out) is det.
 
 fix_calls_to_expanded_procs(TransformMap, !ModuleInfo) :-
-    module_info_get_valid_predids(PredIds, !ModuleInfo),
+    module_info_get_valid_pred_ids(!.ModuleInfo, PredIds),
     list.foldl(fix_calls_in_pred(TransformMap), PredIds, !ModuleInfo).
 
 :- pred fix_calls_in_pred(transform_map::in, pred_id::in, module_info::in,
@@ -462,7 +482,7 @@ fix_calls_to_expanded_procs(TransformMap, !ModuleInfo) :-
 
 fix_calls_in_pred(TransformMap, PredId, !ModuleInfo) :-
     module_info_pred_info(!.ModuleInfo, PredId, PredInfo),
-    ProcIds = pred_info_non_imported_procids(PredInfo),
+    ProcIds = pred_info_valid_non_imported_procids(PredInfo),
     list.foldl(fix_calls_in_proc(TransformMap, PredId), ProcIds, !ModuleInfo).
 
 :- pred fix_calls_in_proc(transform_map::in, pred_id::in, proc_id::in,
@@ -477,7 +497,9 @@ fix_calls_in_proc(TransformMap, PredId, ProcId, !ModuleInfo) :-
         proc_info_get_varset(!.ProcInfo, VarSet0),
         fix_calls_in_goal(Goal0, Goal, VarSet0, VarSet,
             VarTypes0, VarTypes, TransformMap, !.ModuleInfo),
-        ( Goal0 \= Goal ->
+        ( if Goal0 = Goal then
+            true
+        else
             proc_info_set_goal(Goal, !ProcInfo),
             proc_info_set_varset(VarSet, !ProcInfo),
             proc_info_set_vartypes(VarTypes, !ProcInfo),
@@ -486,8 +508,6 @@ fix_calls_in_proc(TransformMap, PredId, ProcId, !ModuleInfo) :-
                 !ProcInfo, !ModuleInfo),
             module_info_set_pred_proc_info(PredId, ProcId,
                 PredInfo, !.ProcInfo, !ModuleInfo)
-        ;
-            true
         )
     ).
 
@@ -507,24 +527,24 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
         Goal = Goal0
     ;
         GoalExpr0 = plain_call(CalleePredId, CalleeProcId, OrigArgs, _, _, _),
-        (
+        ( if
             map.search(TransformMap, proc(CalleePredId, CalleeProcId),
                 transformed_proc(_, hlds_goal(CallAux0, CallAuxInfo)))
-        ->
+        then
             module_info_get_type_table(ModuleInfo, TypeTable),
             module_info_pred_proc_info(ModuleInfo, CalleePredId,
                 CalleeProcId, _CalleePredInfo, CalleeProcInfo),
             proc_info_get_argmodes(CalleeProcInfo, OrigArgModes),
             expand_call_args(OrigArgs, OrigArgModes, Args,
                 EnterUnifs, ExitUnifs, !VarSet, !VarTypes, TypeTable),
-            ( CallAux = CallAux0 ^ call_args := Args ->
+            ( if CallAux = CallAux0 ^ call_args := Args then
                 Call = hlds_goal(CallAux, CallAuxInfo),
                 ConjList = EnterUnifs ++ [Call] ++ ExitUnifs,
                 conj_list_to_goal(ConjList, GoalInfo0, Goal)
-            ;
-                unexpected($module, $pred, "not a call template")
+            else
+                unexpected($pred, "not a call template")
             )
-        ;
+        else
             Goal = hlds_goal(GoalExpr0, GoalInfo0)
         )
     ;
@@ -535,15 +555,15 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
         Goal = hlds_goal(GoalExpr, GoalInfo0)
     ;
         GoalExpr0 = scope(Reason, SubGoal0),
-        (
+        ( if
             Reason = from_ground_term(_, FGT),
             ( FGT = from_ground_term_construct
             ; FGT = from_ground_term_deconstruct
             )
-        ->
+        then
             % There are no calls in these scopes.
             Goal = Goal0
-        ;
+        else
             fix_calls_in_goal(SubGoal0, SubGoal, !VarSet, !VarTypes,
                 TransformMap, ModuleInfo),
             GoalExpr = scope(Reason, SubGoal),
@@ -558,7 +578,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
         ;
             ConjType = parallel_conj,
             % I am not sure whether parallel conjunctions should be treated
-            % with fix_calls_in_goal or fix_calls_in_goal_list.  At any rate,
+            % with fix_calls_in_goal or fix_calls_in_goal_list. At any rate,
             % this is untested.
             fix_calls_in_goal_list(Goals0, Goals, !VarSet, !VarTypes,
                 TransformMap, ModuleInfo)
@@ -590,7 +610,7 @@ fix_calls_in_goal(Goal0, Goal, !VarSet, !VarTypes, TransformMap, ModuleInfo) :-
     ;
         GoalExpr0 = shorthand(_),
         % These should have been expanded out by now.
-        unexpected($module, $pred, "shorthand")
+        unexpected($pred, "shorthand")
     ).
 
 %-----------------------------------------------------------------------------%
@@ -606,9 +626,9 @@ fix_calls_in_conj([Goal0 | Goals0], Goals, !VarSet, !VarTypes, TransformMap,
         ModuleInfo),
     fix_calls_in_conj(Goals0, Goals1, !VarSet, !VarTypes, TransformMap,
         ModuleInfo),
-    ( Goal1 = hlds_goal(conj(plain_conj, ConjGoals), _) ->
+    ( if Goal1 = hlds_goal(conj(plain_conj, ConjGoals), _) then
         Goals = ConjGoals ++ Goals1
-    ;
+    else
         Goals = [Goal1 | Goals1]
     ).
 
@@ -654,6 +674,10 @@ expand_call_args(Args0, ArgModes0, Args, EnterUnifs, ExitUnifs,
     vartypes::in, vartypes::out, list(mer_type)::in, type_table::in) is det.
 
 expand_call_args_2([], [], [], [], [], !VarSet, !VarTypes, _, _).
+expand_call_args_2([], [_|_], _, _, _, !_, !_, _, _) :-
+    unexpected($pred, "length mismatch").
+expand_call_args_2([_|_], [], _, _, _, !_, !_, _, _) :-
+    unexpected($pred, "length mismatch").
 expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
         EnterUnifs, ExitUnifs, !VarSet, !VarTypes,
         ContainerTypes0, TypeTable) :-
@@ -666,22 +690,22 @@ expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
         vartypes_add_corresponding_lists(ReplacementArgs, Types, !VarTypes),
         list.duplicate(NumVars, ArgMode, ReplacementModes),
         ContainerTypes = [Arg0Type | ContainerTypes0],
-        ( ArgMode = in_mode ->
+        ( if ArgMode = in_mode then
             deconstruct_functor(Arg0, ConsId, ReplacementArgs, Unif),
             EnterUnifs = [Unif | EnterUnifs1],
             expand_call_args_2(ReplacementArgs ++ Args0,
                 ReplacementModes ++ ArgModes,
                 Args, EnterUnifs1, ExitUnifs, !VarSet,
                 !VarTypes, ContainerTypes, TypeTable)
-        ; ArgMode = out_mode ->
+        else if ArgMode = out_mode then
             construct_functor(Arg0, ConsId, ReplacementArgs, Unif),
             ExitUnifs = ExitUnifs1 ++ [Unif],
             expand_call_args_2(ReplacementArgs ++ Args0,
                 ReplacementModes ++ ArgModes,
                 Args, EnterUnifs, ExitUnifs1, !VarSet,
                 !VarTypes, ContainerTypes, TypeTable)
-        ;
-            unexpected($module, $pred, "unsupported mode")
+        else
+            unexpected($pred, "unsupported mode")
         )
     ;
         Expansion = no_expansion,
@@ -690,36 +714,31 @@ expand_call_args_2([Arg0 | Args0], [ArgMode | ArgModes], Args,
             ExitUnifs, !VarSet, !VarTypes, TypeTable)
     ).
 
-expand_call_args_2([], [_|_], _, _, _, !_, !_, _, _) :-
-    unexpected($module, $pred, "length mismatch").
-expand_call_args_2([_|_], [], _, _, _, !_, !_, _, _) :-
-    unexpected($module, $pred, "length mismatch").
-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 :- type expansion_result
     --->    expansion(
+                % The cons_id of the expanded constructor.
                 cons_id,
-                    % the cons_id of the expanded constructor
+
+                % The types of the arguments for the expanded constructor.
                 list(mer_type)
-                    % the types of the arguments for the
-                    % expanded constructor
             )
     ;       no_expansion.
 
-    % This predicate tries to expand the argument of the given mode and
-    % type.  If this is possible then Expansion is unified with the
-    % `expansion' functor, giving the details of the expansion.
-    % Otherwise it is unified with `no_expansion'.
+    % This predicate tries to expand the argument of the given mode and type.
+    % If this is possible then Expansion is unified with the `expansion'
+    % functor, giving the details of the expansion. Otherwise, it is
+    % unified with `no_expansion'.
     %
 :- pred expand_argument(mer_mode::in, mer_type::in, list(mer_type)::in,
     type_table::in, expansion_result::out) is det.
 
 expand_argument(ArgMode, ArgType, ContainerTypes, TypeTable, Expansion) :-
-    ( expandable_arg_mode(ArgMode) ->
+    ( if expandable_arg_mode(ArgMode) then
         expand_type(ArgType, ContainerTypes, TypeTable, Expansion)
-    ;
+    else
         Expansion = no_expansion
     ).
 
@@ -735,34 +754,35 @@ expandable_arg_mode(out_mode).
     expansion_result::out) is det.
 
 expand_type(Type, ContainerTypes, TypeTable, Expansion) :-
-    (
+    ( if
         % Always expand tuple types.
         type_to_ctor_and_args(Type, TypeCtor, TypeArgs),
         type_ctor_is_tuple(TypeCtor)
-    ->
+    then
         Arity = list.length(TypeArgs),
         ConsId = tuple_cons(Arity),
         Expansion = expansion(ConsId, TypeArgs)
-    ;
+    else if
         % Expand a discriminated union type if it has only a
         % single functor and the type has no parameters.
         type_to_ctor_and_args(Type, TypeCtor, []),
         search_type_ctor_defn(TypeTable, TypeCtor, TypeDefn),
         get_type_defn_tparams(TypeDefn, []),
         get_type_defn_body(TypeDefn, TypeBody),
-        TypeBody ^ du_type_ctors = [SingleCtor],
-        SingleCtor ^ cons_exist = [],
+        TypeBody = hlds_du_type(TypeBodyDu),
+        TypeBodyDu ^ du_type_ctors = one_or_more(SingleCtor, []),
+        SingleCtor ^ cons_maybe_exist = no_exist_constraints,
         SingleCtorName = SingleCtor ^ cons_name,
         SingleCtorArgs = SingleCtor ^ cons_args,
         SingleCtorArgs = [_ | _],
         % Prevent infinite loop with recursive types.
-        \+ list.member(Type, ContainerTypes)
-    ->
+        not list.member(Type, ContainerTypes)
+    then
         Arity = list.length(SingleCtorArgs),
         ConsId = cons(SingleCtorName, Arity, TypeCtor),
         ExpandedTypes = list.map(func(C) = C ^ arg_type, SingleCtorArgs),
         Expansion = expansion(ConsId, ExpandedTypes)
-    ;
+    else
         Expansion = no_expansion
     ).
 
