@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2011-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: autopar_calc_overlap.m
 % Author: pbone.
@@ -12,7 +12,7 @@
 % This module contains the code that calculates the likely overlap
 % between conjuncts in a parallelized conjunction.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module mdprof_fb.automatic_parallelism.autopar_calc_overlap.
 :- interface.
@@ -30,8 +30,8 @@
     incomplete_parallelisation::in, incomplete_parallelisation::out,
     parallelisation_cost_data::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -56,7 +56,7 @@
 :- import_module set.
 :- import_module string.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 calculate_parallel_cost(Info, !Parallelisation, CostData) :-
     ParConj = ip_get_par_conjs(!.Parallelisation),
@@ -65,11 +65,11 @@ calculate_parallel_cost(Info, !Parallelisation, CostData) :-
     maybe_calc_sequential_cost(
         (func(P) = P ^ ip_maybe_goals_before_cost),
         (func(P0, MaybeCost) = P0 ^ ip_maybe_goals_before_cost := MaybeCost),
-        ip_get_goals_before, CostBeforePercall, NumCalls, !Parallelisation),
+        ip_get_goals_before, NumCalls, CostBeforePercall, !Parallelisation),
     maybe_calc_sequential_cost(
         (func(P) = P ^ ip_maybe_goals_after_cost),
         (func(P0, MaybeCost) = P0 ^ ip_maybe_goals_after_cost := MaybeCost),
-        ip_get_goals_after, CostAfterPercall, NumCalls, !Parallelisation),
+        ip_get_goals_after, NumCalls, CostAfterPercall, !Parallelisation),
 
     Opts = Info ^ ipi_opts,
     SparkCost = Opts ^ cpcp_sparking_cost,
@@ -91,10 +91,10 @@ calculate_parallel_cost(Info, !Parallelisation, CostData) :-
 
 :- pred maybe_calc_sequential_cost((func(T) = maybe(goal_cost_csq))::in,
     (func(T, maybe(goal_cost_csq)) = T)::in,
-    (func(T) = list(pard_goal_detail))::in, float::out,
-    int::in, T::in, T::out) is det.
+    (func(T) = list(pard_goal_detail))::in,
+    int::in, float::out, T::in, T::out) is det.
 
-maybe_calc_sequential_cost(GetMaybe, SetMaybe, GetGoals, CostPercall, Calls,
+maybe_calc_sequential_cost(GetMaybe, SetMaybe, GetGoals, Calls, CostPercall,
         !Acc) :-
     MaybeCost = GetMaybe(!.Acc),
     (
@@ -121,9 +121,9 @@ calculate_parallel_cost_step(Info, NumMiddleGoals, Conjunct, !ConjNum,
     !.CostData = parallelisation_cost_data(SharedVars, Overlap0, Metrics0,
         PM0),
     !:NumGoals = !.NumGoals + length(Conjuncts),
-    ( !.NumGoals = NumMiddleGoals ->
+    ( if !.NumGoals = NumMiddleGoals then
         IsLastConjunct = is_last_par_conjunct
-    ;
+    else
         IsLastConjunct = not_last_par_conjunct
     ),
     Conjunct = seq_conj(Conjuncts),
@@ -149,8 +149,7 @@ calculate_parallel_cost_step(Info, AllSharedVars, IsLastConjunct, Conjunct,
         set.init, RightProducedVars0, set.init, RightConsumedVars0),
     RightProducedVars = set.intersect(RightProducedVars0, AllSharedVars),
     RightConsumedVars = set.intersect(RightConsumedVars0, AllSharedVars),
-    ProducedVars =
-        set.from_sorted_list(map.sorted_keys(!.ProductionsMap)),
+    ProducedVars = set.sorted_list_to_set(map.sorted_keys(!.ProductionsMap)),
     Vars = set.intersect(ProducedVars, RightConsumedVars),
 
     % This conjunct will actually start after it has been sparked by
@@ -189,7 +188,7 @@ calculate_parallel_cost_step(Info, AllSharedVars, IsLastConjunct, Conjunct,
             [], RevExecution0, map.init, ConsumptionsMap),
 
         % Calculate the point at which this conjunct finishes execution
-        % and complete the RevExecutions structure..
+        % and complete the RevExecutions structure.
         list.reverse(RevExecution, Execution),
         CostBParElapsed = LastParConsumeTime + (CostB - LastSeqConsumeTime),
         RevExecution = [ (LastResumeTime - CostBParElapsed) | RevExecution0 ],
@@ -202,7 +201,6 @@ calculate_parallel_cost_step(Info, AllSharedVars, IsLastConjunct, Conjunct,
     ;
         ( Algorithm = do_not_parallelise_dep_conjs
         ; Algorithm = parallelise_dep_conjs(estimate_speedup_naively)
-        ; Algorithm = parallelise_dep_conjs(estimate_speedup_by_num_vars)
         ),
 
         CostBPar = CostB + SparkCost,
@@ -213,10 +211,10 @@ calculate_parallel_cost_step(Info, AllSharedVars, IsLastConjunct, Conjunct,
         DeadTime = 0.0
     ),
 
-    % CostB    - the cost of B if it where to be executed in sequence.
-    % CostBPar - CostB plus the overheads of parallel exection (not including
+    % CostB    - the cost of B if it were to be executed in sequence.
+    % CostBPar - CostB plus the overheads of parallel execution (not including
     %            the dead time).
-    % DeadTime - The time that B spends blocked on other computations.
+    % DeadTime - The time that B spends blocked, waiting on other computations.
     % XXX: Need to account for SparkDelay here,
     !:Metrics = init_parallel_exec_metrics_incomplete(!.Metrics, CostSignals,
         CostWaits, CostB, CostBPar, DeadTime),
@@ -264,7 +262,7 @@ calculate_parallel_cost_step(Info, AllSharedVars, IsLastConjunct, Conjunct,
     %   is used by our caller to calculate the production times of this
     %   conjunct for later ones.
     %
-    % * !ConsumptionsMap: Accumuates a map of variable consumptions.
+    % * !ConsumptionsMap: Accumulates a map of variable consumptions.
     %
 :- pred calculate_dependent_parallel_cost_2(implicit_parallelism_info::in,
     map(var_rep, float)::in, pair(var_rep, production_or_consumption)::in,
@@ -315,19 +313,19 @@ calculate_dependent_parallel_cost_consumption(Info, ProductionsMap,
     ParConsTime0 = max(ParConsTimeBlocked, ParConsTimeNotBlocked) +
         float(Info ^ ipi_opts ^ cpcp_future_wait_cost),
 
-    (
+    ( if
         % True if Q had to suspend waiting for P. Note that we don't include
         % FutureSyncTime here. This is true if Q has to block at all even if
-        % it can be made runable before the context switch is complete.
+        % it can be made runnable before the context switch is complete.
         ProdTime > ParConsTimeNotBlocked
-    ->
+    then
         % Include the time that it may take to resume this thread.
         ParConsTime = ParConsTime0 +
             float(Info ^ ipi_opts ^ cpcp_context_wakeup_delay),
         !:RevExecution =
             [(!.ResumeTime - ParConsTimeNotBlocked) | !.RevExecution],
         !:ResumeTime = ParConsTime
-    ;
+    else
         ParConsTime = ParConsTime0
     ),
 
@@ -361,7 +359,8 @@ calculate_dependent_parallel_cost_production(Info,
 
 graph_do_lookup(Lookup, Graph, GoalNum, Deps) :-
     Lookup(Graph, lookup_key(Graph, GoalNum), DepsKeys),
-    Deps = set(map(lookup_vertex(Graph), set.to_sorted_list(DepsKeys))).
+    Deps = set.list_to_set(map(lookup_vertex(Graph),
+        set.to_sorted_list(DepsKeys))).
 
     % foldl(get_productions_map(Goals, 0,0, _, Vars, _, map.init, Map).
     %
@@ -391,21 +390,20 @@ adjust_time_for_waits(!Time, !Executions) :-
     (
         !.Executions = [Execution | NextExecution],
         ( Start - End ) = Execution,
-        ( (!.Time + adjust_time_for_waits_epsilon) < Start ->
-            error("adjust_time_for_waits: " ++
-                "Time occurs before the current execution")
-        ; !.Time =< (End + adjust_time_for_waits_epsilon) ->
+        ( if (!.Time + adjust_time_for_waits_epsilon) < Start then
+            unexpected($pred, "Time occurs before the current execution")
+        else if !.Time =< (End + adjust_time_for_waits_epsilon) then
             % The production is within the current execution, no adjustment is
             % necessary.
             true
-        ;
+        else
             % The time is after this execution.
             !:Executions = NextExecution,
             adjust_time_for_waits_2(End, !Time, !Executions)
         )
     ;
         !.Executions = [],
-        error("adjust_time_for_waits: Time occurs after all executions")
+        unexpected($pred, "Time occurs after all executions")
     ).
 
 :- pred adjust_time_for_waits_2(float::in, float::in, float::out,
@@ -419,14 +417,15 @@ adjust_time_for_waits_2(LastEnd, !Time, !Executions) :-
         % Do the adjustment.
         !:Time = !.Time + (Start - LastEnd),
 
-        ( (!.Time + adjust_time_for_waits_epsilon) < Start ->
-            error(format("adjust_time_for_waits: Adjustment didn't work, " ++
+        ( if (!.Time + adjust_time_for_waits_epsilon) < Start then
+            unexpected($pred,
+                string.format("Adjustment didn't work, " ++
                 "time occurs before the current execution. " ++
                 "Time: %f, Start: %f.", [f(!.Time), f(Start)]))
-        ; !.Time =< (End + adjust_time_for_waits_epsilon) ->
+        else if !.Time =< (End + adjust_time_for_waits_epsilon) then
             % The adjustment worked.
             true
-        ;
+        else
             % Further adjustment is needed.
             !:Executions = NextExecution,
             adjust_time_for_waits_2(End, !Time, !Executions)
@@ -540,11 +539,11 @@ merge_consumptions_and_productions([Var - Time | Cons0], [],
     merge_consumptions_and_productions(Cons0, [], Cons).
 merge_consumptions_and_productions(Cons@[ConsVar - ConsTime | Cons0],
         Prods@[ProdVar - ProdTime | Prods0], [ProdOrCons | ProdsAndCons]) :-
-    ( ProdTime < ConsTime ->
+    ( if ProdTime < ConsTime then
         % Order earlier events first,
         ProdOrCons = ProdVar - production(ProdTime),
         merge_consumptions_and_productions(Cons, Prods0, ProdsAndCons)
-    ;
+    else
         % In this branch either the consumption occurs first or the events
         % occur at the same time in which case we order consumptions first.
         ProdOrCons = ConsVar - consumption(ConsTime),
@@ -569,9 +568,9 @@ var_productions(TimeBefore, Goal, Var, Var - Time) :-
 
     % var_first_use_time(FindProdOrCons, Time0, Goal, Var, Time).
     %
-    % if FindProdOrCons = find_production
+    % if FindProdOrCons = find_production:
     %   Time is Time0 + the time that Goal produces Var.
-    % elif FindProdOrCons = find_consumption
+    % if FindProdOrCons = find_consumption:
     %   Time is Time0 + the time that Goal first consumes Var.
     %
 :- pred var_first_use_time(find_production_or_consumption::in,
@@ -595,14 +594,14 @@ var_first_use_time(FindProdOrCons, TimeBefore, Goal, Var, Time) :-
                 FindProdOrCons = find_production
             ;
                 FindProdOrCons = find_consumption,
-                unexpected($module, $pred,
+                unexpected($pred,
                     "Found production when looking for consumption")
             )
         ;
             UseType = var_use_consumption,
             (
                 FindProdOrCons = find_production,
-                unexpected($module, $pred,
+                unexpected($pred,
                     "Found consumption when looking for production")
             ;
                 FindProdOrCons = find_consumption
@@ -624,4 +623,6 @@ var_first_use_time(FindProdOrCons, TimeBefore, Goal, Var, Time) :-
     ),
     Time = TimeBefore + UseTime.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module autopar_calc_overlap.
+%---------------------------------------------------------------------------%
