@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2011-2012 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: autopar_find_best_par.m
 % Author: pbone.
@@ -15,7 +15,7 @@
 % The following compile-time flags may introduce trace goals:
 %   debug_branch_and_bound
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module mdprof_fb.automatic_parallelism.autopar_find_best_par.
 :- interface.
@@ -44,8 +44,8 @@
     maybe(full_parallelisation)::out,
     cord(message)::in, cord(message)::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -68,7 +68,7 @@
 find_best_parallelisation(Info, Location, Goals, MaybeBestParallelisation,
         !Messages) :-
     % Decide which algorithm to use.
-    ConjunctionSize = length(Goals),
+    ConjunctionSize = list.length(Goals),
     choose_algorithm(Info, ConjunctionSize, Algorithm),
     preprocess_conjunction(Goals, MaybePreprocessedGoals, Location, !Messages),
     (
@@ -80,39 +80,39 @@ find_best_parallelisation(Info, Location, Goals, MaybeBestParallelisation,
         MaybeBestParallelisation = no
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- type best_par_algorithm_simple
-    --->    bpas_complete(maybe(int))
-    ;       bpas_greedy.
+:- type alg_for_finding_best_par_simple
+    --->    affbps_complete(maybe(int))
+    ;       affbps_greedy.
 
-:- pred choose_algorithm(implicit_parallelism_info::in,
-    int::in, best_par_algorithm_simple::out) is det.
+:- pred choose_algorithm(implicit_parallelism_info::in, int::in,
+    alg_for_finding_best_par_simple::out) is det.
 
 choose_algorithm(Info, ConjunctionSize, Algorithm) :-
-    Algorithm0 = Info ^ ipi_opts ^ cpcp_best_par_alg,
+    Algorithm0 = Info ^ ipi_opts ^ cpcp_alg_for_best_par,
     (
         (
-            Algorithm0 = bpa_complete_branches(BranchesLimit),
+            Algorithm0 = affbp_complete_branches(BranchesLimit),
             MaybeBranchesLimit = yes(BranchesLimit)
         ;
-            Algorithm0 = bpa_complete,
+            Algorithm0 = affbp_complete,
             MaybeBranchesLimit = no
         ),
-        Algorithm = bpas_complete(MaybeBranchesLimit)
+        Algorithm = affbps_complete(MaybeBranchesLimit)
     ;
-        Algorithm0 = bpa_complete_size(SizeLimit),
-        ( SizeLimit < ConjunctionSize ->
-            Algorithm = bpas_greedy
-        ;
-            Algorithm = bpas_complete(no)
+        Algorithm0 = affbp_complete_size(SizeLimit),
+        ( if SizeLimit < ConjunctionSize then
+            Algorithm = affbps_greedy
+        else
+            Algorithm = affbps_complete(no)
         )
     ;
-        Algorithm0 = bpa_greedy,
-        Algorithm = bpas_greedy
+        Algorithm0 = affbp_greedy,
+        Algorithm = affbps_greedy
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type goal_group(T)
     --->    gg_single(
@@ -142,7 +142,7 @@ gg_get_details(gg_multiple(Index, Num, P), Index, Num, P).
     % The valid placement decisions that can be made for a given goal.
     %
     % Note that this type is not as expressive as it could be, this is
-    % deliberate to avoid symmetry.  It is however still complete.
+    % deliberate to avoid symmetry. It is however still complete.
     %
     %  + place_independent is just gpe_place_in_new_conj with the next goal
     %    having gpe_place_in_new_conj.
@@ -150,17 +150,17 @@ gg_get_details(gg_multiple(Index, Num, P), Index, Num, P).
     %  + place_with_next is just gpe_place_in_new_conj with the next goal
     %    having place_with_previous.
     %
-%:- type goal_placement_enum
-%    --->    gpe_place_before_par_conj
-%    ;       gpe_place_after_par_conj
-%    ;       gpe_place_with_previous
-%    ;       gpe_place_in_new_conj.
+% :- type goal_placement_enum
+%     --->    gpe_place_before_par_conj
+%     ;       gpe_place_after_par_conj
+%     ;       gpe_place_with_previous
+%     ;       gpe_place_in_new_conj.
 %
-%:- type can_split_group
-%    --->    can_split_group
-%    ;       cannot_split_group.
+% :- type can_split_group
+%     --->    can_split_group
+%     ;       cannot_split_group.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type goal_classification
     --->    gc_cheap_goals
@@ -168,26 +168,28 @@ gg_get_details(gg_multiple(Index, Num, P), Index, Num, P).
 
     % A summary of goals before parallelisation.
     %
-    % All indexes are 0-based except where otherwise noded.
+    % All indexes are 0-based except where otherwise noted.
     %
 :- type goals_for_parallelisation
     --->    goals_for_parallelisation(
-                gfp_goals                   :: array(pard_goal_detail),
+                gfp_goals               :: array(pard_goal_detail),
 
-                gfp_first_costly_goal       :: int,
-                gfp_last_costly_goal        :: int,
+                gfp_first_costly_goal   :: int,
+                gfp_last_costly_goal    :: int,
 
-                gfp_groups                  ::
-                    list(goal_group(goal_classification)),
+                gfp_groups              :: list(goal_group(
+                                                goal_classification)),
 
                 % The indexes in the dependency graph are 1-based.
-                gfp_dependency_graphs       :: dependency_graphs,
+                % XXX Why? This unnecessary difference looks to be
+                % a likely source of bugs.
+                gfp_dependency_graphs   :: dependency_graphs,
 
-                gfp_costly_goal_indexes     :: list(int),
-                gfp_num_calls               :: int
+                gfp_costly_goal_indexes :: list(int),
+                gfp_num_calls           :: int
             ).
 
-:- inst goals_for_parallelisation
+:- inst goals_for_parallelisation for goals_for_parallelisation/0
     --->    goals_for_parallelisation(
                 ground, ground, ground,
                 non_empty_list,
@@ -197,8 +199,8 @@ gg_get_details(gg_multiple(Index, Num, P), Index, Num, P).
             ).
 
 :- pred preprocess_conjunction(list(pard_goal_detail)::in,
-    maybe(goals_for_parallelisation)::out(maybe(goals_for_parallelisation)),
-    program_location::in, cord(message)::in, cord(message)::out) is det.
+    maybe(goals_for_parallelisation)::out, program_location::in,
+    cord(message)::in, cord(message)::out) is det.
 
 preprocess_conjunction(Goals, MaybeGoalsForParallelisation, Location,
         !Messages) :-
@@ -209,14 +211,14 @@ preprocess_conjunction(Goals, MaybeGoalsForParallelisation, Location,
 
     % Phase 2: Find the costly calls.
     identify_costly_goals(Goals, 0, CostlyGoalsIndexes),
-    (
+    ( if
         CostlyGoalsIndexes = [FirstCostlyGoalIndexPrime | OtherIndexes],
         list.last(OtherIndexes, LastCostlyGoalIndexPrime)
-    ->
+    then
         FirstCostlyGoalIndex = FirstCostlyGoalIndexPrime,
         LastCostlyGoalIndex = LastCostlyGoalIndexPrime
-    ;
-        unexpected($module, $pred, "too few costly goals")
+    else
+        unexpected($pred, "too few costly goals")
     ),
 
     % Phase 3: Check that all the middle goals are model det.
@@ -232,7 +234,7 @@ preprocess_conjunction(Goals, MaybeGoalsForParallelisation, Location,
             FirstCostlyGoalIndex, LastCostlyGoalIndex, [], RevGoalGroups),
         list.reverse(RevGoalGroups, GoalGroups),
 
-        FirstCostlyGoal = lookup(GoalsArray, FirstCostlyGoalIndex),
+        FirstCostlyGoal = array.lookup(GoalsArray, FirstCostlyGoalIndex),
         Cost = FirstCostlyGoal ^ goal_annotation ^ pgd_cost,
         NumCalls = goal_cost_get_calls(Cost),
 
@@ -249,58 +251,75 @@ preprocess_conjunction(Goals, MaybeGoalsForParallelisation, Location,
         ; Detism = cc_nondet_rep
         ),
         MaybeGoalsForParallelisation = no,
-        append_message(Location, notice_candidate_conjunction_not_det(Detism),
-            !Messages)
+        Msg = notice_candidate_conjunction_not_det(Detism),
+        append_message(Location, Msg, !Messages)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred build_dependency_graphs(list(pard_goal_detail)::in,
     dependency_graphs::out) is det.
 
 build_dependency_graphs(Goals, Maps) :-
     Graph0 = digraph.init,
-    build_dependency_graph(Goals, 1, map.init, _VarDepMap, Graph0, Graph),
+    build_dependency_graph_for_conjuncts(Goals, 1,
+        map.init, _VarDepMap, Graph0, Graph),
     Maps = dependency_graphs(Graph, tc(Graph)).
 
-:- pred build_dependency_graph(list(pard_goal_detail)::in, int::in,
-    map(var_rep, int)::in, map(var_rep, int)::out,
+    % This type maps each variable to the conjunct number of the conjunct
+    % that produces it.
+:- type producer_map == map(var_rep, int).
+
+:- pred build_dependency_graph_for_conjuncts(list(pard_goal_detail)::in,
+    int::in, producer_map::in, producer_map::out,
     digraph(int)::in, digraph(int)::out) is det.
 
-build_dependency_graph([], _ConjNum, !VarDepMap, !Graph).
-build_dependency_graph([PG | PGs], ConjNum, !VarDepMap, !Graph) :-
+build_dependency_graph_for_conjuncts([], _ConjNum,
+        !VarProducerMap, !Graph).
+build_dependency_graph_for_conjuncts([PG | PGs], ConjNum,
+        !VarProducerMap, !Graph) :-
+    build_dependency_graph_for_conjunct(PG, ConjNum,
+        !VarProducerMap, !Graph),
+    build_dependency_graph_for_conjuncts(PGs, ConjNum + 1,
+        !VarProducerMap, !Graph).
+
+:- pred build_dependency_graph_for_conjunct(pard_goal_detail::in, int::in,
+    producer_map::in, producer_map::out,
+    digraph(int)::in, digraph(int)::out) is det.
+
+build_dependency_graph_for_conjunct(PG, ConjNum, !ProducerMap, !Graph) :-
     InstMapInfo = PG ^ goal_annotation ^ pgd_inst_map_info,
 
-    % For each variable consumed by a goal we find out which goals instantiate
-    % that variable and add them as its dependencies along with their
-    % dependencies.  NOTE: We only consider variables that are read
-    % and not those that are set. This is safe because we only analyse
-    % single assignment code.
-    RefedVars = InstMapInfo ^ im_consumed_vars,
+    % For each variable consumed by a goal, we find out which previous
+    % conjunct, if any, instantiates that variable, and add it as one of
+    % this conjunct's dependencies.
+    ConsumedVars = InstMapInfo ^ im_consumed_vars,
     digraph.add_vertex(ConjNum, ThisConjKey, !Graph),
-    MaybeAddEdge = ( pred(RefedVar::in, GraphI0::in, GraphI::out) is det :-
-        ( map.search(!.VarDepMap, RefedVar, DepConj) ->
-            % DepConj should already be in the graph.
-            digraph.lookup_key(GraphI0, DepConj, DepConjKey),
-            digraph.add_edge(DepConjKey, ThisConjKey, GraphI0, GraphI)
-        ;
-            GraphI = GraphI0
-        )
-    ),
-    list.foldl(MaybeAddEdge, set.to_sorted_list(RefedVars), !Graph),
+    MaybeAddEdge =
+        ( pred(ConsumedVar::in, Graph0::in, Graph::out) is det :-
+            ( if map.search(!.ProducerMap, ConsumedVar, ProducerConjNum) then
+                % ProducerConjNum should already be in the graph.
+                digraph.lookup_key(Graph0, ProducerConjNum, ProducerKey),
+                digraph.add_edge(ProducerKey, ThisConjKey, Graph0, Graph)
+            else
+                % ConsumedVar must have been instantiated *before*
+                % this whole conjunction, possibly but not necessarily
+                % in the clause head.
+                Graph = Graph0
+            )
+        ),
+    list.foldl(MaybeAddEdge, set.to_sorted_list(ConsumedVars), !Graph),
 
-    % For each variable instantiated by this goal, add it to the VarDepMap
-    % with this goal as its instantiator. That is a mapping from the variable
-    % to the conj num.
-    InstVars = InstMapInfo ^ im_bound_vars,
-    InsertForConjNum = ( pred(InstVar::in, MapI0::in, MapI::out) is det :-
-        map.det_insert(InstVar, ConjNum, MapI0, MapI)
-    ),
-    set.fold(InsertForConjNum, InstVars, !VarDepMap),
+    % For each variable instantiated by this goal, add it to !ProducerMap
+    % with this goal as its producer.
+    ProducedVars = InstMapInfo ^ im_bound_vars,
+    InsertThisConjAsProducer =
+        ( pred(ProducedVar::in, ProducerMap0::in, ProducerMap::out) is det :-
+            map.det_insert(ProducedVar, ConjNum, ProducerMap0, ProducerMap)
+        ),
+    set.fold(InsertThisConjAsProducer, ProducedVars, !ProducerMap).
 
-    build_dependency_graph(PGs, ConjNum + 1, !VarDepMap, !Graph).
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred goal_accumulate_detism(int::in, pard_goal_detail::in,
     detism_rep::in, detism_rep::out) is det.
@@ -340,25 +359,25 @@ goal_accumulate_detism(_, Goal, !Detism) :-
         GoalCommittedChoice = committed_choice,
         CommittedChoice = CommittedChoice0
     ;
-        GoalCommittedChoice = not_committed_cnoice,
-        CommittedChoice = not_committed_cnoice
+        GoalCommittedChoice = not_committed_choice,
+        CommittedChoice = not_committed_choice
     ),
-    (
+    ( if
         promise_equivalent_solutions [FinalDetism] (
             detism_components(FinalDetism, Solutions, CanFail),
             detism_committed_choice(FinalDetism, CommittedChoice)
         )
-    ->
+    then
         !:Detism = FinalDetism
-    ;
-        unexpected($module, $pred, "cannot compute detism from components.")
+    else
+        unexpected($pred, "cannot compute detism from components.")
     ).
 
     % foldl(preprocess_conjunction_into_groups, Goals, FirstCostlyGoalIndex,
     %   LastCostlyGoalIndex, RevGoalGroups)).
     %
     % GoalGroups are Goals divided into groups of single costly goals, Larger
-    % groups are not currently used.  Only the goals within the range of costly
+    % groups are not currently used. Only the goals within the range of costly
     % goals inclusive are analysed.
     %
 :- pred preprocess_conjunction_into_groups(int::in, pard_goal_detail::in,
@@ -378,18 +397,20 @@ preprocess_conjunction_into_groups(Index, Goal, !RevGoalGroups) :-
     GoalGroup = gg_single(Index, GoalClassification),
     !:RevGoalGroups = [GoalGroup | !.RevGoalGroups].
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Find the best parallelisation using the branch and bound algorithm.
     %
 :- pred find_best_parallelisation_complete_bnb(implicit_parallelism_info::in,
-    program_location::in, best_par_algorithm_simple::in,
+    program_location::in, alg_for_finding_best_par_simple::in,
     goals_for_parallelisation::in, maybe(full_parallelisation)::out) is det.
 
 find_best_parallelisation_complete_bnb(Info, Location, Algorithm,
         PreprocessedGoals, MaybeBestParallelisation) :-
     trace [compile_time(flag("debug_branch_and_bound")), io(!IO)] (
-        io.format("D: Find best parallelisation for:\n%s\n",
+        io.output_stream(OutputStream, !IO),
+        io.format(OutputStream,
+            "D: Find best parallelisation for:\n%s\n",
             [s(LocationString)], !IO),
         location_to_string(1, Location, LocationCord),
         string.append_list(cord.list(LocationCord), LocationString),
@@ -398,9 +419,10 @@ find_best_parallelisation_complete_bnb(Info, Location, Algorithm,
         NumGoalsAfter = NumGoals -
             PreprocessedGoals ^ gfp_last_costly_goal - 1,
         NumGoalsMiddle = NumGoals - NumGoalsBefore - NumGoalsAfter,
-        io.format("D: Problem size (before, middle, after): %d,%d,%d\n",
+        io.format(OutputStream,
+            "D: Problem size (before, middle, after): %d,%d,%d\n",
             [i(NumGoalsBefore), i(NumGoalsMiddle), i(NumGoalsAfter)], !IO),
-        io.flush_output(!IO)
+        io.flush_output(OutputStream, !IO)
     ),
 
     promise_equivalent_solutions [GenParTime, EqualBestSolns, Profile] (
@@ -410,13 +432,14 @@ find_best_parallelisation_complete_bnb(Info, Location, Algorithm,
     ),
 
     trace [compile_time(flag("debug_branch_and_bound")), io(!IO)] (
-        io.format("D: Solutions: %d\n",
+        io.output_stream(OutputStream, !IO),
+        io.format(OutputStream, "D: Solutions: %d\n",
             [i(list.length(EqualBestSolns))], !IO),
-        io.format("D: Branch and bound profile: %s\n",
+        io.format(OutputStream, "D: Branch and bound profile: %s\n",
             [s(string(Profile))], !IO),
-        io.format("D: Time: %d ms\n\n",
+        io.format(OutputStream, "D: Time: %d ms\n\n",
             [i(GenParTime)], !IO),
-        io.flush_output(!IO)
+        io.flush_output(OutputStream, !IO)
     ),
 
     (
@@ -466,14 +489,14 @@ find_best_parallelisation_complete_bnb(Info, Location, Algorithm,
             ).
 
 :- pred generate_parallelisations(implicit_parallelism_info::in,
-    best_par_algorithm_simple::in, goals_for_parallelisation::in,
+    alg_for_finding_best_par_simple::in, goals_for_parallelisation::in,
     pair(list(incomplete_parallelisation), bnb_profile)::out) is det.
 
 generate_parallelisations(Info, Algorithm, GoalsForParallelisation,
         EqualBestSolns - FinalProfile) :-
-    some [!GoalGroups, !MaybeBestSolns, !Profile] (
+    some [!GoalGroups, !MaybeBestSolns, !Profile, !IncompleteParallelisation] (
         start_building_parallelisation(GoalsForParallelisation,
-            IncompleteParallelisation0),
+            !:IncompleteParallelisation),
 
         % Set the last scheduled goal to the goal at the end of the first
         % group, popping the first group off the list. This initialises the
@@ -488,33 +511,32 @@ generate_parallelisations(Info, Algorithm, GoalsForParallelisation,
         !:GoalGroups = GoalsForParallelisation ^ gfp_groups,
         (
             !.GoalGroups = [],
-            unexpected($module, $pred, "no goal groups")
+            unexpected($pred, "no goal groups")
         ;
             !.GoalGroups = [_],
-            unexpected($module, $pred, "only one goal group")
+            unexpected($pred, "only one goal group")
         ;
-            !.GoalGroups = [Group, _ | _],
-            !.GoalGroups = [_ | !:GoalGroups],
-            gg_get_details(Group, Index, Num, _),
+            !.GoalGroups = [Group1 | !:GoalGroups],
+            !.GoalGroups = [_ | _],
+            gg_get_details(Group1, Index, Num, _),
             LastScheduledGoal = Index + Num - 1,
-            IncompleteParallelisation1 =
-                IncompleteParallelisation0 ^ ip_last_scheduled_goal
-                    := LastScheduledGoal
+            !IncompleteParallelisation ^ ip_last_scheduled_goal
+                := LastScheduledGoal
         ),
 
         !:MaybeBestSolns = no_best_solutions,
         !:Profile = bnb_profile(0, 0, 0, 0, 0, 0),
 
         generate_parallelisations_loop(Info, Algorithm, !.GoalGroups,
-            IncompleteParallelisation1, !MaybeBestSolns, !Profile),
+            !.IncompleteParallelisation, !MaybeBestSolns, !Profile),
 
 % XXX
-%       ( semipure should_expand_search(BNBState, Algorithm) ->
+%       ( if if semipure should_expand_search(BNBState, Algorithm) then
 %           % Try to push goals into the first and last parallel conjuncts
 %           % from outside the parallel conjunction.
 %           semipure add_goals_into_first_par_conj(BNBState, !Parallelisation),
 %           semipure add_goals_into_last_par_conj(BNBState, !Parallelisation)
-%       ;
+%       else
 %           true
 %       ),
 
@@ -528,7 +550,8 @@ generate_parallelisations(Info, Algorithm, GoalsForParallelisation,
     ).
 
 :- pred generate_parallelisations_loop(implicit_parallelism_info::in,
-    best_par_algorithm_simple::in, list(goal_group(goal_classification))::in,
+    alg_for_finding_best_par_simple::in,
+    list(goal_group(goal_classification))::in,
     incomplete_parallelisation::in,
     best_solutions(incomplete_parallelisation)::in,
     best_solutions(incomplete_parallelisation)::out,
@@ -536,17 +559,20 @@ generate_parallelisations(Info, Algorithm, GoalsForParallelisation,
 
 generate_parallelisations_loop(_, _, [],
         !.IncompleteParallelisation, !MaybeBestSolns, !Profile) :-
-    % Verify that we have generated at least two parallel conjuncts.
-    ( ip_get_num_parallel_conjuncts(!.IncompleteParallelisation) >= 2 ->
+    % Check whether we have generated at least two parallel conjuncts.
+    ( if ip_get_num_parallel_conjuncts(!.IncompleteParallelisation) >= 2 then
         maybe_update_best_complete_parallelisation(!.IncompleteParallelisation,
             !MaybeBestSolns, !Profile)
-    ;
-        % This is not a solution, so do not try to update !MaybeBestSolns.
+    else
+        % We have not. That means that this is not a solution, so
+        % do not try to update !MaybeBestSolns.
         !Profile ^ bnbp_complete_non_solution :=
             !.Profile ^ bnbp_complete_non_solution + 1
     ).
 generate_parallelisations_loop(Info, Algorithm, [GoalGroup | GoalGroups],
         !.IncompleteParallelisation, !MaybeBestSolns, !Profile) :-
+    % XXX The algorithm could be closer to the one in the paper.
+
     LastScheduledGoal0 = !.IncompleteParallelisation ^ ip_last_scheduled_goal,
     gg_get_details(GoalGroup, _Index, Num, _Classification),
 
@@ -575,11 +601,11 @@ generate_parallelisations_loop(Info, Algorithm, [GoalGroup | GoalGroups],
             MaybeAddToLastCost = yes(AddToLastCost),
             (
                 MaybeAddToNewCost = yes(AddToNewCost),
-                ( AddToNewCost > AddToLastCost ->
+                ( if AddToNewCost > AddToLastCost then
                     % Adding to the last parallel conjunct is better.
                     Best0 = !.AddToLastParallelisation,
                     MaybeNextBest0 = yes(!.AddToNewParallelisation)
-                ;
+                else
                     % Adding to a new parallel conjunct is better.
                     Best0 = !.AddToNewParallelisation,
                     MaybeNextBest0 = yes(!.AddToLastParallelisation)
@@ -598,26 +624,26 @@ generate_parallelisations_loop(Info, Algorithm, [GoalGroup | GoalGroups],
         )
     ),
 
-    % XXX: This ite could be simpler, and the algorithm would be closer to the
-    % one in the paper.
+    CurProfile = !.Profile,
+
+    is_incomplete_parallelisation_good_enough(Info, !.MaybeBestSolns,
+        Best0, Best, !Profile, BestGoodEnough),
     (
+        BestGoodEnough = is_good_enough,
+        generate_parallelisations_loop(Info, Algorithm,
+            GoalGroups, Best, !MaybeBestSolns, !Profile)
+    ;
+        BestGoodEnough = is_not_good_enough
+    ),
+
+    ( if
         % Can we create an alternative branch here?
         MaybeNextBest0 = yes(NextBest0),
         % Should we create an alternative branch here?
-        should_expand_search(Algorithm, !.Profile)
-    ->
-        % Create a branch.
-        incomplete_parallelisation_is_good_enough(Info, !.MaybeBestSolns,
-            Best0, Best, !Profile, BestGoodEnough),
-        (
-            BestGoodEnough = is_good_enough,
-            generate_parallelisations_loop(Info, Algorithm,
-                GoalGroups, Best, !MaybeBestSolns, !Profile)
-        ;
-            BestGoodEnough = is_not_good_enough
-        ),
-
-        incomplete_parallelisation_is_good_enough(Info, !.MaybeBestSolns,
+        may_add_choice_point(Algorithm, CurProfile) = add_choice_point
+    then
+        % Create the alternative branch.
+        is_incomplete_parallelisation_good_enough(Info, !.MaybeBestSolns,
             NextBest0, NextBest, !Profile, NextBestGoodEnough),
         (
             NextBestGoodEnough = is_good_enough,
@@ -626,16 +652,8 @@ generate_parallelisations_loop(Info, Algorithm, [GoalGroup | GoalGroups],
         ;
             NextBestGoodEnough = is_not_good_enough
         )
-    ;
-        incomplete_parallelisation_is_good_enough(Info, !.MaybeBestSolns,
-            Best0, Best, !Profile, BestGoodEnough),
-        (
-            BestGoodEnough = is_good_enough,
-            generate_parallelisations_loop(Info, Algorithm,
-                GoalGroups, Best, !MaybeBestSolns, !Profile)
-        ;
-            BestGoodEnough = is_not_good_enough
-        )
+    else
+        true
     ).
 
 :- pred start_building_parallelisation(goals_for_parallelisation::in,
@@ -665,7 +683,7 @@ finalise_parallelisation(Incomplete, Best) :-
         MaybeCostData = yes(CostData)
     ;
         MaybeCostData = no,
-        unexpected($module, $pred, "parallelisation has no cost data")
+        unexpected($pred, "parallelisation has no cost data")
     ),
     CostData = parallelisation_cost_data(_, Overlap, Metrics0, _),
 
@@ -675,22 +693,36 @@ finalise_parallelisation(Incomplete, Best) :-
     Best = fp_parallel_execution(GoalsBefore, ParConjs, GoalsAfter,
         IsDependent, Metrics).
 
+:- type maybe_add_choice_point
+    --->    do_not_add_choice_point
+    ;       add_choice_point.
+
     % True if we should expand the search for parallelisation alternatives by
     % creating a choice point.
     %
-:- pred should_expand_search(best_par_algorithm_simple::in, bnb_profile::in)
-    is semidet.
+:- func may_add_choice_point(alg_for_finding_best_par_simple, bnb_profile)
+    = maybe_add_choice_point.
 
-should_expand_search(Algorithm, Profile) :-
-    Algorithm = bpas_complete(MaybeLimit),
+may_add_choice_point(Algorithm, Profile) = MayAddChoicePoint :-
     (
-        MaybeLimit = yes(Limit),
-        NumIncompleteTests =
-            Profile ^ bnbp_incomplete_not_good_enough +
-            Profile ^ bnbp_incomplete_good_enough,
-        NumIncompleteTests < Limit
+        Algorithm = affbps_greedy,
+        MayAddChoicePoint = do_not_add_choice_point
     ;
-        MaybeLimit = no
+        Algorithm = affbps_complete(MaybeLimit),
+        (
+            MaybeLimit = yes(Limit),
+            NumIncompleteTests =
+                Profile ^ bnbp_incomplete_not_good_enough +
+                Profile ^ bnbp_incomplete_good_enough,
+            ( if NumIncompleteTests < Limit then
+                MayAddChoicePoint = add_choice_point
+            else
+                MayAddChoicePoint = do_not_add_choice_point
+            )
+        ;
+            MaybeLimit = no,
+            MayAddChoicePoint = add_choice_point
+        )
     ).
 
 :- pred maybe_update_best_complete_parallelisation(
@@ -711,16 +743,16 @@ maybe_update_best_complete_parallelisation(CurSoln,
             !.Profile ^ bnbp_complete_best_solution + 1
     ;
         MaybeBestSolns0 = best_solutions(BestSolns0, BestCost0),
-        ( CurSolnCost < BestCost0 ->
+        ( if CurSolnCost < BestCost0 then
             MaybeBestSolns = best_solutions([CurSoln], CurSolnCost),
             !Profile ^ bnbp_complete_best_solution :=
                 !.Profile ^ bnbp_complete_best_solution + 1
-        ; CurSolnCost = BestCost0 ->
+        else if CurSolnCost = BestCost0 then
             BestSolns = [CurSoln | BestSolns0],
             MaybeBestSolns = best_solutions(BestSolns, BestCost0),
             !Profile ^ bnbp_complete_equal_solution :=
                 !.Profile ^ bnbp_complete_equal_solution + 1
-        ;
+        else
             % Do not update !MaybeBestSolns.
             MaybeBestSolns = MaybeBestSolns0,
             !Profile ^ bnbp_complete_worse_solution :=
@@ -735,16 +767,16 @@ maybe_update_best_complete_parallelisation(CurSoln,
     % Test the parallelisation against the best one known to the branch and
     % bound solver.
     %
-:- pred incomplete_parallelisation_is_good_enough(
+:- pred is_incomplete_parallelisation_good_enough(
     implicit_parallelism_info::in,
     best_solutions(incomplete_parallelisation)::in,
     incomplete_parallelisation::in, incomplete_parallelisation::out,
     bnb_profile::in, bnb_profile::out, is_good_enough::out) is det.
 
-incomplete_parallelisation_is_good_enough(Info, MaybeBestSolns,
+is_incomplete_parallelisation_good_enough(Info, MaybeBestSolns,
         !IncompleteParallelisation, !Profile, GoodEnough) :-
     calculate_parallel_cost(Info, !IncompleteParallelisation, CostData),
-    ( test_dependence(Info, CostData) ->
+    ( if test_dependence(Info, CostData) then
         (
             MaybeBestSolns = no_best_solutions,
             !Profile ^ bnbp_incomplete_good_enough :=
@@ -754,17 +786,17 @@ incomplete_parallelisation_is_good_enough(Info, MaybeBestSolns,
             MaybeBestSolns = best_solutions(_, BestSolnCost),
             CurIncompleteCost =
                 incomplete_parallelisation_cost(!.IncompleteParallelisation),
-            ( CurIncompleteCost > BestSolnCost ->
+            ( if CurIncompleteCost > BestSolnCost then
                 !Profile ^ bnbp_incomplete_not_good_enough :=
                     !.Profile ^ bnbp_incomplete_not_good_enough + 1,
                 GoodEnough = is_not_good_enough
-            ;
+            else
                 !Profile ^ bnbp_incomplete_good_enough :=
                     !.Profile ^ bnbp_incomplete_good_enough + 1,
                 GoodEnough = is_good_enough
             )
         )
-    ;
+    else
         !Profile ^ bnbp_incomplete_not_good_enough :=
             !.Profile ^ bnbp_incomplete_not_good_enough + 1,
         GoodEnough = is_not_good_enough
@@ -799,9 +831,9 @@ par_conj_overlap_is_dependent(peo_conjunction(Left, _, VarSet0),
         IsDependent = conjuncts_are_dependent(VarSet)
     ;
         IsDependent0 = conjuncts_are_independent,
-        ( set.empty(VarSet0) ->
+        ( if set.is_empty(VarSet0) then
             IsDependent = conjuncts_are_independent
-        ;
+        else
             IsDependent = conjuncts_are_dependent(VarSet0)
         )
     ).
@@ -815,10 +847,10 @@ par_conj_overlap_is_dependent(peo_conjunction(Left, _, VarSet0),
 update_incomplete_parallelisation_cost(Info, !IncompleteParallelisation,
         MaybeCost) :-
     calculate_parallel_cost(Info, !IncompleteParallelisation, CostData),
-    ( test_dependence(Info, CostData) ->
+    ( if test_dependence(Info, CostData) then
         Cost = incomplete_parallelisation_cost(!.IncompleteParallelisation),
         MaybeCost = yes(Cost)
-    ;
+    else
         MaybeCost = no
     ).
 
@@ -830,8 +862,7 @@ incomplete_parallelisation_cost(IncompleteParallelisation) = Cost :-
         MaybeCostData = yes(CostData)
     ;
         MaybeCostData = no,
-        unexpected($module, $pred,
-            "incomplete parallelisation has no cost data")
+        unexpected($pred, "incomplete parallelisation has no cost data")
     ),
     IncompleteMetrics = CostData ^ pcd_par_exec_metrics,
     FullMetrics = finalise_parallel_exec_metrics(IncompleteMetrics),
@@ -858,7 +889,7 @@ full_parallelisation_cost(FullParallelisation) = Cost :-
     FullMetrics = FullParallelisation ^ fp_par_exec_metrics,
     Cost = full_parallelisation_metrics_cost(FullMetrics).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % XXX
 % :- semipure pred add_goals_into_first_par_conj(
@@ -906,7 +937,7 @@ full_parallelisation_cost(FullParallelisation) = Cost :-
 %         true
 %     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred add_one_goal_into_first_par_conj(incomplete_parallelisation::in,
     incomplete_parallelisation::out) is det.
@@ -928,19 +959,21 @@ add_one_goal_into_last_par_conj(!Parallelisation) :-
     !Parallelisation ^ ip_maybe_goals_after_cost := no,
     !Parallelisation ^ ip_maybe_par_cost_data := no.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred foldl_sub_array(pred(int, T, A, A), array(T), int, int, A, A).
 :- mode foldl_sub_array(pred(in, in, in, out) is det,
     in, in, in, in, out) is det.
 
 foldl_sub_array(Pred, Array, Index, Last, !Acc) :-
-    ( Index =< Last ->
+    ( if Index =< Last then
         array.lookup(Array, Index, X),
         Pred(Index, X, !Acc),
         foldl_sub_array(Pred, Array, Index + 1, Last, !Acc)
-    ;
+    else
         true
     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module autopar_find_best_par.
+%---------------------------------------------------------------------------%
