@@ -1,17 +1,17 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2008-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Authors: pbone, zs.
 %
 % This file implements the coverage propagation algorithm, which attaches
 % coverage information to the component goals of a procedure body.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module coverage.
 
@@ -52,7 +52,7 @@
     int::out, int::out) is det.
 :- pred get_coverage_after_det(coverage_info::in, int::out) is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % This is similar to the coverage_point type in
     % mdbcomp/program_representation.m, however it includes an integer count
@@ -80,7 +80,7 @@
 :- pred coverage_point_arrays_to_list(array(coverage_point_info)::in,
     array(int)::in, list(coverage_point)::out) is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Annotate the program representation structure with coverage information.
     %
@@ -100,7 +100,7 @@
     containing_goal_map::in, goal_id::in,
     goal_attr_array(coverage_info)::gaa_uo) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % foldl(add_coverage_point, CoveragePoints, map.init, SolnsCPs,
     %   map.init, BranchCPs),
@@ -111,7 +111,7 @@
     map(reverse_goal_path, coverage_point)::in,
     map(reverse_goal_path, coverage_point)::out) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -123,7 +123,7 @@
 :- import_module require.
 :- import_module string.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 get_coverage_before(coverage_known(Before, _), Before).
 get_coverage_before(coverage_known_zero, 0).
@@ -140,40 +140,40 @@ get_coverage_after(coverage_known_same(After), After).
 get_coverage_after(coverage_known_after(After), After).
 
 get_coverage_before_det(Coverage, Before) :-
-    ( get_coverage_before(Coverage, BeforePrime) ->
+    ( if get_coverage_before(Coverage, BeforePrime) then
         Before = BeforePrime
-    ;
+    else
         complete_coverage_error
     ).
 
 get_coverage_before_and_after_det(Coverage, Before, After) :-
-    ( get_coverage_before_and_after(Coverage, BeforePrime, AfterPrime) ->
+    ( if get_coverage_before_and_after(Coverage, BeforePrime, AfterPrime) then
         Before = BeforePrime,
         After = AfterPrime
-    ;
+    else
         complete_coverage_error
     ).
 
 get_coverage_after_det(Coverage, After) :-
-    ( get_coverage_after(Coverage, AfterPrime) ->
+    ( if get_coverage_after(Coverage, AfterPrime) then
         After = AfterPrime
-    ;
+    else
         complete_coverage_error
     ).
 
 :- pred complete_coverage_error is erroneous.
 
 complete_coverage_error :-
-    unexpected($module, $pred, "Expected complete coverage information").
+    unexpected($pred, "Expected complete coverage information").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 coverage_point_arrays_to_list(StaticArray, DynamicArray, CoveragePoints) :-
     array.bounds(StaticArray, Min, Max),
-    ( array.bounds(DynamicArray, Min, Max) ->
+    ( if array.bounds(DynamicArray, Min, Max) then
         true
-    ;
-        unexpected($module, $pred, "bounds do not match")
+    else
+        unexpected($pred, "bounds do not match")
     ),
     coverage_point_arrays_to_list_2(Min, Max, StaticArray, DynamicArray,
         [], CoveragePoints).
@@ -184,18 +184,18 @@ coverage_point_arrays_to_list(StaticArray, DynamicArray, CoveragePoints) :-
 
 coverage_point_arrays_to_list_2(Num, Max, StaticArray, DynamicArray,
         !CoveragePoints) :-
-    ( Num =< Max ->
+    ( if Num =< Max then
         array.lookup(StaticArray, Num, coverage_point_info(GoalPath, CPType)),
         array.lookup(DynamicArray, Num, Count),
         CP = coverage_point(Count, GoalPath, CPType),
         !:CoveragePoints = [CP | !.CoveragePoints],
         coverage_point_arrays_to_list_2(Num + 1, Max,
             StaticArray, DynamicArray, !CoveragePoints)
-    ;
+    else
         true
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type coverage_before
     --->    before_unknown
@@ -322,9 +322,9 @@ goal_annotate_coverage(Goal, Info, Before, After, !Array) :-
             ; AtomicGoal = higher_order_call_rep(_, _)
             ; AtomicGoal = method_call_rep(_, _, _)
             ),
-            (
+            ( if
                 map.search(Info ^ cri_call_sites, RevGoalPath, Cost)
-            ->
+            then
                 % Entry due to redo is not counted at the point before the
                 % goal, it is represented when the number of exists is greater
                 % than the number of calls. XXX This won't work with nondet
@@ -334,8 +334,8 @@ goal_annotate_coverage(Goal, Info, Before, After, !Array) :-
                 require(unify(Before, before_coverage(Calls)),
                   "Coverage before call doesn't match calls port on call site"),
                 After0 = after_coverage(Exits)
-            ;
-                unexpected($module, $pred,
+            else
+                unexpected($pred,
                     "Couldn't look up call site for port counts GP: " ++
                     rev_goal_path_to_string(RevGoalPath))
             )
@@ -359,41 +359,43 @@ goal_annotate_coverage(Goal, Info, Before, After, !Array) :-
     % even when the coverage has been calculated from inner goals, since this
     % is used to perform an assertion that these two sources agree about the
     % coverage after this goal.
-    (
+    ( if
         map.search(Info ^ cri_solns_coverage_points, RevGoalPath,
             CoveragePoint)
-    ->
+    then
         CoveragePoint = coverage_point(CoverageAfterCount, _, _),
         after_count_from_either_source(after_coverage(CoverageAfterCount),
             After0, After)
-    ;
+    else
         After0 = After
     ),
     GoalCoverage = construct_before_after_coverage(Before, After),
     update_goal_attribute(GoalId, GoalCoverage, !Array),
 
     trace [compile_time(flag("debug_coverage_propagation")), io(!IO)] (
-        io.write_string("goal_annotate_coverage: done\n", !IO),
-        io.format("\tGoalPath: %s\n\tDetism %s\n\tCoverage; %s\n",
-            [s(rev_goal_path_to_string(RevGoalPath)),
-             s(string(Detism)),
-             s(string(GoalCoverage))], !IO)
+        io.output_stream(OutputStream, !IO),
+        io.write_string(OutputStream,
+            "goal_annotate_coverage: done\n", !IO),
+        io.format(OutputStream,
+            "\tGoalPath: %s\n\tDetism %s\n\tCoverage; %s\n",
+            [s(rev_goal_path_to_string(RevGoalPath)), s(string(Detism)),
+            s(string(GoalCoverage))], !IO)
     ),
     trace [compile_time(not flag("no_coverage_propagation_assertions"))] (
-        ( check_coverage_complete(GoalCoverage, GoalExpr) ->
+        ( if check_coverage_complete(GoalCoverage, GoalExpr) then
             true
-        ;
-            unexpected($module, $pred,
+        else
+            unexpected($pred,
                 string.format("check_coverage_complete failed\n" ++
                     "\tCoverage: %s\n\tGoalPath: %s\n\tProc: %s\n",
                     [s(string(GoalCoverage)),
                      s(rev_goal_path_to_string(RevGoalPath)),
                      s(string(Info ^ cri_proc))]))
         ),
-        ( check_coverage_regarding_detism(GoalCoverage, Detism) ->
+        ( if check_coverage_regarding_detism(GoalCoverage, Detism) then
             true
-        ;
-            unexpected($module, $pred,
+        else
+            unexpected($pred,
                 string.format("check_coverage_regarding_detism failed: %s %s",
                     [s(string(GoalCoverage)), s(string(Detism))]))
         )
@@ -434,9 +436,9 @@ construct_before_after_coverage(Before, After) = Coverage :-
             Coverage = coverage_known_before(BeforeExecCount)
         ;
             After = after_known(AfterExecCount),
-            ( BeforeExecCount = AfterExecCount ->
+            ( if BeforeExecCount = AfterExecCount then
                 Coverage = coverage_known_same(BeforeExecCount)
-            ;
+            else
                 Coverage = coverage_known(BeforeExecCount, AfterExecCount)
             )
         ;
@@ -508,7 +510,9 @@ disj_annotate_coverage([Disj | Disjs], Info, Solutions, Before0, !SumAfter,
 
 switch_annotate_coverage(Cases, Info, CanFail, Before, After, !Array) :-
     trace [compile_time(flag("debug_coverage_propagation")), io(!IO)] (
-        io.format("Switch: Before0: %s\n", [s(string(Before))], !IO)
+        io.output_stream(OutputStream, !IO),
+        io.format(OutputStream, "Switch: Before0: %s\n",
+            [s(string(Before))], !IO)
     ),
 
     switch_annotate_coverage_2(Cases, Info, CanFail, Before,
@@ -535,10 +539,12 @@ switch_annotate_coverage(Cases, Info, CanFail, Before, After, !Array) :-
             Result = yes
         ;
             Result = no,
-            error(string.format("check_switch_coverage failed\n\t" ++
-                "CanFail: %s\n\tCases: %s\n\tBefore: %s, After: %s\n",
-                [s(string(CanFail)), s(string(Cases)),
-                s(string(Before)), s(string(After))]))
+            unexpected($pred,
+                string.format(
+                    "check_switch_coverage failed\n\t" ++
+                    "CanFail: %s\n\tCases: %s\n\tBefore: %s, After: %s\n",
+                    [s(string(CanFail)), s(string(Cases)),
+                    s(string(Before)), s(string(After))]))
         )
     ).
 
@@ -572,7 +578,7 @@ switch_annotate_coverage_2([Case | Cases], Info, CanFail, SwitchBefore,
     %
     % If we cannot calculate this case's coverage information, then try to
     % retrieve the information from a coverage point associated with the case.
-    (
+    ( if
         Cases = [],
         CanFail = switch_can_not_fail_rep,
         (
@@ -587,10 +593,10 @@ switch_annotate_coverage_2([Case | Cases], Info, CanFail, SwitchBefore,
             !.SumBefore = sum_before_zero,
             SumBeforeExecCount = 0
         )
-    ->
+    then
         BeforeCase =
             before_coverage(SwitchBeforeExecCount - SumBeforeExecCount)
-    ;
+    else
         % Search for a coverage point for this case.
         RevCaseGoalPath = map.lookup(Info ^ cri_goal_path_map,
             Goal ^ goal_annotation),
@@ -648,7 +654,7 @@ ite_annotate_coverage(Cond, Then, Else, Info, RevGoalPath, Before, After,
         ( CondSolns = at_most_one_rep
         ; CondSolns = at_most_zero_rep
         ),
-        % Assumeing that Cond never throws an exception and we know the
+        % Assuming that Cond never throws an exception and we know the
         % coverage before the ite and at the beginning of Then, then the
         % coverage at the beginning of Else can be inferred.
         (
@@ -680,7 +686,9 @@ ite_annotate_coverage(Cond, Then, Else, Info, RevGoalPath, Before, After,
     ),
 
     trace [compile_time(flag("debug_coverage_propagation")), io(!IO)] (
-        io.format("ITE Coverage inferred before then and else branches:\n" ++
+        io.output_stream(OutputStream, !IO),
+        io.format(OutputStream,
+            "ITE Coverage inferred before then and else branches:\n" ++
             "\tWhole: %s \n\tThen: %s\n\tElse: %s\n" ++
             "\tGoalPath %s\n",
             [s(string(Before)), s(string(BeforeThen)), s(string(BeforeElse)),
@@ -724,21 +732,23 @@ ite_annotate_coverage(Cond, Then, Else, Info, RevGoalPath, Before, After,
     ),
 
     trace [compile_time(not flag("no_coverage_propagation_assertions"))] (
-        (
+        ( if
             check_ite_coverage(Before, After, Before, AfterCond,
                 BeforeThen, AfterThen, BeforeElse, AfterElse, CondDetism)
-        ->
+        then
             true
-        ;
-            error(string.format("check_ite_coverage/4 failed\n" ++
-                "\tWhole: %s %s\n" ++
-                "\tCond: %s %s\n\tThen: %s %s\n\tElse: %s %s\n" ++
-                "\tGoalPath: %s\n",
-                [s(string(Before)), s(string(After)),
-                s(string(Before)), s(string(AfterCond)),
-                s(string(BeforeThen)), s(string(AfterThen)),
-                s(string(BeforeElse)), s(string(AfterElse)),
-                s(rev_goal_path_to_string(RevGoalPath))]))
+        else
+            unexpected($pred,
+                string.format(
+                    "check_ite_coverage/4 failed\n" ++
+                    "\tWhole: %s %s\n" ++
+                    "\tCond: %s %s\n\tThen: %s %s\n\tElse: %s %s\n" ++
+                    "\tGoalPath: %s\n",
+                    [s(string(Before)), s(string(After)),
+                    s(string(Before)), s(string(AfterCond)),
+                    s(string(BeforeThen)), s(string(AfterThen)),
+                    s(string(BeforeElse)), s(string(AfterElse)),
+                    s(rev_goal_path_to_string(RevGoalPath))]))
         )
     ).
 
@@ -749,10 +759,10 @@ ite_annotate_coverage(Cond, Then, Else, Info, RevGoalPath, Before, After,
     reverse_goal_path::in, coverage_before::out) is det.
 
 get_branch_start_coverage(Info, RevGoalPath, Before) :-
-    ( map.search(Info ^ cri_branch_coverage_points, RevGoalPath, CP) ->
+    ( if map.search(Info ^ cri_branch_coverage_points, RevGoalPath, CP) then
         CP = coverage_point(ExecCount, _, _),
         Before = before_coverage(ExecCount)
-    ;
+    else
         Before = before_unknown
     ).
 
@@ -767,7 +777,9 @@ negation_annotate_coverage(Goal, Info, Before, After, !Array) :-
     % The coverage after a negation is always unknown.
     After = after_unknown,
     trace [compile_time(flag("debug_coverage_propagation")), io(!IO)] (
-        io.format("Negation: setting negation: before %s, after %s\n",
+        io.output_stream(OutputStream, !IO),
+        io.format(OutputStream,
+            "Negation: setting negation: before %s, after %s\n",
             [s(string(Before)), s(string(After))], !IO)
     ).
 
@@ -787,7 +799,7 @@ scope_annotate_coverage(Goal, Info, MaybeCut, Before, After, !Array) :-
         After = AfterScopedGoal
     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % These predicates are used to check that computed coverage counts make sense.
 %
@@ -819,9 +831,9 @@ detism_coverage_ok(Coverage, Detism) = OK :-
             % Execution may leave via the Excp port rather than the exit port.
             % so the exit port count may be smaller than or equal to the entry
             % port count.
-            ( Entry >= Exit ->
+            ( if Entry >= Exit then
                 OK = yes
-            ;
+            else
                 OK = no
             )
         ;
@@ -846,9 +858,9 @@ detism_coverage_ok(Coverage, Detism) = OK :-
             OK = yes
         ;
             Coverage = coverage_known(Entry, Exit),
-            ( Entry >= Exit ->
+            ( if Entry >= Exit then
                 OK = yes
-            ;
+            else
                 OK = no
             )
         )
@@ -881,9 +893,9 @@ detism_coverage_ok(Coverage, Detism) = OK :-
             ; Coverage = coverage_known_same(Exit)
             ; Coverage = coverage_known_after(Exit)
             ),
-            ( Exit = 0 ->
+            ( if Exit = 0 then
                 OK = yes
-            ;
+            else
                 OK = no
             )
         ;
@@ -923,9 +935,9 @@ check_switch_coverage(CanFail, Cases, Before, Array, Result) :-
                         Before = before_zero,
                         SumB = 0
                     ),
-                    ( SumA = SumB ->
+                    ( if SumA = SumB then
                         Result = yes
-                    ;
+                    else
                         Result = no
                     )
                 )
@@ -972,36 +984,36 @@ sum_switch_case_coverage(Array, case_rep(_, _, Goal), !Acc) :-
 
 check_ite_coverage(Before, After, BeforeCond, AfterCond,
         BeforeThen, AfterThen, _BeforeElse, AfterElse, CondDetism) :-
-    (
+    ( if
         Before = before_known(BeforeExecCount),
         BeforeCond = before_known(BeforeCondExecCount)
-    ->
+    then
         BeforeExecCount = BeforeCondExecCount
-    ;
+    else
         true
     ),
-    (
+    ( if
         After = after_known(AfterExecCount),
         AfterThen = after_known(AfterThenExecCount),
         AfterElse = after_known(AfterElseExecCount)
-    ->
+    then
         AfterExecCount = AfterThenExecCount + AfterElseExecCount
-    ;
+    else
         true
     ),
-    (
+    ( if
         AfterCond = after_known(AfterCondExecCount),
         BeforeThen = before_known(BeforeKnownExecCount)
-    ->
+    then
         AfterCondExecCount = BeforeKnownExecCount
-    ;
+    else
         true
     ),
     % Since the condition may throw exceptions and exception count information
     % is not propagated checking the coverage before the else goal based on the
     % coverage before and after the condition goal cannot be done.
 
-    ( AfterCond = after_known(AfterCondExecCount2) ->
+    ( if AfterCond = after_known(AfterCondExecCount2) then
         NumSolutions = detism_get_solutions(CondDetism),
         (
             NumSolutions = at_most_zero_rep,
@@ -1011,7 +1023,7 @@ check_ite_coverage(Before, After, BeforeCond, AfterCond,
             ; NumSolutions = at_most_many_rep
             )
         )
-    ;
+    else
         true
     ).
 
@@ -1030,7 +1042,7 @@ check_coverage_complete(coverage_known_zero, _GoalExpr).
 %    ),
 %    goal_expr_is_trivial(GoalExpr).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Coverage information helper predicates.
 %
@@ -1288,22 +1300,22 @@ before_count_sum_before_count(sum_before_zero, before_zero).
 :- func after_coverage(int) = coverage_after.
 
 after_coverage(Count) =
-    ( Count = 0 ->
+    ( if Count = 0 then
         after_zero
-    ;
+    else
         after_known(Count)
     ).
 
 :- func before_coverage(int) = coverage_before.
 
 before_coverage(Count) =
-    ( Count = 0 ->
+    ( if Count = 0 then
         before_zero
-    ;
+    else
         before_known(Count)
     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 add_coverage_point_to_map(CoveragePoint, !SolnsMap, !BranchMap) :-
     CoveragePoint = coverage_point(_, GoalPath, CPType),
@@ -1315,5 +1327,6 @@ add_coverage_point_to_map(CoveragePoint, !SolnsMap, !BranchMap) :-
         map.det_insert(GoalPath, CoveragePoint, !BranchMap)
     ).
 
-%----------------------------------------------------------------------------%
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module coverage.
+%---------------------------------------------------------------------------%
