@@ -1,10 +1,11 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2001-2012 The University of Melbourne.
+% Copyright (C) 2013, 2015, 2017, 2020-2021 The Mercury team.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: mdprof_cgi.m.
 % Author of initial version: conway.
@@ -13,19 +14,19 @@
 % This file contains the CGI "script" that is executed by the web server
 % to handle web page requests implemented by the Mercury deep profiler server.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module mdprof_cgi.
 :- interface.
 
 :- import_module io.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred main(io::di, io::uo) is cc_multi.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -47,7 +48,7 @@
 :- import_module require.
 :- import_module string.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % The web server should always set QUERY_STRING. It may also pass its contents
 % as arguments, but if any characters special to the shell occur in the query,
@@ -56,12 +57,14 @@
 % from the command line for debugging.
 
 main(!IO) :-
-    write_html_header(!IO),
+    io.stdin_stream(StdIn, !IO),
+    io.stdout_stream(StdOut, !IO),
+    write_html_header(StdOut, !IO),
     io.get_environment_var("QUERY_STRING", MaybeQueryString, !IO),
     (
         MaybeQueryString = yes(QueryString0),
-        getopt.process_options(
-            option_ops_multi(short, long, defaults), [], _, MaybeOptions),
+        OptionOps = option_ops_multi(short, long, defaults),
+        getopt.process_options(OptionOps, [], _, MaybeOptions),
         (
             MaybeOptions = ok(Options)
         ;
@@ -78,27 +81,28 @@ main(!IO) :-
                 MaybeCmd = no,
                 Cmd = default_command
             ),
-            process_query(Cmd, DeepFileName, MaybePrefs, Options, !IO)
+            process_query(StdOut, Cmd, DeepFileName, MaybePrefs, Options, !IO)
         ;
             MaybeDeepQuery = no,
             io.set_exit_status(1, !IO),
             % Give the simplest URL in the error message.
-            io.write_string("Bad URL; expected filename \n", !IO)
+            io.write_string(StdOut, "Bad URL; expected filename\n", !IO)
         )
     ;
         MaybeQueryString = no,
-        process_command_line(!IO)
+        process_command_line(StdIn, StdOut, !IO)
     ).
 
-:- pred process_command_line(io::di, io::uo) is cc_multi.
+:- pred process_command_line(io.text_input_stream::in,
+    io.text_output_stream::in, io::di, io::uo) is cc_multi.
 
-process_command_line(!IO) :-
+process_command_line(InputStream, OutputStream, !IO) :-
     io.progname_base(mdprof_cgi_progname, ProgName, !IO),
     io.command_line_arguments(Args0, !IO),
     trace [compiletime(flag("debug-args")), io(!DIO)] (
-        io.write_string("command line: ", !DIO),
-        io.write_list(Args0, " ", write_bracketed_string, !DIO),
-        io.nl(!DIO)
+        BracketedArgs0 = list.map(bracket_string, Args0),
+        io.format(OutputStream, "command line: %s\n",
+            [s(string.join_list(" ", BracketedArgs0))], !DIO)
     ),
     getopt.process_options(option_ops_multi(short, long, defaults),
         Args0, Args, MaybeOptions),
@@ -111,109 +115,115 @@ process_command_line(!IO) :-
         lookup_bool_option(Options, decode_prefs, DecodePrefs),
         (
             Help = yes,
-            write_help_message(ProgName, !IO)
+            write_help_message(OutputStream, ProgName, !IO)
         ;
             Help = no
         ),
         (
             Version = yes,
-            write_version_message(ProgName, !IO)
+            write_version_message(OutputStream, ProgName, !IO)
         ;
             Version = no
         ),
-        (
+        ( if
             Decode = no,
             DecodeCmd = no,
             DecodePrefs = no
-        ->
+        then
             true
-        ;
-            decode_input_lines(Decode, DecodeCmd, DecodePrefs, !IO)
+        else
+            decode_input_lines(InputStream, OutputStream,
+                Decode, DecodeCmd, DecodePrefs, !IO)
         ),
-        (
+        ( if
             Help = no,
             Version = no,
             Decode = no,
             DecodeCmd = no,
             DecodePrefs = no
-        ->
-            process_args(ProgName, Args, Options, !IO)
-        ;
+        then
+            process_args(OutputStream, ProgName, Args, Options, !IO)
+        else
             true
         )
     ;
-        MaybeOptions = error(Msg),
+        MaybeOptions = error(Error),
+        Msg = option_error_to_string(Error),
         io.set_exit_status(1, !IO),
-        io.format("%s: error parsing options: %s\n",
+        io.format(OutputStream, "%s: error parsing options: %s\n",
             [s(ProgName), s(Msg)], !IO)
     ).
 
-:- pred decode_input_lines(bool::in, bool::in, bool::in, io::di, io::uo)
-    is det.
+:- pred decode_input_lines(io.text_input_stream::in, io.text_output_stream::in,
+    bool::in, bool::in, bool::in, io::di, io::uo) is det.
 
-decode_input_lines(Decode, DecodeCmd, DecodePrefs, !IO) :-
-    io.read_line_as_string(LineResult, !IO),
+decode_input_lines(InputStream, OutputStream, Decode, DecodeCmd, DecodePrefs,
+        !IO) :-
+    io.read_line_as_string(InputStream, LineResult, !IO),
     (
         LineResult = ok(LineStr),
         (
             Decode = no
         ;
             Decode = yes,
-            io.write_string("considering as query string:\n", !IO),
+            io.write_string(OutputStream,
+                "considering as query string:\n", !IO),
             string_to_maybe_query(LineStr) = MaybeQuery,
             (
                 MaybeQuery = yes(deep_query(MaybeCmd, DeepFileName,
                     MaybePrefs)),
-                io.write_string("Maybe Command:\n", !IO),
-                io.write(MaybeCmd, !IO),
-                io.nl(!IO),
-                io.format("Deep File Name: %s\n", [s(DeepFileName)], !IO),
+                io.write_string(OutputStream, "Maybe Command:\n", !IO),
+                io.write_line(OutputStream, MaybeCmd, !IO),
+                io.format(OutputStream, "Deep File Name: %s\n",
+                    [s(DeepFileName)], !IO),
                 % The preferences may fail to parse, in this case no
                 % preferences are assumed.
-                io.write_string("Maybe Preferences:\n", !IO),
-                io.write(MaybePrefs, !IO),
-                io.nl(!IO)
+                io.write_string(OutputStream, "Maybe Preferences:\n", !IO),
+                io.write_line(OutputStream, MaybePrefs, !IO)
             ;
                 MaybeQuery = no,
-                io.write_string("invalid query string: " ++
-                    "cannot split into components\n", !IO)
+                io.write_string(OutputStream,
+                    "invalid query string: cannot split into components\n",
+                    !IO)
             )
         ),
         (
             DecodeCmd = no
         ;
             DecodeCmd = yes,
-            io.write_string("considering as cmd string:\n", !IO),
+            io.write_string(OutputStream, "considering as cmd string:\n", !IO),
             MaybeCmd1 = string_to_maybe_cmd(LineStr),
             (
                 MaybeCmd1 = no,
-                io.format("invalid command string %s\n", [s(LineStr)], !IO)
+                io.format(OutputStream, "invalid command string %s\n",
+                    [s(LineStr)], !IO)
             ;
                 MaybeCmd1 = yes(Cmd),
-                io.write(Cmd, !IO),
-                io.nl(!IO)
+                io.write_line(OutputStream, Cmd, !IO)
             )
         ),
         (
             DecodePrefs = no
         ;
             DecodePrefs = yes,
-            io.write_string("considering as preference string:\n", !IO),
+            io.write_string(OutputStream,
+                "considering as preference string:\n", !IO),
             MaybePref = string_to_maybe_pref(LineStr),
             (
                 MaybePref = no,
-                io.format("invalid preferences string %s\n", [s(LineStr)], !IO)
+                io.format(OutputStream,
+                    "invalid preferences string %s\n", [s(LineStr)], !IO)
             ;
                 MaybePref = yes(Pref),
-                io.write(Pref, !IO),
-                io.nl(!IO)
+                io.write_line(OutputStream, Pref, !IO)
             )
         ),
-        decode_input_lines(Decode, DecodeCmd, DecodePrefs, !IO)
+        decode_input_lines(InputStream, OutputStream,
+            Decode, DecodeCmd, DecodePrefs, !IO)
     ;
         LineResult = error(Error),
         io.error_message(Error, Msg),
-        io.format("%s\n", [s(Msg)], !IO)
+        io.format(OutputStream, "%s\n", [s(Msg)], !IO)
     ;
         LineResult = eof
     ).
@@ -222,74 +232,74 @@ decode_input_lines(Decode, DecodeCmd, DecodePrefs, !IO) :-
 
 mdprof_cgi_progname = "mdprof_cgi".
 
-:- pred write_version_message(string::in, io::di, io::uo) is det.
+:- pred write_version_message(io.text_output_stream::in, string::in,
+    io::di, io::uo) is det.
 
-write_version_message(ProgName, !IO) :-
-    library.version(Version),
-    io.write_string(ProgName, !IO),
-    io.write_string(": Mercury deep profiler", !IO),
-    io.nl(!IO),
-    io.write_string(Version, !IO),
-    io.nl(!IO).
+write_version_message(OutputStream, ProgName, !IO) :-
+    library.version(Version, Fullarch),
+    io.format(OutputStream, "%s: Mercury deep profiler\n", [s(ProgName)], !IO),
+    io.format(OutputStream, "version: %s, on %s.\n",
+        [s(Version), s(Fullarch)], !IO).
 
-:- pred write_help_message(string::in, io::di, io::uo) is det.
+:- pred write_help_message(io.text_output_stream::in, string::in,
+    io::di, io::uo) is det.
 
-write_help_message(ProgName, !IO) :-
-    % The options are deliberately not documented; they change
+write_help_message(OutputStream, ProgName, !IO) :-
+    % The options are deliberately not documented; they can change
     % quite rapidly, based on the debugging needs of the moment.
     % The optional filename argument is also for implementors only.
-    io.format("Usage: %s\n", [s(ProgName)], !IO),
-    io.format("This program doesn't expect any arguments;\n", [], !IO),
-    io.format("instead it decides what to do based on the\n", [], !IO),
-    io.format("QUERY_STRING environment variable.\n", [], !IO).
+    io.format(OutputStream, "Usage: %s\n", [s(ProgName)], !IO),
+    io.write_strings(OutputStream,
+        ["This program doesn't expect any arguments;\n",
+        "instead it decides what to do based on the\n",
+        "QUERY_STRING environment variable.\n"], !IO).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- pred process_args(string::in, list(string)::in, option_table::in,
-    io::di, io::uo) is cc_multi.
+:- pred process_args(io.text_output_stream::in, string::in, list(string)::in,
+    option_table::in, io::di, io::uo) is cc_multi.
 
-process_args(ProgName, Args, Options, !IO) :-
-    ( Args = [DeepFileName] ->
+process_args(OutputStream, ProgName, Args, Options, !IO) :-
+    ( if Args = [DeepFileName] then
         % Although this mode of usage is not intended for production use,
         % allowing the filename and a limited range of commands to be supplied
         % on the command line makes debugging very much easier.
-        process_query(default_cmd(Options), DeepFileName, no, Options, !IO)
-    ;
+        process_query(OutputStream, default_cmd(Options), DeepFileName,
+            no, Options, !IO)
+    else
         io.set_exit_status(1, !IO),
-        write_help_message(ProgName, !IO),
+        write_help_message(OutputStream, ProgName, !IO),
         trace [compiletime(flag("debug-args")), io(!DIO)] (
-            io.write_string("processed args: ", !DIO),
-            io.write_list(Args, " ", write_bracketed_string, !DIO)
+            BracketedArgs = list.map(bracket_string, Args),
+            io.format(OutputStream, "processed args: %s\n",
+                [s(string.join_list(" ", BracketedArgs))], !DIO)
         )
     ).
 
 % This predicate is for debugging the command line given to mdprof_cgi by the
 % web server, should that be necessary.
 
-:- pred write_bracketed_string(string::in, io::di, io::uo) is det.
+:- func bracket_string(string) = string.
 
-write_bracketed_string(S, !IO) :-
-    io.write_string("<", !IO),
-    io.write_string(S, !IO),
-    io.write_string(">", !IO).
+bracket_string(S) = "<" ++ S ++ ">".
 
-:- pred write_html_header(io::di, io::uo) is det.
+:- pred write_html_header(io.text_output_stream::in, io::di, io::uo) is det.
 
-write_html_header(!IO) :-
-    io.write_string(html_header_text, !IO),
-    io.flush_output(!IO).
+write_html_header(OutputStream, !IO) :-
+    io.write_string(OutputStream, html_header_text, !IO),
+    io.flush_output(OutputStream, !IO).
 
 :- func html_header_text = string.
 
 html_header_text = "Content-type: text/html\n\n".
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- pred process_query(cmd::in, string::in, maybe(preferences)::in,
-    option_table::in, io::di, io::uo) is cc_multi.
+:- pred process_query(io.text_output_stream::in, cmd::in, string::in,
+    maybe(preferences)::in, option_table::in, io::di, io::uo) is cc_multi.
 
-process_query(Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
-    ( Cmd0 = deep_cmd_restart ->
+process_query(OutputStream, Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
+    ( if Cmd0 = deep_cmd_restart then
         % This process got started because there was no server, and this
         % process will become the new server, so the user just got the freshly
         % started server they asked for. There is no point in starting it
@@ -297,7 +307,7 @@ process_query(Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
         % deep_cmd_restart, expecting it to be filtered out by its usual caller
         % server_loop. To avoid the exception, we have to filter it out too.
         Cmd = deep_cmd_menu
-    ;
+    else
         Cmd = Cmd0
     ),
     (
@@ -307,14 +317,16 @@ process_query(Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
         MaybePref = no,
         PrefInd = default_pref
     ),
-    ( string.remove_suffix(DeepFileName0, ".localhost", DeepFileNamePrime) ->
+    ( if
+        string.remove_suffix(DeepFileName0, ".localhost", DeepFileNamePrime)
+    then
         DeepFileName = DeepFileNamePrime,
         map.det_update(localhost, bool(yes), Options0, Options)
-    ;
+    else
         DeepFileName = DeepFileName0,
         Options = Options0
     ),
-    ( remove_suffix(DeepFileName, ".data", _BaseFileName) ->
+    ( if string.remove_suffix(DeepFileName, ".data", _BaseFileName) then
         ToServerPipe = to_server_pipe_name(DeepFileName),
         FromServerPipe = from_server_pipe_name(DeepFileName),
         StartupFile = server_startup_name(DeepFileName),
@@ -332,25 +344,26 @@ process_query(Cmd0, DeepFileName0, MaybePref, Options0, !IO) :-
             setup_signals(MutexFile, want_dir, want_prefix, !IO)
         ),
         check_for_existing_fifos(ToServerPipe, FromServerPipe, FifoCount, !IO),
-        ( FifoCount = 0 ->
-            handle_query_from_new_server(Cmd, PrefInd, DeepFileName,
-                ToServerPipe, FromServerPipe, StartupFile, MutexFile, WantFile,
-                Options, !IO)
-        ; FifoCount = 2 ->
+        ( if FifoCount = 0 then
+            handle_query_from_new_server(OutputStream, Cmd, PrefInd,
+                DeepFileName, ToServerPipe, FromServerPipe,
+                StartupFile, MutexFile, WantFile, Options, !IO)
+        else if FifoCount = 2 then
             handle_query_from_existing_server(Cmd, PrefInd,
                 ToServerPipe, FromServerPipe, MutexFile, WantFile, Options,
                 !IO)
-        ;
+        else
             release_lock(Debug, MutexFile, !IO),
             remove_want_file(WantFile, !IO),
             io.set_exit_status(1, !IO),
-            io.write_string("mdprof internal error: bad fifo count", !IO)
+            io.write_string(OutputStream,
+                "mdprof internal error: bad fifo count\n", !IO)
         )
-    ;
+    else
         io.set_exit_status(1, !IO),
-        io.format("<h3> Invalid file name %s.<h3>\n\n",
+        io.format(OutputStream, "<h3> Invalid file name %s.<h3>\n\n",
             [s(DeepFileName)], !IO),
-        io.write_string(
+        io.write_string(OutputStream,
             "Deep profiling data files must have a .data suffix, " ++
             "to allow the deep profiler to locate any related files.\n", !IO)
     ).
@@ -413,12 +426,14 @@ handle_query_from_existing_server(Cmd, PrefInd, ToServerPipe, FromServerPipe,
     % Handle the given query and then become the new server. Delete the mutex
     % and want files when we get out of the critical region.
     %
-:- pred handle_query_from_new_server(cmd::in, preferences_indication::in,
-    string::in, string::in, string::in, string::in, string::in, string::in,
+:- pred handle_query_from_new_server(io.text_output_stream::in, cmd::in,
+    preferences_indication::in, string::in,
+    string::in, string::in, string::in, string::in, string::in,
     option_table::in, io::di, io::uo) is cc_multi.
 
-handle_query_from_new_server(Cmd, PrefInd, FileName, ToServerPipe,
-        FromServerPipe, StartupFile, MutexFile, WantFile, Options, !IO) :-
+handle_query_from_new_server(OutputStream, Cmd, PrefInd, FileName,
+        ToServerPipe, FromServerPipe, StartupFile, MutexFile, WantFile,
+        Options, !IO) :-
     lookup_bool_option(Options, localhost, LocalHost),
     (
         LocalHost = no,
@@ -452,7 +467,7 @@ handle_query_from_new_server(Cmd, PrefInd, FileName, ToServerPipe,
     (
         StartupResult = ok(Deep),
         Pref = solidify_preference(Deep, PrefInd),
-        try_exec(Cmd, Pref, Deep, HTML, !IO),
+        try_exec(Cmd, Pref, Deep, HTML),
         (
             MaybeStartupStream = yes(StartupStream1),
             io.format(StartupStream1, "query 0 output:\n%s\n", [s(HTML)], !IO),
@@ -467,14 +482,14 @@ handle_query_from_new_server(Cmd, PrefInd, FileName, ToServerPipe,
             % --no-server-process should be specified only during debugging.
             release_lock(Debug, MutexFile, !IO),
             remove_want_file(WantFile, !IO),
-            io.write_string(HTML, !IO)
+            io.write_string(OutputStream, HTML, !IO)
         ;
             ServerProcess = yes,
             make_pipes(FileName, Success, !IO),
             (
                 Success = yes,
-                io.write_string(HTML, !IO),
-                io.flush_output(!IO),
+                io.write_string(OutputStream, HTML, !IO),
+                io.flush_output(OutputStream, !IO),
                 start_server(Options, ToServerPipe, FromServerPipe,
                     MaybeStartupStream, MutexFile, WantFile, Deep, !IO)
             ;
@@ -482,7 +497,7 @@ handle_query_from_new_server(Cmd, PrefInd, FileName, ToServerPipe,
                 release_lock(Debug, MutexFile, !IO),
                 remove_want_file(WantFile, !IO),
                 io.set_exit_status(1, !IO),
-                io.write_string("could not make pipes\n", !IO)
+                io.write_string(OutputStream, "could not make pipes\n", !IO)
             )
         )
     ;
@@ -490,7 +505,7 @@ handle_query_from_new_server(Cmd, PrefInd, FileName, ToServerPipe,
         release_lock(Debug, MutexFile, !IO),
         remove_want_file(WantFile, !IO),
         io.set_exit_status(1, !IO),
-        io.format("%s\n", [s(Error)], !IO)
+        io.format(OutputStream, "%s\n", [s(Error)], !IO)
     ).
 
     % Become the new server. Delete the mutex and want files when we get out
@@ -603,7 +618,7 @@ server_loop(ToServerPipe, FromServerPipe, TimeOut0, MaybeStartupStream,
     ),
     CmdPref0 = cmd_pref(Cmd0, PrefInd0),
 
-    ( Cmd0 = deep_cmd_restart ->
+    ( if Cmd0 = deep_cmd_restart then
         read_and_startup_default_deep_options(Deep0 ^ server_name_port,
             Deep0 ^ script_name, Deep0 ^ data_file_name, Canonical,
             MaybeStartupStream, [], MaybeDeep, !IO),
@@ -617,7 +632,7 @@ server_loop(ToServerPipe, FromServerPipe, TimeOut0, MaybeStartupStream,
             Deep = Deep0,
             Cmd = deep_cmd_quit
         )
-    ;
+    else
         Deep = Deep0,
         MaybeMsg = no,
         Cmd = Cmd0
@@ -627,7 +642,7 @@ server_loop(ToServerPipe, FromServerPipe, TimeOut0, MaybeStartupStream,
         MaybeMsg = yes(HTML)
     ;
         MaybeMsg = no,
-        try_exec(Cmd, Pref0, Deep, HTML, !IO)
+        try_exec(Cmd, Pref0, Deep, HTML)
     ),
 
     ResponseFileName = response_file_name(Deep0 ^ data_file_name, QueryNum),
@@ -652,21 +667,21 @@ server_loop(ToServerPipe, FromServerPipe, TimeOut0, MaybeStartupStream,
         MaybeStartupStream = no
     ),
 
-    ( Cmd = deep_cmd_quit ->
+    ( if Cmd = deep_cmd_quit then
         % The lack of a recursive call here shuts down the server.
         %
         % This deletes all the files created by the process, including
         % WantFile and MutexFile, with MutexFile being deleted last.
         delete_cleanup_files(!IO)
-    ; Cmd = deep_cmd_timeout(TimeOut) ->
+    else if Cmd = deep_cmd_timeout(TimeOut) then
         server_loop(ToServerPipe, FromServerPipe, TimeOut, MaybeStartupStream,
             Debug, Canonical, QueryNum, Deep, !IO)
-    ;
+    else
         server_loop(ToServerPipe, FromServerPipe, TimeOut0, MaybeStartupStream,
             Debug, Canonical, QueryNum, Deep, !IO)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred make_pipes(string::in, bool::out, io::di, io::uo) is det.
 
@@ -677,14 +692,14 @@ make_pipes(FileName, Success, !IO) :-
     MakeFromServerPipeCmd = make_pipe_cmd(FromServerPipe),
     io.call_system(MakeToServerPipeCmd, ToServerRes, !IO),
     io.call_system(MakeFromServerPipeCmd, FromServerRes, !IO),
-    (
+    ( if
         ToServerRes = ok(0),
         FromServerRes = ok(0)
-    ->
+    then
         register_file_for_cleanup(ToServerPipe, !IO),
         register_file_for_cleanup(FromServerPipe, !IO),
         Success = yes
-    ;
+    else
         % In case one of the pipes *was* created. We ignore the return values
         % because at least one of these calls *will* fail (since we did not
         % create both pipes), and if we can't remove a named pipe we did
@@ -695,7 +710,7 @@ make_pipes(FileName, Success, !IO) :-
         Success = no
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_decl("C", "
 #ifdef  MR_DEEP_PROFILER_ENABLED
@@ -712,7 +727,7 @@ make_pipes(FileName, Success, !IO) :-
 
 :- pragma foreign_proc("C",
     check_for_existing_fifos(Fifo1::in, Fifo2::in, FifoCount::out,
-        S0::di, S::uo),
+        _IO0::di, _IO::uo),
     [will_not_call_mercury, promise_pure, tabled_for_io],
 "
 #ifdef  MR_DEEP_PROFILER_ENABLED
@@ -728,8 +743,6 @@ make_pipes(FileName, Success, !IO) :-
     if ((status == 0) && (S_ISFIFO(statbuf.st_mode))) {
         FifoCount++;
     }
-
-    S = S0;
 #else
     MR_fatal_error(""deep profiling not enabled"");
 #endif
@@ -748,11 +761,11 @@ make_pipes(FileName, Success, !IO) :-
 
 detach_process(Result, !IO) :-
     raw_detach_process(ResCode, !IO),
-    ( ResCode < 0 ->
+    ( if ResCode < 0 then
         Result = fork_failed
-    ; ResCode > 0 ->
+    else if ResCode > 0 then
         Result = in_parent
-    ;
+    else
         Result = in_child(child_has_parent)
     ).
 
@@ -770,7 +783,7 @@ detach_process(Result, !IO) :-
 :- pred raw_detach_process(int::out, io::di, io::uo) is cc_multi.
 
 :- pragma foreign_proc("C",
-    raw_detach_process(ResCode::out, S0::di, S::uo),
+    raw_detach_process(ResCode::out, _IO0::di, _IO::uo),
     [will_not_call_mercury, promise_pure],
 "{
 #ifdef  MR_DEEP_PROFILER_ENABLED
@@ -796,14 +809,12 @@ detach_process(Result, !IO) :-
 #endif
         ResCode = 0;
     }
-
-    S = S0;
 #else
     MR_fatal_error(""deep profiling not enabled"");
 #endif
 }").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type option
     --->    canonical_clique
@@ -893,20 +904,20 @@ default_cmd(Options) = Cmd :-
     lookup_bool_option(Options, modules, Modules),
     lookup_int_option(Options, clique, CliqueNum),
     lookup_int_option(Options, proc, ProcProcNum),
-    ( Root = yes ->
+    ( if Root = yes then
         Cmd = deep_cmd_root(no)
-    ; Modules = yes ->
+    else if Modules = yes then
         Cmd = deep_cmd_program_modules
-    ; CliqueNum > 0 ->
+    else if CliqueNum > 0 then
         Cmd = deep_cmd_clique(clique_ptr(CliqueNum))
-    ; ProcProcNum > 0 ->
+    else if ProcProcNum > 0 then
         Cmd = deep_cmd_proc(proc_static_ptr(ProcProcNum))
-    ; Quit = yes ->
+    else if Quit = yes then
         Cmd = deep_cmd_quit
-    ;
+    else
         Cmd = deep_cmd_menu
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module mdprof_cgi.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
