@@ -1,21 +1,21 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2008, 2010-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: program_representation_utils.m.
 % Author: pbone.
 %
 % Utilities for working with the program representation structures in the
-% mdbcomp library.  This file is not part of the mdbcomp library, since it
-% contains routines only used by the deep profiling tools.  Code here should be
-% moved into the mdbcomp.program_representation.m module if it is to be used by
-% other tools.
+% mdbcomp library. This file is not part of the mdbcomp library, since it
+% contains routines only used by the deep profiling tools. Code here
+% should be moved into the mdbcomp.program_representation.m module
+% if it is to be used by other tools.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module program_representation_utils.
 :- interface.
@@ -29,7 +29,7 @@
 :- import_module set.
 :- import_module unit.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Ugly-print a module to a string representation. We return a cord of
     % strings is returned rather than a string, since this reduces the cost
@@ -68,7 +68,7 @@
     reverse_goal_path::in, goal_rep(T)::in, cord(string)::out) is det
     <= goal_annotation(GoalAnn).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- typeclass goal_annotation(T) where [
     % Print the goal annotation for inclusion by print_proc_to_strings
@@ -83,7 +83,7 @@
     %
 :- instance goal_annotation(unit).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Goal IDs are a more efficient way to identify goals than goal paths.
     %
@@ -92,7 +92,7 @@
 :- pred label_goals(goal_id::out, containing_goal_map::out,
     goal_rep(T)::in, goal_rep(goal_id)::out) is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Search a program representation for the given procedure and return its
     % procedure representation if found, otherwise fail.
@@ -100,7 +100,7 @@
 :- pred progrep_search_proc(prog_rep::in, string_proc_label::in, proc_rep::out)
     is semidet.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % A map of variables to instantiation states,  Like inst_map within the
     % compiler.
@@ -125,10 +125,27 @@
 :- pred inst_map_ground_vars(list(var_rep)::in, set(var_rep)::in, inst_map::in,
     inst_map::out, seen_duplicate_instantiation::out) is det.
 
+    % Retrieve the instantiatedness of a variable, and variables that it's
+    % binding depends upon from the instmap, if the variable is new ir_free is
+    % returned and the variables it depends upon is the empty set.
+    %
+:- pred inst_map_get(inst_map::in, var_rep::in, inst_rep::out,
+    set(var_rep)::out) is det.
+
+    % Retrieve all the variables this variable depends upon,  indirect
+    % dependencies are also returned.
+    %
+:- pred inst_map_get_var_deps(inst_map::in, var_rep::in, set(var_rep)::out)
+    is det.
+
+    % Merge two inst maps from different branches of execution.
+    %
+:- func merge_inst_map(inst_map, detism_rep, inst_map, detism_rep) = inst_map.
+
     % This type represents whether a traversal has seen more than one
-    % instantiation of a variable within a single branch.  If at the end of a
-    % traversal a duplicate instantiation has been seen we can either accept a
-    % pessimistic default or abort parallelisation of this particular
+    % instantiation of a variable within a single branch. If at the end of a
+    % traversal a duplicate instantiation has been seen, we can either
+    % accept a pessimistic default, or abort parallelisation of this particular
     % conjunction.
     %
 :- type seen_duplicate_instantiation
@@ -138,26 +155,9 @@
 :- func merge_seen_duplicate_instantiation(seen_duplicate_instantiation,
     seen_duplicate_instantiation) = seen_duplicate_instantiation.
 
-    % Retrieve the instantiatedness of a variable, and variables that it's
-    % binding depends upon from the instmap, if the variable is new ir_free is
-    % returned and the variables it depends upon is the empty set.
-    %
-:- pred inst_map_get(inst_map::in, var_rep::in, inst_rep::out,
-    set(var_rep)::out) is det.
+%---------------------------------------------------------------------------%
 
-    % Merge two inst maps from different branches of execution.
-    %
-:- func merge_inst_map(inst_map, detism_rep, inst_map, detism_rep) = inst_map.
-
-    % Retrieve all the variables this variable depends upon,  indirect
-    % dependencies are also returned.
-    %
-:- pred inst_map_get_var_deps(inst_map::in, var_rep::in, set(var_rep)::out)
-    is det.
-
-%----------------------------------------------------------------------------%
-
-    % A difference between too inst maps.  This lists the variables that are
+    % A difference between too inst maps. This lists the variables that are
     % instantiated by a particular goal.
     %
 :- type inst_map_delta.
@@ -167,7 +167,7 @@
 :- pred inst_map_delta_get_var_set(inst_map_delta::in, set(var_rep)::out)
     is det.
 
-    % The empty inst_map_delta.  Nothing is instantiated.
+    % The empty inst_map_delta. Nothing is instantiated.
     %
 :- pred empty_inst_map_delta(inst_map_delta::out) is det.
 :- func empty_inst_map_delta = inst_map_delta.
@@ -182,13 +182,13 @@
 :- pred calc_inst_map_delta(inst_map::in, inst_map::in, inst_map_delta::out)
     is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Retrieve a set of all the vars involved with this atomic goal.
     %
 :- pred atomic_goal_get_vars(atomic_goal_rep::in, set(var_rep)::out) is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type atomic_goal_is_call
     --->    atomic_goal_is_call(list(var_rep))
@@ -197,11 +197,12 @@
 :- pred atomic_goal_is_call(atomic_goal_rep::in, atomic_goal_is_call::out)
     is det.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module mdbcomp.prim_data.
+:- import_module mdbcomp.sym_name.
 
 :- import_module bool.
 :- import_module counter.
@@ -212,7 +213,7 @@
 :- import_module std_util.
 :- import_module string.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 print_module_to_strings(ModuleRep, Strings) :-
     ModuleRep = module_rep(ModuleName, _StringTable, OISUTypesProcs,
@@ -222,9 +223,9 @@ print_module_to_strings(ModuleRep, Strings) :-
         cord.empty, OISUStrs),
     map.foldl(accumulate_print_type_table_entries_to_strings, TypeTableMap,
         cord.empty, TypeTableStrs0),
-    ( cord.is_empty(TypeTableStrs0) ->
+    ( if cord.is_empty(TypeTableStrs0) then
         TypeTableStrs = TypeTableStrs0
-    ;
+    else
         TypeTableStrs = cord.singleton("\nType table:\n") ++
             TypeTableStrs0 ++ nl
     ),
@@ -233,7 +234,7 @@ print_module_to_strings(ModuleRep, Strings) :-
     Strings = cord.singleton(HeaderString) ++ OISUStrs ++ TypeTableStrs
         ++ ProcRepStrs.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred accumulate_print_oisu_type_procs_to_strings(oisu_type_procs::in,
     cord(string)::in, cord(string)::out) is det.
@@ -262,7 +263,7 @@ print_oisu_type_procs_to_strings(OISUTypeProcs, Str) :-
         ++ cord.cons("\nDestructor procs:\n",
             cord.cord_list_to_cord(DestructorNlCords)).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred accumulate_print_type_table_entries_to_strings(int::in,
     type_rep::in, cord(string)::in, cord(string)::out) is det.
@@ -299,6 +300,33 @@ type_rep_to_strings(TypeRep, Cord) :-
         (
             BuiltinTypeRep = builtin_type_int_rep,
             TypeNameStr = "int"
+        ;
+            BuiltinTypeRep = builtin_type_uint_rep,
+            TypeNameStr = "uint"
+        ;
+            BuiltinTypeRep = builtin_type_int8_rep,
+            TypeNameStr = "int8"
+        ;
+            BuiltinTypeRep = builtin_type_uint8_rep,
+            TypeNameStr = "uint8"
+        ;
+            BuiltinTypeRep = builtin_type_int16_rep,
+            TypeNameStr = "int16"
+        ;
+            BuiltinTypeRep = builtin_type_uint16_rep,
+            TypeNameStr = "uint16"
+        ;
+            BuiltinTypeRep = builtin_type_int32_rep,
+            TypeNameStr = "int32"
+        ;
+            BuiltinTypeRep = builtin_type_uint32_rep,
+            TypeNameStr = "uint32"
+        ;
+            BuiltinTypeRep = builtin_type_int64_rep,
+            TypeNameStr = "int64"
+        ;
+            BuiltinTypeRep = builtin_type_uint64_rep,
+            TypeNameStr = "uint64"
         ;
             BuiltinTypeRep = builtin_type_float_rep,
             TypeNameStr = "float"
@@ -371,7 +399,7 @@ arg_type_reps_to_strings(HeadTypeRep, [HeadTailTypeRep | TailTailTypeReps],
     arg_type_reps_to_strings(HeadTailTypeRep, TailTailTypeReps, TailCord),
     Cord = HeadCord ++ cord.singleton(", ") ++ TailCord.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Print a procedure to a string representation.
     %
@@ -423,12 +451,12 @@ print_proc_to_strings_2(Lookup, ProcRep, Strings) :-
 accumulate_var_type_table_entry_strings(VarNameTable, VarNum, TypeRep,
         !Strings) :-
     string.int_to_string(VarNum, VarNumStr),
-    (
+    ( if
         search_var_name(VarNameTable, VarNum, VarName),
         not VarName = ""
-    ->
+    then
         VarIdStrs = cord.from_list([VarName, " ", VarNumStr, " -> "])
-    ;
+    else
         VarIdStrs = cord.from_list(["unnamed_var ", VarNumStr, " -> "])
     ),
     type_rep_to_strings(TypeRep, TypeRepStrs),
@@ -455,7 +483,7 @@ print_proc_label_to_string(ProcLabel, String) :-
             [s(Name), s(TypeModule), s(TypeName), i(Arity), i(Mode)], String)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 print_goal_to_strings(Info, Indent, RevGoalPath, GoalRep, Strings) :-
     GoalRep = goal_rep(GoalExprRep, DetismRep, AnnotationKey),
@@ -526,10 +554,10 @@ print_goal_to_strings(Info, Indent, RevGoalPath, GoalRep, Strings) :-
         ExprString = indent(Indent) ++ ExprString0
     ),
 
-    ( GoalExprRep = conj_rep(_) ->
+    ( if GoalExprRep = conj_rep(_) then
         LinePrefix = indent(Indent) ++ singleton("% conjunction: "),
         ExtraLineForConjunctions = nl
-    ;
+    else
         LinePrefix = indent(Indent) ++ singleton("% "),
         ExtraLineForConjunctions = empty
     ),
@@ -539,18 +567,18 @@ print_goal_to_strings(Info, Indent, RevGoalPath, GoalRep, Strings) :-
     GoalAnnotation = LookupAnnotation(AnnotationKey),
     print_goal_annotation_to_strings(VarTable, GoalAnnotation,
         GoalAnnotationLines0),
-    ( is_empty(GoalAnnotationLines0) ->
+    ( if is_empty(GoalAnnotationLines0) then
         GoalAnnotationLines = empty
-    ;
+    else
         GoalAnnotationLines1 = map((func(Line) = LinePrefix ++ Line ++ nl),
             GoalAnnotationLines0),
         GoalAnnotationLines = foldr(++, GoalAnnotationLines1, empty)
     ),
 
     GoalPathString0 = rev_goal_path_to_string(RevGoalPath),
-    ( GoalPathString0 = "" ->
+    ( if GoalPathString0 = "" then
         GoalPathString = "root goal"
-    ;
+    else
         GoalPathString = GoalPathString0
     ),
     GoalPathLine = LinePrefix ++ cord.singleton(GoalPathString) ++ nl,
@@ -661,7 +689,7 @@ print_cons_id_and_arity_to_strings(Indent, ConsIdArityRep, Strings) :-
     string.format("%% case %s/%d\n", [s(ConsIdRep), i(Arity)], String),
     Strings = cord.snoc(indent(Indent), String).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred print_atomic_goal_to_strings(var_name_table::in, atomic_goal_rep::in,
     cord(string)::out) is det.
@@ -752,7 +780,7 @@ print_atomic_goal_to_strings(VarTable, AtomicGoalRep, Strings) :-
     ),
     Strings = Strings0 ++ nl.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred print_args_to_strings(pred(var_name_table, T, string), var_name_table,
     list(T), cord(string)).
@@ -804,12 +832,14 @@ inst_rep_to_string(ir_free_rep, "free").
 inst_rep_to_string(ir_ground_rep, "ground").
 inst_rep_to_string(ir_other_rep, "other").
 
+%---------------------------------------------------------------------------%
+
 :- func indent(int) = cord(string).
 
 indent(N) =
-    ( N =< 0 ->
+    ( if N =< 0 then
         cord.empty
-    ;
+    else
         cord.singleton("  ") ++ indent(N - 1)
     ).
 
@@ -817,6 +847,16 @@ indent(N) =
 
 nl_indent(N) = nl ++ indent(N).
 
+:- func nl = cord(string).
+
+nl = cord.singleton("\n").
+
+:- func add_nl(string) = cord(string).
+
+add_nl(Str) = cord.from_list([Str, "\n"]).
+
+%---------------------------------------------------------------------------%
+
 :- pred detism_to_string(detism_rep::in, cord(string)::out) is det.
 
 detism_to_string(Detism, DetismStrCord) :-
@@ -847,17 +887,7 @@ detism_to_string(Detism, DetismStrCord) :-
     ),
     DetismStrCord = cord.singleton(DetismStr).
 
-%----------------------------------------------------------------------------%
-
-:- func nl = cord(string).
-
-nl = cord.singleton("\n").
-
-:- func add_nl(string) = cord(string).
-
-add_nl(Str) = cord.from_list([Str, "\n"]).
-
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- instance goal_annotation(unit) where [
     pred(print_goal_annotation_to_strings/3) is print_unit_to_strings
@@ -868,33 +898,7 @@ add_nl(Str) = cord.from_list([Str, "\n"]).
 
 print_unit_to_strings(_, _, cord.empty).
 
-%----------------------------------------------------------------------------%
-
-progrep_search_proc(ProgRep, ProcLabel, ProcRep) :-
-    ( ProcLabel = str_ordinary_proc_label(_, _DeclModule, DefModule, _, _, _)
-    ; ProcLabel = str_special_proc_label(_, _DeclModule, DefModule, _, _, _)
-    ),
-    progrep_search_module(ProgRep, DefModule, ModuleRep),
-    modulerep_search_proc(ModuleRep, ProcLabel, ProcRep).
-
-    % Search for a module within a program representation.
-    %
-:- pred progrep_search_module(prog_rep::in, string::in, module_rep::out)
-    is semidet.
-
-progrep_search_module(ProgRep, ModuleName, ModuleRep) :-
-   ProgRep = prog_rep(ModuleReps),
-   map.search(ModuleReps, ModuleName, ModuleRep).
-
-    % Search for a procedure within a module representation.
-    %
-:- pred modulerep_search_proc(module_rep::in, string_proc_label::in,
-    proc_rep::out) is semidet.
-
-modulerep_search_proc(ModuleRep, ProcLabel, ProcRep) :-
-    map.search(ModuleRep ^ mr_procs, ProcLabel, ProcRep).
-
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Label goals with IDs.
 %
@@ -953,11 +957,12 @@ label_goal(ContainingGoal, !Goal, !Counter, !Map) :-
     ),
     !:Goal = goal_rep(GoalExpr, Detism, GoalId).
 
-:- pred label_goal_wrapper(func(int) = goal_path_step, goal_id,
-    goal_rep(T), goal_rep(goal_id), int, int, counter, counter,
-    map(goal_id, containing_goal), map(goal_id, containing_goal)) is det.
-:- mode label_goal_wrapper(func(in) = out is det, in,
-    in, out, in, out, in, out, in, out) is det.
+:- pred label_goal_wrapper(
+    (func(int) = goal_path_step)::in(func(in) = out is det), goal_id::in,
+    goal_rep(T)::in, goal_rep(goal_id)::out, int::in, int::out,
+    counter::in, counter::out,
+    map(goal_id, containing_goal)::in, map(goal_id, containing_goal)::out)
+    is det.
 
 label_goal_wrapper(MakePathStep, ParentGoalId, !Goal, !GoalNum, !Counter,
         !Map) :-
@@ -976,7 +981,33 @@ label_case(ParentGoalId, !Case, !CaseNum, !Counter, !Map) :-
         ParentGoalId, Goal0, Goal, !CaseNum, !Counter, !Map),
     !:Case = case_rep(MainConsId, OtherConsIds, Goal).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
+progrep_search_proc(ProgRep, ProcLabel, ProcRep) :-
+    ( ProcLabel = str_ordinary_proc_label(_, _DeclModule, DefModule, _, _, _)
+    ; ProcLabel = str_special_proc_label(_, _DeclModule, DefModule, _, _, _)
+    ),
+    progrep_search_module(ProgRep, DefModule, ModuleRep),
+    modulerep_search_proc(ModuleRep, ProcLabel, ProcRep).
+
+    % Search for a module within a program representation.
+    %
+:- pred progrep_search_module(prog_rep::in, string::in, module_rep::out)
+    is semidet.
+
+progrep_search_module(ProgRep, ModuleName, ModuleRep) :-
+    ProgRep = prog_rep(ModuleReps),
+    map.search(ModuleReps, ModuleName, ModuleRep).
+
+    % Search for a procedure within a module representation.
+    %
+:- pred modulerep_search_proc(module_rep::in, string_proc_label::in,
+    proc_rep::out) is semidet.
+
+modulerep_search_proc(ModuleRep, ProcLabel, ProcRep) :-
+    map.search(ModuleRep ^ mr_procs, ProcLabel, ProcRep).
+
+%---------------------------------------------------------------------------%
 
 :- type inst_map
     --->    inst_map(
@@ -1010,6 +1041,75 @@ add_inst_mapping(VarRep, InstRep, DepVars, InstMap0, InstMap) :-
     map.det_insert(VarRep, DepVars, VarToDepVars0, VarToDepVars),
     InstMap = inst_map(VarToInst, VarToDepVars).
 
+%---------------------------------------------------------------------------%
+
+inst_map_ground_vars(Vars, DepVars, !InstMap, SeenDuplicateInstantiation) :-
+    list.foldl2(inst_map_ground_var(DepVars), Vars, !InstMap,
+        have_not_seen_duplicate_instantiation, SeenDuplicateInstantiation).
+
+:- pred inst_map_ground_var(set(var_rep)::in, var_rep::in,
+    inst_map::in, inst_map::out, seen_duplicate_instantiation::in,
+    seen_duplicate_instantiation::out) is det.
+
+inst_map_ground_var(DepVars0, Var, InstMap0, InstMap,
+        !SeenDuplicateInstantiation) :-
+    InstMap0 = inst_map(VarToInst0, VarToDepVars0),
+    ( if map.search(VarToInst0, Var, InstPrime) then
+        Inst = InstPrime
+    else
+        Inst = ir_free_rep
+    ),
+    (
+        Inst = ir_free_rep,
+        NewInst = ir_ground_rep,
+        DepVars = DepVars0
+    ;
+        ( Inst = ir_ground_rep
+        ; Inst = ir_other_rep
+        ),
+        NewInst = ir_other_rep,
+        map.lookup(VarToDepVars0, Var, DepVarsFromIM),
+        DepVars = set.union(DepVars0,  DepVarsFromIM),
+        !:SeenDuplicateInstantiation = seen_duplicate_instantiation
+    ),
+    map.set(Var, NewInst, VarToInst0, VarToInst),
+    map.set(Var, DepVars, VarToDepVars0, VarToDepVars),
+    InstMap = inst_map(VarToInst, VarToDepVars).
+
+%---------------------------------------------------------------------------%
+
+inst_map_get(inst_map(VarToInst, VarToDepVars), Var, Inst, DepVars) :-
+    ( if map.search(VarToInst, Var, InstPrime) then
+        Inst = InstPrime,
+        map.lookup(VarToDepVars, Var, DepVars)
+    else
+        Inst = ir_free_rep,
+        DepVars = set.init
+    ).
+
+inst_map_get_var_deps(inst_map(_, VarToDepVars), VarRep, DepVars) :-
+    inst_map_get_var_deps_2(VarToDepVars, VarRep, set.init, DepVars).
+
+:- pred inst_map_get_var_deps_2(map(var_rep, set(var_rep))::in, var_rep::in,
+    set(var_rep)::in, set(var_rep)::out) is det.
+
+inst_map_get_var_deps_2(VarToDepVars, VarRep, !Set) :-
+    ( if set.contains(!.Set, VarRep) then
+        true
+        % This variable has already been visited. Stopping here prevents
+        % following any cycles in the graph (which should be impossible anyway)
+        % or following the same path twice if there are diamonds in the graph.
+    else
+        ( if map.search(VarToDepVars, VarRep, DepVars) then
+            !:Set = set.union(!.Set, DepVars),
+            set.fold(inst_map_get_var_deps_2(VarToDepVars), DepVars, !Set)
+        else
+            true
+        )
+    ).
+
+%---------------------------------------------------------------------------%
+
 merge_inst_map(InstMapA, DetismA, InstMapB, DetismB) = InstMap :-
     (
         ( DetismA = erroneous_rep
@@ -1057,70 +1157,19 @@ inst_intersect(ir_other_rep,    ir_free_rep,    ir_other_rep).
 inst_intersect(ir_other_rep,    ir_ground_rep,  ir_other_rep).
 inst_intersect(ir_other_rep,    ir_other_rep,   ir_other_rep).
 
-inst_map_ground_vars(Vars, DepVars, !InstMap, SeenDuplicateInstantiation) :-
-    list.foldl2(inst_map_ground_var(DepVars), Vars, !InstMap,
-        have_not_seen_duplicate_instantiation, SeenDuplicateInstantiation).
+%---------------------------------------------------------------------------%
 
-:- pred inst_map_ground_var(set(var_rep)::in, var_rep::in,
-    inst_map::in, inst_map::out, seen_duplicate_instantiation::in,
-    seen_duplicate_instantiation::out) is det.
-
-inst_map_ground_var(DepVars0, Var, InstMap0, InstMap,
-        !SeenDuplicateInstantiation) :-
-    InstMap0 = inst_map(VarToInst0, VarToDepVars0),
-    ( map.search(VarToInst0, Var, InstPrime) ->
-        Inst = InstPrime
-    ;
-        Inst = ir_free_rep
-    ),
-    (
-        Inst = ir_free_rep,
-        NewInst = ir_ground_rep,
-        DepVars = DepVars0
-    ;
-        ( Inst = ir_ground_rep
-        ; Inst = ir_other_rep
-        ),
-        NewInst = ir_other_rep,
-        map.lookup(VarToDepVars0, Var, DepVarsFromIM),
-        DepVars = set.union(DepVars0,  DepVarsFromIM),
-        !:SeenDuplicateInstantiation = seen_duplicate_instantiation
-    ),
-    map.set(Var, NewInst, VarToInst0, VarToInst),
-    map.set(Var, DepVars, VarToDepVars0, VarToDepVars),
-    InstMap = inst_map(VarToInst, VarToDepVars).
-
-inst_map_get(inst_map(VarToInst, VarToDepVars), Var, Inst, DepVars) :-
-    ( map.search(VarToInst, Var, InstPrime) ->
-        Inst = InstPrime,
-        map.lookup(VarToDepVars, Var, DepVars)
-    ;
-        Inst = ir_free_rep,
-        DepVars = set.init
+merge_seen_duplicate_instantiation(A, B) = R :-
+    ( if
+        A = have_not_seen_duplicate_instantiation,
+        B = have_not_seen_duplicate_instantiation
+    then
+        R = have_not_seen_duplicate_instantiation
+    else
+        R = seen_duplicate_instantiation
     ).
 
-inst_map_get_var_deps(inst_map(_, VarToDepVars), VarRep, DepVars) :-
-    inst_map_get_var_deps_2(VarToDepVars, VarRep, set.init, DepVars).
-
-:- pred inst_map_get_var_deps_2(map(var_rep, set(var_rep))::in, var_rep::in,
-    set(var_rep)::in, set(var_rep)::out) is det.
-
-inst_map_get_var_deps_2(VarToDepVars, VarRep, !Set) :-
-    ( set.contains(!.Set, VarRep) ->
-        true
-        % This variable has already been visited, this prevents following any
-        % (impossible) cycles in the graph, or following the same path twice
-        % when there are diamonds in the graph.
-    ;
-        ( map.search(VarToDepVars, VarRep, DepVars) ->
-            !:Set = set.union(!.Set, DepVars),
-            set.fold(inst_map_get_var_deps_2(VarToDepVars), DepVars, !Set)
-        ;
-            true
-        )
-    ).
-
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type inst_map_delta
     --->    inst_map_delta(set(var_rep)).
@@ -1133,10 +1182,11 @@ empty_inst_map_delta = InstMap :-
     empty_inst_map_delta(InstMap).
 
 calc_inst_map_delta(Before, After, inst_map_delta(DeltaVars)) :-
-    map.foldl((pred(Var::in, Inst::in, Set0::in, Set::out) is det :-
-            (
+    map.foldl(
+        ( pred(Var::in, Inst::in, Set0::in, Set::out) is det :-
+            ( if
                 map.search(Before ^ im_inst_map, Var, BeforeInst)
-            ->
+            then
                 (
                     BeforeInst = ir_free_rep,
                     (
@@ -1153,8 +1203,8 @@ calc_inst_map_delta(Before, After, inst_map_delta(DeltaVars)) :-
                     BeforeInst = ir_ground_rep,
                     (
                         Inst = ir_free_rep,
-                        error("calc_inst_map_delta: " ++
-                            "Variables cannot become less instantiated.")
+                        unexpected($pred,
+                            "variable should become less instantiated")
                     ;
                         ( Inst = ir_ground_rep
                         ; Inst = ir_other_rep
@@ -1165,8 +1215,8 @@ calc_inst_map_delta(Before, After, inst_map_delta(DeltaVars)) :-
                     BeforeInst = ir_other_rep,
                     (
                         Inst = ir_free_rep,
-                        error("calc_inst_map_delta: " ++
-                            "Variables cannot become less instantiated.")
+                        unexpected($pred,
+                            "variable should become less instantiated")
                     ;
                         ( Inst = ir_ground_rep
                         ; Inst = ir_other_rep
@@ -1174,9 +1224,9 @@ calc_inst_map_delta(Before, After, inst_map_delta(DeltaVars)) :-
                     ),
                     Set = Set0
                 )
-            ;
-                % If we couldn't find the variable then it was free, It may
-                % have been in the head of the procedure.
+            else
+                % If we couldn't find the variable then it was free;
+                % it may have been in the head of the procedure.
                 (
                     Inst = ir_free_rep,
                     Set = Set0
@@ -1190,7 +1240,7 @@ calc_inst_map_delta(Before, After, inst_map_delta(DeltaVars)) :-
             )
         ), After ^ im_inst_map, set.init, DeltaVars).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 atomic_goal_get_vars(AtomicGoal, Vars) :-
     (
@@ -1205,14 +1255,16 @@ atomic_goal_get_vars(AtomicGoal, Vars) :-
         ( AtomicGoal = partial_construct_rep(Var, _, MaybeVars)
         ; AtomicGoal = partial_deconstruct_rep(Var, _, MaybeVars)
         ),
-        list.foldl((pred(MaybeVar::in, Set0::in, Set::out) is det :-
+        list.foldl(
+            ( pred(MaybeVar::in, Set0::in, Set::out) is det :-
                 (
                     MaybeVar = yes(VarI),
                     set.insert(VarI, Set0, Set)
                 ;
                     MaybeVar = no,
                     Set = Set0
-                )), MaybeVars, set.init, Vars0),
+                )
+            ), MaybeVars, set.init, Vars0),
         set.insert(Var, Vars0, Vars)
     ;
         ( AtomicGoal = unify_assign_rep(VarA, VarB)
@@ -1229,17 +1281,7 @@ atomic_goal_get_vars(AtomicGoal, Vars) :-
         Vars = list_to_set(VarsL)
     ).
 
-merge_seen_duplicate_instantiation(A, B) = R :-
-    (
-        A = have_not_seen_duplicate_instantiation,
-        B = have_not_seen_duplicate_instantiation
-    ->
-        R = have_not_seen_duplicate_instantiation
-    ;
-        R = seen_duplicate_instantiation
-    ).
-
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 atomic_goal_is_call(AtomicGoal, IsCall) :-
     (
@@ -1263,4 +1305,6 @@ atomic_goal_is_call(AtomicGoal, IsCall) :-
         IsCall = atomic_goal_is_call(Args)
     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module program_representation_utils.
+%---------------------------------------------------------------------------%
