@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ft=mercury ts=4 sw=4 et
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Copyright (C) 2008-2011 The University of Melbourne.
 % This file may only be copied under the terms of the GNU General
 % Public License - see the file COPYING in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % File: report.m.
 % Author: pbone.
@@ -15,7 +15,7 @@
 % html_format.m, but some tools may use the data structures constructed here
 % directly.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module report.
 :- interface.
@@ -38,7 +38,7 @@
 :- import_module set.
 :- import_module unit.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type deep_report
     --->    report_message(
@@ -215,7 +215,7 @@
                 % The type of recursion in this clique.
                 crr_recursion_type          :: recursion_type,
 
-                % The number of procuedures in the clique.
+                % The number of procedures in the clique.
                 crr_num_procs               :: int
             ).
 
@@ -248,13 +248,13 @@
 
     % Recursion types for which call costs at any level can be calculated.
     %
-:- inst recursion_type_known_costs
+:- inst recursion_type_known_costs for recursion_type/0
     --->    rt_not_recursive
     ;       rt_single(ground, ground, ground, ground, ground).
 
     % Recursion types for which call costs at any level can not be calculated.
     %
-:- inst recursion_type_unknown_costs
+:- inst recursion_type_unknown_costs for recursion_type/0
     --->    rt_divide_and_conquer(ground, ground)
     ;       rt_mutual_recursion(ground)
     ;       rt_other(ground)
@@ -697,7 +697,7 @@
                 cdesc_other_members             :: list(proc_desc)
             ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Predicates to make common actions on these types easier.
 %
@@ -714,12 +714,12 @@
     map(reverse_goal_path, clique_call_site_report)::in,
     map(reverse_goal_path, clique_call_site_report)::out) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 proc_label_from_proc_desc(Deep, ProcDesc, ProcLabel) :-
     PSPtr = ProcDesc ^ pdesc_ps_ptr,
@@ -731,6 +731,6 @@ add_call_site_report_to_map(CallSite, !Map) :-
         ^ csdesc_goal_path,
     map.det_insert(GoalPath, CallSite, !Map).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module report.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
