@@ -1,13 +1,15 @@
 %-----------------------------------------------------------------------------%
 % Copyright (C) 1997-1998, 2003 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2015, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
 % mercury_opengl -- a Mercury binding for OpenGL.
 
 :- module mercury_opengl.
+:- interface.
 
-:- import_module mglu, mogl.
+:- import_module mglu.
+:- import_module mogl.
 
