@@ -1,23 +1,23 @@
 %-----------------------------------------------------------------------------%
-% vim: ts=4 sw=4 et tw=0 wm=0 ff=unix
+% vim: ts=4 sw=4 et tw=0 wm=0 ff=unix ft=mercury
 %-----------------------------------------------------------------------------%
 %
 % lex.m
 % Copyright (C) 2001-2002 Ralph Becket <rbeck@microsoft.com>
 % Sun Aug 20 09:08:46 BST 2000
 % Copyright (C) 2001-2002 The Rationalizer Intelligent Software AG
-%   The changes made by Rationalizer are contributed under the terms 
+%   The changes made by Rationalizer are contributed under the terms
 %   of the GNU Lesser General Public License, see the file COPYING.LGPL
 %   in this directory.
 % Copyright (C) 2002, 2006, 2010-2011 The University of Melbourne
+% Copyright (C) 2014, 2017-2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% This module puts everything together, compiling a list of lexemes into state
+% machines and turning the input stream into a token stream.
 %
-% Thu Jul 26 07:45:47 UTC 2001
-%
-% This module puts everything together, compiling a list of lexemes
-% into state machines and turning the input stream into a token stream.
+% Note that the astral characters (in Unicode) are not included in the range of
+% Unicode characters, as the astral planes are very sparsely assigned.
 %
 %-----------------------------------------------------------------------------%
 
@@ -27,19 +27,17 @@
 :- import_module char.
 :- import_module io.
 :- import_module list.
-:- import_module maybe.
 :- import_module pair.
 :- import_module string.
+:- import_module sparse_bitset.
+:- import_module enum.
 
 %-----------------------------------------------------------------------------%
 
-:- type token_creator(Token)
-    ==                        (func(string) = Token).
-:- inst token_creator
-    ==                        (func(in) = out is det).
+:- type token_creator(Token) ==  (func(string) = Token).
+:- inst token_creator ==         (func(in) = out is det).
 
-:- type lexeme(Token)
-    ==                        pair(regexp, token_creator(Token)).
+:- type lexeme(Token) == pair(regexp, token_creator(Token)).
 
 :- inst lexeme(Inst)
     ---> (ground - Inst).
@@ -48,8 +46,8 @@
 
 :- type lexer_state(Token, Source).
 
-:- type offset
-    ==      int.                        % Byte offset into the source data.
+    % Byte offset into the source data.
+:- type offset == int.
 
     % Any errors should be reported by raising an exception.
     %
@@ -60,17 +58,17 @@
     % read_pred(Offset, Result, SrcIn, SrcOut) reads the char at
     % Offset from SrcIn and returns SrcOut.
     %
-:- type read_pred(T)
-    ==      pred(offset, read_result, T, T).
-:- inst read_pred
-    ==      ( pred(in, out, di, uo) is det ).
+:- type read_pred(T) == pred(offset, read_result, T, T).
+:- inst read_pred ==   (pred(in, out, di, uo) is det).
 
     % ignore_pred(Token): if it does not fail, Token must be ignored
     %
-:- type ignore_pred(Tok)
-    ==      pred(Tok).
-:- inst ignore_pred
-    ==      ( pred(in) is semidet ).
+:- type ignore_pred(Tok) == pred(Tok).
+:- inst ignore_pred ==     (pred(in) is semidet).
+
+    % Represents a set of Unicode characters
+    %
+:- type charset == sparse_bitset(char).
 
     % The type of regular expressions.
     %
@@ -79,27 +77,28 @@
     % The typeclass for types having a natural converter to regexp's
     %
 :- typeclass regexp(T) where [
-           func re(T) = regexp
+    func re(T) = regexp
 ].
 
     % Handling regexp's based on the typeclass regexp(T)
     %
-:- func  null      = regexp.
-:- func  T1 ++ T2  = regexp  <= (regexp(T1), regexp(T2)).
-:- func  *(T)      = regexp  <= (regexp(T)).
-    % One of the following two functions may be deprecated
-    % in future, depending upon whether there's a concensus
-    % concerning which is preferable.  Both express
-    % alternation.
-    %
-:- func  T1 \/ T2  = regexp  <= (regexp(T1), regexp(T2)).
-:- func (T1 or T2) = regexp  <= (regexp(T1), regexp(T2)).
+:- func null      = regexp.
+:- func T1 ++ T2  = regexp <= (regexp(T1), regexp(T2)).
+:- func *(T)      = regexp <= (regexp(T)).
 
-    % Some instances of typeclass regexp(T)
+    % One of the following two functions may be deprecated in future,
+    % depending upon whether there's a concensus concerning
+    % which is preferable. Both express alternation.
+    %
+:- func  T1 \/ T2  = regexp <= (regexp(T1), regexp(T2)).
+:- func (T1 or T2) = regexp <= (regexp(T1), regexp(T2)).
+
+    % Some instances of typeclass regexp(T).
     %
 :- instance regexp(regexp).
 :- instance regexp(char).
 :- instance regexp(string).
+:- instance regexp(sparse_bitset(T)) <= (regexp(T),enum(T)).
 
     % Some basic non-primitive regexps.
     %
@@ -107,6 +106,8 @@
 :- func anybut(string) = regexp.     % anybut("abc") is complement of any("abc")
 :- func ?(T) = regexp <= regexp(T).  % ?(R)       = R or null
 :- func +(T) = regexp <= regexp(T).  % +(R)       = R ++ *(R)
+:- func range(char, char) = regexp.  % range('a', 'z') = any("ab...xyz")
+:- func (T * int) = regexp <= regexp(T). % R * N = R ++ ... ++ R
 
     % Some useful single-char regexps.
     %
@@ -117,14 +118,14 @@
 :- func alphanum = regexp.      % alphanum   = alpha or digit
 :- func identstart = regexp.    % identstart = alpha or "_"
 :- func ident = regexp.         % ident      = alphanum or "_"
-:- func nl = regexp.            % nl         = re("\n")
 :- func tab = regexp.           % tab        = re("\t")
 :- func spc = regexp.           % spc        = re(" ")
 :- func wspc = regexp.          % wspc       = any(" \t\n\r\f\v")
-:- func dot = regexp.           % dot        = anybut("\n")
+:- func dot = regexp.           % dot        = anybut("\r\n")
 
     % Some useful compound regexps.
     %
+:- func nl = regexp.            % nl         = ?("\r") ++ re("\n")
 :- func nat = regexp.           % nat        = +(digit)
 :- func signed_int = regexp.    % signed_int = ?("+" or "-") ++ nat
 :- func real = regexp.          % real       = \d+((.\d+([eE]int)?)|[eE]int)
@@ -132,9 +133,42 @@
 :- func whitespace = regexp.    % whitespace = *(wspc)
 :- func junk = regexp.          % junk       = *(dot)
 
+    % A range of characters, inclusive of both the first and last values.
+    %
+:- type char_range
+    --->    char_range(
+                cr_first        :: int,
+                cr_last         :: int
+            ).
+
+    % charset(Start, End) = charset(Start `..` End)
+    %
+    % Throws an exception if Start > End.
+    %
+:- func charset(int, int) = charset.
+
+    % Function to create a sparse bitset from a range of Unicode codepoints.
+    % These codepoints are checked for validity, any invalid codepoints
+    % are ignored. Throws an exception if cr_first value is less than cr_last.
+    %
+:- func charset(char_range) = charset.
+
+    % Creates a union of all char ranges in the list. Returns the empty set
+    % if the list is empty. Any invalid codepoints are ignored.
+    %
+:- func charset_from_ranges(list(char_range)) = charset.
+
+    % Latin is comprised of the following Unicode blocks:
+    %  * Basic Latin
+    %  * Latin1 Supplement
+    %  * Latin Extended-A
+    %  * Latin Extended-B
+    %
+:- func latin_chars = charset is det.
+
    % Utility predicate to create ignore_pred's.
    % Use it in the form `ignore(my_token)' to ignore just `my_token'.
-   % 
+   %
 :- pred ignore(Token::in, Token::in) is semidet.
 
    % Utility function to return noval tokens.
@@ -145,7 +179,7 @@
    % Utility operator to create lexemes.
    %
 :- func (T1 -> token_creator(Tok)) = pair(regexp, token_creator(Tok))
-            <= regexp(T1).
+    <= regexp(T1).
 
     % Construct a lexer from which we can generate running
     % instances.
@@ -154,8 +188,8 @@
     % the token generated by the one closest to the start
     % of the list of lexemes is returned.
     %
-:- func init(list(lexeme(Tok)), read_pred(Src)) = lexer(Tok, Src).
-:- mode init(in, in(read_pred)) = out is det.
+:- func init(list(lexeme(Tok))::in, read_pred(Src)::in(read_pred))
+    = (lexer(Tok, Src)::out) is det.
 
     % Construct a lexer from which we can generate running
     % instances. If we construct a lexer with init/4, we
@@ -165,72 +199,62 @@
     % the token generated by the one closest to the start
     % of the list of lexemes is returned.
     %
-:- func init(list(lexeme(Tok)), read_pred(Src), ignore_pred(Tok)) =
-            lexer(Tok, Src).
-:- mode init(in, in(read_pred), in(ignore_pred)) = out is det.
+:- func init(list(lexeme(Tok))::in, read_pred(Src)::in(read_pred),
+    ignore_pred(Tok)::in(ignore_pred)) = (lexer(Tok, Src)::out) is det.
 
     % Handy read predicates.
     %
-:- pred read_from_stdin(offset, read_result, io.state, io.state).
-:- mode read_from_stdin(in, out, di, uo) is det.
-
-:- pred read_from_string(offset, read_result, string, string).
-:- mode read_from_string(in, out, di, uo) is det.
+:- pred read_from_stdin(offset::in, read_result::out, io::di, io::uo) is det.
+:- pred read_from_string(offset::in, read_result::out,
+    string::di, string::uo) is det.
 
     % Generate a running instance of a lexer on some input source.
     % If you want to lex strings, you must ensure they are unique
     % by calling either copy/1 or unsafe_promise_unique/1 on the
     % source string argument.
     %
-    % Note that you can't get the input source back until you stop
-    % lexing.
+    % Note that you can't get the input source back until you stop lexing.
     %
-:- func start(lexer(Tok, Src), Src) = lexer_state(Tok, Src).
-:- mode start(in, di) = uo is det.
+:- func start(lexer(Tok, Src)::in, Src::di) = (lexer_state(Tok, Src)::uo)
+    is det.
 
     % Read the next token from the input stream.
     %
-    % CAVEAT: if the token returned happened to match the empty
-    % string then you must use read_char/3 (below) to consume
-    % the next char in the input stream before calling read/3
-    % again, since matching the empty string does not consume
-    % any chars from the input stream and will otherwise mean
-    % you simply get the same match ad infinitum.
+    % CAVEAT: if the token returned happened to match the empty string,
+    % then you must use read_char/3 (below) to consume the next char
+    % in the input stream before calling read/3 again, since matching
+    % the empty string does not consume any chars from the input stream
+    % and will otherwise mean you simply get the same match ad infinitum.
     %
-    % An alternative solution is to always include a "catch all"
-    % lexeme that matches any unexpected char at the end of the
-    % list of lexemes.
+    % An alternative solution is to always include a "catch all" lexeme
+    % that matches any unexpected char at the end of the list of lexemes.
     %
-:- pred read(io.read_result(Tok),
-            lexer_state(Tok, Src), lexer_state(Tok, Src)).
-:- mode read(out, di, uo) is det.
+:- pred read(io.read_result(Tok)::out,
+    lexer_state(Tok, Src)::di, lexer_state(Tok, Src)::uo) is det.
 
     % Calling offset_from_start/3 immediately prior to calling read/3
     % will give the offset in chars from the start of the input stream
     % for the result returned by the read/3 operation.
     %
-:- pred offset_from_start(offset,
-            lexer_state(Tok, Src), lexer_state(Tok, Src)).
-:- mode offset_from_start(out, di, uo) is det.
+:- pred offset_from_start(offset::out,
+    lexer_state(Tok, Src)::di, lexer_state(Tok, Src)::uo) is det.
 
     % Stop a running instance of a lexer and retrieve the input source.
     %
-:- func stop(lexer_state(_Tok, Src)) = Src.
-:- mode stop(di) = uo is det.
+:- func stop(lexer_state(_Tok, Src)::di) = (Src::uo) is det.
 
     % Sometimes (e.g. when lexing the io.io) you want access to the
-    % input stream without interrupting the lexing process.  This pred
-    % provides that sort of access.
+    % input stream without interrupting the lexing process.
+    % This pred provides that sort of access.
     %
-:- pred manipulate_source(pred(Src, Src),
-            lexer_state(Tok, Src), lexer_state(Tok, Src)).
-:- mode manipulate_source(pred(di, uo) is det, di, uo) is det.
+:- pred manipulate_source(pred(Src, Src)::in(pred(di, uo) is det),
+    lexer_state(Tok, Src)::di, lexer_state(Tok, Src)::uo) is det.
 
-    % This is occasionally useful.  It reads the next char from the
+    % This is occasionally useful. It reads the next char from the
     % input stream, without attempting to match it against a lexeme.
     %
-:- pred read_char(read_result, lexer_state(Tok, Src), lexer_state(Tok, Src)).
-:- mode read_char(out, di, uo) is det.
+:- pred read_char(read_result::out,
+    lexer_state(Tok, Src)::di, lexer_state(Tok, Src)::uo) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -245,10 +269,11 @@
 
 :- import_module array.
 :- import_module bool.
-:- import_module char.
 :- import_module exception.
+:- import_module require.
 :- import_module int.
 :- import_module map.
+:- import_module maybe.
 
 :- import_module lex.automata.
 :- import_module lex.buf.
@@ -280,30 +305,24 @@
 
 :- inst lexer_instance
     --->    lexer_instance(
-                live_lexeme_list, 
-                init_winner_func, 
-                live_lexeme_list, 
-                winner, 
+                live_lexeme_list,
+                init_winner_func,
+                live_lexeme_list,
+                winner,
                 buf.buf_state,
                 ignore_pred
             ).
 
-:- type live_lexeme(Token)
-    ==      compiled_lexeme(Token).
-:- inst live_lexeme
-    ==      compiled_lexeme.
-:- inst live_lexeme_list
-    ==      list.list_skel(live_lexeme).
+:- type live_lexeme(Token) == compiled_lexeme(Token).
+:- inst live_lexeme == compiled_lexeme.
+:- inst live_lexeme_list ==  list.list_skel(live_lexeme).
 
 :- type init_winner_func(Token)
     ==      ( func(offset) = winner(Token) ).
 :- inst init_winner_func
     ==      ( func(in)     = out is det    ).
 
-
-
-:- type winner(Token)
-    ==      maybe(pair(token_creator(Token), offset)).
+:- type winner(Token) == maybe(pair(token_creator(Token), offset)).
 :- inst winner
     --->    yes(pair(token_creator, ground))
     ;       no.
@@ -322,13 +341,16 @@ return(Token, _) = Token.
 
 %-----------------------------------------------------------------------------%
 
-init(Lexemes, BufReadPred) = init(Lexemes, BufReadPred, DontIgnoreAnything) :-
-    DontIgnoreAnything = ( pred(_::in) is semidet :- semidet_fail ).
+init(Lexemes, BufReadPred) = Lexer :-
+    DontIgnoreAnything =
+        ( pred(_::in) is semidet :-
+            semidet_fail
+        ),
+    Lexer = init(Lexemes, BufReadPred, DontIgnoreAnything).
 
-init(Lexemes, BufReadPred, IgnorePred) =
-    lexer(CompiledLexemes, IgnorePred, BufReadPred)
- :-
-    CompiledLexemes = list.map(compile_lexeme, Lexemes).
+init(Lexemes, BufReadPred, IgnorePred) = Lexer :-
+    CompiledLexemes = list.map(compile_lexeme, Lexemes),
+    Lexer = lexer(CompiledLexemes, IgnorePred, BufReadPred).
 
 %-----------------------------------------------------------------------------%
 
@@ -347,10 +369,24 @@ start(Lexer0, Src) = State :-
     Lexer = Lexer0;
 ").
 
+:- pragma foreign_proc("Java",
+    lexer_inst_cast(Lexer0::in) = (Lexer::out(lexer)),
+    [will_not_call_mercury, promise_pure, thread_safe],
+"
+    Lexer = Lexer0;
+").
+
+:- pragma foreign_proc("C#",
+    lexer_inst_cast(Lexer0::in) = (Lexer::out(lexer)),
+    [will_not_call_mercury, promise_pure, thread_safe],
+"
+    Lexer = Lexer0;
+").
+
 %-----------------------------------------------------------------------------%
 
-:- pred init_lexer_instance(lexer(Tok, Src), lexer_instance(Tok, Src), buf).
-:- mode init_lexer_instance(in(lexer), out(lexer_instance), array_uo) is det.
+:- pred init_lexer_instance(lexer(Tok, Src)::in(lexer),
+    lexer_instance(Tok, Src)::out(lexer_instance), buf::array_uo) is det.
 
 init_lexer_instance(Lexer, Instance, Buf) :-
     buf.init(Lexer ^ lex_buf_read_pred, BufState, Buf),
@@ -365,20 +401,19 @@ init_lexer_instance(Lexer, Instance, Buf) :-
 %-----------------------------------------------------------------------------%
 
     % Lexing may *start* with a candidate winner if one of the lexemes
-    % accepts the empty string.  We pick the first such, if any, since
+    % accepts the empty string. We pick the first such, if any, since
     % that lexeme has priority.
     %
-:- func initial_winner_func(list(live_lexeme(Token))) = init_winner_func(Token).
-:- mode initial_winner_func(in(live_lexeme_list)    ) = out(init_winner_func)
-            is det.
+:- func initial_winner_func(list(live_lexeme(Token))::in(live_lexeme_list))
+    = (init_winner_func(Token)::out(init_winner_func)) is det.
 
-initial_winner_func([]       ) =
+initial_winner_func([]) =
     ( func(_) = no ).
-
-initial_winner_func( [L | Ls]) =
-    ( if   in_accepting_state(L)
-      then ( func(Offset) = yes(L ^ token - Offset) )
-      else initial_winner_func(Ls)
+initial_winner_func([L | Ls]) =
+    ( if in_accepting_state(L) then
+        ( func(Offset) = yes(L ^ token - Offset) )
+    else
+        initial_winner_func(Ls)
     ).
 
 %----------------------------------------------------------------------------%
@@ -395,7 +430,6 @@ stop(State) = Src :-
 %-----------------------------------------------------------------------------%
 
 read(Result, State0, State) :-
-
     lexer_state_args(State0, Instance0, Buf0, Src0),
     BufState0  = Instance0 ^ buf_state,
     Start      = BufState0 ^ start_offset,
@@ -404,22 +438,16 @@ read(Result, State0, State) :-
     read_2(Result, Instance1, Instance, Buf0, Buf, Src0, Src),
     State      = args_lexer_state(Instance, Buf, Src).
 
-
-
-:- pred read_2(io.read_result(Tok),
-            lexer_instance(Tok, Src), lexer_instance(Tok, Src),
-            buf, buf, Src, Src).
-:- mode read_2(out,
-            in(lexer_instance), out(lexer_instance),
-            array_di, array_uo, di, uo) is det.
-
     % Basically, just read chars from the buf and advance the live lexemes
     % until we have a winner or hit an error (no parse).
     %
+:- pred read_2(io.read_result(Tok)::out,
+    lexer_instance(Tok, Src)::in(lexer_instance),
+        lexer_instance(Tok, Src)::out(lexer_instance),
+    buf::array_di, buf::array_uo, Src::di, Src::uo) is det.
+
 read_2(Result, !Instance, !Buf, !Src) :-
-
     BufState0 = !.Instance ^ buf_state,
-
     buf.read(BufReadResult, BufState0, BufState, !Buf, !Src),
     (
         BufReadResult = ok(Char),
@@ -431,61 +459,56 @@ read_2(Result, !Instance, !Buf, !Src) :-
 
 %-----------------------------------------------------------------------------%
 
-:- pred process_char(io.read_result(Tok), char,
-            lexer_instance(Tok, Src), lexer_instance(Tok, Src),
-            buf_state(Src), buf, buf, Src, Src).
-:- mode process_char(out, in, in(lexer_instance), out(lexer_instance),
-            in(buf_state), array_di, array_uo, di, uo) is det.
+:- pred process_char(io.read_result(Tok)::out, char::in,
+    lexer_instance(Tok, Src)::in(lexer_instance),
+        lexer_instance(Tok, Src)::out(lexer_instance),
+    buf_state(Src)::in(buf_state),
+    buf::array_di, buf::array_uo, Src::di, Src::uo) is det.
 
 process_char(Result, Char, !Instance, BufState, !Buf, !Src) :-
-
     LiveLexemes0 = !.Instance ^ live_lexemes,
     Winner0      = !.Instance ^ current_winner,
-
     advance_live_lexemes(Char, BufState ^ cursor_offset,
-            LiveLexemes0, LiveLexemes, Winner0, Winner),
+        LiveLexemes0, LiveLexemes, Winner0, Winner),
     (
-        LiveLexemes = [],               % Nothing left to consider.
-
+        LiveLexemes = [],
+        % Nothing left to consider.
         process_any_winner(Result, Winner, !Instance, BufState, !Buf, !Src)
     ;
-        LiveLexemes = [_ | _],          % Still some open possibilities.
-
-        !:Instance  = (((!.Instance ^ live_lexemes   := LiveLexemes )
-                                    ^ current_winner := Winner      )
-                                    ^ buf_state      := BufState    ),
+        LiveLexemes = [_ | _],
+        % Still some open possibilities.
+        !:Instance  = (((!.Instance
+            ^ live_lexemes   := LiveLexemes )
+            ^ current_winner := Winner      )
+            ^ buf_state      := BufState    ),
         read_2(Result, !Instance, !Buf, !Src)
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred process_any_winner(io.read_result(Tok), winner(Tok),
-            lexer_instance(Tok, Src), lexer_instance(Tok, Src), 
-            buf_state(Src), buf, buf, Src, Src).
-:- mode process_any_winner(out, in(winner),
-            in(lexer_instance), out(lexer_instance),
-            in(buf_state), array_di, array_uo, di, uo) is det.
+:- pred process_any_winner(io.read_result(Tok)::out, winner(Tok)::in(winner),
+    lexer_instance(Tok, Src)::in(lexer_instance),
+        lexer_instance(Tok, Src)::out(lexer_instance),
+    buf_state(Src)::in(buf_state),
+    buf::array_di, buf::array_uo, Src::di, Src::uo) is det.
 
 process_any_winner(Result, yes(TokenCreator - Offset), Instance0, Instance,
         BufState0, Buf0, Buf, Src0, Src) :-
-
     BufState1  = rewind_cursor(Offset, BufState0),
     String     = string_to_cursor(BufState1, Buf0),
     Token      = TokenCreator(String),
     IgnorePred = Instance0 ^ ignore_pred,
     InitWinner = ( Instance0 ^ init_winner_func )(Offset),
     Instance1  = ((( Instance0
-                       ^ live_lexemes       := Instance0 ^ init_lexemes )
-                       ^ current_winner     := InitWinner               )
-                       ^ buf_state          := commit(BufState1)        ),
+        ^ live_lexemes       := Instance0 ^ init_lexemes )
+        ^ current_winner     := InitWinner               )
+        ^ buf_state          := commit(BufState1)        ),
 
     ( if IgnorePred(Token) then
-
-            % We have to be careful to avoid an infinite loop here.
-            % If the longest match was the empty string, then the
-            % next char in the input stream cannot start a match,
-            % so it must be reported as an error.
-            %
+        % We have to be careful to avoid an infinite loop here.
+        % If the longest match was the empty string, then the next char
+        % in the input stream cannot start a match, so it must be reported
+        % as an error.
         ( if String = "" then
             buf.read(BufResult, BufState1, BufState, Buf0, Buf, Src0, Src),
             (
@@ -496,38 +519,34 @@ process_any_winner(Result, yes(TokenCreator - Offset), Instance0, Instance,
                 Result    = eof
             ),
             Instance = ( Instance1 ^ buf_state := commit(BufState) )
-          else
+        else
             read_2(Result, Instance1, Instance, Buf0, Buf, Src0, Src)
         )
-      else
+    else
         Result   = ok(Token),
         Instance = Instance1,
         Buf      = Buf0,
         Src      = Src0
     ).
-
-process_any_winner(Result, no, !Instance,
-        BufState0, !Buf, !Src) :-
-
-    Start      = BufState0 ^ start_offset,
-    BufState   = rewind_cursor(Start + 1, BufState0),
-    Result     = error("input not matched by any regexp", Start),
+process_any_winner(Result, no, !Instance, BufState0, !Buf, !Src) :-
+    Start    = BufState0 ^ start_offset,
+    BufState = rewind_cursor(Start + 1, BufState0),
+    Result   = error("input not matched by any regexp", Start),
 
     InitWinner = ( !.Instance ^ init_winner_func )(Start),
-    !:Instance = ((( !.Instance ^ live_lexemes   := !.Instance ^ init_lexemes )
-                                ^ current_winner := InitWinner                )
-                                ^ buf_state      := commit(BufState)          ).
+    !:Instance = ((( !.Instance
+        ^ live_lexemes   := !.Instance ^ init_lexemes )
+        ^ current_winner := InitWinner                )
+        ^ buf_state      := commit(BufState)          ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred process_eof(io.read_result(Tok),
-            lexer_instance(Tok, Src), lexer_instance(Tok, Src),
-            buf_state(Src), buf).
-:- mode process_eof(out, in(lexer_instance), out(lexer_instance),
-            in(buf_state), array_ui) is det.
+:- pred process_eof(io.read_result(Tok)::out,
+    lexer_instance(Tok, Src)::in(lexer_instance),
+        lexer_instance(Tok, Src)::out(lexer_instance),
+    buf_state(Src)::in(buf_state), buf::array_ui) is det.
 
 process_eof(Result, !Instance, !.BufState, !.Buf) :-
-
     CurrentWinner = !.Instance ^ current_winner,
     (
         CurrentWinner = no,
@@ -541,9 +560,10 @@ process_eof(Result, !Instance, !.BufState, !.Buf) :-
         Result        = ( if IgnorePred(Token) then eof else ok(Token) )
     ),
     InitWinner = ( !.Instance ^ init_winner_func )(Offset),
-    !:Instance = ((( !.Instance ^ live_lexemes   := !.Instance ^ init_lexemes )
-                                ^ current_winner := InitWinner                )
-                                ^ buf_state      := commit(!.BufState)        ).
+    !:Instance = ((( !.Instance
+        ^ live_lexemes   := !.Instance ^ init_lexemes )
+        ^ current_winner := InitWinner                )
+        ^ buf_state      := commit(!.BufState)        ).
 
 %-----------------------------------------------------------------------------%
 
@@ -551,81 +571,70 @@ process_eof(Result, !Instance, !.BufState, !.Buf) :-
     % string, the win is given to the earliest such lexeme in the list.
     % This matches the behaviour of standard C lex.
     %
-:- pred advance_live_lexemes(char, offset,
-            list(live_lexeme(Token)), list(live_lexeme(Token)),
-            winner(Token), winner(Token)).
-:- mode advance_live_lexemes(in, in, in(live_lexeme_list), 
-            out(live_lexeme_list), 
-            in(winner), out(winner)) is det.
+:- pred advance_live_lexemes(char::in, offset::in,
+    list(live_lexeme(Token))::in(live_lexeme_list),
+        list(live_lexeme(Token))::out(live_lexeme_list),
+    winner(Token)::in(winner), winner(Token)::out(winner)) is det.
 
 advance_live_lexemes(_Char, _Offset, [], [], !Winner).
-
 advance_live_lexemes(Char, Offset, [L | Ls0], Ls, !Winner) :-
-
-    State0        = L ^ state,
-
+    State0 = L ^ state,
     ( if next_state(L, State0, Char, State, IsAccepting) then
-
         (
             IsAccepting = no
         ;
             IsAccepting = yes,
-            !:Winner    = ( if   !.Winner = yes(_ - Offset)
-                            then !.Winner
-                            else yes(L ^ token - Offset)
-                          )
+            ( if !.Winner = yes(_ - Offset) then
+                true
+            else
+                !:Winner = yes(L ^ token - Offset)
+            )
         ),
         advance_live_lexemes(Char, Offset, Ls0, Ls1, !Winner),
-        Ls = [( L ^ state := State ) | Ls1]
-
-      else
-
+        Ls = [(L ^ state := State) | Ls1]
+    else
         advance_live_lexemes(Char, Offset, Ls0, Ls, !Winner)
     ).
 
 %-----------------------------------------------------------------------------%
 
-:- pred live_lexeme_in_accepting_state(list(live_lexeme(Tok)),
-                token_creator(Tok)).
-:- mode live_lexeme_in_accepting_state(in(live_lexeme_list), 
-                out(token_creator)) is semidet.
+:- pred live_lexeme_in_accepting_state(
+    list(live_lexeme(Tok))::in(live_lexeme_list),
+    token_creator(Tok)::out(token_creator)) is semidet.
 
 live_lexeme_in_accepting_state([L | Ls], Token) :-
-    ( if   in_accepting_state(L)
-      then Token = L ^ token
-      else live_lexeme_in_accepting_state(Ls, Token)
+    ( if in_accepting_state(L) then
+        Token = L ^ token
+    else
+        live_lexeme_in_accepting_state(Ls, Token)
     ).
 
-
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
 
-    % It's much more convenient (especially for integration with, e.g.
-    % parsers such as moose) to package up the lexer_instance, buf
-    % and Src in a single object.
-
+    % It is much more convenient (especially for integration with parsers
+    % such as moose) to package up the lexer_instance, buf and Src
+    % in a single object.
 :- type lexer_state(Tok, Src)
     --->    lexer_state(
-                run                     :: lexer_instance(Tok, Src),
-                buf                     :: buf,
-                src                     :: Src
+                run     :: lexer_instance(Tok, Src),
+                buf     :: buf,
+                src     :: Src
             ).
 
 %-----------------------------------------------------------------------------%
 
-:- func args_lexer_state(lexer_instance(Tok, Src), buf, Src) =
-            lexer_state(Tok, Src).
-:- mode args_lexer_state(in(lexer_instance), array_di, di) = uo is det.
+:- func args_lexer_state(lexer_instance(Tok, Src)::in(lexer_instance),
+    buf::array_di, Src::di) = (lexer_state(Tok, Src)::uo) is det.
 
 args_lexer_state(Instance, Buf, Src) = LexerState :-
     unsafe_promise_unique(lexer_state(Instance, Buf, Src), LexerState).
 
 %-----------------------------------------------------------------------------%
 
-:- pred lexer_state_args(lexer_state(Tok, Src), lexer_instance(Tok, Src),
-            buf, Src).
-:- mode lexer_state_args(di, out(lexer_instance),
-            array_uo, uo)  is det.
+:- pred lexer_state_args(lexer_state(Tok, Src)::di,
+    lexer_instance(Tok, Src)::out(lexer_instance),
+    buf::array_uo, Src::uo) is det.
 
 lexer_state_args(lexer_state(Instance, Buf0, Src0), Instance, Buf, Src) :-
     unsafe_promise_unique(Buf0, Buf),
@@ -641,7 +650,6 @@ manipulate_source(P, !State) :-
 %----------------------------------------------------------------------------%
 
 read_char(Result, !State) :-
-
     lexer_state_args(!.State, Instance0, Buf0, Src0),
 
     BufState0 = Instance0 ^ buf_state,
@@ -652,34 +660,43 @@ read_char(Result, !State) :-
 
 %-----------------------------------------------------------------------------%
 
-read_from_stdin(_Offset, Result) -->
-    io.read_char(IOResult),
-    {   IOResult = ok(Char),              Result = ok(Char)
-    ;   IOResult = eof,                   Result = eof
-    ;   IOResult = error(_E),             throw(IOResult)
-    }.
-
-%-----------------------------------------------------------------------------%
-
-    % XXX This is bad for long strings!  We should cache the string
-    % length somewhere rather than recomputing it each time we read
-    % a char.
-    %
-read_from_string(Offset, Result, String, unsafe_promise_unique(String)) :-
-    ( if   Offset < string.length(String)
-      then Result = ok(string.unsafe_index(String, Offset))
-      else Result = eof
+read_from_stdin(_Offset, Result, !IO) :-
+    io.read_char(IOResult, !IO),
+    (
+        IOResult = ok(Char),
+        Result = ok(Char)
+    ;
+        IOResult = eof,
+        Result = eof
+    ;
+        IOResult = error(_E),
+        throw(IOResult)
     ).
 
 %-----------------------------------------------------------------------------%
+
+    % XXX This is bad for long strings! We should cache the string length
+    % somewhere rather than recomputing it each time we read a char.
+    %
+read_from_string(Offset, Result, String, unsafe_promise_unique(String)) :-
+    ( if Offset < string.length(String) then
+        Result = ok(string.unsafe_index(String, Offset))
+    else
+        Result = eof
+    ).
+
+%-----------------------------------------------------------------------------%
+%
 % The type of regular expressions.
+%
 
 :- type regexp
     --->    eps                    % The empty regexp
     ;       atom(char)             % Match a single char
     ;       conc(regexp, regexp)   % Concatenation
     ;       alt(regexp, regexp)    % Alternation
-    ;       star(regexp).          % Kleene closure
+    ;       star(regexp)           % Kleene closure
+    ;       charset(charset).      % Matches any char in the set
 
 %-----------------------------------------------------------------------------%
 
@@ -696,85 +713,141 @@ read_from_string(Offset, Result, String, unsafe_promise_unique(String)) :-
         ( if S = "" then
             R = null
           else
-            L = string.length(S),
-            C = string.det_index(S, L - 1),
-            R = str_foldr(func(Cx, Rx) = (Cx ++ Rx), S, re(C), L - 2)
+            R = string.foldl(func(Char, R0) = R1 :-
+                ( if R0 = eps then R1 = re(Char) else R1 = R0 ++ re(Char) ),
+                S,
+                eps)
         )
 ].
 
+:- instance regexp(sparse_bitset(T)) <= (regexp(T),enum(T)) where [
+    re(SparseBitset) = charset(Charset) :-
+        Charset = sparse_bitset.foldl(
+            func(Enum, Set0) = insert(Set0, char.det_from_int(to_int(Enum))),
+            SparseBitset,
+            sparse_bitset.init)
+].
+
 %-----------------------------------------------------------------------------%
+%
 % Basic primitive regexps.
+%
 
- null      = eps.
- R1 ++ R2  = conc(re(R1), re(R2)).
- R1 \/ R2  = alt(re(R1), re(R2)).
+null       = eps.
+R1 ++ R2   = conc(re(R1), re(R2)).
+R1 \/ R2   = alt(re(R1), re(R2)).
 (R1 or R2) = alt(re(R1), re(R2)).
- *(R1)     = star(re(R1)).
+*(R1)      = star(re(R1)).
 
 %-----------------------------------------------------------------------------%
+%
 % Some basic non-primitive regexps.
+%
+
+    % int_is_valid_char(Int) = Char.
+    %
+    % True iff Int is Char and is in [0x0..0x10ffff] and not a surrogate
+    % character.
+    %
+:- func int_is_valid_char(int) = char is semidet.
+
+int_is_valid_char(Value) = Char :-
+    char.from_int(Value, Char),
+    not char.is_surrogate(Char).
+
+charset(Start, End) = build_charset(Start, End, sparse_bitset.init) :-
+    expect(Start =< End, $file, $pred,
+        "Start must be less than or equal to End").
+
+charset(char_range(First, Last)) = charset(First, Last).
+
+:- func build_charset(int, int, charset) = charset.
+
+build_charset(First, Last, Charset0) = Charset :-
+    ( if First =< Last then
+        ( if int_is_valid_char(First) = Char then
+            Charset1 = sparse_bitset.insert(Charset0, Char)
+        else
+            Charset1 = Charset0
+        ),
+        Charset = build_charset(First + 1, Last, Charset1)
+    else
+        Charset = Charset0
+    ).
+
+charset_from_ranges(ListOfRanges) =
+    union_list(map(charset, ListOfRanges)).
+
+latin_chars =
+    charset_from_ranges([
+        char_range(0x40, 0x7d),
+        char_range(0xc0, 0xff),
+        char_range(0x100, 0x2ff)
+    ]).
+
+:- func valid_unicode_chars = charset.
+
+valid_unicode_chars = charset(char_range(0x01, 0xffff)).
 
 any(S) = R :-
     ( if S = "" then
         R = null
-      else
-        L = string.length(S),
-        C = string.det_index(S, L - 1),
-        R = str_foldr(func(Cx, Rx) = (Cx or Rx), S, re(C), L - 2)
+    else
+        R = re(sparse_bitset.list_to_set(string.to_char_list(S)))
     ).
 
-anybut(S0) = R :-
-    S = string.from_char_list(
-            list.filter_map(
-                ( func(X) = C is semidet :-
-                    char.to_int(C, X),
-                    not string.contains_char(S0, C)
-                ),
-                0x01 `..` 0xff
-            )
-        ),
-    R = any(S).
-
-:- func str_foldr(func(char, T) = T, string, T, int) = T.
-
-str_foldr(Fn, S, X, I) =
-    ( if I < 0 then X
-               else str_foldr(Fn, S, Fn(string.det_index(S, I), X), I - 1)
-    ).
+anybut(S) = R :-
+    ExcludedChars = sparse_bitset.list_to_set(string.to_char_list(S)),
+    R = re(sparse_bitset.difference(valid_unicode_chars, ExcludedChars)).
 
 ?(R) = (R or null).
 
 +(R) = (R ++ *(R)).
 
+range(Start, End) = re(charset(char.to_int(Start), char.to_int(End))).
+
+R * N = Result :-
+    ( if N < 0 then
+        unexpected($pred, "N must be a non-negative number")
+    else if N = 0 then
+        Result = null
+    else if N = 1 then
+        Result = re(R)
+    else
+        Result = conc(re(R), (R * (N - 1)))
+    ).
+
 %-----------------------------------------------------------------------------%
+%
 % Some useful single-char regexps.
+%
 
     % We invite the compiler to memo the values of these constants that
     % (a) are likely to be quite common in practice and (b) take *some*
     % time to compute.
     %
-:- pragma memo(digit/0).
-:- pragma memo(lower/0).
-:- pragma memo(upper/0).
-:- pragma memo(wspc/0).
-:- pragma memo(dot/0).
+:- pragma memo(func(digit/0)).
+:- pragma memo(func(lower/0)).
+:- pragma memo(func(upper/0)).
+:- pragma memo(func(wspc/0)).
+:- pragma memo(func(dot/0)).
 
 digit      = any("0123456789").
 lower      = any("abcdefghijklmnopqrstuvwxyz").
 upper      = any("ABCDEFGHIJKLMNOPQRSTUVWXYZ").
 wspc       = any(" \t\n\r\f\v").
-dot        = anybut("\n").
+dot        = anybut("\r\n").
 alpha      = (lower or upper).
 alphanum   = (alpha or digit).
 identstart = (alpha or ('_')).
 ident      = (alphanum or ('_')).
-nl         = re('\n').
 tab        = re('\t').
 spc        = re(' ').
 
 %-----------------------------------------------------------------------------%
 % Some useful compound regexps.
 
+nl         = (?('\r') ++ '\n').  % matches both Posix and Windows newline.
 nat        = +(digit).
 signed_int = ?("+" or "-") ++ nat.
 real       = signed_int ++ (
