@@ -1,9 +1,12 @@
 %-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et
+% vim: ft=mercury ts=4 sw=4 et ft=mercury
 %-----------------------------------------------------------------------------%
+%
 % regex.m
 % Ralph Becket <rafe@cs.mu.oz.au>
 % Copyright (C) 2002, 2006, 2010 The University of Melbourne
+% Copyright (C) 2014, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %
 % This module provides basic string matching and search and replace
 % functionality using regular expressions defined as strings of the
@@ -17,14 +20,15 @@
 % - character classes [:space:] in character sets are not recognised;
 % - special inter-character patterns such as ^, $, \<, \> are not recognised;
 % - to include a literal `-' in a character set, use the range `---' or
-% include `-' as one end-point of a range (e.g. [!--]);
+%   include `-' as one end-point of a range (e.g. [!--]);
 % - regex will complain if `-' appears other than as an end-point
-% of a range or the range delimiter in a character set;
+%   of a range or the range delimiter in a character set;
 % - regex is a little more sensible about including `[' and `]' in character
-% sets - either character can appear as and end-point of a range;
+%   sets - either character can appear as and end-point of a range;
 % - literal `)' must be backslash escaped, even in the absence of an `(';
 % - `.' matches any character except `\n';
 % - [^...] matches any character not in ... or `\n'.
+%
 % NOTE: these minor differences may go away in a future revision of this
 % module.
 %
@@ -49,13 +53,13 @@
     % suitable for the string matching operations provided in this
     % module.
     %
-    % An exception is thrown if the regex string is malformed.  We
-    % memoize this function for efficiency (it is cheaper to look up a
+    % An exception is thrown if the regex string is malformed.
+    % We memoize this function for efficiency (it is cheaper to look up a
     % string in a hash table than to parse it and recompute the regex.)
     %
-    % A regex string obeys the following grammar.  Note that alternation
+    % A regex string obeys the following grammar. Note that alternation
     % has lowest priority, followed by concatenation, followed by
-    % *, + and ?.  Hence "ab*" is equivalent to "a(b*)" and not "(ab)*"
+    % *, + and ?. Hence "ab*" is equivalent to "a(b*)" and not "(ab)*"
     % while "ab|cd" is equivalent to "(ab)|(cd)" and not "a(b|c)d".
     %
     % <regex> ::= <char>                % Single char
@@ -70,7 +74,7 @@
     %
     % (Note the need to use double-backslashes in Mercury strings.
     % The following chars must be escaped if they are intended
-    % literally: .|*+?()[]\.  Escapes should not be used in sets.)
+    % literally: .|*+?()[]\. Escapes should not be used in sets.)
     %
     % <char>   ::= \<any char>          % Literal char used in regexes
     %           |  <ordinary char>      % All others
@@ -78,7 +82,7 @@
     % <escaped char> ::= . | | | * | + | ? | ( | ) | [ | ] | \
     %
     % (If the first char in a <set'> is ] then it is taken as part of
-    % the char set and not the closing bracket.  Similarly, ] may appear
+    % the char set and not the closing bracket. Similarly, ] may appear
     % as the end char in a range and it will not be taken as the closing
     % bracket.)
     %
@@ -89,8 +93,7 @@
     %           |  <any char>           % Literal char
     %           |  <set'><set'>         % Set union
     %
-:- func regex(string) = regex.
-:- mode regex(in    ) = out(regex) is det.
+:- func regex(string::in) = (regex::out(regex)) is det.
 
     % This is a utility function for lex - it compiles a string into a
     % regexp (not a regex, which is for use with this module) suitable
@@ -110,8 +113,8 @@
     %       {Substring, Start, Count} = head(matches(Regex, String)),
     %       Start = 0
     %
-:- pred left_match(regex,     string, string, int, int).
-:- mode left_match(in(regex), in,     out,    out, out) is semidet.
+:- pred left_match(regex::in(regex), string::in, string::out,
+    int::out, int::out) is semidet.
 
     % right_match(Regex, String, Substring, Start, Count)
     %   succeeds iff Regex maximally matches the last Count characters
@@ -122,8 +125,8 @@
     %       {Substring, Start, Count} = last(matches(Regex, String)),
     %       Start + Count = length(String)
     %
-:- pred right_match(regex,     string, string, int, int).
-:- mode right_match(in(regex), in,     out,    out, out) is semidet.
+:- pred right_match(regex::in(regex), string::in, string::out,
+    int::out, int::out) is semidet.
 
     % first_match(Regex, String, Substring, Start, Count)
     %   succeeds iff Regex matches some Substring of String,
@@ -134,50 +137,51 @@
     %
     %       {Substring, Start, Count} = head(matches(Regex, String))
     %
-:- pred first_match(regex,     string, string, int, int).
-:- mode first_match(in(regex), in,     out,    out, out) is semidet.
+:- pred first_match(regex::in(regex), string::in, string::out,
+    int::out, int::out) is semidet.
 
     % exact_match(Regex, String)
     %   succeeds iff Regex exactly matches String.
     %
-:- pred exact_match(regex,     string).
-:- mode exact_match(in(regex), in    ) is semidet.
+:- pred exact_match(regex::in(regex), string::in) is semidet.
 
     % matches(Regex, String) = [{Substring, Start, Count}, ...]
     %   Regex exactly matches Substring = substring(String, Start, Count).
     %   None of the {Start, Count} regions will overlap and are in
     %   ascending order with respect to Start.
     %
-:- func matches(regex,     string) = list({string, int, int}).
-:- mode matches(in(regex), in    ) = out is det.
+:- func matches(regex::in(regex), string::in) =
+    (list({string, int, int})::out) is det.
 
     % replace_first(Regex, Replacement, String)
     %   computes the string formed by replacing the maximal first match
     %   of Regex (if any) in String with Replacement.
     %
-:- func replace_first(regex,     string, string) = string.
-:- mode replace_first(in(regex), in,     in    ) = out is det.
+:- func replace_first(regex::in(regex), string::in, string::in)
+    = (string::out) is det.
 
     % replace_all(Regex, Replacement, String)
     %   computes the string formed by replacing the maximal non-overlapping
     %   matches of Regex in String with Replacement.
     %
-:- func replace_all(regex,     string, string) = string.
-:- mode replace_all(in(regex), in,     in    ) = out is det.
+:- func replace_all(regex::in(regex), string::in, string::in)
+    = (string::out) is det.
 
     % change_first(Regex, ChangeFn, String)
     %   computes the string formed by replacing the maximal first match
     %   of Regex (Substring, if any) in String with ChangeFn(Substring).
     %
-:- func change_first(regex,     func(string) = string,     string) = string.
-:- mode change_first(in(regex), func(in    ) = out is det, in    ) = out is det.
+:- func change_first(regex::in(regex),
+    (func(string) = string)::in(func(in) = out is det), string::in)
+    = (string::out) is det.
 
     % change_all(Regex, ChangeFn, String)
     %   computes the string formed by replacing the maximal non-overlapping
     %   matches of Regex, Substring, in String with ChangeFn(Substring).
     %
-:- func change_all(regex,     func(string) = string,     string) = string.
-:- mode change_all(in(regex), func(in    ) = out is det, in    ) = out is det.
+:- func change_all(regex::in(regex),
+    (func(string) = string)::in(func(in) = out is det), string::in)
+    = (string::out) is det.
 
 %-----------------------------------------------------------------------------%
 %-----------------------------------------------------------------------------%
@@ -194,13 +198,10 @@
 
 %-----------------------------------------------------------------------------%
 
-
 :- type regex == lexer(string, string).
 
 :- type lexer_state == lexer_state(string, string).
 
-
-
     % This states of the regex parser.
     %
 :- type parser_state
@@ -232,64 +233,68 @@ regex(S) = init([regexp(S) - id], read_from_string).
 
 :- pragma memo(regexp/1).
 
-regexp(S) = finish_regex(S, foldl(compile_regex(S), S, res([]))).
+regexp(S) = finish_regex(S, string.foldl(compile_regex(S), S, res([]))).
 
 %-----------------------------------------------------------------------------%
 
 :- func compile_regex(string, char, parser_state) = parser_state.
 
-    % res: we are looking for the next regex or operator.
-    %
-compile_regex(S, C, res(REs)) =
-    (      if C = ('.')  then res([re(dot) | REs])
-      else if C = ('|')  then res(alt(S, REs))
-      else if C = ('*')  then res(star(S, REs))
-      else if C = ('+')  then res(plus(S, REs))
-      else if C = ('?')  then res(opt(S, REs))
-      else if C = ('(')  then res([lpar | REs])
-      else if C = (')')  then res(rpar(S, REs))
-      else if C = ('[')  then set1(REs)
-      else if C = (']')  then regex_error("`]' without opening `['", S)
-      else if C = ('\\') then esc(REs)
-      else                    res([char(re(C)) | REs])
-    ).
-
-    % esc: the current char has been \ escaped.
-    %
-compile_regex(_, C, esc(REs)) =
-    res([char(re(C)) | REs]).
-
-    % set1: we have just seen the opening [.
-    %
-compile_regex(_, C, set1(REs)) =
-    (      if C = ('^') then set2(yes, REs)
-      else                   set3(no,  C, [], REs)
-    ).
-
-    % set2: we are looking for the first char in the set, which may
-    % include ].
-    %
-compile_regex(_, C, set2(Complement, REs)) =
-                             set3(Complement, C, [], REs).
-
-    % set3: we are looking for a char or - or ].
-    %
-compile_regex(_, C, set3(Complement, C0, Cs, REs)) =
-    (      if C = (']') then res([char_set(Complement, [C0 | Cs]) | REs])
-      else if C = ('-') then set4(Complement, C0, Cs, REs)
-      else                   set3(Complement, C, [C0 | Cs], REs)
-    ).
-
-    % set4: we have just seen a `-' for a range.
-    %
-compile_regex(_, C, set4(Complement, C0, Cs, REs)) =
-                             set5(Complement, push_range(C0, C, Cs), REs).
-
-    % set5: we are looking for a char or ].
-    %
-compile_regex(_, C, set5(Complement, Cs, REs)) =
-    (      if C = (']') then res([char_set(Complement, Cs) | REs])
-      else                   set3(Complement, C, Cs, REs)
+compile_regex(S, C, ParserState0) = ParserState :-
+    (
+        ParserState0 = res(REs),
+        % res: we are looking for the next regex or operator.
+        (      if C = ('.')  then ParserState = res([re(dot) | REs])
+          else if C = ('|')  then ParserState = res(alt(S, REs))
+          else if C = ('*')  then ParserState = res(star(S, REs))
+          else if C = ('+')  then ParserState = res(plus(S, REs))
+          else if C = ('?')  then ParserState = res(opt(S, REs))
+          else if C = ('(')  then ParserState = res([lpar | REs])
+          else if C = (')')  then ParserState = res(rpar(S, REs))
+          else if C = ('[')  then ParserState = set1(REs)
+          else if C = (']')  then ParserState =
+                                    regex_error("`]' without opening `['", S)
+          else if C = ('\\') then ParserState = esc(REs)
+          else                    ParserState = res([char(re(C)) | REs])
+        )
+    ;
+        ParserState0 = esc(REs),
+        % esc: the current char has been \ escaped.
+        ParserState = res([char(re(C)) | REs])
+    ;
+        ParserState0 = set1(REs),
+        % set1: we have just seen the opening [.
+        ( if C = ('^') then
+            ParserState = set2(yes, REs)
+        else
+            ParserState = set3(no,  C, [], REs)
+        )
+    ;
+        ParserState0 = set2(Complement, REs),
+        % set2: we are looking for the first char in the set,
+        % which may include ].
+        ParserState = set3(Complement, C, [], REs)
+    ;
+        ParserState0 = set3(Complement, C0, Cs, REs),
+        % set3: we are looking for a char or - or ].
+        ( if C = (']') then
+            ParserState = res([char_set(Complement, [C0 | Cs]) | REs])
+        else if C = ('-') then
+            ParserState = set4(Complement, C0, Cs, REs)
+        else
+            ParserState = set3(Complement, C, [C0 | Cs], REs)
+        )
+    ;
+        ParserState0 = set4(Complement, C0, Cs, REs),
+        % set4: we have just seen a `-' for a range.
+        ParserState = set5(Complement, push_range(C0, C, Cs), REs)
+    ;
+        ParserState0 = set5(Complement, Cs, REs),
+        % set5: we are looking for a char or ].
+        ( if C = (']') then
+            ParserState = res([char_set(Complement, Cs) | REs])
+        else
+            ParserState = set3(Complement, C, Cs, REs)
+        )
     ).
 
 %-----------------------------------------------------------------------------%
@@ -312,7 +317,6 @@ push_range(A, B, Cs) = Rg ++ Cs :-
     Hi = max(to_int(A), to_int(B)),
     Rg = map(int_to_char, Lo `..` Hi).
 
-
 :- func int_to_char(int) = char.
 
 int_to_char(X) =
@@ -341,9 +345,10 @@ finish_regex(S, set5(_, _, _)) =
     regex_error("`[' without closing `]'", S).
 
 finish_regex(S, res(REs)) =
-    ( if   rpar(S, REs ++ [lpar]) = [RE]
-      then extract_regex(RE)
-      else regex_error("`(' without closing `)'", S)
+    ( if rpar(S, REs ++ [lpar]) = [RE] then
+        extract_regex(RE)
+    else
+        regex_error("`(' without closing `)'", S)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -353,25 +358,28 @@ finish_regex(S, res(REs)) =
 :- func star(string, list(re)) = list(re).
 
 star(S, REs) =
-    ( if   ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] )
-      then [re(*(RE)) | REs0]
-      else regex_error("`*' without preceding regex", S)
+    ( if ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] ) then
+        [re(*(RE)) | REs0]
+    else
+        regex_error("`*' without preceding regex", S)
     ).
 
 :- func plus(string, list(re)) = list(re).
 
 plus(S, REs) =
-    ( if   ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] )
-      then [re(+(RE)) | REs0]
-      else regex_error("`+' without preceding regex", S)
+    ( if ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] ) then
+        [re(+(RE)) | REs0]
+    else
+        regex_error("`+' without preceding regex", S)
     ).
 
 :- func opt(string, list(re)) = list(re).
 
 opt(S, REs) =
-    ( if   ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] )
-      then [re(?(RE)) | REs0]
-      else regex_error("`?' without preceding regex", S)
+    ( if ( REs = [re(RE) | REs0] ; REs = [char(RE) | REs0] ) then
+        [re(?(RE)) | REs0]
+    else
+        regex_error("`?' without preceding regex", S)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -381,25 +389,20 @@ opt(S, REs) =
 :- func alt(string, list(re)) = list(re).
 
 alt(S, REs) =
-    (      if REs =   [alt(_)                                         | _   ]
-      then    regex_error("`|' immediately following `|'", S)
-
-      else if REs =   [lpar                                           | _   ]
-      then    regex_error("`|' immediately following `('", S)
-
-      else if REs =   [RE_B, alt(RE_A)                                | REs0]
-      then            [alt(RE_A or extract_regex(RE_B))               | REs0]
-
-      else if REs =   [RE, lpar                                       | REs0]
-      then            [alt(extract_regex(RE)), lpar                   | REs0]
-
-      else if REs =   [RE_B, RE_A                                     | REs0]
-      then    alt(S,  [re(extract_regex(RE_A) ++ extract_regex(RE_B)) | REs0])
-
-      else if REs =   [RE]
-      then            [alt(extract_regex(RE))]
-
-      else regex_error("`|' without preceding regex", S)
+    ( if REs = [alt(_) | _] then
+        regex_error("`|' immediately following `|'", S)
+    else if REs = [lpar | _] then
+        regex_error("`|' immediately following `('", S)
+    else if REs = [RE_B, alt(RE_A)| REs0] then
+        [alt(RE_A or extract_regex(RE_B)) | REs0]
+    else if REs =   [RE, lpar | REs0] then
+        [alt(extract_regex(RE)), lpar | REs0]
+    else if REs = [RE_B, RE_A | REs0] then
+        alt(S,  [re(extract_regex(RE_A) ++ extract_regex(RE_B)) | REs0])
+    else if REs = [RE] then
+        [alt(extract_regex(RE))]
+    else
+        regex_error("`|' without preceding regex", S)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -409,22 +412,18 @@ alt(S, REs) =
 :- func rpar(string, list(re)) = list(re).
 
 rpar(S, REs) =
-    (      if REs =   [alt(_)                                         | _   ]
-      then    regex_error("`)' immediately following `|'", S)
-
-      else if REs =   [RE_B, alt(RE_A)                                | REs0]
-      then    rpar(S, [re(RE_A or extract_regex(RE_B))                | REs0])
-
-      else if REs =   [lpar                                           | REs0]
-      then            [nil                                            | REs0]
-
-      else if REs =   [RE, lpar                                       | REs0]
-      then            [RE                                             | REs0]
-
-      else if REs =   [RE_B, RE_A                                     | REs0]
-      then    rpar(S, [re(extract_regex(RE_A) ++ extract_regex(RE_B)) | REs0])
-
-      else    regex_error("`)' without opening `('", S)
+    ( if REs = [alt(_) | _] then
+        regex_error("`)' immediately following `|'", S)
+    else if REs = [RE_B, alt(RE_A) | REs0] then
+        rpar(S, [re(RE_A or extract_regex(RE_B)) | REs0])
+    else if REs = [lpar | REs0] then
+        [nil  | REs0]
+    else if REs = [RE, lpar | REs0] then
+        [RE   | REs0]
+    else if REs = [RE_B, RE_A | REs0] then
+        rpar(S, [re(extract_regex(RE_A) ++ extract_regex(RE_B)) | REs0])
+    else
+        regex_error("`)' without opening `('", S)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -467,20 +466,19 @@ left_match(Regex, String, Substring, 0, length(Substring)) :-
     % we find a complete match.
     %
 right_match(Regex, String, Substring, Start, length(Substring)) :-
-    right_match_2(Regex, String, 0, length(String), Substring, Start).
+    right_match_2(Regex, String, length(String), 0, Start, Substring).
 
+:- pred right_match_2(regex::in(regex), string::in, int::in,
+    int::in, int::out, string::out) is semidet.
 
-:- pred right_match_2(regex,     string, int, int, string, int).
-:- mode right_match_2(in(regex), in,     in,  in,  out,    out) is semidet.
-
-right_match_2(Regex, String, I, Length, Substring, Start) :-
-    I =< Length,
-    Substring0 = substring(String, I, max_int),
+right_match_2(Regex, String, Length, !Start, Substring) :-
+    !.Start =< Length,
+    Substring0 = right_by_codepoint(String, Length - !.Start),
     ( if exact_match(Regex, Substring0) then
-        Substring = Substring0,
-        Start     = I
+        Substring = Substring0
       else
-        right_match_2(Regex, String, I + 1, Length, Substring, Start)
+        !:Start = !.Start + 1,
+        right_match_2(Regex, String, Length, !Start, Substring)
     ).
 
 %-----------------------------------------------------------------------------%
@@ -489,13 +487,11 @@ first_match(Regex, String, Substring, Start, length(Substring)) :-
     State = start(Regex, unsafe_promise_unique(String)),
     first_match_2(Substring, Start, State).
 
-
-:- pred first_match_2(string, int, lexer_state).
-:- mode first_match_2(out,    out, di         ) is semidet.
+:- pred first_match_2(string::out, int::out, lexer_state::di) is semidet.
 
 first_match_2(Substring, Start, !.State) :-
     lex.offset_from_start(Start0, !State),
-    lex.read(Result,         !State),
+    lex.read(Result, !State),
     (
         Result = error(_, _),
         first_match_2(Substring, Start, !.State)
@@ -516,9 +512,8 @@ matches(Regex, String) = Matches :-
     State   = start(Regex, unsafe_promise_unique(String)),
     Matches = matches_2(length(String), -1, State).
 
-
-:- func matches_2(int, offset, lexer_state) = list({string, int, int}).
-:- mode matches_2(in,  in,     di)          = out is det.
+:- func matches_2(int::in, offset::in, lexer_state::di)
+    = (list({string, int, int})::out) is det.
 
 matches_2(Length, LastEnd, State0) = Matches :-
     lex.offset_from_start(Start0, State0, State1),
@@ -535,44 +530,38 @@ matches_2(Length, LastEnd, State0) = Matches :-
         Start   = Start0,
         Count   = End - Start,
 
-            % If we matched the empty string then we have to advance
-            % at least one char (and finish if we get eof.)
-            %
-            % If we've reached the end of the input then also finish
-            % (this avoids the situation where, say, ".*" produces
-            % two matches for "foo" - "foo" and the notional null string
-            % at the end.)
-            %
-            % If we matched the empty string at the same point the
-            % last match ended, then we ignore this solution and
-            % move on.
-            %
-        Matches =
-            ( if Count = 0, Start = LastEnd then
-
-                    % This is an empty match at the same point as the end
-                    % of our last match.  We have to ignore it and move on.
-                    %
-                ( if   lex.read_char(ok(_), State3, State4)
-                  then matches_2(Length, End, State4)
-                  else []
-                )
-
-              else
-
-                [ {Substring, Start, Count} |
-                  ( if End = Length then
-                        []
-                    else if Count = 0 then
-                        ( if   lex.read_char(ok(_), State3, State4)
-                          then matches_2(Length, End, State4)
-                          else []
-                        )
-                    else
-                      matches_2(Length, End, State3)
-                  )
-                ]
+        % If we matched the empty string then we have to advance
+        % at least one char (and finish if we get eof.)
+        %
+        % If we have reached the end of the input then also finish
+        % (this avoids the situation where, say, ".*" produces two matches
+        % for "foo" - "foo" and the notional null string at the end.)
+        %
+        % If we matched the empty string at the same point the last
+        % match ended, then we ignore this solution and move on.
+        ( if Count = 0, Start = LastEnd then
+            % This is an empty match at the same point as the end
+            % of our last match. We have to ignore it and move on.
+            ( if lex.read_char(ok(_), State3, State4) then
+                Matches = matches_2(Length, End, State4)
+            else
+                Matches = []
             )
+        else
+            FirstMatch = {Substring, Start, Count},
+            ( if End = Length then
+                LaterMatches = []
+            else if Count = 0 then
+                ( if lex.read_char(ok(_), State3, State4) then
+                    LaterMatches = matches_2(Length, End, State4)
+                else
+                    LaterMatches = []
+                )
+            else
+                LaterMatches = matches_2(Length, End, State3)
+            ),
+            Matches = [FirstMatch | LaterMatches]
+        )
     ).
 
 %-----------------------------------------------------------------------------%
@@ -590,9 +579,9 @@ replace_all(Regex, Replacement, String) =
 change_first(Regex, ChangeFn, String) =
     ( if first_match(Regex, String, Substring, Start, Count) then
         append_list([
-            substring(String, 0, Start),
+            left_by_codepoint(String, Start),
             ChangeFn(Substring),
-            substring(String, Start + Count, max_int)
+            between(String, Start + Count, max_int)
         ])
       else
         String
@@ -603,17 +592,15 @@ change_first(Regex, ChangeFn, String) =
 change_all(Regex, ChangeFn, String) =
     append_list(change_all_2(String, ChangeFn, 0, matches(Regex, String))).
 
-
 :- func change_all_2(string, func(string) = string, int,
-            list({string, int, int})) = list(string).
+    list({string, int, int})) = list(string).
 
 change_all_2(String, _ChangeFn, I, []) =
-    [ substring(String, I, max_int) ].
-
+    [between(String, I, max_int)].
 change_all_2(String, ChangeFn, I, [{Substring, Start, Count} | Matches]) =
-    [ substring(String, I, Start - I),
+    [between(String, I, Start),
       ChangeFn(Substring)
-    | change_all_2(String, ChangeFn, Start + Count, Matches) ].
+    | change_all_2(String, ChangeFn, Start + Count, Matches)].
 
 %-----------------------------------------------------------------------------%
 :- end_module regex.
