@@ -2,8 +2,8 @@
 % vim: ft=mercury ts=4 sw=4 et
 %-----------------------------------------------------------------------------%
 % Copyright (C) 2005-2006 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %-----------------------------------------------------------------------------%
 % any_array.m
 % Ralph Becket <rafe@cs.mu.oz.au>
@@ -300,7 +300,6 @@
 :- use_module    any_list.
 :- import_module exception.
 :- import_module int.
-:- import_module list.
 :- import_module require.
 :- import_module string.
 
