@@ -2,8 +2,8 @@
 % vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
 %-----------------------------------------------------------------------------%
 % Copyright (C) 1998-2000, 2002-2003, 2005-2007, 2010 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2014, 2016, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %-----------------------------------------------------------------------------%
 %
 % File: var.m
@@ -1046,7 +1046,7 @@ debug_freeze(Msg, X, Pred, Y) :-
     ).
 
 :- /* impure */
-   pred debug_pred(string, pred(T), T) is semidet.
+   pred debug_pred(string, pred(T), T).
 :- mode debug_pred(in, pred(in) is semidet, in) is semidet.
 
     % XXX the `pragma promise_pure' here is a lie,
@@ -1080,7 +1080,7 @@ debug_pred(Msg, Pred, Var) :-
     ).
 
 :- /* impure */
-   pred debug_pred2(string, pred(T1, T2), T1, T2) is semidet.
+   pred debug_pred2(string, pred(T1, T2), T1, T2).
 :- mode debug_pred2(in, pred(in, out) is semidet, in, out) is semidet.
 
     % XXX the `pragma promise_pure' here is a lie,
