@@ -1,83 +1,83 @@
 %---------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
 % Copyright (C) 1994-2007, 2010-2012 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2014-2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %---------------------------------------------------------------------------%
-% 
+%
 % File: builtin.m.
 % Main author: fjh.
 % Stability: low.
-% 
+%
 % This file is automatically imported into every module.
 % It is intended for things that are part of the language,
 % but which are implemented just as normal user-level code
 % rather than with special coding in the compiler.
-% 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module builtin.
 :- interface.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Types
+% Types.
 %
 
-% The types `character', `int', `float', and `string',
+% The types `character', `int', `int8', `int16', `int32', `int64',
+% `uint', `uint8', `uint16', `uint32', `uint64', `float', and `string',
 % and tuple types `{}', `{T}', `{T1, T2}', ...
 % and the types `pred', `pred(T)', `pred(T1, T2)', `pred(T1, T2, T3)', ...
 % and `func(T1) = T2', `func(T1, T2) = T3', `func(T1, T2, T3) = T4', ...
-% are builtin and are implemented using special code in the
-% type-checker.  (XXX TODO: report an error for attempts to redefine
-% these types.)
+% are builtin and are implemented using special code in the type-checker.
 
-    % The type c_pointer can be used by predicates that use the
-    % C interface.
+    % The type c_pointer can be used by predicates that use the C interface.
     %
-    % NOTE: we strongly recommend using a `foreign_type' pragma instead
-    %       of using this type. 
+    % NOTE: We *strongly* recommend using a `foreign_type' pragma instead
+    % of using this type.
     %
 :- type c_pointer.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Insts
+% Insts.
 %
 
-% The standard insts `free', `ground', and `bound(...)' are builtin
-% and are implemented using special code in the parser and mode-checker.
+% The standard insts `free', `ground', and `bound(...)' are builtin and are
+% implemented using special code in the parser and mode-checker.
 %
-% So are the standard unique insts `unique', `unique(...)',
-% `mostly_unique', `mostly_unique(...)', and `clobbered'.
+% So are the standard unique insts `unique', `unique(...)', `mostly_unique',
+% `mostly_unique(...)', and `clobbered'.
 %
 % Higher-order predicate insts `pred(<modes>) is <detism>'
-% and higher-order functions insts `func(<modes>) = <mode> is det'
+% and higher-order function insts `func(<modes>) = <mode> is <detism>'
 % are also builtin.
+%
+% The `any' inst used for constraint solver interfaces is builtin and so are
+% its higher-order variants: `any_pred(<modes>) is <detism>' and
+% `any_func(<modes>) = <mode> is <detism>'.
 
     % The name `dead' is allowed as a synonym for `clobbered'.
-    % Similarly `mostly_dead' is a synonym for `mostly_clobbered'.
+    % Similarly, `mostly_dead' is a synonym for `mostly_clobbered'.
     %
 :- inst dead == clobbered.
 :- inst mostly_dead == mostly_clobbered.
 
-    % The `any' inst used for the constraint solver interface is also
-    % builtin.  The insts `new' and `old' are allowed as synonyms for
-    % `free' and `any', respectively, since some of the literature uses
-    % this terminology.
-    %
-:- inst old == any.
-:- inst new == free.
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Standard modes
+% Standard modes.
 %
 
 :- mode unused == free >> free.
+
+    % This mode is deprecated, use `out' instead.
+    %
 :- mode output == free >> ground.
+
+    % This mode is deprecated, use `in' instead.
+    %
 :- mode input  == ground >> ground.
 
 :- mode in  == ground >> ground.
@@ -88,9 +88,9 @@
 :- mode di(Inst)  == Inst >> clobbered.
 :- mode mdi(Inst) == Inst >> mostly_clobbered.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Unique modes
+% Unique modes.
 %
 
 % XXX These are still not fully implemented.
@@ -107,9 +107,9 @@
     %
 :- mode di == unique >> clobbered.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% "Mostly" unique modes
+% "Mostly" unique modes.
 %
 
 % Unique except that they may be referenced again on backtracking.
@@ -126,25 +126,19 @@
     %
 :- mode mdi == mostly_unique >> mostly_clobbered.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Dynamic modes
+% Dynamic modes.
 %
-    
+
     % Solver type modes.
     %
 :- mode ia == any >> any.
 :- mode oa == free >> any.
 
-    % The modes `no' and `oo' are allowed as synonyms, since some of the
-    % literature uses this terminology.
-    %
-:- mode no == new >> old.
-:- mode oo == old >> old.
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
-% Predicates
+% Predicates.
 %
 
     % copy/2 makes a deep copy of a data structure.
@@ -157,14 +151,14 @@
 
     % unsafe_promise_unique/2 is used to promise the compiler that you
     % have a `unique' copy of a data structure, so that you can use
-    % destructive update.  It is used to work around limitations in
+    % destructive update. It is used to work around limitations in
     % the current support for unique modes.
     % `unsafe_promise_unique(X, Y)' is the same as `Y = X' except that
     % the compiler will assume that `Y' is unique.
     %
-    % Note that misuse of this predicate may lead to unsound results: if
-    % there is more than one reference to the data in question, i.e. it is
-    % not `unique', then the behaviour is undefined.
+    % Note that misuse of this predicate may lead to unsound results:
+    % if there is more than one reference to the data in question,
+    % i.e. it is not `unique', then the behaviour is undefined.
     % (If you lie to the compiler, the compiler will get its revenge!)
     %
 :- func unsafe_promise_unique(T::in) = (T::uo) is det.
@@ -175,24 +169,24 @@
     %
 :- pred false is failure.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % This function is useful for converting polymorphic non-solver type
     % values with inst any to inst ground (the compiler recognises that
     % inst any is equivalent to ground for non-polymorphic non-solver
     % type values.)
     %
-    % Do not call this on solver type values unless you are absolutely
-    % sure that they are semantically ground.
+    % Do not call this on solver type values unless you are *absolutely sure*
+    % that they are semantically ground.
     %
 :- func unsafe_cast_any_to_ground(T::ia) = (T::out) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % A call to the function `promise_only_solution(Pred)' constitutes a
     % promise on the part of the caller that `Pred' has at most one
     % solution, i.e. that `not some [X1, X2] (Pred(X1), Pred(X2), X1 \=
-    % X2)'.  `promise_only_solution(Pred)' presumes that this assumption is
+    % X2)'. `promise_only_solution(Pred)' presumes that this assumption is
     % satisfied, and returns the X for which Pred(X) is true, if there is
     % one.
     %
@@ -200,12 +194,13 @@
     % `cc_multi' or `cc_nondet' code inside a `det' or `semidet' procedure.
     %
     % Note that misuse of this function may lead to unsound results: if the
-    % assumption is not satisfied, the behaviour is undefined.  (If you lie
+    % assumption is not satisfied, the behaviour is undefined. (If you lie
     % to the compiler, the compiler will get its revenge!)
     %
-    % NOTE: we recommend using the a `promise_equivalent_solutions' goal
-    %       instead of this function.
+    % NOTE: This function is deprecated and will be removed in a future
+    % release. Use a `promise_equivalent_solutions' goal instead.
     %
+:- pragma obsolete(func(promise_only_solution/1)).
 :- func promise_only_solution(pred(T)) = T.
 :- mode promise_only_solution(pred(out) is cc_multi) = out is det.
 :- mode promise_only_solution(pred(uo) is cc_multi) = uo is det.
@@ -223,17 +218,18 @@
     % IO0, IO)' is true.
     %
     % Note that misuse of this predicate may lead to unsound results: if
-    % the assumption is not satisfied, the behaviour is undefined.  (If you
+    % the assumption is not satisfied, the behaviour is undefined. (If you
     % lie to the compiler, the compiler will get its revenge!)
     %
-    % NOTE: we recommend using a `promise_equivalent_solutions' goal
-    %       instead of this predicate.
+    % NOTE: This predicate is deprecated and will be removed in a future
+    % release. Use a `promise_equivalent_solutions' goal instead.
     %
+:- pragma obsolete(pred(promise_only_solution_io/4)).
 :- pred promise_only_solution_io(
     pred(T, IO, IO)::in(pred(out, di, uo) is cc_multi), T::out,
     IO::di, IO::uo) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % unify(X, Y) is true iff X = Y.
     %
@@ -256,9 +252,10 @@
     % X is =, <, or > Y in the standard ordering.
     %
 :- pred compare(comparison_result, T, T).
-    % Note to implementors: the modes must appear in this order:
-    % compiler/higher_order.m depends on it, as does
-    % compiler/simplify.m (for the inequality simplification.)
+    % NOTE_TO_IMPLEMENTORS The modes must appear in this order:
+    % NOTE_TO_IMPLEMENTORS compiler/higher_order.m depends on it, as does
+    % NOTE_TO_IMPLEMENTORS compiler/simplify.m (for the inequality
+    % NOTE_TO_IMPLEMENTORS simplification.)
 :- mode compare(uo, in, in) is det.
 :- mode compare(uo, ui, ui) is det.
 :- mode compare(uo, ui, in) is det.
@@ -327,7 +324,7 @@
     % Comparison functions are expected to obey analogous laws.
     %
     % Note that binary relations <, > and = can be defined from a
-    % comparison predicate or function in an obvious way.  The following
+    % comparison predicate or function in an obvious way. The following
     % facts about these relations are entailed by the above constraints:
     % = is an equivalence relation (not necessarily the usual equality),
     % and the equivalence classes of this relation are totally ordered
@@ -361,10 +358,10 @@
 %   all Vars pred
 %   call/N
 
-%-----------------------------------------------------------------------------%
-    
+%---------------------------------------------------------------------------%
+
     % `semidet_succeed' is exactly the same as `true', except that
-    % the compiler thinks that it is semi-deterministic.  You can use
+    % the compiler thinks that it is semi-deterministic. You can use
     % calls to `semidet_succeed' to suppress warnings about determinism
     % declarations that could be stricter.
     %
@@ -379,10 +376,10 @@
     %
 :- pred semidet_true is semidet.
 
-    % A synonym for semidet_fail/0
+    % A synonym for semidet_fail/0.
     %
 :- pred semidet_false is semidet.
-    
+
     % `cc_multi_equal(X, Y)' is the same as `X = Y' except that it
     % is cc_multi rather than det.
     %
@@ -398,44 +395,23 @@
     %
 :- semipure pred semipure_true is det.
 
-%-----------------------------------------------------------------------------%
-    
+%---------------------------------------------------------------------------%
+
     % dynamic_cast(X, Y) succeeds with Y = X iff X has the same ground type
     % as Y (so this may succeed if Y is of type list(int), say, but not if
     % Y is of type list(T)).
     %
 :- pred dynamic_cast(T1::in, T2::out) is semidet.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 % Everything below here is not intended to be part of the public interface,
 % and will not be included in the Mercury library reference manual.
 
-% This import is needed by the Mercury clauses for semidet_succeed/0
-% and semidet_fail/0.
-%
-:- import_module int.
-
-%-----------------------------------------------------------------------------%
-
 :- interface.
-    
-    % `get_one_solution' and `get_one_solution_io' are impure alternatives
-    % to `promise_one_solution' and `promise_one_solution_io', respectively.
-    % They get a solution to the procedure, without requiring any promise
-    % that there is only one solution.  However, they can only be used in
-    % impure code.
-    %
-:- impure func get_one_solution(pred(T)) = T.
-:-        mode get_one_solution(pred(out) is cc_multi) = out is det.
-:-        mode get_one_solution(pred(out) is cc_nondet) = out is semidet.
-
-:- impure pred get_one_solution_io(pred(T, IO, IO), T, IO, IO).
-:-        mode get_one_solution_io(pred(out, di, uo) is cc_multi,
-        out, di, uo) is det.
 
     % compare_representation(Result, X, Y):
     %
@@ -457,26 +433,47 @@
 :- pred compare_representation(comparison_result, T, T).
 :- mode compare_representation(uo, in, in) is cc_multi.
 
+    % `get_one_solution' and `get_one_solution_io' are impure alternatives
+    % to `promise_one_solution' and `promise_one_solution_io', respectively.
+    % They get a solution to the procedure, without requiring any promise
+    % that there is only one solution. However, they can only be used in
+    % impure code.
+    %
+:- pragma obsolete(func(get_one_solution/1)).
+:- impure func get_one_solution(pred(T)) = T.
+:-        mode get_one_solution(pred(out) is cc_multi) = out is det.
+:-        mode get_one_solution(pred(out) is cc_nondet) = out is semidet.
+
+:- pragma obsolete(pred(get_one_solution_io/4)).
+:- impure pred get_one_solution_io(pred(T, IO, IO), T, IO, IO).
+:-        mode get_one_solution_io(pred(out, di, uo) is cc_multi,
+        out, di, uo) is det.
+
+    % Set up Mercury runtime to call special predicates implemented in this
+    % module.
+    %
+:- impure pred init_runtime_hooks is det.
+
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
 :- implementation.
 
-%-----------------------------------------------------------------------------%
+    % This import is needed by the Mercury clause for semidet_fail/0,
+    % and for e.g. {unify,compare}_tuple_pos.
+    %
+:- import_module int.
+
+%---------------------------------------------------------------------------%
 
 false :-
     fail.
 
-%-----------------------------------------------------------------------------%
-
-% NOTE: dynamic_cast/2 is handled specially compiler/const_prop.m.
-% Any changes here may need to be reflected here.
-
-dynamic_cast(X, Y) :-
-    private_builtin.typed_unify(X, Y).
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % XXX The calls to unsafe_promise_unique below work around
     % mode checker limitations.
-:- pragma promise_pure(promise_only_solution/1).
+:- pragma promise_pure(func(promise_only_solution/1)).
 promise_only_solution(CCPred::(pred(out) is cc_multi)) = (OutVal::out) :-
     impure OutVal = get_one_solution(CCPred).
 promise_only_solution(CCPred::(pred(uo) is cc_multi)) = (OutVal::uo) :-
@@ -488,34 +485,21 @@ promise_only_solution(CCPred::(pred(uo) is cc_nondet)) = (OutVal::uo) :-
     impure OutVal0 = get_one_solution(CCPred),
     OutVal = unsafe_promise_unique(OutVal0).
 
-get_one_solution(CCPred) = OutVal :-
-    promise_equivalent_solutions [OutVal] (
-        CCPred(OutVal),
-        impure impure_true
-    ).
-
-:- pragma promise_pure(promise_only_solution_io/4).
+:- pragma promise_pure(pred(promise_only_solution_io/4)).
 promise_only_solution_io(Pred, X, !IO) :-
     impure get_one_solution_io(Pred, X, !IO).
 
-get_one_solution_io(Pred, X, !IO) :-
-    promise_equivalent_solutions [!:IO, X] (
-        Pred(X, !IO),
-        impure impure_true
-    ).
+%---------------------------------------------------------------------------%
 
-%-----------------------------------------------------------------------------%
-%
 % IMPORTANT: any changes or additions to external predicates should be
 % reflected in the definition of pred_is_external in
-% mdbcomp/program_representation.m.  The debugger needs to know what predicates
+% mdbcomp/program_representation.m. The debugger needs to know what predicates
 % are defined externally, so that it knows not to expect events for those
 % predicates.
 %
-
-:- external(unify/2).
-:- external(compare/3).
-:- external(compare_representation/3).
+:- pragma external_pred(unify/2).
+:- pragma external_pred(compare/3).
+:- pragma external_pred(compare_representation/3).
 
 :- pragma foreign_export_enum("C#", comparison_result/0, [],
     [
@@ -542,34 +526,150 @@ get_one_solution_io(Pred, X, !IO) :-
 ordering(X, Y) = R :-
     compare(R, X, Y).
 
-    % simplify.goal automatically inlines these definitions.
-    %
-X  @< Y :-
+% simplify_goal_call.m automatically inlines these definitions.
+
+X @< Y :-
     compare((<), X, Y).
+
 X @=< Y :-
     not compare((>), X, Y).
+
 X @>  Y :-
     compare((>), X, Y).
+
 X @>= Y :-
     not compare((<), X, Y).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%
+% Unify/compare of tuples.
+%
 
-:- pragma foreign_decl("C", "#include ""mercury_type_info.h""").
+% We implement these predicates in Mercury mainly to allow the compiler
+% to perform the deep profiling transformation on them. init_runtime_hooks
+% sets fields in `MR_special_pred_hooks' structure to point to the actual
+% implementations, because we do not want the runtime to have unresolved
+% references into the library when it is built.
 
-:- interface.
+:- pragma foreign_decl("C", "#include ""mercury_ho_call.h""").
 
-:- pred call_rtti_generic_unify(T::in, T::in) is semidet.
-:- pred call_rtti_generic_compare(comparison_result::out, T::in, T::in) is det.
+:- pred unify_tuple(T::in, T::in) is semidet.
 
-:- implementation.
-:- use_module erlang_rtti_implementation.
-:- use_module rtti_implementation.
+:- pragma foreign_export("C", unify_tuple(in, in), "ML_unify_tuple").
 
-call_rtti_generic_unify(X, Y) :-
-    rtti_implementation.generic_unify(X, Y).
-call_rtti_generic_compare(Res, X, Y) :-
-    rtti_implementation.generic_compare(Res, X, Y).
+unify_tuple(TermA, TermB) :-
+    tuple_arity(TermA, Arity),
+    unify_tuple_pos(TermA, TermB, 0, Arity).
+
+:- pred unify_tuple_pos(T::in, T::in, int::in, int::in) is semidet.
+
+unify_tuple_pos(TermA, TermB, Index, Arity) :-
+    ( if Index >= Arity then
+        true
+    else
+        tuple_arg(TermA, Index, SubTermA),
+        tuple_arg(TermB, Index, SubTermB),
+        private_builtin.unsafe_type_cast(SubTermB, CastSubTermB),
+        ( if builtin.unify(SubTermA, CastSubTermB) then
+            unify_tuple_pos(TermA, TermB, Index + 1, Arity)
+        else
+            fail
+        )
+    ).
+
+:- pred compare_tuple(comparison_result::uo, T::in, T::in) is det.
+
+:- pragma foreign_export("C", compare_tuple(uo, in, in), "ML_compare_tuple").
+
+compare_tuple(Result, TermA, TermB) :-
+    tuple_arity(TermA, Arity),
+    compare_tuple_pos(Result, TermA, TermB, 0, Arity).
+
+:- pred compare_tuple_pos(comparison_result::uo, T::in, T::in,
+    int::in, int::in) is det.
+
+compare_tuple_pos(Result, TermA, TermB, Index, Arity) :-
+    ( if Index >= Arity then
+        Result = (=)
+    else
+        tuple_arg(TermA, Index, SubTermA),
+        tuple_arg(TermB, Index, SubTermB),
+        private_builtin.unsafe_type_cast(SubTermB, CastSubTermB),
+        builtin.compare(SubResult, SubTermA, CastSubTermB),
+        (
+            SubResult = (=),
+            compare_tuple_pos(Result, TermA, TermB, Index + 1, Arity)
+        ;
+            ( SubResult = (<)
+            ; SubResult = (>)
+            ),
+            Result = SubResult
+        )
+    ).
+
+:- pred compare_rep_tuple(comparison_result::uo, T::in, T::in) is cc_multi.
+
+:- pragma foreign_export("C", compare_rep_tuple(uo, in, in),
+    "ML_compare_rep_tuple").
+
+compare_rep_tuple(Result, TermA, TermB) :-
+    tuple_arity(TermA, Arity),
+    compare_rep_tuple_pos(Result, TermA, TermB, 0, Arity).
+
+:- pred compare_rep_tuple_pos(comparison_result::uo, T::in, T::in,
+    int::in, int::in) is cc_multi.
+
+compare_rep_tuple_pos(Result, TermA, TermB, Index, Arity) :-
+    ( if Index >= Arity then
+        Result = (=)
+    else
+        tuple_arg(TermA, Index, SubTermA),
+        tuple_arg(TermB, Index, SubTermB),
+        private_builtin.unsafe_type_cast(SubTermB, CastSubTermB),
+        builtin.compare_representation(SubResult, SubTermA, CastSubTermB),
+        (
+            SubResult = (=),
+            compare_rep_tuple_pos(Result, TermA, TermB, Index + 1, Arity)
+        ;
+            ( SubResult = (<)
+            ; SubResult = (>)
+            ),
+            Result = SubResult
+        )
+    ).
+
+:- pred tuple_arity(T::in, int::out) is det.
+:- pragma no_determinism_warning(pred(tuple_arity/2)).
+
+:- pragma foreign_proc("C",
+    tuple_arity(_Term::in, Arity::out),
+    [will_not_call_mercury, promise_pure, thread_safe],
+"
+    Arity = MR_TYPEINFO_GET_VAR_ARITY_ARITY((MR_TypeInfo) TypeInfo_for_T);
+").
+
+tuple_arity(_, _) :-
+    private_builtin.sorry("tuple_arity/2").
+
+:- some [ArgT] pred tuple_arg(T::in, int::in, ArgT::out) is det.
+:- pragma no_determinism_warning(pred(tuple_arg/3)).
+
+:- pragma foreign_proc("C",
+    tuple_arg(Term::in, Index::in, Arg::out),
+    [will_not_call_mercury, promise_pure, thread_safe],
+"
+    MR_TypeInfo type_info = (MR_TypeInfo) TypeInfo_for_T;
+    MR_Word     *arg_vector = (MR_Word *) Term;
+
+    TypeInfo_for_ArgT =
+        (MR_Word) MR_TYPEINFO_GET_VAR_ARITY_ARG_VECTOR(type_info)[1 + Index];
+    Arg = arg_vector[Index];
+").
+
+tuple_arg(_, _, -1) :-
+    private_builtin.sorry("tuple_arg/3").
+
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_code("C#", "
     //
@@ -747,19 +847,17 @@ __Compare____tuple_0_0(object x, object y)
 
 :- pragma foreign_code("Java",
 "
-    /*
-    ** Two other approaches to implement deep copy might be:
-    **
-    ** 1. Get all mercury objects to implement the Serializable interface.
-    **    Then this whole function could be replaced with code that writes
-    **    the Object out via an ObjectOutputStream into a byte array (or
-    **    something), then reads it back in again, thus creating a copy.
-    **    This would copy non-Mercury objects as well, though.
-    **
-    ** 2. Get all mercury objects to implement a clone() method (either the
-    **    one in Object or our own). The MLDS doesn't have method calls
-    **    and probably we wouldn't want to clutter it up just for this.
-    */
+    // Two other approaches to implement deep copy might be:
+    //
+    // 1. Get all mercury objects to implement the Serializable interface.
+    //    Then this whole function could be replaced with code that writes
+    //    the Object out via an ObjectOutputStream into a byte array (or
+    //    something), then reads it back in again, thus creating a copy.
+    //    This would copy non-Mercury objects as well, though.
+    //
+    // 2. Get all mercury objects to implement a clone() method (either the
+    //    one in Object or our own). The MLDS doesn't have method calls
+    //    and probably we wouldn't want to clutter it up just for this.
 
     public static <T> T
     deep_copy(final T original) throws
@@ -769,7 +867,8 @@ __Compare____tuple_0_0(object x, object y)
             return null;
         }
 
-        final java.lang.Class<T> cls = (java.lang.Class<T>) original.getClass();
+        final java.lang.Class<T> cls =
+            (java.lang.Class<T>) original.getClass();
 
         if (cls.isArray()) {
             int length = java.lang.reflect.Array.getLength(original);
@@ -783,8 +882,8 @@ __Compare____tuple_0_0(object x, object y)
             return clone;
         }
 
-        // We'll only copy objects of Mercury-defined types.  We could copy
-        // more but that's what we do for C backends and it's enough.
+        // We'll only copy objects of Mercury-defined types. We could copy
+        // more but that is what we do for C backends and it is enough.
         // We don't copy enumeration instances.
         if (!(original instanceof jmercury.runtime.MercuryType)) {
             return original;
@@ -810,60 +909,11 @@ __Compare____tuple_0_0(object x, object y)
     }
 ").
 
-:- pragma foreign_code("Erlang", "
-
-    '__Compare____c_pointer_0_0'(_, _) ->
-        throw(""called compare/3 for type `c_pointer'"").
-
-    '__Unify____c_pointer_0_0'(_, _) ->
-        throw(""called unify for type `c_pointer'"").
-
-    compare_3_p_0(TypeInfo, X, Y) ->
-        mercury__erlang_rtti_implementation:generic_compare_3_p_0(
-            TypeInfo, X, Y).
-
-    compare_3_p_1(TypeInfo, X, Y) ->
-        compare_3_p_0(TypeInfo, X, Y).
-
-    compare_3_p_2(TypeInfo, X, Y) ->
-        compare_3_p_0(TypeInfo, X, Y).
-
-    compare_3_p_3(TypeInfo, X, Y) ->
-        compare_3_p_0(TypeInfo, X, Y).
-
-    % XXX what is this supposed to do?
-    compare_representation_3_p_0(TypeInfo, X, Y) ->
-        compare_3_p_0(TypeInfo, X, Y).
-
-    unify_2_p_0(TypeInfo, X, Y) ->
-        mercury__erlang_rtti_implementation:generic_unify_2_p_0(
-            TypeInfo, X, Y).
-
-    '__Unify____tuple_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to unify for tuple/0"").
-
-    '__Compare____tuple_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to compare for tuple/0"").
-
-    '__Unify____void_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to unify for void/0"").
-
-    '__Compare____void_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to compare for void/0"").
-
-    '__Unify____func_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to unify for func/0"").
-
-    '__Compare____func_0_0'(X, Y) ->
-        mercury__require:error_1_p_0(""call to compare for func/0"").
-").
-
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 % unsafe_promise_unique is a compiler builtin.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_proc("C",
     copy(Value::ui, Copy::uo),
@@ -925,25 +975,11 @@ __Compare____tuple_0_0(object x, object y)
     }
 ").
 
-:- pragma foreign_proc("Erlang",
-    copy(X::ui, Y::uo),
-    [may_call_mercury, thread_safe, promise_pure, terminates],
-"
-    Y = X
-").
-
-:- pragma foreign_proc("Erlang",
-    copy(X::in, Y::uo),
-    [may_call_mercury, thread_safe, promise_pure, terminates],
-"
-    Y = X
-").
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 %
 % A definition of the Mercury type void/0 is needed because we can generate
-% references to it in code.  See tests/hard_coded/nullary_ho_func.m for an
+% references to it in code. See tests/hard_coded/nullary_ho_func.m for an
 % example of code which does.
 %
 :- pragma foreign_code("C#", "
@@ -968,7 +1004,7 @@ __Compare____tuple_0_0(object x, object y)
     }
 ").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_code("Java", "
 
@@ -1047,14 +1083,16 @@ __Compare____tuple_0_0(object x, object y)
     __Unify____tuple_0_0(java.lang.Object[] x, java.lang.Object[] y)
     {
         // stub only
-        throw new java.lang.Error (""unify/2 for tuple types not implemented"");
+        throw new java.lang.Error (
+            ""unify/2 for tuple types not implemented"");
     }
 
     public static boolean
     __Unify____func_0_0(java.lang.Object[] x, java.lang.Object[] y)
     {
         // stub only
-        throw new java.lang.Error (""unify/2 for tuple types not implemented"");
+        throw new java.lang.Error (
+            ""unify/2 for tuple types not implemented"");
     }
 
 
@@ -1107,7 +1145,7 @@ __Compare____tuple_0_0(object x, object y)
     }
 ").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % semidet_succeed and semidet_fail
 %
@@ -1116,6 +1154,15 @@ __Compare____tuple_0_0(object x, object y)
 % interface to make sure that the compiler doesn't issue any determinism
 % warnings for them.
 
+% We can't just use "true" and "fail" in the Mercury versions, because that
+% provokes warnings from determinism analysis, and the library is compiled
+% with --halt-at-warn. So instead we use 0+0 = (or \=) 0.
+% This is guaranteed to succeed or fail (respectively),
+% and with a bit of luck will even get optimized by constant propagation.
+% But this optimization won't happen until after determinism analysis,
+% which doesn't know anything about integer arithmetic,
+% so this code won't provide a warning from determinism analysis.
+
 :- pragma foreign_proc("C",
     semidet_succeed,
     [will_not_call_mercury, thread_safe, promise_pure,
@@ -1123,6 +1170,16 @@ __Compare____tuple_0_0(object x, object y)
 "
     SUCCESS_INDICATOR = MR_TRUE;
 ").
+:- pragma foreign_proc("C#",
+    semidet_succeed,
+    [will_not_call_mercury, thread_safe, promise_pure],
+"
+    SUCCESS_INDICATOR = true;
+").
+
+semidet_succeed :-
+    0 + 0 = 0.
+
 :- pragma foreign_proc("C",
     semidet_fail,
     [will_not_call_mercury, thread_safe, promise_pure,
@@ -1130,13 +1187,6 @@ __Compare____tuple_0_0(object x, object y)
 "
     SUCCESS_INDICATOR = MR_FALSE;
 ").
-
-:- pragma foreign_proc("C#",
-    semidet_succeed,
-    [will_not_call_mercury, thread_safe, promise_pure],
-"
-    SUCCESS_INDICATOR = true;
-").
 :- pragma foreign_proc("C#",
     semidet_fail,
     [will_not_call_mercury, thread_safe, promise_pure],
@@ -1144,39 +1194,16 @@ __Compare____tuple_0_0(object x, object y)
     SUCCESS_INDICATOR = false;
 ").
 
-:- pragma foreign_proc("Erlang",
-    semidet_succeed,
-    [will_not_call_mercury, thread_safe, promise_pure],
-"
-    SUCCESS_INDICATOR = true
-").
-:- pragma foreign_proc("Erlang",
-    semidet_fail,
-    [will_not_call_mercury, thread_safe, promise_pure],
-"
-    SUCCESS_INDICATOR = false
-").
-
-% We can't just use "true" and "fail" here, because that provokes warnings
-% from determinism analysis, and the library is compiled with --halt-at-warn.
-% So instead we use 0+0 = (or \=) 0.
-% This is guaranteed to succeed or fail (respectively),
-% and with a bit of luck will even get optimized by constant propagation.
-% But this optimization won't happen until after determinism analysis,
-% which doesn't know anything about integer arithmetic,
-% so this code won't provide a warning from determinism analysis.
-
-semidet_succeed :-
-    0 + 0 = 0.
 semidet_fail :-
     0 + 0 \= 0.
 
 semidet_true :-
     semidet_succeed.
+
 semidet_false :-
     semidet_fail.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % cc_multi_equal
 %
@@ -1184,7 +1211,7 @@ semidet_false :-
 % NOTE: cc_multi_equal/2 is handled specially in browser/declarative_tree.m.
 % Any changes here may need to be reflected there.
 
-:- pragma promise_equivalent_clauses(cc_multi_equal/2).
+:- pragma promise_equivalent_clauses(pred(cc_multi_equal/2)).
 
 :- pragma foreign_proc("C",
     cc_multi_equal(X::in, Y::out),
@@ -1227,20 +1254,7 @@ semidet_false :-
     Y = X;
 ").
 
-:- pragma foreign_proc("Erlang",
-    cc_multi_equal(X::in, Y::out),
-    [will_not_call_mercury, thread_safe, promise_pure],
-"
-    Y = X
-").
-:- pragma foreign_proc("Erlang",
-    cc_multi_equal(X::di, Y::uo),
-    [will_not_call_mercury, thread_safe, promise_pure],
-"
-    Y = X
-").
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 impure_true :-
     impure private_builtin.imp.
@@ -1248,7 +1262,15 @@ impure_true :-
 semipure_true :-
     semipure private_builtin.semip.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
+% NOTE: dynamic_cast/2 is handled specially compiler/const_prop.m.
+% Any changes here may need to be reflected here.
+
+dynamic_cast(X, Y) :-
+    private_builtin.typed_unify(X, Y).
+
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_proc("C",
     unsafe_cast_any_to_ground(X::ia) = (Y::out),
@@ -1271,13 +1293,42 @@ semipure_true :-
     Y = X;
 ").
 
-:- pragma foreign_proc("Erlang",
-    unsafe_cast_any_to_ground(X::ia) = (Y::out),
-    [promise_pure, will_not_call_mercury, thread_safe, will_not_modify_trail],
+%---------------------------------------------------------------------------%
+
+get_one_solution(CCPred) = OutVal :-
+    promise_equivalent_solutions [OutVal] (
+        CCPred(OutVal),
+        impure impure_true
+    ).
+
+get_one_solution_io(Pred, X, !IO) :-
+    promise_equivalent_solutions [!:IO, X] (
+        Pred(X, !IO),
+        impure impure_true
+    ).
+
+%---------------------------------------------------------------------------%
+
+init_runtime_hooks.
+
+:- pragma foreign_proc("C",
+    init_runtime_hooks,
+    [will_not_call_mercury, thread_safe, may_not_duplicate],
 "
-    Y = X
+#ifdef MR_HIGHLEVEL_CODE
+    MR_special_pred_hooks.MR_unify_tuple_pred = ML_unify_tuple;
+    MR_special_pred_hooks.MR_compare_tuple_pred = ML_compare_tuple;
+    MR_special_pred_hooks.MR_compare_rep_tuple_pred = ML_compare_rep_tuple;
+#else
+    MR_special_pred_hooks.MR_unify_tuple_pred =
+        MR_ENTRY(mercury__builtin__unify_tuple_2_0);
+    MR_special_pred_hooks.MR_compare_tuple_pred =
+        MR_ENTRY(mercury__builtin__compare_tuple_3_0);
+    MR_special_pred_hooks.MR_compare_rep_tuple_pred =
+        MR_ENTRY(mercury__builtin__compare_rep_tuple_3_0);
+#endif
 ").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module builtin.
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
