@@ -1,10 +1,10 @@
-%-----------------------------------------------------------------------------%
-% vim:ts=4 sw=4 expandtab tw=0 wm=0 ft=mercury
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+% vim:ts=4 sw=4 expandtab ft=mercury
+%---------------------------------------------------------------------------%
 % Copyright (C) 2000-2007, 2010-2011 The University of Melbourne
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
-%-----------------------------------------------------------------------------%
+% Copyright (C) 2014-2018, 2020 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
+%---------------------------------------------------------------------------%
 %
 % File: pprint.m
 % Main author: rafe
@@ -27,6 +27,7 @@
 % terms without thrashing the VM system.
 %
 % Wadler's approach has three main advantages:
+%
 % 1. the layout algebra is small and quite intuitive (more so than Hughes');
 % 2. the pretty printer is optimal in the sense that it will never generate
 %    output that over-runs the specified width unless that is unavoidable; and
@@ -39,7 +40,7 @@
 % allowing line-breaks to be converted into spaces at the pretty
 % printer's discretion) over docs, I have extended the doc type to
 % include a `GROUP' constructor and made the appropriate algorithmic
-% changes.  Because `UNION' only arises as a consequence of processing
+% changes. Because `UNION' only arises as a consequence of processing
 % a 'GROUP' it turns out to be simpler to do away with `UNION'
 % altogether and convert clauses that process `UNION' terms to
 % processing `GROUP's.
@@ -48,7 +49,7 @@
 %
 % (c) The third change is the introduction of the `LABEL' constructor,
 % which acts much like `NEST', except that indentation is defined
-% using a string rather than a number of spaces.  This is useful for,
+% using a string rather than a number of spaces. This is useful for,
 % e.g., multi-line compiler errors and warnings that should be
 % prefixed with the offending source file and line number.
 %
@@ -59,16 +60,16 @@
 % resulting doc is significantly larger than the original term.
 % Worse, any sharing structure in the original term leads to
 % duplicated sub-docs, which can cause an exponential blow-up in the
-% size of the doc w.r.t. the source term.  To get around this problem
+% size of the doc w.r.t. the source term. To get around this problem
 % I have introduced the 'DOC' constructor which causes on-demand
 % conversion of arguments.
 %
 % [This is not true laziness in the sense that the 'DOC', once
-% evaluated, will be overwritten with its value.  This approach would
+% evaluated, will be overwritten with its value. This approach would
 % lead to garbage retention and not solve the page thrashing behaviour
 % otherwise experienced when converting extremely large terms.
-% Instead, each 'DOC' is reevaluated each time it is examined.  This
-% trades off computation time for space.]
+% Instead, each 'DOC' is reevaluated each time it is examined.
+% This trades off computation time for space.]
 %
 % I have added several obvious general purpose formatting functions.
 %
@@ -133,8 +134,8 @@
 %      world
 %
 % 6. label("Look! ", line ++
-%                    group("Goodbye " ++
-%                          nest(3, line ++ group("cruel " ++ line ++ "world"))))
+%        group("Goodbye " ++
+%              nest(3, line ++ group("cruel " ++ line ++ "world"))))
 %
 %   Look! Goodbye cruel world
 %
@@ -145,8 +146,8 @@
 %   Look!    cruel
 %   Look!    world
 %
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module pprint.
 :- interface.
@@ -158,7 +159,7 @@
 :- import_module string.
 :- import_module univ.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Clients must translate data structures into docs for
     % the pretty printer to display.
@@ -186,6 +187,15 @@
 :- instance doc(doc).
 :- instance doc(string).
 :- instance doc(int).
+:- instance doc(int8).
+:- instance doc(int16).
+:- instance doc(int32).
+:- instance doc(int64).
+:- instance doc(uint).
+:- instance doc(uint8).
+:- instance doc(uint16).
+:- instance doc(uint32).
+:- instance doc(uint64).
 :- instance doc(float).
 :- instance doc(char).
 
@@ -242,13 +252,13 @@
     %
 :- func group(T)            = doc <= (doc(T)).
 
-    % This function can be used to convert strings, chars, ints and floats
-    % to their text doc equivalents.
+    % This function can be used to convert strings, chars, ints, uints and
+    % floats to their text doc equivalents.
     %
     % NOTE: since these types are now instances of the doc/1 type class,
     % it is simpler to just apply the doc/1 method to these types.
     %
-:- func poly(string.poly_type) = doc.
+:- func poly(poly_type) = doc.
 
     % Shorthand for doc ++ line ++ doc.
     %
@@ -331,6 +341,9 @@
     % Performs word wrapping at the end of line, taking whitespace sequences
     % as delimiters separating words.
     %
+    % See `char.is_whitespace' for the definition of whitespace characters
+    % used by this predicate.
+    %
 :- func word_wrapped(string) = doc.
 
     % Convert arbitrary terms to docs. This requires std_util.functor/3 to work
@@ -361,8 +374,8 @@
 :- pred write(Stream::in, int::in, T::in, State::di, State::uo) is det
     <= ( doc(T), stream.writer(Stream, string, State) ).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -370,6 +383,10 @@
 :- import_module deconstruct.
 :- import_module enum.
 :- import_module int.
+:- import_module int8.
+:- import_module int16.
+:- import_module int32.
+:- import_module int64.
 :- import_module map.
 :- import_module ops.
 :- import_module pair.
@@ -377,9 +394,13 @@
 :- import_module sparse_bitset.
 :- import_module term.
 :- import_module type_desc.
+:- import_module uint8.
+:- import_module uint16.
+:- import_module uint32.
+:- import_module uint64.
 :- import_module version_array.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type doc
     --->    'NIL'
@@ -406,38 +427,62 @@
     %
 :- type depth == int.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 doc(X) = doc(int.max_int, X).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- instance doc(doc)       where [ doc(_, Doc)    = Doc            ].
-:- instance doc(string)    where [ doc(_, String) = text(String)   ].
-:- instance doc(int)       where [ doc(_, Int)    = poly(i(Int))   ].
-:- instance doc(float)     where [ doc(_, Float)  = poly(f(Float)) ].
-:- instance doc(char)      where [ doc(_, Char)   = poly(c(Char))  ].
+:- instance doc(doc)    where [ doc(_, Doc)    = Doc            ].
+:- instance doc(string) where [ doc(_, String) = text(String)   ].
+:- instance doc(int)    where [ doc(_, Int)    = poly(i(Int))   ].
+:- instance doc(int8)   where [ doc(_, Int8) = text(int8_to_string(Int8))].
+:- instance doc(int16) where [ doc(_, Int16) = text(int16_to_string(Int16))].
+:- instance doc(int32) where [ doc(_, Int32) = text(int32_to_string(Int32))].
+:- instance doc(int64) where [ doc(_, Int64) = text(int64_to_string(Int64))].
+:- instance doc(uint)   where [ doc(_, UInt) = text(uint_to_string(UInt))].
+:- instance doc(uint8)  where [ doc(_, UInt8) = text(uint8_to_string(UInt8))].
+:- instance doc(uint16) where [
+    doc(_, UInt16) = text(uint16_to_string(UInt16))
+].
+:- instance doc(uint32) where [
+    doc(_, UInt32) = text(uint32_to_string(UInt32))
+].
+:- instance doc(uint64) where [
+    doc(_, UInt64) = text(uint64_to_string(UInt64))
+].
+:- instance doc(float)  where [ doc(_, Float)  = poly(f(Float)) ].
+:- instance doc(char)   where [ doc(_, Char)   = poly(c(Char))  ].
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 Doc1 ++ Doc2 = doc(Doc1) `<>` doc(Doc2).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-nil                     = 'NIL'.
-X `<>` Y                = 'SEQ'(X, Y).
-nest(I, X)              = 'NEST'(I, doc(X)).
-label(L, X)             = 'LABEL'(L, doc(X)).
-text(S)                 = 'TEXT'(S).
-line                    = 'LINE'.
-group(X)                = 'GROUP'(doc(X)).
+nil             = 'NIL'.
+X `<>` Y        = 'SEQ'(X, Y).
+nest(I, X)      = 'NEST'(I, doc(X)).
+label(L, X)     = 'LABEL'(L, doc(X)).
+text(S)         = 'TEXT'(S).
+line            = 'LINE'.
+group(X)        = 'GROUP'(doc(X)).
 
-poly(s(S))              = text(string.format("%s", [s(S)])).
-poly(c(C))              = text(string.format("%c", [c(C)])).
-poly(i(I))              = text(string.format("%d", [i(I)])).
-poly(f(F))              = text(string.format("%f", [f(F)])).
+poly(s(S))      = text(string.format("%s", [s(S)])).
+poly(c(C))      = text(string.format("%c", [c(C)])).
+poly(i(I))      = text(string.format("%d", [i(I)])).
+poly(i8(I8))    = text(string.format("%d", [i(int8.cast_to_int(I8))])).
+poly(i16(I16))  = text(string.format("%d", [i(int16.cast_to_int(I16))])).
+poly(i32(I32))  = text(string.format("%d", [i(int32.cast_to_int(I32))])).
+poly(i64(I64))  = text(string.format("%d", [i(int64.cast_to_int(I64))])).
+poly(u(U))      = text(string.format("%u", [u(U)])).
+poly(u8(U8))    = text(string.format("%u", [u(uint8.cast_to_uint(U8))])).
+poly(u16(U16))  = text(string.format("%u", [u(uint16.cast_to_uint(U16))])).
+poly(u32(U32))  = text(string.format("%u", [u(uint32.cast_to_uint(U32))])).
+poly(u64(U64))  = text(string.format("%u", [u(uint64.cast_to_uint(U64))])).
+poly(f(F))      = text(string.format("%f", [f(F)])).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 to_string(W, X) = S :-
     layout_best(pred(H::in, T::in, [H | T]::out) is det, W, X, [], Ss),
@@ -449,7 +494,7 @@ write(W, X, !IO) :-
 write(Stream, W, X, !State) :-
     layout_best(put(Stream), W, doc(X), !State).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % This is a contraction of Wadler's pretty, layout and be functions,
     % adapted to work with a strict evaluation order.
@@ -473,9 +518,9 @@ layout_best(P, W, X, S0, S) :-
     %   S  is the resulting layout stream value after laying out X.
     %
     % This predicate is somewhat different to the function `be' described
-    % by Wadler.  In the first place, the decision procedure has been
+    % by Wadler. In the first place, the decision procedure has been
     % recoded (in fits_flat/2) to preserve linear running times under
-    % a strict language.  The second important change is that lb/8
+    % a strict language. The second important change is that lb/8
     % handles output strings as they are identified (e.g. writing them
     % out or accumulating them in a list), doing away with the need for
     % a more elaborate simple_doc type.
@@ -513,7 +558,7 @@ lb(P, _, K0, K, _, 'TEXT'(T),     S0, S) :-
     K = K0 + string.count_codepoints(T),
     P(T, S0, S).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Decide if a flattened doc will fit on the remainder of the line.
     %
@@ -535,7 +580,7 @@ ff('TEXT'(S),     R) = R - L :-
     L = string.count_codepoints(S),
     R > L.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Lay out a doc in its flattened form.
     %
@@ -567,24 +612,24 @@ layout_flat(P, K0, K, 'TEXT'(T),     S0, S) :-
     K = K0 + string.count_codepoints(T),
     P(T, S0, S).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func extend(string, int) = string.
 
 extend(I, J) = I ++ string.duplicate_char(' ', J).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 X `</>` Y               = X ++ line ++ Y.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 bracketed(L, R, D)      = L ++ D ++ R.
 parentheses(D)          = bracketed("(", ")", D).
 brackets(D)             = bracketed("[", "]", D).
 braces(D)               = bracketed("{", "}", D).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 separated(_,  _,   []) = nil.
 
@@ -593,7 +638,7 @@ separated(PP, Sep, [X | Xs]) =
                  else PP(X) ++ (Sep ++ separated(PP, Sep, Xs))
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed(_N, _Sep, []           ) =
     nil.
@@ -607,35 +652,35 @@ packed(N,  Sep,  [X1, X2 | Xs]) =
       else group(line ++ ellipsis)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed(Sep, Xs) = packed(int.max_int, Sep, Xs).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed_cs(N, Xs) = packed(N, ", ", Xs).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed_cs(Xs) = packed(", ", Xs).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed_cs_to_depth(Depth, Xs) =
     packed_cs(Depth, list.map(to_doc(Depth), Xs)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 packed_cs_univ_args(Depth, UnivArgs) =
     packed_cs(Depth, list.map(func(UA) = 'DOC'(Depth, UA), UnivArgs)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 word_wrapped(String) =
     packed(space, list.map(func(Word) = text(Word),
         string.words_separator(char.is_whitespace, String))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 comma                   = text(",").
 semic                   = text(";").
@@ -653,17 +698,17 @@ semic_space_line        = "; " ++ line.
 colon_space_line        = ": " ++ line.
 ellipsis                = text("...").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 to_doc(X) = to_doc(int.max_int, X).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Infix "," has precedence 1000 in standard Mercury.
     %
 to_doc(Depth, X) = to_doc(Depth, 1000, X).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % This may throw an exception or cause a runtime abort if the term
     % in question has user-defined equality.
@@ -704,13 +749,13 @@ to_doc(Depth, Priority, X) =
       else    generic_term_to_doc(Depth, Priority, X)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func generic_term_to_doc(depth, priority, T) = doc.
 
 generic_term_to_doc(Depth, Priority, X) = Doc :-
     ( if
-    	Depth =< 0
+        Depth =< 0
       then
         functor(X, canonicalize, Name, Arity),
         Doc = ( if Arity = 0 then text(Name) else Name ++ "/" ++ Arity )
@@ -785,7 +830,7 @@ generic_term_to_doc(Depth, Priority, X) = Doc :-
             )
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % We need to put parentheses around a subterm if its top-level
     % functor has a higher priority than its parent functor.
@@ -795,7 +840,7 @@ generic_term_to_doc(Depth, Priority, X) = Doc :-
 maybe_parens(ParentPriority, OpPriority, Doc) =
     ( if ParentPriority < OpPriority then parentheses(Doc) else Doc ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % An x priority adjustment lowers the effective priority by one.
     % A y priority adjustment does not affect the effective priority.
@@ -805,7 +850,7 @@ maybe_parens(ParentPriority, OpPriority, Doc) =
 Priority `adjusted_by` x = Priority - 1.
 Priority `adjusted_by` y = Priority.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % Convert a univ encapsulated value into a doc.
     %
@@ -813,7 +858,7 @@ Priority `adjusted_by` y = Priority.
 
 univ_to_doc(Depth, Priority, Univ) = to_doc(Depth, Priority, univ_value(Univ)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2] pred dynamic_cast_to_var(T1::in, var(T2)::out) is semidet.
 
@@ -827,7 +872,7 @@ dynamic_cast_to_var(X, V) :-
     % Constrain the type of V to be var(ArgType) and do the cast.
     dynamic_cast(X, V : var(ArgType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred dynamic_cast_to_sparse_bitset_of_int(T1::in, sparse_bitset(int)::out)
     is semidet.
@@ -841,7 +886,7 @@ dynamic_cast_to_sparse_bitset_of_int(X, A) :-
 dynamic_cast_to_sparse_bitset_of_var(X, A) :-
     dynamic_cast(X, A : sparse_bitset(var)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2] pred dynamic_cast_to_version_array(T1::in,
     version_array(T2)::out) is semidet.
@@ -856,7 +901,7 @@ dynamic_cast_to_version_array(X, VA) :-
     % Constrain the type of VA to be array(ArgType) and do the cast.
     dynamic_cast(X, VA : version_array(ArgType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2] pred dynamic_cast_to_list(T1::in, list(T2)::out) is semidet.
 
@@ -870,7 +915,7 @@ dynamic_cast_to_list(X, L) :-
     % Constrain the type of L to be list(ArgType) and do the cast.
     dynamic_cast(X, L : list(ArgType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2, T3] pred dynamic_cast_to_map(T1::in, map(T2, T3)::out) is semidet.
 
@@ -885,7 +930,7 @@ dynamic_cast_to_map(X, M) :-
     % Constrain the type of M to be map(KeyType, ValueType) and do the cast.
     dynamic_cast(X, M : map(KeyType, ValueType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2, T3] pred dynamic_cast_to_map_pair(T1::in, map_pair(T2, T3)::out)
     is semidet.
@@ -902,7 +947,7 @@ dynamic_cast_to_map_pair(X, MP) :-
     % and do the cast.
     dynamic_cast(X, MP : map_pair(KeyType, ValueType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred dynamic_cast_to_tuple(T::in, T::out) is semidet.
 
@@ -910,7 +955,7 @@ dynamic_cast_to_tuple(X, X) :-
     % If X is a tuple then it's functor name is {}.
     functor(X, canonicalize, "{}", _Arity).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- some [T2] pred dynamic_cast_to_robdd(T1, robdd(T2)).
 :-           mode dynamic_cast_to_robdd(in, out) is semidet.
@@ -925,14 +970,14 @@ dynamic_cast_to_robdd(X, R) :-
     % Constrain the type of R to be robdd(ArgType) and do the cast.
     dynamic_cast(X, R : robdd(ArgType)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func var_to_doc(int, var(T)) = doc.
 
 var_to_doc(Depth, V) =
     to_doc(Depth, to_int(V)).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % XXX Ideally we'd just walk the sparse bitset. But that is an optimization
     % for another day.
@@ -943,16 +988,16 @@ sparse_bitset_to_doc(Depth, A) =
     group("sparse_bitset" ++
         parentheses(list_to_doc(Depth - 1, sparse_bitset.to_sorted_list(A)))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func list_to_doc(int, list(T)) = doc.
 
 list_to_doc(Depth, Xs) =
     brackets(nest(1, packed_cs_to_depth(Depth - 1, Xs))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-    % XXX Ideally we'd just walk the array.  But that's an optimization
+    % XXX Ideally we wouldd just walk the array. But that is an optimization
     % for another day.
     %
 :- func array_to_doc(int, array(T)) = doc.
@@ -960,10 +1005,10 @@ list_to_doc(Depth, Xs) =
 array_to_doc(Depth, A) =
     group("array" ++ parentheses(list_to_doc(Depth - 1, array.to_list(A)))).
 
-%-----------------------------------------------------------------------------%
-    
-    % XXX Ideally we'd just walk the version array.  But that's an optimization
-    % for another day.
+%---------------------------------------------------------------------------%
+
+    % XXX Ideally we wouldd just walk the version array. But that is
+    % an optimization for another day.
     %
 :- func version_array_to_doc(int, version_array(T)) = doc.
 
@@ -971,7 +1016,7 @@ version_array_to_doc(Depth, A) =
     group("version_array"
         ++ parentheses(list_to_doc(Depth - 1, version_array.to_list(A)))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % This should only really be used if the item in question really
     % is a tuple.
@@ -982,7 +1027,7 @@ tuple_to_doc(Depth, Tuple) = Doc :-
     deconstruct(Tuple, canonicalize, _Name, _Arity, UnivArgs),
     Doc = group(braces(nest(1, packed_cs_univ_args(Depth - 1, UnivArgs)))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func map_to_doc(int, map(T1, T2)) = doc.
 
@@ -994,7 +1039,7 @@ map_to_doc(Depth, X) = Doc :-
 
 mk_map_pair(K - V) = map_pair(K, V).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func map_pair_to_doc(int, map_pair(T1, T2)) = doc.
 
@@ -1002,11 +1047,13 @@ map_pair_to_doc(Depth, map_pair(Key, Value)) =
     to_doc(Depth - 1, Key) ++ text(" -> ") ++
         group(nest(2, line ++ to_doc(Depth - 1, Value))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- func robdd_to_doc(int, robdd(T)) = doc.
 
 robdd_to_doc(Depth, R) =
     "robdd_dnf" ++ parentheses(list_to_doc(Depth - 1, dnf(R))).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+:- end_module pprint.
+%---------------------------------------------------------------------------%
