@@ -1,9 +1,9 @@
 %---------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
+% vim: ft=mercury ts=4 sw=4 et
 %---------------------------------------------------------------------------%
 % Copyright (C) 2007-2009, 2011 The University of Melbourne.
-% This file may only be copied under the terms of the GNU Library General
-% Public License - see the file COPYING.LIB in the Mercury distribution.
+% Copyright (C) 2014-2015, 2018 The Mercury team.
+% This file is distributed under the terms specified in COPYING.LIB.
 %---------------------------------------------------------------------------%
 %
 % File: stm_builtin.m.
@@ -13,15 +13,15 @@
 % This module defines types and predicates that can be used with the
 % Software Transactional Memory constructs.
 %
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module stm_builtin.
 :- interface.
 
 :- import_module io.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Transaction state
 %
@@ -39,7 +39,7 @@
     %
 :- type stm.
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Transaction variables
 %
@@ -69,19 +69,19 @@
     %
 :- pred read_stm_var(stm_var(T)::in, T::out, stm::di, stm::uo) is det.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Retry
 %
 
     % Abort the current transaction and restart it from the beginning.
-    % Operationally this casuses the calling thread to block until the value
+    % Operationally this causes the calling thread to block until the value
     % of at least one transaction variable read during the attempted
     % transaction is written by another thread.
     %
 :- pred retry(stm::ui) is erroneous.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Closure versions of atomic transactions.  These predicates can be used
 % to perform Software Transactional Memory without using the atomic scope.
@@ -105,12 +105,12 @@
     pred(T, stm, stm)::in(pred(out, di, uo) is det),
     T::out, stm::di, stm::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- interface.
 
@@ -200,15 +200,15 @@
 
 :- impure pred stm_merge_nested_logs(stm::di, stm::di, stm::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module exception.
 :- import_module univ.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_decl("C", "#include \"mercury_stm.h\"").
 
@@ -226,7 +226,7 @@
 :- type stm
     --->    stm(c_pointer).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_proc("C",
     new_stm_var(T::in, TVar::out, _IO0::di, _IO::uo),
@@ -302,7 +302,7 @@
     stm_merge_nested_logs(Child::di, Parent0::di, Parent::uo),
     [will_not_call_mercury, thread_safe],
 "
-    /* Avoid a warning: Child, Parent0, Parent */
+    // Avoid a warning: Child, Parent0, Parent
 #if defined(MR_STM_DEBUG)
     fprintf(stderr, \"STM Calling Merge Nested: log <0x%.8lx>\\n\",
         (MR_Word)(Child));
@@ -365,7 +365,7 @@
     IO = MR_initial_io_state();
 ").
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 
 % In high level C grades, stm_block calls a C procedure in the runtime that
@@ -431,7 +431,7 @@ INIT mercury_sys_init_stm_builtin_modules
 
 #endif
 
-    /* forward decls to suppress gcc warnings */
+    // Forward decls to suppress gcc warnings.
     void mercury_sys_init_stm_builtin_modules_init(void);
     void mercury_sys_init_stm_builtin_modules_init_type_tables(void);
     #ifdef  MR_DEEP_PROFILING
@@ -448,20 +448,20 @@ INIT mercury_sys_init_stm_builtin_modules
 
     void mercury_sys_init_stm_builtin_modules_init_type_tables(void)
     {
-        /* no types to register */
+        // No types to register.
     }
 
     #ifdef  MR_DEEP_PROFILING
     void mercury_sys_init_stm_builtin_modules_write_out_proc_statics(
         FILE *deep_fp, FILE *procrep_fp)
     {
-        /* no proc_statics to write out */
+        // No proc_statics to write out.
     }
     #endif
 ").
 
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Retry
 %
@@ -469,7 +469,7 @@ INIT mercury_sys_init_stm_builtin_modules
 retry(_) :-
     throw(rollback_retry).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 %
 % Atomic transactions
 %
@@ -479,63 +479,66 @@ atomic_transaction(Goal, Result, !IO) :-
         impure atomic_transaction_impl(Goal, Result)
     ).
 
-:- pragma promise_pure(or_else/5).
 or_else(TransA, TransB, Result, OuterSTM0, OuterSTM) :-
-    impure stm_create_nested_transaction_log(OuterSTM0, InnerSTM_A0),
-    promise_equivalent_solutions [ResultA, InnerSTM_A] (
-        unsafe_try_stm(TransA, ResultA,
-            InnerSTM_A0, InnerSTM_A)
-    ),
-    (
-        ResultA = succeeded(Result),
-        impure stm_merge_nested_logs(InnerSTM_A, OuterSTM0, OuterSTM)
-    ;
-        ResultA = exception(ExcpA),
-
-        % If transaction A retried, then we should attemp transaction B.
-        % Otherwise we just propagate the exception upwards.
-
-        ( ExcpA = univ(rollback_retry) ->
-            impure stm_create_nested_transaction_log(OuterSTM0, InnerSTM_B0),
-            promise_equivalent_solutions [ResultB, InnerSTM_B] (
-                unsafe_try_stm(TransB, ResultB,
-                    InnerSTM_B0, InnerSTM_B)
-            ),
-            (
-                ResultB = succeeded(Result),
-                impure stm_merge_nested_logs(InnerSTM_B, OuterSTM0, OuterSTM)
-            ;
-                ResultB = exception(ExcpB),
-                ( ExcpB = univ(rollback_retry) ->
-                    impure stm_lock,
-                    impure stm_validate(InnerSTM_A, IsValidA),
-                    impure stm_validate(InnerSTM_B, IsValidB),
-                    (
-                        IsValidA = stm_transaction_valid,
-                        IsValidB = stm_transaction_valid
-                    ->
-                        % We want to wait on the union of the transaction
-                        % variables accessed during both alternatives.
-                        % We merge the transaction logs (the order does not
-                        % matter) and then propagate the retry upwards.
-                        impure stm_merge_nested_logs(InnerSTM_A, OuterSTM0,
-                            OuterSTM1),
-                        impure stm_merge_nested_logs(InnerSTM_B, OuterSTM1,
-                            OuterSTM),
-                        impure stm_unlock,
-                        retry(OuterSTM)
-                    ;
-                        impure stm_unlock,
-                        throw(rollback_invalid_transaction)
-                    )
-                ;
-                    impure stm_discard_transaction_log(InnerSTM_B),
-                    rethrow(ResultB)
-                )
-            )
+    promise_pure (
+        impure stm_create_nested_transaction_log(OuterSTM0, InnerSTM_A0),
+        promise_equivalent_solutions [ResultA, InnerSTM_A] (
+            unsafe_try_stm(TransA, ResultA,
+                InnerSTM_A0, InnerSTM_A)
+        ),
+        (
+            ResultA = succeeded(Result),
+            impure stm_merge_nested_logs(InnerSTM_A, OuterSTM0, OuterSTM)
         ;
-            impure stm_discard_transaction_log(InnerSTM_A),
-            rethrow(ResultA)
+            ResultA = exception(ExcpA),
+
+            % If transaction A retried, then we should attempt transaction B.
+            % Otherwise we just propagate the exception upwards.
+
+            ( if ExcpA = univ(rollback_retry) then
+                impure
+                    stm_create_nested_transaction_log(OuterSTM0, InnerSTM_B0),
+                promise_equivalent_solutions [ResultB, InnerSTM_B] (
+                    unsafe_try_stm(TransB, ResultB,
+                        InnerSTM_B0, InnerSTM_B)
+                ),
+                (
+                    ResultB = succeeded(Result),
+                    impure
+                        stm_merge_nested_logs(InnerSTM_B, OuterSTM0, OuterSTM)
+                ;
+                    ResultB = exception(ExcpB),
+                    ( if ExcpB = univ(rollback_retry) then
+                        impure stm_lock,
+                        impure stm_validate(InnerSTM_A, IsValidA),
+                        impure stm_validate(InnerSTM_B, IsValidB),
+                        ( if
+                            IsValidA = stm_transaction_valid,
+                            IsValidB = stm_transaction_valid
+                        then
+                            % We want to wait on the union of the transaction
+                            % variables accessed during both alternatives.
+                            % We merge the transaction logs (the order does not
+                            % matter) and then propagate the retry upwards.
+                            impure stm_merge_nested_logs(InnerSTM_A, OuterSTM0,
+                                OuterSTM1),
+                            impure stm_merge_nested_logs(InnerSTM_B, OuterSTM1,
+                                OuterSTM),
+                            impure stm_unlock,
+                            retry(OuterSTM)
+                        else
+                            impure stm_unlock,
+                            throw(rollback_invalid_transaction)
+                        )
+                    else
+                        impure stm_discard_transaction_log(InnerSTM_B),
+                        rethrow(ResultB)
+                    )
+                )
+            else
+                impure stm_discard_transaction_log(InnerSTM_A),
+                rethrow(ResultA)
+            )
         )
     ).
 
@@ -551,10 +554,10 @@ atomic_transaction_impl(Goal, Result) :-
         Result0 = succeeded(Result)
     ;
         Result0 = exception(Excp),
-        ( Excp = univ(rollback_invalid_transaction) ->
+        ( if Excp = univ(rollback_invalid_transaction) then
             impure stm_discard_transaction_log(STM),
             impure atomic_transaction_impl(Goal, Result)
-        ; Excp = univ(rollback_retry) ->
+        else if Excp = univ(rollback_retry) then
             impure stm_lock,
             impure stm_validate(STM, IsValid),
             (
@@ -567,7 +570,7 @@ atomic_transaction_impl(Goal, Result) :-
             ),
             impure stm_discard_transaction_log(STM),
             impure atomic_transaction_impl(Goal, Result)
-        ;
+        else
             impure stm_lock,
             impure stm_validate(STM, IsValid),
             impure stm_unlock,
@@ -601,6 +604,6 @@ call_atomic_goal(Goal, Result, !STM) :-
         )
     ).
 
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 :- end_module stm_builtin.
-%----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
