@@ -1,11 +1,8 @@
-/*
-** vim: ts=4 sw=4 expandtab
-*/
-/*
-** Copyright (C) 1996-2007, 2010 The University of Melbourne.
-** This file may only be copied under the terms of the GNU Library General
-** Public License - see the file COPYING.LIB in the Mercury distribution.
-*/
+// vim: ts=4 sw=4 expandtab ft=c
+
+// Copyright (C) 1996-2007, 2010 The University of Melbourne.
+// Copyright (C) 2013-2014, 2016, 2018 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
 
 #include    "mercury_imp.h"
 #include    "mercury_dlist.h"
@@ -17,20 +14,23 @@
 #include    <stdio.h>
 #include    <stdarg.h>
 
-static  MR_bool MR_find_zone_for_det_ptr_in_context(const MR_Word *ptr,
+static MR_bool  MR_find_zone_for_det_ptr_in_context(const MR_Word *ptr,
                     MR_Context *ctxt,
                     MR_MemoryZone **zone_ptr, int *zone_num_ptr);
-static  MR_bool MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr,
+static MR_bool  MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr,
                     MR_Context *ctxt,
                     MR_MemoryZone **zone_ptr, int *zone_num_ptr);
-static  MR_bool MR_find_zone_for_det_ptr(const MR_Word *ptr,
+static MR_bool  MR_find_zone_for_ptr(const MR_Word *ptr,
+                    MR_MemoryZone *first_zone, MR_MemoryZones *later_zones,
+                    MR_MemoryZone **zone_ptr, int *zone_num_ptr);
+static MR_bool  MR_find_zone_for_det_ptr(const MR_Word *ptr,
                     MR_Context **ctxt_ptr,
                     MR_MemoryZone **zone_ptr, int *zone_num_ptr);
-static  MR_bool MR_find_zone_for_nondet_ptr(const MR_Word *ptr,
+static MR_bool  MR_find_zone_for_nondet_ptr(const MR_Word *ptr,
                     MR_Context **ctxt_ptr,
                     MR_MemoryZone **zone_ptr, int *zone_num_ptr);
 
-/*--------------------------------------------------------------------*/
+////////////////////////////////////////////////////////////////////////////
 
 #ifdef  MR_DEEP_PROFILING
 static void     MR_check_watch_csd_start(MR_Code *proc);
@@ -55,7 +55,7 @@ static MR_bool  MR_proc_matches_name(const MR_Code *proc, const char *name);
 
 static  MR_bool MR_print_raw_addrs = MR_PRINT_RAW_ADDRS;
 
-/* debugging messages */
+// Debugging messages.
 
 #ifdef MR_DEBUG_HEAP_ALLOC
 
@@ -93,7 +93,7 @@ MR_debug_tag_offset_incr_hp_base_msg(FILE *fp, MR_Word ptr,
     }
 }
 
-#endif /* MR_DEBUG_HEAP_ALLOC */
+#endif // MR_DEBUG_HEAP_ALLOC
 
 #ifdef MR_LOWLEVEL_DEBUG
 
@@ -109,24 +109,27 @@ MR_mkframe_msg(FILE *fp, const char *predname)
     fprintf(fp, "\nnew choice point for procedure %s\n", predname);
     fprintf(fp, "new  fr: ");
     MR_printnondetstack(fp, MR_curfr);
-    fprintf(fp, "prev fr: ");
+    fprintf(fp, "\nprev fr: ");
     MR_printnondetstack(fp, MR_prevfr_slot(MR_curfr));
-    fprintf(fp, "succ fr: ");
+    fprintf(fp, "\nsucc fr: ");
     MR_printnondetstack(fp, MR_succfr_slot(MR_curfr));
-    fprintf(fp, "succ ip: ");
+    fprintf(fp, "\nsucc ip: ");
     MR_printlabel(fp, MR_succip_slot(MR_curfr));
     fprintf(fp, "redo fr: ");
     MR_printnondetstack(fp, MR_redofr_slot(MR_curfr));
-    fprintf(fp, "redo ip: ");
+    fprintf(fp, "\nredo ip: ");
     MR_printlabel(fp, MR_redoip_slot(MR_curfr));
 #ifdef  MR_USE_MINIMAL_MODEL_OWN_STACKS
-    fprintf(fp, "det fr:  ");
+    fprintf(fp, "\ndet fr:  ");
     MR_printdetstack(fp, MR_table_detfr_slot(MR_curfr));
 #endif
+    fprintf(fp, "\n");
 
     if (MR_detaildebug) {
         MR_dumpnondetstack(fp);
     }
+
+    fflush(fp);
 }
 
 void
@@ -138,19 +141,21 @@ MR_mktempframe_msg(FILE *fp)
         return;
     }
 
-    fprintf(fp, "\nnew temp nondet frame\n");
-    fprintf(fp, "new  fr: ");
+    fprintf(fp, "\nnew temp nondet frame");
+    fprintf(fp, "\nnew  fr: ");
     MR_printnondetstack(fp, MR_maxfr);
-    fprintf(fp, "prev fr: ");
+    fprintf(fp, "\nprev fr: ");
     MR_printnondetstack(fp, MR_prevfr_slot(MR_maxfr));
-    fprintf(fp, "redo fr: ");
+    fprintf(fp, "\nredo fr: ");
     MR_printnondetstack(fp, MR_redofr_slot(MR_maxfr));
-    fprintf(fp, "redo ip: ");
+    fprintf(fp, "\nredo ip: ");
     MR_printlabel(fp, MR_redoip_slot(MR_maxfr));
 
     if (MR_detaildebug) {
         MR_dumpnondetstack(fp);
     }
+
+    fflush(fp);
 }
 
 void
@@ -162,17 +167,18 @@ MR_mkdettempframe_msg(FILE *fp)
         return;
     }
 
-    fprintf(fp, "\nnew det temp nondet frame\n");
-    fprintf(fp, "new  fr: ");
+    fprintf(fp, "\nnew det temp nondet frame");
+    fprintf(fp, "\nnew  fr: ");
     MR_printnondetstack(fp, MR_maxfr);
-    fprintf(fp, "prev fr: ");
+    fprintf(fp, "\nprev fr: ");
     MR_printnondetstack(fp, MR_prevfr_slot(MR_maxfr));
-    fprintf(fp, "redo fr: ");
+    fprintf(fp, "\nredo fr: ");
     MR_printnondetstack(fp, MR_redofr_slot(MR_maxfr));
-    fprintf(fp, "redo ip: ");
+    fprintf(fp, "\nredo ip: ");
     MR_printlabel(fp, MR_redoip_slot(MR_maxfr));
     fprintf(fp, "det fr:  ");
     MR_printdetstack(fp, MR_tmp_detfr_slot(MR_maxfr));
+    fprintf(fp, "\n");
 
     if (MR_detaildebug) {
         MR_dumpnondetstack(fp);
@@ -241,10 +247,11 @@ MR_fail_msg(FILE *fp)
     fprintf(fp, "\nfailing from procedure\n");
     fprintf(fp, "curr fr: ");
     MR_printnondetstack(fp, MR_curfr);
-    fprintf(fp, "fail fr: ");
+    fprintf(fp, "\nfail fr: ");
     MR_printnondetstack(fp, MR_prevfr_slot(MR_curfr));
-    fprintf(fp, "fail ip: ");
+    fprintf(fp, "\nfail ip: ");
     MR_printlabel(fp, MR_redoip_slot(MR_prevfr_slot(MR_curfr)));
+    fprintf(fp, "\n");
 }
 
 void
@@ -258,12 +265,12 @@ MR_redo_msg(FILE *fp)
         return;
     }
 
-    fprintf(fp, "\nredo from procedure\n");
-    fprintf(fp, "curr fr: ");
+    fprintf(fp, "\nredo from procedure");
+    fprintf(fp, "\ncurr fr: ");
     MR_printnondetstack(fp, MR_curfr);
-    fprintf(fp, "redo fr: ");
+    fprintf(fp, "\nredo fr: ");
     MR_printnondetstack(fp, MR_maxfr);
-    fprintf(fp, "redo ip: ");
+    fprintf(fp, "\nredo ip: ");
     MR_printlabel(fp, MR_redoip_slot(MR_maxfr));
 }
 
@@ -274,7 +281,7 @@ MR_call_msg(FILE *fp, const MR_Code *proc, const MR_Code *succ_cont)
 
 #ifdef  MR_DEEP_PROFILING
     MR_check_watch_csd_start(proc);
-#endif  /* MR_DEEP_PROFILING */
+#endif  // MR_DEEP_PROFILING
 
     MR_do_watches(fp);
 
@@ -305,7 +312,7 @@ MR_tailcall_msg(FILE *fp, const MR_Code *proc)
 
 #ifdef  MR_DEEP_PROFILING
     MR_check_watch_csd_start(proc);
-#endif  /* MR_DEEP_PROFILING */
+#endif  // MR_DEEP_PROFILING
 
     MR_do_watches(fp);
 
@@ -443,7 +450,7 @@ MR_decr_sp_msg(FILE *fp, MR_Word val, const MR_Word *addr)
     MR_printdetstack(fp, addr);
 }
 
-#endif /* defined(MR_LOWLEVEL_DEBUG) */
+#endif // defined(MR_LOWLEVEL_DEBUG)
 
 #ifdef MR_DEBUG_GOTOS
 
@@ -473,7 +480,7 @@ MR_reg_msg(FILE *fp)
         return;
     }
 
-    for(i=1; i<=8; i++) {
+    for (i=1; i<=8; i++) {
         x = (MR_Integer) MR_get_reg(i);
 #ifndef MR_CONSERVATIVE_GC
         if ((MR_Integer) MR_ENGINE(MR_eng_heap_zone)->MR_zone_min <= x
@@ -487,13 +494,13 @@ MR_reg_msg(FILE *fp)
     fprintf(fp, "\n");
 }
 
-#endif /* defined(MR_DEBUG_GOTOS) */
+#endif // defined(MR_DEBUG_GOTOS)
 
-/*--------------------------------------------------------------------*/
+////////////////////////////////////////////////////////////////////////////
 
 #ifdef MR_LOWLEVEL_DEBUG
 
-/* debugging printing tools */
+// Debugging printing tools.
 
 static void
 MR_count_call(FILE *fp, const MR_Code *proc)
@@ -542,10 +549,10 @@ MR_count_call(FILE *fp, const MR_Code *proc)
     }
 #endif
 
-    /* the bitwise ORs implement logical OR */
+    // The bitwise ORs implement logical OR.
     MR_lld_print_enabled = MR_lld_print_region_enabled
         | MR_lld_print_name_enabled | MR_lld_print_csd_enabled
-        | MR_lld_debug_enabled;
+        | MR_lld_debug_enabled | MR_lld_print_always_enabled;
 }
 
 void
@@ -572,7 +579,7 @@ MR_printheap(FILE *fp, const MR_Word *h)
         fprintf(fp, "ptr %p, ", (const void *) h);
     }
 
-    fprintf(fp, "offset %3ld words\n",
+    fprintf(fp, "offset %6ld words\n",
         (long) (MR_Integer) (h - MR_ENGINE(MR_eng_heap_zone)->min));
 #else
     fprintf(fp, "ptr %p\n", (const void *) h);
@@ -609,7 +616,7 @@ MR_dumpnondetstack(FILE *fp)
     MR_Word *fr;
 
     fprintf(fp, "\nnondetstack dump\n");
-    for (fr = MR_maxfr; fr > MR_nondet_stack_trace_bottom;
+    for (fr = MR_maxfr; MR_above_bottom_nondet_frame(fr);
         fr = MR_prevfr_slot(fr))
     {
         MR_dumpframe(fp, fr);
@@ -672,34 +679,36 @@ MR_print_ordinary_regs(FILE *fp)
     }
 }
 
+////////////////////////////////////////////////////////////////////////////
+
 #ifdef  MR_DEEP_PROFILING
 
 static struct MR_CallSiteDynamic_Struct MR_watched_csd_last_value =
 {
-    /* MR_csd_callee_ptr */ NULL,
+    /* MR_csd_callee_ptr */     NULL,
     {
   #ifdef MR_DEEP_PROFILING_PORT_COUNTS
     #ifdef MR_DEEP_PROFILING_EXPLICIT_CALL_COUNTS
-    /* MR_own_calls */ 0,
+    /* MR_own_calls */          0,
     #else
-    /* calls are computed from the other fields */
+    // Calls are computed from the other fields.
     #endif
-    /* MR_own_exits */ 0,
-    /* MR_own_fails */ 0,
-    /* MR_own_redos */ 0,
+    /* MR_own_exits */          0,
+    /* MR_own_fails */          0,
+    /* MR_own_redos */          0,
   #endif
   #ifdef MR_DEEP_PROFILING_TIMING
-    /* MR_own_quanta */ 0,
+    /* MR_own_quanta */         0,
   #endif
   #ifdef MR_DEEP_PROFILING_CALL_SEQ
-    /* MR_own_call_seqs */ 0,
+    /* MR_own_call_seqs */      0,
   #endif
   #ifdef MR_DEEP_PROFILING_MEMORY
-    /* MR_own_allocs */ 0,
-    /* MR_own_words */ 0,
+    /* MR_own_allocs */         0,
+    /* MR_own_words */          0,
   #endif
     },
-    /* MR_csd_depth_count */ 0
+    /* MR_csd_depth_count */    0
 };
 
 static void
@@ -712,9 +721,8 @@ MR_check_watch_csd_start(MR_Code *proc)
 
     if (MR_proc_matches_name(proc, MR_watch_csd_start_name)) {
         if (MR_watch_csd_addr == MR_next_call_site_dynamic) {
-            /*
-            ** Optimize future checks and make MR_watch_csd_addr static.
-            */
+            // Optimize future checks and make MR_watch_csd_addr static.
+
             MR_watch_csd_started = MR_TRUE;
             MR_watch_csd_start_name = NULL;
         }
@@ -805,7 +813,9 @@ MR_assign_csd(MR_CallSiteDynamic *csd1, const MR_CallSiteDynamic *csd2)
     csd1->MR_csd_depth_count = csd2->MR_csd_depth_count;
 };
 
-#endif  /* MR_DEEP_PROFILING */
+#endif  // MR_DEEP_PROFILING
+
+////////////////////////////////////////////////////////////////////////////
 
 static void
 MR_do_watches(FILE *fp)
@@ -827,7 +837,7 @@ MR_do_watches(FILE *fp)
             }
         }
     }
-#endif  /* MR_DEEP_PROFILING */
+#endif  // MR_DEEP_PROFILING
 }
 
 static MR_bool
@@ -845,11 +855,11 @@ MR_proc_matches_name(const MR_Code *proc, const char *name)
         }
     }
 
-#endif  /* MR_NEED_ENTRY_LABEL_ARRAY */
+#endif  // MR_NEED_ENTRY_LABEL_ARRAY
     return MR_FALSE;
 }
 
-#endif /* defined(MR_DEBUG_GOTOS) */
+#endif // defined(MR_DEBUG_GOTOS)
 
 #ifndef MR_HIGHLEVEL_CODE
 
@@ -876,6 +886,33 @@ MR_print_detstackptr(FILE *fp, const MR_Word *s)
     }
 }
 
+void
+MR_print_zone(FILE *fp, const MR_MemoryZone *zone)
+{
+    fprintf(fp, "zone %p:\n", zone);
+    fprintf(fp, "  bottom %p, top %p\n",
+        zone->MR_zone_bottom, zone->MR_zone_top);
+    fprintf(fp, "  min    %p, max %p",
+        zone->MR_zone_min, zone->MR_zone_max);
+#if defined(MR_PROTECTPAGE)
+    fprintf(fp, ", hardmax %p", zone->MR_zone_hardmax);
+#endif
+#if defined(MR_STACK_SEGMENTS) && !defined(MR_HIGHLEVEL_CODE)
+    fprintf(fp, ", extend %p",
+        zone->MR_zone_extend_threshold);
+#endif
+    fprintf(fp, "\n");
+}
+
+void
+MR_print_zones(FILE *fp, const MR_MemoryZones *zones)
+{
+    while (zones != NULL) {
+        MR_print_zone(fp, zones->MR_zones_head);
+        zones = zones->MR_zones_tail;
+    }
+}
+
 void
 MR_printdetstack(FILE *fp, const MR_Word *s)
 {
@@ -888,10 +925,10 @@ MR_printdetstack(FILE *fp, const MR_Word *s)
         }
 
         if (zone_num == 0) {
-            fprintf(fp, "offset %3ld words",
+            fprintf(fp, "offset %6ld words",
                 (long) (MR_Integer) (s - zone->MR_zone_min));
         } else {
-            fprintf(fp, "offset %3ld words in segment %d",
+            fprintf(fp, "offset %6ld words in segment %d",
                 (long) (MR_Integer) (s - zone->MR_zone_min), zone_num);
         }
     } else {
@@ -934,10 +971,10 @@ MR_printnondetstack(FILE *fp, const MR_Word *s)
         }
 
         if (zone_num == 0) {
-            fprintf(fp, "offset %3ld words",
+            fprintf(fp, "offset %6ld words",
                 (long) (MR_Integer) (s - zone->MR_zone_min));
         } else {
-            fprintf(fp, "offset %3ld words in segment %d",
+            fprintf(fp, "offset %6ld words in segment %d",
                 (long) (MR_Integer) (s - zone->MR_zone_min), zone_num);
         }
     } else {
@@ -945,7 +982,7 @@ MR_printnondetstack(FILE *fp, const MR_Word *s)
     }
 }
 
-#endif /* !MR_HIGHLEVEL_CODE */
+#endif // !MR_HIGHLEVEL_CODE
 
 void
 MR_print_heapptr(FILE *fp, const MR_Word *s)
@@ -962,11 +999,9 @@ MR_print_heapptr(FILE *fp, const MR_Word *s)
     }
 }
 
-/*
-** The code of MR_print_label is similar to, but significantly more powerful
-** than, MR_lookup_entry_or_internal in mercury_label.c, since it does not
-** have to return a single short constant string.
-*/
+// The code of MR_print_label is similar to, but significantly more powerful
+// than, MR_lookup_entry_or_internal in mercury_label.c, since it does not
+// have to return a single short constant string.
 
 void
 MR_print_label(FILE *fp, const MR_Code *w)
@@ -1074,7 +1109,9 @@ MR_print_deep_prof_var(FILE *fp, const char *name, MR_CallSiteDynamic *csd)
 #endif
 }
 
-/* auxiliary routines for the code that prints debugging messages */
+////////////////////////////////////////////////////////////////////////////
+
+// Auxiliary routines for the code that prints debugging messages.
 
 static MR_bool
 MR_find_zone_for_det_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
@@ -1082,46 +1119,12 @@ MR_find_zone_for_det_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
 {
 #ifdef  MR_HIGHLEVEL_CODE
     return MR_FALSE;
-#else   /* !MR_HIGHLEVEL_CODE */
-    int             segment_number;
-    MR_MemoryZones  *remaining_zones;
-    MR_MemoryZone   *cur_zone;
-
-    remaining_zones = ctxt->MR_ctxt_prev_detstack_zones;
-    if (MR_in_zone(ptr, ctxt->MR_ctxt_detstack_zone)) {
-        if (zone_ptr != NULL) {
-            *zone_ptr = ctxt->MR_ctxt_detstack_zone;
-        }
-    } else {
-        while (remaining_zones != NULL) {
-            cur_zone = remaining_zones->MR_zones_head;
-            if (MR_in_zone(ptr, cur_zone)) {
-                if (zone_ptr != NULL) {
-                    *zone_ptr = cur_zone;
-                }
-
-                remaining_zones = remaining_zones->MR_zones_tail;
-                break;
-            }
-
-            remaining_zones = remaining_zones->MR_zones_tail;
-        }
-
-        return MR_FALSE;
-    }
-
-    if (zone_num_ptr != NULL) {
-        segment_number = 0;
-        while (remaining_zones != NULL) {
-            segment_number++;
-            remaining_zones = remaining_zones->MR_zones_tail;
-        }
-
-        *zone_num_ptr = segment_number;
-    }
-
-    return MR_TRUE;
-#endif  /* !MR_HIGHLEVEL_CODE */
+#else   // !MR_HIGHLEVEL_CODE
+    return MR_find_zone_for_ptr(ptr,
+        ctxt->MR_ctxt_detstack_zone,
+        ctxt->MR_ctxt_prev_detstack_zones,
+        zone_ptr, zone_num_ptr);
+#endif  // !MR_HIGHLEVEL_CODE
 }
 
 static MR_bool
@@ -1130,17 +1133,35 @@ MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
 {
 #ifdef  MR_HIGHLEVEL_CODE
     return MR_FALSE;
-#else   /* !MR_HIGHLEVEL_CODE */
+#else   // !MR_HIGHLEVEL_CODE
+    return MR_find_zone_for_ptr(ptr,
+        ctxt->MR_ctxt_nondetstack_zone,
+        ctxt->MR_ctxt_prev_nondetstack_zones,
+        zone_ptr, zone_num_ptr);
+#endif  // !MR_HIGHLEVEL_CODE
+}
+
+static MR_bool
+MR_find_zone_for_ptr(const MR_Word *ptr,
+    MR_MemoryZone *first_zone, MR_MemoryZones *later_zones,
+    MR_MemoryZone **zone_ptr, int *zone_num_ptr)
+{
+#ifdef  MR_HIGHLEVEL_CODE
+    return MR_FALSE;
+#else   // !MR_HIGHLEVEL_CODE
     int             segment_number;
     MR_MemoryZones  *remaining_zones;
     MR_MemoryZone   *cur_zone;
 
-    remaining_zones = ctxt->MR_ctxt_prev_nondetstack_zones;
-    if (MR_in_zone(ptr, ctxt->MR_ctxt_nondetstack_zone)) {
+    remaining_zones = later_zones;
+    if (MR_in_zone(ptr, first_zone)) {
         if (zone_ptr != NULL) {
-            *zone_ptr = ctxt->MR_ctxt_nondetstack_zone;
+            *zone_ptr = first_zone;
         }
     } else {
+        MR_bool found;
+
+        found = MR_FALSE;
         while (remaining_zones != NULL) {
             cur_zone = remaining_zones->MR_zones_head;
             if (MR_in_zone(ptr, cur_zone)) {
@@ -1148,6 +1169,7 @@ MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
                     *zone_ptr = cur_zone;
                 }
 
+                found = MR_TRUE;
                 remaining_zones = remaining_zones->MR_zones_tail;
                 break;
             }
@@ -1155,7 +1177,9 @@ MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
             remaining_zones = remaining_zones->MR_zones_tail;
         }
 
-        return MR_FALSE;
+        if (!found) {
+            return MR_FALSE;
+        }
     }
 
     if (zone_num_ptr != NULL) {
@@ -1169,7 +1193,7 @@ MR_find_zone_for_nondet_ptr_in_context(const MR_Word *ptr, MR_Context *ctxt,
     }
 
     return MR_TRUE;
-#endif  /* !MR_HIGHLEVEL_CODE */
+#endif  // !MR_HIGHLEVEL_CODE
 }
 
 static MR_bool
@@ -1204,7 +1228,7 @@ MR_find_zone_for_det_ptr(const MR_Word *ptr, MR_Context **ctxt_ptr,
         }
     }
 
-#else   /* !MR_USE_MINIMAL_MODEL_OWN_STACKS */
+#else   // !MR_USE_MINIMAL_MODEL_OWN_STACKS
 
     if (MR_find_zone_for_det_ptr_in_context(ptr, &MR_ENGINE(MR_eng_context),
         zone_ptr, zone_num_ptr))
@@ -1216,7 +1240,7 @@ MR_find_zone_for_det_ptr(const MR_Word *ptr, MR_Context **ctxt_ptr,
         return MR_TRUE;
     }
 
-#endif  /* MR_USE_MINIMAL_MODEL_OWN_STACKS */
+#endif  // MR_USE_MINIMAL_MODEL_OWN_STACKS
 
     return MR_FALSE;
 }
@@ -1253,7 +1277,7 @@ MR_find_zone_for_nondet_ptr(const MR_Word *ptr, MR_Context **ctxt_ptr,
         }
     }
 
-#else   /* !MR_USE_MINIMAL_MODEL_OWN_STACKS */
+#else   // !MR_USE_MINIMAL_MODEL_OWN_STACKS
 
     if (MR_find_zone_for_nondet_ptr_in_context(ptr, &MR_ENGINE(MR_eng_context),
         zone_ptr, zone_num_ptr))
@@ -1265,43 +1289,39 @@ MR_find_zone_for_nondet_ptr(const MR_Word *ptr, MR_Context **ctxt_ptr,
         return MR_TRUE;
     }
 
-#endif  /* MR_USE_MINIMAL_MODEL_OWN_STACKS */
+#endif  // MR_USE_MINIMAL_MODEL_OWN_STACKS
 
     return MR_FALSE;
 }
 
-void 
-MR_debug_log_message(const char *format, ...) {
+////////////////////////////////////////////////////////////////////////////
+
+void
+MR_debug_log_message(const char *format, ...)
+{
     char    *buffer;
-    int     len, result;
+    int     len;
+    int     result;
     va_list args;
 
-    /*
-    ** This should be a reasonable estimate of the size of the buffer that we
-    ** need.  At least twice the size of the format string or 128 bytes.
-    */
+    // This should be a reasonable estimate of the size of the buffer that we
+    // need. At least twice the size of the format string or 128 bytes.
+
     len = strlen(format);
-    len = len*2;
-    len = len<128 ? 128 : len;
+    len = len * 2;
+    len = len < 128 ? 128 : len;
 
     buffer = MR_GC_malloc(len);
     while (1) {
         va_start(args, format);
-#ifdef MR_HAVE_VSNPRINTF
-        result = vsnprintf(buffer, len, format, args);
-#elif MR_HAVE__VSNPRINTF
-        result = _vsnprintf(buffer, len, formst, args);
-#else
-        MR_fatal_error(
-            "MR_debug_log_message: Don't have vsnprintf or _vsnprintf\n");
-#endif
+        result = MR_vsnprintf(buffer, len, format, args);
         va_end(args);
         if (result < len) {
             break;
         }
 
-        /* Make the buffer bigger. */
-        len = len*2;
+        // Make the buffer bigger.
+        len = len * 2;
         buffer = MR_GC_realloc(buffer, len);
     }
 
@@ -1313,4 +1333,3 @@ MR_debug_log_message(const char *format, ...) {
     printf("%s\n", buffer);
 #endif
 }
-
