@@ -1,14 +1,15 @@
+// vim: ts=4 sw=4 expandtab ft=cs
 //
 // Copyright (C) 2003-2004, 2010-2011 The University of Melbourne.
-// This file may only be copied under the terms of the GNU Library General
-// Public License - see the file COPYING.LIB in the Mercury distribution.
+// Copyright (C) 2014-2018, 2021 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
 //
 
 // mercury_dotnet.cs - This file defines the system runtime types and
 // methods that are used when generating code for the .NET backend.
 
-// the following "using" declaration works around a bug in GNU Portable.NET's
-// C# compiler
+// The following "using" declaration works around a bug in GNU Portable.NET's
+// C# compiler.
 using System.Runtime.InteropServices;
 
 namespace mercury.runtime {
@@ -300,45 +301,54 @@ public enum TypeCtorRep {
     MR_TYPECTOR_REP_EQUIV                   = 6,
     MR_TYPECTOR_REP_FUNC                    = 7,
     MR_TYPECTOR_REP_INT                     = 8,
-    MR_TYPECTOR_REP_CHAR                    = 9,
-    MR_TYPECTOR_REP_FLOAT                   = 10,
-    MR_TYPECTOR_REP_STRING                  = 11,
-    MR_TYPECTOR_REP_PRED                    = 12,
-    MR_TYPECTOR_REP_SUBGOAL                 = 13,
-    MR_TYPECTOR_REP_VOID                    = 14,
-    MR_TYPECTOR_REP_C_POINTER               = 15,
-    MR_TYPECTOR_REP_TYPEINFO                = 16,
-    MR_TYPECTOR_REP_TYPECLASSINFO           = 17,
-    MR_TYPECTOR_REP_ARRAY                   = 18,
-    MR_TYPECTOR_REP_SUCCIP                  = 19,
-    MR_TYPECTOR_REP_HP                      = 20,
-    MR_TYPECTOR_REP_CURFR                   = 21,
-    MR_TYPECTOR_REP_MAXFR                   = 22,
-    MR_TYPECTOR_REP_REDOFR                  = 23,
-    MR_TYPECTOR_REP_REDOIP                  = 24,
-    MR_TYPECTOR_REP_TRAIL_PTR               = 25,
-    MR_TYPECTOR_REP_TICKET                  = 26,
-    MR_TYPECTOR_REP_NOTAG_GROUND            = 27,
-    MR_TYPECTOR_REP_NOTAG_GROUND_USEREQ     = 28,
-    MR_TYPECTOR_REP_EQUIV_GROUND            = 29,
-    MR_TYPECTOR_REP_TUPLE                   = 30,
-    MR_TYPECTOR_REP_RESERVED_ADDR           = 31,
-    MR_TYPECTOR_REP_RESERVED_ADDR_USEREQ    = 32,
-    MR_TYPECTOR_REP_TYPECTORINFO            = 33,
-    MR_TYPECTOR_REP_BASETYPECLASSINFO       = 34,
-    MR_TYPECTOR_REP_TYPEDESC                = 35,
-    MR_TYPECTOR_REP_TYPECTORDESC            = 36,
-    MR_TYPECTOR_REP_FOREIGN                 = 37,
-    MR_TYPECTOR_REP_REFERENCE               = 38,
-    MR_TYPECTOR_REP_STABLE_C_POINTER        = 39,
-    MR_TYPECTOR_REP_STABLE_FOREIGN          = 40,
-    MR_TYPECTOR_REP_PSEUDOTYPEDESC          = 41,
-    MR_TYPECTOR_REP_DUMMY                   = 42,
-    MR_TYPECTOR_REP_BITMAP                  = 43,
-    MR_TYPECTOR_REP_FOREIGN_ENUM            = 44,
-    MR_TYPECTOR_REP_FOREIGN_ENUM_USEREQ     = 45,
-    MR_TYPECTOR_REP_UNKNOWN                 = 46,
-    MR_TYPECTOR_REP_MAX                     = 47
+    MR_TYPECTOR_REP_UINT                    = 9,
+    MR_TYPECTOR_REP_CHAR                    = 10,
+    MR_TYPECTOR_REP_FLOAT                   = 11,
+    MR_TYPECTOR_REP_STRING                  = 12,
+    MR_TYPECTOR_REP_PRED                    = 13,
+    MR_TYPECTOR_REP_SUBGOAL                 = 14,
+    MR_TYPECTOR_REP_VOID                    = 15,
+    MR_TYPECTOR_REP_C_POINTER               = 16,
+    MR_TYPECTOR_REP_TYPEINFO                = 17,
+    MR_TYPECTOR_REP_TYPECLASSINFO           = 18,
+    MR_TYPECTOR_REP_ARRAY                   = 19,
+    MR_TYPECTOR_REP_SUCCIP                  = 20,
+    MR_TYPECTOR_REP_HP                      = 21,
+    MR_TYPECTOR_REP_CURFR                   = 22,
+    MR_TYPECTOR_REP_MAXFR                   = 23,
+    MR_TYPECTOR_REP_REDOFR                  = 24,
+    MR_TYPECTOR_REP_REDOIP                  = 25,
+    MR_TYPECTOR_REP_TRAIL_PTR               = 26,
+    MR_TYPECTOR_REP_TICKET                  = 27,
+    MR_TYPECTOR_REP_NOTAG_GROUND            = 28,
+    MR_TYPECTOR_REP_NOTAG_GROUND_USEREQ     = 29,
+    MR_TYPECTOR_REP_EQUIV_GROUND            = 30,
+    MR_TYPECTOR_REP_TUPLE                   = 31,
+    MR_TYPECTOR_REP_RESERVED_ADDR           = 32,
+    MR_TYPECTOR_REP_RESERVED_ADDR_USEREQ    = 33,
+    MR_TYPECTOR_REP_TYPECTORINFO            = 34,
+    MR_TYPECTOR_REP_BASETYPECLASSINFO       = 35,
+    MR_TYPECTOR_REP_TYPEDESC                = 36,
+    MR_TYPECTOR_REP_TYPECTORDESC            = 37,
+    MR_TYPECTOR_REP_FOREIGN                 = 38,
+    MR_TYPECTOR_REP_REFERENCE               = 39,
+    MR_TYPECTOR_REP_STABLE_C_POINTER        = 40,
+    MR_TYPECTOR_REP_STABLE_FOREIGN          = 41,
+    MR_TYPECTOR_REP_PSEUDOTYPEDESC          = 42,
+    MR_TYPECTOR_REP_DUMMY                   = 43,
+    MR_TYPECTOR_REP_BITMAP                  = 44,
+    MR_TYPECTOR_REP_FOREIGN_ENUM            = 45,
+    MR_TYPECTOR_REP_FOREIGN_ENUM_USEREQ     = 46,
+    MR_TYPECTOR_REP_INT8                    = 47,
+    MR_TYPECTOR_REP_UINT8                   = 48,
+    MR_TYPECTOR_REP_INT16                   = 49,
+    MR_TYPECTOR_REP_UINT16                  = 50,
+    MR_TYPECTOR_REP_INT32                   = 51,
+    MR_TYPECTOR_REP_UINT32                  = 52,
+    MR_TYPECTOR_REP_INT64                   = 53,
+    MR_TYPECTOR_REP_UINT64                  = 54,
+    MR_TYPECTOR_REP_UNKNOWN                 = 55,
+    MR_TYPECTOR_REP_MAX                     = 56
 }
 
 public class PseudoTypeInfo {
@@ -359,21 +369,24 @@ public class PseudoTypeInfo {
 }
 
 public class TypeCtorInfo_Struct : PseudoTypeInfo {
-    public int              arity;
-    public int              type_ctor_version;
-    public int              type_ctor_num_ptags;
-    public TypeCtorRep      type_ctor_rep;
-    public object           unify_pred;
-    public object           compare_pred;
-    public string           type_ctor_module_name;
-    public string           type_ctor_name;
-    public TypeFunctors     type_functors;
-    public TypeLayout       type_layout;
-    public int              type_ctor_num_functors;
-    public short            type_ctor_flags;
-    public int[]            type_functor_number_map;
+    public int                  arity;
+    public byte                 type_ctor_version;
+    public sbyte                type_ctor_num_ptags;
+    public TypeCtorRep          type_ctor_rep;
+    public object               unify_pred;
+    public object               compare_pred;
+    public string               type_ctor_module_name;
+    public string               type_ctor_name;
+    public TypeFunctors         type_functors;
+    public TypeLayout           type_layout;
+    public int                  type_ctor_num_functors;
+    public ushort               type_ctor_flags;
+    public int[]                type_functor_number_map;
+    public TypeCtorInfo_Struct  type_ctor_base; // null unless subtype
 
-    // This attribute is also to to prevent cyclic copying in deep_copy.
+    private const ushort MR_TYPE_CTOR_FLAG_LAYOUT_INDEXABLE = 0x8;
+
+    // This attribute is also to prevent cyclic copying in deep_copy.
     [System.NonSerialized]
     private TypeInfo_Struct cached_type_info;
 
@@ -394,43 +407,46 @@ public class TypeCtorInfo_Struct : PseudoTypeInfo {
             other.type_layout,
             other.type_ctor_num_functors,
             other.type_ctor_flags,
-            other.type_functor_number_map
+            other.type_functor_number_map,
+            other.type_ctor_base
         );
     }
 
     public void init(
         int type_arity,
-        int version,
-        int num_ptags,
+        byte version,
+        sbyte num_ptags,
         TypeCtorRep rep,
         object unify_proc,
         object compare_proc,
         string module,
         string name,
-        object name_ordered_functor_descs, // TypeFunctors
-        object value_ordered_functor_descs, // TypeLayout
+        object name_ordered_functor_descs,  // TypeFunctors
+        object ordinal_ordered_functor_descs, // TypeLayout
         int num_functors,
-        short flags,
-        int[] functor_number_map)
+        ushort flags,
+        int[] functor_number_map,
+        TypeCtorInfo_Struct type_ctor_base)
     {
-        arity = type_arity;
-        type_ctor_version = version;
-        type_ctor_num_ptags = num_ptags;
-        type_ctor_rep = rep;
-        unify_pred = unify_proc;
-        compare_pred = compare_proc;
-        type_ctor_module_name = module;
-        type_ctor_name = name;
-        type_functors = (TypeFunctors) name_ordered_functor_descs;
-        type_layout = (TypeLayout) value_ordered_functor_descs;
-        type_ctor_num_functors = num_functors;
-        type_ctor_flags = flags;
-        type_functor_number_map = functor_number_map;
+        this.arity = type_arity;
+        this.type_ctor_version = version;
+        this.type_ctor_num_ptags = num_ptags;
+        this.type_ctor_rep = rep;
+        this.unify_pred = unify_proc;
+        this.compare_pred = compare_proc;
+        this.type_ctor_module_name = module;
+        this.type_ctor_name = name;
+        this.type_functors = (TypeFunctors) name_ordered_functor_descs;
+        this.type_layout = (TypeLayout) ordinal_ordered_functor_descs;
+        this.type_ctor_num_functors = num_functors;
+        this.type_ctor_flags = flags;
+        this.type_functor_number_map = functor_number_map;
+        this.type_ctor_base = type_ctor_base;
     }
 
     public override bool Equals(object other) {
         TypeCtorInfo_Struct tci = other as TypeCtorInfo_Struct;
-        if ((System.Object)tci == null) {
+        if ((System.Object) tci == null) {
             return false;
         }
         return this.Equals(tci);
@@ -464,6 +480,40 @@ public class TypeCtorInfo_Struct : PseudoTypeInfo {
         }
         return cached_type_info;
     }
+
+    public int index_or_search_enum_functor_ordinal(int enum_value) {
+        if (flags_is_layout_indexable()) {
+            return enum_value; // same as ordinal
+        }
+
+        EnumFunctorDesc[] layout_enum = type_layout.layout_enum();
+        for (int i = 0; i < layout_enum.Length; i++) {
+            if (layout_enum[i].enum_functor_value == enum_value) {
+                return i;
+            }
+        }
+        return -1;
+    }
+
+    // Return the ptag layout with the given ptag, or null if not found.
+    public DuPtagLayout index_or_search_ptag_layout(byte ptag) {
+        DuPtagLayout[] layout_du = type_layout.layout_du();
+
+        if (flags_is_layout_indexable()) {
+            return layout_du[ptag];
+        }
+
+        foreach (DuPtagLayout ptag_layout in layout_du) {
+            if (ptag_layout.du_ptag == ptag) {
+                return ptag_layout;
+            }
+        }
+        return null;
+    }
+
+    private bool flags_is_layout_indexable() {
+        return (type_ctor_flags & MR_TYPE_CTOR_FLAG_LAYOUT_INDEXABLE) != 0;
+    }
 }
 
 public class TypeInfo_Struct : PseudoTypeInfo {
@@ -533,7 +583,7 @@ public class TypeInfo_Struct : PseudoTypeInfo {
 
     public override bool Equals(object other) {
         TypeInfo_Struct ti = other as TypeInfo_Struct;
-        if ((System.Object)ti == null) {
+        if ((System.Object) ti == null) {
             return false;
         }
         return this.Equals(ti);
@@ -582,7 +632,7 @@ public class TypeInfo_Struct : PseudoTypeInfo {
     private TypeInfo_Struct collapse_equivalences() {
         TypeInfo_Struct ti = this;
 
-        /* Look past equivalences */
+        // Look past equivalences.
         while (ti.type_ctor.type_ctor_rep ==
                 TypeCtorRep.MR_TYPECTOR_REP_EQUIV_GROUND
             || ti.type_ctor.type_ctor_rep ==
@@ -659,33 +709,37 @@ public class TypeFunctors {
 
 public class DuFunctorDesc {
     public string           du_functor_name;
-    public int              du_functor_orig_arity;
-    public int              du_functor_arg_type_contains_var;
+    public short            du_functor_orig_arity;
+    public ushort           du_functor_arg_type_contains_var;
     public Sectag_Locn      du_functor_sectag_locn;
-    public int              du_functor_primary;
+    public byte             du_functor_primary;
     public int              du_functor_secondary;
     public int              du_functor_ordinal;
     public PseudoTypeInfo[] du_functor_arg_types;
     public string[]         du_functor_arg_names;
     public DuArgLocn[]      du_functor_arg_locns;
     public DuExistInfo      du_functor_exist_info;
+    public FunctorSubtypeInfo   du_functor_subtype_info;
+    public byte             du_functor_num_sectag_bits; // unused in C# grades
 
     public DuFunctorDesc() {
     }
 
     public void init(
         string functor_name,
-        int orig_arity,
-        int arg_type_contains_var,
+        short orig_arity,
+        ushort arg_type_contains_var,
         Sectag_Locn sectag_locn,
-        int primary,
+        byte primary,
         int secondary,
         int ordinal,
         // XXX why do we need to use object here?
         object arg_types,
         object arg_names,
         object arg_locns,
-        object exist_info)
+        object exist_info,
+        FunctorSubtypeInfo functor_subtype_info,
+        byte num_sectag_bits)
     {
         du_functor_name = functor_name;
         du_functor_orig_arity = orig_arity;
@@ -699,39 +753,97 @@ public class DuFunctorDesc {
         du_functor_arg_names = (string[]) arg_names;
         du_functor_arg_locns = (DuArgLocn[]) arg_locns;
         du_functor_exist_info = (DuExistInfo) exist_info;
+        du_functor_subtype_info = functor_subtype_info;
+        du_functor_num_sectag_bits = num_sectag_bits;
     }
 }
 
 public enum Sectag_Locn {
-    MR_SECTAG_NONE              = 0,
-    MR_SECTAG_NONE_DIRECT_ARG   = 1,
-    MR_SECTAG_LOCAL             = 2,
-    MR_SECTAG_REMOTE            = 3
+    MR_SECTAG_NONE               = 0,
+    MR_SECTAG_NONE_DIRECT_ARG    = 1,
+    MR_SECTAG_LOCAL              = 2,
+    MR_SECTAG_LOCAL_REST_OF_WORD = 2,       // synonym
+    MR_SECTAG_REMOTE             = 3,
+    MR_SECTAG_REMOTE_FULL_WORD   = 3        // synonym
+    // These are never used.
+    // MR_SECTAG_VARIABLE        = 4
+    // These are never used in C# grades.
+    // MR_SECTAG_LOCAL_BITS      = 5
+    // MR_SECTAG_REMOTE_BITS     = 6
+}
+
+public enum FunctorSubtypeInfo {
+    MR_FUNCTOR_SUBTYPE_NONE     = 0,
+    MR_FUNCTOR_SUBTYPE_EXISTS   = 1
 }
 
 public class DuPtagLayout {
-    public int          sectag_sharers;
+    public uint         sectag_sharers;
     public Sectag_Locn  sectag_locn;
     public /* final */ DuFunctorDesc[] sectag_alternatives;
+    public sbyte        sectag_numbits;     // unused in C# grades
+    public byte         du_ptag;
+    public byte         du_ptag_flags;
 
-    public DuPtagLayout(int sharers, Sectag_Locn locn, DuFunctorDesc[] alts) {
-        sectag_sharers = sharers;
-        sectag_locn = locn;
-        sectag_alternatives = alts;
+    private const byte MR_DU_PTAG_FLAG_SECTAG_ALTERNATIVES_INDEXABLE = 0x1;
+
+    public DuPtagLayout(
+        uint sectag_sharers,
+        Sectag_Locn sectag_locn,
+        DuFunctorDesc[] sectag_alts,
+        sbyte sectag_numbits,
+        byte du_ptag,
+        byte du_ptag_flags)
+    {
+        this.sectag_sharers = sectag_sharers;
+        this.sectag_locn = sectag_locn;
+        this.sectag_alternatives = sectag_alts;
+        this.sectag_numbits = sectag_numbits;
+        this.du_ptag = du_ptag;
+        this.du_ptag_flags = du_ptag_flags;
     }
 
-    public DuPtagLayout(int sharers, int locn, DuFunctorDesc[] alts)
-        : this(sharers, (Sectag_Locn)locn, alts)
+    public DuPtagLayout(
+        uint sectag_sharers,
+        int sectag_locn,
+        DuFunctorDesc[] sectag_alts,
+        sbyte sectag_numbits,
+        byte du_ptag,
+        byte du_ptag_flags)
+        : this(sectag_sharers,
+            (Sectag_Locn) sectag_locn,
+            sectag_alts,
+            sectag_numbits,
+            du_ptag,
+            du_ptag_flags)
     {
     }
+
+    public DuFunctorDesc index_or_search_sectag_functor(int sectag) {
+        if (flags_is_sectag_alternatives_indexable()) {
+            return sectag_alternatives[sectag];
+        }
+
+        foreach (DuFunctorDesc desc in sectag_alternatives) {
+            if (desc.du_functor_secondary == sectag) {
+                return desc;
+            }
+        }
+        return null;
+    }
+
+    private bool flags_is_sectag_alternatives_indexable() {
+        return (du_ptag_flags & MR_DU_PTAG_FLAG_SECTAG_ALTERNATIVES_INDEXABLE)
+                != 0;
+    }
 }
 
 public class DuArgLocn {
-    public int arg_offset;
-    public int arg_shift;
-    public int arg_bits;
+    public short arg_offset;
+    public sbyte arg_shift;
+    public sbyte arg_bits;
 
-    public DuArgLocn(int arg_offset, int arg_shift, int arg_bits) {
+    public DuArgLocn(short arg_offset, sbyte arg_shift, sbyte arg_bits) {
         this.arg_offset = arg_offset;
         this.arg_shift = arg_shift;
         this.arg_bits = arg_bits;
@@ -739,9 +851,9 @@ public class DuArgLocn {
 }
 
 public class DuExistInfo {
-    public int exist_typeinfos_plain;
-    public int exist_typeinfos_in_tci;
-    public int exist_tcis;
+    public short exist_typeinfos_plain;
+    public short exist_typeinfos_in_tci;
+    public short exist_tcis;
     public /* final */ DuExistLocn[] exist_typeinfo_locns;
     public /* final */ TypeClassConstraint[] exist_constraints;
 
@@ -749,9 +861,9 @@ public class DuExistInfo {
     }
 
     public void init(
-        int typeinfos_plain,
-        int typeinfos_in_tci,
-        int tcis,
+        short typeinfos_plain,
+        short typeinfos_in_tci,
+        short tcis,
         DuExistLocn[] typeinfo_locns,
         TypeClassConstraint[] constraints)
     {
@@ -764,10 +876,10 @@ public class DuExistInfo {
 }
 
 public class DuExistLocn {
-    public int exist_arg_num;
-    public int exist_offset_in_tci;
+    public short exist_arg_num;
+    public short exist_offset_in_tci;
 
-    public DuExistLocn(int arg_num, int offset_in_tci) {
+    public DuExistLocn(short arg_num, short offset_in_tci) {
         exist_arg_num = arg_num;
         exist_offset_in_tci = offset_in_tci;
     }
@@ -862,14 +974,14 @@ public class TypeClassMethod {
 
 public class EnumFunctorDesc {
     public string   enum_functor_name;
-    public int      enum_functor_ordinal;
+    public int      enum_functor_value;
 
     public EnumFunctorDesc() {
     }
 
-    public void init(string name, int ordinal) {
+    public void init(string name, int val) {
         enum_functor_name = name;
-        enum_functor_ordinal = ordinal;
+        enum_functor_value = val;
     }
 }
 
@@ -956,11 +1068,11 @@ public class UnreachableDefault : System.Exception {
 
 public class ThreadLocalMutables
 {
-    // Child threads do not inherit [ThreadStatic] values so we cannot use the
-    // attribute to implement `thread_local' mutables directly.  Instead, all
-    // thread-local mutables are stored in a single array, which is copied from
-    // the parent thread when a new thread is spawned.  The array indices are
-    // allocated by module initialisation predicates.
+    // Child threads do not inherit [ThreadStatic] values, so we cannot use
+    // the attribute to implement `thread_local' mutables directly. Instead,
+    // all thread-local mutables are stored in a single array, which is copied
+    // from the parent thread when a new thread is spawned. The array indices
+    // are allocated by module initialisation predicates.
     [System.ThreadStatic]
     private static object[] array = null;
 
@@ -1008,8 +1120,8 @@ public class Constants
     public static readonly int MR_PREDICATE     = 0;
     public static readonly int MR_FUNCTION      = 1;
 
-    public static readonly string MR_VERSION    = "14.01.1";
-    public static readonly string MR_FULLARCH   = "x86_64-apple-darwin13.3.0";
+    public static readonly string MR_VERSION    = "22.01.1";
+    public static readonly string MR_FULLARCH   = "x86_64-pc-linux-gnu";
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1021,7 +1133,7 @@ public class Environment
 
 public class LowLevelData
 {
-	// Make a Mercury enumeration with the given integer value.
+    // Make a Mercury enumeration with the given integer value.
     public static object[] make_enum(int enum_value)
     {
         object[] e;
@@ -1029,7 +1141,7 @@ public class LowLevelData
         return e;
     }
 
-	// Make a MR_Word with the given tag and arity.
+    // Make a MR_Word with the given tag and arity.
     public static object[] make_MR_Word(int tag, int arity)
     {
         object[] o = new object[arity + 1];
@@ -1037,15 +1149,15 @@ public class LowLevelData
         return o;
     }
 
-	// Set a field of an MR_Word with a given value.
-	// The first field is at index 1.
+    // Set a field of an MR_Word with a given value.
+    // The first field is at index 1.
     public static void set_MR_Word_field(object[] w, int index, object value)
     {
         w[index] = value;
     }
 
-	// Get the value from an MR_Word.
-	// The first field is at index 1.
+    // Get the value from an MR_Word.
+    // The first field is at index 1.
     public static object get_MR_Word_field(object[] w, int index)
     {
         return w[index];
@@ -1053,7 +1165,7 @@ public class LowLevelData
 
     public static bool list_is_cons(object[] w)
     {
-	    return (System.Convert.ToInt32(w[0]) != 0);
+        return (System.Convert.ToInt32(w[0]) != 0);
     }
 
     public static object list_get_head(object[] w)
@@ -1073,6 +1185,69 @@ class PInvoke {
     public static extern int _putenv(string env);
 }
 
+// A bitmap class for bitmap.m and the rewind code in version_array.m.
+[System.Serializable]
+public class MercuryBitmap {
+    public int num_bits;
+    public byte[] elements;
+    public static int BITS_PER_BYTE = 8;
+
+    public MercuryBitmap(int numBits) {
+        num_bits = numBits;
+        elements = new byte[numBits / 8 + (((numBits % 8) != 0) ? 1: 0)];
+    }
+
+    public override bool Equals(object that) {
+        MercuryBitmap other = that as MercuryBitmap;
+        if (other == null) {
+            return false;
+        }
+        if (num_bits != other.num_bits) {
+            return false;
+        }
+        for (int i = 0; i < elements.Length; i++) {
+            if (elements[i] != other.elements[i]) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    public override int GetHashCode() {
+        return num_bits ^ elements.GetHashCode();
+    }
+
+    public bool GetBit(int bit)
+    {
+        return (elements[ByteIndexForBit(bit)]
+            & (1 << BitIndexWithinByte(bit))) != 0;
+    }
+
+    public void SetBit(int bit)
+    {
+        byte b;
+
+        b = elements[ByteIndexForBit(bit)];
+        b |= (byte) (1 << BitIndexWithinByte(bit));
+        elements[ByteIndexForBit(bit)] = b;
+    }
+
+    public void ClearBit(int bit)
+    {
+        byte b;
+
+        b = elements[ByteIndexForBit(bit)];
+        b &= (byte) ~(1 << BitIndexWithinByte(bit));
+        elements[ByteIndexForBit(bit)] = b;
+    }
+
+    public int ByteIndexForBit(int bit) {
+        return bit / BITS_PER_BYTE;
+    }
+
+    public int BitIndexWithinByte(int bit) {
+        return bit % BITS_PER_BYTE;
+    }
 }
 
-/* vim: set sts=4 sw=4 et: */
+}
