@@ -1,32 +1,27 @@
-/*
-** vim:sw=4 ts=4 expandtab
-*/
-/*
-** Copyright (C) 1995-2006, 2012 The University of Melbourne.
-** This file may only be copied under the terms of the GNU Library General
-** Public License - see the file COPYING.LIB in the Mercury distribution.
-*/
+// vim: ts=4 sw=4 expandtab ft=c
 
-/*
-** mercury_type_info.c -
-**  Definitions for dealing with type_infos needed by the Mercury
-**  runtime system.
-*/
+// Copyright (C) 1995-2006, 2012 The University of Melbourne.
+// Copyright (C) 2014, 2016, 2018 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
+
+// mercury_type_info.c -
+//  Definitions for dealing with type_infos needed by the Mercury
+//  runtime system.
 
 #include "mercury_conf.h"
 #ifndef MR_HIGHLEVEL_CODE
   #include "mercury_imp.h"
 #endif
 #include "mercury_type_info.h"
-#include "mercury_misc.h"           /* for MR_fatal_error() */
-#include "mercury_heap.h"           /* for MR_incr_saved_hp() */
-#include "mercury_builtin_types.h"  /* for void/0's type_ctor_info */
+#include "mercury_misc.h"           // for MR_fatal_error()
+#include "mercury_heap.h"           // for MR_incr_saved_hp()
+#include "mercury_builtin_types.h"  // for void/0's type_ctor_info
 
 MR_ConstString  MR_ctor_rep_name[] = {
     MR_CTOR_REP_NAMES
 };
 
-/*---------------------------------------------------------------------------*/
+////////////////////////////////////////////////////////////////////////////
 
 static MR_PseudoTypeInfo    MR_get_arg_pseudo_type_info(
                                 const MR_PseudoTypeInfoParams params,
@@ -40,7 +35,7 @@ static MR_TypeInfo          MR_get_arg_type_info(
                                 const MR_Word *data_value,
                                 const MR_DuFunctorDesc *functor_desc);
 
-/*---------------------------------------------------------------------------*/
+////////////////////////////////////////////////////////////////////////////
 
 #define usual_func              MR_make_type_info
 #define exist_func              MR_make_type_info_maybe_existq
@@ -87,17 +82,19 @@ static MR_TypeInfo          MR_get_arg_type_info(
 #ifdef MR_NATIVE_GC
   #define ALLOCATE_WORDS(target, size)                              \
     do {                                                            \
-        /* reserve one extra word for GC forwarding pointer */      \
-        /* (see comments in compiler/mlds_to_c.m for details) */    \
+        /*                                                          \
+        ** Reserve one extra word for GC forwarding pointer         \
+        ** (see comments in compiler/mlds_to_c.m for details).      \
+        */                                                          \
         MR_offset_incr_saved_hp((target), 0, 1, NULL, NULL);        \
         MR_offset_incr_saved_hp((target), 0, (size), NULL, NULL);   \
     } while (0)
-#else /* !MR_NATIVE_GC */
+#else // !MR_NATIVE_GC
   #define ALLOCATE_WORDS(target, size)                              \
     do {                                                            \
         MR_offset_incr_saved_hp((target), 0, (size), NULL, NULL);   \
     } while (0)
-#endif /* !MR_NATIVE_GC */
+#endif // !MR_NATIVE_GC
 
 #include "mercury_make_type_info_body.h"
 #undef  usual_func
@@ -121,17 +118,19 @@ static MR_TypeInfo          MR_get_arg_type_info(
 #ifdef MR_NATIVE_GC
   #define ALLOCATE_WORDS(target, size)                              \
     do {                                                            \
-        /* reserve one extra word for GC forwarding pointer */      \
-        /* (see comments in compiler/mlds_to_c.m for details) */    \
+        /*                                                          \
+        ** Reserve one extra word for GC forwarding pointer         \
+        ** (see comments in compiler/mlds_to_c.m for details).      \
+        */                                                          \
         MR_offset_incr_saved_hp((target), 0, 1, NULL, NULL);        \
         MR_offset_incr_saved_hp((target), 0, (size), NULL, NULL);   \
     } while (0)
-#else /* !MR_NATIVE_GC */
+#else // !MR_NATIVE_GC
   #define ALLOCATE_WORDS(target, size)                              \
     do {                                                            \
         MR_offset_incr_saved_hp((target), 0, (size), NULL, NULL);   \
     } while (0)
-#endif /* !MR_NATIVE_GC */
+#endif // !MR_NATIVE_GC
 
 #include "mercury_make_type_info_body.h"
 #undef  usual_func
@@ -156,24 +155,17 @@ MR_get_arg_pseudo_type_info(const MR_PseudoTypeInfoParams params,
     int                     slot;
     int                     offset;
 
-    /*
-    ** Most changes here should also be reflected in MR_get_arg_type_info
-    ** below.
-    */
+    // Most changes here should also be reflected in MR_get_arg_type_info
+    // below.
 
     arg_num = (MR_Unsigned) pseudo_type_info;
 
     if (MR_TYPE_VARIABLE_IS_UNIV_QUANT(pseudo_type_info)) {
-        /*
-        ** This is a universally quantified type variable.
-        */
+        // This is a universally quantified type variable.
         return params[arg_num];
     }
 
-    /*
-    ** This is an existentially quantified type variable.
-    */
-
+    // This is an existentially quantified type variable.
     exist_info = functor_desc->MR_du_functor_exist_info;
     if (exist_info == NULL) {
         MR_fatal_error("MR_get_arg_pseudo_type_info: no exist_info");
@@ -198,38 +190,31 @@ MR_get_arg_type_info(const MR_TypeInfoParams params,
 {
     MR_Unsigned             arg_num;
     const MR_DuExistInfo    *exist_info;
-    MR_DuExistLocn          exist_locn;
+    const MR_DuExistLocn    *exist_locn;
     int                     exist_varnum;
     int                     slot;
     int                     offset;
 
-    /*
-    ** Most changes here should also be reflected in
-    ** MR_get_arg_pseudo_type_info above.
-    */
+    // Most changes here should also be reflected in
+    // MR_get_arg_pseudo_type_info above.
 
     arg_num = (MR_Unsigned) pseudo_type_info;
 
     if (MR_TYPE_VARIABLE_IS_UNIV_QUANT(pseudo_type_info)) {
-        /*
-        ** This is a universally quantified type variable.
-        */
+        // This is a universally quantified type variable.
         return params[arg_num];
     }
 
-    /*
-    ** This is an existentially quantified type variable.
-    */
-
+    // This is an existentially quantified type variable.
     exist_info = functor_desc->MR_du_functor_exist_info;
     if (exist_info == NULL) {
         MR_fatal_error("MR_get_arg_type_info: no exist_info");
     }
 
     exist_varnum = arg_num - MR_PSEUDOTYPEINFO_EXIST_VAR_BASE - 1;
-    exist_locn = exist_info->MR_exist_typeinfo_locns[exist_varnum];
-    slot = exist_locn.MR_exist_arg_num;
-    offset = exist_locn.MR_exist_offset_in_tci;
+    exist_locn = &exist_info->MR_exist_typeinfo_locns[exist_varnum];
+    slot = exist_locn->MR_exist_arg_num;
+    offset = exist_locn->MR_exist_offset_in_tci;
     if (offset < 0) {
         return (MR_TypeInfo) data_value[slot];
     } else {
@@ -250,35 +235,26 @@ MR_compare_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
     int                 i;
     int                 comp;
 
-    /*
-    ** Try to optimize a common case:
-    ** If type_info addresses are equal, they must represent the
-    ** same type.
-    */
+    // Try to optimize a common case:
+    // If type_info addresses are equal, they must represent the same type.
 
     if (pti1 == pti2) {
         return MR_COMPARE_EQUAL;
     }
 
-    /*
-    ** Otherwise, we need to expand equivalence types, if any.
-    */
+    // Otherwise, we need to expand equivalence types, if any.
 
     pti1 = MR_collapse_equivalences_pseudo(pti1);
     pti2 = MR_collapse_equivalences_pseudo(pti2);
 
-    /*
-    ** Perhaps they are equal now...
-    */
+    // Perhaps they are equal now...
 
     if (pti1 == pti2) {
         return MR_COMPARE_EQUAL;
     }
 
-    /*
-    ** Handle the comparison if either pseudo_type_info is a variable.
-    ** Any non-variable is greater than a variable.
-    */
+    // Handle the comparison if either pseudo_type_info is a variable.
+    // Any non-variable is greater than a variable.
 
     if (MR_PSEUDO_TYPEINFO_IS_VARIABLE(pti1) &&
         MR_PSEUDO_TYPEINFO_IS_VARIABLE(pti2))
@@ -300,9 +276,7 @@ MR_compare_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         return MR_COMPARE_GREATER;
     }
 
-    /*
-    ** Otherwise find the type_ctor_infos, and compare those.
-    */
+    // Otherwise find the type_ctor_infos, and compare those.
 
     tci1 = MR_PSEUDO_TYPEINFO_GET_TYPE_CTOR_INFO(pti1);
     tci2 = MR_PSEUDO_TYPEINFO_GET_TYPE_CTOR_INFO(pti2);
@@ -312,19 +286,17 @@ MR_compare_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         return comp;
     }
 
-    /*
-    ** If the type_ctor_infos are equal, we don't need to compare
-    ** the arity of the types - they must be the same - unless they are
-    ** higher-order (which are all mapped to pred/0 or func/0) or tuples
-    ** (which are all mapped to tuple/0), in which cases we must compare
-    ** the arities before we can check the argument types.
-    */
+    // If the type_ctor_infos are equal, we don't need to compare
+    // the arity of the types - they must be the same - unless they are
+    // higher-order (which are all mapped to pred/0 or func/0) or tuples
+    // (which are all mapped to tuple/0), in which cases we must compare
+    // the arities before we can check the argument types.
 
     if (MR_type_ctor_has_variable_arity(tci1)) {
         num_arg_types_1 = MR_PSEUDO_TYPEINFO_GET_VAR_ARITY_ARITY(pti1);
         num_arg_types_2 = MR_PSEUDO_TYPEINFO_GET_VAR_ARITY_ARITY(pti2);
 
-            /* Check arity */
+        // Check the arities.
         if (num_arg_types_1 < num_arg_types_2) {
             return MR_COMPARE_LESS;
         } else if (num_arg_types_1 > num_arg_types_2) {
@@ -339,7 +311,7 @@ MR_compare_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         arg_vector_2 = MR_PSEUDO_TYPEINFO_GET_FIXED_ARITY_ARG_VECTOR(pti2);
     }
 
-        /* compare the argument types */
+    // Compare the argument types.
     for (i = 1; i <= num_arg_types_1; i++) {
         comp = MR_compare_pseudo_type_info(arg_vector_1[i], arg_vector_2[i]);
         if (comp != MR_COMPARE_EQUAL) {
@@ -361,35 +333,26 @@ MR_unify_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
     int                 num_arg_types_2;
     int                 i;
 
-    /*
-    ** Try to optimize a common case:
-    ** If type_info addresses are equal, they must represent the
-    ** same type.
-    */
+    // Try to optimize a common case:
+    // If type_info addresses are equal, they must represent the same type.
 
     if (pti1 == pti2) {
         return MR_TRUE;
     }
 
-    /*
-    ** Otherwise, we need to expand equivalence types, if any.
-    */
+    // Otherwise, we need to expand equivalence types, if any.
 
     pti1 = MR_collapse_equivalences_pseudo(pti1);
     pti2 = MR_collapse_equivalences_pseudo(pti2);
 
-    /*
-    ** Perhaps they are equal now...
-    */
+    // Perhaps they are equal now...
 
     if (pti1 == pti2) {
         return MR_TRUE;
     }
 
-    /*
-    ** Handle the comparison if either pseudo_type_info is a variable.
-    ** Any non-variable is greater than a variable.
-    */
+    // Handle the comparison if either pseudo_type_info is a variable.
+    // Any non-variable is greater than a variable.
 
     if (MR_PSEUDO_TYPEINFO_IS_VARIABLE(pti1) &&
         MR_PSEUDO_TYPEINFO_IS_VARIABLE(pti2))
@@ -409,9 +372,7 @@ MR_unify_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         return MR_FALSE;
     }
 
-    /*
-    ** Otherwise find the type_ctor_infos, and compare those.
-    */
+    // Otherwise find the type_ctor_infos, and compare those.
 
     tci1 = MR_PSEUDO_TYPEINFO_GET_TYPE_CTOR_INFO(pti1);
     tci2 = MR_PSEUDO_TYPEINFO_GET_TYPE_CTOR_INFO(pti2);
@@ -420,19 +381,17 @@ MR_unify_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         return MR_FALSE;
     }
 
-    /*
-    ** If the type_ctor_infos are equal, we don't need to compare
-    ** the arity of the types - they must be the same - unless they are
-    ** higher-order (which are all mapped to pred/0 or func/0) or tuples
-    ** (which are all mapped to tuple/0), in which cases we must compare
-    ** the arities before we can check the argument types.
-    */
+    // If the type_ctor_infos are equal, we don't need to compare
+    // the arity of the types - they must be the same - unless they are
+    // higher-order (which are all mapped to pred/0 or func/0) or tuples
+    // (which are all mapped to tuple/0), in which cases we must compare
+    // the arities before we can check the argument types.
 
     if (MR_type_ctor_has_variable_arity(tci1)) {
         num_arg_types_1 = MR_PSEUDO_TYPEINFO_GET_VAR_ARITY_ARITY(pti1);
         num_arg_types_2 = MR_PSEUDO_TYPEINFO_GET_VAR_ARITY_ARITY(pti2);
 
-            /* Check arity */
+        // Check the arities.
         if (num_arg_types_1 != num_arg_types_2) {
             return MR_FALSE;
         }
@@ -445,7 +404,7 @@ MR_unify_pseudo_type_info(MR_PseudoTypeInfo pti1, MR_PseudoTypeInfo pti2)
         arg_vector_2 = MR_PSEUDO_TYPEINFO_GET_FIXED_ARITY_ARG_VECTOR(pti2);
     }
 
-        /* compare the argument types */
+    // Compare the argument types.
     for (i = 1; i <= num_arg_types_1; i++) {
         if (! MR_unify_pseudo_type_info(arg_vector_1[i], arg_vector_2[i])) {
             return MR_FALSE;
@@ -482,34 +441,25 @@ MR_compare_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
     int             i;
     int             comp;
 
-    /*
-    ** Try to optimize a common case:
-    ** If type_info addresses are equal, they must represent the
-    ** same type.
-    */
+    // Try to optimize a common case:
+    // If type_info addresses are equal, they must represent the same type.
 
     if (ti1 == ti2) {
         return MR_COMPARE_EQUAL;
     }
 
-    /*
-    ** Otherwise, we need to expand equivalence types, if any.
-    */
+    // Otherwise, we need to expand equivalence types, if any.
 
     ti1 = MR_collapse_equivalences(ti1);
     ti2 = MR_collapse_equivalences(ti2);
 
-    /*
-    ** Perhaps they are equal now...
-    */
+    // Perhaps they are equal now...
 
     if (ti1 == ti2) {
         return MR_COMPARE_EQUAL;
     }
 
-    /*
-    ** Otherwise find the type_ctor_infos, and compare those.
-    */
+    // Otherwise find the type_ctor_infos, and compare those.
 
     tci1 = MR_TYPEINFO_GET_TYPE_CTOR_INFO(ti1);
     tci2 = MR_TYPEINFO_GET_TYPE_CTOR_INFO(ti2);
@@ -519,19 +469,17 @@ MR_compare_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
         return comp;
     }
 
-    /*
-    ** If the type_ctor_infos are equal, we don't need to compare
-    ** the arity of the types - they must be the same - unless they are
-    ** higher-order (which are all mapped to pred/0 or func/0) or tuples
-    ** (which are all mapped to tuple/0), in which cases we must compare
-    ** the arities before we can check the argument types.
-    */
+    // If the type_ctor_infos are equal, we don't need to compare
+    // the arity of the types - they must be the same - unless they are
+    // higher-order (which are all mapped to pred/0 or func/0) or tuples
+    // (which are all mapped to tuple/0), in which cases we must compare
+    // the arities before we can check the argument types.
 
     if (MR_type_ctor_has_variable_arity(tci1)) {
         num_arg_types_1 = MR_TYPEINFO_GET_VAR_ARITY_ARITY(ti1);
         num_arg_types_2 = MR_TYPEINFO_GET_VAR_ARITY_ARITY(ti2);
 
-            /* Check arity */
+        // Check the arities.
         if (num_arg_types_1 < num_arg_types_2) {
             return MR_COMPARE_LESS;
         } else if (num_arg_types_1 > num_arg_types_2) {
@@ -546,7 +494,7 @@ MR_compare_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
         arg_vector_2 = MR_TYPEINFO_GET_FIXED_ARITY_ARG_VECTOR(ti2);
     }
 
-        /* compare the argument types */
+    // Compare the argument types.
     for (i = 1; i <= num_arg_types_1; i++) {
         comp = MR_compare_type_info(arg_vector_1[i], arg_vector_2[i]);
         if (comp != MR_COMPARE_EQUAL) {
@@ -568,34 +516,25 @@ MR_unify_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
     int             num_arg_types_2;
     int             i;
 
-    /*
-    ** Try to optimize a common case:
-    ** If type_info addresses are equal, they must represent the
-    ** same type.
-    */
+    // Try to optimize a common case:
+    // If type_info addresses are equal, they must represent the same type.
 
     if (ti1 == ti2) {
         return MR_TRUE;
     }
 
-    /*
-    ** Otherwise, we need to expand equivalence types, if any.
-    */
+    // Otherwise, we need to expand equivalence types, if any.
 
     ti1 = MR_collapse_equivalences(ti1);
     ti2 = MR_collapse_equivalences(ti2);
 
-    /*
-    ** Perhaps they are equal now...
-    */
+    // Perhaps they are equal now...
 
     if (ti1 == ti2) {
         return MR_TRUE;
     }
 
-    /*
-    ** Otherwise find the type_ctor_infos, and compare those.
-    */
+    // Otherwise find the type_ctor_infos, and compare those.
 
     tci1 = MR_TYPEINFO_GET_TYPE_CTOR_INFO(ti1);
     tci2 = MR_TYPEINFO_GET_TYPE_CTOR_INFO(ti2);
@@ -604,19 +543,17 @@ MR_unify_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
         return MR_FALSE;
     }
 
-    /*
-    ** If the type_ctor_infos are equal, we don't need to compare
-    ** the arity of the types - they must be the same - unless they are
-    ** higher-order (which are all mapped to pred/0 or func/0) or tuples
-    ** (which are all mapped to tuple/0), in which cases we must compare
-    ** the arities before we can check the argument types.
-    */
+    // If the type_ctor_infos are equal, we don't need to compare
+    // the arity of the types - they must be the same - unless they are
+    // higher-order (which are all mapped to pred/0 or func/0) or tuples
+    // (which are all mapped to tuple/0), in which cases we must compare
+    // the arities before we can check the argument types.
 
     if (MR_type_ctor_has_variable_arity(tci1)) {
         num_arg_types_1 = MR_TYPEINFO_GET_VAR_ARITY_ARITY(ti1);
         num_arg_types_2 = MR_TYPEINFO_GET_VAR_ARITY_ARITY(ti2);
 
-            /* Check arity */
+        // Check the arities.
         if (num_arg_types_1 != num_arg_types_2) {
             return MR_FALSE;
         }
@@ -629,7 +566,7 @@ MR_unify_type_info(MR_TypeInfo ti1, MR_TypeInfo ti2)
         arg_vector_2 = MR_TYPEINFO_GET_FIXED_ARITY_ARG_VECTOR(ti2);
     }
 
-        /* compare the argument types */
+    // Compare the argument types.
     for (i = 1; i <= num_arg_types_1; i++) {
         if (! MR_unify_type_info(arg_vector_1[i], arg_vector_2[i])) {
             return MR_FALSE;
@@ -650,14 +587,12 @@ MR_compare_type_ctor_info(MR_TypeCtorInfo tci1, MR_TypeCtorInfo tci2)
     int             arity1;
     int             arity2;
 
-    /*
-    ** We are relying on the fact that type_ctor_infos are always
-    ** statically allocated to ensure that two type_ctor_infos are
-    ** for the same type iff their address is the same.
-    **
-    ** The casts to (MR_Unsigned) here are in the hope of increasing
-    ** the chance that this will work on a segmented architecture.
-    */
+    // We are relying on the fact that type_ctor_infos are always
+    // statically allocated to ensure that two type_ctor_infos are
+    // for the same type iff their address is the same.
+    //
+    // The casts to (MR_Unsigned) here are in the hope of increasing
+    // the chance that this will work on a segmented architecture.
 
     if ((MR_Unsigned) tci1 == (MR_Unsigned) tci2) {
         return MR_COMPARE_EQUAL;
@@ -696,14 +631,12 @@ MR_compare_type_ctor_info(MR_TypeCtorInfo tci1, MR_TypeCtorInfo tci2)
 MR_bool
 MR_unify_type_ctor_info(MR_TypeCtorInfo tci1, MR_TypeCtorInfo tci2)
 {
-    /*
-    ** We are relying on the fact that type_ctor_infos are always
-    ** statically allocated to ensure that two type_ctor_infos are
-    ** for the same type iff their address is the same.
-    **
-    ** The casts to (MR_Unsigned) here are in the hope of increasing
-    ** the chance that this will work on a segmented architecture.
-    */
+    // We are relying on the fact that type_ctor_infos are always
+    // statically allocated to ensure that two type_ctor_infos are
+    // for the same type iff their address is the same.
+    //
+    // The casts to (MR_Unsigned) here are in the hope of increasing
+    // the chance that this will work on a segmented architecture.
 
     if ((MR_Unsigned) tci1 == (MR_Unsigned) tci2) {
         return MR_TRUE;
@@ -719,7 +652,7 @@ MR_collapse_equivalences(MR_TypeInfo maybe_equiv_type_info)
 
     type_ctor_info = MR_TYPEINFO_GET_TYPE_CTOR_INFO(maybe_equiv_type_info);
 
-        /* Look past equivalences */
+    // Look past equivalences.
     while (MR_type_ctor_rep(type_ctor_info) == MR_TYPECTOR_REP_EQUIV_GROUND
         || MR_type_ctor_rep(type_ctor_info) == MR_TYPECTOR_REP_EQUIV)
     {
@@ -745,7 +678,7 @@ MR_collapse_equivalences_pseudo(MR_PseudoTypeInfo maybe_equiv_pseudo_type_info)
     type_ctor_info = MR_PSEUDO_TYPEINFO_GET_TYPE_CTOR_INFO(
         maybe_equiv_pseudo_type_info);
 
-        /* Look past equivalences */
+    // Look past equivalences.
     while (MR_type_ctor_rep(type_ctor_info) == MR_TYPECTOR_REP_EQUIV_GROUND
         || MR_type_ctor_rep(type_ctor_info) == MR_TYPECTOR_REP_EQUIV)
     {
@@ -765,19 +698,16 @@ MR_collapse_equivalences_pseudo(MR_PseudoTypeInfo maybe_equiv_pseudo_type_info)
     return maybe_equiv_pseudo_type_info;
 }
 
-/*
-** MR_deallocate() frees up a list of memory cells
-*/
+// MR_deallocate() frees up a list of memory cells.
 
 void
 MR_deallocate(MR_MemoryList allocated)
 {
     while (allocated != NULL) {
         MR_MemoryList next = allocated->next;
-        /*
-        ** These were allocated with MR_GC_NEW_ARRAY_ATTRIB so we must free
-        ** using MR_GC_free_attrib.
-        **/
+        // These were allocated with MR_GC_NEW_ARRAY_ATTRIB so we must free
+        // using MR_GC_free_attrib.
+
         MR_GC_free_attrib(allocated->data);
         MR_GC_free_attrib(allocated);
         allocated = next;
@@ -828,7 +758,7 @@ MR_arg_name_vector_to_list(int arity, const MR_ConstString *arg_names)
     arg_names_list = MR_list_empty();
 
     if (arg_names == NULL) {
-        /* No arguments have names. */
+        // No arguments have names.
         while (arity > 0) {
             --arity;
             arg_names_list = MR_string_list_cons((MR_Word) NULL,
@@ -859,7 +789,7 @@ MR_pseudo_type_info_vector_to_pseudo_type_info_list(int arity,
     pseudo_type_info_list = MR_list_empty();
 
     while (--arity >= 0) {
-            /* Get the argument type_info */
+        // Get the argument type_info.
 
         pseudo = arg_pseudo_type_infos[arity];
         if (MR_PSEUDO_TYPEINFO_IS_VARIABLE(pseudo) &&
@@ -908,25 +838,80 @@ MR_typeclass_ref_error(MR_Word tci, int n, const char *msg)
     fprintf(stderr, "access parameters: %s, %d\n", msg, n);
     MR_fatal_error("typeclass_info reference error");
 
-    /* not reached */
+    /*NOTREACHED*/
     return 0;
 }
 
 int
 MR_cell_size_for_args(int arity, const MR_DuArgLocn *arg_locns)
 {
-    const MR_DuArgLocn *last_arg;
+    int                 last_arg_num;
+    const MR_DuArgLocn  *last_arg;
 
     if (arg_locns == NULL) {
         return arity;
     }
 
-    last_arg = &arg_locns[arity - 1];
-    if (last_arg->MR_arg_bits == -1) {
-        return last_arg->MR_arg_offset + 2;
-    } else {
-        return last_arg->MR_arg_offset + 1;
+    // The meanings of the various special values of MR_arg_{offset,bits}
+    // are documented next to the definition of the MR_DuArgLocn type
+    // in mercury_type_info.h.
+
+    // We are looking for the last argument that occupies any storage.
+    // Since we loop backwards, we stop at the *first* argument we find
+    // that occupies any storage, and decide what to return based on
+    // where that argument ends.
+    last_arg_num = arity - 1;
+    while (last_arg_num >= 0) {
+        last_arg = &arg_locns[last_arg_num];
+
+        if (last_arg->MR_arg_offset < 0) {
+            // We reached the tagword; nothing but dummy args follow.
+            return 1;
+        }
+
+        if (last_arg->MR_arg_bits >= 0) {
+            // This means that argument #last_arg_num takes either
+            // one full word, or part of a word. In the latter case,
+            // the rest of the bits in the word (if any are left) are padding.
+            return last_arg->MR_arg_offset + 1;
+        } else {
+            switch (last_arg->MR_arg_bits) {
+                case -1:
+                case -2:
+                case -3:
+                    // This means that argument #last_arg_num takes two words;
+                    // it can be a double precision float, int64 or uint64.
+                    return last_arg->MR_arg_offset + 2;
+
+                case -4:
+                case -5:
+                case -6:
+                case -7:
+                case -8:
+                case -9:
+                    // This means that argument #last_arg_num is a
+                    // sub-word-sized integer. Treat this case as we treat
+                    // enums.
+                    return last_arg->MR_arg_offset + 1;
+
+                case -10:
+                    // This indicates that argument #last_arg_num is a dummy,
+                    // which means that its offset field is not meaningful.
+                    // Try again with the previous argument (if there is one).
+                    last_arg_num = last_arg_num - 1;
+                    break;
+
+                default:
+                    MR_fatal_error("unknown code value in MR_arg_bits");
+            }
+        }
     }
+
+    // We get here for cells that contain nothing but dummy values.
+    // We can generate them now, but we should be able to optimize them away.
+    // We return 1 because memory allocators should not be asked to allocate
+    // zero words.
+    return 1;
 }
 
 void
@@ -962,4 +947,4 @@ MR_print_type(FILE *fp, MR_TypeInfo type_info)
     }
 }
 
-/*---------------------------------------------------------------------------*/
+////////////////////////////////////////////////////////////////////////////
