@@ -1,5 +1,9 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 % An example program to illustrate the use of all-solutions predicates
-% in Mercury.  This program just prints out all solutions to the
+% in Mercury. This program just prints out all solutions to the
 % predicate hello/1.
 
 % This source file is hereby placed in the public domain.  -fjh (the author).
@@ -8,17 +12,16 @@
 :- interface.
 :- import_module io.
 
-:- pred main(io__state::di, io__state::uo) is det.
+:- pred main(io::di, io::uo) is det.
 
 :- implementation.
 :- import_module solutions.
 
-main --> 
-	{ solutions(hello, List) },
-	io__write_strings(List).
+main(!IO) :-
+    solutions(hello, List),
+    io.write_strings(List, !IO).
 
 :- pred hello(string::out) is multi.
 
 hello("Hello, world\n").
 hello("Hello again, world\n").
-
