@@ -1,5 +1,8 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
 % Regression test.
+%---------------------------------------------------------------------------%
 
 :- module bad_direct_reuse.
 :- interface.
@@ -8,12 +11,12 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- type sym_name
     --->    unqualified(string)
@@ -33,10 +36,7 @@ main(!IO) :-
     io.nl(!IO).
 
 :- func mk = (sym_name::uo).
-:- pragma no_inline(mk/0).
+:- pragma no_inline(func(mk/0)).
 
 mk = SymName :-
     copy(unqualified("builder"), SymName).
-
-%-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=8 sts=4 sw=4 et
