@@ -1,4 +1,8 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
 % Regression test.
+%---------------------------------------------------------------------------%
 
 :- module bad_indirect_reuse2.
 :- interface.
@@ -7,16 +11,16 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
 :- import_module list.
 
 main(!IO) :-
-    copy([1,3,3], U1),
-    copy([1,3,7], U2),
+    copy([1, 3, 3], U1),
+    copy([1, 3, 7], U2),
     H1 = hide(U1),
     H2 = hide(U2),
     my_merge(U1, U2, M),    % bad indirect reuse
@@ -29,8 +33,8 @@ main(!IO) :-
 
     io.write_string("--------\n", !IO),
 
-    copy([0,0,3,5], U3),
-    copy([1,2,2,4], U4),
+    copy([0, 0, 3, 5], U3),
+    copy([1, 2, 2, 4], U4),
     my_merge(U3, U4, MB),   % good indirect reuse
     io.write(MB, !IO),
     io.nl(!IO).
@@ -45,14 +49,11 @@ my_merge([], [], []).
 my_merge([A | As], [], [A | As]).
 my_merge([], [B | Bs], [B | Bs]).
 my_merge([A | As], [B | Bs], [C | Cs]) :-
-    ( compare(>, A, B) ->
+    ( if compare(>, A, B) then
         C = B,
         my_merge([A | As], Bs, Cs)
-    ;
+    else
         % If compare((=), A, B), take A first.
         C = A,
         my_merge(As, [B | Bs], Cs)
     ).
-
-%-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=8 sw=4 et wm=0 tw=0
