@@ -1,6 +1,7 @@
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et tw=0 wm=0 ft=mercury
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+
 :- module bit_buffer_test.
 
 :- interface.
@@ -12,7 +13,6 @@
 :- implementation.
 
 :- import_module assoc_list.
-:- import_module bitmap.
 :- import_module bit_buffer.
 :- import_module bit_buffer.read.
 :- import_module bit_buffer.write.
@@ -27,12 +27,11 @@
 :- import_module univ.
 
 :- type request
-    --->    bits(word, num_bits) 
+    --->    bits(word, num_bits)
     ;       bitmap(bitmap, bit_index, num_bits)
     ;       flush
     ;       pad_to_byte
-    ;       check_buffer_status(stream.result(univ))
-    .
+    ;       check_buffer_status(stream.result(univ)).
 
 :- type read_error
         --->    bits(expected_word :: word, found_word :: word, num_bits)
@@ -48,16 +47,14 @@ main(!IO) :-
     Byte2 = 0b11001100,
     Byte3 = 0b01000110,
     Byte4 = 0b10111001,
-        
+
     % Uncomment this to debug read errors.
     % bit_buffer.read.set_logging_level(1, !IO),
 
     some [!Seq, !ShortSeq, !LongSeq, !ShortBM, !LongBM, !ErrorSeq] (
-        %
         % Test with request sequences that are a minimum of 8 bytes to
         % test overflow even on 64-bit machines (buffers are at least
         % as big as the word size).
-        %
 
         io.write_string("Test reading and writing full bytes.\n", !IO),
         !:Seq = condense(duplicate(4,
@@ -66,19 +63,19 @@ main(!IO) :-
             ++ [check_buffer_status(eof)],
         Seq1 = !.Seq,
         test_sequence(8, !.Seq, !IO),
-            
+
         io.write_string("Test reading and writing partial bytes.\n", !IO),
 
         % This also tests a request split over a flush and handling of
         % a list of requests for which the length is not a multiple of
         % the buffer size.
-        %
+
         !:Seq = condense(duplicate(6,
                     [bits(Byte1, 7), bits(1, 1), bits(Byte2, 6),
                         bits(Byte3, 7), bits(Byte4, 4)])),
         Seq2 = !.Seq,
         test_sequence(8, !.Seq, !IO),
-            
+
         io.write_string(
     "Test flushes when the stream is at a byte boundary and when it is not.\n",
             !IO),
@@ -95,7 +92,7 @@ main(!IO) :-
         % A longer bitmap.
         %
         BM2 = requests_to_bitmap(Seq1 ++ Seq2),
- 
+
         io.write_string("Test simple reading and writing of bitmaps.\n", !IO),
         !:Seq = [bitmap(BM1, 0, num_bits(BM1))],
         test_sequence(8, !.Seq, !IO),
@@ -119,7 +116,7 @@ main(!IO) :-
             !IO),
         !:Seq = [bitmap(BM2, 0, num_bits(BM2))],
         test_sequence(8, !.Seq, !IO),
-        
+
         io.write_string(
     "Test a bitmap starting at a position that isn't on a byte boundary.\n",
             !IO),
@@ -140,7 +137,7 @@ main(!IO) :-
         io.write_string(
             "Test read sequence of bitmaps one byte too long.\n", !IO),
         !:LongBM = shrink_without_copying(copy(BM2), 136),
-        !:ShortBM = shrink_without_copying(copy(BM2), 128), 
+        !:ShortBM = shrink_without_copying(copy(BM2), 128),
         !:ShortSeq = [bitmap(!.ShortBM, 0, num_bits(!.ShortBM))],
         !:LongSeq = [bitmap(!.LongBM, 0, num_bits(!.LongBM))],
         test_error_sequence(io_and_bitmap, 8, !.ShortSeq, !.LongSeq, !IO),
@@ -148,7 +145,7 @@ main(!IO) :-
         io.write_string(
             "Test read sequence of bitmaps one byte too long.\n", !IO),
         !:LongBM = shrink_without_copying(copy(BM2), 136),
-        !:ShortBM = shrink_without_copying(copy(BM2), 128), 
+        !:ShortBM = shrink_without_copying(copy(BM2), 128),
         !:ShortSeq = [bitmap(!.ShortBM, 0, num_bits(!.ShortBM))],
         !:LongSeq = [bitmap(!.LongBM, 0, num_bits(!.LongBM))],
         test_error_sequence(io_and_bitmap, 8, !.ShortSeq, !.LongSeq, !IO),
@@ -186,17 +183,15 @@ main(!IO) :-
     ),
     io.remove_file(bit_buffer_test_tmp_file, _, !IO).
 
-:- pred test_sequence(num_bytes::in, list(request)::in,
-            io::di, io::uo) is det.
+:- pred test_sequence(num_bytes::in, list(request)::in, io::di, io::uo) is det.
 
 test_sequence(BufferSize, Requests0, !IO) :-
-
     % This makes the results for bitmap and I/O buffers consistent.
     Requests = Requests0 ++ [pad_to_byte],
 
-    io.format("Testing with buffer size %d.\n", [i(BufferSize)], !IO), 
+    io.format("Testing with buffer size %d.\n", [i(BufferSize)], !IO),
     TempFile = bit_buffer_test_tmp_file,
-    io.write_string("Testing writes: [", !IO), 
+    io.write_string("Testing writes: [", !IO),
     io.write_list(Requests, ", ", output_request, !IO),
     io.write_string("]\n", !IO),
     io.write_string("Expected result: ", !IO),
@@ -205,7 +200,7 @@ test_sequence(BufferSize, Requests0, !IO) :-
     io.nl(!IO),
     io.flush_output(!IO),
     test_writes(BufferSize, TempFile, Requests, ExpectedBM, !IO),
-    io.write_string("Testing reads:\n", !IO), 
+    io.write_string("Testing reads:\n", !IO),
     test_reads(BufferSize, TempFile, Requests, ExpectedBM, !IO),
     io.write_string("\n", !IO),
     io.flush_output(!IO).
@@ -216,16 +211,14 @@ test_sequence(BufferSize, Requests0, !IO) :-
 :- type error_test_type
     --->    io_and_bitmap
     ;       bitmap_only
-    ;       timebomb(timer)
-    .
+    ;       timebomb(timer).
 
     % SetupRequests will set up a bitmap and the file returned by
     % `bit_buffer_test_tmp_file'.  Requests is a list of requests
     % that will result in a read error when applied to that input.
-    %    
+    %
 :- pred test_error_sequence(error_test_type::in, num_bytes::in,
-        list(request)::in, list(request)::in,
-        io::di, io::uo) is cc_multi.
+    list(request)::in, list(request)::in, io::di, io::uo) is cc_multi.
 
 test_error_sequence(ErrorTestType, BufferSize,
         SetupRequests0, Requests0, !IO) :-
@@ -253,42 +246,42 @@ test_error_sequence(ErrorTestType, BufferSize,
     io.nl(!IO),
     ExpectedBM = requests_to_bitmap(SetupRequests),
     TempFile = bit_buffer_test_tmp_file,
-    (
+    ( if
         ( ErrorTestType = io_and_bitmap
         ; ErrorTestType = timebomb(_)
         )
-    ->
+    then
         test_writes(8, TempFile, SetupRequests, ExpectedBM, !IO)
-    ;
+    else
         true
     ),
-    (
+    ( if
         ( ErrorTestType = io_and_bitmap
         ; ErrorTestType = bitmap_only
         )
-    ->
+    then
         check_that_error_occurs("bitmap",
             test_bitmap_reads(Requests, ExpectedBM),
             !IO)
-    ;
+    else
         true
     ),
-    (
+    ( if
         ErrorTestType = io_and_bitmap
-    ->
+    then
         check_that_error_occurs("I/O",
             test_io_reads(BufferSize, TempFile, Requests),
             !IO)
-    ;
+    else
         true
     ),
-    (
+    ( if
         ErrorTestType = timebomb(Timer)
-    ->
+    then
         check_that_error_occurs("stream read error",
             test_io_timebomb_reads(BufferSize, Timer, TempFile, Requests),
             !IO)
-    ;
+    else
         true
     ),
 
@@ -337,7 +330,7 @@ output_request(check_buffer_status(BufferStatus), !IO) :-
     io.write_string(")", !IO).
 
 :- pred test_writes(num_bytes::in, string::in, list(request)::in,
-        bitmap::in, io::di, io::uo) is det.
+    bitmap::in, io::di, io::uo) is det.
 
 test_writes(BufferSize, FileName, Writes, ExpectedBM, !IO) :-
     io.open_binary_output(FileName, WriteOpenRes, !IO),
@@ -355,9 +348,9 @@ test_writes(BufferSize, FileName, Writes, ExpectedBM, !IO) :-
             io.close_binary_output(WriteStream, !IO)
         ),
 
-        ( BM = ExpectedBM ->
+        ( if BM = ExpectedBM then
             io.write_string("Collected bitmap compares OK.\n", !IO)
-        ;
+        else
             io.write_string("Collected bitmap differs: \n", !IO),
             io.write_string(to_byte_string(BM), !IO),
             io.nl(!IO),
@@ -370,10 +363,10 @@ test_writes(BufferSize, FileName, Writes, ExpectedBM, !IO) :-
             io.read_binary_file_as_bitmap(ReadStream, BMReadResult, !IO),
             (
                 BMReadResult = ok(ReadBM),
-                ( ReadBM = ExpectedBM ->
+                ( if ReadBM = ExpectedBM then
                     io.write_string("I/O bitmap compares OK.\n", !IO),
                     io.flush_output(!IO)
-                ;
+                else
                     io.write_string("I/O bitmap differs: \n", !IO),
                     io.write_string(to_byte_string(ReadBM), !IO),
                     io.nl(!IO),
@@ -405,18 +398,18 @@ test_writes(BufferSize, FileName, Writes, ExpectedBM, !IO) :-
     <= stream.writer(S, bitmap.slice, St).
 
 do_write(bits(Word, NumBits), !Buffer) :-
-    ( NumBits = 1 ->
-        Bit = ( Word = 0 -> no ; yes ),
+    ( if NumBits = 1 then
+        Bit = ( if Word = 0 then no else yes ),
         put_bit(Bit, !Buffer)
-    ; NumBits = 8 ->
+    else if NumBits = 8 then
         put_byte(Word, !Buffer)
-    ;
+    else
         put_bits(Word, NumBits, !Buffer)
     ).
 do_write(bitmap(BM, Index, NumBits), !Buffer) :-
-    ( Index = 0, NumBits = BM ^ num_bits ->
+    ( if Index = 0, NumBits = BM ^ num_bits then
         put_bitmap(BM, !Buffer)
-    ;
+    else
         put_bitmap(BM, Index, NumBits, !Buffer)
     ).
 do_write(pad_to_byte, !Buffer) :-
@@ -428,12 +421,12 @@ do_write(check_buffer_status(_), !Buffer).
 
     % Create a bitmap directly from the list of requests.
 :- func requests_to_bitmap(list(request)::in) =
-            (bitmap::bitmap_uo) is det.
+    (bitmap::bitmap_uo) is det.
 
 requests_to_bitmap(Requests) = !:BM :-
-        Size = request_list_length(Requests, 0),
-        !:BM = bitmap.init(Size),
-        list.foldl2(request_to_bitmap, Requests, 0, _, !BM).
+    Size = request_list_length(Requests, 0),
+    !:BM = bitmap.init(Size),
+    list.foldl2(request_to_bitmap, Requests, 0, _, !BM).
 
 :- func request_list_length(list(request), int) = int.
 
@@ -442,7 +435,7 @@ request_list_length([Req | Reqs], L0) = L :-
     ( Req = bits(_, NumBits)
     ; Req = bitmap(_, _, NumBits)
     ; Req = pad_to_byte, Rem = L0 `rem` bits_per_byte,
-        NumBits = ( Rem = 0 -> 0 ; bits_per_byte - Rem )
+        NumBits = ( if Rem = 0 then 0 else bits_per_byte - Rem )
     ; Req = flush, NumBits = 0
     ; Req = check_buffer_status(_), NumBits = 0
     ),
@@ -459,14 +452,14 @@ request_to_bitmap(bitmap(OtherBM, Start, NumBits), !Index, !BM) :-
     !:Index = !.Index + NumBits.
 request_to_bitmap(pad_to_byte, !Index, !BM) :-
     Rem = !.Index `rem` bits_per_byte,
-    NumBits = ( Rem = 0 -> 0 ; bits_per_byte - Rem ),
+    NumBits = ( if Rem = 0 then 0 else bits_per_byte - Rem ),
     !:BM = !.BM ^ bits(!.Index, NumBits) := 0,
     !:Index = !.Index + NumBits.
 request_to_bitmap(flush, !Index, !BM).
 request_to_bitmap(check_buffer_status(_), !Index, !BM).
 
 :- pred test_reads(num_bytes::in, string::in, list(request)::in,
-        bitmap::in, io::di, io::uo) is det.
+    bitmap::in, io::di, io::uo) is det.
 
 test_reads(BufferSize, FileName, Requests, ExpectedBM, !IO) :-
     test_bitmap_reads(Requests, ExpectedBM, !IO),
@@ -480,9 +473,9 @@ test_bitmap_reads(Requests, ExpectedBM, !IO) :-
         !:BMBuffer = new_bitmap_reader(ExpectedBM, 0, ExpectedBM ^ num_bits),
         do_reads("bitmap", 1, Requests, !BMBuffer),
         finalize(!.BMBuffer, _, _, _, _, BMNumFinalBits),
-        ( BMNumFinalBits = 0 ->
+        ( if BMNumFinalBits = 0 then
             true
-        ;
+        else
             throw(string.format("bitmap reader has %d bits left over: \n",
                 [i(BMNumFinalBits)]) : string)
         ),
@@ -499,9 +492,9 @@ test_io_reads(BufferSize, FileName, Requests, !IO) :-
         !:IOBuffer = new(BufferSize, ReadStream, !.IO) : io_read_buffer,
         do_reads("I/O", 1, Requests, !IOBuffer),
         finalize(!.IOBuffer, _, !:IO, _, _, IONumFinalBits),
-        ( IONumFinalBits = 0 ->
+        ( if IONumFinalBits = 0 then
             true
-        ;
+        else
             throw(string.format("I/O reader has %d bits left over: \n",
                 [i(IONumFinalBits)]): string)
         ),
@@ -521,16 +514,16 @@ test_io_timebomb_reads(BufferSize, Countdown, FileName, Requests, !IO) :-
         ReadOpenRes = ok(ReadStream),
         ErrorState0 = 'new timebomb_state'(ReadStream, !.IO, Countdown),
         !:ErrorBuffer = bit_buffer.read.new(BufferSize,
-                            timebomb_byte_stream,
-                            unsafe_promise_unique(ErrorState0)) :
-                            read_buffer(timebomb_byte_stream,
-                                timebomb_state, timebomb_error),
-        
+            timebomb_byte_stream,
+            unsafe_promise_unique(ErrorState0)) :
+            read_buffer(timebomb_byte_stream,
+                timebomb_state, timebomb_error),
+
         do_reads("timebomb", 1, Requests, !ErrorBuffer),
         finalize(!.ErrorBuffer, _, ErrorState, _, _, ErrorNumFinalBits),
-        ( ErrorNumFinalBits = 0 ->
+        ( if ErrorNumFinalBits = 0 then
             true
-        ;
+        else
             throw(string.format("timebomb reader has %d bits left over: \n",
                 [i(ErrorNumFinalBits)]) : string)
         ),
@@ -560,16 +553,16 @@ do_reads(Desc, Index, [Req | Reqs], !Buffer) :-
 
 do_read(Desc, ReqIndex, bits(ExpectedWord0, NumBits), !Buffer) :-
     ExpectedWord = mask_word(ExpectedWord0, NumBits),
-    ( NumBits = 1 ->
+    ( if NumBits = 1 then
         get_bit(GetResult, !Buffer),
         (
             GetResult = ok(ResultBit),
-            ResultWord = ( ResultBit = yes -> 1 ; 0 ),
-            ( ResultWord = ExpectedWord ->
+            ResultWord = ( if ResultBit = yes then 1 else 0 ),
+            ( if ResultWord = ExpectedWord then
                 true
-            ;
+            else
                 throw_read_result_error(bits(ExpectedWord, ResultWord, 1),
-                    Desc, ReqIndex) 
+                    Desc, ReqIndex)
             )
         ;
             GetResult = eof,
@@ -578,17 +571,17 @@ do_read(Desc, ReqIndex, bits(ExpectedWord0, NumBits), !Buffer) :-
             GetResult = error(Err),
             throw(Err)
         )
-    ;
+    else
         get_bits(bits_per_int - NumBits, NumBits, 0, ResultWord,
             NumBitsRead, GetResult, !Buffer),
         (
             GetResult = ok,
-            ( NumBitsRead = NumBits, ExpectedWord = ResultWord ->
+            ( if NumBitsRead = NumBits, ExpectedWord = ResultWord then
                 true
-            ;
+            else
                 throw_read_result_error(
                     bits(ExpectedWord, ResultWord, NumBits),
-                    Desc, ReqIndex) 
+                    Desc, ReqIndex)
             )
         ;
             GetResult = error(Err),
@@ -599,9 +592,9 @@ do_read(Desc, ReqIndex, bits(ExpectedWord0, NumBits), !Buffer) :-
 do_read(Desc, ReqIndex, bitmap(SourceBM, Index, NumBits), !Buffer) :-
     some [!BM] (
         !:BM = bitmap.init(SourceBM ^ num_bits),
-        ( Index = 0, NumBits = SourceBM ^ num_bits ->
+        ( if Index = 0, NumBits = SourceBM ^ num_bits then
             get_bitmap(!BM, BitsRead, GetResult, !Buffer)
-        ;
+        else
             get_bitmap(Index, NumBits, !BM, BitsRead, GetResult, !Buffer)
         ),
         (
@@ -609,12 +602,12 @@ do_read(Desc, ReqIndex, bitmap(SourceBM, Index, NumBits), !Buffer) :-
             ExpectedBM0 = bitmap.init(SourceBM ^ num_bits),
             ExpectedBM = copy_bits(SourceBM, Index,
                             ExpectedBM0, Index, NumBits),
-            ( ExpectedBM = !.BM ->
+            ( if ExpectedBM = !.BM then
                 true
-            ;
+            else
                 throw_read_result_error(
                     bitmap(ExpectedBM, !.BM, NumBits, BitsRead),
-                    Desc, ReqIndex) 
+                    Desc, ReqIndex)
             )
         ;
             GetResult = error(Err),
@@ -629,12 +622,11 @@ do_read(Desc, ReqIndex, check_buffer_status(ExpectedStatus), !Buffer) :-
     buffer_status(FoundStatus0, !Buffer),
     ( FoundStatus0 = ok, FoundStatus = ok
     ; FoundStatus0 = eof, FoundStatus = eof
-    ; FoundStatus0 = error(Err),
-        FoundStatus = error(univ(Err))
+    ; FoundStatus0 = error(Err), FoundStatus = error(univ(Err))
     ),
-    ( ExpectedStatus = FoundStatus ->
+    ( if ExpectedStatus = FoundStatus then
         true
-    ;
+    else
         throw_read_result_error(
             check_buffer_status(ExpectedStatus, FoundStatus),
             Desc, ReqIndex)
@@ -657,11 +649,11 @@ throw_read_stream_error(Error, Desc, ReqIndex) :-
 :- func mask_word(word, num_bits) = word.
 
 mask_word(ExpectedWord0, N) = ExpectedWord :-
-    ( N \= 0 ->
+    ( if N \= 0 then
         BitMask  = 1 `unchecked_left_shift` (N - 1),
         BitsMask = BitMask \/ (BitMask - 1),
         ExpectedWord = ExpectedWord0 /\ BitsMask
-    ;
+    else
         ExpectedWord = 0
     ).
 
@@ -680,7 +672,7 @@ bit_buffer_test_tmp_file = "bit_buffer_test_tmp".
         timebomb_state(timebombed_stream :: Stream,
             timebombed_state :: State, countdown :: int) =>
         (reader(Stream, byte, State, Error),
-        bulk_reader(Stream, int, bitmap, State, Error)). 
+        bulk_reader(Stream, int, bitmap, State, Error)).
 
 :- type timebomb_error
         --->    bang
@@ -705,12 +697,12 @@ bit_buffer_test_tmp_file = "bit_buffer_test_tmp".
 [
     (get(_Stream, Result, !.State, unsafe_promise_unique(!:State)) :-
         !.State = timebomb_state(TStream, TState0, Countdown0),
-        ( Countdown0 < 0 ->
+        ( if Countdown0 < 0 then
             Result = error(already_exploded)
-        ; Countdown0 = 0 ->
+        else if Countdown0 = 0 then
             Result = error(bang),
             !:State = 'new timebomb_state'(TStream, TState0, -1)
-        ;
+        else
             get(TStream, ByteResult, unsafe_promise_unique(TState0), TState),
             (
                 ByteResult = ok(Byte),
@@ -737,25 +729,25 @@ bit_buffer_test_tmp_file = "bit_buffer_test_tmp".
     (bulk_get(_, Index, NumBytes, !BM, NumBytesRead, Result,
             !.State, unsafe_promise_unique(!:State)) :-
         !.State = timebomb_state(TStream, TState0, Countdown0),
-        ( Countdown0 < 0 ->
+        ( if Countdown0 < 0 then
             NumBytesRead = 0,
             Result = error(already_exploded)
-        ; Countdown0 = 0 ->
+        else if Countdown0 = 0 then
             NumBytesRead = 0,
             Result = error(bang),
             !:State = 'new timebomb_state'(TStream, TState0, -1)
-        ;
+        else
             unsafe_promise_unique(TState0, TState1),
             bulk_get(TStream, Index, NumBytes, !BM, NumBytesRead0,
                 BulkGetResult, TState1, TState),
             (
                 BulkGetResult = ok,
-                ( NumBytesRead0 >= Countdown0 ->
+                ( if NumBytesRead0 >= Countdown0 then
                     NumBytesRead = Countdown0,
                     Result = error(bang)
-                ;
+                else
                     NumBytesRead = NumBytesRead0,
-                    Result = ok    
+                    Result = ok
                 ),
                 Countdown = Countdown0 - NumBytesRead
             ;
@@ -768,4 +760,3 @@ bit_buffer_test_tmp_file = "bit_buffer_test_tmp".
         )
     )
 ].
-
