@@ -1,6 +1,11 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 % A construction of a direct argument functor with a free argument was
-% completely ignored.  This test case caused a compiler abort during LLDS
+% completely ignored. This test case caused a compiler abort during LLDS
 % code generation.
+%
 
 :- module direct_arg_partial_inst.
 :- interface.
@@ -9,8 +14,8 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -27,14 +32,14 @@
 :- pred foo(list(fruit)::in, string::out) is det.
 
 foo(Xs, R) :-
-    (
+    ( if
         % X was not recorded in the var_state_map.
         X = apple(_),
         % Compiler tries to flush X to the stack.
-        member(X, Xs)
-    ->
+        list.member(X, Xs)
+    then
         R = "found"
-    ;
+    else
         R = "not found"
     ).
 
@@ -48,6 +53,3 @@ main(!IO) :-
     foo(ListB, ResultB),
     io.write_string(ResultB, !IO),
     io.nl(!IO).
-
-%-----------------------------------------------------------------------------%
-% vim: set sts=4 sw=4 et:
