@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 :- module foreign_import_module_2.
 
 :- interface.
@@ -9,16 +13,15 @@
 :- import_module int.
 
 :- pragma foreign_export("C", foo(in, out), "foo").
-:- pragma foreign_export("IL", foo(in, out), "foo").
 :- pragma foreign_export("C#", foo(in, out), "foo").
 :- pragma foreign_export("Java", foo(in, out), "foo").
-:- pragma foreign_export("Erlang", foo(in, out), "foo").
 
 foo(X, X+1).
 
-:- pragma foreign_code("C#", "
-	static void foo2(int X, ref int Y)
-	{
-		Y = X + 1;
-	}
+:- pragma foreign_code("C#",
+"
+    static void foo2(int X, ref int Y)
+    {
+        Y = X + 1;
+    }
 ").
