@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 :- module prince_frameopt_css.style.
 
 :- interface.
@@ -11,11 +15,11 @@
 
 :- type max_width
     --->    length(length)
-    ;	    percent(float).
+    ;       percent(float).
 
 :- type spec(T)
     --->    inherit
-    ;	    value(T).
+    ;       value(T).
 
 :- implementation.
 
@@ -23,17 +27,19 @@
 
 get_max_width(V) =
     ( if get_percent(V) = W then
-	percent(W)
+        percent(W)
     else
-	length(get_length(V))
+        length(get_length(V))
     ).
 
 :- func spec((func(value) = T), value) = spec(T).
 :- mode spec((func(in) = out is det), in) = out is det.
 
 spec(F, V) =
-    (    if V = ident("inherit") then inherit
-    else value(F(V))
+    ( if V = ident("inherit") then
+        inherit
+    else
+        value(F(V))
     ).
 
 new_prules = [max_width(spec(get_max_width, percent(int(100))))].
