@@ -1,7 +1,12 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 % Regression test.
 % Previously a singleton switch could be replaced by the case goal, possibly
 % with a functor test beforehand, but that's only true if the case arm is
 % applicable to only a single functor.
+%
 
 :- module simplify_multi_arm_switch.
 :- interface.
@@ -10,8 +15,8 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -41,11 +46,8 @@ squishy(Fruit) :-
     IsSquishy = yes.
 
 main(!IO) :-
-    ( squishy(apple) ->
+    ( if squishy(apple) then
         io.write_string("bug!\n", !IO)
-    ;
+    else
         io.write_string("ok\n", !IO)
     ).
-
-%-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=8 sts=4 sw=4 et
