@@ -3,5 +3,5 @@
 
 void c_write_string(char *string)
 {
-	printf(string);
+	printf("%s", string);
 }
