@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
 % Compiling this module should generate an error message since
 % it tries to cast a non-standard func inst to ground.
 
@@ -6,26 +10,29 @@
 :- interface.
 :- import_module io.
 
-:- pred main(io__state, io__state).
-:- mode main(di, uo) is det.
+:- pred main(io::di, io::uo) is det.
 
 :- implementation.
 
+:- include_module ho_default_func_2__sub.
+:- include_module ho_default_func_2__id.
+
 :- import_module ho_default_func_2__sub.
 :- import_module ho_default_func_2__id.
 
-:- import_module int, std_util.
+:- import_module int.
+:- import_module std_util.
 
-main -->
-	{ baz(IdF), eq(getval(IdF), F) },
-	do_io(F).
+main(!IO) :-
+    baz(IdF),
+    eq(getval(IdF), F),
+    do_io(F, !IO).
 
 :- func foo(int) = int.
+
 foo(X) = X + 1.
 
 :- func bar(int) = int.
 :- mode bar(out) = in is det.
-bar(X) = X + 1.
 
-:- include_module ho_default_func_2__sub.
-:- include_module ho_default_func_2__id.
+bar(X) = X + 1.
