@@ -1,6 +1,6 @@
-%-----------------------------------------------------------------------------%
-% vim: ft=mercury ts=4 sw=4 et wm=0 tw=0
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+% vim: ft=mercury ts=4 sw=4 et
+%---------------------------------------------------------------------------%
 %
 % This is a regression test. In versions of the compiler before 7 Aug 2007,
 % it used to cause a link error.
@@ -13,7 +13,7 @@
 % eliminate the procedure itself, since doing so would also
 % eliminate the definition of the global variable.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % mercury_java_parser_memoed.m
 % Ralph Becket <rafe@cs.mu.oz.au>
 % Mon Feb 21 09:42:40 EST 2005
@@ -36,7 +36,7 @@
 % pragma memo lines except those for literal/2, qualified_identifier/2, and
 % punct/2.
 %
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- module mercury_java_parser_dead_proc_elim_bug.
 
@@ -46,8 +46,8 @@
 
 :- pred main(io::di, io::uo) is det.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- implementation.
 
@@ -71,7 +71,7 @@
     %
 :- type ps == int.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 main(!IO) :-
     io.command_line_arguments(Args, !IO),
@@ -97,22 +97,15 @@ parse_file(Filename, !IO) :-
         throw(Result)
     ).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Low-level predicates.
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
-
+%---------------------------------------------------------------------------%
 
 :- pragma foreign_decl("C", "
-
     MR_String   input_string = NULL;
     MR_Word     input_length = (MR_Word) 0;
-
 ").
 
-%-----------------------------------------------------------------------------%
-
 :- impure pred set_input_string(string::in) is det.
 
 :- pragma foreign_proc("C",
@@ -123,8 +116,6 @@ parse_file(Filename, !IO) :-
     input_length = strlen(Str);
 ").
 
-%-----------------------------------------------------------------------------%
-
 :- semipure pred input_string_and_length(string::out, int::out) is det.
 
 :- pragma foreign_proc("C",
@@ -135,23 +126,21 @@ parse_file(Filename, !IO) :-
     Length = input_length;
 ").
 
-
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred current_offset(int::out, int::in, int::out) is det.
 
 current_offset(Offset, Offset, Offset).
 
-%-----------------------------------------------------------------------------%
-
 :- pred eof(ps::in, ps::out) is semidet.
 
 eof(Offset, Offset) :-
-    promise_pure (                    semipure input_string_and_length(_Str, Length),
-    Offset = Length
+    promise_pure (
+        semipure input_string_and_length(_Str, Length),
+        Offset = Length
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
     % XXX These are really semipure, but I'm being naughty and promising them
     % to be pure because I don't want to pollute my code with impurity
@@ -162,31 +151,34 @@ eof(Offset, Offset) :-
 :- pred char(char::out, ps::in, ps::out) is semidet.
 
 char(Char, Offset, Offset + 1) :-
-    promise_pure (                    semipure input_string_and_length(Str, Length),
-    Offset < Length,
-    Char = Str ^ unsafe_elem(Offset)
+    promise_pure (
+        semipure input_string_and_length(Str, Length),
+        Offset < Length,
+        Char = Str ^ unsafe_elem(Offset)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred input_substring(int::in, int::in, string::out,
     ps::in, ps::out) is semidet.
 
 input_substring(Start, End, Substring, Offset, Offset) :-
-    promise_pure (                    semipure input_string_and_length(Str, Length),
-    End =< Length,
-    Substring = unsafe_substring(Str, Start, End - Start)
+    promise_pure (
+        semipure input_string_and_length(Str, Length),
+        End =< Length,
+        Substring = unsafe_between(Str, Start, End)
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred match_string(string::in, int::in, int::out) is semidet.
 
 match_string(MatchStr, Offset, Offset + N) :-
-    promise_pure (                    semipure input_string_and_length(Str, Length),
-    N = length(MatchStr),
-    Offset + N =< Length,
-    match_string_2(0, N, MatchStr, Offset, Str)
+    promise_pure (
+        semipure input_string_and_length(Str, Length),
+        N = length(MatchStr),
+        Offset + N =< Length,
+        match_string_2(0, N, MatchStr, Offset, Str)
     ).
 
 :- pred match_string_2(int::in, int::in, string::in, int::in, string::in)
@@ -200,11 +192,9 @@ match_string_2(I, N, MatchStr, Offset, Str) :-
         true
     ).
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Utility predicates.
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred optional_det(
     pred(ps, ps):: in(pred(in, out) is semidet),
@@ -217,8 +207,6 @@ optional_det(P) -->
         []
     ).
 
-%-----------------------------------------------------------------------------%
-
 :- pred zero_or_more_det(
     pred(ps, ps)::in(pred(in, out) is semidet),
     ps::in, ps::out) is det.
@@ -230,8 +218,6 @@ zero_or_more_det(P) -->
         []
     ).
 
-%-----------------------------------------------------------------------------%
-
 :- pred one_or_more(
     pred(ps, ps)::in(pred(in, out) is semidet),
     ps::in, ps::out) is semidet.
@@ -240,8 +226,6 @@ one_or_more(P) -->
     P,
     zero_or_more_det(P).
 
-%-----------------------------------------------------------------------------%
-
 :- pred brackets(string::in,
     pred(ps, ps)::in(pred(in, out) is semidet),
     string::in, ps::in, ps::out) is semidet.
@@ -260,8 +244,6 @@ brackets_detarg(L, P, R) -->
     P,
     punct(R).
 
-%-----------------------------------------------------------------------------%
-
 :- pred seq(
     pred(ps, ps)::in(pred(in, out) is semidet),
     pred(ps, ps)::in(pred(in, out) is semidet),
@@ -273,17 +255,13 @@ seq(P, Q) -->
     P,
     Q.
 
-%-----------------------------------------------------------------------------%
-
 :- pred comma_separated_list(
     pred(ps, ps)::in(pred(in, out) is semidet),
     ps::in, ps::out) is semidet.
 
 comma_separated_list(P) -->
     P,
-    zero_or_more_det(seq(punct(","), P)).
-
-%-----------------------------------------------------------------------------%
+    zero_or_more_det(seq(punct(", "), P)).
 
 :- pred whitespace(ps::in, ps::out) is semidet.
 
@@ -320,7 +298,7 @@ skip_to_end_of_trad_comment -->
         skip_to_end_of_trad_comment
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred punct(string::in, ps::in, ps::out) is semidet.
 
@@ -328,8 +306,6 @@ punct(Punct) -->
     match_string(Punct),
     whitespace.
 
-%-----------------------------------------------------------------------------%
-
 :- pred keyword(string::in, ps::in, ps::out) is semidet.
 
 keyword(Keyword) -->
@@ -337,8 +313,6 @@ keyword(Keyword) -->
     not(java_identifier_part),
     whitespace.
 
-%-----------------------------------------------------------------------------%
-
 :- pred keyword(string::in,
     pred(ps, ps)::in(pred(in, out) is semidet),
     ps::in, ps::out) is semidet.
@@ -349,12 +323,11 @@ keyword(Keyword, P) -->
     whitespace,
     P.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
-:- pred java_identifier( /*string::out,*/ ps::in, ps::out)
+:- pred java_identifier( /*string::out, */ ps::in, ps::out)
     is semidet.
 
-
 java_identifier/*(Identifier)*/ -->
 %   current_offset(Start),
     java_identifier_start,
@@ -375,7 +348,7 @@ java_identifier_part -->
     char(C),
     { char.is_alnum_or_underscore(C) ; C = ('$') }.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred floating_point_literal(ps::in, ps::out) is semidet.
 
@@ -416,7 +389,7 @@ float_type_suffix -->
     char(C),
     { C = ('F') ; C = ('f') ; C = ('D') ; C = ('d') }.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred integer_literal(ps::in, ps::out) is semidet.
 
@@ -473,9 +446,9 @@ digits(Base) -->
 
 digit(Base) -->
     char(C),
-    { char.digit_to_int(C, D), D < Base }.
+    { char.decimal_digit_to_int(C, D), D < Base }.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred character_literal(ps::in, ps::out) is semidet.
 
@@ -509,7 +482,7 @@ possibly_escaped_char -->
         []
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred string_literal(ps::in, ps::out) is semidet.
 
@@ -524,7 +497,7 @@ string_char -->
     not(char('"')),
     possibly_escaped_char.
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred boolean_literal(ps::in, ps::out) is semidet.
 
@@ -537,28 +510,22 @@ boolean_literal -->
         keyword("false")
     ).
 
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred null_literal(ps::in, ps::out) is semidet.
 
 null_literal -->
     keyword("null").
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 % Taken from
 % http://java.sun.com/docs/books/jls/second_edition/html/syntax.doc.html
-
-%      _________________________________________________________________
 %
 %    [5]Contents | [6]Prev | [7]Next | [8]Index Java Language Specification
 %    Second Edition
 %
 %    [9]Copyright 2000 Sun Microsystems, Inc. All rights reserved
 %    Please send any comments or corrections to the [10]JLS team
-
-%                                     Syntax
-%      _________________________________________________________________
 %
 %    This chapter presents a grammar for the Java programming language.
 %
@@ -572,7 +539,7 @@ null_literal -->
 %      * [x] denotes zero or one occurrences of x.
 %      * {x} denotes zero or more occurrences of x.
 %      * x | y means one of either x or y.
-
+%
 % Identifier:
 %         IDENTIFIER
 %
@@ -652,7 +619,7 @@ assignment_operator_expression1 -->
 %         *=
 %         /=
 %         &=
-%         |=
+%         | =
 %         ^=
 %         %=
 %         <<=
@@ -687,7 +654,7 @@ assignment_operator -->
     then
         []
     else if
-        punct("|=")
+        punct(" | =")
     then
         []
     else if
@@ -798,7 +765,7 @@ infix_op_expression3 -->
     expression3.
 
 % Infixop:
-%         ||
+%         | |
 %         &&
 %         |
 %         ^
@@ -822,7 +789,7 @@ infix_op_expression3 -->
 
 infix_op -->
     ( if
-        punct("||")
+        punct(" | |")
     then
         []
     else if
@@ -830,7 +797,7 @@ infix_op -->
     then
         []
     else if
-        punct("|")
+        punct(" | ")
     then
         []
     else if
@@ -1269,7 +1236,7 @@ class_creator_rest -->
     optional_det(class_body).
 
 % ArrayInitializer:
-%         { [VariableInitializer {, VariableInitializer} [,]] }
+%         { [VariableInitializer {, VariableInitializer} [, ]] }
 
 :- pred array_initializer(ps::in, ps::out) is semidet.
 
@@ -1283,7 +1250,7 @@ array_initializer -->
 
 array_initializer_body -->
     comma_separated_list(variable_initializer),
-    optional_det(punct(",")).
+    optional_det(punct(", ")).
 
 % VariableInitializer:
 %         ArrayInitializer
@@ -1924,7 +1891,7 @@ method_or_field_decl -->
 %         VariableDeclaratorRest
 %         MethodDeclaratorRest
 %         XXX First should be
-%           VariableDeclaratorRest [',' VariableDeclarators]
+%           VariableDeclaratorRest [', ' VariableDeclarators]
 
 :- pred method_or_field_rest(ps::in, ps::out) is semidet.
 
@@ -1935,7 +1902,7 @@ method_or_field_rest -->
         []
     else
         variable_declarator_rest,
-        ( if punct(",") then
+        ( if punct(", ") then
             variable_declarators
         else
             []
@@ -2116,8 +2083,8 @@ formal_parameter -->
 method_body -->
     block.
 
-%-----------------------------------------------------------------------------%
-%-----------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
+%---------------------------------------------------------------------------%
 
 :- pred global_table_reset(io::di, io::uo) is det.
 
