@@ -1,3 +1,7 @@
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+
 :- module call_failure.
 :- interface.
 
@@ -9,10 +13,10 @@
 :- import_module require.
 
 foo(X, Y) :-
-    ( X = 42 ->
+    ( if X = 42 then
         bar(X, Y0),
         Y = Y0 + 1
-    ;
+    else
         error("foo")
     ).
 
