@@ -1,7 +1,12 @@
-% The removal of automatic initialisation for solver types in 
+%---------------------------------------------------------------------------%
+% vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
+%
+% The removal of automatic initialisation for solver types in
 % rotd-2007-10-31 inadvertently weakened the scheduling of
 % delayed goals such as the trace goal below.
-:- module trace_goal_reorder. 
+
+:- module trace_goal_reorder.
 :- interface.
 
 :- import_module io.
@@ -16,8 +21,7 @@ read_deep_byte(Res, !IO) :-
     read_byte(Res0, !IO),
     trace [compile_time(flag("debug_read_profdeep")), io(!IO)] (
         io.write_string("byte ", !IO),
-        io.write(Res, !IO),
-        io.write_string("\n", !IO)
+        io.write_line(Res, !IO)
     ),
     (
         Res0 = ok(Byte),
