@@ -1,7 +1,11 @@
+%---------------------------------------------------------------------------%
 % vim: ts=4 sw=4 et ft=mercury
+%---------------------------------------------------------------------------%
 %
 % This is a test case for what error message the compiler generates
 % when a trace goal attempts to bind a nonlocal variable.
+%
+%---------------------------------------------------------------------------%
 
 :- module trace_goal_dupl_defn.
 
@@ -25,25 +29,25 @@ main(!IO) :-
 
 p(!N, S) :-
     trace [compiletime(flag("abc")), io(!IO)] (
-        ( is_even(!.N) ->
+        ( if is_even(!.N) then
             % M is the nonlocal we attempt to bind. It is nonlocal here
             % because it also appears in the second trace goal.
             M = !.N,
             io.write_string("<abc>", !IO),
             io.write_int(M, !IO),
             io.write_string("<abc>\n", !IO)
-        ;
+        else
             true
         )
     ),
     S = "xx",
     trace [compiletime(flag("abc")), io(!IO)] (
-        ( is_even(!.N) ->
+        ( if is_even(!.N) then
             M = !.N,
             io.write_string("<abc>", !IO),
             io.write_int(M, !IO),
             io.write_string("<abc>\n", !IO)
-        ;
+        else
             true
         )
     ).
