@@ -1,24 +1,19 @@
-/*
-** vim: ts=4 sw=4 expandtab
-*/
-/*
-** Copyright (C) 1998-2008,2010,2012 The University of Melbourne.
-** This file may only be copied under the terms of the GNU Library General
-** Public License - see the file COPYING.LIB in the Mercury distribution.
-*/
+// vim: ts=4 sw=4 expandtab ft=c
 
-/*
-** This module implements the mdb commands in the "browsing" category.
-**
-** The structure of these files is:
-**
-** - all the #includes
-** - local macros and declarations of local static functions
-** - one function for each command in the category
-** - any auxiliary functions
-** - any command argument strings
-** - option processing functions.
-*/
+// Copyright (C) 1998-2008,2010,2012 The University of Melbourne.
+// Copyright (C) 2017-2018, 2020 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
+
+// This module implements the mdb commands in the "browsing" category.
+//
+// The structure of these files is:
+//
+// - all the #includes
+// - local macros and declarations of local static functions
+// - one function for each command in the category
+// - any auxiliary functions
+// - any command argument strings
+// - option processing functions.
 
 #include "mercury_std.h"
 #include "mercury_getopt.h"
@@ -38,7 +33,7 @@
 #include "mdb.declarative_execution.mh"
 #include "mdbcomp.program_representation.mh"
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
 
 static  void        MR_trace_set_level_and_report(int ancestor_level,
                         MR_bool detailed, MR_bool print_optionals);
@@ -48,11 +43,11 @@ static  const char  *MR_trace_browse_exception(MR_EventInfo *event_info,
 static  const char  *MR_trace_browse_proc_body(MR_EventInfo *event_info,
                         MR_Browser browser, MR_BrowseCallerType caller,
                         MR_BrowseFormat format);
-                        
-/* Functions to invoke the user's XML browser on terms or goals */
-static  void        MR_trace_browse_xml(MR_Word type_info, MR_Word value,
+
+// Functions to invoke the user's web browser on terms or goals.
+static  void        MR_trace_browse_web(MR_Word type_info, MR_Word value,
                         MR_BrowseCallerType caller, MR_BrowseFormat format);
-static  void        MR_trace_browse_goal_xml(MR_ConstString name,
+static  void        MR_trace_browse_goal_web(MR_ConstString name,
                         MR_Word arg_list, MR_Word is_func,
                         MR_BrowseCallerType caller, MR_BrowseFormat format);
 
@@ -65,26 +60,31 @@ static  const char  *MR_trace_new_source_window(const char *window_cmd,
                         int timeout, MR_bool force, MR_bool verbose,
                         MR_bool split);
 
-static  MR_bool     MR_trace_options_detailed(MR_bool *detailed, char ***words,
-                        int *word_count);
+static  MR_bool     MR_trace_options_detailed(MR_bool *detailed,
+                        char ***words, int *word_count);
 static  MR_bool     MR_trace_options_stack_trace(MR_bool *print_all,
                         MR_bool *detailed, MR_SpecLineLimit *line_limit,
                         MR_SpecLineLimit *clique_line_limit,
                         MR_FrameLimit *frame_limit,
                         char ***words, int *word_count);
-static  MR_bool     MR_trace_options_format(MR_BrowseFormat *format,
-                        MR_bool *xml, char ***words, int *word_count);
+static  MR_bool     MR_trace_options_print(MR_BrowseFormat *format,
+                        MR_Unsigned *max_printed_actions,
+                        MR_bool *set_max_printed_actions,
+                        char ***words, int *word_count);
+static  MR_bool     MR_trace_options_browse(MR_BrowseFormat *format,
+                        MR_bool *web,
+                        char ***words, int *word_count);
 static  MR_bool     MR_trace_options_view(const char **window_cmd,
                         const char **server_cmd, const char **server_name,
                         MR_Unsigned *timeout, MR_bool *force, MR_bool *verbose,
-                        MR_bool *split, MR_bool *close_window, char ***words,
-                        int *word_count);
+                        MR_bool *split, MR_bool *close_window,
+                        char ***words, int *word_count);
 static  MR_bool     MR_trace_options_diff(MR_Unsigned *start,
                         MR_Unsigned *max, char ***words, int *word_count);
 static  MR_bool     MR_trace_options_dump(MR_bool *quiet, MR_bool *xml,
                         char ***words, int *word_count);
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
 
 MR_Next
 MR_trace_cmd_level(char **words, int word_count, MR_TraceCmdInfo *cmd,
@@ -96,25 +96,28 @@ MR_trace_cmd_level(char **words, int word_count, MR_TraceCmdInfo *cmd,
 
     detailed = MR_FALSE;
     if (! MR_trace_options_detailed(&detailed, &words, &word_count)) {
-        ; /* the usage message has already been printed */
+        // The usage message has already been printed.
+        ;
     } else if (word_count == 2 &&
         ( MR_streq(words[1], "clique") || MR_streq(words[1], "clentry") ))
     {
         if (MR_find_clique_entry_mdb(event_info, MR_CLIQUE_ENTRY_FRAME,
             &selected_level))
         {
-            /* the error message has already been printed */
+            // The error message has already been printed.
             return KEEP_INTERACTING;
         }
     } else if (word_count == 2 && MR_streq(words[1], "clparent")) {
         if (MR_find_clique_entry_mdb(event_info, MR_CLIQUE_ENTRY_PARENT_FRAME,
             &selected_level))
         {
-            /* the error message has already been printed */
+            // The error message has already been printed.
             return KEEP_INTERACTING;
         }
     } else if (word_count == 2 && MR_trace_is_natural_number(words[1], &n)) {
         selected_level = n;
+    } else if (word_count == 1) {
+        selected_level = 0;
     } else {
         MR_trace_usage_cur_cmd();
         return KEEP_INTERACTING;
@@ -134,7 +137,8 @@ MR_trace_cmd_up(char **words, int word_count, MR_TraceCmdInfo *cmd,
 
     detailed = MR_FALSE;
     if (! MR_trace_options_detailed(&detailed, &words, &word_count)) {
-        ; /* the usage message has already been printed */
+        // The usage message has already been printed.
+        ;
     } else if (word_count == 2 && MR_trace_is_natural_number(words[1], &n)) {
         MR_trace_set_level_and_report(MR_trace_current_level() + n, detailed,
             MR_print_optionals);
@@ -157,7 +161,8 @@ MR_trace_cmd_down(char **words, int word_count, MR_TraceCmdInfo *cmd,
 
     detailed = MR_FALSE;
     if (! MR_trace_options_detailed(&detailed, &words, &word_count)) {
-        ; /* the usage message has already been printed */
+        // The usage message has already been printed.
+        ;
     } else if (word_count == 2 && MR_trace_is_natural_number(words[1], &n)) {
         MR_trace_set_level_and_report(MR_trace_current_level() - n, detailed,
             MR_print_optionals);
@@ -203,14 +208,16 @@ MR_trace_cmd_held_vars(char **words, int word_count, MR_TraceCmdInfo *cmd,
     return KEEP_INTERACTING;
 }
 
-#define MR_MAX_NUM_IO_ACTIONS_TO_PRINT  20
+#define MR_NEXT_NUM_IO_ACTIONS_TO_PRINT     20
+#define MR_ALL_NUM_IO_ACTIONS_TO_PRINT      500
 
 MR_Next
 MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
     MR_EventInfo *event_info, MR_Code **jumpaddr)
 {
     MR_BrowseFormat     format;
-    MR_bool             xml;
+    MR_Unsigned         max_printed_actions;
+    MR_bool             set_max_printed_actions;
     const char          *problem;
     MR_Unsigned         action;
     MR_Unsigned         lo_action;
@@ -218,11 +225,11 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
     static MR_bool      have_next_io_action = MR_FALSE;
     static MR_Unsigned  next_io_action = 0;
 
-    if (! MR_trace_options_format(&format, &xml, &words, &word_count)) {
-        ; /* the usage message has already been printed */
-    } else if (xml) {
-        /* the --xml option is not valid for print */
-        MR_trace_usage_cur_cmd();
+    if (! MR_trace_options_print(&format,
+        &max_printed_actions, &set_max_printed_actions, &words, &word_count))
+    {
+        // The usage message has already been printed.
+        ;
     } else if (word_count == 1) {
         problem = MR_trace_browse_one_goal(MR_mdb_out,
             MR_trace_browse_goal_internal, MR_BROWSE_CALLER_PRINT, format);
@@ -248,6 +255,10 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
         {
             MR_Unsigned num_printed_actions;
 
+            if (!set_max_printed_actions) {
+                max_printed_actions = MR_NEXT_NUM_IO_ACTIONS_TO_PRINT;
+            }
+
             if (MR_io_tabling_phase == MR_IO_TABLING_BEFORE) {
                 fflush(MR_mdb_out);
                 fprintf(MR_mdb_err,
@@ -275,7 +286,7 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
                 lo_action = MR_io_tabling_start;
             }
 
-            hi_action = lo_action + MR_MAX_NUM_IO_ACTIONS_TO_PRINT;
+            hi_action = lo_action + max_printed_actions;
             if (hi_action >= MR_io_tabling_counter_hwm) {
                 hi_action = MR_io_tabling_counter_hwm - 1;
             }
@@ -285,7 +296,7 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
                 fprintf(MR_mdb_out, "There are no I/O actions to print\n");
                 have_next_io_action = MR_FALSE;
             } else {
-                for (action = lo_action; action <= hi_action; action++) { 
+                for (action = lo_action; action <= hi_action; action++) {
                     fprintf(MR_mdb_out,
                         "action %" MR_INTEGER_LENGTH_MODIFIER "u: ", action);
                     problem = MR_trace_browse_action(MR_mdb_out, action,
@@ -347,6 +358,19 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
                 MR_io_tabling_start, MR_io_tabling_counter_hwm - 1);
             fflush(MR_mdb_out);
         } else if (MR_trace_is_natural_number(words[2], &action)) {
+            if (! (MR_io_tabling_start <= action
+                && action < MR_io_tabling_counter_hwm))
+            {
+                fflush(MR_mdb_out);
+                fprintf(MR_mdb_err,
+                    "I/O tabling has only recorded actions "
+                    "%" MR_INTEGER_LENGTH_MODIFIER "u to "
+                    "%" MR_INTEGER_LENGTH_MODIFIER "u.\n",
+                    MR_io_tabling_start, MR_io_tabling_counter_hwm - 1);
+                have_next_io_action = MR_FALSE;
+                return KEEP_INTERACTING;
+            }
+
             problem = MR_trace_browse_action(MR_mdb_out, action,
                 MR_trace_browse_goal_internal,
                 MR_BROWSE_CALLER_PRINT, format);
@@ -363,7 +387,7 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
             &lo_action, &hi_action))
         {
             if (lo_action >= hi_action) {
-                /* swap lo_action and hi_action */
+                // Swap lo_action and hi_action.
                 MR_Unsigned tmp;
 
                 tmp = lo_action;
@@ -384,7 +408,58 @@ MR_trace_cmd_print(char **words, int word_count, MR_TraceCmdInfo *cmd,
                 return KEEP_INTERACTING;
             }
 
-            for (action = lo_action; action <= hi_action; action++) { 
+            for (action = lo_action; action <= hi_action; action++) {
+                fprintf(MR_mdb_out,
+                    "action %" MR_INTEGER_LENGTH_MODIFIER "u: ", action);
+                problem = MR_trace_browse_action(MR_mdb_out, action,
+                    MR_trace_browse_goal_internal,
+                    MR_BROWSE_CALLER_PRINT, format);
+
+                if (problem != NULL) {
+                    fflush(MR_mdb_out);
+                    fprintf(MR_mdb_err, "mdb: %s.\n", problem);
+                    return KEEP_INTERACTING;
+                }
+            }
+
+            next_io_action = hi_action + 1;
+            have_next_io_action = MR_TRUE;
+        } else if (MR_streq(words[2], "*")) {
+            if (!set_max_printed_actions) {
+                max_printed_actions = MR_ALL_NUM_IO_ACTIONS_TO_PRINT;
+            }
+
+            lo_action = MR_io_tabling_start;
+            hi_action = MR_io_tabling_counter_hwm - 1;
+
+            if (lo_action + max_printed_actions < hi_action) {
+                fflush(MR_mdb_out);
+                fprintf(MR_mdb_err,
+                    "I/O tabling has recorded "
+                    "%" MR_INTEGER_LENGTH_MODIFIER "d actions, "
+                    "numbered %" MR_INTEGER_LENGTH_MODIFIER "u to "
+                    "%" MR_INTEGER_LENGTH_MODIFIER "u.\n",
+                    MR_io_tabling_counter_hwm + MR_io_tabling_start - 1,
+                    MR_io_tabling_start, MR_io_tabling_counter_hwm - 1);
+                if (set_max_printed_actions) {
+                    fprintf(MR_mdb_err,
+                        "Following your request via the -m option, "
+                        "only the first %" MR_INTEGER_LENGTH_MODIFIER "u "
+                        "actions will be printed.\n",
+                        max_printed_actions);
+                } else {
+                    fprintf(MR_mdb_err,
+                        "Without your explicit request via the -m option, "
+                        "only the default maximum "
+                        "of %" MR_INTEGER_LENGTH_MODIFIER "u "
+                        "actions will be printed.\n",
+                        max_printed_actions);
+                }
+
+                hi_action = lo_action + max_printed_actions - 1;
+            }
+
+            for (action = lo_action; action <= hi_action; action++) {
                 fprintf(MR_mdb_out,
                     "action %" MR_INTEGER_LENGTH_MODIFIER "u: ", action);
                 problem = MR_trace_browse_action(MR_mdb_out, action,
@@ -415,18 +490,19 @@ MR_trace_cmd_browse(char **words, int word_count, MR_TraceCmdInfo *cmd,
     MR_EventInfo *event_info, MR_Code **jumpaddr)
 {
     MR_BrowseFormat     format;
-    MR_bool             xml;
+    MR_bool             web;
     MR_IoActionNum      action;
     MR_GoalBrowser      goal_browser;
     MR_Browser          browser;
     const char          *problem;
 
-    if (! MR_trace_options_format(&format, &xml, &words, &word_count)) {
-        ; /* the usage message has already been printed */
+    if (! MR_trace_options_browse(&format, &web, &words, &word_count)) {
+        // The usage message has already been printed.
+        ;
     } else {
-        if (xml) {
-            goal_browser = MR_trace_browse_goal_xml;
-            browser = MR_trace_browse_xml;
+        if (web) {
+            goal_browser = MR_trace_browse_goal_web;
+            browser = MR_trace_browse_web;
         } else {
             goal_browser = MR_trace_browse_goal_internal;
             browser = MR_trace_browse_internal;
@@ -501,7 +577,7 @@ MR_trace_cmd_stack(char **words, int word_count, MR_TraceCmdInfo *cmd,
     if (! MR_trace_options_stack_trace(&print_all, &detailed,
         &line_limit, &clique_line_limit, &frame_limit, &words, &word_count))
     {
-        /* the usage message has already been printed */
+        // The usage message has already been printed.
         return KEEP_INTERACTING;
     } else if (word_count == 1) {
         line_limit = MR_stack_default_line_limit;
@@ -567,7 +643,7 @@ MR_trace_cmd_view(char **words, int word_count, MR_TraceCmdInfo *cmd,
     const char      *window_cmd = NULL;
     const char      *server_cmd = NULL;
     const char      *server_name = NULL;
-    MR_Unsigned     timeout = 8;    /* seconds */
+    MR_Unsigned     timeout = 8;    // In seconds.
     MR_bool         force = MR_FALSE;
     MR_bool         verbose = MR_FALSE;
     MR_bool         split = MR_FALSE;
@@ -578,7 +654,8 @@ MR_trace_cmd_view(char **words, int word_count, MR_TraceCmdInfo *cmd,
         &timeout, &force, &verbose, &split, &close_window,
         &words, &word_count))
     {
-        ; /* the usage message has already been printed */
+        // The usage message has already been printed.
+        ;
     } else if (word_count != 1) {
         MR_trace_usage_cur_cmd();
     } else if (close_window) {
@@ -620,13 +697,13 @@ MR_trace_cmd_hold(char **words, int word_count, MR_TraceCmdInfo *cmd,
     }
 
     if (strpbrk(held_var_name, "^/") != NULL) {
-        /* Don't allow path separators in variable names. */
+        // Don't allow path separators in variable names.
         MR_trace_usage_cur_cmd();
         return KEEP_INTERACTING;
     }
 
     if (held_var_name[0] == '$') {
-        /* Ignore any unneeded initial $ signs. */
+        // Ignore any unneeded initial $ signs.
         held_var_name = &held_var_name[1];
     }
 
@@ -666,11 +743,12 @@ MR_trace_cmd_diff(char **words, int word_count, MR_TraceCmdInfo *cmd,
     const char  *problem2;
     MR_bool     bad_subterm1;
     MR_bool     bad_subterm2;
+    MercuryFile mdb_out;
 
     start = 0;
     max = 20;
     if (! MR_trace_options_diff(&start, &max, &words, &word_count)) {
-        /* the usage message has already been printed */
+        // The usage message has already been printed.
         return KEEP_INTERACTING;
     } else if (word_count != 3) {
         MR_trace_usage_cur_cmd();
@@ -696,10 +774,12 @@ MR_trace_cmd_diff(char **words, int word_count, MR_TraceCmdInfo *cmd,
         return KEEP_INTERACTING;
     }
 
+    MR_c_file_to_mercury_file(MR_mdb_out, &mdb_out);
     MR_TRACE_CALL_MERCURY(
         MR_new_univ_on_hp(univ1, type_info1, value1);
         MR_new_univ_on_hp(univ2, type_info2, value2);
-        ML_report_diffs(start, max, univ1, univ2);
+        ML_report_diffs(MR_wrap_output_stream(&mdb_out), start, max,
+            univ1, univ2);
     );
 
     return KEEP_INTERACTING;
@@ -714,13 +794,13 @@ MR_trace_cmd_dump(char **words, int word_count, MR_TraceCmdInfo *cmd,
     MR_bool         quiet = MR_FALSE;
     MR_bool         xml = MR_FALSE;
 
-    /*
-    ** Set this to NULL to avoid uninitialization warnings.
-    */
+    // Set this to NULL to avoid uninitialization warnings.
+
     browser_term = (MR_Word) NULL;
 
     if (! MR_trace_options_dump(&quiet, &xml, &words, &word_count)) {
-        ; /* the usage message has already been printed */
+        // The usage message has already been printed.
+        ;
     } else if (word_count != 3) {
         MR_trace_usage_cur_cmd();
     } else {
@@ -793,18 +873,18 @@ MR_trace_cmd_dump(char **words, int word_count, MR_TraceCmdInfo *cmd,
     return KEEP_INTERACTING;
 }
 
-/*
-** list [num]
-**  List num lines of context around the line number of the context of the
-**  current point (i.e., level in the call stack).  If num is not given,
-**  the number of context lines defaults to the value of the context_lines
-**  setting.
-**
-** TODO: add the following (use MR_parse_source_locn()):
-** list filename:num[-num]
-**  List a range of lines from a given file.  If only one number is
-**  given, the default number of lines of context is used.
-*/
+// list [num]
+//
+// List num lines of context around the line number of the context of the
+// current point (i.e. level in the call stack). If num is not given,
+// the number of context lines defaults to the value of the context_lines
+// setting.
+//
+// TODO: add the following (use MR_parse_source_locn()):
+// list filename:num[-num]
+//
+// List a range of lines from a given file. If only one number is given,
+// the default number of lines of context is used.
 
 MR_Next
 MR_trace_cmd_list(char **words, int word_count,
@@ -813,6 +893,8 @@ MR_trace_cmd_list(char **words, int word_count,
     const MR_ProcLayout     *entry_ptr;
     const char              *filename;
     int                     lineno;
+    int                     first_lineno;
+    int                     last_lineno;
     MR_Word                 *base_sp_ptr;
     MR_Word                 *base_curfr_ptr;
     MR_Unsigned             num = MR_num_context_lines;
@@ -835,15 +917,30 @@ MR_trace_cmd_list(char **words, int word_count,
         MR_make_aligned_string(aligned_filename, (MR_String) filename);
     );
 
-    MR_TRACE_CALL_MERCURY(
-        ML_LISTING_list_file(MR_mdb_out, MR_mdb_err, (char *) aligned_filename,
-            lineno - num, lineno + num, lineno, MR_listing_path);
-    );
+    first_lineno = lineno - num;
+    last_lineno = lineno + num;
+    if (first_lineno < 1) {
+        first_lineno = 1;
+    }
+
+    if (MR_listing_cmd != NULL && strlen(MR_listing_cmd) > 0) {
+        MR_TRACE_CALL_MERCURY(
+            ML_LISTING_list_file_with_command(MR_mdb_out, MR_mdb_err,
+                MR_listing_cmd, (char *) aligned_filename,
+                first_lineno, last_lineno, lineno, MR_listing_path);
+        );
+    } else {
+        MR_TRACE_CALL_MERCURY(
+            ML_LISTING_list_file(MR_mdb_out, MR_mdb_err,
+                (char *) aligned_filename,
+                first_lineno, last_lineno, lineno, MR_listing_path);
+        );
+    }
 
     return KEEP_INTERACTING;
 }
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
 
 static void
 MR_trace_set_level_and_report(int ancestor_level, MR_bool detailed,
@@ -865,10 +962,8 @@ MR_trace_set_level_and_report(int ancestor_level, MR_bool detailed,
             &base_sp, &base_curfr);
         fprintf(MR_mdb_out, "%4d ", ancestor_level);
         if (detailed) {
-            /*
-            ** We want to print the trace info first regardless
-            ** of the value of MR_context_position.
-            */
+            // We want to print the trace info first regardless
+            // of the value of MR_context_position.
 
             MR_print_call_trace_info(MR_mdb_out, entry, base_sp, base_curfr);
             indent = 26;
@@ -978,7 +1073,7 @@ MR_trace_browse_proc_body(MR_EventInfo *event_info, MR_Browser browser,
 }
 
 static void
-MR_trace_browse_xml(MR_Word type_info, MR_Word value,
+MR_trace_browse_web(MR_Word type_info, MR_Word value,
     MR_BrowseCallerType caller, MR_BrowseFormat format)
 {
     MR_Word     browser_term;
@@ -986,26 +1081,24 @@ MR_trace_browse_xml(MR_Word type_info, MR_Word value,
     browser_term = MR_type_value_to_browser_term((MR_TypeInfo) type_info,
         value);
 
-    MR_trace_save_and_invoke_xml_browser(browser_term);
+    MR_trace_save_and_invoke_web_browser(browser_term);
 }
 
 static void
-MR_trace_browse_goal_xml(MR_ConstString name, MR_Word arg_list,
+MR_trace_browse_goal_web(MR_ConstString name, MR_Word arg_list,
     MR_Word is_func, MR_BrowseCallerType caller, MR_BrowseFormat format)
 {
     MR_Word     browser_term;
 
     browser_term = MR_synthetic_to_browser_term(name, arg_list, is_func);
-    MR_trace_save_and_invoke_xml_browser(browser_term);
+    MR_trace_save_and_invoke_web_browser(browser_term);
 }
 
-/*
-** Implement the `view' command. First, check if there is a server attached.
-** If so, either stop it or abort the command, depending on whether '-f'
-** was given. Then, if a server name was not supplied, start a new server
-** with a unique name (which has been MR_malloc'd), otherwise attach to the
-** server with the supplied name (and make a MR_malloc'd copy of the name).
-*/
+// Implement the `view' command. First, check if there is a server attached.
+// If so, either stop it or abort the command, depending on whether '-f'
+// was given. Then, if a server name was not supplied, start a new server
+// with a unique name (which has been MR_malloc'd), otherwise attach to the
+// server with the supplied name (and make a MR_malloc'd copy of the name).
 
 static const char *
 MR_trace_new_source_window(const char *window_cmd, const char *server_cmd,
@@ -1015,9 +1108,8 @@ MR_trace_new_source_window(const char *window_cmd, const char *server_cmd,
     const char  *msg;
 
     if (MR_trace_source_server.server_name != NULL) {
-        /*
-        ** We are already attached to a server.
-        */
+        // We are already attached to a server.
+
         if (force) {
             MR_trace_maybe_close_source_window(verbose);
         } else {
@@ -1040,10 +1132,9 @@ MR_trace_new_source_window(const char *window_cmd, const char *server_cmd,
         msg = MR_trace_source_attach(&MR_trace_source_server, timeout,
             verbose);
         if (msg != NULL) {
-            /*
-            ** Something went wrong, so we should free the
-            ** strings we allocated just above.
-            */
+            // Something went wrong, so we should free the
+            // strings we allocated just above.
+
             MR_free(MR_trace_source_server.server_name);
             MR_trace_source_server.server_name = NULL;
             MR_free(MR_trace_source_server.server_cmd);
@@ -1062,7 +1153,7 @@ MR_trace_maybe_sync_source_window(MR_EventInfo *event_info, MR_bool verbose)
     int                     lineno;
     const char              *parent_filename;
     int                     parent_lineno;
-    const char              *problem; /* not used */
+    const char              *problem; // Not used.
     MR_Word                 *base_sp;
     MR_Word                 *base_curfr;
     const char              *msg;
@@ -1079,10 +1170,9 @@ MR_trace_maybe_sync_source_window(MR_EventInfo *event_info, MR_bool verbose)
                 &filename, &lineno);
         }
 
-        /*
-        ** At interface ports we send both the parent context and the
-        ** current context. Otherwise, we just send the current context.
-        */
+        // At interface ports we send both the parent context and the
+        // current context. Otherwise, we just send the current context.
+
         if (MR_port_is_interface(event_info->MR_trace_port)) {
             base_sp = MR_saved_sp(event_info->MR_saved_regs);
             base_curfr = MR_saved_curfr(event_info->MR_saved_regs);
@@ -1125,27 +1215,25 @@ MR_trace_maybe_close_source_window(MR_bool verbose)
     }
 }
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
 
 const char *const    MR_trace_print_cmd_args[] =
     { "-f", "-p", "-v", "--flat", "--pretty", "--verbose",
     "exception", "goal", "*", NULL };
-    
-/*
-** It is better to have a single completion where possible,
-** so don't include `-d' here.
-*/  
+
+// It is better to have a single completion where possible,
+// so don't include `-d' here.
 
 const char *const    MR_trace_stack_cmd_args[] =
     { "--detailed", NULL };
-    
+
 const char *const    MR_trace_view_cmd_args[] =
     { "-c", "-f", "-n", "-s", "-t", "-v", "-w", "-2",
-    "--close", "--verbose", "--force", "--split-screen", 
+    "--close", "--verbose", "--force", "--split-screen",
     "--window-command", "--server-command", "--server-name",
     "--timeout", NULL };
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
 
 static struct MR_option MR_trace_detailed_opts[] =
 {
@@ -1227,27 +1315,30 @@ MR_trace_options_stack_trace(MR_bool *print_all, MR_bool *detailed,
     return MR_TRUE;
 }
 
-static struct MR_option MR_trace_format_opts[] =
+static struct MR_option MR_trace_print_opts[] =
 {
-    { "flat",       MR_no_argument, NULL,   'f' },
-    { "raw_pretty", MR_no_argument, NULL,   'r' },
-    { "verbose",    MR_no_argument, NULL,   'v' },
-    { "pretty",     MR_no_argument, NULL,   'p' },
-    { "xml",        MR_no_argument, NULL,   'x' },
-    { NULL,         MR_no_argument, NULL,   0   }
+    // Please keep the formatting options in sync with MR_trace_browse_opts.
+    { "flat",       MR_no_argument,         NULL,   'f' },
+    { "raw_pretty", MR_no_argument,         NULL,   'r' },
+    { "verbose",    MR_no_argument,         NULL,   'v' },
+    { "pretty",     MR_no_argument,         NULL,   'p' },
+    { "max",        MR_required_argument,   NULL,   'm' },
+    { NULL,         MR_no_argument,         NULL,   0   }
 };
 
 static MR_bool
-MR_trace_options_format(MR_BrowseFormat *format, MR_bool *xml, char ***words,
-    int *word_count)
+MR_trace_options_print(MR_BrowseFormat *format,
+    MR_Unsigned *max_printed_actions, MR_bool *set_max_printed_actions,
+    char ***words, int *word_count)
 {
     int c;
 
     *format = MR_BROWSE_DEFAULT_FORMAT;
-    *xml = MR_FALSE;
+    *max_printed_actions = -1;
+    *set_max_printed_actions = MR_FALSE;
     MR_optind = 0;
-    while ((c = MR_getopt_long(*word_count, *words, "frvpx",
-        MR_trace_format_opts, NULL)) != EOF)
+    while ((c = MR_getopt_long(*word_count, *words, "frvpm:",
+        MR_trace_print_opts, NULL)) != EOF)
     {
         switch (c) {
 
@@ -1267,8 +1358,76 @@ MR_trace_options_format(MR_BrowseFormat *format, MR_bool *xml, char ***words,
                 *format = MR_BROWSE_FORMAT_PRETTY;
                 break;
 
-            case 'x':
-                *xml = MR_TRUE;
+            case 'm':
+                if (! MR_trace_is_natural_number(MR_optarg,
+                    max_printed_actions))
+                {
+                    MR_trace_usage_cur_cmd();
+                    return MR_FALSE;
+                }
+                if (*max_printed_actions == 0) {
+                    // Printing a maximum of zero I/O actions
+                    // does not make any sense.
+                    MR_trace_usage_cur_cmd();
+                    return MR_FALSE;
+                }
+                *set_max_printed_actions = MR_TRUE;
+                break;
+
+            default:
+                MR_trace_usage_cur_cmd();
+                return MR_FALSE;
+        }
+    }
+
+    *words = *words + MR_optind - 1;
+    *word_count = *word_count - MR_optind + 1;
+    return MR_TRUE;
+}
+
+static struct MR_option MR_trace_browse_opts[] =
+{
+    // Please keep the formatting options in sync with MR_trace_print_opts.
+    { "flat",       MR_no_argument, NULL,   'f' },
+    { "raw_pretty", MR_no_argument, NULL,   'r' },
+    { "verbose",    MR_no_argument, NULL,   'v' },
+    { "pretty",     MR_no_argument, NULL,   'p' },
+    { "web",        MR_no_argument, NULL,   'w' },
+    { NULL,         MR_no_argument, NULL,   0   }
+};
+
+static MR_bool
+MR_trace_options_browse(MR_BrowseFormat *format, MR_bool *web,
+    char ***words, int *word_count)
+{
+    int c;
+
+    *format = MR_BROWSE_DEFAULT_FORMAT;
+    *web = MR_FALSE;
+    MR_optind = 0;
+    while ((c = MR_getopt_long(*word_count, *words, "frvpw",
+        MR_trace_browse_opts, NULL)) != EOF)
+    {
+        switch (c) {
+
+            case 'f':
+                *format = MR_BROWSE_FORMAT_FLAT;
+                break;
+
+            case 'r':
+                *format = MR_BROWSE_FORMAT_RAW_PRETTY;
+                break;
+
+            case 'v':
+                *format = MR_BROWSE_FORMAT_VERBOSE;
+                break;
+
+            case 'p':
+                *format = MR_BROWSE_FORMAT_PRETTY;
+                break;
+
+            case 'w':
+                *web = MR_TRUE;
                 break;
 
             default:
@@ -1308,10 +1467,9 @@ MR_trace_options_view(const char **window_cmd, const char **server_cmd,
     while ((c = MR_getopt_long(*word_count, *words, "cw:s:n:t:fv2",
         MR_trace_view_opts, NULL)) != EOF)
     {
-        /*
-        ** Option '-c' is mutually incompatible with '-f', '-t',
-        ** '-s', '-n', '-w' and '-2'.
-        */
+        // Option '-c' is mutually incompatible with '-f', '-t',
+        // '-s', '-n', '-w' and '-2'.
+
         switch (c) {
 
             case 'c':
@@ -1481,4 +1639,4 @@ MR_trace_options_dump(MR_bool *quiet, MR_bool *xml,
     return MR_TRUE;
 }
 
-/****************************************************************************/
+////////////////////////////////////////////////////////////////////////////
