@@ -1,26 +1,21 @@
-/*
-** vim: ts=4 sw=4 expandtab
-*/
-/*
-** Copyright (C) 1998-2008,2011-2012 The University of Melbourne.
-** This file may only be copied under the terms of the GNU Library General
-** Public License - see the file COPYING.LIB in the Mercury distribution.
-*/
+// vim: ts=4 sw=4 expandtab ft=c
 
-/*
-** This file contains the code that interfaces between the Mercury program
-** being debugged and the external trace analysis process.
-**
-** For the general basis of trace analysis systems, see the paper
-** "Opium: An extendable trace analyser for Prolog" by Mireille Ducasse,
-** available from http://www.irisa.fr/lande/ducasse.
-**
-** The code for using an external debugger is conditionalized
-** on MR_USE_EXTERNAL_DEBUGGER which is enabled for systems that support
-** sockets.
-**
-** Main authors: Erwan Jahier and Fergus Henderson.
-*/
+// Copyright (C) 1998-2008,2011-2012 The University of Melbourne.
+// Copyright (C) 2014, 2016, 2018 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
+
+// This file contains the code that interfaces between the Mercury program
+// being debugged and the external trace analysis process.
+//
+// For the general basis of trace analysis systems, see the paper
+// "Opium: An extendable trace analyser for Prolog" by Mireille Ducasse,
+// available from http://www.irisa.fr/lande/ducasse.
+//
+// The code for using an external debugger is conditionalized
+// on MR_USE_EXTERNAL_DEBUGGER which is enabled for systems that support
+// sockets.
+//
+// Main authors: Erwan Jahier and Fergus Henderson.
 
 #include "mercury_imp.h"
 
@@ -55,77 +50,75 @@
   #include <dlfcn.h>
 #endif
 
-/*
-** This type must match the definition of classify_request in
-** browser/debugger_interface.m.
-*/
+// This type must match the definition of classify_request in
+// browser/debugger_interface.m.
 
 typedef enum {
-    /* initiate debugging session */
+    // initiate debugging session
     MR_REQUEST_HELLO_REPLY              = 0,
 
-    /* go to the next matching trace event */
+    // go to the next matching trace event
     MR_REQUEST_FORWARD_MOVE             = 1,
 
-    /* report data for current_vars query */
+    // report data for current_vars query
     MR_REQUEST_CURRENT_VARS             = 2,
 
-    /* report data for current_slots query */
+    // report data for current_slots query
     MR_REQUEST_CURRENT_SLOTS            = 3,
 
-    /* continue to end, not tracing */
+    // continue to end, not tracing
     MR_REQUEST_NO_TRACE                 = 4,
 
-    /* abort the current execution */
+    // abort the current execution
     MR_REQUEST_ABORT_PROG               = 5,
 
-    /* something went wrong */
+    // something went wrong
     MR_REQUEST_ERROR                    = 6,
 
-    /* report data for current_live_var_names query */
+    // report data for current_live_var_names query
     MR_REQUEST_CURRENT_LIVE_VAR_NAMES   = 7,
 
-    /* report data for current_nth_var query */
+    // report data for current_nth_var query
     MR_REQUEST_CURRENT_NTH_VAR          = 8,
 
-    /* restart the execution to the call port of the current event */
+    // restart the execution to the call port of the current event
     MR_REQUEST_RETRY                    = 9,
 
-    /* print the ancestors list */
+    // print the ancestors list
     MR_REQUEST_STACK                    = 10,
 
-    /* print the nondet stack */
+    // print the nondet stack
     MR_REQUEST_NONDET_STACK             = 11,
 
-    /* prints the contents of the virtual machine registers. */
+    // prints the contents of the virtual machine registers.
     MR_REQUEST_STACK_REGS               = 12,
 
-    /* wait for a normal interactive query */
+    // wait for a normal interactive query
     MR_REQUEST_INTERACTIVE_QUERY_NORMAL = 13,
 
-    /* wait for a cc interactive query */
+    // wait for a cc interactive query
     MR_REQUEST_INTERACTIVE_QUERY_CC     = 14,
 
-    /* wait for a io interactive query */
+    // wait for a io interactive query
     MR_REQUEST_INTERACTIVE_QUERY_IO     = 15,
-    /* pass down new options to compile queries with */
+    // pass down new options to compile queries with
     MR_REQUEST_MMC_OPTIONS              = 16,
 
-    /* call the term browser */
+    // call the term browser
     MR_REQUEST_BROWSE                   = 17,
-    /* dynamically link the collect module */
+    // dynamically link the collect module
     MR_REQUEST_LINK_COLLECT             = 18,
 
-    /* collecting monitoring informations */
+    // collecting monitoring informations
     MR_REQUEST_COLLECT                  = 19,
 
-    /* retrieving the grade of the current program has been compiled with */
+    // retrieving the grade of the current program has been compiled with
     MR_REQUEST_CURRENT_GRADE            = 20,
 
-    /* switch the arguments collecting on */
+    // switch the arguments collecting on
     MR_REQUEST_COLLECT_ARG_ON           = 21,
 
-    /* switch the arguments collecting off */
+    // switch the arguments collecting off
     MR_REQUEST_COLLECT_ARG_OFF          = 22
 
 } MR_debugger_request_type;
@@ -135,15 +128,13 @@ MercuryFile         MR_debugger_socket_out;
 
 static MR_String    MR_external_mmc_options;
 
-/*
-** Type of a static variable that indicates in which mode the external
-** debugger is. When the external debugger is in mode:
-** (1) `MR_searching', it tries to find an event that matches a forward
-**     move request,
-** (2) `MR_reading_request', it reads a new request on the socket,
-** (3) `MR_collecting', it is collecting information (after a `collect'
-**     request).
-*/
+// Type of a static variable that indicates in which mode the external
+// debugger is. When the external debugger is in mode:
+// (1) `MR_searching', it tries to find an event that matches a forward
+//     move request,
+// (2) `MR_reading_request', it reads a new request on the socket,
+// (3) `MR_collecting', it is collecting information (after a `collect'
+//     request).
 
 typedef enum {
     MR_searching, MR_reading_request, MR_collecting
@@ -152,58 +143,44 @@ typedef enum {
 static  MR_external_debugger_mode_type
     external_debugger_mode = MR_reading_request;
 
-/*
-** Global variable that is used to store the information collected during
-** a collect request.
-*/
+// Global variable that is used to store the information collected during
+// a collect request.
 
 static  MR_Word MR_accumulator_variable;
 
-/*
-** Global variable that is sent to collect caller.
-*/
+// Global variable that is sent to collect caller.
 
 static  MR_Word MR_collected_variable;
 
-/*
-** Function pointer used to post-process the result of the collect activity
-*/
+// Function pointer used to post-process the result of the collect activity
 
 static void (*post_process_ptr)(MR_Word, MR_Word *);
 
-/*
-** Function pointer used to sent the collecting variable to the external
-** debugger.
-*/
+// Function pointer used to sent the collecting variable to the external
+// debugger.
 
 static  void    (*send_collect_result_ptr)(MR_Word, MR_Word);
 
-/*
-** Variable generated during the dynamic linking that is needed to close
-** this linking properly.
-*/
+// Variable generated during the dynamic linking that is needed to close
+// this linking properly.
 
 static  MR_Word collect_lib_maybe_handle;
 
-/*
-** Static variable that tells whether the list of arguments is available
-** within a collect module.
-*/
+// Static variable that tells whether the list of arguments is available
+// within a collect module.
 
 static  MR_bool MR_collect_arguments = MR_FALSE;
 
-/*
-** Use a GNU C extension to enforce static type checking
-** for printf-style functions.
-** (See the "Function attributes" section of "C extensions"
-** chapter of the GNU C manual for detailed documentation.)
-*/
+// Use a GNU C extension to enforce static type checking
+// for printf-style functions.
+// (See the "Function attributes" section of "C extensions"
+// chapter of the GNU C manual for detailed documentation.)
 
 #ifdef __GNUC__
   #define MR_LIKE_PRINTF(format_argnum, vars_argnum) \
     __attribute__ ((format (printf, (format_argnum), (vars_argnum))))
 #else
-  #define MR_LIKE_PRINTF(n, m) /* nothing */
+  #define MR_LIKE_PRINTF(n, m) // nothing
 #endif
 
 static void     MR_send_message_to_socket_format(const char *format, ...)
@@ -248,8 +225,8 @@ static void     MR_send_collect_result(void);
 #if 0
 This pseudocode should go in the debugger process:
 
-#define SOCKET_PATH "/var/tmp/"     /* +5 for pid = 14 chars */
-#define SOCKET_PERM S_IRWXU     /* rwx for user only */
+#define SOCKET_PATH "/var/tmp/"     // +5 for pid = 14 chars
+#define SOCKET_PERM S_IRWXU         // rwx for user only
 
 sprintf(Name, "%s%05d", SOCKET_PATH, getpid());
 
@@ -277,11 +254,9 @@ accept(Sock, _, New_Sock).
 static void
 MR_init_unix_address(const char *name, struct sockaddr_un *unix_addr)
 {
-    /*
-    ** The code here is adapted from Stevens, "Advanced Programming
-    ** in the UNIX environment", page 501.
-    ** Don't blame me, I'm just copying this code from Stevens ;-)
-    */
+    // The code here is adapted from Stevens, "Advanced Programming
+    // in the UNIX environment", page 501.
+    // Don't blame me, I'm just copying this code from Stevens ;-)
 
     memset(unix_addr, 0, sizeof(unix_addr));
     unix_addr->sun_family = AF_UNIX;
@@ -320,23 +295,19 @@ MR_trace_init_external(void)
     MR_Integer          debugger_request_type;
     char                errbuf[MR_STRERROR_BUF_SIZE];
 
-    /*
-    ** MR_external_mmc_options contains the options to pass to mmc
-    ** when compiling queries. We initialize it to the MR_String "".
-    */
+    // MR_external_mmc_options contains the options to pass to mmc
+    // when compiling queries. We initialize it to the MR_String "".
 
     MR_TRACE_CALL_MERCURY(
         ML_DI_init_mercury_string(&MR_external_mmc_options)
     );
 
-    /*
-    ** We presume that the user's program has been invoked from
-    ** within the debugger (e.g. Morphine).
-    ** The debugger (or the user) should set the
-    ** MERCURY_DEBUGGER_UNIX_SOCKET or MERCURY_DEBUGGER_INET_SOCKET
-    ** environment variable to tell the user's program which socket
-    ** it needs to connect to.
-    */
+    // We presume that the user's program has been invoked from
+    // within the debugger (e.g. Morphine).
+    // The debugger (or the user) should set the
+    // MERCURY_DEBUGGER_UNIX_SOCKET or MERCURY_DEBUGGER_INET_SOCKET
+    // environment variable to tell the user's program which socket
+    // it needs to connect to.
 
     unix_socket = getenv("MERCURY_DEBUGGER_UNIX_SOCKET");
     inet_socket = getenv("MERCURY_DEBUGGER_INET_SOCKET");
@@ -350,14 +321,12 @@ MR_trace_init_external(void)
             "and MERCURY_DEBUGGER_INET_SOCKET environment variables");
     }
     if (unix_socket) {
-        /*
-        ** We use `(memset)(...)' rather than `memset'
-        ** to prevent macro expansion; this is needed because
-        ** with GNU libc 2.1.2 and gcc 2.95, gcc barfs on the latter,
-        ** with an error message about impossible asm due to a conflict
-        ** between our use of global register variables and the inline
-        ** assembler macro definition of memset.
-        */
+        // We use `(memset)(...)' rather than `memset'
+        // to prevent macro expansion; this is needed because
+        // with GNU libc 2.1.2 and gcc 2.95, gcc barfs on the latter,
+        // with an error message about impossible asm due to a conflict
+        // between our use of global register variables and the inline
+        // assembler macro definition of memset.
 
         addr_family = AF_UNIX;
         (memset)(&unix_address, 0, sizeof(unix_address));
@@ -371,11 +340,9 @@ MR_trace_init_external(void)
         unsigned short  port;
         int             host_addr;
 
-        /*
-        ** Parse the MERCURY_DEBUGGER_INET_SOCKET environment variable.
-        ** It should be in the format "<hostname> <port>",
-        ** where <hostname> is numeric (e.g. "123.456.78.90").
-        */
+        // Parse the MERCURY_DEBUGGER_INET_SOCKET environment variable.
+        // It should be in the format "<hostname> <port>",
+        // where <hostname> is numeric (e.g. "123.456.78.90").
 
         if (sscanf(inet_socket, "%254s %254s", hostname, port_string) != 2) {
             MR_fatal_error("MERCURY_DEBUGGER_INET_SOCKET invalid");
@@ -400,9 +367,7 @@ MR_trace_init_external(void)
         len = sizeof(inet_address);
     }
 
-    /*
-    ** Open the socket.
-    */
+    // Open the socket.
 
     fd = socket(addr_family, SOCK_STREAM, 0);
     if (fd < 0) {
@@ -413,9 +378,7 @@ MR_trace_init_external(void)
         fprintf(stderr,"Mercury runtime: creation of socket ok\n");
     }
 
-    /*
-    ** Connect to the socket.
-    */
+    // Connect to the socket.
 
     if (connect(fd, addr, len) < 0) {
         fprintf(stderr, "Mercury runtime: connect() failed: %s\n",
@@ -425,9 +388,7 @@ MR_trace_init_external(void)
         fprintf(stderr, "Mercury runtime: connection to socket: ok\n");
     }
 
-    /*
-    ** Convert the socket fd to a Mercury stream.
-    */
+    // Convert the socket fd to a Mercury stream.
 
     file_in = fdopen(fd, "r");
     file_out = fdopen(fd, "w");
@@ -442,18 +403,14 @@ MR_trace_init_external(void)
     MR_mercuryfile_init(file_in, 1, &MR_debugger_socket_in);
     MR_mercuryfile_init(file_out, 1, &MR_debugger_socket_out);
 
-    /*
-    ** Send hello.
-    */
+    // Send hello.
 
     MR_send_message_to_socket("hello");
     if (MR_debug_socket) {
         fprintf(stderr, "Mercury runtime: Send hello\n");
     }
 
-    /*
-    ** Wait for hello_reply.
-    */
+    // Wait for hello_reply.
 
     MR_read_request_from_socket(&debugger_request, &debugger_request_type);
 
@@ -463,9 +420,7 @@ MR_trace_init_external(void)
         fprintf(stderr, "Mercury runtime: read hello_reply\n");
     }
 
-    /*
-    ** Send start to start the synchronous communication with the debugger.
-    */
+    // Send start to start the synchronous communication with the debugger.
 
     MR_send_message_to_socket("start");
     if (MR_debug_socket) {
@@ -476,11 +431,9 @@ MR_trace_init_external(void)
 void
 MR_trace_final_external(void)
 {
-    /*
-    ** This can only happen during a forward_move or a collect request.
-    ** In the first case, we want to tell the debugger that no match was found;
-    ** in the second one we send the result of the collect activity.
-    */
+    // This can only happen during a forward_move or a collect request.
+    // In the first case, we want to tell the debugger that no match was found;
+    // in the second one we send the result of the collect activity.
 
     switch(external_debugger_mode) {
         case MR_searching:
@@ -496,12 +449,10 @@ MR_trace_final_external(void)
             MR_fatal_error("Error in the external debugger");
     }
 
-    /*
-    ** Maybe we should loop to process requests from the debugger socket here?
-    ** Currently we just return, which will result in the debuggee terminating.
-    ** (This will need to change if the debuggee is to record trace history
-    ** and support a `move_backward' request.)
-    */
+    // Maybe we should loop to process requests from the debugger socket here?
+    // Currently we just return, which will result in the debuggee terminating.
+    // (This will need to change if the debuggee is to record trace history
+    // and support a `move_backward' request.)
 }
 
 MR_Code *
@@ -543,10 +494,9 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
 
     switch(external_debugger_mode) {
         case MR_searching:
-            /*
-            ** XXX Should also pass the registers here, since they are needed
-            ** for checking for matches with the arguments.
-            */
+            // XXX Should also pass the registers here, since they are needed
+            // for checking for matches with the arguments.
+
             if (MR_found_match(layout, port, seqno, depth,
                 /* XXX registers, */ path, search_data))
             {
@@ -571,7 +521,7 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
 
     lineno = MR_get_line_number(event_info->MR_saved_regs, layout, port);
 
-    /* Loop to process requests read from the debugger socket. */
+    // Loop to process requests read from the debugger socket.
     for(;;) {
         MR_read_request_from_socket(&debugger_request, &debugger_request_type);
         switch((int) debugger_request_type) {
@@ -668,16 +618,14 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
                         "\nMercury runtime: REQUEST_NONDET_STACK\n");
                 }
                 MR_trace_init_modules();
-                /*
-                ** XXX As in stack dump, we could send the output of this
-                ** function on the socket. But the outputs are done via
-                ** fprintf() and printlabel(), so we would need to define new
-                ** fprintf() and printlabel() and pass them down as parameters
-                ** of MR_dump_nondet_stack() (as we do with
-                ** MR_dump_stack_record_print()).
-                */
-                MR_dump_nondet_stack(stdout, NULL, 0, 0,
-                    MR_saved_maxfr(saved_regs));
+                // XXX As in stack dump, we could send the output of this
+                // function on the socket. But the outputs are done via
+                // fprintf() and printlabel(), so we would need to define new
+                // fprintf() and printlabel() and pass them down as parameters
+                // of MR_dump_nondet_stack() (as we do with
+                // MR_dump_stack_record_print()).
+
+                MR_dump_nondet_stack(stdout, 0, 0, MR_saved_maxfr(saved_regs));
                 MR_send_message_to_socket("ok");
                 break;
 
@@ -806,10 +754,8 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
                         (*initialize_ptr)(&MR_accumulator_variable);
                     );
 
-                    /*
-                    ** In order to perform the collect from the current
-                    ** event, we need to call filter once here.
-                    */
+                    // In order to perform the collect from the current
+                    // event, we need to call filter once here.
 
                     MR_COLLECT_filter(cmd->MR_filter_ptr, seqno, depth,
                         port, layout, path, lineno, &stop_collecting);
@@ -819,15 +765,14 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
                         MR_send_message_to_socket("execution_continuing");
                         break;
                     } else {
-                        /*
-                        ** For efficiency, the remaining calls
-                        ** to MR_COLLECT_filter() are done in MR_trace_real().
-                        */
+                        // For efficiency, the remaining calls
+                        // to MR_COLLECT_filter() are done in MR_trace_real().
 
                         cmd->MR_trace_cmd = MR_CMD_COLLECT;
                         cmd->MR_trace_must_check = MR_FALSE;
                         cmd->MR_trace_strict = MR_TRUE;
                         MR_init_trace_check_integrity(cmd);
+                        cmd->MR_trace_print_level_specified = MR_TRUE;
                         cmd->MR_trace_print_level = MR_PRINT_LEVEL_NONE;
                         goto done;
                     }
@@ -870,10 +815,9 @@ MR_trace_event_external(MR_TraceCmdInfo *cmd, MR_EventInfo *event_info)
     }
 
 done:
-    /*
-    ** Recalculate the `must_check' flag, in case we changed the command
-    ** strictness or print-level.
-    */
+    // Recalculate the `must_check' flag, in case we changed the command
+    // strictness or print-level.
+
     cmd->MR_trace_must_check = (! cmd->MR_trace_strict) ||
         (cmd->MR_trace_print_level != MR_PRINT_LEVEL_NONE);
 
@@ -908,7 +852,7 @@ MR_output_current_slots(const MR_LabelLayout *layout,
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_def_module,
                 (MR_String)
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_pred_name,
-                /* is the type_ctor's arity what is wanted? XXX */
+                // is the type_ctor's arity what is wanted? XXX
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_type_arity,
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_mode,
                 layout->MR_sll_entry->MR_sle_detism,
@@ -989,13 +933,13 @@ MR_read_request_from_socket(MR_Word *debugger_request_ptr,
 static MR_bool
 MR_found_match(const MR_LabelLayout *layout,
     MR_TracePort port, MR_Unsigned seqno, MR_Unsigned depth,
-    /* XXX live vars */
+    // XXX live vars
     const char *path, MR_Word search_data)
 {
     MR_bool result;
 
-    /* XXX get live vars from registers */
-    MR_Word arguments = /* XXX FIXME!!! */ 0;
+    // XXX Get live vars from registers.
+    MR_Word arguments = 0;                                  // XXX FIXME!!!
     if (MR_PROC_LAYOUT_IS_UCI(layout->MR_sll_entry)) {
         MR_TRACE_CALL_MERCURY(
             result = ML_DI_found_match_comp(
@@ -1011,7 +955,7 @@ MR_found_match(const MR_LabelLayout *layout,
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_def_module,
                 (MR_String)
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_pred_name,
-                /* is the type_ctor's arity what is wanted? XXX */
+                // Is the type_ctor's arity what is wanted? XXX
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_type_arity,
                 layout->MR_sll_entry->MR_sle_uci.MR_uci_mode,
                 layout->MR_sll_entry->MR_sle_detism,
@@ -1065,14 +1009,12 @@ MR_send_message_to_socket(const char *message)
     MR_line_number(MR_debugger_socket_out)++;
 }
 
-/*
-** This function returns the list of the currently live variables
-** as a list of univs.
-** The memory needed will be allocated on the Mercury heap.
-**
-** If no information on live variables is available, or if there
-** are no live variables, return the empty list.
-*/
+// This function returns the list of the currently live variables
+// as a list of univs.
+// The memory needed will be allocated on the Mercury heap.
+//
+// If no information on live variables is available, or if there
+// are no live variables, return the empty list.
 
 static MR_Word
 MR_trace_make_var_list(void)
@@ -1094,7 +1036,7 @@ MR_trace_make_var_list(void)
     for (i = var_count; i > 0; i--) {
         problem = MR_trace_return_var_info(i, NULL, &type_info, &value);
         if (problem != NULL) {
-            MR_fatal_error(problem);
+            MR_fatal_error("%s", problem);
         }
 
         MR_TRACE_USE_HP(
@@ -1109,14 +1051,12 @@ MR_trace_make_var_list(void)
     return var_list;
 }
 
-/*
-** This function returns the list of the internal names of currently live
-** variables.
-** The memory needed will be allocated on the Mercury heap.
-**
-** If no information on live variables is available, or if there
-** are no live variables, return the empty list.
-*/
+// This function returns the list of the internal names of currently live
+// variables.
+// The memory needed will be allocated on the Mercury heap.
+//
+// If no information on live variables is available, or if there
+// are no live variables, return the empty list.
 
 static MR_Word
 MR_trace_make_var_names_list(void)
@@ -1136,7 +1076,7 @@ MR_trace_make_var_names_list(void)
     for (i = var_count; i > 0; i--) {
         problem = MR_trace_return_var_info(i, &name, NULL, NULL);
         if (problem != NULL) {
-            MR_fatal_error(problem);
+            MR_fatal_error("%s", problem);
         }
 
         MR_TRACE_USE_HP(
@@ -1148,13 +1088,11 @@ MR_trace_make_var_names_list(void)
     return var_names_list;
 }
 
-/*
-** This function returns the list of types of currently live variables.
-** The memory needed will be allocated on the Mercury heap.
-**
-** If no information on live variables is available, or if there
-** are no live variables, return the empty list.
-*/
+// This function returns the list of types of currently live variables.
+// The memory needed will be allocated on the Mercury heap.
+//
+// If no information on live variables is available, or if there
+// are no live variables, return the empty list.
 
 static MR_Word
 MR_trace_make_type_list(void)
@@ -1175,7 +1113,7 @@ MR_trace_make_type_list(void)
     for (i = var_count; i > 0; i--) {
         problem = MR_trace_return_var_info(i, NULL, &type_info, NULL);
         if (problem != NULL) {
-            MR_fatal_error(problem);
+            MR_fatal_error("%s", problem);
         }
 
         MR_TRACE_CALL_MERCURY(
@@ -1190,10 +1128,8 @@ MR_trace_make_type_list(void)
     return type_list;
 }
 
-/*
-** This function returns the requested live variable, as a univ.
-** Any memory needed will be allocated on the Mercury heap.
-*/
+// This function returns the requested live variable, as a univ.
+// Any memory needed will be allocated on the Mercury heap.
 
 static MR_Word
 MR_trace_make_nth_var(MR_Word debugger_request)
@@ -1205,7 +1141,7 @@ MR_trace_make_nth_var(MR_Word debugger_request)
     MR_Word     univ;
 
     var_number = MR_get_var_number(debugger_request);
-    /* debugger_request should be of the form: current_nth_var(var_number) */
+    // debugger_request should be of the form: current_nth_var(var_number)
 
     problem = MR_trace_return_var_info(var_number, NULL, &type_info, &value);
     if (problem == NULL) {
@@ -1213,21 +1149,17 @@ MR_trace_make_nth_var(MR_Word debugger_request)
             MR_new_univ_on_hp(univ, type_info, value);
         );
     } else {
-        /*
-        ** Should never occur since we check in the external debugger process
-        ** if a variable is live before retrieving it.
-        */
+        // Should never occur since we check in the external debugger process
+        // if a variable is live before retrieving it.
 
-        MR_fatal_error(problem);
+        MR_fatal_error("%s", problem);
     }
 
     return univ;
 }
 
-/*
-** This function is called only when debugger_request = current_nth_var(n).
-** It returns the integer 'n'.
-*/
+// This function is called only when debugger_request = current_nth_var(n).
+// It returns the integer 'n'.
 
 static int
 MR_get_var_number(MR_Word debugger_request)
@@ -1239,42 +1171,38 @@ MR_get_var_number(MR_Word debugger_request)
     return num;
 }
 
-/*
-** The protocol between the debugged Mercury program and the external debugger
-** is the following:
-** 1) The debugger sends "stack";
-** 2) For each procedure in the stack that is not generated by the compiler, the
-**    debuggee sends:
-**  - level(int) (the level of the procedure in the stack)
-**  - detail(unsigned long, unsigned long, unsigned long) if available
-**    (the call event number, call sequence number and depth of the goal
-**    of the procedure)
-**  - the atom 'pred' or 'func' depending if the procedure is a function
-**    or not
-**  - proc(string, string, long, long) (the procedure)
-**  - det(string) (the determinism of the procedure)
-**  - def_module(string) (the name of the defining module) if different from
-**    the current one.
-**
-**    For each compiler generated procedures, the debuggee sends:
-**  - level(int) (as above)
-**  - detail(unsigned long, unsigned long, unsigned long) (as above)
-**  - proc(string, string, string, long, long) (the name of the
-**    compiler-generated procedure)
-**  - det(string) (as above)
-**  - def_module(string) (as above)
-** 3) The debuggee sends "end_stack"
-*/
+// The protocol between the debugged Mercury program and the external debugger
+// is the following:
+// 1) The debugger sends "stack";
+// 2) For each procedure in the stack that is not generated by the compiler, the
+//    debuggee sends:
+//  - level(int) (the level of the procedure in the stack)
+//  - detail(unsigned long, unsigned long, unsigned long) if available
+//    (the call event number, call sequence number and depth of the goal
+//    of the procedure)
+//  - the atom 'pred' or 'func' depending if the procedure is a function
+//    or not
+//  - proc(string, string, long, long) (the procedure)
+//  - det(string) (the determinism of the procedure)
+//  - def_module(string) (the name of the defining module) if different from
+//    the current one.
+//
+//    For each compiler generated procedures, the debuggee sends:
+//  - level(int) (as above)
+//  - detail(unsigned long, unsigned long, unsigned long) (as above)
+//  - proc(string, string, string, long, long) (the name of the
+//    compiler-generated procedure)
+//  - det(string) (as above)
+//  - def_module(string) (as above)
+// 3) The debuggee sends "end_stack"
 
 static void
 MR_dump_stack_record_print_to_socket(FILE *fp, MR_bool include_trace_data,
     const MR_StackFrameDumpInfo *frame_dump_info)
 {
-    /*
-    ** XXX If the external debugger is ever needed again, it should be updated
-    ** to send information across the socket about about any reuse of a stack
-    ** frame by tail recursion events.
-    */
+    // XXX If the external debugger is ever needed again, it should be updated
+    // to send information across the socket about any reuse of a stack
+    // frame by tail recursion events.
 
     if (frame_dump_info->MR_sdi_min_level != frame_dump_info->MR_sdi_max_level)
     {
@@ -1302,11 +1230,10 @@ MR_print_proc_id_to_socket(const MR_ProcLayout *entry,
         if (MR_PROC_LAYOUT_HAS_EXEC_TRACE(entry)) {
             MR_Integer maybe_from_full = entry->MR_sle_maybe_from_full;
             if (maybe_from_full > 0) {
-                /*
-                ** For procedures compiled with shallow tracing, the details
-                ** will be valid only if the value of MR_from_full saved in
-                ** the appropriate stack slot was MR_TRUE.
-                */
+                // For procedures compiled with shallow tracing, the details
+                // will be valid only if the value of MR_from_full saved in
+                // the appropriate stack slot was MR_TRUE.
+
                 if (MR_DETISM_DET_STACK(entry->MR_sle_detism)) {
                     print_details = MR_based_stackvar(base_sp,
                         maybe_from_full);
@@ -1315,10 +1242,9 @@ MR_print_proc_id_to_socket(const MR_ProcLayout *entry,
                         maybe_from_full);
                 }
             } else {
-                /*
-                ** For procedures compiled with full tracing,
-                ** always print out the details.
-                */
+                // For procedures compiled with full tracing,
+                // always print out the details.
+
                 print_details = MR_TRUE;
             }
         }
@@ -1341,7 +1267,7 @@ MR_print_proc_id_to_socket(const MR_ProcLayout *entry,
 
     if (MR_PROC_LAYOUT_IS_UCI(entry)) {
         MR_send_message_to_socket_format(
-            /* XXX Names with " may cause some problems here */
+            // XXX Names with " may cause some problems here.
             "proc(\"%s\",\"%s\",\"%s\",%ld,%ld).\n",
             entry->MR_sle_uci.MR_uci_pred_name,
             entry->MR_sle_uci.MR_uci_type_module,
@@ -1365,7 +1291,7 @@ MR_print_proc_id_to_socket(const MR_ProcLayout *entry,
         }
 
         MR_send_message_to_socket_format(
-            /* XXX Names with " may cause some problems here */
+            // XXX Names with " may cause some problems here.
             "proc(\"%s\",\"%s\",%ld,%ld).\n",
             entry->MR_sle_user.MR_user_decl_module,
             entry->MR_sle_user.MR_user_name,
@@ -1419,12 +1345,10 @@ MR_get_variable_name(MR_Word debugger_request, MR_String *var_name_ptr)
     );
 }
 
-/*
-** This function does the same thing as MR_trace_browse_one() defined in
-** mercury_trace_internal.c except it sends/receives program-readable terms
-** from/to the socket instead of sending human-readable strings from/to
-** mdb_in/mdb_out.
-*/
+// This function does the same thing as MR_trace_browse_one() defined in
+// mercury_trace_internal.c except it sends/receives program-readable terms
+// from/to the socket instead of sending human-readable strings from/to
+// mdb_in/mdb_out.
 
 static void
 MR_trace_browse_one_external(MR_VarSpec var_spec)
@@ -1440,10 +1364,9 @@ MR_trace_browse_one_external(MR_VarSpec var_spec)
     }
 }
 
-/*
-** This function calls the collect filtering predicate defined by the user
-** and dynamically link with the execution.
-*/
+// This function calls the collect filtering predicate defined by the user
+// and dynamically link with the execution.
+
 void
 MR_COLLECT_filter(MR_FilterFuncPtr filter_ptr, MR_Unsigned seqno,
     MR_Unsigned depth, MR_TracePort port, const MR_LabelLayout *layout,
@@ -1452,15 +1375,11 @@ MR_COLLECT_filter(MR_FilterFuncPtr filter_ptr, MR_Unsigned seqno,
     MR_Char result;
     MR_Word arguments;
 
-    /*
-    ** Only pass the arguments list down filter
-    ** if required, i.e. if MR_collect_arguments
-    ** is set to MR_TRUE. We need to do that in
-    ** order to not penalize the performance
-    ** of collect in the cases where the argument
-    ** list (which might be very big) is not used.
-    **
-    */
+    // Only pass the arguments list down filter if required,
+    // i.e. if MR_collect_arguments is set to MR_TRUE. We need to do that
+    // in order to not penalize the performance of collect in the cases where
+    // the argument list (which might be very big) is not used.
+
     if (MR_collect_arguments) {
         arguments = MR_trace_make_var_list();
     } else {
@@ -1491,9 +1410,7 @@ MR_COLLECT_filter(MR_FilterFuncPtr filter_ptr, MR_Unsigned seqno,
     *stop_collecting = (result == 'y');
 }
 
-/*
-** This function retrieves the line number of the current goal.
-*/
+// This function retrieves the line number of the current goal.
 
 int
 MR_get_line_number(MR_Word *saved_regs, const MR_LabelLayout *layout,
@@ -1508,16 +1425,14 @@ MR_get_line_number(MR_Word *saved_regs, const MR_LabelLayout *layout,
     MR_Unsigned             reused_frames;
 
     if MR_port_is_interface(port) {
-        /*
-        ** At external events, we want the line number where the call is made,
-        ** not the one where the procedure is defined.
-        */
+        // At external events, we want the line number where the call is made,
+        // not the one where the procedure is defined.
 
         base_sp = MR_saved_sp(saved_regs);
         base_curfr = MR_saved_curfr(saved_regs);
         parent_layout = MR_find_nth_ancestor(layout, 1, &base_sp, &base_curfr,
             &reused_frames, &problem);
-        /* The external debugger does not (yet) know about reused frames. */
+        // The external debugger does not (yet) know about reused frames.
         assert(reused_frames == 0);
         if (parent_layout != NULL) {
             (void) MR_find_context(parent_layout, &filename, &lineno);
@@ -1546,4 +1461,4 @@ MR_send_collect_result(void)
 #endif
 }
 
-#endif /* MR_USE_EXTERNAL_DEBUGGER */
+#endif // MR_USE_EXTERNAL_DEBUGGER
