@@ -1,18 +1,13 @@
-/*
-** vim: ts=4 sw=4 expandtab
-*/
-/*
-** Copyright (C) 1998-2007, 2011 The University of Melbourne.
-** This file may only be copied under the terms of the GNU Library General
-** Public License - see the file COPYING.LIB in the Mercury distribution.
-*/
+// vim: ts=4 sw=4 expandtab ft=c
 
-/*
-** This file manages a table listing the debuggable modules of the program,
-** and subsidiary tables listing the procedures of each of those modules.
-**
-** Main author: Zoltan Somogyi.
-*/
+// Copyright (C) 1998-2007, 2011 The University of Melbourne.
+// Copyright (C) 2014-2016, 2018 The Mercury team.
+// This file is distributed under the terms specified in COPYING.LIB.
+
+// This file manages a table listing the debuggable modules of the program,
+// and subsidiary tables listing the procedures of each of those modules.
+//
+// Main author: Zoltan Somogyi.
 
 #include "mercury_imp.h"
 #include "mercury_label.h"
@@ -29,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 
 MR_TraceEventSet    *MR_trace_event_sets = NULL;
 int                 MR_trace_event_set_next = 0;
@@ -37,19 +33,17 @@ int                 MR_trace_event_set_max = 0;
 MR_bool             MR_trace_event_sets_are_all_consistent = MR_TRUE;
 int                 MR_trace_event_sets_max_num_attr = -1;
 
-/*
-** We record module layout structures in two tables. The MR_module_infos
-** array contains one pointer to every module layout structure, and is ordered
-** by the fully qualified module name. The MR_module_nickname array contains
-** one reference to every module layout structure by every one of the module's
-** (zero or more) less-than-fully-qualified names (which we call `nickname'
-** here), ordered by the nickname.
-*/
+// We record module layout structures in two tables. The MR_module_infos
+// array contains one pointer to every module layout structure, and is ordered
+// by the fully qualified module name. The MR_module_nickname array contains
+// one reference to every module layout structure by every one of the module's
+// (zero or more) less-than-fully-qualified names (which we call `nickname'
+// here), ordered by the nickname.
 
 typedef struct {
     const char          *MR_nick_name;
     MR_Dlist            *MR_nick_layouts;
-                        /* the list entries are MR_ModuleLayouts */
+                        // The list entries are MR_ModuleLayouts.
 } MR_Module_Nick;
 
 static  MR_Module_Nick  *MR_module_nicks;
@@ -75,7 +69,8 @@ static  void            MR_process_matching_procedures_in_module(
 static  void            MR_process_line_layouts(const MR_ModuleFileLayout
                             *file_layout, int line,
                             MR_file_line_callback callback_func,
-                            int callback_arg);
+                            int callback_arg, int *num_line_matches_ptr,
+                            int *next_lower, int *next_higher);
 
 static  MR_bool         MR_module_in_arena(const char *name, char **names,
                             int num_names);
@@ -95,25 +90,22 @@ static  char            *MR_get_module_info_name(int slot);
 typedef struct {
     MR_PredFunc MR_complete_pf;
 
-                /*
-                ** The word to complete, with `__' translated into '.'.
-                */
+                // The word to complete, with `__' translated into '.'.
+
     char        *MR_complete_name;
     size_t      MR_complete_name_len;
     MR_bool     MR_complete_name_is_qualified;
 
-                /*
-                ** Slot number of a module for which we should return all
-                ** procedures as completions, -1 if there is none.
-                */
+                // Slot number of a module for which we should return all
+                // procedures as completions, -1 if there is none.
+
     int         MR_unambiguous_matching_module;
 
-                /*
-                ** Length of the part of the word to skip when testing for module
-                ** qualified completions in the current module, zero if we
-                ** shouldn't test for module qualified completions in the
-                ** current module.
-                */
+                // Length of the part of the word to skip when testing for
+                // module qualified completions in the current module,
+                // zero if we shouldn't test for module qualified completions
+                // in the current module.
+
     int         MR_complete_word_matches_module;
     int         MR_complete_current_module;
     int         MR_complete_current_proc;
@@ -163,12 +155,10 @@ MR_register_all_modules_and_procs(FILE *fp, MR_bool verbose)
 void
 MR_register_module_layout_real(const MR_ModuleLayout *module)
 {
-    /*
-    ** MR_register_module_layout_real should only be called from
-    ** the initialization function of a module, which should be
-    ** called only once. The check here whether the module layout
-    ** already exists in the table is really only for paranoia.
-    */
+    // MR_register_module_layout_real should only be called from
+    // the initialization function of a module, which should be
+    // called only once. The check here whether the module layout
+    // already exists in the table is really only for paranoia.
 
     if (MR_search_module_info_by_name(module->MR_ml_name) == NULL) {
         MR_insert_module_info(module);
@@ -178,7 +168,7 @@ MR_register_module_layout_real(const MR_ModuleLayout *module)
             MR_bool             found;
             const char          *event_set_name;
             MR_TraceEventSet    *trace_event_set;
-            
+
             event_set_name = module->MR_ml_user_event_set_name;
 
             found = MR_FALSE;
@@ -218,7 +208,7 @@ MR_register_module_layout_real(const MR_ModuleLayout *module)
                     module->MR_ml_num_user_event_specs;
                 trace_event_set->MR_tes_specs =
                     module->MR_ml_user_event_specs;
-                trace_event_set->MR_tes_event_set = 
+                trace_event_set->MR_tes_event_set =
                     MR_read_event_set("no input file",
                         trace_event_set->MR_tes_desc);
 
@@ -315,7 +305,7 @@ MR_insert_module_info(const MR_ModuleLayout *module)
 
     nickname = strchr(module->MR_ml_name, '.');
     while (nickname != NULL) {
-        nickname++; /* step over the '.' */
+        nickname++; // Step over the '.'.
         MR_bsearch(MR_module_nick_next, slot, found,
             strcmp(MR_module_nicks[slot].MR_nick_name, nickname));
         if (found) {
@@ -337,18 +327,28 @@ MR_insert_module_info(const MR_ModuleLayout *module)
 
 void
 MR_process_file_line_layouts(const char *file, int line,
-    MR_file_line_callback callback_func, int callback_arg)
+    MR_file_line_callback callback_func, int callback_arg,
+    int *num_file_matches_ptr, int *num_line_matches_ptr,
+    int *next_lower, int *next_higher)
 {
     const MR_ModuleFileLayout   *file_layout;
     size_t                      i;
     int                         j;
 
+    *num_file_matches_ptr = 0;
+    *num_line_matches_ptr = 0;
+    *next_lower = -1;
+    *next_higher = INT_MAX;
+    // The next_higher sentinel, INT_MAX must match the corresponding value
+    // in mercury_trace_spy.c.
     for (i = 0; i < MR_module_info_next; i++) {
         for (j = 0; j < MR_module_infos[i]->MR_ml_filename_count; j++) {
             file_layout = MR_module_infos[i]->MR_ml_module_file_layout[j];
             if (MR_streq(file_layout->MR_mfl_filename, file)) {
+                *num_file_matches_ptr = *num_file_matches_ptr + 1;
                 MR_process_line_layouts(file_layout, line, callback_func,
-                    callback_arg);
+                    callback_arg, num_line_matches_ptr,
+                    next_lower, next_higher);
             }
         }
     }
@@ -356,7 +356,8 @@ MR_process_file_line_layouts(const char *file, int line,
 
 static void
 MR_process_line_layouts(const MR_ModuleFileLayout *file_layout, int line,
-    MR_file_line_callback callback_func, int callback_arg)
+    MR_file_line_callback callback_func, int callback_arg,
+    int *num_line_matches_ptr, int *next_lower, int *next_higher)
 {
     int         k;
     MR_bool     found;
@@ -365,21 +366,57 @@ MR_process_line_layouts(const MR_ModuleFileLayout *file_layout, int line,
         file_layout->MR_mfl_label_lineno[k] - line);
 
     if (found) {
-        /*
-        ** The binary search found *one* label with the given linenumber;
-        ** we now find the *first* such label.
-        */
+        // The binary search found *one* label with the given linenumber.
+        // We now find the *first* such label, ...
 
         while (k > 0 && file_layout->MR_mfl_label_lineno[k - 1] == line) {
             k--;
         }
 
+        // ... and then do a callback for *all* such labels.
+
         while (k < file_layout->MR_mfl_label_count
             && file_layout->MR_mfl_label_lineno[k] == line)
         {
             (*callback_func)(file_layout->MR_mfl_label_layout[k],
                 callback_arg);
             k++;
+            *num_line_matches_ptr = *num_line_matches_ptr + 1;
+        }
+    } else {
+        // It would be nice if we could ask MR_bsearch to tell us
+        // which parts of the array cannot contain the next lower
+        // and next higher line numbers, but since we cannot,
+        // a binary search will have to do.
+
+        // These variables hold the next lower and higher line numbers
+        // (if any) *in this file*. *next_lower and *next_higher apply
+        // to *all file structures* with the specified name. (Theoretically,
+        // there may be more than one, especially in the presence of
+        // nested submodules and/or #line directives.)
+        int lower = -1;
+        int higher = INT_MAX;
+
+        k = 0;
+        while (k < file_layout->MR_mfl_label_count) {
+            if (file_layout->MR_mfl_label_lineno[k] < line) {
+                lower = file_layout->MR_mfl_label_lineno[k];
+                k++;
+            } else {
+                break;
+            }
+        }
+
+        if (k < file_layout->MR_mfl_label_count) {
+            higher = file_layout->MR_mfl_label_lineno[k];
+        }
+
+        if (lower > *next_lower) {
+            *next_lower = lower;
+        }
+
+        if (higher < *next_higher) {
+            *next_higher = higher;
         }
     }
 }
@@ -396,7 +433,7 @@ MR_dump_module_tables(FILE *fp, MR_bool separate, MR_bool uci,
     if (module_name != NULL) {
         module = MR_search_module_info_by_unique_name(fp, module_name);
         if (module == NULL) {
-            /* The error message has already been printed */
+            // The error message has already been printed.
             return;
         }
     } else {
@@ -441,16 +478,14 @@ MR_dump_module_procs(FILE *fp, const char *name)
 
     module = MR_search_module_info_by_unique_name(fp, name);
     if (module == NULL) {
-        /* The error message has already been printed */
+        // The error message has already been printed.
         return;
     }
 
     fprintf(fp, "List of procedures in module `%s'\n\n", name);
     for (i = 0; i < module->MR_ml_proc_count; i++) {
         decl_module = MR_get_proc_decl_module(module->MR_ml_procs[i]);
-        /*
-        ** Only show procs which are declared in the module.
-        */
+        // Only show procs which are declared in the module.
         if (MR_streq(decl_module, module->MR_ml_name)) {
             MR_print_proc_id_and_nl(fp, module->MR_ml_procs[i]);
         }
@@ -530,11 +565,9 @@ MR_compare_proc_layout_by_name(const void *ptr1, const void *ptr2)
         return result;
     }
 
-    /*
-    ** Return equal only if the module name, pred_or_func and the arity
-    ** are the same as well, in order to group all procedures of a predicate
-    ** or function together.
-    */
+    // Return equal only if the module name, pred_or_func and the arity
+    // are the same as well, in order to group all procedures of a predicate
+    // or function together.
 
     result = MR_proc_compare_module_name(proc1, proc2);
     if (result != 0) {
@@ -613,7 +646,8 @@ typedef struct {
     ((functor1).MR_functor_arity - (functor2).MR_functor_arity)
 
 #define MR_functor_compare_type_module(functor1, functor2)              \
-    strcmp((functor1).MR_functor_type_module, (functor2).MR_functor_type_module)
+    strcmp((functor1).MR_functor_type_module,                           \
+        (functor2).MR_functor_type_module)
 
 #define MR_functor_compare_type_name(functor1, functor2)                \
     strcmp((functor1).MR_functor_type_name, (functor2).MR_functor_type_name)
@@ -682,7 +716,6 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
     MR_bool                     *report;
     MR_EnumFunctorDesc          **enum_functors;
     MR_DuFunctorDesc            **du_functors;
-    MR_MaybeResAddrFunctorDesc  *res_functors;
     MR_NotagFunctorDesc         *notag_functor;
     int                         num_distinct;
     int                         num_ambiguous;
@@ -690,22 +723,18 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
     int                         num_ambiguous_max;
     int                         i;
 
-    /*
-    ** We compute each data structure regardless of the settings of
-    ** print_procs, print_types and print_functors because the implementation
-    ** of one may depend on the other; e.g. we calculate how many function
-    ** symbols to reserve space for while we build the types array.
-    */
+    // We compute each data structure regardless of the settings of
+    // print_procs, print_types and print_functors because the implementation
+    // of one may depend on the other; e.g. we calculate how many function
+    // symbols to reserve space for while we build the types array.
 
     num_procs = 0;
     for (module_num = 0; module_num < MR_module_info_next; module_num++) {
         num_procs += MR_module_infos[module_num]->MR_ml_proc_count;
     }
 
-    /*
-    ** num_procs is a conservative estimate of the number of user defined
-    ** procedures.
-    */
+    // num_procs is a conservative estimate of the number of user defined
+    // procedures.
 
     procs = malloc(sizeof(const MR_ProcLayout *) * num_procs);
     if (procs == NULL) {
@@ -830,15 +859,13 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
                 case MR_TYPECTOR_REP_DUMMY:
                 case MR_TYPECTOR_REP_DU:
                 case MR_TYPECTOR_REP_DU_USEREQ:
-                case MR_TYPECTOR_REP_RESERVED_ADDR:
-                case MR_TYPECTOR_REP_RESERVED_ADDR_USEREQ:
                 case MR_TYPECTOR_REP_NOTAG:
                 case MR_TYPECTOR_REP_NOTAG_USEREQ:
                 case MR_TYPECTOR_REP_NOTAG_GROUND:
                 case MR_TYPECTOR_REP_NOTAG_GROUND_USEREQ:
                     num_functors += type_ctor_info->MR_type_ctor_num_functors;
 #if 0
-                    /* for debugging only */
+                    // for debugging only
                     fprintf(fp,
                         "TYPE %s.%s/%" MR_INTEGER_LENGTH_MODIFIER "d\t%s\t%d\n",
                         type_ctors[type_num]->MR_type_ctor_module_name,
@@ -920,7 +947,7 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
         type_ctor_info = type_ctors[type_num];
 
 #if 0
-        /* for debugging only */
+        // for debugging only
         fprintf(fp, "FUNCTYPE %s.%s/%" MR_INTEGER_LENGTH_MODIFIER "d\n",
             type_ctors[type_num]->MR_type_ctor_module_name,
             type_ctors[type_num]->MR_type_ctor_name,
@@ -970,28 +997,6 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
                 }
                 break;
 
-            case MR_TYPECTOR_REP_RESERVED_ADDR:
-            case MR_TYPECTOR_REP_RESERVED_ADDR_USEREQ:
-                res_functors =
-                    MR_type_ctor_functors(type_ctor_info).MR_functors_res;
-                i = 0;
-                while (i < type_ctor_info->MR_type_ctor_num_functors) {
-                    functors[functor_num].MR_functor_name =
-                        res_functors[i].MR_maybe_res_name;
-                    functors[functor_num].MR_functor_arity =
-                        res_functors[i].MR_maybe_res_arity;
-                    functors[functor_num].MR_functor_type_module =
-                        type_ctor_info->MR_type_ctor_module_name;
-                    functors[functor_num].MR_functor_type_name =
-                        type_ctor_info->MR_type_ctor_name;
-                    functors[functor_num].MR_functor_type_arity =
-                        type_ctor_info->MR_type_ctor_arity;
-                    functor_num++;
-                    i++;
-                }
-                break;
-                break;
-
             case MR_TYPECTOR_REP_NOTAG:
             case MR_TYPECTOR_REP_NOTAG_USEREQ:
             case MR_TYPECTOR_REP_NOTAG_GROUND:
@@ -1011,10 +1016,8 @@ MR_print_ambiguities(FILE *fp, MR_bool print_procs, MR_bool print_types,
                 break;
 
             default:
-                /*
-                ** Other kinds of types do not have user-defined function
-                ** symbols.
-                */
+                // Other kinds of types do not have user-defined function
+                // symbols.
                 break;
         }
     }
@@ -1091,33 +1094,31 @@ MR_parse_proc_spec(char *str, MR_ProcSpec *spec)
 
     len = strlen(str);
 
-    /*
-    ** Check for the optional trailing arity and mode number.
-    ** This also checks for filename:linenumber breakpoint specifiers.
-    */
+    // Check for the optional trailing arity and mode number.
+    // This also checks for filename:linenumber breakpoint specifiers.
+
     end = str + len - 1;
     if (MR_parse_trailing_number(str, &end, &n)) {
         if (end == str) {
-            /* the string contains only a number */
+            // The string contains only a number.
             return MR_FALSE;
         }
         end--;
         if (*end == ':') {
-            /* filename:linenumber */
+            // filename:linenumber
             return MR_FALSE;
         } else if (*end == '-') {
             spec->MR_proc_mode = n;
 
-            /*
-            ** Avoid modifying the string until we're sure
-            ** the parse can't fail.
-            */
+            // Avoid modifying the string until we're sure
+            // the parse can't fail.
+
             dash = end;
 
             end--;
             if (MR_parse_trailing_number(str, &end, &n)) {
                 if (end == str) {
-                    /* the string contains only a number */
+                    // The string contains only a number.
                     return MR_FALSE;
                 }
                 end--;
@@ -1155,10 +1156,8 @@ MR_parse_proc_spec(char *str, MR_ProcSpec *spec)
         str += 5;
     }
 
-    /*
-    ** Search backwards for the end of the final module qualifier.
-    ** There must be at least one character before the qualifier.
-    */
+    // Search backwards for the end of the final module qualifier.
+    // There must be at least one character before the qualifier.
 
     while (end > str) {
         if (*end == '.' || (*end == '_' && *(end + 1) == '_')) {
@@ -1172,9 +1171,8 @@ MR_parse_proc_spec(char *str, MR_ProcSpec *spec)
                 spec->MR_proc_name = NULL;
             }
 
-            /*
-            ** Convert all occurences of '__' to '.'.
-            */
+            // Convert all occurences of '__' to '.'.
+
             *end = '\0';
             MR_translate_double_underscores(str);
 
@@ -1186,7 +1184,7 @@ MR_parse_proc_spec(char *str, MR_ProcSpec *spec)
         }
     }
 
-    /* There was no module qualifier. */
+    // There was no module qualifier.
     spec->MR_proc_name = str;
     if (strlen(spec->MR_proc_name) == 0) {
         spec->MR_proc_name = NULL;
@@ -1194,9 +1192,7 @@ MR_parse_proc_spec(char *str, MR_ProcSpec *spec)
     return MR_TRUE;
 }
 
-/*
-** Convert all occurrences of `__' to `.'.
-*/
+// Convert all occurrences of `__' to `.'.
 
 static void
 MR_translate_double_underscores(char *start)
@@ -1218,12 +1214,10 @@ MR_translate_double_underscores(char *start)
     *(str - double_underscores) = '\0';
 }
 
-/*
-** Go backwards over a string starting at `end', stopping at `start',
-** parsing the trailing integer and storing it in `*n'.
-** On return, `*end' points to the start of the trailing number.
-** If no number was found, `*end' is unchanged.
-*/
+// Go backwards over a string starting at `end', stopping at `start',
+// parsing the trailing integer and storing it in `*n'.
+// On return, `*end' points to the start of the trailing number.
+// If no number was found, `*end' is unchanged.
 
 static MR_bool
 MR_parse_trailing_number(char *start, char **end, int *number)
@@ -1275,10 +1269,8 @@ MR_search_for_matching_procedures(MR_ProcSpec *spec)
     return m;
 }
 
-/*
-** This struct is for communication between
-** MR_register_match and MR_search_for_matching_procedure.
-*/
+// This struct is for communication between
+// MR_register_match and MR_search_for_matching_procedure.
 
 typedef struct {
     const MR_ProcLayout     *matching_entry;
@@ -1463,11 +1455,6 @@ MR_trace_proc_spec_completer(const char *word, size_t word_len)
         data->MR_complete_pf = MR_FUNCTION;
         word += 5;
     } else {
-        /*
-        ** We don't complete on the names of special (unify, index compare,
-        ** or init) predicates.
-        */
-
         data->MR_complete_pf = -1;
     }
 
@@ -1480,19 +1467,20 @@ MR_trace_proc_spec_completer(const char *word, size_t word_len)
     data->MR_complete_current_module = -1;
     data->MR_complete_current_proc= -1;
 
-    /*
-    ** Handle the case where the word matches the first part of a module name.
-    ** If the word unambiguously determines the module name we want to return
-    ** module qualified completions for all the procedures in that module.
-    ** Otherwise, we just complete on the names of all of the matching modules
-    ** and unqualified procedure names.
-    **
-    ** For example, given word to complete `f' and modules `foo' and `bar',
-    ** we want to return all the procedures in module `foo' as completions,
-    ** as well as all procedures whose unqualified names begin with `f'.
-    ** Given word to complete `foo.' and modules `foo' and `foo.bar' we want to
-    ** return `foo.bar.' and all the procedures in module `foo' as completions.
-    */
+    // Handle the case where the word matches the first part of a module name.
+    // If the word unambiguously determines the module name, we want to return
+    // module qualified completions for all the procedures in that module.
+    // Otherwise, we just complete on the names of all of the matching modules
+    // and unqualified procedure names.
+    //
+    // For example, given word to complete `f' and modules `foo' and `bar',
+    // we want to return all the procedures in module `foo' as completions,
+    // as well as all procedures whose unqualified names begin with `f'.
+    // Given word to complete `foo.' and modules `foo' and `foo.bar' we want to
+    // return `foo.bar.' and all the procedures in module `foo' as completions.
+    //
+    // We don't complete on the names of special (unify, index or compare)
+    // predicates.
 
     MR_bsearch(MR_module_info_next, slot, found,
         strncmp(MR_module_infos[slot]->MR_ml_name,
@@ -1538,11 +1526,10 @@ try_completion:
     if (data->MR_complete_current_module == -1 ||
         data->MR_complete_current_proc == -1 ||
         data->MR_complete_current_proc >=
-            MR_module_infos[data->MR_complete_current_module]->MR_ml_proc_count)
+            MR_module_infos[data->MR_complete_current_module]->
+                MR_ml_proc_count)
     {
-        /*
-        ** Move on to the next module.
-        */
+        // Move on to the next module.
 
         data->MR_complete_current_module++;
         if (data->MR_complete_current_module >= MR_module_info_next) {
@@ -1550,10 +1537,8 @@ try_completion:
         }
         MR_trace_proc_spec_completer_init_module(data);
 
-        /*
-        ** Complete on the module name if we aren't finding
-        ** qualified completions in this module.
-        */
+        // Complete on the module name if we aren't finding
+        // qualified completions in this module.
 
         module_name = MR_module_infos[data->MR_complete_current_module]
             ->MR_ml_name;
@@ -1566,9 +1551,8 @@ try_completion:
             goto try_completion;
         }
     } else {
-        /*
-        ** Complete on the next procedure in the current module.
-        */
+        // Complete on the next procedure in the current module.
+
         completion = MR_trace_complete_proc(data);
 
         if (completion != NULL) {
@@ -1579,9 +1563,7 @@ try_completion:
     }
 }
 
-/*
-** Set up the completer data for processing a module.
-*/
+// Set up the completer data for processing a module.
 
 static void
 MR_trace_proc_spec_completer_init_module(MR_ProcCompleterData *data)
@@ -1598,42 +1580,35 @@ MR_trace_proc_spec_completer_init_module(MR_ProcCompleterData *data)
         MR_module_infos[data->MR_complete_current_module]->MR_ml_name;
     module_name_len = strlen(module_name);
 
-    /*
-    ** Work out whether we should find qualified completions
-    ** for procedures in this module.
-    */
+    // Work out whether we should find qualified completions
+    // for procedures in this module.
 
     if (MR_strneq(module_name, name, module_name_len)
         && name_len > module_name_len
         && name[module_name_len] == '.'
         && strchr(name + module_name_len + 1, '.') == NULL)
     {
-        /*
-        ** The name to complete matches the module name completely.
-        ** When searching for qualified completions skip past
-        ** the module name and the trailing '.'.
-        */
+        // The name to complete matches the module name completely.
+        // When searching for qualified completions skip past
+        // the module name and the trailing '.'.
 
         data->MR_complete_word_matches_module = module_name_len + 1;
     } else if (data->MR_complete_current_module ==
         data->MR_unambiguous_matching_module)
     {
-        /*
-        ** The name to complete matches the module name partially,
-        ** and does not match any other module name. We will be
-        ** matching all procedures, use the empty string as the
-        ** name to match against.
-        */
+        // The name to complete matches the module name partially,
+        // and does not match any other module name. We will be
+        // matching all procedures, use the empty string as the
+        // name to match against.
 
         data->MR_complete_word_matches_module = name_len;
     } else {
         data->MR_complete_word_matches_module = 0;
     }
 
-    /*
-    ** If the name to complete is qualified, we should only
-    ** complete on procedures if the module name matches.
-    */
+    // If the name to complete is qualified, we should only
+    // complete on procedures if the module name matches.
+
     if (data->MR_complete_name_is_qualified &&
         data->MR_complete_word_matches_module == 0)
     {
@@ -1643,10 +1618,8 @@ MR_trace_proc_spec_completer_init_module(MR_ProcCompleterData *data)
     }
 }
 
-/*
-** Check whether the current procedure matches the word to be completed.
-** To do: complete on arity and mode number.
-*/
+// Check whether the current procedure matches the word to be completed.
+// To do: complete on arity and mode number.
 
 static char *
 MR_trace_complete_proc(MR_ProcCompleterData *data)
@@ -1688,9 +1661,7 @@ MR_trace_complete_proc(MR_ProcCompleterData *data)
         completion = NULL;
     }
 
-    /*
-    ** Move on to the next procedure in the current module.
-    */
+    // Move on to the next procedure in the current module.
 
     data->MR_complete_current_proc++;
 
@@ -1698,11 +1669,10 @@ MR_trace_complete_proc(MR_ProcCompleterData *data)
         && data->MR_complete_current_proc >= module_layout->MR_ml_proc_count
         && ! data->MR_complete_name_is_qualified)
     {
-        /*
-        ** We've finished checking for module qualified completions
-        ** in this module, now check for unqualified completions
-        ** if the word to complete doesn't contain a qualifier.
-        */
+        // We have finished checking for module qualified completions
+        // in this module, now check for unqualified completions
+        // if the word to complete doesn't contain a qualifier.
+
         data->MR_complete_word_matches_module = 0;
         data->MR_complete_current_proc = 0;
     }
@@ -1724,11 +1694,11 @@ MR_format_proc_spec_completion(MR_PredFunc pred_or_func,
         size += 5;
     }
     if (module != NULL) {
-        /* +1 for the '.' */
+        // +1 for the '.'
         module_len = strlen(module);
         size += module_len + 1;
     } else {
-        module_len = 0;     /* avoid a warning */
+        module_len = 0;     // Avoid a compiler warning.
     }
     completion = MR_malloc(size + 1);
 
