@@ -172,8 +172,6 @@ demangle(const char *orig_name)
     static const char compare2[] = "__Compare____";
     static const char index1[]  = "__Index___";
     static const char index2[]  = "__Index____";
-    static const char initialise1[]  = "__Initialise___";
-    static const char initialise2[]  = "__Initialise____";
 
     static const char introduced[]  = "IntroducedFrom__";
     static const char deforestation[]  = "DeforestationIn__";
@@ -213,7 +211,7 @@ demangle(const char *orig_name)
         accumulator,
         type_spec,
         unused_arg,
-        unify1, compare1, index1, initialise1,
+        unify1, compare1, index1,
         NULL
     };
 
@@ -262,7 +260,7 @@ demangle(const char *orig_name)
     char        *lambda_pred_name = NULL;
     char        *end_of_lambda_pred_name = NULL;
     const char  *lambda_kind = NULL;
-    enum        { ORDINARY, UNIFY, COMPARE, INDEX, INITIALISE,
+    enum        { ORDINARY, UNIFY, COMPARE, INDEX,
                 LAMBDA, DEFORESTATION, ACCUMULATOR, TYPE_SPEC } category;
     enum        { COMMON, INFO, LAYOUT, FUNCTORS } data_category;
     const char  *class_name;
@@ -369,8 +367,7 @@ demangle(const char *orig_name)
         */
         high_level = (strstr(start, unify2) ||
             strstr(start, compare2) ||
-            strstr(start, index2) ||
-            strstr(start, initialise2));
+            strstr(start, index2));
         pred_or_func = "predicate";
     }
 
@@ -406,8 +403,6 @@ demangle(const char *orig_name)
     } else if (strip_prefix(&start, index1)) {
         category = INDEX;
         if (mode_num != 0) goto not_plain_mercury;
-    } else if (strip_prefix(&start, initialise1)) {
-        category = INITIALISE;
     } else {
         category = ORDINARY;
         /*
@@ -643,11 +638,6 @@ demangle(const char *orig_name)
                 module, start, arity);
             break;
 
-        case INITIALISE:
-            printf("initialisation predicate for type '%s.%s'/%d",
-                module, start, arity);
-            break;
-
         case LAMBDA:
             printf("%s goal (#%d) from '%s' in module '%s' line %d",
                 lambda_kind, lambda_seq_number,
@@ -1129,7 +1119,7 @@ find_double_underscore(char **start, char *end)
 ** The compiler changes all names starting with `f_' so that they start with
 ** `f__' instead, and uses names starting with `f_' for mangled names
 ** which are either descriptions (such as `f_greater_than' for `>')
-** or sequences of decimal reprententations of ASCII codes separated by
+** or sequences of decimal representations of ASCII codes separated by
 ** underscores. If the name starts with `f__', we must change it back to
 ** start with `f_'. Otherwise, if it starts with `f_' we must convert
 ** the mnemonic or list of ASCII codes back into an identifier.
